package ClickRatioReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# リンク元ランキングモジュール
# ClickRatioReport.pm ver1.0.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-02-11
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	利用可能なスクリーン幅
# 	$$data[9]	利用可能なスクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;
    my $uri     = $$data[2];
    my $referer = $$data[5];
    return 0 if !$uri || !$referer;
    if (   $setting::stat{'click'}{'option'} eq 'ignore'
        || $setting::stat{'click'}{'option'} eq 'noquery' )
    {
        $uri =~ s/\?.*$//;
        $referer =~ s/\?.*$//;
    }
    else {
        $uri     = $this->url_filter( $uri,     1 );
        $referer = $this->url_filter( $referer, 1 );
    }
    if (   $setting::stat{'click'}{'option'} eq 'ignore'
        || $setting::stat{'click'}{'option'} eq 'noanchor' )
    {
        $uri =~ s/\#.*$//;
        $referer =~ s/\#.*$//;
    }
    $this->{'total_click'}++ if $uri eq $setting::stat{'click'}{'target'};
    if ( $referer eq $setting::stat{'click'}{'target'} ) {
        $this->{'click_count'}->{$uri}++;
        $this->{'checked_click'}++;
        return 1;
    }
    else {
        return 0;
    }
}

sub _stat_setting ($) {
    my $this = shift;
    $main::setting{'click_target'} = $main::clicktarget
        if defined $main::clicktarget && $main::clicktarget;
    return;
}

######################################################################
# 解析環境（追加）
sub print_sub_stat_env ($) {
    my $this = shift;
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('対象ページ');
    print $this->{'infoTable'}->td(
        '<input type="text" name="setting:click_target" size="30" value="'
            . $this->escape( $setting::stat{'click'}{'target'} ) . '" />'
            . ' <select name="setting:click_option">'
            . '<option value="">URL単位で集計する</option>'
            . '<option value="noquery"'
            . (
            ( $setting::stat{'click'}{'option'} eq 'noquery' )
            ? ' selected="selected"'
            : ''
            )
            . '>CGIの条件部分を無視する</option>'
            . '<option value="noanchor"'
            . (
            ( $setting::stat{'click'}{'option'} eq 'noanchor' )
            ? ' selected="selected"'
            : ''
            )
            . '>アンカーを無視する</option>'
            . '<option value="ignore"'
            . (
            ( $setting::stat{'click'}{'option'} eq 'ignore' )
            ? ' selected="selected"'
            : ''
            )
            . '>CGIの条件部分とアンカーを無視する</option>'
            . '</select>'
            . $this->submit_button(
            $env::image_dir . '/stat/node.gif',
            14,
            14,
            'このファイルからのリンクのクリック率をランキングします。',
            '',
            'm',
            'ClickRatioReport',
            'k',
            'click'
            )
    );
    print $this->{'infoTable'}->_tr;
}

######################################################################
# キー項目調整
sub stat_key ($$) {
    my ( $this, $key ) = @_;
    return $this->url_to_title($key);
}

######################################################################
# 追加項目
sub stat_option ($$$) {
    my ( $this, $key, $value ) = @_;
    if ( $key =~ m!^http://!i ) {
        return $this->submit_button(
            $env::image_dir . '/stat/node.gif',
            14,
            14,
            'このファイルからのリンクのクリック率をランキングします。',
            '',
            'm',
            'ClickRatioReport',
            'k',
            'click',
            'clicktarget',
            $key
        );
    }
    else {
        return '&nbsp;';
    }
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return 'ページ別クリック率ランキング';
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    return if defined $this->{'noLog'} && $this->{'noLog'};
    $this->tie_title_db;
    my (@ranking);
    $this->{'total_click'}   = 0 if !defined $this->{'total_click'};
    $this->{'checked_click'} = 0 if !defined $this->{'checked_click'};
    $setting::stat{'click'}{'sort'} = 'descend'
        if !defined $setting::stat{'click'}{'sort'}
        || $setting::stat{'click'}{'sort'} eq '';
    $this->{'total_click'} = $this->{'checked_click'}
        if $this->{'total_click'}
        && $this->{'checked_click'}
        && $this->{'total_click'} < $this->{'checked_click'};
    $this->{'click_count'}->{ ( ($default::english) ? 'Missing' : '不明' ) }
        = $this->{'total_click'} - $this->{'checked_click'}
        if $this->{'total_click'} - $this->{'checked_click'} > 0;
    @ranking = sort {
        $this->sort_by(
            \$this->{'click_count'}->{$a},
            \$this->{'click_count'}->{$b},
            $setting::stat{'click'}{'sort'}
            )
            || $a cmp $b
    } keys %{ $this->{'click_count'} };
    $this->print_stat_report(
        '',
        \@ranking,
        $this->{'click_count'},
        '',
        '',
        $this->{'total_click'},
        'click',
        1,
        'リンク先',
        'left',
        300,
        'アクセス数',
        100,
        '%',
        'クリック率',
        60,
        'アクセス数/' . $this->{'total_click'} . '件[総数]'
    );
    $this->untie_title_db;
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'click' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'click', 'ClickRatioReport' );

}

1;

