package LanguageReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# 訪問者の言語環境モジュール
# LanguageReport.pm ver1.0.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-01-30
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	利用可能なスクリーン幅
# 	$$data[9]	利用可能なスクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;
    my ($lang);

    $lang
        = ( defined $$data[4] && $$data[4] =~ /^([0-9a-zA-Z_\-]+)/ )
        ? $1
        : '';
    return 0 if $lang eq '';

    if ( !$this->{'is_counted'}{ $$data[1] . $$data[3] } ) {
        $lang = lc $lang;
        $lang =~ s/^ja-[0-9a-zA-Z_]+$/ja/;
        $this->{'http_accept_language'}->{$lang}++;
        $this->{'is_counted'}{ $$data[1] . $$data[3] } = 1;
        $this->{'user_count'}++;
    }
    return 1;
}

######################################################################
# キー項目調整
sub stat_key ($$) {
    my ( $this, $key ) = @_;
    return '<strong>' . $this->get_lang($key) . '</strong>';
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return '訪問者の言語環境';
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    undef %{ $this->{'is_counted'} };
    delete $this->{'is_counted'};
    return if defined $this->{'noLog'} && $this->{'noLog'};

    my (@ranking);
    @ranking = sort {
        $this->sort_by( \$this->{'http_accept_language'}->{$a},
            \$this->{'http_accept_language'}->{$b}, 'descend' )
            || $a cmp $b
    } keys %{ $this->{'http_accept_language'} };

    $this->print_stat_report(
        '',                              \@ranking,
        $this->{'http_accept_language'}, '',
        '',                              $this->{'user_count'},
        '',                              1,
        '言語',                        'center',
        180,                             'ユーザー数',
        100,                             '%'
    );
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'language' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'language', 'LanguageReport' );

}

1;

