package MonitorReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# 訪問者のモニター環境モジュール
# MonitorReport.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	スクリーン幅
# 	$$data[9]	スクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;

    # データ取得不能
    return 0 if ( !$$data[6] || !$$data[7] ) && ( !$$data[8] || !$$data[9] );
    my ($platform);

    if ( $$data[3] =~ /Win/ ) {
        $platform = 'Windows';
    }
    elsif ( $$data[3] =~ /Mac/ ) {
        $platform = 'Macintosh';
    }
    elsif ( $$data[3] =~ /X[0-9]{2}/ ) {
        $platform = 'UNIX';
    }
    else {
        return 0;
    }

    if ( !$this->{'is_counted'}{ $$data[1] . $$data[3] } ) {

        # ウィンドウサイズ集計
        if ( $$data[6] > 0 && $$data[7] > 0 ) {
            $this->{'window_count'}->{$platform}++;
            $this->{'window_innerWidth'}->{$platform}  += $$data[6];
            $this->{'window_innerHeight'}->{$platform} += $$data[7];
        }

        # スクリーンサイズ集計
        if ( $$data[8] > 0 && $$data[9] > 0 ) {
            $this->{'screen_count'}->{$platform}++;
            $this->{'screen_width'}->{$platform}  += $$data[8];
            $this->{'screen_height'}->{$platform} += $$data[9];
            $this->{'screen_size'}->{ $$data[8] . '×' . $$data[9] }++;
        }

        # 最大化率集計
        if ( $$data[6] > 0 && $$data[8] > 0 ) {
            $this->{'both_count'}->{$platform}++;
            $this->{'window_maximized'}->{$platform}++
                if $$data[8] - $$data[6] < 50;
        }
        $this->{'is_counted'}{ $$data[1] . $$data[3] } = 1;
    }

    return 1;
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return '訪問者のモニター環境';
}

######################################################################
# 解析結果表示
sub print_report ($) {
    no warnings qw(numeric);
    my $this = shift;
    undef %{ $this->{'is_counted'} };
    delete $this->{'is_counted'};
    return if defined $this->{'noLog'} && $this->{'noLog'};
    my (@order);

    # ウィンドウサイズ/スクリーンサイズ平均値集計
    my ( %screen_size, %window_size, %graf_size );
    $this->screen_average(
        $this->{'screen_count'},
        $this->{'screen_width'},
        $this->{'screen_height'},
        0, \%screen_size, \%graf_size
    );
    $this->screen_average(
        $this->{'window_count'},
        $this->{'window_innerWidth'},
        $this->{'window_innerHeight'},
        1, \%window_size, \%graf_size
    );

    @order = (
        '全体 (スクリーン)',
        'Windows (スクリーン)',
        'Macintosh (スクリーン)',
        'UNIX (スクリーン)'
    );
    $this->print_stat_report(
        '平均スクリーンサイズ',
        \@order,
        \%screen_size,
        \%window_size,
        \%graf_size,
        (   ( !ref $this->{'screen_width'} )
            ? ''
            : $graf_size{ $this->maximum( \%screen_size ) }
        ),
        '', '',
        'ユーザー環境',
        'center', 200,
        '平均サイズ',
        100,
        'ピクセル[面積]'
    );

    # 最大化率
    my ( %window_maximized, $total_count, $total_maximized );
    foreach ( keys %{ $this->{'both_count'} } ) {
        $total_count     += $this->{'both_count'}->{$_};
        $total_maximized += $this->{'window_maximized'}->{$_};

        $window_maximized{$_} = ( $this->{'both_count'}->{$_} )
            ? int(
            (         $this->{'window_maximized'}->{$_}
                    / $this->{'both_count'}->{$_}
            ) * 10000
            ) / 100
            : 0;
    }
    $window_maximized{'全体'}
        = ($total_count)
        ? int( ( $total_maximized / $total_count ) * 10000 ) / 100
        : 0;

    @order = ( '全体', 'Windows', 'Macintosh', 'UNIX' );
    $this->print_stat_report(
        'ウィンドウ最大化率', \@order,
        \%window_maximized,            '',
        '',                            100,
        '',                            '',
        'ユーザー環境',          'center',
        200,                           '最大化率 (%)',
        100,                           '%'
    );

    # スクリーンサイズ分布
    @order = sort { $a <=> $b || $a cmp $b } keys %{ $this->{'screen_size'} };
    $this->print_stat_report(
        'スクリーンサイズ分布',
        \@order,
        $this->{'screen_size'},
        '', '',
        (   ( !%{ $this->{'screen_size'} } )
            ? ''
            : $this->{'screen_size'}
                ->{ $this->maximum( $this->{'screen_size'} ) }
        ),
        '', '',
        'スクリーンサイズ',
        'center', 200,
        'アクセス数',
        100, '人'
    );

}

sub screen_average ($$$$$$$) {
    my ( $this, $count, $width, $height, $window, $average, $graf ) = @_;
    my ( $platform, $w, $h, $total_count, $total_height, $total_width );
    while ( ( $platform, $w ) = each %$width ) {
        $w = ( $$count{$platform} ) ? int( $w / $$count{$platform} ) : 0;
        $h
            = ( $$count{$platform} )
            ? int( $$height{$platform} / $$count{$platform} )
            : 0;
        if ( defined $window && $window ) {
            $$average{ $platform . ' (スクリーン)' }
                { $platform . ' (ウィンドウ内側)' } = $w . '×' . $h;
            $$graf{ $platform . ' (ウィンドウ内側)' } = $w * $h;
        }
        else {
            $$average{ $platform . ' (スクリーン)' } = $w . '×' . $h;
            $$graf{ $platform . ' (スクリーン)' }    = $w * $h;
        }

        $total_count  += $$count{$platform};
        $total_width  += $$width{$platform};
        $total_height += $$height{$platform};
    }

    $w = ($total_count) ? int( $total_width / $total_count )  : 0;
    $h = ($total_count) ? int( $total_height / $total_count ) : 0;
    if ( defined $window && $window ) {
        $$average{'全体 (スクリーン)'}
            {'全体 (ウィンドウ内側)'} = $w . '×' . $h;
        $$graf{'全体 (ウィンドウ内側)'} = $w * $h;
    }
    else {
        $$average{'全体 (スクリーン)'} = $w . '×' . $h;
        $$graf{'全体 (スクリーン)'}    = $w * $h;
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'monitor' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'monitor', 'MonitorReport' );

}

1;

