package RequestReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# ファイル別アクセス数ランキングモジュール
# RequestReport.pm ver1.0.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-08-07
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	利用可能なスクリーン幅
# 	$$data[9]	利用可能なスクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;
    my ($request);
    $request = $$data[2];

    # 翻訳サイト対策
    if ( $request
        && ( $request =~ /\?(.+)$/ || $request =~ /^.+(http.*?)$/ ) )
    {
        my $query = $1;
        utf8::encode($query);
        $query =~ s/\+/ /g;
        $query
            =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/(($1 eq '26' || uc $1 eq '3D') ? '%' . $1 : pack("C",hex($1)))/ge;
        foreach my $t (@default::target) {
            if ( $query =~ s/^.*\Q$t\E/$t/ ) {
                $query =~ s/[\&\^].*$//;
                $query =~ s/%26/\&/gi;
                $query =~ s/%3D/=/gi;
                $request = $query;
                $request = 'http://' . $request
                    if $request !~ /^https?:\/\//i;
                last;
            }
        }
    }
    return 0 if !$request;

# パスワードの埋め込まれたURL→安全のため、パスワード部分を削除
#$request =~ s!^(http[s]?://)[0-9a-zA-Z_]+:[0-9a-zA-Z_]+\@!$1!si;

    # CGIの条件部分を無視
    $request =~ s/\?.*$//s
        if defined $setting::stat{'request'}{'option'}
        && $setting::stat{'request'}{'option'};

    # アンカーを無視
    if ( $setting::stat{'request'}{'option2'} ) {
        $request =~ s/\#.*$//s;
        $request =~ s/(\.[s]?htm[l]?)(?=[^0-9a-zA-Z_]).*?$/$1/s;
    }
    $request = $this->url_filter( $request, 1 );
    return 0 if !$request;
    if ( $setting::stat{'request'}{'option3'} ) {
        $this->{'access_file'}->{$request}++
            if !$this->{'access_ip'}->{$request}->{ $$data[1] };
        $this->{'access_ip'}->{$request}->{ $$data[1] }++;
    }
    else {
        $this->{'access_file'}->{$request}++;
    }
    return 1;
}

######################################################################
# キー項目調整
sub stat_key ($$;$) {
    my ( $this, $key ) = @_;
    return $this->url_to_title($key);
}

######################################################################
# 追加項目
sub stat_option ($$$) {
    my ( $this, $key, $value ) = @_;
    return $this->submit_button(
        $env::image_dir . '/stat/node.gif',
        14,
        14,
        'このファイルからのリンクのクリック率をランキングします。',
        '',
        'm',
        'ClickRatioReport',
        'k',
        'click',
        'setting:click_target',
        $key
    );
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return 'ファイル別アクセス数ランキング';
}

######################################################################
# 解析環境（追加）
sub print_sub_stat_env ($) {
    my $this = shift;
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('集計方法');
    print $this->{'infoTable'}->td(
              '<select name="setting:request_option">'
            . '<option value="">CGIの条件部分を含める</option>'
            . '<option value="1"'
            . (
            ( $setting::stat{'request'}{'option'} ) ? ' selected="selected"'
            : ''
            )
            . '>CGIの条件部分を無視する</option>'
            . '</select>'
            . ' <select name="setting:request_option2">'
            . '<option value="">アンカーを含める</option>'
            . '<option value="1"'
            . (
            ( $setting::stat{'request'}{'option2'} ) ? ' selected="selected"'
            : ''
            )
            . '>アンカーを無視する</option>'
            . '</select> '
            . ' <select name="setting:request_option3">'
            . '<option value="">同一IPからの再アクセスをカウントする</option>'
            . '<option value="1"'
            . (
            ( $setting::stat{'request'}{'option3'} ) ? ' selected="selected"'
            : ''
            )
            . '>同一IPからの再アクセスをカウントしない</option>'
            . '</select> '
            . $this->submit_button(
            $env::image_dir . '/stat/request.gif',
            20,
            20,
            'ファイル別アクセス数ランキング',
            '',
            'm',
            'RequestReport',
            'k',
            'request'
            )
    );
    print $this->{'infoTable'}->_tr;
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    return if defined $this->{'noLog'} && $this->{'noLog'};
    $this->tie_title_db;

    my (@ranking);
    $setting::stat{'request'}{'sort'} = 'descend'
        if !defined $setting::stat{'request'}{'sort'}
        || $setting::stat{'request'}{'sort'} eq '';
    @ranking = sort {
        $this->sort_by(
            \$this->{'access_file'}->{$a},
            \$this->{'access_file'}->{$b},
            $setting::stat{'request'}{'sort'}
            )
            || $a cmp $b
    } keys %{ $this->{'access_file'} };

    $this->print_stat_report(
        '',
        \@ranking,
        $this->{'access_file'},
        '', '',
        (     ( !@ranking ) ? ''
            : ( $setting::stat{'request'}{'sort'} eq 'ascend' )
            ? $this->{'access_file'}->{ $ranking[$#ranking] }
            : $this->{'access_file'}->{ $ranking[0] }
        ),
        'request',
        1, 'URL', 'left', 300,
        'アクセス数',
        100, '件',
        'クリック率',
        60
    );
    $this->untie_title_db;
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'request' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'request', 'RequestReport' );

}

1;

