package GeneralSetting;
require CMONOS::CGI_Lib::Tools::General;
@ISA = qw(General);
use strict;
use utf8;
######################################################################
# 一般設定モジュール
# GeneralSetting.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-02-09
######################################################################
# 設定チェック項目取得
sub get_sub_setting_items ($) {
    my $this = shift;

    #チェックボックス
    push( @check::checkbox, 'realtime', 'getShortTitle' );

    #数字チェック
    $check::num{'limit'} = 'ログ保存容量制限';

    #URLチェック
    $check::url{'whois'}    = 'WHOIS URL';
    $check::url{'redirect'} = 'リダイレクト URL';

    #複数メール書式チェック
    $check::emails{'mailto'} = '削除ファイル送付先';

    return;
}

sub ogtype_disabled ($) {
    my $this = shift;
    return 1;
}

######################################################################
# サブ設定パネル
sub print_sub_preference_panel ($$) {
    my ( $this, $master ) = @_;
    print '<fieldset><legend>ログ管理設定</legend>';
    print '<table class="field-table"><tbody>';

    if ( defined $master && $master ) {
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><label for="data:limit">ログ保存容量制限</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:limit" size="10" value="'
            . $this->escape( $main::data{'limit'} )
            . '" title="アクセスログ容量の最大値を設定します。アクセスログの容量が設定された容量を超えると自動的に古いログが削除されます。" /> <span class="unit">byte</span></td>';
        print
            '<td class="field-name"><label for="data:realtime">当日ログの解析</label></td>';
        print
            '<td class="q-ield"><input type="checkbox" name="data:realtime" id="data:realtime" value="1"'
            . ( ( $main::data{'realtime'} ) ? ' checked="checked"' : '' )
            . ' /> 許可する<span class="note">（許可しないことを推奨）</span></td>';
        print '</tr>';

        print '</tbody><tbody>';

        print '<tr>';
        print
            '<td class="field-name"><label for="data:mailto">削除ファイル送付先</label></td>';
        print
            '<td class="field" colspan="3"><textarea name="data:mailto" id="data:mailto" cols="56" rows="4" wrap="off" title="アクセスログ容量が制限値を超えたときに削除されるファイルは、ここで設定されたアドレスに添付メールで送信されます。">'
            . $this->escape( $main::data{'mailto'} )
            . '</textarea>';
        print
            '<div class="note">送付先が複数の時はカンマ（+改行）で区切ります。</div></td>';
        print '</tr>';
    }
    else {
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><span class="datatitle">ログ保存容量制限</span></td>';
        print '<td class="q-field">'
            . $this->digit( ( $main::data{'limit'} )
            ? $main::data{'limit'}
            : $default::limit )
            . ' <span class="unit">byte</span></td></tr>';
        print
            '<td class="field-name"><span class="datatitle">当日ログの解析</span></td>';
        print '<td class="q-field">'
            . ( ( $main::data{'realtime'} ) ? '可' : '禁止' ) . '</td>';
        print '</tr>';

        print '</tbody><tbody>';

        print '<tr>';
        print
            '<td class="field-name"><span class="datatitle">削除ファイル送付先</span></td>';
        print '<td class="field" colspan="3">';
        my $mailto = $this->escape( $main::data{'mailto'} );
        $mailto
            =~ s/[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/<br \/>/g;
        print $mailto;
        print '</td>';
        print '</tr>';
    }
    print '</tbody></table></fieldset>';
    print '<fieldset><legend>ページタイトル取得</legend>';
    if ( defined $master && $master ) {
        print $this->mk_select_menu(
            'data:getTitle',
            'ページタイトルを取得する対象を設定します。',
            '', $main::data{'getTitle'},
            ''       => 'ページタイトルを取得しない',
            'static' => 'CGI 以外のページタイトルを取得する',
            'all'    => '全てのページタイトルを取得する'
        );
        print
            ' <label><input type="checkbox" name="data:getShortTitle" id="data:getShortTitle" value="1"'
            . ( ( $main::data{'getShortTitle'} ) ? ' checked="checked"' : '' )
            . ' /> 短いタイトルを優先する</label>';
    }
    else {
        print(
            (          $main::data{'getTitle'}
                    && $main::data{'getTitle'} eq 'static'
            ) ? 'CGI 以外のページタイトルを取得する'
            : ( $main::data{'getTitle'} && $main::data{'getTitle'} eq 'all' )
            ? '全てのページタイトルを取得する'
            : 'ページタイトルを取得しない'
        );
        print( ( $main::data{'getShortTitle'} )
            ? '(短いタイトルを優先する)'
            : '' );
    }
    print '</fieldset>';
    print
        '<fieldset><legend><label for="data:whois">WHOIS URL</label></legend>';
    if ( defined $master && $master ) {
        print
            '<input type="text" name="data:whois" id="data:whois" size="80" value="'
            . $this->escape( $main::data{'whois'} )
            . '" title="ホスト情報を得るための WHOIS サービスを登録します。" />';
        print
            '<div class="note">ホスト名が入る場所を %s とします。</div>';
    }
    else {
        print $this->escape( $main::data{'whois'} );
    }
    print '</fieldset>';
    print
        '<fieldset><legend><label for="data:whois">リダイレクト URL</label></legend>';
    if ( defined $master && $master ) {
        print
            '<input type="text" name="data:redirect" id="data:redirect" size="80" value="'
            . $this->escape( $main::data{'redirect'} )
            . '" title="リンク元を隠すための URL リダイレクトサービスを登録します。" />';
        print
            '<div class="note">URL が入る場所を %u とします。</div>';
    }
    else {
        print $this->escape( $main::data{'redirect'} );
    }
    print '</fieldset>';
    return;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    #クラス登録
    Common::class_register( '0000', 'GeneralSetting' );

    #ツール登録
    $setting::jump_menu{'0000'} = '一般設定';
}

1;
