package LogSweeper;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# ログ整理モジュール
# LogSweeper.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( @access_log, $size, $removed );

    #ページ設定読み込み
    $this->import_db( $default::preference_dir . '/page',
        \%setting::page, 1, 'stat' );

    # 削除
    if (   $main::delete{'year'}
        && $main::delete{'mon'}
        && $main::delete{'mday'} )
    {
        no warnings qw(numeric);
        my $delete
            = sprintf( "%.4d", $main::delete{'year'} )
            . sprintf( "%.2d", $main::delete{'mon'} )
            . sprintf( "%.2d", $main::delete{'mday'} );
        ( $size, @access_log ) = $this->log_size;
        $removed = $this->log_sweeper( $main::option,
            grep( $_ <= $delete, @access_log ) );
        $removed += 0;
        push(
            @{ $this->{'dialog'} },
            $removed . 'のアクセスログを削除しました。'
        );
    }

    ( $size, @access_log ) = $this->log_size;

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print
        '<h1 title="アクセスログを一括削除します。">アクセスログの整理</h1>';

    if (@access_log) {
        $this->print_log_env( $size, \@access_log );
        $this->print_delete_setting( \@access_log );

    }
    else {
        print
            '<fieldset>現在のところ、アクセスログは一件も記録されていません。</fieldset>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# ログサイズ
sub log_size ($) {
    no warnings qw(numeric);
    my $this = shift;
    my ( @access_log, $size );

    # アクセスログディレクトリのサイズを求める
    opendir( LOG, $default::access_log );
    @access_log
        = grep( $_ =~ /[0-9]{8}\.csv$/
            && ( $size += -s $default::access_log . '/' . $_ ),
        readdir(LOG) );
    closedir(LOG);

    @access_log = sort { $a <=> $b || $a cmp $b } @access_log;
    return ( $size, @access_log );
}

######################################################################
# ログの状態
sub print_log_env ($$$) {
    my ( $this, $size, $log ) = @_;

    #解析条件テーブル
    my $this_table = Table->new;
    if ( ref $table::obj{ $setting::page{'stat'}{'infoTable'} } ) {
        %{$this_table}
            = %{ $table::obj{ $setting::page{'stat'}{'infoTable'} } };
        $this_table->{'fixed'} = 1;
    }
    else {
        $this_table->set(
            'fixed'      => 1,
            'tableclass' => 'tooltable',
            'trclass'    => 'odd,even',
            'class0'     => 'odd,even',
            'style0'     => 'text-align: left'
        );
    }

    #グラフモジュール読み込み
    require CMONOS::CGI_Lib::BarGraph;
    my $bargraph = BarGraph->new(
        \%setting::deco,
        $setting::page{'stat'}{'barColor'},
        $setting::page{'stat'}{'bgColor'},
        $setting::page{'stat'}{'barImage'},
        $setting::page{'stat'}{'bgImage'},
        $setting::page{'stat'}{'barWidth'}
    );

    print '<fieldset><legend>ログの状態</legend>';
    print $this_table->table;

    #ログ期間
    print $this_table->tr;
    print $this_table->th('アクセスログ保存期間');
    print $this_table->td( $this->log_span( $$log[0], $$log[ $#{$log} ] ) );
    print $this_table->_tr;

    #アクセスログスペース
    my $limit = $setting::general{'limit'};
    $limit = $default::limit if !defined $limit || !$limit;
    $limit = 100000          if !defined $limit || !$limit;
    print $this_table->tr;
    print $this_table->th('ログスペース');
    print $this_table->td( '最大 ' . $this->digit($limit) . ' byte' );
    print $this_table->_tr;

    #アクセスログサイズ
    print $this_table->tr;
    print $this_table->th('ログスペース使用量');
    print $this_table->td( $this->digit($size) . ' byte' );
    print $this_table->_tr;

    #アクセスログスペース使用率
    print $this_table->tr;
    print $this_table->th('ログスペース使用率');
    print $this_table->td( $bargraph->bargraph( $size, $limit, '%' ) );
    print $this_table->_tr;

    print $this_table->_table;
    print '</fieldset>';
}

sub log_span ($$$) {
    my ( $this, $from, $to ) = @_;
    $from = $1 if $from =~ /^([0-9]{8})/;
    $to   = $1 if $to =~ /^([0-9]{8})/;
    return $this->date(
        $from,
        $setting::page{'stat'}{'datelimit'},
        $setting::page{'stat'}{'datetype'}
        )
        . '〜'
        . $this->date(
        $to,
        $setting::page{'stat'}{'datelimit'},
        $setting::page{'stat'}{'datetype'}
        );
}

sub print_delete_setting ($$) {
    my ( $this, $log ) = @_;
    my ( $year, $mon, $mday, $year_end );

    ( $year, $mon, $mday ) = ( $1, $2, $3 )
        if $$log[0] =~ /^([0-9]{4})([0-9]{2})([0-9]{2})/;
    $year_end = $1 if $$log[ $#{$log} ] =~ /^([0-9]{4})([0-9]{2})([0-9]{2})/;

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'required' => \@setting::form
    );
    print '<fieldset class="delete">';
    print '<label><select name="delete:year">';
    foreach ( $year .. $year_end ) {
        print '<option value="'
            . $_ . '"'
            . ( ( $_ == $year ) ? ' selected="selected"' : '' ) . '>'
            . $_
            . '</option>';
    }
    print '</select>年</label> ';
    print '<label><select name="delete:mon">';
    foreach ( 1 .. 12 ) {
        print '<option value="'
            . $_ . '"'
            . ( ( $_ == $mon ) ? ' selected="selected"' : '' ) . '>'
            . $_
            . '</option>';
    }
    print '</select>月</label> ';
    print '<label><select name="delete:mday">';
    foreach ( 1 .. 31 ) {
        print '<option value="'
            . $_ . '"'
            . ( ( $_ == $mday ) ? ' selected="selected"' : '' ) . '>'
            . $_
            . '</option>';
    }
    print '</select>日</label>';
    print ' <input type="submit" value="以前のログを削除する" />';

    print
        '<div class="external-field"><label><input type="checkbox" name="option" value="1" /> 削除したファイルを指定されたアドレスに送信する</label></div>'
        if defined $setting::general{'mailto'} && $setting::general{'mailto'};
    print '</fieldset></form>';
}

######################################################################
# アクセスログの整理
sub log_sweeper ($$;@) {
    my ( $this, $option, @unlink ) = @_;
    my ( $file, $removed );
    return 0 if !@unlink;
    map( $_ = $default::access_log . '/' . $_, @unlink );

    # 削除ファイルのバックアップ
    if ( $option && $setting::general{'mailto'} ) {
        my ($msg);
        $msg = '【報告】' . $setting::general{'cginame'} . "\x0A";
        $msg .= "\x0A";
        $msg
            .= '管理人により古いアクセスログが削除されました。'
            . "\x0A";

        $msg .= "\x0A";
        $msg
            .= '※添付ファイルは削除されたアクセスログです。'
            . "\x0A";
        $msg .= 'ファイル名：日付（yyyymmdd）' . "\x0A";
        $msg
            .= '形　　　式：csv形式（カンマ区切りテキスト）'
            . "\x0A";
        $msg .= "\x0A";
        $msg .= '時間（hhmmss）,' . "\x0A";
        $msg .= 'IP（REMOTE_HOST）,' . "\x0A";
        $msg .= 'アクセスファイル,' . "\x0A";
        $msg .= 'クライアント環境（HTTP_USER_AGENT）,' . "\x0A";
        $msg .= 'クライアント主言語（HTTP_ACCEPT_LANGUAGE）,'
            . "\x0A";
        $msg .= 'リンク元（HTTP_REFERER）,' . "\x0A";
        $msg .= 'ウィンドウ内側の幅,' . "\x0A";
        $msg .= 'ウィンドウ内側の高さ,' . "\x0A";
        $msg .= 'スクリーン幅,' . "\x0A";
        $msg .= 'スクリーン高さ' . "\x0A";
        $msg .= "\x0A";

        unless ( ref $cgi::SendMail ) {
            require CMONOS::CGI_Lib::SendMail;
            $cgi::SendMail
                = SendMail->new( $env::sendmail, $env::master_email, '',
                $cgi::input, 1 );
        }

        $cgi::SendMail->send_mail(
            'Bcc'  => $setting::general{'mailto'},
            'From' => (
                ( $setting::general{'mastername'} )
                ? $setting::general{'mastername'} . ' <'
                    . $setting::general{'master'} . '>'
                : $setting::general{'master'}
            ),
            'Subject' => '古いアクセスログを削除しました。',
            'message' => $msg,
            'file'    => \@unlink
        );
    }
    $removed = 0;
    foreach $file (@unlink) {
        $removed++ if unlink $file;
    }
    return $removed;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 'a200', 'LogSweeper' );

    #メニュー登録
    $setting::jump_menu{'a200'} = 'ログの整理';

}

1;
