package BbArticle;
require CMONOS::Baobab::BbLeaf;
@ISA = qw(BbLeaf);
use strict;
use utf8;
######################################################################
# バオバブ記事表示モジュール
# BbArticle.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-08-13
######################################################################
# オブジェクト作成
sub new ($) {
    my $pkg  = shift;
    my $this = {};
    bless $this, $pkg;
    return $this;
}

######################################################################
# ページセッティング
sub css ($;$) {
    my ( $this, $list ) = @_;
    return ''
        if $setting::page{$setting::key}{'treeNoLine'}
        || !@$list
        || @$list > 1;
    return <<END;
ul.bb-tree, ul.bb-tree ul {
  background: transparent none;
  border: none;
  list-style: none;
  padding: 0;
  text-align: left;
  font-family: "ヒラギノ角ゴ Pro W3","ＭＳ　Ｐゴシック",sans-serif;
}
ul.bb-tree ul { margin: 0; }
ul.bb-tree li {
  background: transparent none;
  border: none;
  margin: 0;
  padding: 0;
  white-space: nowrap;
}
ul.bb-tree * { line-height: 97%; }
ul.bb-tree li span.bb-indent { font-size: 1.3em; }
ul.bb-tree li span.bb-list { font-size: 0.8em; }
ul.bb-tree li span.nobr:after {
  content: "";
  white-space: nowrap;
}
END
}

######################################################################
# 記事カウント
sub list_count ($$$$$$$) {
    my ( $this, $output, $db, $list, $p, $t, $n ) = @_;
    my ( $page, $e );
    $p = 0 if !defined $p || !$p;
    $this->{'count'} = $page = 0;
    foreach my $no (@$list) {
        if ( $n && !$output && !$e ) {
            $page++
                if $this->{'count'}
                && $setting::page{$setting::key}{'articleLimit'}
                && $this->{'count'}
                % $setting::page{$setting::key}{'articleLimit'} == 0;
            $e = 1 if $no eq $n;
        }
        $this->{'count'}++;
    }
    return ( !$output && $n ) ? $page : $p;
}

######################################################################
#記事リスト
sub print_articles ($$$$$$) {
    my ( $this, $handle, $output, $db, $list, $p ) = @_;
    $this->set_article_frame($output);
    $p = 0 if !$p;

    my ( $count, $from, $to );
    $from  = $p * $setting::page{$setting::key}{'articleLimit'};
    $to    = $from + $setting::page{$setting::key}{'articleLimit'};
    $count = 0;

    my ($n);
    foreach my $n (@$list) {
        next if $count < $from;
        last if $count >= $to;
        $this->print_article_box( $handle, $db, $n );
    }
    continue {
        $count++;
    }

    #ツリーナビ
    if (   @$list == 1
        && defined $$list[0]
        && defined $db->{'tree'}->{ $this->encode_utf8( $$list[0] ) } )
    {
        my @child = split(
            /,/,
            $this->decode_utf8(
                $db->{'tree'}->{ $this->encode_utf8( $$list[0] ) }
            )
        );
        $child[1] = $$list[0] if !defined $child[1] || $child[1] eq '';
        $child[2] = $$list[0] if !defined $child[2] || $child[2] eq '';
        if ( $$list[0] ne $child[1] || $child[3] ) {

            #フィールド条件操作
            if ( defined $setting::page{$setting::key}{'naviIcon'}
                && $setting::page{$setting::key}{'naviIcon'} )
            {
                $ThreadCategory::is_tree = 1;
                $ThreadStatus::is_tree   = 1;
                $BbGrader::is_tree       = 1;
                $ViewSelector::type      = 'tree';
            }

            require CMONOS::Baobab::BbTree;
            my $tree_obj = BbTree->new;
            $this->tie_db( $db, $tree_obj->required_fields );

            my $query
                = ( !$output )
                ? $this->amp_escape(
                $this->set_query( 'url', \@setting::form1 ) )
                : '';
            $query .= ($query) ? '&amp;' : '?';

            print $handle $this->frame_start(
                $setting::page{$setting::key}{'naviFrame'},
                $setting::page{$setting::key}{'naviWidth'},
                '',
                'baobab-tree-navi-frame',
                '',
                $output
                )
                if $setting::page{$setting::key}{'naviFrame'}
                || $setting::page{$setting::key}{'naviWidth'};
            if ( $setting::page{$setting::key}{'treeNaviEnabled'} ) {
                $tree_obj->print_articles( $handle, $output, $db,
                    [ $child[1] ],
                    0, $child[1], $$list[0], 1 );
            }
            else {
                print $handle '<ul id="baobab-tree-navi" class="box">';
                if ( $child[1] ne $$list[0] ) {
                    print $handle '<li class="root-article"><a href="'
                        . $cgi::path
                        . $query . 'n='
                        . $child[1]
                        . '&amp;v=listarticle">○</a>';
                    $tree_obj->print_article_info( $handle, $db, $child[1],
                        \$query );
                    print $handle '</li>';
                }
                if ( $child[2] ne $$list[0] && $child[1] ne $child[2] ) {
                    print $handle '<li class="parent-article"><a href="'
                        . $cgi::path
                        . $query . 'n='
                        . $child[2]
                        . '&amp;v=listarticle">△</a>';
                    $tree_obj->print_article_info( $handle, $db, $child[2],
                        \$query );
                    print $handle '</li>';
                }
                splice( @child, 0, 3 );
                if (@child) {
                    print $handle '<li class="child-article"><ul>';
                    foreach my $n (@child) {
                        print $handle '<li><a href="'
                            . $cgi::path
                            . $query . 'n='
                            . $n
                            . '&amp;v=listarticle">▽</a>';
                        $tree_obj->print_article_info( $handle, $db, $n,
                            \$query );
                        print $handle '</li>';
                    }
                    print $handle '</ul></li>';
                }
                print $handle '</ul>';
            }
            print $handle $this->frame_end(
                $setting::page{$setting::key}{'naviFrame'},
                $setting::page{$setting::key}{'naviWidth'},
                '',
                'baobab-tree-navi-frame',
                1
                )
                if $setting::page{$setting::key}{'naviFrame'}
                || $setting::page{$setting::key}{'naviWidth'};
        }
    }
}

1;
