package Editor;
require CMONOS::Baobab::BbCommon;
@ISA = qw(BbCommon);
use strict;
use utf8;
######################################################################
# バオバブ記事編集モジュール
# Editor.pm ver1.1.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-06-27
######################################################################
# 実行
sub run ($;$) {
    my $this = shift;
    my ( %parent, $thread_name );
    use Fcntl qw(:DEFAULT :flock);
    $main::mailing = '' if !defined $main::mailing || !$main::mailing;
    undef $main::n if $main::n && $main::n =~ /[^0-9]/;
    undef $main::parent if !$this->tree_enabled;
    $thread_name = ( ( $this->tree_enabled ) ? 'スレッド' : '記事' );
    $setting::general{'limit'} = 1000 if !$setting::general{'limit'};

    if (   $main::new_parent
        && $main::n
        && ( !$main::parent || $main::parent ne $main::new_parent )
        && $this->is_secure('tools') )
    {
        if ( $main::new_parent eq 'new' ) {
            $main::new_parent = $main::n;
        }
        elsif ( $main::new_parent =~ /[&\?]n=([0-9]{14})/ ) {
            $main::new_parent = $1;
        }
        elsif ( $main::new_parent =~ /[^0-9]/ ) {
            undef $main::new_parent;
        }
    }
    else {
        undef $main::new_parent;
    }

    #レコード番号判定
    if ( defined $main::parent && $main::parent ne '' ) {
        undef $main::n;
        my ( @info, %tree );
        tie( %tree, $env::dbm_file, $default::db . '/tree',
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %tree ) {
            @info = split(
                /,/,
                $this->decode_utf8(
                    $tree{ $this->encode_utf8($main::parent) } || ''
                )
            );
        }
        untie(%tree);
        if ( defined $info[0] && $info[0] ) {
            $this->print_dialog( 'editor',
                '削除された記事に返信をつけることはできません。'
            ) if $info[0] =~ /^d/;
            $thread::id
                = ( defined $info[1] && $info[1] ne '' )
                ? $info[1]
                : $main::parent;
        }
        else {
            $this->print_dialog( 'editor',
                '返信を投稿することができません。<br />指定された記事が存在しません。'
            );
        }
    }
    elsif ( defined $main::n && $main::n ne '' ) {
        my ( @info, %tree );
        tie( %tree, $env::dbm_file, $default::db . '/tree',
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %tree ) {
            @info = split( /,/,
                $this->decode_utf8( $tree{ $this->encode_utf8($main::n) } ) );
        }
        untie(%tree);
        if ( defined $info[0] && $info[0] ) {
            if ( $info[0] =~ /^dm/ && !$this->is_secure('tools') ) {
                $this->print_dialog( 'editor',
                    '指定された記事は管理人により削除されています。'
                );
            }
            elsif ( $info[0] =~ /^d/ && !$main::delete && !$main::rmthread ) {
                push(
                    @{ $this->{'dialog'} },
                    '一旦削除された記事を復活させることはマナー上お勧めできません。'
                );
            }
            $thread::id
                = ( defined $info[1] && $info[1] ne '' )
                ? $info[1]
                : $main::n;
            $main::parent = $info[2] if defined $info[2] && $info[2] ne '';
        }
        else {
            $this->print_dialog( 'editor',
                '指定された記事は存在しません。' );
        }
        if (   $main::new_parent
            && $main::new_parent eq $main::n
            && !$main::parent )
        {
            push(
                @{ $this->{'dialog'} },
                'この記事はルート記事です。'
            );
            undef $main::new_parent;
        }
    }

    #スレッド情報確認
    if ( defined $thread::id && $thread::id ) {
        my ( %tree, @info );
        tie( %tree, $env::dbm_file, $default::db . '/tree',
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %tree ) {
            @info = split(
                /,/,
                $this->decode_utf8(
                    $tree{ $this->encode_utf8($thread::id) }
                )
            );
        }
        untie(%tree);
        if ( defined $info[0] && $info[0] ) {
            $this->print_dialog( 'editor',
                'ルート記事が削除されたスレッドに返信をつけることはできません。'
                )
                if $info[0] =~ /^d/ && $setting::security_key eq 'inputchild';
        }
        else {
            $this->print_dialog( 'editor',
                'エラー：指定されたスレッドのスレッド情報が壊れています。'
            );
        }
        if ( $field::for{'category'} ) {
            my ( %thread_category, $category_id );
            tie( %thread_category, $env::dbm_file,
                $default::db . '/thread_category',
                O_RDONLY, $env::dbm_mode );
            if ( ref tied %thread_category ) {
                $category_id = $this->decode_utf8(
                    $thread_category{ $this->encode_utf8($thread::id) } );
            }
            untie(%thread_category);
            if (   defined $category_id
                && ref $thread::category{$category_id}
                && $thread::category{$category_id}{'closed'}
                && !$this->is_secure('closed') )
            {
                push(
                    @{ $this->{'dialog'} },
                    'このスレッドに投稿する権限がありません。'
                );
                $this->print_password_check_form('editor');
            }
            elsif (defined $category_id
                && ref $thread::category{$category_id}
                && $thread::category{$category_id}{'name'} )
            {
                $thread::category_name
                    = $thread::category{$category_id}{'name'};
            }
        }
        if ( $field::for{'status'} ) {
            my (%thread_status);
            tie( %thread_status, $env::dbm_file,
                $default::db . '/thread_status',
                O_RDONLY, $env::dbm_mode );
            if ( ref tied %thread_status ) {
                $thread::status = $this->decode_utf8(
                    $thread_status{ $this->encode_utf8($thread::id) } );
            }
            untie(%thread_status);
        }
        if ( $field::for{'name'} ) {
            my (%name);
            tie( %name, $env::dbm_file,
                $default::db . '/' . $field::obj{ $field::for{'name'} }->db,
                O_RDONLY, $env::dbm_mode );
            if ( ref tied %name ) {
                $thread::root = $this->decode_utf8(
                    $name{ $this->encode_utf8($thread::id) } );
            }
            untie(%name);
        }
    }

    #データ読み込み
    %parent = ();
    $main::a = '' if $main::delete || !%main::data || !defined $main::a;
    $this->dbm( \%parent )
        if ( !$main::a && $main::n ) || ( !$main::n && $main::parent );
    if (   $field::NAMECHECK
        && $field::for{'name'}
        && !defined $main::data{ $field::for{'name'} }{'str'}
        && $this->use_register )
    {
        $main::data{ $field::for{'name'} }{'str'}
            = $cgi::Register->email_to_name( $main::pswd, $main::email,
            $security::registered );
    }

    #更新権限チェック
    my ( $input_only_user, $input_is_valid );
    $input_only_user = $this->is_input_only_user;

    #データ削除
    if (   $main::delete
        && defined $main::n
        && $main::n ne ''
        && !$input_only_user
        && $this->is_secure( 'delete', 0, 'revise' ) )
    {
        my ( %tree, $is_deleted );

        #データ削除
        $this->delete_from_dbs( 0, $main::n );

        #データ削除者情報更新
        $this->record_info_modify;

        #ツリー更新
        tie(%tree, $env::dbm_file,
            $default::db . '/tree',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %tree ) {
            my @info = split( /,/,
                $this->decode_utf8( $tree{ $this->encode_utf8($main::n) } ) );
            $is_deleted = 1 if $info[0] =~ s/^dm?//;
            $info[0]++;
            $info[0]
                = ( ( $this->is_one_of_owners ) ? 'dm' : 'd' ) . $info[0];
            $tree{ $this->encode_utf8($main::n) }
                = $this->encode_utf8( join( ',', @info ) );
        }
        untie %tree;

        #スレッド記事数情報更新
        if ( !$is_deleted && $thread::id ne $main::n ) {
            my (%thread_count);
            tie(%thread_count, $env::dbm_file,
                $default::db . '/thread_count',
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );
            if ( ref tied %thread_count ) {
                $thread_count{ $this->encode_utf8($thread::id) }--
                    if $thread_count{ $this->encode_utf8($thread::id) } > 0;
            }
            untie %thread_count;
        }

        #添付ファイル削除
        use File::Path;
        rmtree "$default::file/$main::n";

        #RSS掃除
        $this->index_sweeper( [$main::n] );

        #デフォルトHTML書き出し
        $this->mk_default_page;

        #オープナーURL指定
        $this->set_opener_location;

        #ページヘッダ
        $this->print_page_header('editor');

        print '<p>指定された記事を削除しました。</p>';
        print '<div id="close-button">'
            . $this->close_button(
            $this->{'return_url'},
            $setting::page{'closeButton'},
            $setting::page{'backButton'}
            ) . '</div>';

        #ページフッタ
        $this->print_page_footer('editor');
        exit;

    }
    elsif ( defined $main::delete && $main::delete ) {
        if ( $input_only_user || !$this->is_secure( 'delete', 0, 'revise' ) )
        {
            push(
                @{ $this->{'dialog'} },
                '削除に失敗しました。削除権限がありません。'
            );
        }
        elsif ( !$input_only_user ) {
            push(
                @{ $this->{'dialog'} },
                '削除に失敗しました。対象記事が指定されていません。'
            );
        }

        #スレッド抹消
    }
    elsif ($main::rmthread
        && defined $main::n
        && $main::n ne ''
        && !$input_only_user )
    {

        if ($this->is_secure('remove')
            && ($this->is_one_of_owners
                || (   ref $main::data{ $field::for{'name'} }
                    && defined $main::data{ $field::for{'name'} }{'str'}
                    && $thread::root eq
                    $main::data{ $field::for{'name'} }{'str'}
                    && $this->is_secure_name )
            )
            )
        {
            my ( %tree, %general, @delete_list, $delete_count );

            #関連記事リスト
            tie( %tree, $env::dbm_file, "$default::db/tree", O_RDONLY,
                $env::dbm_mode );
            if ( ref tied %tree ) {
                push( @delete_list, $thread::id );
                $this->scan_my_thread( \%tree, \@delete_list, $thread::id );
            }
            untie %tree;
            $delete_count = @delete_list;

            #データ削除
            $this->delete_all_data( $default::db, @delete_list );
            $this->index_sweeper( \@delete_list );

            #スレッド記事数情報更新
            tie(%general, $env::dbm_file,
                $default::preference_dir . '/general',
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );
            if ( ref tied %general ) {
                $general{'count'} -= $delete_count;
            }
            untie %general;

            #添付ファイル削除
            use File::Path;
            foreach (@delete_list) {
                next if !defined $_ || $_ eq '';
                rmtree "$default::file/$_";
            }

            #デフォルトHTML書き出し
            $this->mk_default_page;

            #オープナーURL指定
            undef $main::n;
            undef $main::t;
            $this->set_opener_location;

            #ページヘッダ
            $this->print_page_header('editor');

            print '<p>指定された'
                . $thread_name
                . 'を削除しました。</p>';
            print '<div id="close-button">'
                . $this->close_button(
                $this->{'return_url'},
                $setting::page{'closeButton'},
                $setting::page{'backButton'}
                ) . '</div>';

            #ページフッタ
            $this->print_page_footer('editor');
            exit;

        }
        else {
            push(
                @{ $this->{'dialog'} },
                $thread_name
                    . '削除に失敗しました。削除権限がありません。'
            ) if !$input_only_user;
        }

    }
    elsif ( defined $main::rmthread && $main::rmthread ) {
        push(
            @{ $this->{'dialog'} },
            $thread_name
                . '削除に失敗しました。対象'
                . $thread_name
                . 'が指定されていません。'
        ) if !$input_only_user;

        #入力のチェック
    }
    elsif ( $main::a eq 'w' ) {
        $input_is_valid = 1;
        foreach my $o ( sort { $a <=> $b } keys %field::ORDER ) {
            my $id = $field::ORDER{$o};
            if (ref $field::obj{$id}
                && (!$field::obj{$id}->{'protected'}
                    || (   defined $main::n
                        && $main::n ne ''
                        && $this->is_secure(
                            'reviseProtectedFields', 0, 'revise'
                        )
                    )
                    || (( !defined $main::n || $main::n eq '' )
                        && $this->is_secure(
                            'inputProtectedFields', 0, 'input'
                        )
                    )
                )
                )
            {
                $input_is_valid = 0
                    if !$field::obj{$id}
                    ->is_valid_data( ( defined $main::n && $main::n ne '' )
                    ? $default::file . '/' . $main::n
                    : '' );
            }
        }

        #連続チェック
        if (   $setting::security_key ne 'revise'
            && $setting::general{'ipStopper'}
            && $setting::general{'latest'} )
        {
            my ( $mtime, $ip ) = split( /:/, $setting::general{'latest'} );
            if (   $ip
                && $mtime
                && $ip eq (
                           $ENV{'REMOTE_HOST'}
                        || $this->to_host( $ENV{'REMOTE_ADDR'} )
                )
                && time - $mtime <= $setting::general{'ipStopper'}
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    '連続投稿は制限されています。しばらくしてから投稿しなおしてください。'
                );
                $input_is_valid = 0;
            }
        }

        #EMAILチェック
        if (( !defined $main::n || $main::n eq '' )
            && ( !defined $setting::security{'revise'}{'passwordCheck'}
                || $setting::security{'revise'}{'passwordCheck'} ne
                'nocheck' )
            && $this->is_secure('revise')
            && $this->is_registered_email
            && !$this->is_secure_email
            && $this->email_field_enabled
            )
        {
            push(
                @{ $this->{'dialog'} },
                '登録メールアドレスがみつかりません。'
            );
            $main::data{ $field::for{'mail'} }{'str'} = $main::email
                if $field::for{'mail'}
                && $main::data{ $field::for{'mail'} }{'str'} ne '';
            $input_is_valid = 0;
        }

        #名前チェック
        if (   $field::NAMECHECK
            && $input_is_valid
            && !$this->is_secure_name(1) )
        {
            $input_is_valid = 0;
        }

        #入力エラー || キャンセル
        if ( !$input_is_valid || $main::cancel ) {

            #確認画面
        }
        elsif ( !defined $main::n || $main::n eq '' ) {
            if (!$setting::general{'skipConfirmation'}
                && ( $input_only_user
                    || ( $main::opener && $main::opener eq 'self' ) )
                && !$cgi::confirmation_disabled
                && !$main::ok
                )
            {

                #ページヘッダ
                $this->set_opener_location( 0, 1 );
                $this->print_page_header('editor');

                #フォーム
                print $this->form_start(
                    'key'    => $setting::security_key,
                    'name'   => 'input',
                    'method' => 'post',
                    'onsubmit' =>
                        'return cmonos.form.item(this).isNotSubmitted()',
                    'required' => \@setting::form
                );
                print '<input type="hidden" name="a" value="w" />';
                print '<input type="hidden" name="opener" value="'
                    . $this->escape($main::opener) . '" />'
                    if defined $main::opener && $main::opener;
                print '<input type="hidden" name="parent" value="'
                    . $this->escape($main::parent) . '" />'
                    if ( !defined $main::n || $main::n eq '' )
                    && $setting::security_key eq 'inputchild';

                print '<p id="form-confirmation">'
                    . ( ($default::english)
                    ? 'Confirmation.'
                    : '下記内容に問題がなければ「送信」ボタンを押してください。'
                    ) . '</p>';
                print '<table class="field-table"><tbody>';

                my (%database);
                tie(%{ $database{'thread_category'} },
                    $env::dbm_file, "$default::db/thread_category",
                    O_RDONLY, $env::dbm_mode
                ) if $field::for{'category'};
                tie( %{ $database{'thread_status'} },
                    $env::dbm_file, "$default::db/thread_status", O_RDONLY,
                    $env::dbm_mode )
                    if $field::for{'status'};
                tie( %{ $database{'tree'} },
                    $env::dbm_file, "$default::db/tree", O_RDONLY,
                    $env::dbm_mode );
                my $path = $default::file . '/'
                    . ( ( defined $main::n ) ? $main::n : '' );
                my $cookie_enabled = 0;

                foreach my $o ( sort { $a <=> $b } keys %field::ORDER ) {
                    my $id = $field::ORDER{$o};
                    if (ref $field::obj{$id}
                        && (!$field::obj{$id}->{'protected'}
                            || $this->is_secure(
                                'inputProtectedFields', 0, 'input'
                            )
                        )
                        && defined $main::data{$id}{'str'}
                        && ( ( defined $main::n && $main::n ne '' )
                            || $field::obj{$id}->{'class'} ne 'user_xtime' )
                        )
                    {
                        $cookie_enabled = 1
                            if $field::obj{$id}->cookie_enabled
                            && $field::obj{$id}->{'cookie'};
                        $field::obj{$id}
                            ->print_confirmation_form_table_row( $path,
                            $main::c, $main::n, \%database );
                    }
                }

                $this->untie_db( \%database );

                if ( defined $main::status && $main::status ) {
                    print
                        '<tr><td class="field-name"><span class="datatitle">'
                        . ( ( ref $field::obj{ $field::for{'status'} } )
                        ? $field::obj{ $field::for{'status'} }->name
                        : 'スレッド状況' )
                        . '</span></td><td class="field">解決済みにする<input type="hidden" name="status" value="'
                        . $this->escape($main::status)
                        . '" /></td></tr>';
                }
                if (   $main::mailing
                    && $setting::general{'resmail'}
                    && $this->is_secure('mail')
                    && $field::for{'mail'}
                    && ref $field::obj{ $field::for{'mail'} }
                    && $field::obj{ $field::for{'mail'} }->{'e-mail'}
                    && $field::obj{ $field::for{'mail'} }->{'class'}
                    =~ /email/
                    && $main::data{ $field::for{'mail'} }{'str'} ne '' )
                {
                    print '</tbody><tbody>';
                    print
                        '<tr class="tbody-start"><td class="field-name"><span class="datatitle">返信通知</span></td><td class="field">希望する<input type="hidden" name="mailing" value="1" /></td></tr>';
                }
                print '</tbody><tbody>';
                print
                    '<tr class="tbody-start"><td class="submit-box" colspan="2">';
                print '<input type="hidden" name="set_cookie" value="1" />'
                    if $cookie_enabled && $main::set_cookie;
                print '<input type="submit" name="cancel=1" value="'
                    . ( ($default::english) ? 'Cancel' : 'キャンセル' )
                    . '" class="cancel" />';
                print '<input type="submit" name="ok=1" value="'
                    . ( ($default::english) ? 'OK' : '送信' )
                    . '" class="save" />';
                print '</td></tr>';
                print '</tbody></table></form>';

                print '<div id="close-button">'
                    . $this->close_button(
                    $this->{'return_url'},
                    $setting::page{'closeButton'},
                    $setting::page{'backButton'}
                    ) . '</div>';

                #ページフッタ
                $this->print_page_footer('editor');
                return;

                #新規登録
            }
            else {
                $main::n = $this->new_record_id(1);
                if (   ( !defined $thread::id || $thread::id eq '' )
                    && ( !defined $main::parent || $main::parent eq '' ) )
                {
                    $thread::id   = $main::n;
                    $thread::root = $main::data{ $field::for{'name'} }{'str'};
                }
                $this->{'new_data'} = 1;
            }
        }

        #カウント
    }
    elsif ( !defined $main::n || $main::n eq '' ) {
        $this->new_record_id;

    }

    #書き込み/新規入力の告知
    if (   $input_is_valid
        && !$main::cancel
        && ( !$input_only_user || $this->{'new_data'} ) )
    {
        $this->dbm( \%parent );

        if (   ( $setting::general{'newmail'} && $this->{'new_record'} )
            || ( $main::parent && $setting::general{'resmail'} ) )
        {

            #メール設定読み込み
            $this->import_db( $default::preference_dir . '/mail',
                \%setting::mail, 1 );

            #返信通知
            $this->delivery_res_notice_mail
                if $main::parent && $setting::general{'resmail'};

            #投稿通知
            $this->notice_new_record_for_master
                if $setting::general{'newmail'} && $this->{'new_record'};
            undef %setting::mail;
        }

        #デフォルトHTML書き出し
        $this->mk_default_page;

        #新しいウィンドウを開かずに書き込んでいる場合
        if ( $main::opener && $main::opener eq 'self' ) {
            $this->mkcookie;
            $this->jump(
                $this->full_path($cgi::path) . '?t='
                    . (
                    ( !$main::parent )
                    ? $main::n
                    : ( $main::t || $main::parent )
                    )
                    . $this->set_query(
                    'url+', [ 'n', 'v', 'p', @setting::form1 ]
                    )
            );
            return;
        }
    }

    #データ修正パスワードチェック
    if ( defined $main::n && $main::n ne '' && $input_only_user ) {

        #新規入力
        if ( defined $this->{'new_data'} && $this->{'new_data'} ) {
            my ( $title, $name, %thanks );
            if ( defined $field::for{'name'} && $field::for{'name'} ) {
                $name = $this->detag_text(
                    $field::obj{ $field::for{'name'} }->usual_format(
                        $main::data{ $field::for{'name'} }{'str'}
                    )
                );
            }
            $name = ($default::english) ? 'Anonymous' : '匿名'
                if !defined $name || $name eq '';
            if ( defined $field::for{'title'} && $field::for{'title'} ) {
                $title = $this->detag_text(
                    $field::obj{ $field::for{'title'} }->usual_format(
                        $main::data{ $field::for{'title'} }{'str'}
                    )
                );
            }
            $title = ($default::english) ? 'This item' : 'これ'
                if !defined $title || $title eq '';

            #入力お礼設定読み込み
            $this->import_db( $default::preference_dir . '/thanks',
                \%thanks );

            #書き込みお礼
            if ( defined $thanks{'thanks'} && $thanks{'thanks'} ne '' ) {
                $thanks{'thanks'} =~ s/<name>/$name/igs;
                $thanks{'thanks'} =~ s/<title>/$title/igs;
            }
            if ( defined $thanks{'thankstitle'}
                && $thanks{'thankstitle'} ne '' )
            {
                $thanks{'thankstitle'} =~ s/<name>/$name/igs;
                $thanks{'thankstitle'} =~ s/<title>/$title/igs;
                $this->push_window_title( $thanks{'thankstitle'} );
                $setting::page{'editor'}{'titleText'}
                    =~ s/((<[^>]+>)*)[^<]+/$1$thanks{'thankstitle'}/s
                    if defined $setting::page{'editor'}{'titleText'}
                    && $setting::page{'editor'}{'titleText'} ne '';
            }
            $this->set_opener_location;
            $this->print_page_header('editor');
            if ( defined $thanks{'thanks'} && $thanks{'thanks'} ne '' ) {
                $this->tag_fix( \$thanks{'thanks'}, 2, 0, 0, 1 );
                print $thanks{'thanks'};
            }
            else {
                print '<p>登録されました。</p>';
            }
            print '<div id="close-button">'
                . $this->close_button(
                $this->{'return_url'},
                $setting::page{'closeButton'},
                $setting::page{'backButton'}
                ) . '</div>';
            $this->print_page_footer('editor');
            return;

#特殊なケース
#} elsif (!$this->email_field_enabled) {
#	$this->print_dialog($setting::key,'メールアドレスフィールドが存在しません。');

            #パスワードチェック
        }
        elsif ( defined $main::a && $main::a eq 'w' ) {
            $this->print_password_check_form(
                'editor',
                (   ( $this->{'user_address'} ) ? $this->{'user_address'}
                    : 'none'
                ),
                (   ( $this->{'need_new_pswd'} ) ? $this->{'need_new_pswd'}
                    : 0
                )
            );
        }
        else {
            $this->print_password_check_form('editor');
        }
    }

    #ページヘッダ
    $this->set_opener_location( 0,
        ( ( $input_is_valid && !$main::cancel ) ? 0 : 1 ) );
    $this->print_page_header('editor');

    #フォーム
    print $this->form_start(
        'key'      => $setting::security_key,
        'name'     => 'input',
        'multi'    => $cgi::enctype_multipart,
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="w" />';
    print '<input type="hidden" name="opener" value="'
        . $this->escape($main::opener) . '" />'
        if defined $main::opener && $main::opener;
    print '<input type="hidden" name="parent" value="'
        . $this->escape($main::parent) . '" />'
        if ( !defined $main::n || $main::n eq '' )
        && $setting::security_key eq 'inputchild';

    #ファイル格納ディレクトリ
    my $path
        = $default::file . '/' . ( ( defined $main::n ) ? $main::n : '' );
    my $parent_path
        = ( defined $main::parent )
        ? $default::file . '/' . $main::parent
        : '';

    print '<table class="field-table"><tbody>';
    my $cookie_enabled = 0;
    foreach my $o ( sort { $a <=> $b } keys %field::ORDER ) {
        my $id = $field::ORDER{$o};
        if (ref $field::obj{$id}
            && (!$field::obj{$id}->{'protected'}
                || (   defined $main::n
                    && $main::n ne ''
                    && $this->is_secure( 'reviseProtectedFields', 0,
                        'revise' ) )
                || ( ( !defined $main::n || $main::n eq '' )
                    && $this->is_secure( 'inputProtectedFields', 0, 'input' )
                )
            )
            && ( ( defined $main::n && $main::n ne '' )
                || $field::obj{$id}->{'class'} ne 'user_xtime' )
            )
        {
            $cookie_enabled = 1
                if $field::obj{$id}->cookie_enabled
                && $field::obj{$id}->{'cookie'};
            print $field::obj{$id}
                ->form( $main::n, $parent{$id}, $path, $parent_path );
        }
    }

    print '</tbody><tbody>';
    if ( $main::n && $this->is_secure('tools') ) {
        print '<tr>';
        print
            '<td class="field-name"><label for="new_parent">親記事変更</label></td><td class="field"><input type="text" size="40" name="new_parent" id="new_parent" value="'
            . $this->escape($main::new_parent)
            . '" title="記事番号または記事 URL を入力します。" placeholder="記事番号または記事 URL を入力します。" /></td>';
        print '</tr>';
        print '</tbody><tbody>';
    }
    print '<tr class="tbody-start">';
    print '<td class="submit-box" colspan="2">';
    print '<div id="submit-button">' if defined $main::n && $main::n ne '';
    if (   $setting::general{'resmail'}
        && $this->is_secure('mail')
        && $field::for{'mail'}
        && ref $field::obj{ $field::for{'mail'} }
        && $field::obj{ $field::for{'mail'} }->{'e-mail'}
        && $field::obj{ $field::for{'mail'} }->{'class'} =~ /email/ )
    {
        print '<label><input type="checkbox" name="mailing" value="1"'
            . ( ($main::mailing) ? ' checked="checked"' : '' )
            . ' />返信通知</label> ';
    }
    if ( defined $cookie_enabled && $cookie_enabled ) {
        print
            '<label title="同じ内容を入力することが多いいくつかの項目を記憶します。"><input type="checkbox" name="set_cookie" value="1"'
            . ( ( $cookie::read{$cookie::name} || $main::set_cookie )
            ? ' checked="checked"'
            : '' )
            . ' />情報を記憶する</label> ';
    }
    print
        '<input type="submit" name="submit_button" value="送信" class="save" />';
    print '</div>' if defined $main::n && $main::n ne '';

    if ( defined $main::n && $main::n ne '' ) {
        print '<div id="delete-button">';
        if ($this->is_secure('remove')
            && ($this->is_one_of_owners
                || (   ref $main::data{ $field::for{'name'} }
                    && defined $main::data{ $field::for{'name'} }{'str'}
                    && $thread::root eq
                    $main::data{ $field::for{'name'} }{'str'}
                    && $this->is_secure_name )
            )
            )
        {
            print '<input type="submit" name="rmthread=1" value="'
                . $thread_name
                . '削除" onclick="return confirm(\'この操作は取り消せません。本当にこの'
                . $thread_name
                . 'を削除しますか？\')" />';
        }
        print
            '<input type="submit" name="delete=1" value="削除" onclick="return confirm(\'この操作は取り消せません。本当に削除しますか？\')" />'
            if $this->tree_enabled
            && $this->is_secure( 'delete', 0, 'revise' );
        print '</div>';
    }

    print '</td></tr></tbody></table></form>';
    print '<div id="close-button">'
        . $this->close_button(
        $this->{'return_url'},
        $setting::page{'closeButton'},
        $setting::page{'backButton'}
        ) . '</div>';

    #ページフッタ
    $this->print_page_footer('editor');
}

########################################################################
#スレッド作成者かどうか
sub is_secure_name ($;$) {
    my ( $this, $input_is_valid ) = @_;
    return $this->{'is_secure_name'}
        if !$input_is_valid && defined $this->{'is_secure_name'};
    if (   $field::NAMECHECK
        && $field::for{'name'}
        && $main::data{ $field::for{'name'} }{'str'} ne ''
        && $this->use_register )
    {
        if (   $main::shift_flag
            && $this->{'old_pswd'}
            && $this->{'old_email'} )
        {
            if ( defined $input_is_valid && $input_is_valid ) {
                if ($cgi::Register->remove(
                        $this->{'old_pswd'}, $this->{'old_email'}
                    )
                    )
                {
                    push(
                        @{ $this->{'dialog'} },
                        'これまで登録されていたメールアドレス（'
                            . $this->{'old_email'}
                            . '）を抹消しました。'
                    );
                    $cgi::Register->success_dialog;
                }
            }
            else {
                push( @setting::form, 'shift_flag', 'old_pswd', 'old_email' );
            }
        }
        my ( $name, $fontcolor );
        $name
            = ( ref $main::data{ $field::for{'name'} }
                && defined $main::data{ $field::for{'name'} }{'str'} )
            ? $main::data{ $field::for{'name'} }{'str'}
            : '';
        $fontcolor = $1 if $name =~ s/<>([#0-9a-zA-z]{3,10})\Z//s;
        if (   defined $name
            && $name ne ''
            && !$this->is_same_name_and_email( $main::n, $name ) )
        {
            if ((      !defined $main::n
                    || $main::n eq ''
                    || $this->is_secure_email
                )
                && $cgi::Register->is_available_name(
                    $main::pswd, $main::email,
                    $name,       $security::registered
                )
                )
            {
                push( @{ $this->{'dialog'} },
                    $cgi::Register->success_dialog );
            }
            else {
                my @msg = $cgi::Register->error_dialog;
                push(
                    @{ $this->{'dialog'} },
                    (   (@msg)
                        ? @msg
                        : '認証に必要なメールアドレスが変更されました。'
                    )
                );
                $main::data{ $field::for{'name'} }{'str'}
                    = $name
                    . ( ( defined $fontcolor && $fontcolor ne '' )
                    ? '<>' . $fontcolor
                    : '' );
                $this->{'is_secure_name'} = 0;
                if ( $input_is_valid && $this->{'old_email'} ) {
                    return 1;
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        'この書き込みは確定していません。'
                    );
                    return 0;
                }
            }
        }
    }
    $this->{'is_secure_name'} = 1;
    return 1;
}

sub is_same_name_and_email ($$$) {
    my ( $this, $n, $name ) = @_;
    return 0 if !defined $n || $n eq '' || !$this->{'is_admin'};
    my ( @check_fields, %db, $is_not_same );
    @check_fields = @{ $field::for{'mail_address'} }
        if ref $field::for{'mail_address'};
    return 1 if !@check_fields;
    push( @check_fields, $field::for{'name'} );
    foreach my $id (@check_fields) {
        if ( ref $field::obj{$id} ) {
            tie( %db, $env::dbm_file,
                $default::db . '/' . $field::obj{$id}->db,
                O_RDONLY, $env::dbm_mode );
            $is_not_same = 1
                if !ref tied %db
                || ( defined $db{ $this->encode_utf8($n) }
                && $this->decode_utf8( $db{ $this->encode_utf8($n) } ) ne
                $main::data{$id}{'str'} );
            untie(%db);
            return 0 if $is_not_same;
        }
    }
    $this->{'keep_original_user_info'} = 1;
    return 1;
}

########################################################################
#デフォルトページ作成
sub mk_default_page ($) {
    my $this = shift;
    my $index = ( ($default::file) ? $default::file : '.' ) . '/index.html';
    if (   $setting::general{'html'}
        && $setting::security{'browse'}{'passwordCheck'} eq 'nocheck'
        && $setting::security{'secret'}{'passwordCheck'} eq 'nocheck'
        && $setting::security{'closed'}{'passwordCheck'} eq 'nocheck' )
    {
        local ( @setting::form, $cgi::action, %main::search, $main::q );
        require CMONOS::Baobab::Viewer;
        $cgi::action = Common->new('viewer');
        $cgi::action->run($index);
    }
    else {
        $this->write_jump_page($index);
    }
}

#ジャンプ
sub write_jump_page ($$) {
    my ( $this, $path ) = @_;
    local (*OUT);
    if ( open( OUT, ">>$path" ) ) {
        binmode OUT, ":utf8";
        flock( OUT, LOCK_EX );
        truncate( OUT, 0 );
        seek( OUT, 0, 0 );
        print OUT '<html><head><meta http-equiv="Refresh" content="0; URL='
            . $cgi::path
            . ( ($main::c) ? '?c=' . $main::c : '' )
            . '" /></head><body></body></html>';
        close(OUT);
    }
    else {
        push(
            @{ $this->{'dialog'} },
            'デフォルトページの書き込みに失敗しました。'
        );
    }
}

######################################################################
# オープナーのURL
sub set_opener_location ($;$$) {
    my ( $this, $nonum_flag, $url_only ) = @_;
    my ( @query, $v );
    $v = $main::v || 'listthread';
    push( @query, 'v', 'p', @setting::form1 );
    if ( $main::opener && $main::opener eq 'self' ) {
        push( @query, 't' )
            if !$nonum_flag && ( !$main::v || $main::v ne 'list' );
        $this->{'return_url'}
            = $cgi::path
            . (
            ( !$nonum_flag && $main::parent ) ? '?n=' . $main::parent : '?' )
            . $this->set_query( 'url+', \@query );
    }
    else {
        if ( !$nonum_flag ) {
            push( @query, 'n' );
            push( @query, 't' ) if !$main::v || $main::v ne 'list';
        }
        $this->{'return_url'}
            = $cgi::path
            . ( ( $main::opener && $main::opener eq 'tree' )
            ? '?k=tree&m=TreeView'
            : '?' )
            . $this->set_query( 'url+', \@query );
    }
    return if $url_only;
    $this->set_onload( 'cmonos.window.refreshOpener(\''
            . $this->escape( $this->{'return_url'} )
            . '\',true)' );
}

######################################################################
# 新規入力のみのユーザーかどうか
sub is_input_only_user ($) {
    my $this = shift;
    if ( !defined $main::n || $main::n eq '' ) {
        return (
            (   !defined $setting::security{'revise'}{'passwordCheck'}
                    || $setting::security{'revise'}{'passwordCheck'} ne
                    'nocheck'
            )
                && ( !$this->is_secure('revise')
                || ( !$this->is_one_of_owners && !$this->email_field_enabled )
                )
        ) ? 1 : 0;
    }
    else {
        if ( defined $setting::security{'revise'}{'passwordCheck'}
            && $setting::security{'revise'}{'passwordCheck'} eq 'nocheck' )
        {
            $this->{'is_admin'} = 1
                if $this->is_secure('revise') && $this->is_one_of_owners;
            $this->is_authorized_email;
        }
        else {
            $this->{'is_admin'} = 1
                if !$this->is_authorized_email
                && $this->is_secure('revise')
                && $this->is_one_of_owners;
        }
        return (
            (   !defined $setting::security{'revise'}{'passwordCheck'}
                    || $setting::security{'revise'}{'passwordCheck'} ne
                    'nocheck'
            )
                && ( !$this->is_secure_email || !$this->is_secure('revise') )
        ) ? 1 : 0;
    }
}

sub is_secure_email ($) {
    my $this = shift;
    if ( defined $this->{'is_secure_email'} ) {
        return $this->{'is_secure_email'};
    }
    elsif ($this->is_master
        || $this->is_owner
        || $this->is_one_of_owners
        || $this->is_staff )
    {
        $main::set_cookie_id = 1
            if !$env::session_enabled
            && $cookie::read{ $this->login_cookie_name };
        $this->{'is_secure_email'} = 1;
    }
    elsif ( $this->is_authorized_email ) {
        $this->{'is_secure_email'} = 1;
    }
    else {
        $this->{'is_secure_email'} = 0;
    }
    return $this->{'is_secure_email'};
}

sub email_field_enabled ($) {
    my $this = shift;
    $this->is_authorized_email if !defined $this->{'email_field_enabled'};
    return $this->{'email_field_enabled'};
}

sub is_authorized_email ($) {
    my $this = shift;
    if ( defined $this->{'is_authorized_email'} ) {
        return $this->{'is_authorized_email'};
    }
    else {
        $this->{'is_authorized_email'} = 0;
        $this->{'email_field_enabled'} = 0;
        $this->{'user_address'}        = '';
        if ( ref $field::for{'mail_address'}
            && @{ $field::for{'mail_address'} } )
        {
            my ( @user_address, @new_user_address );
            $this->{'email_field_enabled'} = 1;
            foreach my $id ( @{ $field::for{'mail_address'} } ) {
                push( @new_user_address, $main::data{$id}{'str'} )
                    if $main::data{$id}{'str'};
                if ( defined $main::n && $main::n ne '' ) {
                    if ( ref $field::obj{$id} ) {
                        my (%db);
                        tie( %db, $env::dbm_file,
                            $default::db . '/' . $field::obj{$id}->db,
                            O_RDONLY, $env::dbm_mode );
                        if ( ref tied %db ) {
                            push(
                                @user_address,
                                $this->decode_utf8(
                                    $db{ $this->encode_utf8($main::n) }
                                )
                            ) if $db{ $this->encode_utf8($main::n) };
                        }
                        untie(%db);
                    }
                }
                elsif ( $main::data{$id}{'str'} ) {
                    push( @user_address, $main::data{$id}{'str'} );
                }
            }
            @user_address     = $this->get_address(@user_address);
            @new_user_address = $this->get_address(@new_user_address);
            if ( @user_address && @new_user_address ) {
                my $user_address_count
                    = grep( defined $_
                        && defined $main::email
                        && $_ eq $main::email,
                    @user_address );
                my $new_user_address_count
                    = grep( defined $_
                        && defined $main::email
                        && $_ eq $main::email,
                    @new_user_address );

#データベースにも新しいデータにも登録されたE-Mailがある
                if (   $main::email
                    && $user_address_count
                    && $new_user_address_count
                    && ($security::registered
                        || ($this->use_register
                            && $cgi::Register->is_registered(
                                $main::pswd, $main::email
                            )
                        )
                    )
                    )
                {
                    $security::registered = 1;
                    $main::set_cookie_id  = 1
                        if !$env::session_enabled
                        && $cookie::read{ $this->login_cookie_name };
                    $this->{'is_authorized_email'} = 1;
                    $this->{'user_address'}        = $main::email;

#データベースには登録されたE-Mailがあるが新しいデータにない
                }
                elsif (
                       $user_address_count
                    && !$new_user_address_count
                    && $this->use_register
                    && $cgi::Register->is_registered(
                        $main::pswd, $main::email
                    )
                    )
                {
                    $this->{'need_new_pswd'} = 1;
                    if (   $main::new_email
                        && $main::new_pswd
                        && grep( defined $_ && $_ eq $main::new_email,
                            @new_user_address )
                        && $cgi::Register->is_registered(
                            $main::new_pswd, $main::new_email
                        )
                        )
                    {
                        $this->{'is_authorized_email'} = 1;
                        $this->{'old_email'}           = $main::email;
                        $this->{'old_pswd'}            = $main::pswd;
                        $this->{'user_address'}        = $main::email
                            = $main::new_email;
                        $main::pswd = $main::new_pswd;
                        undef $main::new_email;
                        undef $main::new_pswd;
                        if ($env::session_enabled) {

                            if ( $this->use_session ) {
                                $session::id = $cgi::Session->set_user_info(
                                    $main::email, $main::pswd, $session::id );
                                $session::is_changed = 1;
                            }
                        }
                        elsif ( $cookie::read{ $this->login_cookie_name } ) {
                            $main::set_cookie_id = 1;
                        }
                    }
                    else {
                        if ($main::new_email
                            && !grep( defined $_ && $_ eq $main::new_email,
                                @new_user_address )
                            )
                        {
                            push(
                                @{ $this->{'dialog'} },
                                '入力された項目に新しいメールアドレスがみつかりません。'
                            );
                        }
                        elsif ($main::new_pswd) {
                            push(
                                @{ $this->{'dialog'} },
                                '新しいパスワードを確認してください。'
                            );
                        }
                        elsif (
                            $setting::security{'revise'}{'passwordCheck'} eq
                            'nocheck' )
                        {
                            $this->{'is_authorized_email'} = 1;
                        }
                        $this->{'user_address'}
                            = (@new_user_address)
                            ? $new_user_address[0]
                            : $user_address[0];
                    }

                    #その他
                }
                else {
                    if ( $setting::security{'revise'}{'passwordCheck'} eq
                        'nocheck' )
                    {
                        $this->{'is_authorized_email'} = 1;
                    }
                    $this->{'user_address'}
                        = (@new_user_address)
                        ? $new_user_address[0]
                        : $user_address[0];
                }
            }
            elsif (
                $setting::security{'revise'}{'passwordCheck'} eq 'nocheck' )
            {
                $this->{'is_authorized_email'} = 1;
            }
        }
        elsif ( $setting::security{'revise'}{'passwordCheck'} eq 'nocheck' ) {
            $this->{'is_authorized_email'} = 1;
        }
    }
    return $this->{'is_authorized_email'};
}

######################################################################
# データ削除
sub delete_from_dbs ($$@) {
    my ( $this, $all_flag, @keys ) = @_;
    my ( @files, %db, %database );
    local (*DIR);

    opendir( DIR, $default::db );
    @files = readdir(DIR);
    closedir(DIR);

    foreach my $db (@files) {
        next
            if $db =~ /^\./
            || $db =~ /\.htm[l]?$/i
            || !-f $default::db . '/' . $db;
        $db =~ s/\.[^\/\.]+$//;
        next if !$all_flag && $db =~ /[^0-9]/;
        $db{$db} = 1;
    }
    while ( my ($db) = each %db ) {
        tie( %database, $env::dbm_file, $default::db . '/' . $db,
            O_RDWR, $env::dbm_mode );
        if ( ref tied %database ) {
            foreach my $k (@keys) {
                utf8::encode($k) if utf8::is_utf8($k);
                delete $database{$k};
            }
        }
        untie %database;
    }
}

######################################################################
# カウント
sub new_record_id ($;$) {
    no warnings qw(untie);
    my ( $this, $count_flag ) = @_;
    my ( %time, $count, $n );
    tie( %time, $env::dbm_file, $default::db . '/mtime',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %time ) {
        my ($dmy);
        while ( ($dmy) = each %time ) {
            $count++;
        }
    }

    #レコード数制限オーバー
    if ( $count && $count >= $setting::general{'limit'} ) {

  #最も古いスレッドを消去して新しいレコードを追加する
        if (   $setting::general{'action'} eq 'byMtime'
            || $setting::general{'action'} eq 'byCtime' )
        {
            $n = $this->time_id;
            $n = $this->new_number( \%time, $n );
            untie %time;

            my (%thread_time, $k,       $v,    $key,
                $value,       $del_top, %tree, %general,
                @delete_list, $delete_count
            );
            tie(%thread_time,
                $env::dbm_file,
                $default::db
                    . (
                    ( $setting::general{'action'} eq 'byCtime' )
                    ? '/thread_ctime'
                    : '/thread_mtime'
                    ),
                O_RDONLY,
                $env::dbm_mode
            );
            while ( ( $k, $v ) = each %thread_time ) {
                ( $key, $value ) = ( $k, $v )
                    if ( !defined $thread::id || $k != $thread::id )
                    && ( !defined $key || $v < $value );
            }
            untie %thread_time;

            #書き込み拒否
            if ( !defined $key || !$key ) {
                $this->print_dialog(
                    'editor',
                    (   ( $setting::general{'cginame'} ne '' )
                        ? $setting::general{'cginame'}
                        : 'このBBS'
                        )
                        . 'には書き込めません。<br />記事数が '
                        . $setting::general{'limit'}
                        . ' 件以内に制限されています。'
                );
            }
            $del_top = $key;

            #関連記事リスト
            tie( %tree, $env::dbm_file, "$default::db/tree", O_RDONLY,
                $env::dbm_mode );
            if ( ref tied %tree ) {
                push( @delete_list, $del_top );
                $this->scan_my_thread( \%tree, \@delete_list, $del_top );
            }
            untie %tree;

            $delete_count = @delete_list;
            $count -= $delete_count;

            #データ削除
            $this->delete_all_data( $default::db, @delete_list );
            $this->index_sweeper( \@delete_list );

            #スレッド記事数情報更新
            if ( !$count_flag ) {
                tie(%general, $env::dbm_file,
                    $default::preference_dir . '/general',
                    O_RDWR | O_CREAT,
                    $env::dbm_mode
                );
                if ( ref tied %general ) {
                    $general{'count'} = $count;
                }
                untie %general;
            }

            #添付ファイル削除
            use File::Path;
            foreach (@delete_list) {
                next if !defined $_ || $_ eq '';
                rmtree "$default::file/$_";
            }

#push(@{$this->{'dialog'}},'記事数が ' . $setting::general{'limit'} . ' 件を超えたため、' . (($setting::general{'action'} eq 'byCtime') ? '投稿日時' : '最終更新日時') . 'が最も古いスレッドを削除しました。');

#デフォルトHTML書き出し
#my $index = (($default::file) ? $default::file : '.') . '/index.html';
#if ($setting::general{'html'} && $setting::security{'browse'}{'passwordCheck'} eq 'nocheck' && $setting::security{'secret'}{'passwordCheck'} eq 'nocheck' && $setting::security{'closed'}{'passwordCheck'} eq 'nocheck') {
#	require CMONOS::Baobab::Viewer;
#	my $html = Common->new('viewer');
#	$html->run($index);
#}

            $this->set_opener_location(1);

  #新しいカテゴリーを作成して新しいレコードを追加する
        }
        elsif (defined $default::subdivided
            && $default::subdivided
            && defined $setting::general{'action'}
            && $setting::general{'action'} eq 'make' )
        {
            untie %time;

            #スレッド入力権限がなければ書き込み拒絶
            if ( !$this->is_secure('inputthread') ) {
                $this->print_dialog(
                    'editor',
                    (   ( $setting::general{'cginame'} ne '' )
                        ? $setting::general{'cginame'}
                        : 'このBBS'
                        )
                        . 'には書き込めません。<br />記事数が '
                        . $setting::general{'limit'}
                        . ' 件以内に制限されています。'
                );

            }
            else {
                $setting::security_key = 'inputthread';
                $thread::id            = $main::n;
                undef $thread::root;
                undef $main::parent;

                my ($oname, $name,  $label, $d, $new_name,
                    $c,     $new_c, $new_n, $new_e
                );
                $name = $oname = $setting::general{'cginame'};
                $label = $setting::category{$main::c}{'label'};
                $c     = $main::c;
                $d     = $1 if $c =~ s/[\x09\x0A\x0C\x0D\x20]*([0-9]+)$//;
                $d     = 1 if !$d;
                while ( $count >= $setting::general{'limit'} ) {
                    $d++;
                    $new_name = $name;
                    $new_name =~ s/[\x09\x0A\x0C\x0D\x20]*([0-9]+)$//;
                    $new_name .= ' ' . $d;
                    $new_c = $c . $d;
                    if ( -d $default::master_preference_dir . '/' . $new_c ) {
                        $new_e = 1;
                        my ( %general, $original_c );
                        tie(%general,
                            $env::dbm_file,
                            "$default::master_preference_dir/$new_c/general",
                            O_RDONLY,
                            $env::dbm_mode
                        );
                        if ( ref tied %general ) {
                            $original_c
                                = $this->decode_utf8( $general{'original'} );
                            if ((      $setting::general{'original'}
                                    && $original_c eq
                                    $setting::general{'original'}
                                )
                                || $original_c eq $main::c
                                )
                            {
                                $setting::general{'limit'}
                                    = $this->decode_utf8( $general{'limit'} )
                                    if $general{'limit'};
                                $count
                                    = ( $general{'count'} )
                                    ? $this->decode_utf8( $general{'count'} )
                                    : 0;
                            }
                        }
                        untie(%general);
                        if ((      $setting::general{'original'}
                                && $original_c eq
                                $setting::general{'original'}
                            )
                            || $original_c eq $main::c
                            )
                        {
                            my (%mtime);
                            tie(%mtime,
                                $env::dbm_file,
                                "$default::master_db_dir/$new_c/mtime",
                                O_RDONLY,
                                $env::dbm_mode
                            );
                            if ( ref tied %mtime ) {
                                if ( $count < $setting::general{'limit'} ) {
                                    $new_n = $this->time_id;
                                    $new_n = $this->new_number( \%mtime,
                                        $new_n );
                                    untie %mtime;
                                    last;
                                }
                                untie %mtime;
                            }
                            else {
                                untie %mtime;
                                last;
                            }
                        }
                    }
                    else {
                        $count = 0;
                        $new_e = 0;
                        last;
                    }
                }
                $this->cp_category(
                    (   ( $setting::general{'original'} )
                        ? $setting::general{'original'}
                        : $main::c
                    ),
                    $new_c,
                    $new_name
                ) if !$new_e;
                $main::c = $new_c;
                $n = ($new_n) ? $new_n : $this->time_id;
                push(
                    @{ $this->{'dialog'} },
                    '記事数が '
                        . $setting::general{'limit'}
                        . ' 件以内に制限されているため、'
                        . $oname
                        . ' には書き込めません。'
                        . $new_name
                        . ' に書き込みます。'
                );
            }

            #新しいレコードの書き込みを拒絶する
        }
        else {
            untie %time;
            $this->print_dialog(
                'editor',
                (   ( $setting::general{'cginame'} ne '' )
                    ? $setting::general{'cginame'}
                    : 'このBBS'
                    )
                    . 'には書き込めません。<br />記事数が '
                    . $setting::general{'limit'}
                    . ' 件以内に制限されています。'
            );

        }
    }
    else {
        $n = $this->time_id;
        $n = $this->new_number( \%time, $n );
        untie %time;
    }

    #カウント記録
    if ($count_flag) {
        my (%general);
        $count++;
        tie(%general, $env::dbm_file,
            $default::preference_dir . '/general',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %general ) {
            $general{'count'} = $count;
        }
        untie %general;
    }
    return $n;
}

######################################################################
#データベース読み書き
sub dbm ($;$) {
    my ( $this, $parent ) = @_;
    my ( $flags, $filedir );

    if ( !$this->set_dir($default::db) ) {
        push(
            @{ $this->{'dialog'} },
            'データベース書き込みに失敗しました。保存用ディレクトリが存在しません。'
        );
        return;
    }

    if ( defined $main::a && $main::a eq 'w' && $main::n ) {
        no warnings qw(untie);
        my (%tree);

        #名前登録
        if (   $field::NAMEREGISTER
            && $field::for{'name'}
            && ref $main::data{ $field::for{'name'} }
            && defined $main::data{ $field::for{'name'} }{'str'}
            && $main::data{ $field::for{'name'} }{'str'} ne ''
            && $this->use_register )
        {
            my $user_name = $main::data{ $field::for{'name'} }{'str'};
            $user_name =~ s/<>([#0-9a-zA-z]{3,10})\Z//s;
            $cgi::Register->user_name_register(
                $main::pswd, $main::email,
                $user_name,  $security::registered
            ) if $user_name ne '';
        }

        #ツリー更新
        tie(%tree, $env::dbm_file,
            $default::db . '/tree',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %tree ) {
            if (   $main::new_parent
                && $tree{ $this->encode_utf8($main::new_parent) }
                && $this->is_secure('tools') )
            {
                my @children = ();
                $this->scan_my_thread( \%tree, \@children, $main::n );
                if ( grep( $_ eq $main::new_parent, @children ) ) {
                    push(
                        @{ $this->{'dialog'} },
                        '子記事を親記事とすることはできません。'
                    );
                }
                else {
                    my ( @info, $child_count, $old_thread,
                        $delete_old_thread );

                    #現在のスレッドID保存　親記事から外す
                    $old_thread = $thread::id;
                    if ($main::parent) {
                        @info = split(
                            /,/,
                            $this->decode_utf8(
                                $tree{ $this->encode_utf8($main::parent) }
                            )
                        );
                        @info = grep( $_ ne $main::n, @info );
                        $tree{ $this->encode_utf8($main::parent) }
                            = $this->encode_utf8( join( ',', @info ) );
                    }

                    #新しい親記事に関連づける
                    if ( $main::new_parent eq $main::n ) {
                        undef $main::parent;
                        $thread::id = $main::n;
                        if ( $field::for{'category'}
                            && ref $field::obj{ $field::for{'category'} } )
                        {
                            my (%thread_category);
                            tie(%thread_category,
                                $env::dbm_file,
                                $default::db . '/thread_category',
                                O_RDONLY,
                                $env::dbm_mode
                            );
                            if ( ref tied %thread_category ) {
                                $main::data{ $field::for{'category'} }{'str'}
                                    = $this->decode_utf8(
                                    $thread_category{ $this->encode_utf8(
                                            $old_thread) } )
                                    if defined
                                    $thread_category{ $this->encode_utf8(
                                        $old_thread) };
                            }
                            untie %thread_category;
                        }
                    }
                    else {
                        $main::parent = $main::new_parent;
                        @info         = split(
                            /,/,
                            $this->decode_utf8(
                                $tree{ $this->encode_utf8($main::parent) }
                            )
                        );
                        $thread::id
                            = ( defined $info[1] && $info[1] ne '' )
                            ? $info[1]
                            : $main::parent;
                        $info[1] = '' if !$info[1];
                        $info[2] = '' if !$info[2];
                        push( @info, $main::n );
                        $tree{ $this->encode_utf8($main::parent) }
                            = $this->encode_utf8( join( ',', @info ) );
                    }
                    $child_count = @children + 1;
                    foreach my $child (@children) {
                        utf8::encode($child) if utf8::is_utf8($child);
                        if ( $tree{$child} ) {
                            @info = split( /,/,
                                $this->decode_utf8( $tree{$child} ) );
                            $info[1] = $thread::id;
                            $tree{$child}
                                = $this->encode_utf8( join( ',', @info ) );
                        }
                    }
                    undef $main::new_parent;
                    undef @children;

                    if ( $thread::id ne $old_thread ) {

                        #スレッド記事数情報更新
                        my (%thread_count);
                        tie( %thread_count, $env::dbm_file,
                            $default::db . '/thread_count',
                            O_RDWR, $env::dbm_mode );
                        if ( ref tied %thread_count ) {
                            $thread_count{ $this->encode_utf8($thread::id) }
                                += $this->encode_utf8( ( !$main::parent )
                                ? $child_count - 1
                                : $child_count );
                            $thread_count{ $this->encode_utf8($old_thread) }
                                -= $this->encode_utf8($child_count);
                            if (!$thread_count{ $this->encode_utf8(
                                        $old_thread) }
                                || $thread_count{ $this->encode_utf8(
                                        $old_thread) } <= 0 )
                            {
                                $delete_old_thread = 1;
                                delete $thread_count{ $this->encode_utf8(
                                        $old_thread) };
                            }
                        }
                        untie %thread_count;

                        #旧スレッド情報削除
                        if ($delete_old_thread) {
                            foreach my $db (
                                'thread_status', 'thread_category',
                                'thread_ctime',  'thread_mtime'
                                )
                            {
                                my (%db);
                                tie( %db, $env::dbm_file,
                                    $default::db . '/' . $db,
                                    O_RDWR, $env::dbm_mode );
                                if ( ref tied %db ) {
                                    delete
                                        $db{ $this->encode_utf8($old_thread)
                                        };
                                }
                                untie %db;
                            }
                        }

                        #状況
                        if ( $child_count > 1 || $main::parent ) {
                            my (%thread_status);
                            tie(%thread_status,
                                $env::dbm_file,
                                $default::db . '/thread_status',
                                O_RDWR,
                                $env::dbm_mode
                            );
                            if ( ref tied %thread_status ) {
                                $thread_status{ $this->encode_utf8(
                                        $thread::id) } = 'operational'
                                    if !$thread_status{ $this->encode_utf8(
                                        $thread::id) };
                            }
                            untie %thread_status;
                        }
                    }
                    push(
                        @{ $this->{'dialog'} },
                        '親記事を変更しました。'
                    );
                }
            }
            my @info
                = ( defined $tree{ $this->encode_utf8($main::n) } )
                ? split( /,/,
                $this->decode_utf8( $tree{ $this->encode_utf8($main::n) } ) )
                : ();
            $info[0] =~ s/^dm?// if defined $info[0];
            $info[0]++;
            $info[1] = ( $thread::id ne $main::n ) ? $thread::id   : '';
            $info[2] = ($main::parent)             ? $main::parent : '';
            $tree{ $this->encode_utf8($main::n) }
                = $this->encode_utf8( join( ',', @info ) );
            undef @info;

            if ( defined $main::parent && $main::parent ) {
                my @child = split(
                    /,/,
                    $this->decode_utf8(
                        $tree{ $this->encode_utf8($main::parent) }
                    )
                );
                my @info = splice( @child, 0, 3 );
                @child   = () if !@child;
                $info[0] = '' if !defined $info[0];
                $info[1] = '' if !defined $info[1];
                $info[2] = '' if !defined $info[2];
                if ( grep( $_ eq $main::n, @child ) ) {
                    untie %tree;
                }
                else {
                    push( @child, $main::n );
                    $tree{ $this->encode_utf8($main::parent) }
                        = $this->encode_utf8( join( ',', @info, @child ) );
                    untie %tree;

                    #スレッド記事数情報更新
                    my (%thread_count);
                    tie(%thread_count, $env::dbm_file,
                        $default::db . '/thread_count',
                        O_RDWR | O_CREAT,
                        $env::dbm_mode
                    );
                    if ( ref tied %thread_count ) {
                        $thread_count{ $this->encode_utf8($thread::id) }++;
                    }
                    untie %thread_count;
                }

                #状況更新
                my (%thread_status);
                tie(%thread_status, $env::dbm_file,
                    $default::db . '/thread_status',
                    O_RDWR | O_CREAT,
                    $env::dbm_mode
                );
                if ( ref tied %thread_status ) {
                    if (   $thread_status{ $this->encode_utf8($thread::id) }
                        && $thread_status{ $this->encode_utf8($thread::id) }
                        ne 'solved'
                        && $main::status
                        && $main::status eq 'solved' )
                    {
                        $thread_status{ $this->encode_utf8($thread::id) }
                            = 'solved';
                    }
                    elsif ( !defined
                           $thread_status{ $this->encode_utf8($thread::id) }
                        || $thread_status{ $this->encode_utf8($thread::id) }
                        eq '' )
                    {
                        $thread_status{ $this->encode_utf8($thread::id) }
                            = 'operational';
                    }
                }
                untie %thread_status;
            }
            else {
                untie %tree;
            }
        }
        else {
            untie %tree;
        }

        #更新情報更新
        $this->record_info_modify;

        #メール予約
        if (   $main::mailing
            && $setting::general{'resmail'}
            && $this->is_secure('mail')
            && $field::for{'mail'}
            && ref $field::obj{ $field::for{'mail'} }
            && $field::obj{ $field::for{'mail'} }->{'e-mail'}
            && $field::obj{ $field::for{'mail'} }->{'class'} =~ /email/
            && $main::data{ $field::for{'mail'} }{'str'} ne '' )
        {
            my (%mailto);
            tie(%mailto, $env::dbm_file,
                $default::db . '/mailto',
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );
            if ( ref tied %mailto ) {
                $mailto{ $this->encode_utf8($main::n) } = $this->encode_utf8(
                    $main::data{ $field::for{'mail'} }{'str'} );
                untie %mailto;

                #クッキー
                $cookie::data{'mailing'} = '1'
                    if defined $main::set_cookie && $main::set_cookie;
            }
        }
        elsif ( $main::mailing && $setting::general{'resmail'} ) {
            push(
                @{ $this->{'dialog'} },
                '返信通知機能は利用できません。利用権限がありません。'
            );
        }

        #ファイル保存
        $filedir = $default::file . '/' . $main::n;
        $cgi::input->fix_all_files($filedir);

        if ( ref $cgi::input->{'dialog'} eq 'ARRAY'
            && @{ $cgi::input->{'dialog'} } > 0 )
        {
            push( @{ $this->{'dialog'} }, @{ $cgi::input->{'dialog'} } );
        }

        $flags = O_RDWR | O_CREAT;
    }
    else {
        $flags = O_RDONLY;
    }

    my ( $id, $field, $category_field_id );
    while ( ( $id, $field ) = each %field::obj ) {
        my ( %database, $db );
        $db = $field->db;
        $category_field_id = $id if $field->{'class'} eq 'xbb_category';
        next if !defined $db || !$db;
        tie( %database, $env::dbm_file, $default::db . '/' . $db,
            $flags, $env::dbm_mode );
        if ( ref tied %database ) {
            if ( defined $main::n && $main::n ne '' ) {

                #書き込み
                if ( $main::a eq 'w' && $field->input_enabled ) {
                    if (!$field->{'protected'}
                        || (!$this->{'new_data'}
                            && $this->is_secure(
                                'reviseProtectedFields', 0, 'revise'
                            )
                        )
                        || ($this->{'new_data'}
                            && $this->is_secure(
                                'inputProtectedFields', 0, 'input'
                            )
                        )
                        )
                    {
                        eval
                            '$database{$this->encode_utf8($main::n)} = $this->encode_utf8($main::data{$id}{\'str\'})';
                        if ( defined $@ && $@ ) {
                            push(
                                @{ $this->{'dialog'} },
                                '書き込みに失敗しました。('
                                    . $field->{'name'} . ')'
                            );
                        }
                        else {
                            $field->settle_up($filedir);
                        }
                    }

                    #読み込み
                }
                else {
                    %{ $main::data{$id} } = $field->elements(
                        $this->decode_utf8(
                            $database{ $this->encode_utf8($main::n) }
                        )
                    );
                }
            }

            #親記事読み込み
            %{ $parent->{$id} } = $field->elements(
                $this->decode_utf8(
                    $database{ $this->encode_utf8($main::parent) }
                )
            ) if defined $main::parent && $main::parent;

        }
        else {
            push(
                @{ $this->{'dialog'} },
                'データベースに接続できませんでした。('
                    . $field->{'name'} . ')'
            );
        }
        untie %database;
    }

    #RSS更新
    if (   defined $main::a
        && $main::a eq 'w'
        && $main::n
        && (   $setting::general{'RSSEnabled'}
            || $setting::general{'masterRSSEnabled'}
            || $setting::general{'commentRSSEnabled'}
            || $setting::general{'siteRSSEnabled'} )
        )
    {
        my $my_url = $this->my_url;
        my $my_category_url
            = $my_url . '?'
            . ( ( $default::subdivided && defined $main::c && $main::c ne '' )
            ? 'c=' . $main::c . '&'
            : '' );
        my $guid = $my_category_url . 'n=' . $main::n . '&v=listarticle';
        if (   $setting::security{'browse'}{'passwordCheck'} eq 'nocheck'
            && $setting::security{'closed'}{'passwordCheck'} eq 'nocheck' )
        {
            my $category
                = (    $default::subdivided
                    && defined $main::c
                    && $main::c ne ''
                    && $setting::general{'cginame'} )
                ? $setting::general{'cginame'}
                : '';
            $category
                .= ( !defined $category_field_id || $category_field_id eq '' )
                ? ''
                : ( defined $thread::category_name
                    && $thread::category_name ne '' )
                ? ( ( $category ne '' ) ? '/' : '' ) . $thread::category_name
                : (    $thread::id eq $main::n
                    && defined $main::data{$category_field_id}{'str'}
                    && ref
                    $thread::category{ $main::data{$category_field_id}{'str'}
                    }
                    && $thread::category{ $main::data{$category_field_id}
                        {'str'} }{'name'} )
                ? ( ( $category ne '' ) ? '/' : '' )
                . $thread::category{ $main::data{$category_field_id}{'str'} }
                {'name'}
                : '';
            $this->feed_rss(
                {   'title' => (
                        (          $field::for{'title'}
                                && ref $field::obj{ $field::for{'title'} }
                                && $field::obj{ $field::for{'title'} }
                                ->{'Open'}
                                && !$field::obj{ $field::for{'title'} }
                                ->{'Hidden'}
                                && $main::data{ $field::for{'title'} }{'str'}
                        )
                        ? $field::obj{ $field::for{'title'} }->usual_format(
                            $main::data{ $field::for{'title'} }{'str'},
                            $default::file . '/' . $main::n,
                            $main::c,
                            $main::n
                            )
                        : '新しい記事が投稿されました。'
                    ),
                    'link' => (
                        (          $setting::general{'html'}
                                && $setting::general{'rssLinkToHtml'}
                        )
                        ? $this->full_path(
                            ( ($default::file) ? $default::file : '.' )
                            . '/index.html'
                            )
                            . '#n'
                            . $main::n
                        : $guid
                    ),
                    'content:encoded' => (
                        (          $field::for{'content'}
                                && ref $field::obj{ $field::for{'content'} }
                                && $field::obj{ $field::for{'content'} }
                                ->{'Open'}
                                && !$field::obj{ $field::for{'content'} }
                                ->{'Hidden'}
                                && $main::data{ $field::for{'content'} }
                                {'str'}
                        )
                        ? $field::obj{ $field::for{'content'} }
                            ->format_for_detail(
                            $main::data{ $field::for{'content'} }{'str'},
                            $default::file . '/' . $main::n,
                            $main::c,
                            $main::n
                            )
                        : '新しい記事が投稿されました。'
                    ),
                    'dc:creator' => (
                        (          $field::for{'name'}
                                && ref $field::obj{ $field::for{'name'} }
                                && $main::data{ $field::for{'name'} }{'str'}
                        )
                        ? $field::obj{ $field::for{'name'} }
                            ->format_for_detail(
                            $main::data{ $field::for{'name'} }{'str'},
                            $default::file . '/' . $main::n,
                            $main::c, $main::n
                            )
                        : ( !$field::for{'name'}
                                && defined $setting::general{'mastername'} )
                        ? $setting::general{'mastername'}
                        : ''
                    ),
                    'category'  => $category,
                    'enclosure' => $this->article_enclosure,
                    'guid'      => $guid,
                    (   (          $env::checkout
                                && $field::for{'showcase'}
                                && ref $field::obj{ $field::for{'showcase'} }
                                && $field::obj{ $field::for{'showcase'} }
                                ->{'Open'}
                                && !$field::obj{ $field::for{'showcase'} }
                                ->{'Hidden'}
                        )
                        ? $field::obj{ $field::for{'showcase'} }
                            ->get_gdata( $guid,
                            $main::data{ $field::for{'showcase'} } )
                        : ()
                    )
                },
                $default::file,
                (   ( $setting::general{'html'} )
                    ? $this->full_path(
                        ( ($default::file) ? $default::file : '.' )
                        . '/index.html'
                        )
                    : $my_category_url
                ),
                ( ($default::subdivided) ? $default::master_file_dir : '' ),
                ( ($default::subdivided) ? $my_url : '' )
            );
        }
        else {
            $this->feed_rss(
                {   'title' => '新しい記事が投稿されました。',
                    'link'  => $guid,
                    'content:encoded' =>
                        '新しい記事が投稿されました。',
                    'guid' => $guid
                },
                $default::file,
                $my_category_url,
                ( ($default::subdivided) ? $default::master_file_dir : '' ),
                ( ($default::subdivided) ? $my_url : '' )
            );
        }
    }
}

sub article_enclosure ($) {
    my $this = shift;
    my $hash
        = (    $field::for{'media'}
            && ref $field::obj{ $field::for{'media'} }
            && $field::obj{ $field::for{'media'} }->{'Open'}
            && !$field::obj{ $field::for{'media'} }->{'Hidden'}
            && $main::data{ $field::for{'media'} }{'str'} )
        ? {
        $field::obj{ $field::for{'media'} }->get_enclosure_attributes(
            $main::data{ $field::for{'media'} }{'str'},
            $default::file . '/' . $main::n,
            $main::c, $main::n
        )
        }
        : '';
    $hash
        = (    $field::for{'media2'}
            && ref $field::obj{ $field::for{'media2'} }
            && $field::obj{ $field::for{'media2'} }->{'Open'}
            && !$field::obj{ $field::for{'media2'} }->{'Hidden'}
            && $main::data{ $field::for{'media2'} }{'str'} )
        ? {
        $field::obj{ $field::for{'media2'} }->get_enclosure_attributes(
            $main::data{ $field::for{'media2'} }{'str'},
            $default::file . '/' . $main::n,
            $main::c, $main::n
        )
        }
        : ''
        if !ref $hash || !%$hash;
    $hash
        = (    $field::for{'media3'}
            && ref $field::obj{ $field::for{'media3'} }
            && $field::obj{ $field::for{'media3'} }->{'Open'}
            && !$field::obj{ $field::for{'media3'} }->{'Hidden'}
            && $main::data{ $field::for{'media3'} }{'str'} )
        ? {
        $field::obj{ $field::for{'media3'} }->get_enclosure_attributes(
            $main::data{ $field::for{'media3'} }{'str'},
            $default::file . '/' . $main::n,
            $main::c, $main::n
        )
        }
        : ''
        if !ref $hash || !%$hash;
    return $hash;
}

sub index_sweeper ($$) {
    my ( $this, $delete_list ) = @_;
    my ( $home_url, $home_id, @index_files, @rss_files );
    $home_id
        = ( $default::subdivided && defined $main::c && $main::c ne '' )
        ? $main::c . '-'
        : '';
    $home_url
        = $this->my_url . '?'
        . ( ( $default::subdivided && defined $main::c && $main::c ne '' )
        ? 'c=' . $main::c . '&'
        : '' );
    @index_files = (
        $env::externalfiles . '/recentcomments.ssi',
        $env::externalfiles . '/recenttrackbacks.ssi',
        $default::file . '/recentcomments.ssi',
        $default::file . '/recenttrackbacks.ssi',
    );
    @rss_files = (
        '../../index.xml',                 '../../comment.xml',
        '../../trackback.xml',             '../../podcast.xml',
        $default::file . '/index.xml',     $default::file . '/comment.xml',
        $default::file . '/trackback.xml', $default::file . '/podcast.xml',
    );
    if ($default::subdivided) {
        push( @index_files,
            $default::master_file_dir . '/recentcomments.ssi',
            $default::master_file_dir . '/recenttrackbacks.ssi' );
        push( @rss_files,
            $default::master_file_dir . '/index.xml',
            $default::master_file_dir . '/comment.xml',
            $default::master_file_dir . '/trackback.xml',
            $default::master_file_dir . '/podcast.xml' );
    }
    require CMONOS::CGI_Lib::IndexSweeper;
    IndexSweeper->new(
        'index_files' => \@index_files,
        'rss_files'   => \@rss_files,
        )->sweep(
        [ map { $home_url . 'n=' . $_ . '&v=listarticle' } @$delete_list ],
        { $default::id => [ map { $home_id . $_ } @$delete_list ] }
        );
}

######################################################################
# 新規カテゴリー
sub cp_category ($$$;$) {
    my ( $this, $old_c, $c, $name ) = @_;
    my ( $old_file_dir, $old_pref_dir, $old_field_pref_dir, $icon, %general );

    $old_file_dir = $default::file;
    $default::file
        = ( ( $default::master_file_dir ne '' )
        ? $default::master_file_dir
        : '.' )
        . '/'
        . $c
        if defined $default::file;
    $this->set_dir($default::file);
    $icon = $this->fileglob( $old_file_dir, '^icon\.(gif|jpg|png)$' );
    if ( defined $icon && $icon ) {
        my ($icon_name);
        $icon_name = $1 if $icon =~ /(icon\.(gif|jpg|png)$)$/;
        $this->file_copy( $icon, $default::file . '/' . $icon_name );
    }

    $default::db
        = (
        ( $default::master_db_dir ne '' ) ? $default::master_db_dir : '.' )
        . '/'
        . $c
        if defined $default::db;
    $this->set_dir($default::db);

    $old_pref_dir            = $default::preference_dir;
    $old_field_pref_dir      = $default::field_pref_dir;
    $default::preference_dir = $default::master_preference_dir;
    $default::field_pref_dir = $default::master_field_pref_dir;
    unless ( $default::field_pref_dir
        =~ s!^$default::preference_dir/!$default::preference_dir/$c/! )
    {
        $default::field_pref_dir
            = ( ( $default::field_pref_dir ne '' )
            ? $default::field_pref_dir
            : '.' )
            . '/'
            . $c;
    }
    $default::preference_dir
        = (
        ( $default::preference_dir ne '' ) ? $default::preference_dir : '.' )
        . '/'
        . $c;

    $this->directory_copy( $old_pref_dir,       $default::preference_dir );
    $this->directory_copy( $old_field_pref_dir, $default::field_pref_dir );

    #一般設定調整
    tie(%general, $env::dbm_file,
        $default::preference_dir . '/general',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %general ) {
        $setting::general{'cginame'} = $general{'cginame'}
            = $this->encode_utf8($name)
            if defined $name && $name;
        $setting::general{'latest'} = $general{'latest'} = '';
        $setting::general{'count'}  = $general{'count'}  = '';
        $setting::general{'original'} = $general{'original'}
            = $this->encode_utf8($old_c);
    }
    untie %general;

    #グループ更新時を登録
    my (%category_mtime);
    tie(%category_mtime, $env::dbm_file,
        $default::master_db_dir . '/category_mtime',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %category_mtime ) {
        $category_mtime{$c} = time;
        $category_mtime{'Last-Modified'} = time;
    }
    untie %category_mtime;
}

######################################################################
sub record_info_modify ($) {
    my $this = shift;
    my ( %mtime, %thread_mtime, %ip, %agent, %general, $time, $last,
        $user_ip );
    $time = time;
    $last = 0;

    #更新時刻の書き込み
    tie(%mtime, $env::dbm_file,
        $default::db . '/mtime',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %mtime ) {
        $last
            = ( $mtime{ $this->encode_utf8($main::n) } )
            ? $this->decode_utf8( $mtime{ $this->encode_utf8($main::n) } )
            : 0;
        $mtime{ $this->encode_utf8($main::n) } = $time;
    }
    else {
        push(
            @{ $this->{'dialog'} },
            '書き込みに失敗しました。(mtime)'
        );
    }
    untie %mtime;
    tie(%thread_mtime, $env::dbm_file,
        $default::db . '/thread_mtime',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %thread_mtime ) {
        $thread_mtime{ $this->encode_utf8($thread::id) } = $time;
    }
    else {
        push(
            @{ $this->{'dialog'} },
            '書き込みに失敗しました。(thread_mtime)'
        );
    }
    untie %thread_mtime;

    $this->{'new_record'} = 1 if $this->{'new_data'} || $time - $last > 10800;

    #作成時刻の書き込み
    if ( defined $this->{'new_data'} && $this->{'new_data'} ) {
        my (%ctime);
        tie(%ctime, $env::dbm_file,
            $default::db . '/ctime',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %ctime ) {
            $ctime{ $this->encode_utf8($main::n) } = $time
                if !$ctime{ $this->encode_utf8($main::n) };
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '書き込みに失敗しました。(ctime)'
            );
        }
        untie %ctime;
        if ( !defined $main::parent || $main::parent eq '' ) {
            my (%thread_ctime);
            tie(%thread_ctime, $env::dbm_file,
                $default::db . '/thread_ctime',
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );
            if ( ref tied %thread_ctime ) {
                $thread_ctime{ $this->encode_utf8($thread::id) } = $time
                    if !$thread_ctime{ $this->encode_utf8($thread::id) };
            }
            else {
                push(
                    @{ $this->{'dialog'} },
                    '書き込みに失敗しました。(thread_ctime)'
                );
            }
            untie %thread_ctime;
        }
    }

    $user_ip
        = ( $ENV{'REMOTE_HOST'} || $this->to_host( $ENV{'REMOTE_ADDR'} ) );
    if ( !$this->{'keep_original_user_info'} ) {

        #投稿者IPの書き込み
        tie(%ip, $env::dbm_file,
            $default::db . '/ip',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %ip ) {
            $ip{ $this->encode_utf8($main::n) }
                = $this->encode_utf8($user_ip);
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '書き込みに失敗しました。(ip)'
            );
        }
        untie %ip;

        #投稿者環境の書き込み
        tie(%agent, $env::dbm_file,
            $default::db . '/agent',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %agent ) {
            $agent{ $this->encode_utf8($main::n) }
                = $this->encode_utf8( $ENV{'HTTP_USER_AGENT'} );
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '書き込みに失敗しました。(agent)'
            );
        }
        untie %agent;
    }

    #カテゴリー更新時更新
    if ( defined $default::subdivided && $default::subdivided ) {
        my (%category_mtime);
        tie(%category_mtime, $env::dbm_file,
            $default::master_db_dir . '/category_mtime',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %category_mtime ) {
            $category_mtime{$main::c} = $time;
            $category_mtime{'Last-Modified'} = $time;
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '書き込みに失敗しました。(category_mtime)'
            );
        }
        untie %category_mtime;
        tie(%general, $env::dbm_file,
            $default::master_db_dir . '/general',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %general ) {
            $general{'mtime'} = $this->encode_utf8($time);
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '書き込みに失敗しました。(/general)'
            );
        }
        untie %general;
    }

    #一般情報更新
    tie(%general, $env::dbm_file,
        $default::preference_dir . '/general',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %general ) {
        $general{'latest'} = $this->encode_utf8( $time . ':' . $user_ip );
        $general{'mtime'}  = $this->encode_utf8($time);
    }
    else {
        push(
            @{ $this->{'dialog'} },
            '書き込みに失敗しました。(latest)'
        );
    }
    untie %general;
    $setting::general{'mtime'} = $time;
}

######################################################################
sub delivery_res_notice_mail ($) {
    my $this = shift;
    my ( @list, %tree, %mailto, %addr, @addr );
    no warnings qw(untie);
    tie( %tree, $env::dbm_file, "$default::db/tree", O_RDONLY,
        $env::dbm_mode );
    if ( ref tied %tree ) {

        #関連記事全てをリストアップ
        if ( $setting::mail{'resdelivery'} eq 'thread' ) {
            push( @list, $thread::id );
            $this->scan_my_thread( \%tree, \@list, $thread::id );
            @list = grep( $_ ne $main::n, @list );

            #上流のみをリストアップ
        }
        else {
            my $n = $main::parent;
            while ( $n ne '' && $n ne $main::n ) {
                push( @list, $n );
                my @info = split( /,/,
                    $this->decode_utf8( $tree{ $this->encode_utf8($n) } ) );
                $n = $info[2];
            }
        }
        untie %tree;
    }
    else {
        untie %tree;
        push(
            @{ $this->{'dialog'} },
            'ツリーデータベースに接続できませんでした。'
        );
        return;
    }
    return if !@list;

    tie( %mailto, $env::dbm_file, $default::db . '/mailto',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %mailto ) {
        my ($n);
        foreach $n (@list) {
            my $addr
                = $this->decode_utf8( $mailto{ $this->encode_utf8($n) } );
            $addr{$1}++
                while $addr
                =~ /([0-9a-zA-Z_\-\.]+\@[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-\.]+)/gs;
        }
    }
    untie %mailto;
    if ( defined $main::data{ $field::for{'mail'} }{'str'}
        && $main::data{ $field::for{'mail'} }{'str'} )
    {
        my @my_addr
            = $this->get_address( $main::data{ $field::for{'mail'} }{'str'} );
        foreach my $m (@my_addr) {
            utf8::encode($m) if utf8::is_utf8($m);
            delete $addr{$m} if defined $addr{$m};
        }
    }
    @addr = keys %addr;
    undef %addr;
    return if !@addr;
    $this->send_notice_mail(
        $setting::mail{'ressubject'},
        $setting::mail{'resmsg'},
        $setting::general{'resmail'}, \@addr
        )
        || $this->notice_mail(
        '新しい返信が投稿されました。',
        '新しい返信が投稿されました。',
        \%field::obj,
        ( ( $setting::general{'resmail'} eq 'data' ) ? \%main::data : '' ),
        $setting::general{'master'},
        @addr
        );
}

sub scan_my_thread ($$$@) {
    my ( $this, $tree, $list, @num ) = @_;
    my ($n);
    foreach $n (@num) {
        my @child = split( /,/,
            $this->decode_utf8( $$tree{ $this->encode_utf8($n) } ) );
        splice( @child, 0, 3 );
        push( @$list, @child );
        $this->scan_my_thread( $tree, $list, @child );
    }
}

sub send_notice_mail ($$$$$) {
    my ( $this, $subject, $msg, $mode, $addr ) = @_;
    my ( $name, $title, $url );
    return 0 if $subject eq '' && $msg eq '';
    if ( defined $field::for{'name'} && $field::for{'name'} ) {
        $name = $main::data{ $field::for{'name'} }{'str'};
        $name =~ s/<>([#0-9a-zA-z]{3,10})\Z//s;
    }
    $name = ($default::english) ? 'Anonymous' : '匿名'
        if !defined $name || $name eq '';
    if ( defined $field::for{'title'} && $field::for{'title'} ) {
        $title = $main::data{ $field::for{'title'} }{'str'};
        $title =~ s/<>([#0-9a-zA-z]{3,10})\Z//s;
    }
    $title = ($default::english) ? 'This item' : 'これ'
        if !defined $title || $title eq '';
    $subject = '' if !defined $subject;
    $msg     = '' if !defined $msg;
    $subject =~ s/<name>/$name/igs;
    $msg =~ s/<name>/$name/igs;
    $subject =~ s/<title>/$title/igs;
    $msg =~ s/<title>/$title/igs;
    $url
        = $this->my_url . '?'
        . ( ( defined $main::c ) ? 'c=' . $main::c . '&' : '' ) . 'n='
        . $main::n;
    $msg =~ s/<url>/$url/igs;

    if ( $mode eq 'data' ) {
        my $list = $this->data_for_notice_mail( \%field::obj, \%main::data );
        $msg .= "\x0A\x0A" . $list . "\x0A" unless $msg =~ s/<data>/$list/igs;
    }
    else {
        $msg =~ s/<data>//igs;
    }
    unless ( ref $cgi::SendMail ) {
        require CMONOS::CGI_Lib::SendMail;
        $cgi::SendMail
            = SendMail->new( $env::sendmail, $env::master_email, '',
            $cgi::input, 1 );
    }
    $cgi::SendMail->send_mail(
        'From' => (
            (          defined $field::for{'mail'}
                    && ref $field::obj{ $field::for{'mail'} }
                    && !$field::obj{ $field::for{'mail'} }->{'Hidden'}
                    && $field::obj{ $field::for{'mail'} }->{'Open'}
                    && $main::data{ $field::for{'mail'} }{'str'}
            )
            ? ( ( $main::data{ $field::for{'name'} }{'str'} )
                ? $main::data{ $field::for{'name'} }{'str'} . ' <'
                    . $main::data{ $field::for{'mail'} }{'str'} . '>'
                : $main::data{ $field::for{'mail'} }{'str'} )
            : $setting::general{'master'}
        ),
        'Bcc'     => join( ', ', @$addr ),
        'Subject' => $subject,
        'message' => $msg . "\x0A\x0A",
        'file' =>
            ( ( %main::data && defined $mail::file ) ? $mail::file : '' )
    );
    return 1;
}

sub notice_new_record_for_master ($) {
    my $this = shift;
    return
        if !defined $setting::general{'newmail'}
        || !$setting::general{'newmail'};
    my @addr = $this->get_address(
        $setting::general{'master'},
        $setting::general{'owners'},
        $setting::general{'staff'}
    );
    return if !@addr;
    $this->send_notice_mail(
        $setting::mail{'newsubject'},
        $setting::mail{'newmsg'},
        $setting::general{'newmail'}, \@addr
        )
        || $this->notice_mail(
        '新しい記事が投稿されました。',
        '新しい記事が投稿されました。',
        \%field::obj,
        ( ( $setting::general{'newmail'} eq 'data' ) ? \%main::data : '' ),
        $setting::general{'master'},
        @addr
        );
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {

        #クッキー
        $cookie::id_enabled = 1;    #ID情報クッキー読み込み
        $cookie::default_enabled
            = 1;    #ユーザー設定情報クッキー読み込み
        $cookie::name
            = ( $default::subdivided && defined $main::c && $main::c ne '' )
            ? 'INPUT_' . ( uc $main::c )
            : 'INPUT';
        $cookie::path = $ENV{'SCRIPT_NAME'};

        #必須項目
        @setting::form = (
            'm',  'k', 'v', 's',   'w',      'tc',
            'st', 'n', 't', 'and', 'search', 'q'
        );
        if ( $ENV{'QUERY_STRING'} ) {
            undef $main::a;
            if ( ( $main::delete || $main::rmthread )
                && !$cgi::input->referer_is_me(1) )
            {
                undef $main::delete;
                undef $main::rmthread;
            }
        }
        elsif ($ENV{'REQUEST_METHOD'}
            && $ENV{'REQUEST_METHOD'} eq 'POST'
            && !$cgi::input->referer_is_me )
        {
            undef $main::a;
            undef $main::delete;
            undef $main::rmthread;
        }

        #各種キー設定
        #デフォルトセキュリティキー
        undef $main::n if defined $main::parent && $main::parent ne '';
        if ( !defined $setting::security_key || $setting::security_key eq '' )
        {
            if ( defined $main::n && $main::n ne '' ) {
                $setting::security_key = 'revise';
            }
            elsif ( defined $main::parent && $main::parent ne '' ) {
                $setting::security_key = 'inputchild';
            }
            else {
                $setting::security_key = 'inputthread';
            }
        }

        #ページ設定キー
        $setting::key = 'editor'
            if !defined $setting::key || $setting::key eq '';

        #デフォルトキー
        $main::k = 'editor' if !defined $main::k || !$main::k;

        #クラス登録
        Common::class_register( 'editor', 'Editor' );

    }
}

1;
