package ArticlePage;
require CMONOS::CGI_Lib::Tools::Page;
@ISA = qw(Page);
use strict;
use utf8;
######################################################################
# 記事ページ設定モジュール
# ArticlePage.pm ver1.0.5
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2013-12-08
######################################################################
# 設定項目取得
sub get_sub_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #チェックボックス
    push( @check::checkbox,
        'articleFrameClass',          'articleFrameTitle',
        'articleFrameStatus',         'staticIndent',
        'headIcon',                   'footIcon',
        'viewSwitch',                 'searchbox',
        'sortMenu',                   'pswdIcon',
        'defaultSearchOparatorFixed', 'treeNoLine',
        'naviIcon',                   'searchFrameTitle',
        'treeNaviEnabled',            'formEnabled' );

    #数字チェック
    $check::num{'listLimit'}
        = 'テーブル表示インデックス表示件数';
    $check::num{'treeLimit'}
        = 'ツリー表示インデックス表示件数';
    $check::num{'articleLimit'}      = 'ソート順記事表示件数';
    $check::num{'articleFrameWidth'} = '記事を囲うフレーム幅';
    $check::num{'indentWidth'}       = 'スレッド表示インデント幅';
    $check::num{'naviWidth'}         = '前後記事ナビ フレーム幅';
    $check::num{'searchFrameWidth'}  = '検索条件入力欄フレーム幅';

    return;
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    $preference::id   = 'article';                        # データキー
    $preference::name = '記事ページレイアウト'; # 設定パネル名
    $preference::title
        = '記事ページのレイアウトについて設定します。';
}

######################################################################
# 設定パネル
sub print_sub_preference_panel ($;$$) {
    my ( $this, $id, $key ) = @_;
    print '<fieldset><legend>共通設定</legend>';
    print
        '<fieldset><legend><label for="data:article_defaultView">表示のデフォルト</label></legend>';
    print $this->mk_select_menu(
        'data:article_defaultView',
        '表示のデフォルト形式を設定します。', '',
        $main::data{'article_defaultView'},
        'list' =>
            'テーブル表示インデックス（親トピックのタイトルのみを一覧表示）',
        'listtree' =>
            'ツリー表示インデックス（タイトルのみのツリーを表示）',
        'listthread' =>
            'ソート順スレッド表示（記事をソート順にツリー表示）',
        'listarticle' =>
            'ソート順記事表示（記事をソート順に表示）'
    );
    print '</fieldset>';
    print '<fieldset><legend>ツールボタン設定</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label for="data:article_headIcon"><input type="checkbox" name="data:article_headIcon" id="data:article_headIcon"  value="1"'
        . (
        (   defined $main::data{'article_headIcon'}
                && $main::data{'article_headIcon'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> ツールボタンをページ上部に表示する</label></li>';
    print
        '<li><label for="data:article_footIcon"><input type="checkbox" name="data:article_footIcon" id="data:article_footIcon"  value="1"'
        . (
        (   defined $main::data{'article_footIcon'}
                && $main::data{'article_footIcon'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> ツールボタンをページ下部に表示する</label></li>';
    print '</ol>';
    print '<ol class="checkbox">';
    print
        '<li><label for="data:article_viewSwitch"><input type="checkbox" name="data:article_viewSwitch" id="data:article_viewSwitch"  value="1"'
        . (
        (   defined $main::data{'article_viewSwitch'}
                && $main::data{'article_viewSwitch'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> ツールボタンに表示切り替えボタンを表示する</label></li>';
    print
        '<li><label for="data:article_searchbox"><input type="checkbox" name="data:article_searchbox" id="data:article_searchbox"  value="1"'
        . (
        (   defined $main::data{'article_searchbox'}
                && $main::data{'article_searchbox'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> 表示切り替えボタンに検索ボックスを追加する</label></li>';
    print
        '<li><label for="data:article_sortMenu"><input type="checkbox" name="data:article_sortMenu" id="data:article_sortMenu"  value="1"'
        . (
        (   defined $main::data{'article_sortMenu'}
                && $main::data{'article_sortMenu'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> 表示切り替えボタンに再ソートメニューを追加する</label></li>';
    print
        '<li><label for="data:article_pswdIcon"><input type="checkbox" name="data:article_pswdIcon" id="data:article_pswdIcon"  value="1"'
        . (
        (   defined $main::data{'article_pswdIcon'}
                && $main::data{'article_pswdIcon'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> ツールボタンにログインボタンを表示する</label></li>';
    print '</ol>';
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset>';
    print
        '<legend><label for="data:article_defaultSearchOparator">「記事検索」のデフォルトの検索方式</label></legend>';
    print $this->mk_select_menu(
        'data:article_defaultSearchOparator',
        '', '',
        (   ( defined $main::data{'article_defaultSearchOparator'} )
            ? $main::data{'article_defaultSearchOparator'}
            : 'OR'
        ),
        'AND' => '全ての条件に一致',
        'OR'  => 'いずれかの条件に一致',
    );
    print
        ' <label><input type="checkbox" name="data:article_defaultSearchOparatorFixed" id="data:article_defaultSearchOparatorFixed" value="1"'
        . ( ( $main::data{'article_defaultSearchOparatorFixed'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 固定する</label>';
    print '</fieldset>';

    print
        '<fieldset><legend>テーブル表示インデックスの設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:article_listLimit">表示件数</label></td>';
    print
        '<td class="field"><input type="text" name="data:article_listLimit" id="data:article_listLimit" value="'
        . $this->escape( $main::data{'article_listLimit'} )
        . '" size="7" maxlength="7" /> 件/1ページ</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:article_listTable">テーブルタイプ</label></td>';
    print '<td class="field">';
    print $this->table_menu( 'data:article_listTable',
        $main::data{'article_listTable'} );
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:article_listLink">タイトルのリンク先</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        'data:article_listLink',
        'タイトルのリンク先の表示形式を設定します。',
        '', $main::data{'article_listLink'},
        'listthread' => '親トピックと返信記事をスレッド表示',
        'listarticle' => '親トピックの記事単体を表示'
    );
    print '</td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print
        '<fieldset class="foldable"><legend>テーブルを囲うフレーム</legend>';
    print $cgi::Deco->frame_menu( 'data:article_listFrame', 'frame',
        $main::data{'article_listFrame'} );
    print '</fieldset>';
    print '</fieldset>';

    print
        '<fieldset><legend>ツリー表示インデックスの設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">表示方法</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="data:article_treeNoLine" id="data:article_treeNoLine" value="1"'
        . (
        ( $main::data{'article_treeNoLine'} ) ? ' checked="checked"' : '' )
        . ' /> 記事階層ガイド罫線を表示しない</label></td>';
    print '</tr><tr>';
    print
        '<td class="field-name"><label for="data:article_treeLimit">表示件数</label></td>';
    print
        '<td class="field"><input type="text" name="data:article_treeLimit" id="data:article_treeLimit" value="'
        . $this->escape( $main::data{'article_treeLimit'} )
        . '" size="7" maxlength="7" /> 件/1ページ</td>';
    print '</tr>';
    print '</tbody>';
    print '<tbody class="foldable">';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:article_treeStyle">表示スタイル</label></td>';
    print
        '<td class="field"><input type="text" name="data:article_treeStyle" id="data:article_treeStyle" value="'
        . $this->escape( $main::data{ $key . 'treeStyle' } )
        . '" size="64" />';
    print
        '<div class="hints">":" を含まない場合はクラス指定と解釈されます。</div></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print
        '<fieldset class="foldable"><legend>ツリーを囲うフレーム</legend>';
    print $cgi::Deco->frame_menu( 'data:article_treeFrame', 'frame',
        $main::data{'article_treeFrame'} );
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset><legend>記事表示の設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:article_articleLimit">表示件数</label></td>';
    print
        '<td class="field"><input type="text" name="data:article_articleLimit" id="data:article_articleLimit" value="'
        . $this->escape( $main::data{'article_articleLimit'} )
        . '" size="7" maxlength="7" /> 件/1ページ</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:article_indentWidth">スレッド表示インデント幅</label></td>';
    print
        '<td class="field"><input type="text" name="data:article_indentWidth" id="data:article_indentWidth" value="'
        . $this->escape( $main::data{'article_indentWidth'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span>';
    print
        '<label for="data:article_staticIndent"><input type="checkbox" name="data:article_staticIndent" id="data:article_staticIndent"  value="1"'
        . (
        (   defined $main::data{'article_staticIndent'}
                && $main::data{'article_staticIndent'}
        ) ? ' checked="checked"' : ''
        ) . ' /> インデント幅を加算しない</label></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">記事ナビゲーション</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="data:article_treeNaviEnabled" id="data:article_treeNaviEnabled" value="1"'
        . ( ( $main::data{'article_treeNaviEnabled'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 記事ナビゲーションをツリー表示にする</label></td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><span class="datatitle">投稿フォーム</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="data:article_formEnabled" id="data:article_formEnabled" value="1"'
        . (
        ( $main::data{'article_formEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> 投稿フォームをページ下部に表示する<span class="note">(静的ページには常に表示されません)</span></label></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '<fieldset><legend>記事を囲うフレーム</legend>';
    print $cgi::Deco->frame_menu( 'data:article_articleFrame', 'frame',
        $main::data{'article_articleFrame'} );
    print '<div class="additional-field">';
    print
        '<label for="data:article_articleFrameWidth">フレーム幅</label><input type="text" name="data:article_articleFrameWidth" id="data:article_articleFrameWidth" value="'
        . $this->escape( $main::data{'article_articleFrameWidth'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span>';
    print
        ' <label for="data:article_articleAlign">記事フレーム内行揃え</label>';
    print $this->mk_select_menu(
        'data:article_articleAlign', '', '',
        $main::data{'article_articleAlign'},
        ''       => '',
        'left'   => '左寄せ',
        'center' => '中央揃え',
        'right'  => '右寄せ'
    );
    print $this->mk_select_menu(
        'data:article_articleValign', '', '',
        $main::data{'article_articleValign'},
        ''       => '',
        'top'    => '上寄せ',
        'middle' => '中央揃え',
        'bottom' => '下寄せ'
    );
    print '</div>';
    print '</fieldset>';
    print '<fieldset><legend>タイトル表示設定</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label for="data:article_articleFrameTitle"><input type="checkbox" name="data:article_articleFrameTitle" id="data:article_articleFrameTitle"  value="1"'
        . (
        (   defined $main::data{'article_articleFrameTitle'}
                && $main::data{'article_articleFrameTitle'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> 記事タイトル部を記事フレームのタイトルバー部分に表示する</label></li>';
    print
        '<li><label for="data:article_articleFrameClass"><input type="checkbox" name="data:article_articleFrameClass" id="data:article_articleFrameClass"  value="1"'
        . (
        (   defined $main::data{'article_articleFrameClass'}
                && $main::data{'article_articleFrameClass'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> 記事カテゴリーアイコンを記事タイトル部に含める</label></li>';
    print
        '<li><label for="data:article_articleFrameStatus"><input type="checkbox" name="data:article_articleFrameStatus" id="data:article_articleFrameStatus"  value="1"'
        . (
        (   defined $main::data{'article_articleFrameStatus'}
                && $main::data{'article_articleFrameStatus'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> 記事状態アイコンを記事タイトル部に含める</label></li>';
    print '</ol>';
    print '</fieldset>';

    $this->use_field( 'field', $default::field_pref_dir . '/inputfield', 2 );
    require CMONOS::CGI_Lib::Template;
    my $this_template = Template->new( \%field::obj );
    $this_template->print_template_form(
        'data:article_template',
        '記事表示テンプレート（上記記事表示設定よりテンプレートが優先されます）',
        \$main::data{'article_template'}
    );

    print '</fieldset>';

    print
        '<fieldset class="foldable"><legend>前後記事ナビの設定</legend>';
    print
        '<span class="datatitle">アイコン表示</span><label><input type="checkbox" name="data:article_naviIcon" id="data:article_naviIcon" value="1"'
        . ( ( $main::data{'article_naviIcon'} ) ? ' checked="checked"' : '' )
        . ' /> ツリー表示時の設定に準じる (デフォルトでは、表示切り替えボタン等のツールボタンをテキストではなくアイコンで表示します)</label>';
    print
        '<fieldset><legend>前後記事ナビを囲うフレーム</legend>';
    print $cgi::Deco->frame_menu( 'data:article_naviFrame', 'frame',
        $main::data{'article_naviFrame'} );
    print '<div class="additional-field">';
    print
        '<label for="data:article_naviWidth">フレーム幅</label><input type="text" name="data:article_naviWidth" id="data:article_naviWidth" value="'
        . $this->escape( $main::data{'article_naviWidth'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span>';
    print '</div>';
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset class="foldable"><legend>検索表示の設定</legend>';
    print
        '<fieldset><legend>検索条件入力欄を囲うフレーム</legend>';
    print $cgi::Deco->frame_menu( 'data:article_searchFrame', 'frame',
        $main::data{'article_searchFrame'} );
    print '<div class="additional-field">';
    print
        '<label for="data:article_searchFrameWidth">フレーム幅</label><input type="text" name="data:article_searchFrameWidth" id="data:article_searchFrameWidth" value="'
        . $this->escape( $main::data{'article_searchFrameWidth'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span>';
    print '</div>';
    print '</fieldset>';
    print
        '<div class="external-field"><label for="data:article_searchFrameTitle"><input type="checkbox" name="data:article_searchFrameTitle" id="data:article_searchFrameTitle"  value="1"'
        . ( ( $main::data{'article_searchFrameTitle'} )
        ? ' checked="checked"'
        : '' )
        . ' />「記事検索」を検索条件入力欄フレームのタイトルバー部分に表示する</label></div>';
    print '</fieldset>';
}

######################################################################
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {

        #クラス登録
        Common::class_register( 'b010', 'ArticlePage' );

        #ツール登録
        $setting::jump_menu{'b010'}
            = 'ページレイアウト';    #これがデフォルト
        $setting::sub_jump_menu{'page'}{'b010'} = '記事ページ';
        $setting::selected_jump_menu{'b010'} = 'b010'
            ; #サブメニューがあるときメインメニューのデフォルトキー

        $preference::page{'b010'} = 'article';

    }
}

1;
