package PageObj;
require CMONOS::CGI_Lib::Tools::Page;
@ISA = qw(Page);
use strict;
use utf8;
######################################################################
# 共通部品モジュール
# PageObj.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 設定チェック項目
sub get_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #チェックボックス
    push( @check::checkbox, 'index' );

    #数字チェック
    $check::num{'pagelistmax'}    = 'ページリンク最大表示数';
    $check::num{'pagelistmargin'} = '前方ページリンク表示数';
    $check::num{'newIconLimit'}   = '新着アイコン対象期間';
    $check::num{'upIconLimit'}    = '更新アイコン対象期間';

    #URL書式チェック
    $check::url{'returnUrl'} = '戻る先URL';
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    $preference::id   = '';                      # データキー
    $preference::name = '共通部品設定';    # 設定パネル名
    $preference::title
        = '各ページが共通して利用するボタン等について設定します。';
}

######################################################################
# 設定パネル
sub print_preference_panel ($) {
    my $this = shift;
    print '<fieldset><legend>タイトル補完設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:defaultTitle">無記入時タイトル</label></td>';
    print
        '<td class="field"><input type="text" size="50" maxlength="100" name="data:defaultTitle" id="data:defaultTitle" value="'
        . $this->escape( $main::data{'defaultTitle'} )
        . '" title="「無題」など、タイトルフィールドが無記入の場合に表示するタイトルを設定します。" /></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:deletedMsg">削除記事タイトル</label></td>';
    print
        '<td class="field"><input type="text" size="50" maxlength="100" name="data:deletedMsg" id="data:deletedMsg" value="'
        . $this->escape( $main::data{'deletedMsg'} )
        . '" title="「削除されました」など、記事が削除されている場合に表示するタイトルを設定します。" /></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>ページナビゲーション</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="radio" name="data:pageinfo" value="1"'
        . ( ( $main::data{'pageinfo'} ) ? ' checked="checked"' : '' )
        . ' /> ページ位置情報のみ表示する</label></li>';
    print '<li><label><input type="radio" name="data:pageinfo" value="0"'
        . ( ( !$main::data{'pageinfo'} ) ? ' checked="checked"' : '' )
        . ' /> 各ページへのリンクを表示する</label>';
    print ' (<span class="additional-field">';
    print
        '<label for="data:pagelistmax">ページリンク最大表示数</label><input type="number" size="3" maxlength="3" name="data:pagelistmax" id="data:pagelistmax" value="'
        . $this->escape( $main::data{'pagelistmax'} )
        . '" /> (7以上)';
    print
        ' / <label for="data:pagelistmargin">前方ページリンク表示数</label><input type="number" size="3" maxlength="3" name="data:pagelistmargin" id="data:pagelistmargin" value="'
        . $this->escape( $main::data{'pagelistmargin'} ) . '" />';
    print '</span>)</li>';
    print '</ol>';
    print '</fieldset>';

    $this->print_page_button_setting;
    $this->print_close_button_setting;
    $this->print_sc_button_setting;

    print '<fieldset><legend>新着アイコン</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name">';
    print '<span class="datatitle">対象</span></td>';
    print
        '<td class="field"><label><input type="text" name="data:newIconLimit" id="data:newIconLimit" value="'
        . $this->escape( $main::data{'newIconLimit'} )
        . '" size="3" /> 日以内に作成された記事</label></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:newIconAlt">代替文字列</label></td>';
    print
        '<td class="field"><input type="text" name="data:newIconAlt" id="data:newIconAlt" value="'
        . $this->escape( $main::data{'newIconAlt'} )
        . '" size="40" /></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '<fieldset><legend>アイコン</legend>';
    print $cgi::Deco->image_menu( 'data:newIcon', 'new', 120,
        $main::data{'newIcon'} );
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset><legend>更新アイコン</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name">';
    print '<span class="datatitle">対象</span></td>';
    print
        '<td class="field"><label><input type="text" name="data:upIconLimit" id="data:upIconLimit" value="'
        . $this->escape( $main::data{'upIconLimit'} )
        . '" size="3" /> 日以内に更新された記事</label><div class="warning">「更新アイコン」よりも「新着アイコン」が優先されます。</div></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:upIconAlt">代替文字列</label></td>';
    print
        '<td class="field"><input type="text" name="data:upIconAlt" id="data:upIconAlt" value="'
        . $this->escape( $main::data{'upIconAlt'} )
        . '" size="40" /></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '<fieldset><legend>アイコン</legend>';
    print $cgi::Deco->image_menu( 'data:upIcon', 'new', 120,
        $main::data{'upIcon'} );
    print '</fieldset>';
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
}

######################################################################
# 設定チェック
sub is_valid_setting ($) {
    my $this = shift;
    $main::data{'pagelistmax'} = 7
        if $main::data{'pagelistmax'} && $main::data{'pagelistmax'} < 7;
    return;
}

######################################################################
# 登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {
        #クラス登録
        Common::class_register( 'b0zz', 'PageObj' );

        #ツール登録
        $setting::sub_jump_menu{'page'}{'b0zz'} = '共通部品設定';
        $setting::selected_jump_menu{'b0zz'} = 'b010'
            ; #サブメニューがあるときメインメニューのデフォルトキー

        #画像
        $setting::deco_dir{'new'}  = '新着アイコン';
        $setting::deco_type{'new'} = 'image';

        if ( !defined $default::subdivided || !$default::subdivided ) {
            $setting::deco_dir{'index'}  = 'インデックス ボタン';
            $setting::deco_type{'index'} = 'button';
            $setting::deco_dir{'next'}   = '「次」ボタン';
            $setting::deco_type{'next'}  = 'button';
            $setting::deco_dir{'prev'}   = '「前」ボタン';
            $setting::deco_type{'prev'}  = 'button';
        }
        $setting::deco_dir{'return'}  = '戻る ボタン';
        $setting::deco_type{'return'} = 'button';
        $setting::deco_dir{'close'}   = '閉じる ボタン';
        $setting::deco_type{'close'}  = 'button';
        if ( defined $env::checkout && $env::checkout ) {
            $setting::deco_dir{'check'}  = 'カートを見る ボタン';
            $setting::deco_type{'check'} = 'button';
        }
    }
}

1;
