package Field;
require CMONOS::CGI_Lib::Base;
@ISA = qw(Base);
use strict;
use utf8;
######################################################################
# フィールドモジュール
# Field.pm 1.2.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-05-22
######################################################################
#
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	(0)-0-0 オブジェクトを作る。
#		require CMONOS::CGI_Lib::Field;
#		obj = Field->new(key,id,tag,field_obj,field_for,setting);
# 			key				クラス登録キー
# 			id				フィールドID
# 			tag				フィールドタグ
# 			field_obj		\%field::obj
# 			field_for		\%field::for
# 			setting			設定（obj->set に準じます。）
#
#	(0)-0-1 Field::class_register('key','子クラスのパッケージ名','フィールド定義名')
#		クラスを登録します。子クラス側でキーワードとクラス名を登録することで、子クラスにオ
#		ブジェクトを結合させることができます。
#			例）子クラスのimportで、ClassRegisterを呼び出します。
#			sub import {
#				Field::class_register('key','子クラスのパッケージ名','フィールド定義名');
#			}
#
#	(0)-1 必要項目を設定する
#		obj->set(
#			'name' => '名前',					# フィールド名
#			'id' => '0',						# 設定DB名（フィールドID名には小文字か数字のみ使用可）
#			'field_style_disabled' => '1',		# 1=スタイル設定を無視する
#												# 2=フォント設定スタイル設定を無視する
#			'field_tag_enabled' => 'no'			# tag=タグのみ/all=全設定を付記/no=タグをつけない
#			'field_tag_name' => 'field'			# フィールドタグ <!--field--><!--/field-->
#		)
#
#		obj->plain			'field_style_disabled' => '2','field_tag_enabled' => 'no' に設定
#		obj->rich(tag)		'field_style_disabled' => '0','field_tag_enabled' => tag に設定
#
#	(0)-2 obj->menu
#		フィールド定義選択メニューを返します。
#
#	(0)-3 obj->db;
#		フィールドに対応するDB名を返します。
#
#	(0)-4 obj->name;
#		フィールドの名前を返します。
#
#	(0)-5 %data = obj->elements(str);
#		strをフィールドごとの書式にしたがって、データに分解します。
#
# ---------------------------------------------------------------------------------------
#	(1)-0 obj->match_enabled;
#		obj->search_formからパターンが入力されていれば"1"を返します。
#
#	(1)-1 obj->match([\x09\x0A\x0C\x0D\x20]tr);
#		obj->search_formから入力されたパターンとのパターンマッチを行います。
#		（成功すれば"1"を返します。）
# 			str				パターンマッチ対象文字列のリファレンス
#
# ---------------------------------------------------------------------------------------
#	(2)-0 obj->input_enabled;
#		入力フォームから入力できるフィールドかどうか(1/0)
#
#	(2)-1 obj->form(status,\ref,path);
#		入力フォームを返します（<form>タグは含みません）。
#		※　前後を<table>...</tbody></table>で囲んでください。
#		※　デフォルト値は、$data{'id'}{'str'}に設定されます。
#		※　送信されたデータは、$data{'id'}{'str'}で取り出せます。
#
# 			status			書き込み時の状態
# 								''=最初の書き込み=デフォルトを表示
# 								上記以外のときは直前に送信されたデータを表示
# 			ref				参照文のリファレンス
# 			path			ファイル保存ディレクトリのパス
#
# ---------------------------------------------------------------------------------------
#	(3) obj->is_valid_data;
#		obj->formから入力された文字列がフィールド定義に反しないかチェックします。
#		問題なければ"1"を返します。また、$main::data{$this->{'id'}}{'str'}を適切な書式に整形します。
#
# 		特殊ネーム
# 			"data:id:file"			添付ファイル専用の名前
# 			"data:id:comment"		コメント専用の名前
#
# 		特殊変数
# 			$mail::file				添付ファイル予約
#
# ---------------------------------------------------------------------------------------
#	(4)-1-0 obj->usual_format_enabled;
#		表示用に書き出せるフィールドかどうか(1/0)
#
#	(4)-1-1 obj->usual_format(output[...]);
#		表示用に整形します。
# 			output			書き出す文字列
# 			...				スクリプト特有の引数
# 								path		ファイル格納ディレクトリのパス
# 								category	カテゴリー番号
# 								num			データ番号
#
#	(4)-2-0 obj->format_for_mail_enabled;
#		表示用に書き出せるフィールドかどうか(1/0)
#
#	(4)-2-1 obj->format_for_mail(output...);
#		メール用に整形します。
#
#	(4)-3-0 obj->format_for_detail_enabled;
#		詳細表示用に書き出せるフィールドかどうか(1/0)
#
#	(4)-3-1 obj->format_for_detail(output...);
#		詳細表示用に整形します。
#
#	(4)-4-0 obj->format_for_text_enabled;
#		テキスト表示用に書き出せるフィールドかどうか(1/0)
#
#	(4)-4-1 obj->format_for_text(output...);
#		テキスト表示用に整形します。
#
#	(4)-5-0 obj->confirmation_enabled;
#		入力確認表示用に書き出せるフィールドかどうか(1/0)
#
#	(4)-5-1 obj->confirmation(str);
#		入力確認表示用に整形します。
#
# ---------------------------------------------------------------------------------------
#	(5) obj->print_setting_form;
#		フィールド設定フォームを返します（FieldEditor.pmと連動／<form>タグは含みません）。
#
# ---------------------------------------------------------------------------------------
#	(6) obj->is_valid_setting;
#		初期設定の入力チェックを行います。
#		エラー情報は$cgi::actionオブジェクトに渡されます。
#
# ---------------------------------------------------------------------------------------
#	(7)-0 obj->search_form_enabled;
#		入力フォームに書き出せるフィールドかどうか(1/0)
#
#	(7)-1 obj->search_form;
#		検索フォームを返します。
#		※　前後を<table>...</tbody></table>で囲んでください。
#		※　送信されたデータは、$data{'id'}{'str'}で取り出せます。
#
# ---------------------------------------------------------------------------------------
#	(8) obj->sort_enabled;
#		ソートできるフィールドかどうか(1/0)
#
# ---------------------------------------------------------------------------------------
#	(9) obj->cookie_enabled;
#		情報を記憶させることができるフィールドかどうか(1/0)
#
# ---------------------------------------------------------------------------------------
#	(10) obj->required_enabled;
#		必須項目に指定することが出来るフィールドかどうか(1/0)
#
# ---------------------------------------------------------------------------------------
#	(11) obj->update(DB,key);
#		特殊フィールド
# 			DB				操作するデータベースのあるディレクトリの位置
# 			key				操作するレコードのキー
#
######################################################################
# 	▼特殊変数
# 			$cgi::output	書き出し先パス
#
######################################################################
#オブジェクト作成
sub new ($;$$$$@) {
    my ( $pkg, $key, $id, $tag, $field_obj, $field_for, @setting ) = @_;
    my $this = {};
    if (   defined $key
        && defined $Field::modules{$key}
        && $Field::modules{$key} ne '' )
    {
        bless $this, $Field::modules{$key};
        $this->{'pkg'}   = $Field::modules{$key};
        $this->{'class'} = $key;
    }
    else {
        bless $this;
        $this->{'pkg'}   = 'Field';
        $this->{'class'} = 'text';
    }
    $this->{'id'}             = ($id)  ? $id  : 0;
    $this->{'field_tag_name'} = ($tag) ? $tag : 'field';
    if ( defined $field_obj && ref $field_obj ) {
        $this->{'field_obj'} = $field_obj;
    }
    else {
        $this->{'field_obj'} = \%field::obj if %field::obj;
    }
    if ( defined $field_for && ref $field_for ) {
        $this->{'field_for'} = $field_for;
    }
    else {
        $this->{'field_for'} = \%field::for if %field::for;
    }
    $this->set(@setting) if @setting;
    $this;
}

#クラス登録
sub class_register ($$$) {
    my ( $key, $pkg, $name ) = @_;
    $Field::modules{$key}   = $pkg;
    $Field::fieldname{$key} = $name;
}

#フィールドキーチェック
sub is_valid_key ($$$) {
    my ($key) = @_;
    return 1 if defined $Field::modules{$key};
}

# --------------------------------------------------------------------
#	(0)-1 初期設定
sub set ($;@) {
    my $this = shift;
    my ( $key, $op, %setting, @images );
    if ( @_ == 1 ) {
        my $setting = shift;
        while ( $setting
            =~ /[\x09\x0A\x0C\x0D\x20]*([0-9a-zA-Z_]+)[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]*)"/sg
            )
        {
            my ( $k, $v ) = ( $1, $2 );
            $v =~ s/&lt;/</sg;
            $v =~ s/&gt;/>/sg;
            $v =~ s/&quot;/"/sg;
            $v =~ s/&#10;/\x0A/sg;
            $v =~ s/&amp;/\&/sg;
            $setting{$k} = $v;
        }
    }
    else {
        %setting = @_;
    }

    my ( $k, $v );
    while ( ( $k, $v ) = each %setting ) {
        next
            if $k eq ''
            || $k eq 'class'
            || $k eq 'pkg'
            || $k eq 'id'
            || $k eq 'field_tag_name'
            || $k eq 'field_obj'
            || $k eq 'field_for'
            || $k eq 'ORIGINAL'
            || $k eq 'additional_setting'
            || !defined $v;
        $this->{'ORIGINAL'}->{$k} = $this->{$k}
            if defined $this->{$k}
            && ( !ref $this->{'ORIGINAL'}
            || !defined $this->{'ORIGINAL'}->{$k} );
        $this->{$k} = $v;
    }

    $this->{'cellAlign'} = $this->align
        if !defined $this->{'cellAlign'} || !$this->{'cellAlign'};
    $this->{'cellValign'} = 'middle'
        if !defined $this->{'cellValign'} || !$this->{'cellValign'};

    $this->{'fontface'} =~ s/"/'/sg
        if defined $this->{'fontface'} && $this->{'fontface'};
    $this->{'style'} =~ s/"/'/sg
        if defined $this->{'style'} && $this->{'style'};
    $this->{'helperEnabled'} = 1
        if defined $this->{'nostyle'} && !defined $this->{'helperEnabled'};

    $this->_set( \%setting );
}

sub set_additional_settings ($$) {
    my ( $this, $setting ) = @_;
    if ( $this->{'additional_setting'} ) {
        while ( $this->{'additional_setting'}
            =~ /[\x09\x0A\x0C\x0D\x20]*([0-9a-zA-Z_]+)="[^"]*"/sg )
        {
            if ( $1 ne 'pkg' && $1 ne 'class' && $1 ne 'id' ) {
                delete $this->{$1};
            }
        }
        delete $this->{'additional_setting'};
    }
    if ( ref $this->{'ORIGINAL'} ) {
        while ( my ( $k, $v ) = each %{ $this->{'ORIGINAL'} } ) {
            $this->{$k} = $v;
        }
        undef %{ $this->{'ORIGINAL'} };
        delete $this->{'ORIGINAL'};
    }
    delete $this->{'head'};
    delete $this->{'foot'};
    if ( $setting && $setting =~ /=/ ) {
        $this->set($setting);
        $setting
            =~ s/[\x09\x0A\x0C\x0D\x20]+(head|foot|id|class|pkg)="[^"]*"//gs;
        $this->{'additional_setting'} = $setting;
    }
}

sub restore ($) {
    my $this = shift;
    $this->set_additional_settings;
    return $this;
}

sub _set ($$) {
    my ( $this, $setting ) = @_;

    #テーブル設定の処理
    if ( ref $this->{'Table'} ) {
        if ( defined $$setting{'tableSetting'}
            && $$setting{'tableSetting'} ne '' )
        {
            $this->{'Table'}->reset_all->set( $$setting{'tableSetting'} );
        }
        elsif ( defined $$setting{'table'} && $$setting{'table'} ne '' ) {
            $this->{'Table'}->reset_all;
            if ( ref $table::obj{ $$setting{'table'} } ) {
                %{ $this->{'Table'} }
                    = %{ $table::obj{ $$setting{'table'} } };
            }
            elsif ( ref $setting::table{ $$setting{'table'} } ) {
                $this->{'Table'}
                    ->set( %{ $setting::table{ $$setting{'table'} } } );
            }
        }
    }
    else {
        require CMONOS::CGI_Lib::Table;
        if ( defined $$setting{'tableSetting'}
            && $$setting{'tableSetting'} ne '' )
        {
            $this->{'Table'} = Table->new( $$setting{'tableSetting'} );
        }
        elsif ( defined $$setting{'table'} && $$setting{'table'} ne '' ) {
            if ( ref $table::obj{ $$setting{'table'} } ) {
                $this->{'Table'} = Table->new;
                %{ $this->{'Table'} }
                    = %{ $table::obj{ $$setting{'table'} } };
            }
            elsif ( ref $setting::table{ $$setting{'table'} } ) {
                $this->{'Table'} = Table->new(
                    %{ $setting::table{ $$setting{'table'} } } );
            }
        }
        else {
            $this->{'Table'} = Table->new;
        }
    }
    $this->__set($setting);
}

sub __set ($$) {
    my ( $this, $setting ) = @_;
}

sub disabled ($) {
    my ( $this, $reset ) = @_;
    if ($reset) {
        delete $this->{'is_disabled'};
    }
    else {
        $this->{'is_disabled'} = 1;
    }
}

sub enabled ($) {
    my ( $this, $disabled ) = @_;
    if ($disabled) {
        $this->{'is_disabled'} = 1;
    }
    else {
        delete $this->{'is_disabled'};
    }
}

sub is_disabled ($) {
    my $this = shift;
    return ( $this->{'is_disabled'} ) ? 1 : 0;
}

sub plain ($) {
    my $this = shift;
    $this->{'field_style_disabled'} = 2;
    $this->{'field_tag_enabled'}    = 'no';
}

sub rich ($;$) {
    my ( $this, $tag ) = @_;
    $this->{'field_style_disabled'} = '';
    $this->{'field_tag_enabled'}    = $tag;
}

sub mk_user_generated_field_id ($;$$$) {
    my ( $this, $c, $n, $id ) = @_;
    $id = 'user-generated-content' if !$id;
    if ($default::id) {
        $id = $default::id;
        $id =~ s/^_+//;
        $id =~ s/_/-/g;
    }
    return $id . $this->_mk_user_generated_field_id( $c, $n );
}

sub _mk_user_generated_field_id ($;$$) {
    my ( $this, $c, $n ) = @_;
    my $id = ( ( defined $c && $c ne '' ) ? '-' . $c : '' )
        . ( ( defined $n && $n ne '' ) ? '-' . $n : '' );
    if ($default::static) {
        if ( $main::p && $main::p eq 'i' ) {
            $id .= '-info';
        }
        else {
            $id .= ($main::p) ? '-' . $main::p : '';
            $id
                .= ( ( defined $main::chap && $main::chap ne '' )
                ? '-' . $main::chap
                : '' );
            $id
                .= ( ( defined $main::sect && $main::sect ne '' )
                ? '-' . $main::sect
                : '' );
        }
    }
    return $id . '-' . $this->{'id'};
}

# 記事のパス
sub article_path ($;$$$) {
    my ( $this, $c, $n, $wantscalar ) = @_;
    my $url    = '';
    my $anchor = '';
    if ($default::static) {
        if ( $default::lib eq 'HomePageMakerLite' ) {
            $url = $default::home;
            $anchor
                = ( $main::p && $main::p eq 'i' )
                ? ''
                : '#a'
                . sprintf( "%.2d", $main::chap )
                . sprintf( "%.2d", $main::sect );
        }
        else {
            $url
                = ( ( !$master::home || $master::home eq $default::home )
                ? $default::home
                    . ( ( defined $c && $c ne '' ) ? '/' . $c : '' )
                : $default::home )
                . ( ( defined $n && $n ne '' ) ? '/' . $n : '' ) . '/'
                . (
                ($main::p) ? ( $main::p eq 'i' ) ? '1' : $main::p : 'index' )
                . '.'
                . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' );
            $anchor
                = ( $main::p && $main::p eq 'i' ) ? ''
                : '#a'
                . ( ( defined $main::p && $main::p ne '' )
                ? sprintf( "%.2d", $main::p )
                : ( $url =~ m!/1\.s?html?$!i ) ? '01'
                :                                '' )
                . sprintf( "%.2d", $main::chap )
                . sprintf( "%.2d", $main::sect );
        }
        $url
            .= 'index.'
            . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' )
            if $url =~ m!/$!;
    }
    else {
        my $dir
            = ( $ENV{'SCRIPT_NAME'} =~ m!/([a-zA-Z0-9]+)/start\.cgi! )
            ? $1
            : '';
        return (wantarray) ? ( '', '' ) : '' if !$dir;
        my $query = '';
        if ( $default::lib eq 'Baobab' ) {
            $query
                = ( ( defined $c && $c ne '' ) ? '&c=' . $c : '' )
                . ( ( defined $n && $n ne '' ) ? '&n=' . $n : '' )
                . '&v=listarticle';
            $query =~ s/^&//;
        }
        else {
            $query
                = 'm=DetailViewer'
                . ( ( defined $c && $c ne '' ) ? '&c=' . $c : '' )
                . ( ( defined $n && $n ne '' ) ? '&n=' . $n : '' );
        }
        $url = '../' . $dir . '/start.cgi?' . $query;
    }
    return ( !$wantscalar && wantarray ) ? ( $url, $anchor ) : $url . $anchor;
}

# --------------------------------------------------------------------
#	(0)-2 フィールドメニュー
sub menu ($$) {
    my ( $this, $name ) = @_;
    my ( $out, $code );
    $code = '';
    $out .= '<select name="' . $name . '">';
    foreach ( sort keys %Field::fieldname ) {
        if ( $_ =~ /^([a-zA-Z0-9]+)/ && $code ne $1 ) {
            $out .= '<option value="' . $this->{'class'} . '"></option>'
                if $code ne '';
            $code = $1;
        }
        $out
            .= '<option value="'
            . $_ . '"'
            . ( ( $_ eq $this->{'class'} ) ? ' selected="selected"' : '' )
            . '>'
            . $Field::fieldname{$_}
            . '</option>';
    }
    $out .= '</select>';
    return $out;
}

# --------------------------------------------------------------------
#	(0)-3 対応するDB
sub db ($) {
    my $this = shift;
    return ( defined $this->{'id'} && $this->{'id'} ne '' )
        ? $this->{'id'}
        : '';
}

# --------------------------------------------------------------------
#	(0)-4 フィールドの名前
sub name ($;$) {
    my ( $this, $name_only ) = @_;
    return
          ( defined $this->{'name'} )
        ? ( $name_only && $this->{'name'} =~ /^.*?:(.*)$/s )
            ? $1
            : $this->{'name'}
        : '[' . $this->{'id'} . ']';
}

# --------------------------------------------------------------------
#	(0)-5 データ要素の取得
sub elements ($$) {
    my ( $this, $str ) = @_;
    return ( 'str' => $str ) if !$this->{'elements'};
    return ( 'str' => '' )   if !defined $str;
    my ( %data, $check );
    while ( $str =~ s/^([0-9a-zA-Z_\.\,\-\+\/]+)\:([^\x0A\x0D]*)\x0A//s ) {
        $data{$1} = $2;
        $data{$1} =~ s/&#10;/\x0A/sg;
        $check = 1;
    }
    $str =~ s/^\x0A//;
    $str =~ s/^\x0A// if !defined $check || !$check;
    $data{'comment'} .= $str if defined $str && $str ne '';
    return %data;
}

# --------------------------------------------------------------------
#	(0)-6 設定の取得
sub get_setting ($;$) {
    my ( $this, $ignore_header ) = @_;
    my $attributes = '';
    while ( my ( $k, $v ) = each %{$this} ) {
        next
            if $k eq ''
            || $v eq ''
            || $k eq 'field_tag_name'
            || ref $v
            || ( $ignore_header && ( $k eq 'head' || $v eq 'foot' ) );
        $v =~ s/\&/&amp;/sg;
        $v =~ s/</&lt;/sg;
        $v =~ s/>/&gt;/sg;
        $v =~ s/"/&quot;/sg;
        $v =~ s/\x0A/\&#10;/g;
        $attributes .= ' ' . $k . '="' . $v . '"';
    }
    return $attributes;
}

# --------------------------------------------------------------------
#	(0)-7 設定のリセット
sub reset_all ($) {
    my $this = shift;
    my $temp = $this->{'class'};
    undef %{$this};
    $this->{'class'} = $temp;
}

# --------------------------------------------------------------------
#	(0)-8 obj->align;
sub align ($) {
    my $this = shift;
    return 'left';
}

######################################################################
#	(1)-0 obj->match_enabled;
sub match_enabled ($) {
    my $this = shift;
    return ( defined $main::search{ $this->{'id'} }{'str'}
            && $main::search{ $this->{'id'} }{'str'} ne '' ) ? 1 : 0;
}

#	(1) obj->match(str);
sub match ($;$) {
    my ( $this, $str ) = @_;
    return 1
        if !defined $main::search{ $this->{'id'} }{'str'}
        || $main::search{ $this->{'id'} }{'str'} eq '';
    $this->text_color($str);
    @{ $this->{'pattern'} } = $this->make_patterns(
        \$main::search{ $this->{'id'} }{'str'},
        (   ( $this->{'ignoreCase'} || $main::search{ $this->{'id'} }{'i'} )
            ? 1
            : 0
        ),
        $main::search{ $this->{'id'} }{'mode'}
    ) if !ref $this->{'pattern'};
    return $this->pattern_matched(
        $str,
        $this->{'pattern'},
        (   ( $this->{'ignoreCase'} || $main::search{ $this->{'id'} }{'i'} )
            ? 1
            : 0
        ),
        $main::search{ $this->{'id'} }{'mode'}
    );
}

######################################################################
#	(2) obj->input_enabled;
sub input_enabled ($) {
    my $this = shift;
    return 1;
}

sub form {
    my ( $this, $status, $original ) = @_;
    my ( $name, $form, @menu, $v, $colorform, $original_text );
    if (   ref $original eq 'HASH'
        && defined $original->{'str'}
        && $original->{'str'} ne '' )
    {
        $original_text = $original->{'str'};
        $original_text =~ s/<>[#0-9a-zA-z]{3,10}\Z//s
            if defined $this->{'color'} && $this->{'color'};
    }

    #色つき
    my $fontcolor = $this->text_color( \$main::data{ $this->{'id'} }{'str'} );
    if ( defined $this->{'color'} && $this->{'color'} ) {
        $main::data{ $this->{'id'} }{'fontcolor'} = $fontcolor
            if !defined $main::data{ $this->{'id'} }{'fontcolor'}
            || $main::data{ $this->{'id'} }{'fontcolor'} eq '';
        $colorform
            = '<div class="extra-field"><label for="data:'
            . $this->{'id'}
            . ':fontcolor" class="datatitle">文字色</label><input type="text" name="data:'
            . $this->{'id'}
            . ':fontcolor" id="data:'
            . $this->{'id'}
            . ':fontcolor" value="'
            . $this->escape( $main::data{ $this->{'id'} }{'fontcolor'} )
            . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:'
            . $this->{'id'}
            . ':fontcolor\',\'dom\')"'
            . ( ( $this->{'is_invalid_field'}{'fontcolor'} )
            ? ' class="invalid-field"'
            : ( $main::data{ $this->{'id'} }{'fontcolor'} )
            ? ' style="background-color: '
                . $this->escape( $main::data{ $this->{'id'} }{'fontcolor'} )
                . '"'
            : '' )
            . ' onchange="cmonos.validate.field(this).asColor()" />'
            . ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:'
            . $this->{'id'}
            . ':fontcolor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:'
            . $this->{'id'}
            . ':fontcolor\',\'window\')">'
            . '<img src="'
            . $env::image_dir
            . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>'
            . ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:'
            . $this->{'id'}
            . ':fontcolor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:'
            . $this->{'id'}
            . ':fontcolor\',\'window\')">'
            . '<img src="'
            . $env::image_dir
            . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
        $colorform .= '</div>';
    }

    #タイトルフィールドの場合
    if (   $this->{'field_for'}->{'title'}
        && $this->{'field_for'}->{'title'} eq $this->{'id'}
        && ( !defined $main::data{ $this->{'id'} }{'str'}
            || $main::data{ $this->{'id'} }{'str'} eq '' )
        && defined $original_text
        && $original_text ne ''
        )
    {
        my ($d);
        while ( $original_text
            =~ s/^(Re[\x09\x0A\x0C\x0D\x20]*:[\x09\x0A\x0C\x0D\x20]*|Re\[([0-9]+)\]:|Re\^([0-9]+):)//i
            )
        {
            if ( defined $2 && $2 ) {
                $d += $2;
            }
            elsif ( defined $3 && $3 ) {
                $d += $3;
            }
            else {
                $d++;
            }
        }
        $main::data{ $this->{'id'} }{'str'}
            = 'Re'
            . ( ($d) ? '[' . ( $d + 1 ) . ']' : '' ) . ':'
            . $original_text;
    }

    #名前フィールドの場合
    if (   $this->{'field_for'}->{'name'}
        && $this->{'field_for'}->{'name'} eq $this->{'id'}
        && ( !defined $main::data{ $this->{'id'} }{'str'}
            || $main::data{ $this->{'id'} }{'str'} eq '' )
        && ( $this->{'required'} || !defined $status || $status eq '' )
        )
    {
        $main::data{ $this->{'id'} }{'str'} = $this->get_user_name;
    }

    #デフォルト作成
    $main::data{ $this->{'id'} }{'str'} = $this->{'default'}
        if ( !defined $status || $status eq '' )
        && ( !defined $main::data{ $this->{'id'} }{'str'}
        || $main::data{ $this->{'id'} }{'str'} eq '' );

    #テーブル復元
    if (   ( !$this->{'nostyle'} || $this->{'nostyle'} <= 2 )
        && $main::data{ $this->{'id'} }{'str'}
        && $main::data{ $this->{'id'} }{'str'}
        =~ /<!--_[ct]sv(?![0-9a-zA-Z_])[^>]*-->/ )
    {
        $main::data{ $this->{'id'} }{'str'}
            = $this->{'Table'}
            ->text_to_sv( $main::data{ $this->{'id'} }{'str'},
            ( ( $this->{'csv'} ) ? 'no' : 'tag' ), 'csv' );
    }

    #テキスト入力欄
    $this->{'cols'} = '40'
        if !defined $this->{'cols'} || $this->{'cols'} eq '';

    #フィールドネーム+ヘッダ
    $form = $this->field_header( $this->{'cols'},
        'data:' . $this->{'id'} . ':str' );

    #高さが1以下の場合
    if ( !$this->{'rows'} || $this->{'rows'} <= 1 ) {
        if ( defined $main::data{ $this->{'id'} }{'str'}
            && $main::data{ $this->{'id'} }{'str'} ne '' )
        {
            if ( defined $this->{'nostyle'} && $this->{'nostyle'} ) {
                $main::data{ $this->{'id'} }{'str'} =~ s/\x0A//gs;
            }
            else {
                $main::data{ $this->{'id'} }{'str'} =~ s/\x0A/<br \/>/gs;
            }
        }
        $form
            .= '<input type="text" name="data:'
            . $this->{'id'}
            . ':str" id="data:'
            . $this->{'id'}
            . ':str" size="'
            . $this->{'cols'} . '"';
        $form .= ' maxlength="' . $this->{'maxlength'} . '"'
            if $this->{'maxlength'};
        $form
            .= ' value="'
            . $this->escape(
            $this->unescape_data_tag( $main::data{ $this->{'id'} }{'str'} ) )
            . '"';
        $form
            .= ' placeholder="'
            . $this->escape( $this->{'placeholder'} ) . '"'
            if defined $this->{'placeholder'} && $this->{'placeholder'} ne '';
        $form .= $this->set_invalid_class
            if $this->{'is_invalid_field'}{'str'};
        $form .= ' required="required"' if $this->{'required'};
        $form .= ' />';

        #単位の付加
        $form .= ' <span class="unit">' . $this->{'unit'} . '</span>'
            if defined $this->{'unit'} && $this->{'unit'};

        #色設定の付加
        $form .= $colorform if defined $this->{'color'} && $this->{'color'};

        #高さが2以上の場合
    }
    else {
        $form
            .= '<textarea name="data:'
            . $this->{'id'}
            . ':str" id="data:'
            . $this->{'id'}
            . ':str" cols="'
            . $this->{'cols'}
            . '" rows="'
            . $this->{'rows'}
            . '" wrap="soft"';
        $form
            .= ' placeholder="'
            . $this->escape( $this->{'placeholder'} ) . '"'
            if defined $this->{'placeholder'} && $this->{'placeholder'} ne '';
        $form .= $this->set_invalid_class
            if $this->{'is_invalid_field'}{'str'};
        $form .= ' required="required"' if $this->{'required'};
        $form .= '>';
        $form .= $this->escape(
            $this->blockquote_to_quotation(
                $this->unescape_data_tag(
                    $main::data{ $this->{'id'} }{'str'}
                ),
                $this->{'cols'}
            )
        );
        $form .= '</textarea>';
        $form .= $this->textarea_tool( 'data:' . $this->{'id'} . ':str',
            $original_text );
        $form .= $colorform if defined $this->{'color'} && $this->{'color'};
        $form
            .= '<div class="hints">カンマ区切り文字列は、自動的に表に変換されます。</div>'
            if defined $this->{'csv'} && $this->{'csv'};
    }

    $form .= $this->field_footer;
    return $form;
}

sub set_invalid_class ($) {
    my $this = shift;
    return
        ' class="invalid-field" onchange="cmonos.validate.field(this).bailOut()"';
}

sub textarea_tool ($$$;$) {
    my ( $this, $name, $original_text, $id ) = @_;
    return '' if $setting::field_for_mail || !$this->{'helperEnabled'};
    $id = $name if !$id;
    my $quote_flag = 0;
    my $form       = '<div class="toolbox">';
    require CMONOS::CGI_Lib::HTMLHelper;
    $form .= HTMLHelper->new->html_helper_menu(
        $name,
        (   ($original_text)
            ? '<option value="'
                . $this->jsquot(
                '\n'
                    . $this->blockquote_to_quotation_for_textarea_tool(
                    $original_text,
                    ( ( $this->{'cols'} ) ? $this->{'cols'} : 40 )
                    )
                )
                . '">オリジナルを引用</option>'
            : ''
        ),
        ( ( !$this->{'nostyle'} || $this->{'nostyle'} <= 1 ) ? 0 : 1 ),
        ( ( !$this->{'nostyle'} || $this->{'nostyle'} <= 2 ) ? 1 : 0 ),
        $id
    );
    $form
        .= ' <label id="'
        . $id
        . '-fit" class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\''
        . $name
        . '\').setAutoFit(this.checked);" />フィット</label>'
        if !$this->{'maxlength'} || $this->{'maxlength'} > 1000;

    if ( !$this->{'nostyle'} || $this->{'nostyle'} <= 1 ) {
        my $panel_type = $this->html_editor_panel_type;
        $form
            .= ' <label id="'
            . $id
            . '-editor" class="toggle-html-editor" title="HTMLエディタを使用する"><input type="checkbox"'
            . ( ( $this->{'editorPrepared'} ) ? ' checked="checked"' : '' )
            . ' onclick="cmonos.tool.htmlEditor.textarea(\''
            . $id
            . '\').toggle(this.checked,\''
            . $panel_type
            . '\');" />エディタ</label>';
        if ( $this->{'editorPrepared'} ) {
            $form
                .= $this->javascript( 'cmonos.tool.htmlEditor.textarea(\''
                    . $id
                    . '\').toggle(true,\''
                    . $panel_type
                    . '\')' );
        }
    }
    $form .= '</div>';
    return $form;
}

sub html_editor_panel_type ($) {
    my $this = shift;
    return
          ( $this->{'nostyle'} ) ? 'nostyle'
        : ( $this->{'noembed'} ) ? 'noimage'
        :                          'full';
}

sub field_header ($;$$) {
    my ( $this, $cols, $label_for ) = @_;
    my ( $form, $is_long, $name, $description, $group_name );
    $name        = $this->name;
    $description = $this->{'description'}
        if defined $this->{'description'} && $this->{'description'};
    if (!defined $this->{'ORDER'}
        || ( defined $Field::LAST_ORDER
            && $this->{'ORDER'} < $Field::LAST_ORDER )
        )
    {
        undef $Field::CURRENT_GROUP_NAME;
        undef $Field::LAST_ORDER;
    }
    if ( defined $Field::CURRENT_GROUP_NAME
        && $name =~ s/^$Field::CURRENT_GROUP_NAME://s )
    {
        #グループの続き
    }
    elsif ( $name =~ s/^(.*?)://s ) {
        $group_name                = $1;
        $Field::CURRENT_GROUP_NAME = quotemeta($group_name);
    }
    elsif ( defined $Field::CURRENT_GROUP_NAME ) {
        $group_name = "";
        undef $Field::CURRENT_GROUP_NAME;
    }
    $is_long
        = ( ( $cols && $cols > 80 ) || bytes::length $name > 45 ) ? 1 : 0;
    $Field::LAST_ORDER = $this->{'ORDER'} if defined $this->{'ORDER'};
    if ( defined $group_name ) {
        $form = '</tbody><tbody>' if !defined $Field::LAST_ORDER;
        $form .= '<tr class="tbody-start">';
        if ( $group_name ne '' ) {
            if ( $description && $description =~ s/^(.*?)\x0A\x0A;?//s ) {
                $form
                    .= '<th class="field-group-name">'
                    . $group_name
                    . '</th><td class="field-group-description">'
                    . $1
                    . '</td></tr><tr'
                    . ( ($is_long) ? ' class="single-cell"' : '' )
                    . ' id="field-'
                    . $this->{'id'} . '">';
            }
            else {
                $form
                    .= '<th class="field-group-name" colspan="2">'
                    . $group_name
                    . '</th></tr><tr'
                    . ( ($is_long) ? ' class="single-cell"' : '' )
                    . ' id="field-'
                    . $this->{'id'} . '">';
            }
        }
    }
    else {
        $form
            = '<tr id="field-'
            . $this->{'id'} . '"'
            . ( ($is_long) ? ' class="single-cell"' : '' ) . '>';
    }
    $form
        .= ($is_long)
        ? '<td colspan="2" class="field"><div class="field-name"'
        . ( ( !$label_for ) ? ' id="fieldlabel-' . $this->{'id'} . '"' : '' )
        . '>'
        : '<td class="field-name">';
    $form
        .= ($label_for)
        ? '<label id="fieldlabel-'
        . $this->{'id'}
        . '" for="'
        . $label_for . '">'
        : ($is_long) ? ''
        :   '<span id="fieldlabel-' . $this->{'id'} . '" class="datatitle">';
    $form .= '<strong>'  if $this->required_enabled && $this->{'required'};
    $form .= $name;
    $form .= '</strong>' if $this->required_enabled && $this->{'required'};
    $form .= ($label_for) ? '</label>' : ($is_long) ? '' : '</span>';
    $form .= ($is_long) ? '</div>' : '</td><td class="field">';
    $form .= '<div class="field-description">' . $description . '</div>'
        if defined $description && $description;
    return $form;
}

sub field_footer ($) {
    my $this = shift;
    return ( ( defined $this->{'note'} && $this->{'note'} ne '' )
        ? '<div class="note">' . $this->{'note'} . '</div>'
        : '' )
        . '</td></tr>';
}

######################################################################
#	(3) obj->is_valid_data(path);
sub is_valid_data ($;$) {
    my $this     = shift;
    my $is_valid = 0;
    delete $this->{'is_invalid_field'} if ref $this->{'is_invalid_field'};
    $this->escape_data_tag('str');

    #入力チェック
    $is_valid = 1;
    $is_valid = 0 if !$this->is_not_empty('str');
    $is_valid = 0 if !$this->is_not_too_long('str');
    $is_valid = 0 if !$this->is_not_taboo('str');
    $is_valid = 0 if !$this->is_valid_type_string('str');

    if ( defined $this->{'color'} && $this->{'color'} ) {
        if ( defined $main::data{ $this->{'id'} }{'fontcolor'}
            && $main::data{ $this->{'id'} }{'fontcolor'} ne '' )
        {
            if ($this->is_invalid_color(
                    $main::data{ $this->{'id'} }{'fontcolor'}
                )
                )
            {
                push(
                    @{ $cgi::action->{'dialog'} },
                    ( ( $this->{'name'} ) ? $this->{'name'} . 'の' : '' )
                        . '文字色書式を確認してください。'
                ) if ref $cgi::action;
                $main::data{ $this->{'id'} }{'fontcolor'} = '';
                $this->{'is_invalid_field'}{'fontcolor'}  = 1;
                $is_valid                                 = 0;
            }
            elsif ( defined $is_valid && $is_valid ) {
                $main::data{ $this->{'id'} }{'str'}
                    .= '<>' . $main::data{ $this->{'id'} }{'fontcolor'};
            }
        }
    }

    #名前フィールドの場合
    if (   defined $this->{'field_for'}->{'name'}
        && $this->{'field_for'}->{'name'} eq $this->{'id'}
        && defined $main::data{ $this->{'id'} }{'str'}
        && ( $main::data{ $this->{'id'} }{'str'}
            =~ s/^[\x09\x0A\x0C\x0D\x20]+//s
            || $main::data{ $this->{'id'} }{'str'}
            =~ s/[\x09\x0A\x0C\x0D\x20]+$//s
            || $main::data{ $this->{'id'} }{'str'} =~ s/^\x{3000}+//s
            || $main::data{ $this->{'id'} }{'str'} =~ s/\x{3000}+$//s
            || $main::data{ $this->{'id'} }{'str'}
            =~ s/[\x09\x0A\x0C\x0D\x20]{2,}/ /gs
            || $main::data{ $this->{'id'} }{'str'} =~ s/\x{3000}{2,}/ /gs )
        )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            ( ( $this->{'name'} ) ? $this->{'name'} . 'に' : '' )
                . '余分な空白があります。'
        ) if ref $cgi::action;
        $is_valid = 0;
        $this->{'is_invalid_field'}{'str'} = 1;
    }

    #クッキー
    if (   $is_valid
        && $this->{'cookie'}
        && defined $main::set_cookie
        && $main::set_cookie )
    {
        if ( $this->{'color'} && $this->{'colorcookie'} ) {
            $cookie::data{ 'data:' . $this->{'id'} . ':str' }
                = '<>' . $main::data{ $this->{'id'} }{'fontcolor'}
                if defined $main::data{ $this->{'id'} }{'fontcolor'}
                && $main::data{ $this->{'id'} }{'fontcolor'} ne '';
        }
        elsif ( defined $main::data{ $this->{'id'} }{'str'} ) {
            $cookie::data{ 'data:' . $this->{'id'} . ':str' }
                = $main::data{ $this->{'id'} }{'str'};
        }
    }
    return $is_valid;
}

#---------------------------------------------------------------------
#必須項目
sub is_valid_type_string ($$) {
    my ( $this, $name ) = @_;
    my $is_valid = 1;
    if (   defined $this->{'type'}
        && $this->{'type'} ne ''
        && defined $main::data{ $this->{'id'} }{$name}
        && $main::data{ $this->{'id'} }{$name} ne '' )
    {
        if ( $this->{'type'} eq 'h' ) {
            if ( $this->is_not_hiragana( $main::data{ $this->{'id'} }{$name} )
                )
            {
                push(
                    @{ $cgi::action->{'dialog'} },
                    ( ( $this->{'name'} ) ? $this->{'name'} : '入力' )
                        . 'を確認してください。ひらがなで入力して下さい。'
                ) if ref $cgi::action;
                $is_valid = 0;
            }
        }
        elsif ( $this->{'type'} eq 'k' ) {
            if ( $this->is_not_katakana( $main::data{ $this->{'id'} }{$name} )
                )
            {
                push(
                    @{ $cgi::action->{'dialog'} },
                    ( ( $this->{'name'} ) ? $this->{'name'} : '入力' )
                        . 'を確認してください。カタカナで入力して下さい。'
                ) if ref $cgi::action;
                $is_valid = 0;
            }
        }
        elsif ( $this->{'type'} eq 'a' ) {
            if ($this->is_not_word( $main::data{ $this->{'id'} }{$name}, 1 ) )
            {
                push(
                    @{ $cgi::action->{'dialog'} },
                    ( ( $this->{'name'} ) ? $this->{'name'} : '入力' )
                        . 'を確認してください。半角英数字で入力して下さい。'
                ) if ref $cgi::action;
                $is_valid = 0;
            }
        }
        elsif ( $this->{'type'} eq '1' ) {
            if ( $this->is_nan( $main::data{ $this->{'id'} }{$name}, 1 ) ) {
                push(
                    @{ $cgi::action->{'dialog'} },
                    ( ( $this->{'name'} ) ? $this->{'name'} : '入力' )
                        . 'を確認してください。数字で入力して下さい。'
                ) if ref $cgi::action;
                $is_valid = 0;
            }
        }
        elsif ( $this->{'type'} eq 'j' ) {
            if ( $this->is_not_japanese( $main::data{ $this->{'id'} }{$name} )
                )
            {
                push(
                    @{ $cgi::action->{'dialog'} },
                    ( ( $this->{'name'} ) ? $this->{'name'} : '入力' )
                        . 'を確認してください。日本語文字列がみつかりません。'
                ) if ref $cgi::action;
                $is_valid = 0;
            }
        }
    }
    if ($is_valid) {
        return 1;
    }
    else {
        $this->{'is_invalid_field'}{$name} = 1;
        return 0;
    }
}

#---------------------------------------------------------------------
#必須項目
sub is_not_empty ($$) {
    my ( $this, $name ) = @_;
    if (   defined $this->{'required'}
        && $this->{'required'}
        && ( !defined $main::data{ $this->{'id'} }{$name}
            || $main::data{ $this->{'id'} }{$name} eq '' )
        )
    {
        if ($default::english) {
            push(
                @{ $cgi::action->{'dialog'} },
                '"'
                    . (
                    ( defined $this->{'name'} && $this->{'name'} )
                    ? $this->{'name'}
                    : ''
                    )
                    . '" field is required.'
            ) if ref $cgi::action;
        }
        else {
            push(
                @{ $cgi::action->{'dialog'} },
                (   ( defined $this->{'name'} && $this->{'name'} )
                    ? $this->{'name'} . 'が'
                    : ''
                    )
                    . '入力されていません。'
            ) if ref $cgi::action;
        }
        $this->{'is_invalid_field'}{$name} = 1;
        return 0;
    }
    return 1;
}

#---------------------------------------------------------------------
#文字数チェック
sub is_not_too_long ($$;$) {
    my ( $this, $name, $subname ) = @_;
    $this->{'maxlength'} = $env::db_maxlength
        if ( !defined $default::static || !$default::static )
        && ( !$this->{'maxlength'}
        || $this->{'maxlength'} > $env::db_maxlength );
    if (   $this->{'maxlength'}
        && defined $main::data{ $this->{'id'} }{$name}
        && $this->str_length( $main::data{ $this->{'id'} }{$name} )
        > $this->{'maxlength'} )
    {
        if ($default::english) {
            push(
                @{ $cgi::action->{'dialog'} },
                '"'
                    . (
                    ( defined $this->{'name'} && $this->{'name'} )
                    ? $this->{'name'}
                    : ''
                    )
                    . '" field is too long.'
            ) if ref $cgi::action;
        }
        else {
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '入力' )
                    . (
                    ( defined $subname && $subname ) ? 'の' . $subname : ''
                    )
                    . 'を確認してください。'
                    . $this->{'maxlength'}
                    . '文字（半角）以内にしてください。'
            ) if ref $cgi::action;
        }
        $this->{'is_invalid_field'}{$name} = 1;
        return 0;
    }
    return 1;
}

#---------------------------------------------------------------------
#禁止文字列チェック
sub is_not_taboo ($$) {
    my ( $this, $name ) = @_;
    if ($this->is_taboo_string(
            $main::data{ $this->{'id'} }{$name}, $this->{'taboowords'},
            $this->{'tabooregexp'}
        )
        )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            (   ( defined $this->{'name'} && $this->{'name'} )
                ? $this->{'name'} . 'に'
                : ''
                )
                . '禁止された語句が存在します。'
        ) if ref $cgi::action;
        $this->{'is_invalid_field'}{$name} = 1;
        return 0;
    }
    return 1;
}

#---------------------------------------------------------------------
#フィールド添付ファイル情報を取得
sub get_field_file_info ($$;$$$$$$$) {
    my ($this, $path,   $width, $height, $ext,
        $name, $ignore, $fixed, $latlng
    ) = @_;
    my $file_path;
    if ( $path && $ext && $ext !~ /[^0-9a-zA-Z_]/ ) {
        $file_path
            = $path . '/'
            . $this->{'id'} . '.'
            . ( ( defined $name && $name ne '' && $name !~ /[^0-9a-zA-Z_]/ )
            ? $name . '.'
            : '' )
            . $ext;
        $file_path = '' if !-f $file_path;
    }
    if ( !$file_path ) {
        local (*DIR);
        return ( '', '', '', '' ) unless opendir( DIR, $path );
        my @files
            = grep( $_ =~ /^$this->{'id'}\.[0-9a-zA-Z_]+/
                && -f $path . '/' . $_
                && ( !$ignore
                || $_ !~ /^$this->{'id'}\.$ignore\.[0-9a-zA-Z_]+$/ ),
            readdir(DIR) );
        closedir(DIR);
        if ( $name && $name !~ /[^0-9a-zA-Z_]/ ) {
            ($file_path)
                = grep( $_ =~ /^$this->{'id'}\.$name\.[0-9a-zA-Z_]+$/,
                @files );
        }
        else {
            ($file_path)
                = grep( $_ =~ /^$this->{'id'}\.[0-9a-zA-Z_]+$/, @files );
        }
        if ( !$fixed && !$file_path && $files[0] ) {
            $file_path = $files[0];
            $name
                = ( $file_path
                    =~ /^$this->{'id'}\.([0-9a-zA-Z_]+)\.[0-9a-zA-Z_]+$/ )
                ? $1
                : '';
        }
        return ( '', '', '', '' ) if !$file_path;
        $ext = ( $file_path =~ /\.([0-9a-zA-Z_]+)$/ ) ? $1 : '';
        $file_path = $path . '/' . $file_path;
    }
    if ( ref $cgi::Image ) {
        my @info = $cgi::Image->info($file_path);
        if ( $info[2] ) {
            ( $width, $height ) = @info;
            if ( ref $info[4] ) {
                my ( $lat, $lng ) = $cgi::Image->latlng_from_exif( $info[4] );
                $latlng = $lat . ',' . $lng if $lat && $lng;
            }
        }
    }
    return ( $width, $height, $ext, $name, $latlng );
}

#---------------------------------------------------------------------
#データ埋め込みタグの除去
sub escape_data_tag ($$) {
    my ( $this, $k ) = @_;
    return if !defined $main::data{ $this->{'id'} }{$k};

    $main::data{ $this->{'id'} }{$k}
        =~ s/(<!--\/?([0-9a-zA-Z_:]+(?![0-9a-zA-Z_]))[^>]*-->)/$this->_escape_data_tag($2,$1)/sge;

#$main::data{$this->{'id'}}{$k} =~ s/<!--field(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/field(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--$this->{'field_tag_name'}(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/$this->{'field_tag_name'}(?![0-9a-zA-Z_])[^>]*-->//sg;
#
#$main::data{$this->{'id'}}{$k} =~ s/<!--data(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/data(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--head(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/head(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--foot(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/foot(?![0-9a-zA-Z_])[^>]*-->//sg;
#
#$main::data{$this->{'id'}}{$k} =~ s/<!--chapter(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/chapter(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--section(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/section(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--info(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/info(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--index(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/index(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--subindex(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/subindex(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--article(?![0-9a-zA-Z_])[^>]*-->//sg;
#$main::data{$this->{'id'}}{$k} =~ s/<!--\/article(?![0-9a-zA-Z_])[^>]*-->//sg;

    $main::data{ $this->{'id'} }{$k} =~ s/<>//sg;
}

sub _escape_data_tag ($$) {
    my ( $this, $tag_name, $tag ) = @_;
    if ( !grep( $tag_name eq $_, 'csv', 'tsv', 'tate', 'm:content' ) ) {
        $tag =~ s/</\&lt;/g;
        $tag =~ s/>/\&gt;/g;
    }
    return $tag;
}

sub unescape_data_tag ($$) {
    my ( $this, $html ) = @_;
    return '' if !defined $html;
    $html
        =~ s/(&lt;\!--\/?m:content(?![0-9a-zA-Z_])[^>]*--&gt;)/$this->unescape($1)/ges;
    $html =~ s/\&lt;(!--\/?[0-9a-zA-Z_]+(?![0-9a-zA-Z_]).*?--)\&gt;/<$1>/sg;
    return $html;
}

#---------------------------------------------------------------------
# データ整形
# データベース記録前処理
sub data ($;@) {
    my ( $this, @ignore ) = @_;
    $main::data{ $this->{'id'} }{'str'} = '';
    if ( ref $main::data{ $this->{'id'} } eq 'HASH' ) {
        while ( my ( $k, $v ) = each %{ $main::data{ $this->{'id'} } } ) {
            next
                if $k eq 'str'
                || $k eq 'comment'
                || grep( $k eq $_, @ignore ) > 0
                || !defined $v
                || $v eq '';
            $v =~ s/\x0A/\&#10;/sg;
            $main::data{ $this->{'id'} }{'str'} .= $k . ':' . $v . "\x0A";
        }
    }
    if ( defined $main::data{ $this->{'id'} }{'comment'}
        && $main::data{ $this->{'id'} }{'comment'} ne '' )
    {
        $main::data{ $this->{'id'} }{'str'} .= "\x0A"
            unless defined $main::data{ $this->{'id'} }{'str'}
            && $main::data{ $this->{'id'} }{'str'} ne '';
        $main::data{ $this->{'id'} }{'str'} .= "\x0A";
        $this->escape_data_tag('comment');
        $main::data{ $this->{'id'} }{'str'}
            .= $main::data{ $this->{'id'} }{'comment'};
    }
}

#---------------------------------------------------------------------
# 後処理
sub settle_up ($;$) {
    my ( $this, $path ) = @_;
    return 1;
}

######################################################################
#	(4)-1 obj->usual_format(str);
sub usual_format_enabled ($) {
    my $this = shift;
    return 1;
}

sub usual_format {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    my $fontcolor = $this->text_color( \$str );
    $this->text_conv( \$str );
    $this->text_fix( \$str, $this->{'csv'},
        ( ( !$this->{'rows'} || $this->{'rows'} <= 1 ) ? 1 : 0 ) )
        if defined $str && $str;
    return $this->field_tag(
        (   ($fontcolor)
            ? $this->data_tag_from_hash( 'fontcolor', $fontcolor )
            : ''
        )
        . $this->style(
            $this->data_tag( 'str', $str ),
            $this->{'style'},
            $this->{'fontsize'},
            (   ( $this->{'color'} && $fontcolor ) ? $fontcolor
                : $this->{'fontcolor'}
            ),
            $this->{'fontface'}
        )
    );
}

sub has_content {
    my ( $this, $data ) = @_;
    if ( ref $data eq 'HASH' ) {
        if ( defined $data->{'str'} ) {
            $this->text_color( \$data->{'str'} );
            return ( $data->{'str'} ne '' ) ? 1 : 0;
        }
        else {
            foreach my $k ( keys %$data ) {
                return 1 if defined $data->{$k} && $data->{$k} ne '';
            }
            return 0;
        }
    }
    else {
        $this->text_color( \$data );
        return ( defined $data && $data ne '' ) ? 1 : 0;
    }
}

sub image_file_url ($$$$) {
    my ( $this, $str, $path, $c, $n ) = @_;
    return '';
}

sub get_latlng ($$$$) {
    my ( $this, $str, $path, $c, $n ) = @_;
    return ( '', '' );
}

sub text_color ($$) {
    my ( $this, $str ) = @_;
    return ( defined $$str && $$str =~ s/<>([#0-9a-zA-z]{3,10})\Z//s )
        ? $1
        : '';
}

sub text_conv ($$) {
    my ( $this, $str ) = @_;
    return unless $this->{'convtable'} && $this->{'convtable'} =~ /=>/;
    while ( $this->{'convtable'} =~ /^(.+?)=>(.*)$/mg ) {
        my ( $f, $t ) = ( $1, $2 );
        if ( $this->{'convregexp'} ) {
            $t =~ s/\@/\\\@/g;
            $t =~ s/\%/\\\%/g;
            $t =~ s/\$(?![0-9])/\\\$/g;
            $t =~ s!/!\\/!g;
        }
        else {
            $f = quotemeta($f);
            $t = quotemeta($t);
        }
        eval( '$$str =~ s/$f/' . $t . '/g' );
    }
}

sub text_fix ($$;$$) {
    my ( $this, $str, $csv, $noquot ) = @_;
    if ( !defined $$str ) {
        $$str = '';
        return;
    }
    $csv++;
    unless ( ref $this->{'Table'} ) {
        require CMONOS::CGI_Lib::Table;
        $this->{'Table'} = Table->new;
    }
    if ( $this->{'nostyle'} && $this->{'nostyle'} == 3 ) {
        $this->detag($str);
    }
    elsif ( $this->{'nostyle'} && $this->{'nostyle'} == 2 ) {
        $$str =~ s/</\&lt;/g;
        $$str =~ s/>/\&gt;/g;
        $$str
            =~ s/&lt;(\!--(\/?[ct]sv|\/?tate|m:content)(?![0-9a-zA-Z_]).*?--)&gt;/<$1>/gs;
        $$str =~ s/&lt;br(?![0-9a-zA-Z_])[^>]*&gt;/<br \/>/igs;
    }
    else {
        $$str
            =~ s/(<!--m:content(?![0-9a-zA-Z_])[\x09\x0A\x0C\x0D\x20]?)(.*?)-->/$1 . $this->tag_fix($this->unescape($2),2,$this->{'noembed'},$this->{'nostyle'},$this->{'p'},$this->{'nofollow'},$this->{'jsenabled'},$this->{'iframeenabled'}) . '-->'/ges;
        $$str
            =~ s!<code class="html-source-code">(.*?)</code>!'<code _class="html-source-code">' . $this->escape_auto($this->escape($this->unescape_data_tag($1))) . '</code>'!igse;
    }

    #引用色付け/自動表作成/タグ修正/自動リンクタグ
    $$str = $this->quotation_to_blockquote($$str) if !$noquot;
    $$str
        = $this->{'Table'}->text_to_table( $$str, 'tag', $csv, \%table::obj );

    $this->autolist($str) if !$this->{'autolistdisabled'};
    $this->tag_fix( $str, 2, $this->{'noembed'}, $this->{'nostyle'},
        $this->{'p'}, $this->{'nofollow'}, $this->{'jsenabled'},
        $this->{'iframeenabled'} );
    $this->autoembed( $str, $this->{'movwidth'}, $this->{'movheight'} )
        if $this->{'autoembed'};
    $this->autolink( $str, $this->{'nofollow'}, $this->{'addressProtector'},
        $this->{'getTitle'} );
    $$str
        =~ s!<code _class="html-source-code">(.*?)</code>!'<code class="html-source-code">' . $this->unescape_auto($1) . '</code>'!igse;

    #縦書き
    if ( $$str
        =~ s/(<\!--tate(?![0-9a-zA-Z_])[^>]*-->)(.*?)(<\!--\/tate-->)/$1 . $this->tag_fix($2,2,$this->{'noembed'},$this->{'nostyle'},$this->{'p'},$this->{'nofollow'},1,$this->{'iframeenabled'}) . $3/egsi
        )
    {
        require CMONOS::CGI_Lib::Taketori;
        $$str
            = Taketori->new(
            { 'class' => 'tategaki', 'clear' => 1, 'css' => 1 } )
            ->text_to_tategaki( $$str, 1 );
        $this->tag_fix( $str, 0, $this->{'noembed'}, $this->{'nostyle'}, 0,
            $this->{'nofollow'}, 1, $this->{'iframeenabled'} )
            if $$str =~ /<!--_tate(?![0-9a-zA-Z_])[^>]*-->/;
    }
}

sub escape_auto ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/\(/\&#40;/g;
    $str =~ s/\)/\&#41;/g;
    $str =~ s/\+/\&#43;/g;
    $str =~ s/\-/\&#45;/g;
    $str =~ s/\./\&#46;/g;
    $str =~ s/:/\&#58;/g;
    $str =~ s/\@/\&#64;/g;
    $str =~ s/・/\&#x30FB;/g;
    return $str;
}

sub unescape_auto ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/\&#40;/\(/g;
    $str =~ s/\&#41;/\)/g;
    $str =~ s/\&#43;/\+/g;
    $str =~ s/\&#45;/\-/g;
    $str =~ s/\&#46;/\./g;
    $str =~ s/\&#58;/:/g;
    $str =~ s/\&#64;/\@/g;
    $str =~ s/\&#x30FB;/・/g;
    return $str;
}

#引用部分の処理
sub quotation_to_blockquote ($$) {
    my ( $this, $str ) = @_;
    return ''   if !defined $str;
    return $str if $str !~ /^(>|\&gt;|＞)/m;
    my ( $out, $line, $q );
    while ( $str =~ /^(.*?)$/mg ) {
        $line = $1 . "\x0A";
        if ( $line =~ /^(>|\&gt;|＞)/ ) {
            $q .= $line;
        }
        else {
            $out .= $this->_quotation_to_blockquote($q) if $q;
            $out .= $line;
            $q = '';
        }
    }
    $out .= $this->_quotation_to_blockquote($q) if $q;
    $out =~ s/\x0A$// if $str !~ /\x0A\Z/;
    return $out;
}

sub _quotation_to_blockquote ($$) {
    my ( $this, $quotation ) = @_;
    my ( $out, $count, $line, $level );
    $level = 1;
    $out   = '';
    while ( $quotation =~ /^(.*?)$/mg ) {
        $line = $1;
        my $this_level
            = ( $line =~ s/\G(>|\&gt;|＞)[\x09\x0A\x0C\x0D\x20]*//g );
        if ( $this_level > $level ) {
            my $n = $this_level - $level;
            $level = $this_level;
            $out .= '<blockquote>' x $n;
            $count += $n;
        }
        elsif ( $this_level < $level ) {
            my $n = $level - $this_level;
            $level = $this_level;
            $n = $count if $n > $count;
            $out .= '</blockquote>' x $n if $count;
            $count -= $n;
        }
        $out
            .= ( $this_level && $line eq '' )
            ? '<br /><br />'
            : $this->escape_except_html_entity($line);
    }
    $out .= '</blockquote>' while 0 < $count--;
    $out =~ s/(<br \/>){2,}/<br \/><br \/>/g;
    $out =~ s/(<br \/>)+$//s;
    $out =~ s/<\/blockquote>(<br \/>)+/<\/blockquote>/sg;
    $out =~ s/<\/blockquote>\x0A+/<\/blockquote>/sg;
    if ( $this->{'p'} ) {
        $out = '<p>' . $out . '</p>';
        $out
            =~ s!<br />[\x09\x0A\x0C\x0D\x20]*<br />[\x09\x0A\x0C\x0D\x20]*!</p><p>!sg;
        $out =~ s/(<\/?blockquote>)/<\/p>$1<p>/sg;
        $out =~ s/<p>[\x09\x0A\x0C\x0D\x20]*<\/p>//sg;
        $out =~ s/<p>(<blockquote>)/$1/sg;
        $out =~ s!(</blockquote>)</p>!$1!sg;
    }
    return '<blockquote class="quotation">' . $out . '</blockquote>';
}

sub blockquote_to_quotation_for_textarea_tool ($$$) {
    my ( $this, $original_text, $col ) = @_;
    $original_text =~ s/^[\x09\x0A\x0C\x0D\x20]+//s;
    $original_text =~ s/[\x09\x0A\x0C\x0D\x20]+$//s;
    return '' if !defined $original_text;
    $original_text = $this->quotation_to_blockquote($original_text)
        if $original_text !~ /<blockquote class="quotation">/
        && $original_text =~ /^(>|\&gt;|＞)/m;
    $original_text
        = $this->_blockquote_to_quotation(
        '<blockquote class="quotation">' . $original_text . '</blockquote>',
        $col );
    $original_text =~ s/\x0A([^>])/\x0A> $1/sg;
    $original_text =~ s/\x0A/\\n/sg;
    return $original_text;
}

sub blockquote_to_quotation ($$$) {
    my ( $this, $html, $col ) = @_;
    return '' if !defined $html;
    $html = $this->quotation_to_blockquote($html)
        if $html !~ /<blockquote class="quotation">/
        && $html =~ /^(>|\&gt;|＞)/m;
    $html
        =~ s/(<blockquote class="quotation">.*<\/blockquote>)/$this->_blockquote_to_quotation($1,$col)/seg;
    $html =~ s/\x0A{2,}/\x0A\x0A/g;
    return $html;
}

sub _blockquote_to_quotation ($$$) {
    my ( $this, $html, $col ) = @_;
    return '' if !defined $html;
    my ( $out, $count );
    $count = 0;
    while ( $html =~ /\G(.*?)(<\/?blockquote(?![0-9a-zA-Z_])[^>]*>|$)/sgi ) {
        my ( $text, $tag ) = ( $1, $2 );
        chomp $text;
        if ( $text ne '' ) {
            if ( $count > int( $col / 4 ) ) {
                my $n = int( $col / 4 );
                $out .= ( '> ' x ( $n - 1 ) ) . "\x0A";
                $out .= $this->unescape(
                    $this->jfold(
                        $text, $col - ( $n * 2 ),
                        'r',    '',
                        "\x0A", '> ' x $n
                    )
                );
            }
            elsif ($count) {
                $out .= $this->unescape(
                    $this->jfold(
                        $text, $col - ( $count * 2 ),
                        'r',    '',
                        "\x0A", '> ' x $count
                    )
                );
            }
            else {
                $out .= $text;
            }
            $out .= "\x0A";
        }
        if ( $tag =~ /<blockquote/ ) {
            $out .= "\x0A" if $tag =~ / class="quotation"/;
            if ( $count || $tag =~ / class="quotation"/ ) {
                $count++;
            }
            else {
                $out .= $tag;
            }
        }
        elsif ( $tag eq '</blockquote>' ) {
            if ($count) {
                $count--;
            }
            else {
                $out .= $tag;
            }
        }
    }
    return $out;
}

sub style ($$;$$$$$) {
    my ( $this, $str, $style, $fontsize, $fontcolor, $fontface, $legacy_mode )
        = @_;
    return $str
        if $this->{'data_only'}
        || ( defined $this->{'field_style_disabled'}
        && $this->{'field_style_disabled'} > 1 )
        || $str eq '';
    my ( $tag, $font_style );
    $tag
        = ( $str
            =~ /<(p(?![0-9a-zA-Z_])|div(?![0-9a-zA-Z_])|table(?![0-9a-zA-Z_])|[ou]l(?![0-9a-zA-Z_]))/
        ) ? 'div' : 'span';
    $style = '' if !defined $style;
    $font_style = '';

    #フォント
    if ($legacy_mode) {
        my $font_attribute = '';
        $font_attribute .= ' size="' . $fontsize . '"'
            if defined $fontsize && $fontsize ne '';
        $font_attribute .= ' color="' . $fontcolor . '"'
            if defined $fontcolor && $fontcolor ne '';
        $font_attribute .= ' face="' . $fontface . '"'
            if defined $fontface && $fontface ne '';
        $str = '<font' . $font_attribute . '>' . $str . '</font>'
            if defined $font_attribute && $font_attribute ne '';
        return $str
            if defined $this->{'field_style_disabled'}
            && $this->{'field_style_disabled'};
    }
    else {
        $font_style
            = ' font-size: ' . $this->font_size_to_style($fontsize) . ';'
            if defined $fontsize && $fontsize ne '';
        $font_style .= ' color: ' . $fontcolor . ';'
            if defined $fontcolor && $fontcolor ne '';
        $font_style .= ' font-family: ' . $fontface . ';'
            if defined $fontface && $fontface ne '';
        if ( defined $this->{'field_style_disabled'}
            && $this->{'field_style_disabled'} )
        {
            $str
                = '<'
                . $tag
                . ' style="'
                . $font_style . '">'
                . $str . '</'
                . $tag . '>'
                if defined $font_style && $font_style ne '';
            return $str;
        }
    }

    if ( $style =~ /:/ ) {
        $style
            = ' style="'
            . (
            ( defined $font_style && $font_style ne '' ) ? $font_style : '' )
            . $style . '"';
    }
    else {
        $style = ' class="' . $style . '"' if $style ne '';
        $style .= ' style="' . $font_style . '"'
            if defined $font_style && $font_style ne '';
    }

    $str = '<' . $tag . $style . '>' . $str . '</' . $tag . '>'
        if $style ne '';
    return $str;
}

sub font_size_to_style ($$) {
    my ( $this, $size ) = @_;
    if ( $size == 1 ) {
        return '0.67em';
    }
    elsif ( $size == 2 ) {
        return '0.83em';
    }
    elsif ( $size == 3 ) {
        return '1em';
    }
    elsif ( $size == 4 ) {
        return '1.17em';
    }
    elsif ( $size == 5 ) {
        return '1.5em';
    }
    elsif ( $size == 6 ) {
        return '2em';
    }
    elsif ( $size == 7 ) {
        return '3em';
    }
    else {
        return '1em';
    }
}

#データタグの作成
sub data_tag ($$$;$) {
    my ( $this, $name, $str, $txt ) = @_;
    my ($tag);
    return '' if !defined $str || $str eq '';
    if (   $this->{'field_tag_enabled'}
        && $this->{'field_tag_enabled'} eq 'no' )
    {
        $str = $txt if defined $txt && $txt ne '';
        $str .= '<span class="unit">' . $this->{'unit'} . '</span>'
            if defined $this->{'unit'} && $this->{'unit'} ne '';
        return $str;
    }
    if ( defined $txt && $txt ne '' ) {
        $tag
            = '<!--data name="'
            . $name
            . '" value="'
            . $this->lf_to_entity( $this->escape($str) ) . '"-->'
            . $txt
            . '<!--/data-->';
    }
    else {
        $tag = '<!--data name="' . $name . '"-->' . $str . '<!--/data-->';
    }
    $tag .= '<span class="unit">' . $this->{'unit'} . '</span>'
        if defined $this->{'unit'} && $this->{'unit'} ne '';
    return $tag;
}

#obj->data_tag(\%data,@ignore);ignoreはデータタグに含まない名前
sub data_tag_from_ref ($$@) {
    my ( $this, $data, @ig ) = @_;
    return ''
        if $this->{'field_tag_enabled'}
        && $this->{'field_tag_enabled'} eq 'no';
    my ( $tag, $k, $v );
    $tag = '';
    while ( ( $k, $v ) = each %$data ) {
        next if !defined $k || $k eq '';
        next if grep( $k eq $_, @ig );
        $tag
            .= '<!--data name="'
            . $k
            . '" value="'
            . $this->lf_to_entity( $this->escape($v) )
            . '"--><!--/data-->';
    }
    return $tag;
}

sub data_tag_from_hash ($@) {
    my ( $this, %data ) = @_;
    return ''
        if $this->{'field_tag_enabled'}
        && $this->{'field_tag_enabled'} eq 'no';
    return $this->data_tag_from_ref( \%data );
}

sub field_tag ($$;$) {
    my ( $this, $str, $invisible_flag ) = @_;
    if ( $this->{'data_only'} ) {
        delete $this->{'data_only'};
        $str =~ s/<!--(?!#include\s)[^>]*-->//gs;
        return ( defined $str ) ? $str : '';
    }
    my ( $tag, $content_included );
    if ( defined $str ) {
        my $content = $str;
        $content =~ s/<!--(?!#)[^>].*?-->//sg;
        $content =~ s/[\x09\x0A\x0C\x0D\x20]+//sg;
        $content_included = 1 if $content ne '';
        if ( $content_included && $this->{'link'} ) {
            $str =~ s/<\/?a(?=[\s>])[^>]*>//igs;
            $str = $this->{'link'} . $str . '</a>';
        }
    }
    else {
        $str = '';
    }
    if ( defined $this->{'field_tag_enabled'}
        && $this->{'field_tag_enabled'} eq 'no' )
    {
        return
            ( !$content_included || $invisible_flag ) ? ''
            : ( ( defined $this->{'head'} && $this->{'head'} ne '' )
            ? $this->field_name_replace( $this->{'head'} )
            : '' )
            . $str
            . ( ( defined $this->{'foot'} && $this->{'foot'} ne '' )
            ? $this->field_name_replace( $this->{'foot'} )
            : '' );
    }
    if ( $content_included && !$invisible_flag ) {
        return
              '<!--'
            . $this->{'field_tag_name'}
            . (
            (   defined $this->{'field_tag_enabled'}
                    && $this->{'field_tag_enabled'} eq 'all'
            ) ? $this->get_setting(1) : ' id="'
                . $this->{'id'} . '"'
                . (
                ( $this->{'additional_setting'} )
                ? $this->{'additional_setting'}
                : ''
                )
            )
            . '-->'
            . ( ( defined $this->{'head'} && $this->{'head'} ne '' )
            ? '<!--head-->'
                . $this->field_name_replace( $this->{'head'} )
                . '<!--/head-->'
            : '' )
            . $str
            . ( ( defined $this->{'foot'} && $this->{'foot'} ne '' )
            ? '<!--foot-->'
                . $this->field_name_replace( $this->{'foot'} )
                . '<!--/foot-->'
            : '' )
            . '<!--/'
            . $this->{'field_tag_name'} . '-->';
    }
    else {
        return
              '<!--'
            . $this->{'field_tag_name'}
            . (
            (   defined $this->{'field_tag_enabled'}
                    && $this->{'field_tag_enabled'} eq 'all'
            ) ? $this->get_setting(1) : ' id="'
                . $this->{'id'} . '"'
                . (
                ( $this->{'additional_setting'} )
                ? $this->{'additional_setting'}
                : ''
                )
            )
            . (
            ( defined $this->{'head'} && $this->{'head'} ne '' ) ? ' head="'
                . $this->lf_to_entity(
                $this->escape( $this->field_name_replace( $this->{'head'} ) )
                )
                . '"' : ''
            )
            . (
            ( defined $this->{'foot'} && $this->{'foot'} ne '' ) ? ' foot="'
                . $this->lf_to_entity(
                $this->escape( $this->field_name_replace( $this->{'foot'} ) )
                )
                . '"' : ''
            )
            . '-->'
            . $str . '<!--/'
            . $this->{'field_tag_name'} . '-->';
    }
}

sub data_only ($) {
    my $this = shift;
    $this->{'data_only'} = 1;
    return $this;
}

sub set_link ($$) {
    my ( $this, $anchor ) = @_;
    delete $this->{'link'};
    if ( defined $anchor ) {
        if ( ref $anchor ) {
            if ( ref $anchor eq 'HASH' ) {
                my $attr = '';
                while ( my ( $k, $v ) = each %$anchor ) {
                    $attr .= ' ' . $k . '="' . $this->escape($v) . '"'
                        if $k && $v;
                }
                $this->{'link'} = '<a' . $attr . '>' if $attr;
            }
        }
        elsif ( $anchor =~ /<a(?=\s)[^>]+>/is ) {
            $this->{'link'} = $anchor;
        }
    }
}

sub field_name_replace ($$) {
    my ( $this, $str ) = @_;
    return '' if !defined $str || $str eq '';
    my $name = $this->name;
    my $groupname = ( $name && $name =~ s/^(.*?)://s ) ? $1 : '';
    if ( defined $this->{'field_tag_enabled'}
        && $this->{'field_tag_enabled'} eq 'no' )
    {
        $str
            =~ s/<!--name(?![0-9a-zA-Z_])[^>]*-->.*?<!--\/name(?![0-9a-zA-Z_])[^>]*-->/$name/sgi;
        $str
            =~ s/<!--groupname(?![0-9a-zA-Z_])[^>]*-->.*?<!--\/groupname(?![0-9a-zA-Z_])[^>]*-->/$groupname/sgi;
    }
    else {
        $str
            =~ s/(<!--name(?![0-9a-zA-Z_])[^>]*-->).*?(<!--\/name(?![0-9a-zA-Z_])[^>]*-->)/$1$name$2/sg;
        $str
            =~ s/(<!--groupname(?![0-9a-zA-Z_])[^>]*-->).*?(<!--\/groupname(?![0-9a-zA-Z_])[^>]*-->)/$1$groupname$2/sgi;
    }
    return $str;
}

#---------------------------------------------------------------------
#	(4)-2 obj->format_for_mail(str);
sub format_for_mail_enabled ($) {
    my $this = shift;
    return 1;
}

sub format_for_mail {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return '' if !defined $str || $str eq '';
    $str = $this->blockquote_to_quotation( $str, 70 )
        if $this->{'rows'} && $this->{'rows'} > 1;
    $this->text_color( \$str );
    $this->text_conv( \$str );
    $str = '[' . $this->name . "]\x0A" . $str;
    $str .= ' ' . $this->{'unit'}
        if defined $this->{'unit'} && $this->{'unit'};
    $str =~ s/[\x0A]*\Z/\x0A\x0A/s;
    return $str;
}

#---------------------------------------------------------------------
#	(4)-3 obj->format_for_detail(str);
sub format_for_detail_enabled ($) {
    my $this = shift;
    return 1;
}

sub format_for_detail {
    my ( $this, $str, @data ) = @_;
    return $this->usual_format( $str, @data );
}

#---------------------------------------------------------------------
#	(4)-4 obj->format_for_text(str);
sub format_for_text_enabled ($) {
    my $this = shift;
    return 1;
}

sub format_for_text {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return '' if !defined $str || $str eq '';
    $str
        = $this->blockquote_to_quotation( $str,
        ( ( $this->{'cols'} ) ? $this->{'cols'} : 40 ) )
        if $this->{'rows'} && $this->{'rows'} > 1;
    $this->text_color( \$str );
    $this->text_conv( \$str );
    $str .= ' ' . $this->{'unit'} if $str ne '' && $this->{'unit'};
    return $str;
}

#---------------------------------------------------------------------
#	(4)-5 obj->confirmation(str);
sub confirmation_enabled ($) {
    my $this = shift;
    return 0;
}

sub confirmation ($$) {
    my ( $this, $str ) = @_;
    return '';
}

sub optional_values ($) {
    my $this = shift;
    my $form = '';
    if ( ref $main::option{ $this->{'id'} } ) {
        while ( my ( $k, $v ) = each %{ $main::option{ $this->{'id'} } } ) {
            next if $k eq 'str';
            if ( ref $v eq 'ARRAY' ) {
                my $n = 0;
                foreach my $value (@$v) {
                    $form
                        .= '<input type="hidden" name="option:'
                        . $this->{'id'} . ':'
                        . $k . '^'
                        . ( $n++ )
                        . '" value="'
                        . $this->escape($value) . '" />';
                }
            }
            else {
                $form
                    .= '<input type="hidden" name="option:'
                    . $this->{'id'} . ':'
                    . $k
                    . '" value="'
                    . $this->escape($v) . '" />';
            }
        }
    }
    return $form;
}

######################################################################
# 入力の確認
sub print_confirmation_form_table_row ($;@) {
    my ( $this, @option ) = @_;
    my ( $sample, $has_sample );
    if ( $this->confirmation_enabled ) {
        $sample = $this->confirmation( $main::data{ $this->{'id'} }{'str'},
            @option );
    }
    elsif ( $this->usual_format_enabled ) {
        local $this->{'is_temporary_data'} = 1;
        local $this->{'maxwidth'}          = 300;
        local $this->{'maxheight'}         = 300;
        $sample = $this->usual_format( $main::data{ $this->{'id'} }{'str'},
            @option );
    }
    if ( defined $sample && $sample ne '' ) {
        print $this->field_header( $this->{'cols'} );
        print $sample;
        $has_sample = 1;
        undef $sample;
    }
    print $this->hidden_confirmation_data;
    print '</td></tr>' if $has_sample;
    return 1;
}

sub hidden_confirmation_data ($) {
    my $this = shift;
    my $form = '';
    if ( $this->{'elements'} ) {
        while ( my ( $k, $v ) = each %{ $main::data{ $this->{'id'} } } ) {
            next if $k eq 'str';
            if ( ref $v eq 'ARRAY' ) {
                my $n = 0;
                foreach my $value (@$v) {
                    $form
                        .= '<input type="hidden" name="data:'
                        . $this->{'id'} . ':'
                        . $k . '^'
                        . ( $n++ )
                        . '" value="'
                        . $this->escape($value) . '" />';
                }
            }
            else {
                $form
                    .= '<input type="hidden" name="data:'
                    . $this->{'id'} . ':'
                    . $k
                    . '" value="'
                    . $this->escape($v) . '" />';
            }
        }
    }
    elsif ( defined $this->{'color'} && $this->{'color'} ) {
        my $fontcolor
            = $this->text_color( \$main::data{ $this->{'id'} }{'str'} );
        $form
            .= '<input type="hidden" name="data:'
            . $this->{'id'}
            . ':str" value="'
            . $this->escape( $main::data{ $this->{'id'} }{'str'} ) . '" />';
        $form
            .= '<input type="hidden" name="data:'
            . $this->{'id'}
            . ':fontcolor" value="'
            . $this->escape($fontcolor) . '" />';
    }
    elsif ( ref $main::data{ $this->{'id'} }{'str'} eq 'ARRAY' ) {
        my $n = 0;
        foreach my $value ( @{ $main::data{ $this->{'id'} }{'str'} } ) {
            $form
                .= '<input type="hidden" name="data:'
                . $this->{'id'} . ':str^'
                . ( $n++ )
                . '" value="'
                . $this->escape($value) . '" />';
        }
    }
    else {
        $form
            .= '<input type="hidden" name="data:'
            . $this->{'id'}
            . ':str" value="'
            . $this->escape( $main::data{ $this->{'id'} }{'str'} ) . '" />';
    }
    return $form;
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my ( $this, $html_disabled, $tips ) = @_;
    my ($name);

    #名前
    $name = 'data:' . $this->{'id'};

    $this->print_field_info_form($name);

    print '<fieldset>';
    print '<legend>入力欄設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_cols">入力欄横幅</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_cols" id="'
        . $name
        . '_cols" value="'
        . $this->escape( $this->{'cols'} )
        . '" size="3" title="入力欄の横幅を設定します。" /> <span class="unit">文字（半角）</span></td>';
    print '<td class="field-name"><label for="' . $name
        . '_rows">入力欄高さ</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_rows" id="'
        . $name
        . '_rows" value="'
        . $this->escape( $this->{'rows'} )
        . '" size="3" title="入力欄の高さを設定します。" /> <span class="unit">文字</span></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_maxlength">最大文字数</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_maxlength" id="'
        . $name
        . '_maxlength" value="'
        . $this->escape( $this->{'maxlength'} )
        . '" size="3" title="入力欄の文字数を制限します。" /> <span class="unit">文字（半角）</span></td>';
    print '<td class="field-name"><label for="' . $name
        . '_unit">単位</label></td>';
    print '<td class="q-field"><input type="text" name="'
        . $name
        . '_unit" id="'
        . $name
        . '_unit" value="'
        . $this->escape( $this->{'unit'} )
        . '" size="8" title="入力する内容に適した単位を設定します。" /></td>';
    print '</tr>';
    print '</tbody>';

    print '<tbody>';
    print '<tr class="tbody-end">';
    print '<td class="field-name"><label for="' . $name
        . '_type">入力チェック</label></td>';
    print '<td'
        . ( ( defined $html_disabled && $html_disabled )
        ? ' class="q-field" colspan="3"'
        : ' class="field"' )
        . '>';
    print '<select name="' . $name . '_type" id="' . $name . '_type">';
    print '<option value=""></option>';
    print '<option value="h"'
        . ( ( $this->{'type'} && $this->{'type'} eq 'h' )
        ? ' selected="selected"'
        : '' )
        . '>ひらがな</option>';
    print '<option value="k"'
        . ( ( $this->{'type'} && $this->{'type'} eq 'k' )
        ? ' selected="selected"'
        : '' )
        . '>カタカナ</option>';
    print '<option value="a"'
        . ( ( $this->{'type'} && $this->{'type'} eq 'a' )
        ? ' selected="selected"'
        : '' )
        . '>英数字</option>';
    print '<option value="1"'
        . ( ( $this->{'type'} && $this->{'type'} eq '1' )
        ? ' selected="selected"'
        : '' )
        . '>数字</option>';
    print '<option value="j"'
        . ( ( $this->{'type'} && $this->{'type'} eq 'j' )
        ? ' selected="selected"'
        : '' )
        . '>日本語文字列必須</option>';
    print '</select></td>';

    if ( !defined $html_disabled || !$html_disabled ) {
        print '<td class="field-name"><label for="' . $name
            . '_color">文字色設定</label></td>';
        print '<td class="q-field"><input type="checkbox" name="'
            . $name
            . '_color" id="'
            . $name
            . '_color" value="1"'
            . ( ( defined $this->{'color'} && $this->{'color'} )
            ? ' checked="checked"'
            : '' )
            . ' /> 使用';
        print ' (<input type="checkbox" name="'
            . $name
            . '_colorcookie" value="1"'
            . ( ( defined $this->{'colorcookie'} && $this->{'colorcookie'} )
            ? ' checked="checked"'
            : '' )
            . ' /> <label for="'
            . $name
            . '_colorcookie">文字色設定のみ Cookie に保存する</label>)</td>';
        print '</tr>';
    }
    else {
        print '</tr>';
    }

    print '</tbody>';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_default">デフォルト</label></td>';
    print '<td class="field" colspan="3"><textarea name="'
        . $name
        . '_default" id="'
        . $name
        . '_default" cols="'
        . ( ( $this->{'cols'} ) ? $this->{'cols'} : '72' )
        . '" rows="'
        . ( ( $this->{'rows'} ) ? $this->{'rows'} : '1' )
        . '" wrap="soft" title="入力欄のデフォルトの値を設定します。">'
        . $this->escape( $this->{'default'} )
        . '</textarea>'
        . ( ( defined $tips ) ? $tips : '' ) . '</td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_placeholder">プレースホルダ</label></td>';
    print '<td class="field" colspan="3"><input type="text" name="'
        . $name
        . '_placeholder" id="'
        . $name
        . '_placeholder" size="'
        . ( ( $this->{'cols'} ) ? $this->{'cols'} : '72' )
        . '" title="入力欄の placeholder 属性の値を設定します。" value="'
        . $this->escape( $this->{'placeholder'} )
        . '" /></td>';
    print '</tr>';
    print '</tbody></table>';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend>入力文字列制御</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_taboowords">禁止文字列</label></td>';
    print '<td class="field" colspan="3"><textarea name="'
        . $name
        . '_taboowords" id="'
        . $name
        . '_taboowords" cols="72" rows="4" wrap="off">'
        . $this->escape( $this->{'taboowords'} )
        . '</textarea>';
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\''
        . $name
        . '_taboowords\').setAutoFit(this.checked);" />フィット</label>';
    print '<div class="extra-field"><label><input type="checkbox" name="'
        . $name
        . '_tabooregexp" value="1"'
        . ( ( defined $this->{'tabooregexp'} && $this->{'tabooregexp'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 正規表現を使用する</label></div>';
    print '<div class="note">禁止文字列ごとに改行</div>';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_convtable">文字変換表</label></td>';
    print '<td class="field" colspan="3"><textarea name="'
        . $name
        . '_convtable" id="'
        . $name
        . '_convtable" cols="72" rows="4" wrap="off" title="下記書式に従って文字変換表を設定します。">'
        . $this->escape( $this->{'convtable'} )
        . '</textarea>';
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\''
        . $name
        . '_convtable\').setAutoFit(this.checked);" />フィット</label>';
    print '<div class="extra-field"><label><input type="checkbox" name="'
        . $name
        . '_convregexp" value="1"'
        . ( ( defined $this->{'convregexp'} && $this->{'convregexp'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 正規表現を使用する</label></div>';
    print
        '<div class="note">書式：変換対象文字列=&gt;変換後文字列[改行]</div>';
    print
        '<div class="example">例）、=>，（句読点の統一例）/SPAM=>spam（語句の統一例）</div>';
    print '</td>';
    print '</tr>';
    print '</tbody></table>';
    print '</fieldset>';

    $this->print_sub_setting_form($name);

    if ( defined $html_disabled && $html_disabled ) {
        print '<input type="hidden" name="' . $name
            . '_nostyle" value="2" />';
        print '<input type="hidden" name="' . $name . '_csv" value="0" />';

    }
    else {

        #タグの処理
        $this->print_html_tag_setting_form($name);

        #大文字と小文字を区別しない
        $this->print_ignorecase_setting_form($name);

        #文字設定
        $this->print_font_setting_form($name);

        #テーブル設定
        $this->print_table_setting_form($name);

    }

    $this->print_setting_save_button;
    return;
}

sub print_ignorecase_setting_form ($$;$) {
    my ( $this, $name, $description ) = @_;
    if ( !$default::static ) {
        print '<fieldset>';
        print '<legend>検索設定'
            . ( ( defined $description ) ? $description : '' )
            . '</legend>';
        print '<label><input type="checkbox" name="'
            . $name
            . '_ignoreCase" value="1"'
            . ( ( defined $this->{'ignoreCase'} && $this->{'ignoreCase'} )
            ? ' checked="checked"'
            : '' )
            . ' /> 常に大文字と小文字を区別しない</label>';
        print '</fieldset>';
    }
}

#---------------------------------------------------------------------
sub print_html_tag_setting_form($$;$) {
    my ( $this, $name, $description ) = @_;
    print '<fieldset class="cmtsetting">';
    print '<legend>HTML タグの処理'
        . ( ( defined $description ) ? $description : '' )
        . '</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="radio" name="'
        . $name
        . '_nostyle" value=""'
        . ( ( !defined $this->{'nostyle'} || $this->{'nostyle'} eq '' )
        ? ' checked="checked"'
        : '' )
        . ' /> HTML タグを許可する</label>';
    print '<ol class="extra-checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_noembed" value="1"'
        . ( ( defined $this->{'noembed'} && $this->{'noembed'} eq '1' )
        ? ' checked="checked"'
        : '' )
        . ' /> &lt;img&gt; や &lt;embed&gt; 等外部ファイルを埋め込むタグは禁止する</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_jsenabled" value="1"'
        . ( ( defined $this->{'jsenabled'} && $this->{'jsenabled'} eq '1' )
        ? ' checked="checked"'
        : '' )
        . ' />  &lt;script&gt; タグを許可する（チェックなしで禁止）</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_iframeenabled" value="1"'
        . (
        (   defined $this->{'iframeenabled'}
                && $this->{'iframeenabled'} eq '1'
        ) ? ' checked="checked"' : ''
        )
        . ' /> &lt;iframe&gt; タグを許可する（チェックなしで禁止）</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_helperEnabled" value="1"'
        . (
        (   defined $this->{'helperEnabled'}
                && $this->{'helperEnabled'} eq '1'
        ) ? ' checked="checked"' : ''
        ) . ' /> 入力支援ツールを使用する</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_editorPrepared" value="1"'
        . (
        (   defined $this->{'editorPrepared'}
                && $this->{'editorPrepared'} eq '1'
        ) ? ' checked="checked"' : ''
        )
        . ' /> 初期状態で HTML エディタ(<a href="http://nicedit.com/" target="_blank" title="NicEdit - Micro Inline WYSIWYG">nicEdit</a>)を使用する</label></li>';
    print '</ol></li>';
    print '<li><label><input type="radio" name="'
        . $name
        . '_nostyle" value="1"'
        . ( ( defined $this->{'nostyle'} && $this->{'nostyle'} eq '1' )
        ? ' checked="checked"'
        : '' )
        . ' /> HTML タグを許可するが、スタイル属性とクラス属性(style=""|class="")は無視する</label></li>';
    print '<li><label><input type="radio" name="'
        . $name
        . '_nostyle" value="2"'
        . ( ( defined $this->{'nostyle'} && $this->{'nostyle'} eq '2' )
        ? ' checked="checked"'
        : '' )
        . ' /> HTML タグをエスケープする</label></li>';
    print '<li><label><input type="radio" name="'
        . $name
        . '_nostyle" value="3"'
        . ( ( defined $this->{'nostyle'} && $this->{'nostyle'} eq '3' )
        ? ' checked="checked"'
        : '' )
        . ' /> HTML タグを除去する</label></li>';
    print '</ol>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_p" value="1"'
        . ( ( defined $this->{'p'} && $this->{'p'} eq '1' )
        ? ' checked="checked"'
        : '' )
        . ' /> 改行の連続を &lt;p&gt; に変換する</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_csv" value="1"'
        . ( ( defined $this->{'csv'} && $this->{'csv'} eq '1' )
        ? ' checked="checked"'
        : '' )
        . ' /> カンマ区切り文字列をテーブルに自動変換する</label></li>';
    print
        '<li><label title="rel=&quot;nofollow&quot;"><input type="checkbox" name="'
        . $name
        . '_nofollow" value="1"'
        . ( ( defined $this->{'nofollow'} && $this->{'nofollow'} )
        ? ' checked="checked"'
        : '' )
        . ' /> リンク先を検索エンジンに無視させる</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_addressProtector" value="1"'
        . (
        (   defined $this->{'addressProtector'} && $this->{'addressProtector'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> メールアドレスを JavaScript + CSS + HTML エンティティで保護する</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_autolistdisabled" value="1"'
        . (
        (   defined $this->{'autolistdisabled'} && $this->{'autolistdisabled'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> リストらしき文字列を自動的にリスト化<strong>しない</strong></label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_getTitle" value="1"'
        . ( ( defined $this->{'getTitle'} && $this->{'getTitle'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 自動リンクの際、ページタイトルを取得する'
        . ( ($default::static) ? '' : '【非推奨】' )
        . '</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_autoembed" value="1"'
        . ( ( defined $this->{'autoembed'} && $this->{'autoembed'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 動画(YouTube・DailyMotion・ニコニコ動画)を自動的に埋め込む</label>';
    print ' (<label><input type="number" name="'
        . $name
        . '_movwidth" size="4" value="'
        . $this->escape( $this->{'movwidth'} )
        . '" placeholder="幅" />px</label> &times; <label><input type="number" name="'
        . $name
        . '_movheight" size="4" value="'
        . $this->escape( $this->{'movheight'} )
        . '" placeholder="高さ" />px</label>)</li>';
    print '</ol>';
    print '</fieldset>';
}

#---------------------------------------------------------------------
sub print_table_setting_form($$;$$$) {
    my ( $this, $name, $description, $hints, $table_name,
        $table_setting_name )
        = @_;
    my $table_setting = $this->escape( $this->{'tableSetting'} );
    $description        = ''             if !defined $description;
    $table_name         = 'table'        if !$table_name;
    $table_setting_name = 'tableSetting' if !$table_setting_name;
    if ( !defined $hints || $hints eq '' ) {
        $hints
            = '<div class="hints">複数行テキストの場合のみ適用されます。</div>';
    }
    elsif ( $hints eq 'none' ) {
        $hints = '';
    }
    print <<FORM;
<fieldset class="cmtsetting">
<legend>テーブル$description</legend>$hints
<table class="field-table">
<tbody>
<tr>
<td class="field-name"><label for="${name}_$table_name">テーブルタイプ</label></td>
<td class="field">
FORM

    #テーブル設定選択
    $this->print_table_menu( $name, $table_name );

    print <<FORM;
</td></tr>
<tr><td class="field-name"><label for="${name}_$table_setting_name">オリジナル設定</label></td>
<td class="field">
<textarea name="${name}_$table_setting_name" id="${name}_$table_setting_name" cols="56" rows="5" wrap="off">$table_setting</textarea>
<div class="note">このフィールド独自の設定をする場合はここで設定します。</div>
</td></tr>
</tbody></table>
</fieldset>
FORM
}

#---------------------------------------------------------------------
sub print_field_info_form ($$;$$) {
    my ( $this, $name, $form_disabled, $memo ) = @_;
    my ( $field_name, $tr_class );
    $field_name = $this->jfold( $this->{'name'}, 50, 'r', '', '<br />' );
    $tr_class
        = ( !defined $form_disabled || !$form_disabled )
        ? ' class="tbody-end"'
        : '';
    $memo
        = ( defined $memo && $memo )
        ? ' <span class="warning">' . $memo . '</span>'
        : '';
    print <<FORM;
<fieldset>
<legend>基本情報</legend>
<table class="field-table">
<tbody>
<tr>
<td class="field-name"><span class="datatitle"><strong>名前</strong></span></td>
<td class="field"><strong>$field_name</strong></td>
</tr>
<tr$tr_class>
<td class="field-name"><span class="datatitle"><strong>タイプ</strong></span></td>
<td class="field"><strong>$Field::fieldname{$this->{'class'}}</strong>$memo</td>
</tr>
</tbody>
FORM
    if ( !defined $form_disabled || !$form_disabled ) {
        my $description = $this->escape( $this->{'description'} );
        my $note        = $this->escape( $this->{'note'} );
        print <<FORM;
<tbody>
<tr>
<td class="field-name"><label for="${name}_description">説明</label></td>
<td class="field"><textarea name="${name}_description" id="${name}_description" cols="56" rows="3" wrap="soft" title="入力欄の説明を設定します。入力欄の手前(上)に表示されます。">$description</textarea></td>
</tr>
<tr>
<td class="field-name"><label for="${name}_note">注意事項</label></td>
<td class="field"><textarea name="${name}_note" id="${name}_note" cols="56" rows="3" wrap="soft" title="入力欄の注意事項を設定します。入力欄の直後(下)に表示されます。">$note</textarea></td>
</tr>
</tbody>
FORM
    }

    print <<FORM;
</table>
</fieldset>
FORM
    return;
}

#---------------------------------------------------------------------
sub print_setting_save_button ($) {
    my $this = shift;
    print <<FORM;
<fieldset class="submit">
<input type="submit" value="保存" class="save" />
</fieldset>
FORM
    return;
}

#---------------------------------------------------------------------
sub print_sub_setting_form ($$) {
    my ( $this, $name ) = @_;
    return;
}

#---------------------------------------------------------------------
sub print_table_menu ($$;$) {
    my ( $this, $name, $name2 ) = @_;
    $name2 = 'table' if !defined $name2 || !$name2;
    print '<select name="'
        . $name . '_'
        . $name2
        . '" id="'
        . $name . '_'
        . $name2 . '">';
    print '<option value=""></option>';
    foreach ( sort { $a <=> $b || $a cmp $b } keys %table::obj ) {
        print '<option value="'
            . $_ . '"'
            . ( ( defined $this->{$name2} && $_ eq $this->{$name2} )
            ? ' selected="selected"'
            : '' )
            . '>'
            . $table::obj{$_}->{'name'}
            . '</option>';
    }
    print '</select>';
    return;
}

#---------------------------------------------------------------------
sub print_font_setting_form ($$;$) {
    my ( $this, $name, $description ) = @_;
    my ( $fontcolor, $style, $fontface );
    $fontcolor = ( $this->{'fontcolor'} ) ? $this->{'fontcolor'} : '';
    $fontface
        = ( $this->{'fontface'} ) ? $this->escape( $this->{'fontface'} ) : '';
    $style = ( $this->{'style'} ) ? $this->escape( $this->{'style'} ) : '';
    $description = '' if !defined $description;
    print <<FORM;
<fieldset class="cmtsetting">
<legend>文字設定$description</legend>
<table class="field-table">
<tbody>
<tr>
<td class="field-name"><label for="${name}_fontsize">文字サイズ</label></td>
<td class="q-field">
FORM
    print $this->mk_select_menu(
        $name . '_fontsize', '', '', $this->{'fontsize'},
        ''  => '',
        '1' => '小さめ',
        '2' => 'やや小さめ',
        '3' => '標準',
        '4' => 'やや大きめ',
        '5' => '大きめ',
        '6' => 'やや大きい',
        '7' => '大きい'
    );
    print <<FORM;
</td>
<td class="field-name"><label for="${name}_fontcolor">文字色</label></td>
<td class="q-field"><input type="text" name="${name}_fontcolor" id="${name}_fontcolor" value="$fontcolor" size="7" maxlength="7" style="background-color: $fontcolor" onchange="cmonos.validate.field(this).asColor()" />
 <a href="JavaScript:cmonos.tool.colorPalette.open('${name}_fontcolor')"><img src="$env::image_dir/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>
 <a href="JavaScript:cmonos.tool.colorSlider.open('${name}_fontcolor')"><img src="$env::image_dir/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>
</td>
</tr>
<tr>
<td class="field-name"><label for="${name}_fontface">フォント</label></td>
<td class="field" colspan="3"><input type="text" name="${name}_fontface" id="${name}_fontface" value="$fontface" size="56" /></td>
</tr>
<tr>
<td class="field-name"><label for="${name}_style">表示スタイル</label></td>
<td class="field" colspan="3"><input type="text" name="${name}_style" id="${name}_style" value="$style" size="56" />
<div class="note">このフィールドのスタイルを指定して下さい。</div>
<div class="note">":" を含まない場合はクラス指定と解釈されます。</div>
</td>
</tr>
</tbody></table>
</fieldset>
FORM
    return;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this     = shift;
    my $is_valid = 1;
    $is_valid = 0 if !$this->is_valid_text_setting;
    $is_valid = 0 if !$this->is_valid_sub_setting;
    $this->tag_fix( \$main::data{ $this->{'id'} . '_unit' }, 2 )
        if defined $main::data{ $this->{'id'} . '_unit' }
        && $main::data{ $this->{'id'} . '_unit' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };
    $main::data{ $this->{'id'} . '_ignoreCase' } = ''
        if !$main::data{ $this->{'id'} . '_ignoreCase' };
    return $is_valid;
}

sub is_valid_sub_setting ($) {
    my $this = shift;
    return 1;
}

sub is_valid_text_setting ($) {
    my $this     = shift;
    my $is_valid = 1;

    if (   defined $main::data{ $this->{'id'} . '_cols' }
        && $main::data{ $this->{'id'} . '_cols' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_cols' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '横幅を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_cols' };
        $is_valid = 0;
    }
    if (   defined $main::data{ $this->{'id'} . '_rows' }
        && $main::data{ $this->{'id'} . '_rows' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_rows' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '高さを更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_rows' };
        $is_valid = 0;
    }
    if (   defined $main::data{ $this->{'id'} . '_maxlength' }
        && $main::data{ $this->{'id'} . '_maxlength' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_maxlength' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '最大文字数を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_maxlength' };
        $is_valid = 0;
    }
    $is_valid = 0 if !$this->is_valid_font_setting;
    $main::data{ $this->{'id'} . '_placeholder' } =~ s/[\x0A\x0D]+//sg
        if defined $main::data{ $this->{'id'} . '_placeholder' };

    #チェックボックス対策
    $this->html_tag_setting_fix;
    $main::data{ $this->{'id'} . '_tabooregexp' } = ''
        if !defined $main::data{ $this->{'id'} . '_tabooregexp' }
        || $main::data{ $this->{'id'} . '_tabooregexp' } eq '';
    $main::data{ $this->{'id'} . '_convregexp' } = ''
        if !defined $main::data{ $this->{'id'} . '_convregexp' }
        || $main::data{ $this->{'id'} . '_convregexp' } eq '';
    $main::data{ $this->{'id'} . '_color' } = ''
        if !defined $main::data{ $this->{'id'} . '_color' }
        || $main::data{ $this->{'id'} . '_color' } eq '';
    $main::data{ $this->{'id'} . '_colorcookie' } = ''
        if !defined $main::data{ $this->{'id'} . '_colorcookie' }
        || $main::data{ $this->{'id'} . '_colorcookie' } eq '';
    return $is_valid;
}

sub html_tag_setting_fix ($) {
    my $this = shift;
    $main::data{ $this->{'id'} . '_nostyle' } = ''
        if !defined $main::data{ $this->{'id'} . '_nostyle' }
        || $main::data{ $this->{'id'} . '_nostyle' } eq '';
    $main::data{ $this->{'id'} . '_noembed' } = ''
        if !defined $main::data{ $this->{'id'} . '_noembed' }
        || $main::data{ $this->{'id'} . '_noembed' } eq '';
    $main::data{ $this->{'id'} . '_jsenabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_jsenabled' }
        || $main::data{ $this->{'id'} . '_jsenabled' } eq '';
    $main::data{ $this->{'id'} . '_iframeenabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_iframeenabled' }
        || $main::data{ $this->{'id'} . '_iframeenabled' } eq '';
    $main::data{ $this->{'id'} . '_helperEnabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_helperEnabled' }
        || $main::data{ $this->{'id'} . '_helperEnabled' } eq '';
    $main::data{ $this->{'id'} . '_editorPrepared' } = ''
        if !defined $main::data{ $this->{'id'} . '_editorPrepared' }
        || $main::data{ $this->{'id'} . '_editorPrepared' } eq '';
    $main::data{ $this->{'id'} . '_p' } = ''
        if !defined $main::data{ $this->{'id'} . '_p' }
        || $main::data{ $this->{'id'} . '_p' } eq '';
    $main::data{ $this->{'id'} . '_csv' } = ''
        if !defined $main::data{ $this->{'id'} . '_csv' }
        || $main::data{ $this->{'id'} . '_csv' } eq '';
    $main::data{ $this->{'id'} . '_autolistdisabled' } = ''
        if !defined $main::data{ $this->{'id'} . '_autolistdisabled' }
        || $main::data{ $this->{'id'} . '_autolistdisabled' } eq '';
    $main::data{ $this->{'id'} . '_nofollow' } = ''
        if !defined $main::data{ $this->{'id'} . '_nofollow' }
        || $main::data{ $this->{'id'} . '_nofollow' } eq '';
    $main::data{ $this->{'id'} . '_addressProtector' } = ''
        if !defined $main::data{ $this->{'id'} . '_addressProtector' }
        || $main::data{ $this->{'id'} . '_addressProtector' } eq '';
    $main::data{ $this->{'id'} . '_getTitle' } = ''
        if !defined $main::data{ $this->{'id'} . '_getTitle' }
        || $main::data{ $this->{'id'} . '_getTitle' } eq '';
    $main::data{ $this->{'id'} . '_autoembed' } = ''
        if !$main::data{ $this->{'id'} . '_autoembed' };

    if (   defined $main::data{ $this->{'id'} . '_movwidth' }
        && $main::data{ $this->{'id'} . '_movwidth' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_movwidth' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '埋め込み動画の幅を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_movwidth' };
    }
    if (   defined $main::data{ $this->{'id'} . '_movheight' }
        && $main::data{ $this->{'id'} . '_movheight' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_movheight' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '埋め込み動画の高さを更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_movheight' };
    }
}

sub is_valid_font_setting ($) {
    my $this     = shift;
    my $is_valid = 1;
    if (   defined $main::data{ $this->{'id'} . '_fontcolor' }
        && $main::data{ $this->{'id'} . '_fontcolor' } ne ''
        && $this->is_invalid_color(
            $main::data{ $this->{'id'} . '_fontcolor' }
        )
        )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '文字色を更新できませんでした。色設定を確認してください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_fontcolor' };
        $is_valid = 0;
    }
    return $is_valid;
}

sub is_valid_file_setting ($;$) {
    my ( $this, $title ) = @_;
    my $is_valid = 1;
    $title = ($title) ? $title . 'の' : '';
    if (   defined $main::data{ $this->{'id'} . '_maxsize' }
        && $main::data{ $this->{'id'} . '_maxsize' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_maxsize' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . 'ファイルサイズ制限設定を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_maxsize' };
        $is_valid = 0;
    }
    if (   defined $main::data{ $this->{'id'} . '_maxsize' }
        && $main::data{ $this->{'id'} . '_maxsize' } ne ''
        && ( $env::post_max_size - 128000 )
        < $main::data{ $this->{'id'} . '_maxsize' } )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . 'ファイルサイズ制限設定を更新できませんでした。'
                . ( $env::post_max_size - 128000 )
                . 'バイトより小さな値にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_maxsize' };
        $is_valid = 0;
    }
    if (   defined $main::data{ $this->{'id'} . '_widthlimit' }
        && $main::data{ $this->{'id'} . '_widthlimit' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_widthlimit' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . '横幅制限設定を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_widthlimit' };
        $is_valid = 0;
    }
    if (   defined $main::data{ $this->{'id'} . '_heightlimit' }
        && $main::data{ $this->{'id'} . '_heightlimit' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_heightlimit' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . '高さ制限設定を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_heightlimit' };
        $is_valid = 0;
    }
    if (   defined $main::data{ $this->{'id'} . '_maxwidth' }
        && $main::data{ $this->{'id'} . '_maxwidth' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_maxwidth' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . '最大横幅を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_maxwidth' };
        $is_valid = 0;
    }
    elsif ($main::data{ $this->{'id'} . '_maxwidth' }
        && $main::data{ $this->{'id'} . '_widthlimit' }
        && $main::data{ $this->{'id'} . '_maxwidth' }
        > $main::data{ $this->{'id'} . '_widthlimit' } )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . '最大横幅は横幅制限設定より小さな値にしてください。'
        ) if ref $cgi::action;
        $main::data{ $this->{'id'} . '_maxwidth' }
            = $main::data{ $this->{'id'} . '_widthlimit' };
    }
    if (   defined $main::data{ $this->{'id'} . '_maxheight' }
        && $main::data{ $this->{'id'} . '_maxheight' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_maxheight' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . '最大高さを更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_maxheight' };
        $is_valid = 0;
    }
    elsif ($main::data{ $this->{'id'} . '_maxheight' }
        && $main::data{ $this->{'id'} . '_heightlimit' }
        && $main::data{ $this->{'id'} . '_maxheight' }
        > $main::data{ $this->{'id'} . '_heightlimit' } )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . '最大高さは高さ制限設定より小さな値にしてください。'
        ) if ref $cgi::action;
        $main::data{ $this->{'id'} . '_maxheight' }
            = $main::data{ $this->{'id'} . '_heightlimit' };
    }
    if (   defined $main::data{ $this->{'id'} . '_previewwidthlimit' }
        && $main::data{ $this->{'id'} . '_previewwidthlimit' } ne ''
        && $this->is_nan(
            $main::data{ $this->{'id'} . '_previewwidthlimit' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . 'サムネイル画像横幅制限設定を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_previewwidthlimit' };
        $is_valid = 0;
    }
    elsif ($main::data{ $this->{'id'} . '_previewwidthlimit' }
        && $main::data{ $this->{'id'} . '_maxwidth' }
        && $main::data{ $this->{'id'} . '_previewwidthlimit' }
        > $main::data{ $this->{'id'} . '_maxwidth' } )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . 'サムネイル画像横幅制限設定は最大横幅より小さな値にしてください。'
        ) if ref $cgi::action;
        $main::data{ $this->{'id'} . '_previewwidthlimit' }
            = $main::data{ $this->{'id'} . '_maxwidth' };
    }
    if (   defined $main::data{ $this->{'id'} . '_previewheightlimit' }
        && $main::data{ $this->{'id'} . '_previewheightlimit' } ne ''
        && $this->is_nan(
            $main::data{ $this->{'id'} . '_previewheightlimit' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . 'サムネイル画像高さ制限設定を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_previewheightlimit' };
        $is_valid = 0;
    }
    elsif ($main::data{ $this->{'id'} . '_previewheightlimit' }
        && $main::data{ $this->{'id'} . '_maxheight' }
        && $main::data{ $this->{'id'} . '_previewheightlimit' }
        > $main::data{ $this->{'id'} . '_maxheight' } )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            $title
                . 'サムネイル画像高さ制限設定は最大高さより小さな値にしてください。'
        ) if ref $cgi::action;
        $main::data{ $this->{'id'} . '_previewheightlimit' }
            = $main::data{ $this->{'id'} . '_maxheight' };
    }

    return $is_valid;
}

######################################################################
#	(7) obj->search_form;
sub search_form_enabled ($) {
    my $this = shift;
    return 1;
}

sub search_form ($) {
    my $this = shift;
    my ($form);
    $form
        = '<tr><td class="field-name"><label for="search:'
        . $this->{'id'}
        . ':str">'
        . $this->{'name'}
        . '</label></td><td class="field">';
    $form
        .= '<input type="text" size="30" id="search:'
        . $this->{'id'}
        . ':str" name="search:'
        . $this->{'id'}
        . ':str" value="'
        . $this->escape( $main::search{ $this->{'id'} }{'str'} ) . '" /> ';

    #英語表記
    if ( defined $default::english && $default::english ) {
        $form .= $this->mk_select_menu(
            'search:' . $this->{'id'} . ':mode', '', '',
            $main::search{ $this->{'id'} }{'mode'},
            'and' => 'Matches on all words',
            'or'  => 'Matches on any words',
            'A'   => 'Begins with any words',
            'Z'   => 'Finishes with any words',
            'eq'  => 'Equal to any words',
        );
        $form
            .= '<div class="extra-field"><label><input type="checkbox" name="search:'
            . $this->{'id'}
            . ':i" value="1"'
            . (
            (   defined $main::search{ $this->{'id'} }{'i'}
                    && $main::search{ $this->{'id'} }{'i'}
            ) ? ' checked="checked"' : ''
            )
            . ' /> Do case-insensitive pattern matching.</label></div>'
            if !$this->{'ignoreCase'};

        #日本語
    }
    else {
        $form .= $this->mk_select_menu(
            'search:' . $this->{'id'} . ':mode', '', '',
            $main::search{ $this->{'id'} }{'mode'},
            'and' => '全ての文字列を含む',
            'or'  => 'いずれかの文字列を含む',
            'A'   => 'いずれかの文字列から始まる',
            'Z'   => 'いずれかの文字列で終わる',
            'eq'  => 'いずれかの文字列に一致する',
        );
        $form
            .= '<div class="extra-field"><label><input type="checkbox" name="search:'
            . $this->{'id'}
            . ':i" value="1"'
            . (
            (   defined $main::search{ $this->{'id'} }{'i'}
                    && $main::search{ $this->{'id'} }{'i'}
            ) ? ' checked="checked"' : ''
            )
            . ' /> 大文字と小文字を区別しない</label></div>'
            if !$this->{'ignoreCase'};
    }
    $form .= '</td></tr>';
    return $form;
}

######################################################################
#	(8) obj->sort_enabled;
sub sort_enabled ($) {
    my $this = shift;
    return 1;

#return ($this->{'class'} !~ /^text/ || !defined $this->{'rows'} || $this->{'rows'} eq '' || $this->{'rows'} <= 1) ? 1 : 0;
}

sub sample ($$) {
    my ( $this, $str ) = @_;
    return '' if !defined $str || $str eq '';
    $this->text_color( \$str );
    $str =~ s!<script[^>]*>.*?</script>!!igs;
    $str =~ s!<style[^>]*>.*?</style>!!igs;
    $str =~ s/<(.*?)>//gs;
    $str =~ s/\s+/\x20/gs;
    $str =~ s/\x0D/\x0A/gs;
    $str = $this->z_to_h( $this->h_to_z( $str, 'utf8' ), 'utf8' );
    $str =~ s/([\x{3041}-\x{3096}])/pack("U",unpack("U",$1) + 96)/ges;
    $str = lc $str;
    return ( $str =~ /^(.{1,64})/s ) ? $1 : $str;
}

######################################################################
#	(9) obj->cookie_enabled;
sub cookie_enabled ($) {
    my $this = shift;
    return 1;
}

######################################################################
#	(10) obj->required_enabled;
sub required_enabled ($) {
    my $this = shift;
    return 1;
}

######################################################################
#	(11) obj->update;
sub update ($;$) {
    my $this = shift;
    return 0;
}

sub update_enabled ($) {
    my $this = shift;
    return 0;
}

sub is_secure_user_for_update ($) {
    my $this = shift;
    return 0;
}

sub save_update_target ($;$) {
    my $this = shift;
    $this->{'updated'}{'c'} = ( defined $main::c ) ? $main::c : '';
    $this->{'updated'}{'n'} = ( defined $main::n ) ? $main::n : '';
}

sub is_updated_record ($$$) {
    my ( $this, $c, $n ) = @_;
    $c = '' if !defined $c;
    $n = '' if !defined $n;
    return (   !$default::static
            && ref $this->{'updated'}
            && defined $this->{'updated'}{'c'}
            && defined $this->{'updated'}{'n'}
            && $this->{'updated'}{'c'} eq $c
            && $this->{'updated'}{'n'} eq $n ) ? 1 : 0;
}

######################################################################
sub get_enclosure_attributes ($$$$$) {
    my $this = shift;
    return ();
}

######################################################################
sub editor_enabled ($) {
    my $this = shift;
    return 0;
}

sub is_secure_user_for_edit ($) {
    my $this = shift;
    return 0;
}

sub set_user_generated_field_editor_page_settings ($) {
    my $this = shift;
    return 0;
}

sub print_user_generated_field_editor ($) {
    my $this = shift;
    $this->error404;
}

sub user_generated_field_edit ($) {
    my $this = shift;
    return 0;
}

######################################################################
#sub import {
BEGIN {
    Field::class_register( 'text', 'Field', '文字列' );
}

1;
