package LinkToSearchEngine;
require CMONOS::CGI_Lib::Field;
@ISA = qw(Field);
use strict;
use utf8;
######################################################################
# フィールド定義モジュール（URL）
# Url.pm 1.1.4
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2013-06-08
######################################################################
sub _set ($$) {
    my ( $this, $setting ) = @_;
    unless ( ref $cgi::Image ) {
        require CMONOS::CGI_Lib::Image;
        $cgi::Image = Image->new;
    }
    unless ( ref $cgi::Deco ) {
        require CMONOS::CGI_Lib::Deco;
        $cgi::Deco = Deco->new;
    }
    $this->{'url'} = 'https://www.google.co.jp/search?q=%w'
        if !$this->{'url'};
    $this->{'label'} = '「%w」を検索する' if !$this->{'label'};
    return;
}

######################################################################
#	(1) obj->match(str);
sub match ($;$) {
    my ( $this, $str ) = @_;
    return 1;
}

######################################################################
#	(2) obj->form;
sub form {
    my ( $this, $status ) = @_;
    my ( $name, $form );
    return '' if $this->{'keywordsfrom'} && $this->{'formdisabled'};

    #名前にデータIDを埋め込む
    $name = 'data:' . $this->{'id'} . ':str';

    #フィールドネーム+ヘッダ
    $form = $this->field_header( '50', $name );

    #デフォルト作成
    $main::data{ $this->{'id'} }{'str'} = $this->{'default'}
        if ( !defined $status || $status eq '' )
        && ( !defined $main::data{ $this->{'id'} }{'str'}
        || $main::data{ $this->{'id'} }{'str'} eq '' );

    $form
        .= '<input type="text" id="'
        . $name
        . '" name="'
        . $name
        . '" size="40"';
    $form
        .= ' value="'
        . $this->escape( $main::data{ $this->{'id'} }{'str'} ) . '"'
        if defined $main::data{ $this->{'id'} }{'str'}
        && $main::data{ $this->{'id'} }{'str'};
    $form .= ' class="invalid-field"' if $this->{'is_invalid_field'}{'str'};
    $form
        .= ' placeholder="'
        . ( ( defined $this->{'placeholder'} && $this->{'placeholder'} ne '' )
        ? $this->escape( $this->{'placeholder'} )
        : '検索キーワード' )
        . '" />';

    $form .= $this->field_footer;
    return $form;
}

######################################################################
#	(3) obj->is_valid_data(path);
sub is_valid_data ($;$) {
    my $this = shift;
    delete $this->{'is_invalid_field'} if ref $this->{'is_invalid_field'};

    if (   $this->{'keywordsfrom'}
        && ref $this->{'field_obj'}->{ $this->{'keywordsfrom'} }
        && $this->{'field_obj'}->{ $this->{'keywordsfrom'} }->{'class'}
        =~ /^text/
        && (  !$this->{'field_obj'}->{ $this->{'keywordsfrom'} }->{'rows'}
            || $this->{'field_obj'}->{ $this->{'keywordsfrom'} }->{'rows'}
            == 1 )
        && (   !defined $main::data{ $this->{'id'} }{'str'}
            || $main::data{ $this->{'id'} }{'str'} eq ''
            || $this->{'formdisabled'} )
        )
    {
        $main::data{ $this->{'id'} }{'str'}
            = ( defined $main::data{ $this->{'keywordsfrom'} }{'str'} )
            ? $main::data{ $this->{'keywordsfrom'} }{'str'}
            : '';
        $main::data{ $this->{'id'} }{'str'} =~ s/<>[#0-9a-zA-z]{3,10}\Z//s;
    }
    $main::data{ $this->{'id'} }{'str'}
        = $this->z_to_h( $main::data{ $this->{'id'} }{'str'} );
    $this->escape_data_tag('str');
    $main::data{ $this->{'id'} }{'str'} =~ s/\s+/ /gs
        if $main::data{ $this->{'id'} }{'str'};

    if ( $this->is_not_empty('str') && $this->is_not_too_long('str') ) {
        $cookie::data{ 'data:' . $this->{'id'} . ':str' }
            = $main::data{ $this->{'id'} }{'str'}
            if $this->{'cookie'} && $main::set_cookie;
        return 1;
    }
    else {
        $this->{'is_invalid_field'}{'str'} = 1;
        return 0;
    }
}

######################################################################
#	(4) obj->usual_format(str);
sub search_engine_url ($$) {
    my ( $this, $str ) = @_;
    my $url = $this->{'url'} || 'https://www.google.co.jp/search?q=%w';
    $str = '' if !defined $str;
    $str = $this->jcode_convert( \$str, $this->{'charset'}, 'utf8' )
        if $this->{'charset'}
        && grep( $_ eq $this->{'charset'}, 'sjis', 'jis', 'euc' );
    $str = $this->url_encode($str);
    $url =~ s/\%w/$str/g;
    return $url;
}

sub search_engine_title ($) {
    my ( $this, $str ) = @_;
    my $label = $this->{'label'} || '「%w」を検索する';
    $str = '' if !defined $str;
    $str = $this->escape($str);
    $label =~ s/\%w/$str/g;
    return $label;
}

sub usual_format {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return $this->field_tag($str) if !defined $str || $str eq '';
    my $url   = $this->search_engine_url($str);
    my $title = $this->search_engine_title($str);
    if ( $this->{'apiurl'} ) {
        my $thumnail
            = $this->get_website_thumnail_url( $this->{'apiurl'}, $url );
        if ($thumnail) {
            return $this->field_tag(
                      $this->data_tag_from_hash( 'str', $str )
                    . '<a href="'
                    . $url . '"'
                    . ( ( !$this->{'sameWindow'} ) ? ' target="_blank"' : '' )
                    . ' title="'
                    . $this->escape_except_html_entity($title) . '"'
                    . ( ( $this->{'nofollow'} ) ? ' rel="nofollow"' : '' )
                    . '>'
                    . $cgi::Image->image_tag(
                    $thumnail,                $this->{'previewwidth'},
                    $this->{'previewheight'}, 0
                    )
                    . '</a>'
            );
        }
    }
    if ( $this->{'button'} ) {
        $this->use_deco($cgi::output);
        return $this->field_tag(
            $this->data_tag_from_hash( 'str', $str )
                . $cgi::Deco->button(
                'jump',
                $this->{'button'},
                $url,
                $title,
                ( ( !$this->{'sameWindow'} ) ? '_blank' : '' ),
                ( ( $this->{'nofollow'} ) ? { 'rel' => 'nofollow' } : '' )
                )
        );

    }
    else {
        return $this->field_tag(
            $this->data_tag_from_hash( 'str', $str )
                . $this->style(
                '<a href="'
                    . $this->escape_except_html_entity($url)
                    . '" title="'
                    . $this->escape_except_html_entity($title) . '"'
                    . ( ( !$this->{'sameWindow'} ) ? ' target="_blank"' : '' )
                    . ( ( $this->{'nofollow'} )    ? ' rel="nofollow"'  : '' )
                    . '>'
                    . $this->escape_except_html_entity($title) . '</a>',
                $this->{'style'},     $this->{'fontsize'},
                $this->{'fontcolor'}, $this->{'fontface'}
                )
        );
    }
}

#---------------------------------------------------------------------
#	(4)-2 obj->format_for_mail(str);
sub format_for_mail {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return ( !defined $str || $str eq '' )
        ? ''
        : '['
        . $this->name . "]\x0A"
        . $this->search_engine_url($str)
        . "\x0A\x0A";
}

#---------------------------------------------------------------------
#	(4)-4 obj->format_for_text(str);
sub format_for_text {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return ( !defined $str || $str eq '' )
        ? ''
        : $this->search_engine_url($str);
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my ( $this, $html_disabled ) = @_;

    #名前
    my $name = 'data:' . $this->{'id'};

    $this->{'default'} =~ s/\x0A//sg if $this->{'default'};
    $this->print_field_info_form($name);

    print '<fieldset>';
    print '<legend>入力欄設定</legend>';
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_default">デフォルト</label></legend>';
    print '<input type="text" name="'
        . $name
        . '_default" id="'
        . $name
        . '_default" value="'
        . $this->escape( $this->{'default'} )
        . '" size="50" title="入力欄のデフォルトの値を設定します。" />';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_placeholder">プレースホルダ</label></legend>';
    print '<input type="text" name="'
        . $name
        . '_placeholder" id="'
        . $name
        . '_placeholder" size="50" title="入力欄の placeholder 属性の値を設定します。" value="'
        . $this->escape( $this->{'placeholder'} ) . '" />';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_titlefrom">自動入力</label></legend>';
    print '検索キーワードがないとき、';
    print '<select name="'
        . $name
        . '_keywordsfrom" id="'
        . $name
        . '_keywordsfrom">';
    print '<option value=""></option>';

    foreach my $id (
        sort {
            $this->{'field_obj'}->{$a}->{'ORDER'}
                <=> $this->{'field_obj'}->{$b}->{'ORDER'}
                || $a <=> $b
        } keys %{ $this->{'field_obj'} }
        )
    {
        next
            unless ref $this->{'field_obj'}->{$id}
            && $this->{'field_obj'}->{$id}->{'class'} =~ /^text/
            && ( !$this->{'field_obj'}->{$id}->{'rows'}
            || $this->{'field_obj'}->{$id}->{'rows'} == 1 );
        print '<option value="'
            . $id . '"'
            . (
            (   defined $this->{'keywordsfrom'}
                    && $this->{'keywordsfrom'} eq $id
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $this->{'field_obj'}->{$id}->name, 32, 'I', 1 )
            . '</option>';
    }
    print '</select>を検索キーワードとする';
    print '<div class="extra-field"><label><input type="checkbox" name="'
        . $name
        . '_formdisabled" value="1"'
        . ( ( $this->{'formdisabled'} ) ? ' checked="checked"' : '' )
        . ' /> 常にこのフィールドの値を検索キーワードにする。</label></div>';
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>リンク先検索エンジン設定</legend>';
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_url">検索エンジン URL</label></legend>';
    print '<input type="text" name="'
        . $name
        . '_url" id="'
        . $name
        . '_url" value="'
        . (
        ( defined $this->{'url'} ) ? $this->escape( $this->{'url'} ) : '' )
        . '" size="80" title="検索エンジンの URL を設定します。" />';
    print ' <span class="note">(%w=検索キーワード)</note>';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_label">検索エンジンラベル</label></legend>';
    print '<input type="text" name="'
        . $name
        . '_label" id="'
        . $name
        . '_label" value="'
        . ( ( defined $this->{'label'} )
        ? $this->escape( $this->{'label'} )
        : '' )
        . '" size="80" title="検索エンジンのラベルを設定します。" />';
    print ' <span class="note">(%w=検索キーワード)</note>';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend><label for="' . $name
        . '_charset">文字コード</label></legend>';
    print $this->mk_select_menu(
        $name . '_charset', '', '', $this->{'charset'},
        ''     => 'UTF-8',
        'sjis' => 'Shift JIS',
        'jis'  => 'JIS',
        'euc'  => 'EUC'
    );
    print '</fieldset>';
    print '</fieldset>';

    if ( !defined $html_disabled || !$html_disabled ) {

        print '<fieldset>';
        print '<legend>リンク設定</legend>';
        print '<ol class="checkbox">';
        print '<li><label><input type="checkbox" name="'
            . $name
            . '_nofollow" id="'
            . $name
            . '_nofollow" value="1"'
            . ( ( defined $this->{'nofollow'} && $this->{'nofollow'} )
            ? ' checked="checked"'
            : '' )
            . ' /> リンク先を検索エンジンに無視させる</label></li>';
        print '<li><label><input type="checkbox" name="'
            . $name
            . '_sameWindow" value="1"'
            . ( ( defined $this->{'sameWindow'} && $this->{'sameWindow'} )
            ? ' checked="checked"'
            : '' )
            . ' /> 同じウィンドウで開く</label></li>';
        print '</ol>';
        print '</fieldset>';

        #ボタンメニュー
        print
            '<fieldset><legend title="設定がない場合、検索エンジンラベルが表示されます。">ボタン</legend>';
        print $cgi::Deco->button_menu( $name . '_button',
            'jump', 120, $this->{'button'} );
        print '</fieldset>';

        print '<fieldset>';
        print
            '<legend>ウェブページ・サムネイル化サービス API</legend>';
        print '<fieldset>';
        print '<legend><label for="' . $name
            . '_apiurl">API URL</label></legend>';
        print '<input type="text" name="'
            . $name
            . '_apiurl" id="'
            . $name
            . '_apiurl" value="'
            . $this->escape( $this->{'apiurl'} )
            . '" size="72" />';
        print
            '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'',
            $name, '_apiurl\').textByMenu(this)">';
        print '<option value="">↓主なサービス</option>';
        print
            '<option value="http://img.simpleapi.net/small/%u">SimpleAPI</option>';
        print
            '<option value="http://mozshot.nemui.org/shot?%u">Mozshot</option>';
        print
            '<option value="http://capture.heartrails.com/medium?%u">HeartRails Capture</option>';
        print
            '<option value="http://images.websnapr.com/?url=%u&amp;size=s">WebSnapr</option>';
        print
            '</select><div class="note">URL が入る場所を %u とします。</div>';
        print '</fieldset>';
        print '<fieldset>';
        print '<legend>サムネイルサイズ</legend>';
        print '<label>幅 <input type="number" name="'
            . $name
            . '_previewwidth" id="'
            . $name
            . '_previewwidth" value="'
            . $this->escape( $this->{'previewwidth'} )
            . '" size="4" /> <span class="unit">ピクセル</span></label> &times <label>高さ <input type="number" name="'
            . $name
            . '_previewheight" id="'
            . $name
            . '_previewheight" value="'
            . $this->escape( $this->{'previewheight'} )
            . '" size="4" /> <span class="unit">ピクセル</span></label>';
        print '</fieldset>';
        print '</fieldset>';

        #文字設定
        $this->print_font_setting_form($name);
    }
    $this->print_setting_save_button;
    return;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this     = shift;
    my $is_valid = 1;

    #タグの除去
    if ( defined $main::data{ $this->{'id'} . '_default' }
        && $main::data{ $this->{'id'} . '_default' } )
    {
        $main::data{ $this->{'id'} . '_default' } =~ s/[\x0A\x0D]//sg;
        $this->detag( \$main::data{ $this->{'id'} . '_default' } );
    }
    $main::data{ $this->{'id'} . '_placeholder' } =~ s/[\x0A\x0D]+//sg
        if defined $main::data{ $this->{'id'} . '_placeholder' };

    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };

    $main::data{ $this->{'id'} . '_nofollow' } = ''
        if !defined $main::data{ $this->{'id'} . '_nofollow' }
        || !$main::data{ $this->{'id'} . '_nofollow' };
    $main::data{ $this->{'id'} . '_sameWindow' } = ''
        if !defined $main::data{ $this->{'id'} . '_sameWindow' }
        || $main::data{ $this->{'id'} . '_sameWindow' } eq '';
    if ( defined $main::data{ $this->{'id'} . '_url' }
        && $main::data{ $this->{'id'} . '_url' } ne '' )
    {
        if ( $this->is_invalid_url( $main::data{ $this->{'id'} . '_url' } ) )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                '検索エンジン URLを確認してください。'
            ) if ref $cgi::action;
            $is_valid = 0;
            delete $main::data{ $this->{'id'} . '_url' };
        }
        elsif ( $main::data{ $this->{'id'} . '_url' } !~ /\%w/ ) {
            push(
                @{ $cgi::action->{'dialog'} },
                '検索エンジン URLを確認してください。キーワードタグがみつかりません。'
            ) if ref $cgi::action;
            $is_valid = 0;
            delete $main::data{ $this->{'id'} . '_url' };
        }
    }
    if (   defined $main::data{ $this->{'id'} . '_apiurl' }
        && $main::data{ $this->{'id'} . '_apiurl' } ne ''
        && $this->is_invalid_url( $main::data{ $this->{'id'} . '_apiurl' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'サムネイル化 API URLを確認してください。'
        ) if ref $cgi::action;
        $is_valid = 0;
        delete $main::data{ $this->{'id'} . '_apiurl' };
    }
    if (   defined $main::data{ $this->{'id'} . '_previewwidth' }
        && $main::data{ $this->{'id'} . '_previewwidth' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_previewwidth' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'サムネイル横幅設定を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_previewwidth' };
        $is_valid = 0;
    }
    if (   defined $main::data{ $this->{'id'} . '_previewheight' }
        && $main::data{ $this->{'id'} . '_previewheight' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_previewheight' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'サムネイル高さ設定を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_previewheight' };
        $is_valid = 0;
    }
    if ( $main::data{ $this->{'id'} . '_apiurl' }
        && !$main::data{ $this->{'id'} . '_previewwidth' } )
    {
        my $thumnail
            = $this->get_website_thumnail_url(
            $main::data{ $this->{'id'} . '_apiurl' },
            'http://www.google.com/' );
        if ($thumnail) {
            require CMONOS::CGI_Lib::HTTP_Client;
            my $res = HTTP_Client->new->request( 'GET' => $thumnail );
            if ( $res->is_success ) {
                my $content = $res->content;
                (   $main::data{ $this->{'id'} . '_previewwidth' },
                    $main::data{ $this->{'id'} . '_previewheight' }
                ) = $cgi::Image->info( \$content );
            }
        }
    }
    $is_valid = 0 if !$this->is_valid_font_setting;
    return $is_valid;
}

sub get_website_thumnail_url ($$$) {
    my ( $this, $api, $url ) = @_;
    $url = $this->url_encode($url) if $url && $api =~ /\?/ && $api =~ /=/;
    return ( $url && $api =~ s/\%u/$url/g ) ? $api : '';
}

######################################################################
#	(7) obj->search_form;
sub search_form_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#	(8) obj->sort_enabled;
sub sort_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#	(9) obj->cookie_enabled;
sub cookie_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#	(10) obj->required_enabled;
sub required_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#sub import {
BEGIN {
    Field::class_register( 'webapi_searchengine', 'LinkToSearchEngine',
        '外部サイトで検索する' );
    $setting::deco_dir{'jump'}  = 'ジャンプボタン';
    $setting::deco_type{'jump'} = 'button';
}

1;
