package Trackback;
use strict;
use utf8;
######################################################################
# Trackback 送信モジュール
# CMONOS::CGI_Lib::Ping::Trackback.pm ver1.0.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-06-08
######################################################################
# 使い方
# require CMONOS::CGI_Lib::Ping::Trackback;
# $trackback = Trackback->new(
#	'ping_url'				=>	Trackback 送信先 URL,
#	'title'					=>	記事のタイトル,
#	'excerpt'				=>	記事の説明,
#	'url'					=>	記事のURL,
#	'trackback'				=>	記事の Trackback Ping URL,
#	'blog_name'				=>	ブログタイトル,
# );
#
# RDF取得
# $trackback->rdf;
#
######################################################################
# オブジェクトを作る。
sub new ($;@) {
    my $pkg  = shift;
    my $this = {@_};
    bless $this;
    return $this;
}

sub set ($;@) {
    my $this = shift;
    $this = {@_};
    return $this;
}

sub ping_url ($) {
    my $this = shift;
    return ( $this->{'ping_url'} )
        ? $this->unescape( $this->{'ping_url'} )
        : '';
}

sub url ($) {
    my $this = shift;
    return ( $this->{'url'} ) ? $this->unescape( $this->{'url'} ) : '';
}

sub excerpt ($) {
    my $this = shift;
    return ( $this->{'excerpt'} )
        ? $this->unescape( $this->{'excerpt'} )
        : '';
}

sub blog_name ($) {
    my $this = shift;
    return ( $this->{'blog_name'} )
        ? $this->unescape( $this->{'blog_name'} )
        : '';
}

sub title ($) {
    my $this = shift;
    return ( $this->{'title'} ) ? $this->unescape( $this->{'title'} ) : '';
}

sub trackback ($) {
    my $this = shift;
    return ( $this->{'trackback'} )
        ? $this->unescape( $this->{'trackback'} )
        : '';
}

sub error ($) {
    my $this = shift;
    return ( $this->{'error'} ) ? $this->{'error'} : 0;
}

sub message ($) {
    my $this = shift;
    return ( $this->{'message'} )
        ? $this->unescape( $this->{'message'} )
        : '';
}

sub ptime ($) {
    my $this = shift;
    return ( $this->{'ptime'} ) ? $this->{'ptime'} : '';
}

sub sender ($) {
    my $this = shift;
    return ( $this->{'sender'} ) ? $this->{'sender'} : $this;
}

sub hash ($;$) {
    my ( $this, $sender_result ) = @_;
    my $result
        = ( $sender_result && ref $this->{'sender'} )
        ? $this->{'sender'}
        : $this;
    return (
        'title'     => $this->title,
        'excerpt'   => $this->excerpt,
        'url'       => $this->url,
        'trackback' => $this->trackback,
        'blog_name' => $this->blog_name,

        'ping_url' => $result->ping_url,
        'error'    => $result->error,
        'message'  => $result->message,
        'ptime'    => $result->ptime,
    );
}

sub hash_ref ($;$) {
    my ( $this, $sender_result ) = @_;
    return { $this->hash($sender_result) };
}

sub rdf ($) {
    my $this = shift;
    return '' if !$this->{'url'};
    my $rdf
        = '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"'
        . "\x0A";
    $rdf
        .= '         xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/"'
        . "\x0A"
        if $this->{'trackback'};
    $rdf .= '         xmlns:dc="http://purl.org/dc/elements/1.1/">' . "\x0A";
    $rdf .= '<rdf:Description' . "\x0A";
    $rdf .= '    rdf:about="' . $this->escape( $this->url ) . '"' . "\x0A";
    $rdf
        .= '    trackback:ping="'
        . $this->escape( $this->trackback ) . '"' . "\x0A"
        if $this->{'trackback'};
    $rdf .= '    dc:title="' . $this->escape( $this->title ) . '"' . "\x0A"
        if defined $this->{'title'} && $this->{'title'} ne '';
    $rdf
        .= '    dc:identifier="' . $this->escape( $this->url ) . '"' . "\x0A";
    $rdf
        .= '    dc:description="'
        . $this->escape( $this->excerpt ) . '"' . "\x0A"
        if defined $this->{'excerpt'} && $this->{'excerpt'} ne '';
    $rdf .= '/>' . "\x0A";
    $rdf .= '</rdf:RDF>' . "\x0A";
    return $rdf;
}

# url へのリンクがあるかどうか調べる
sub linked ($;$$) {
    my ( $this, $url, $loosely ) = @_;
    return 0 if !$this->{'url'};
    $url =~ s/[\?#].*?$// if $loosely;
    require CMONOS::CGI_Lib::HTTP_Client;
    my $res = HTTP_Client->new->request(
        'GET' => $this->url,
        { 'Accept' => 'text/html' }
    );
    return (   $res->is_success
            && $res->content_type eq 'text/html'
            && ${ $res->content_ref } =~ /\Q$url\E/ ) ? 1 : 0;
}

# ping url が未設定の場合、url の RDF から ping url を取得する
sub get ($) {
    my $this = shift;
    if ( $this->{'url'} && !$this->{'trackback'} ) {
        require CMONOS::CGI_Lib::HTTP_Client;
        my $res = HTTP_Client->new->request(
            'GET' => $this->url,
            { 'Accept' => 'text/html' }
        );
        if ( $res->is_success && $res->content_type eq 'text/html' ) {
            my $content = $res->content('utf8');
            while (
                $content =~ /<rdf:Description(?![0-9a-zA-Z_])([^>]*?)\/>/g )
            {
                my ( $data, $url );
                $data = $1;
                $url  = $1
                    if $data =~ /[\x09\x0A\x0C\x0D\x20]+rdf:about="(.*?)"/is
                    || $data
                    =~ /[\x09\x0A\x0C\x0D\x20]+dc:identifier="(.*?)"/is;
                if ( $url eq $this->{'url'} ) {
                    $this->{'trackback'} = $this->unescape($1)
                        if $data
                        =~ /[\x09\x0A\x0C\x0D\x20]+trackback:ping="(.*?)"/is;
                    $this->{'title'} = $this->unescape($1)
                        if $data
                        =~ /[\x09\x0A\x0C\x0D\x20]+dc:title="(.*?)"/is;
                    $this->{'excerpt'} = $this->unescape($1)
                        if $data
                        =~ /[\x09\x0A\x0C\x0D\x20]+dc:description="(.*?)"/is;
                    last;
                }
            }
        }
    }
    return $this;
}

sub ename_to_jname ($$) {
    my ( $this, $ename ) = @_;
    return 'utf8' if !$ename;
    $ename = lc $ename;
    my %ename2j = (
        'shiftjis'    => 'sjis',
        'x-sjis'      => 'sjis',
        'shift_jis'   => 'sjis',
        'euc-jp'      => 'euc',
        'x-euc-jp'    => 'euc',
        '7bit-jis'    => 'jis',
        'x-jis'       => 'jis',
        'iso-2022-jp' => 'jis',
        'utf-16be'    => 'ucs2',
        'utf-8'       => 'utf8',
    );
    if ( $ename2j{$ename} ) {
        return $ename2j{$ename};
    }
    else {
        $ename =~ tr/-/_/;
        return $ename;
    }
}

# ping 送信
sub ping ($) {
    my $this = shift;
    if ( !$this->{'ping_url'} ) {
        $this->{'message'}
            = 'トラックバック URL がみつかりませんでした。';
        return $this;
    }
    require CMONOS::CGI_Lib::HTTP_Client;
    my $res = HTTP_Client->new->request(
        'POST' => $this->ping_url,
        { 'Accept' => '*/*', },
        {   'title'     => $this->title,
            'excerpt'   => $this->excerpt,
            'url'       => $this->url,
            'blog_name' => $this->blog_name,
        },
        'utf8'
    );
    if ( $res->is_success ) {
        my $hash = $res->hash('utf8');
        if ( defined $hash->{'response'}->{'error'} ) {
            $this->{'error'}   = $hash->{'response'}->{'error'};
            $this->{'message'} = $hash->{'response'}->{'message'}
                if defined $hash->{'response'}->{'message'};
            $this->{'ptime'} = time;
            return $this;
        }
        else {
            $this->{'error'}   = 1;
            $this->{'message'} = '';
        }
    }
    else {
        $this->{'error'}   = 1;
        $this->{'message'} = $res->message;
    }
    if ( !ref $this->{'sender'} && !ref $this->{'receiver'} ) {
        $this->{'receiver'}
            = Trackback->new( 'sender' => $this, 'url' => $this->ping_url )
            ->get;
        if ( $this->{'receiver'}->trackback ) {
            $this->{'ping_url'} = $this->{'receiver'}->trackback;
            $this->ping;
        }
        else {
            $this->{'message'}
                = 'トラックバック URL が正しくありません。';
            delete $this->{'receiver'};
        }
    }
    return $this;
}

sub receiver ($) {
    my $this = shift;
    return ( ref $this->{'receiver'} )
        ? $this->{'receiver'}
        : Trackback->new( 'sender' => $this, 'trackback' => $this->ping_url );
}

sub escape ($$) {
    my ( $this, $str ) = @_;
    $str
        =~ s/\&(?!(?:#[0-9]{2,6}|#x[a-fA-F0-9]+|[a-zA-Z0-9]{2,10});)/\&amp;/g;
    $str =~ s/"/\&quot;/g;
    $str =~ s/</\&lt;/g;
    $str =~ s/>/\&gt;/g;
    $str =~ s/[\x09\x0A\x0C\x0D\x20]+/ /sg;
    return $str;
}

sub unescape ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/\&quot;/"/g;
    $str =~ s/\&lt;/</g;
    $str =~ s/\&gt;/>/g;
    $str =~ s/\&amp;/&/g;
    $str =~ s/[\x09\x0A\x0C\x0D\x20]+/ /sg;
    return $str;
}

1;
