package ProgressBar;
use strict;
use utf8;
######################################################################
# プログレスバーモジュール
# ProgressBar.pm 1.0
# Copyright (C) 2007-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-05-07
######################################################################
sub new ($;@) {
    my $pkg  = shift;
    my $this = {};
    bless $this;
    $this;
}

sub css ($;$) {
    my ( $this, $width, $body ) = @_;
    $width = 1 if !$width;
    my $css = '';
    if ($body) {
        return <<CSS
html, body { height: 100%; margin: 0; padding: 0; }
body {
  background: #4C4C4C none;
  width: 100%;
  height: 100%;
  color: #FFF;
  text-align: center;
}
div#progress-bar-dialog {
  text-align: left;
  padding-top: 200px;
  width: 482px;
  margin: 0 auto;
}
div#progress-bar-bg {
  text-align: left;
  width: 480px;
  height: 24px;
  margin: 0 auto;
  background: #262626;
  border: 1px #CCC solid;
}
div#progress-bar {
  width: ${width}px;
  height: 24px;
  margin: 0;
  background-color: #69F;
  border: none;
}
CSS

    }
    else {
        return <<CSS
html,body{ height: 100%; }
div#progress-bar-box-bg {
  width: 100%;
  height: 100%;
  position: fixed !important;
  position: absolute;
  top: 0;
  left: 0;
  background-color: #000;
  z-index: 100;
  opacity: 0.7;
  -moz-opacity: 0.7;
  -webkit-opacity: 0.7;
  -o-opacity: 0.7;
  -ms-filter:"alpha(opacity=70)";
  filter:alpha(opacity=70);
}
div#progress-bar-box {
  width: 100%;
  height: 100%;
  position: fixed !important;
  position: absolute;
  top: 0;
  left: 0;
  z-index: 101;
  color: #FFF;
  text-align: center;
}
div#progress-bar-dialog {
  text-align: left;
  padding-top: 200px;
  width: 482px;
  margin: 0 auto;
}
div#progress-bar-bg {
  text-align: left;
  width: 480px;
  height: 24px;
  margin: 0 auto;
  background-color: rgba(0, 0, 0, 0.5) !important;
  background-color: transparent;
  border: 1px #CCC solid;
}
div#progress-bar {
  width: ${width}px;
  height: 24px;
  margin: 0;
  background-color: #69F;
  border: none;
}
CSS
    }
}

sub print_progress_bar ($$;$) {
    my ( $this, $name, $total ) = @_;
    print
        '<div id="progress-bar-box-bg"></div><div id="progress-bar-box"><div id="progress-bar-dialog"><span id="progress-bar-count"></span>/<span id="progress-bar-total">'
        . ( ($total) ? $total : '' )
        . '</span> '
        . ( ($name) ? $name : '' )
        . ' <span id="time-remain">(このまましばらくお待ち下さい)</span></div><div id="progress-bar-bg"><div id="progress-bar"></div></div></div>';
}

sub remove ($) {
    my $this = shift;
    print
        '<style type="text/css">div#progress-bar-box-bg, div#progress-bar-box { display: none; }</style>';
}

sub javascript ($$$) {
    my $this = shift;
    return <<'JAVASCRIPT'
cmonos.progressbar = {
	start : (new Date()).getTime(),
	update : function (count,total) {
		if (typeof document.body.style.maxHeight == "undefined") {
			document.getElementById('progress-bar-box-bg').style.height = (cmonos.window.getPageYOffset() + cmonos.window.getInnerHeight()) + 'px';
			document.getElementById('progress-bar-box-bg').style.width = (cmonos.window.getPageXOffset() + cmonos.window.getInnerWidth()) + 'px';
		}
		var max_width = 480;
		if (cmonos.window.getInnerWidth() <= 500) max_width = cmonos.window.getInnerWidth() - 20;
		if (cmonos.window.getInnerHeight() < 300) document.getElementById('progress-bar-dialog').style.paddingTop = (parseInt(cmonos.window.getInnerHeight() / 2) - 20) + 'px';
		document.getElementById('progress-bar-dialog').style.width = (max_width + 2) + 'px';
		document.getElementById('progress-bar-bg').style.width = max_width + 'px';
		document.getElementById('progress-bar-count').innerHTML = count;
		document.getElementById('progress-bar-total').innerHTML = total;
		if (count == total) {
			document.getElementById('progress-bar').style.width = max_width + 'px';
			document.getElementById('time-remain').innerHTML = '[完了]';
			document.getElementById('progress-bar-box-bg').style.display = 'none';
			document.getElementById('progress-bar-box').style.display = 'none';
		} else {
			var w = Math.floor(count / total * max_width);
			document.getElementById('progress-bar').style.width = ((w > 0) ? w : 1) + 'px';
			if (count > 0) {
				var rmin = Math.ceil(((new Date()).getTime() - this.start) * (total - count) / count / 60000);
				document.getElementById('time-remain').innerHTML = ' [あと' + rmin + '分]';
			}
		}
	}
};
JAVASCRIPT
}

sub update ($$$;$$) {
    my ( $this, $count, $total, $step, $script ) = @_;
    return 0
        if $step
        && $this->{'prev'}
        && $count > $this->{'prev'}
        && $count < ( $this->{'prev'} + ( $total / $step ) )
        && $count < $total;
    $count = $total if $count > $total;
    my $is_xhtml = (
        ( $default::doctype && $default::doctype =~ /XHTML/ )
            || ( !$default::doctype
            && $env::doctype
            && $env::doctype =~ /XHTML/ )
    ) ? 1 : 0;
    binmode STDOUT, ":utf8";
    print '<script type="text/javascript" language="JavaScript">' . "\x0A"
        . '<!--'
        . ( ($is_xhtml) ? ' <![CDATA[' : '' ) . "\x0A"
        . 'cmonos.progressbar.update('
        . $count . ','
        . $total . ');'
        . ( ($script)   ? $script : '' ) . "\x0A" . '// '
        . ( ($is_xhtml) ? ']]> '  : '' ) . '-->' . "\x0A"
        . '</script>'
        . '<noscript><style type="text/css">div#progress-bar { width: '
        . int( $count / $total * 480 )
        . 'px; }</style></noscript>';
    $this->{'prev'} = $count;
    binmode STDOUT;
    return 1;
}

sub set_sig ($) {
    my $this = shift;
    $SIG{'INT'} = $SIG{'HUP'} = $SIG{'QUIT'} = $SIG{'TERM'} = sub {
        binmode STDOUT, ":utf8";
        print <<JAVASCRIPT;
<script type="text/javascript" language="JavaScript">
<!-- <![CDATA[
document.getElementById('progress-bar-box-bg').style.display = 'none';
document.getElementById('progress-bar-box').style.display = 'none';
alert('処理が中断されました。');
// ]]> -->
</script>
JAVASCRIPT
        exit;
    };
}

1;
