package Register;
use strict;
use utf8;
use Fcntl qw(:DEFAULT);
eval 'use ' . $env::dbm_file . ';';
######################################################################
# パスワード管理モジュール
# メールアドレスをキーに登録者のパスワード照合を行います。
# Register.pm 2.2.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2012-09-14
# 注意：このスクリプトは
# 　　　Sendmail.pm,Tcp.pm および DBMとの連携を前提としています。
######################################################################
#
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	(0)-1 オブジェクトを作る。
#		require CMONOS::CGI_Lib::Register;
#		obj = Register->new(sendmail,cgi_master_name,cgi_master,dbm_path[,master_password,master_password_encrypted[,fix_enabled[,openid_request_parameter[,check[,threshold,latency]]]]]);
#
#		新しいオブジェクトを作ります。
#			sendmail					sendmailのパス（あなたのサーバーにあわせる/なくてもよい）
#			cgi_master_name				CGI管理人の名前
#			cgi_master					CGI管理人のメールアドレス
#			dbm_path					ID情報管理DBディレクトリのパス
#			master_password				管理パスワード（認証に必ず成功するパスワード）
#			master_password_encrypted	1=管理パスワード（認証に必ず成功するパスワード）は暗号化されている
#			fix_enabled					1=呼びだし元でパスワードをハッシュのリファレンスにする
#			openid_enabled				1=OpenID を利用可能
#			check						1 = パスワードの重複チェックを行う
#										※ データベースが大きいと重複チェックには時間がかかります。
#			threshold					許可するパスワード認証の連続失敗数
#			latency						パスワード認証に連続して失敗した場合の待ち時間（分）
#
#	(0)-2 obj->get_preference;
#		初期設定情報を取得します。
#
#	(0)-2-1 obj->bad_addr(@addr);
#		登録拒絶メールアドレスを設定します。
#			addr					登録拒絶メールアドレス
#
#			※ 設定された文字列を含むメールアドレスの登録を拒否します。
#			※ 正規表現を使えます。
#
#	(0)-2-2 obj->good_addr(@addr);
#		登録許可メールアドレスを設定します。
#			addr					登録許可メールアドレス
#
#			※ 設定された文字列を含むメールアドレスのみ登録します。
#			※ 正規表現を使えます。
#
#	(0)-2-3 obj->free_mail_disabled;
#		あらかじめ登録されているフリーメールの登録を拒否します。
#
# ---------------------------------------------------------------------------------------
#	(1) obj->rand_char(num);
#		num個の文字からなる乱文字を返します。
#		numの設定がない場合、デフォルトで8文字の乱文字を返します。
#
# ---------------------------------------------------------------------------------------
#	(2) obj->encrypt($text);　$textを暗号化します。
#	    obj->encrypt($text,$crypt);　$textを$cryptと同じキーで暗号化します。
#
# ---------------------------------------------------------------------------------------
#	(3) obj->is_identified(input,crypt);
#		暗号化されたパスワードcryptとinputの照合を行います。
#		認証に成功すれば、「1」を返します。
#
# ---------------------------------------------------------------------------------------
#	(4) obj->read_crypt(email);
#		dbmファイルに接続し、登録者のメールアドレスをキーにして対応するパスワードを、呼び
#		出します。呼び出したパスワードをを返します。
#		例）obj->ReadPswd('xxx@yyy.ne.jp',"./data/id");
#
# ---------------------------------------------------------------------------------------
#	(5) obj->read_email(pswd);
#		dbmファイルに接続し、暗号化されたパスワードをキーにして対応するメールアドレスを、
#		呼び出します。呼び出したメールアドレスを返します。
#
# ---------------------------------------------------------------------------------------
#	(6) obj->is_registered(pswd,email[,master_disabled[,threshold_check]]);
#		dbmファイルに接続し、登録者のメールアドレスをキーにして対応するパスワードを、呼び
#		出します。暗号化されているパスワードとpswdを照合します。
#		認証が成功すれば、「1」を返し、失敗すれば「0」を返します。
#			master_disabled				1=マスターパスワードによる認証を行わない
#			threshold_check				1=パスワード認証の連続失敗数をチェックする
#
# ---------------------------------------------------------------------------------------
#	(7) obj->register(email,subject,message);
#		乱文字8文字のパスワードを自動生成します。
#		dbmファイルに接続し、登録者のメールアドレスをキーにして生成したパスワードを、暗号
#		化して記録します。また、登録者のメールアドレスに、パスワードを送り届けます。登録が
#		成功すれば、登録したパスワードを返します。
#			email		登録するE-Mail
#			subject		パスワード通知メールの表題
#			message		パスワード通知メールのメッセージ
#
# ---------------------------------------------------------------------------------------
#	(8) obj->remove(pswd,email);
#		登録の抹消を行います。
#			pswd			登録パスワード
#			email			登録を抹消するメールアドレス
#
#	(8)-2 obj->remove_in_bulk(\email[,mailto,reason,refuse]);
#		登録の抹消を行います。
#			\email		登録を抹消するメールアドレスを納めた配列のリファレンス
#			notice		1 = 登録抹消通知メールを送る
#			reason		登録抹消の理由
#			refuse		1 = 登録を抹消すると同時に登録制限対象に指定する
#
#	(8)-3 obj->cancel_in_bulk(\email);
#		登録制限解除を行います。
#			\email		登録制限を解除するメールアドレスを納めた配列のリファレンス
#
# ---------------------------------------------------------------------------------------
#	(9) (count,total) = obj->gc(time[,pending]);
#		登録メールアドレスのうち、最後に利用した時間が time 日前より以前のものを削除します。
#			time			リストアップ範囲（time 日前）
#			pending			1=一度もログインされていない登録メールアドレスのみ削除する
#
#			count			削除した登録メールアドレス数
#			total			登録メールアドレスの総数
#
# ---------------------------------------------------------------------------------------
#	(10) obj->print_email_register_form([email,input,script_name]);
#		メールアドレス登録フォームを出力します。
#			email			デフォルトで表示するアドレス
#			input			E-Mail入力欄以外のフォーム
#			script_name	送信先URL（初期値は呼び出し側CGI）
#
# ---------------------------------------------------------------------------------------
#	(11) obj->is_available_name(pswd,email,name);
#		名前が登録済みネームと重複しないかどうかチェックします。
#		重複しない場合は 1 を返し、その名前を登録します。
#			pswd			登録パスワード
#			email			登録メールアドレス
#			name			登録ネーム
#
#	(11)-2 email = obj->name_to_email(pswd,name);
#		登録ネームから登録メールアドレスを呼び出します。
#
#	(11)-3 name = obj->email_to_name(pswd,email);
#		登録メールアドレスから登録ネームを呼び出します。
#
#	(11)-4 name = obj->name_and_email(pswd,email);
#		登録メールアドレスから登録ネーム付きメールアドレスを返します。
#
# ---------------------------------------------------------------------------------------
#	(12) obj->user_name_register(pswd,email,username);
#		重複をチェックしない名前を登録します。
#			pswd			登録パスワード
#			email			登録メールアドレス
#			username		登録ユーザー名
#
#	(12)-2 username = obj->email_to_user_name(pswd,email);
#		登録メールアドレスから登録ユーザー名を呼び出します。
#
# ---------------------------------------------------------------------------------------
#	(13) @error_dialog = obj->error_dialog;@success_dialog = obj->success_dialog;
#		行った処理に関する成功メッセージとエラーメッセージを返します。
#
######################################################################
BEGIN {
    srand;
}

#オブジェクト作成
sub new ($$$;$$$$$$$$$) {
    my ($pkg,                       $sendmail,
        $cgi_master_name,           $cgi_master,
        $dbm_path,                  $master_password,
        $master_password_encrypted, $fix_enabled,
        $openid_enabled,            $check,
        $threshold,                 $latency
    ) = @_;
    $cgi_master_name = '管理人'
        if !defined $cgi_master_name || $cgi_master_name eq '';
    $cgi_master_name =~ s/<//g;
    $cgi_master_name =~ s/>//g;
    $cgi_master_name =~ s/"//g;
    my $this = {
        'sendmail'        => $sendmail,
        'cgi_master_name' => $cgi_master_name,
        'cgi_master'      => $cgi_master,
        'dbm_path'        => $dbm_path,
        'master_password' =>
            ( ( defined $master_password ) ? $master_password : '' ),
        'master_password_encrypted' =>
            ( ($master_password_encrypted) ? 1 : 0 ),
        'fix_enabled'    => ( ($fix_enabled)         ? 1          : 0 ),
        'openid_enabled' => ( ($openid_enabled)      ? 1          : 0 ),
        'check'          => ( ( defined $check )     ? $check     : 0 ),
        'threshold'      => ( ( defined $threshold ) ? $threshold : 5 ),
        'latency'        => (
              ( defined $latency )
            ? ( $latency eq '' )
                    ? ''
                    : $latency * 60
            : 300
        ),
    };
    bless $this;
    $env::dbm_file = 'AnyDBM_File'
        if !defined $env::dbm_file || !$env::dbm_file;
    $env::dbm_mode = 0666 if !defined $env::dbm_mode || !$env::dbm_mode;
    $this;
}

sub encode_utf8 ($$) {
    my ( $this, $str ) = @_;
    utf8::encode($str) if defined $str && utf8::is_utf8($str);
    return $str;
}

sub decode_utf8 ($$) {
    my ( $this, $str ) = @_;
    utf8::decode($str) if defined $str && !utf8::is_utf8($str);
    return $str;
}

######################################################################
#初期設定取得
sub get_preference ($) {
    my $this = shift;
    return if $this->{'preference_enabled'};
    my ( %pref, $bad_addr, $good_addr, $nofreemail );
    tie( %pref, $env::dbm_file, $this->{'dbm_path'} . '/preference',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %pref ) {
        $bad_addr = $this->decode_utf8( $pref{'bad_addr'} )
            if defined $pref{'bad_addr'};
        $good_addr = $this->decode_utf8( $pref{'good_addr'} )
            if defined $pref{'good_addr'};
        $nofreemail = $this->decode_utf8( $pref{'nofreemail'} )
            if defined $pref{'nofreemail'};
        $this->{'mail_subject'} = $this->decode_utf8( $pref{'mail_subject'} )
            if defined $pref{'mail_subject'};
        $this->{'mail_msg'} = $this->decode_utf8( $pref{'mail_msg'} )
            if defined $pref{'mail_msg'};
        $this->{'remove_subject'}
            = $this->decode_utf8( $pref{'remove_subject'} )
            if defined $pref{'remove_subject'};
        $this->{'remove_msg'} = $this->decode_utf8( $pref{'remove_msg'} )
            if defined $pref{'remove_msg'};
    }
    untie %pref;
    if ( $this->{'preference_enabled'} ) {
        $this->bad_addr( split( /[\x0A\x0D]+/, $bad_addr ) )
            if defined $bad_addr && $bad_addr;
        $this->good_addr( split( /[\x0A\x0D]+/, $good_addr ) )
            if defined $good_addr && $good_addr;
        $this->free_mail_disabled if defined $nofreemail && $nofreemail;
    }
    $this->{'preference_enabled'} = 1;
    return $this;
}

######################################################################
#登録拒絶メールアドレス設定
sub bad_addr ($@) {
    my ( $this, @addr ) = @_;
    return $this if !@addr;
    @addr = grep( defined $_ && $_ ne '', @addr );
    return $this if !@addr;
    push( @{ $this->{'bad_addr'} }, @addr );
    return $this;
}

######################################################################
#登録許可メールアドレス設定
sub good_addr ($@) {
    my ( $this, @addr ) = @_;
    return if !@addr;
    @addr = grep( defined $_ && $_ ne '', @addr );
    return if !@addr;
    push( @{ $this->{'good_addr'} }, @addr );
}

######################################################################
#フリーメールの登録
sub free_mail_disabled ($) {
    my $this = shift;
    @{ $this->{'freemail'} } = (
        '@hotmail.com',    '@mail.goo.ne.jp',
        '@freemail.ne.jp', '@mailbank.ne.jp',
        '@freeweb.ne.jp',  '@anet.ne.jp',
        '@club.ne.jp',     '@design.co.jp',
        '@office.ne.jp',   '@vip.co.jp',
        '@hot.co.jp',      '@forum.ne.jp',
        '@104.net',        '@goukaku.com',
        '@mcn.ne.jp',      '@gmail.com',
        'google',          'yahoo',
        'infoseek',        'excite',
        'lycos',
    );
    return $this;
}

sub free_mail_enabled ($) {
    my $this = shift;
    delete $this->{'freemail'};
    return $this;
}

######################################################################
#利用時刻記録
sub mtime ($$;$) {
    my ( $this, $email, $pending ) = @_;
    my (%time);
    tie(%time, $env::dbm_file,
        $this->{'dbm_path'} . '/mtime',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    $time{ $this->encode_utf8($email) } = time . ( ($pending) ? '.0' : '' );
    untie %time;
    return $this;
}

######################################################################
#乱文字発生
sub rand_char ($$) {
    my ( $this, $num ) = @_;
    my (@charset) = ( 'a' .. 'z', 'A' .. 'Z', '0' .. '9' );
    my ($char);
    $num = 8 if !$num;
    $num--;
    foreach ( 0 .. $num ) {
        $char .= $charset[ int( rand(62) ) ];
    }
    return $char;
}

######################################################################
#暗号化
sub salt ($) {
    my $this = shift;
    my (@saltset) = ( 'a' .. 'z', 'A' .. 'Z', '0' .. '9', '.', '/' );
    my ( $p1, $p2, $now, $mday, $salt );
    ( $p1, $p2 ) = unpack( "C2", $this->rand_char );
    $now  = time;
    $mday = $now / ( 60 * 60 * 24 ) + $p1 + $p2;
    $salt = $saltset[ $mday % 64 ] . $saltset[ $now % 64 ];
    return $salt;
}

sub encrypt ($$;$) {
    my ( $this, $text, $salt ) = @_;
    my ( $crypt, @salts, $no );
    if ( defined $salt && $salt =~ s/^\$1\$// ) {
        @salts = split( /\$1\$/, $salt );
        map( $_ = substr( $_, 0, 2 ), @salts );
    }
    elsif ( defined $salt && $salt ne '' ) {
        @salts
            = split( /([a-zA-Z0-9\/\.]{2})(?:[a-zA-Z0-9\/\.]{11})/, $salt );
        @salts = grep( $_ =~ /^[a-zA-Z0-9\/\.]{2}$/, @salts );
    }
    $no = 0;
    while ( $text =~ /([a-zA-Z0-9\/\.]{1,8})/g ) {
        $crypt .= crypt( $1,
            ( ( @salts && defined $salts[$no] ) ? $salts[$no] : $this->salt )
        );
        $no++;
    }
    return $crypt;
}

######################################################################
#パスワードの照合
sub is_identified ($$$;$) {
    my ( $this, $pswd, $crypt, $fix_enabled ) = @_;
    return 0 if !defined $pswd || !defined $crypt || $crypt eq '';
    if ( ref $pswd ) {
        if ( $this->{'openid_enabled'} && $crypt =~ m!^(https?|xri)://!i ) {
            if ( defined $pswd->{'claimed_id'}
                && $pswd->{'claimed_id'} eq $crypt )
            {
                return 1;
            }
        }
        elsif ( defined $pswd->{'encrypted_password'} ) {
            return 1 if $pswd->{'encrypted_password'} eq $crypt;
        }
    }
    elsif ( $pswd ne '' && $crypt eq $this->encrypt( $pswd, $crypt ) ) {
        if ( $fix_enabled && $this->{'fix_enabled'} ) {
            $_[1] = { 'encrypted_password' => $crypt };
        }
        return 1;
    }
    return 0;
}

######################################################################
#暗号化されたパスワード呼び出し
sub read_crypt ($$) {
    my ( $this, $email ) = @_;
    my ( $pswd, %id );
    return '' if !$email || $this->is_invalid_email_address($email);
    tie( %id, $env::dbm_file, $this->{'dbm_path'} . '/users',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %id ) {
        $pswd = $this->decode_utf8( $id{ $this->encode_utf8($email) } );
    }
    untie %id;
    $this->mtime($email) if defined $pswd && $pswd;
    return ( defined $pswd ) ? $pswd : '';
}

sub exist ($$) {
    my ( $this, $email ) = @_;
    my ( $pswd, %id );
    return 0 if !$email || $this->is_invalid_email_address($email);
    tie( %id, $env::dbm_file, $this->{'dbm_path'} . '/users',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %id ) {
        $pswd = $this->decode_utf8( $id{ $this->encode_utf8($email) } );
    }
    untie %id;
    return ( defined $pswd ) ? 1 : 0;
}

######################################################################
#メールアドレス呼び出し
sub read_email ($$) {
    my ( $this, $pswd ) = @_;
    my ( $email, %id );
    return '' if !$pswd || $pswd =~ /:/;
    tie( %id, $env::dbm_file, $this->{'dbm_path'} . '/users',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %id ) {
        while ( my ( $k, $v ) = each %id ) {
            if ( $v && $v !~ /:/ && $this->is_identified( $pswd, $v ) ) {
                $email = $k;
                last;
            }
        }
    }
    untie %id;
    return ($email) ? $email : '';
}

sub is_identifier ($$;$) {
    my ( $this, $email, $pswd ) = @_;
    return 0 if !$this->{'openid_enabled'};
    $pswd = $this->read_crypt($email) if !$pswd;
    return ( $pswd =~ m!^(https?|xri)://!i ) ? 1 : 0;
}

######################################################################
#認証
sub is_registered ($$$;$$) {
    my ( $this, $pswd, $email, $master_password_disabled, $threshold_check )
        = @_;
    return 0
        if !defined $pswd
        || ( !ref $pswd && $pswd eq '' )
        || !defined $email
        || ( $threshold_check && $this->is_locked($email) );
    my $fix_enabled = ( !ref $pswd && $this->{'fix_enabled'} ) ? 1 : 0;
    if (  !$master_password_disabled
        && $this->is_master( $pswd, $fix_enabled ) )
    {
        $_[1] = $pswd if $fix_enabled;
        $this->unlock($email);
        return 1;
    }
    if ($email
        && $this->is_identified(
            $pswd, $this->read_crypt($email), $fix_enabled
        )
        )
    {
        $_[1] = $pswd if $fix_enabled;
        $this->unlock($email);
        return 1;
    }
    $this->lock($email) if $threshold_check;
    return 0;
}

######################################################################
#パスロック
sub is_locked ($$) {
    my ( $this, $email ) = @_;
    return $this->_lock($email);
}

sub lock ($$) {
    my ( $this, $email ) = @_;
    return $this->_lock( $email, 1 );
}

sub unlock ($$) {
    my ( $this, $email ) = @_;
    return ( $this->_lock( $email, 0 ) ) ? 0 : 1;
}

sub _lock ($$;$) {
    my ( $this, $email, $lock ) = @_;
    return 0
        if !defined $email || !$this->{'threshold'} || !$this->{'latency'};
    $email = 'no_email' if $email eq '';
    my (%locked);
    my $host      = $ENV{'REMOTE_HOST'} || $ENV{'REMOTE_ADDR'};
    my $time      = time;
    my $is_locked = 0;
    tie(%locked, $env::dbm_file,
        $this->{'dbm_path'} . '/locked',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );

    if ( ref tied %locked ) {
        my $new_data = '';
        my $data = $this->decode_utf8( $locked{ $this->encode_utf8($email) } )
            || '';
        my $done = 0;
        while ( $data =~ /^([^\t]+)\t([0-9]+)\t([0-9]+)\x0A?$/mg ) {
            my ( $h, $q, $t ) = ( $1, $2, $3 );
            if ( $host eq $h ) {
                next if $done;
                $done++;
                if ( !defined $lock ) {
                    if ( $time - $t > $this->{'latency'} ) {
                        if ( $this->{'threshold'} <= 3 ) {
                            next;
                        }
                        else {
                            $q = $this->{'threshold'} - 2;
                        }
                    }
                    elsif ( $q >= $this->{'threshold'} ) {
                        $is_locked = 1;
                    }
                    $t = $time;
                }
                elsif ($lock) {
                    $q++
                        if !defined $this->{'counted_user'}
                        || !$this->{'counted_user'}{$email};
                    $t = $time;
                    $is_locked = 1 if $q >= $this->{'threshold'};
                    $this->{'counted_user'}{$email} = 1;
                }
                else {
                    next;
                }
                $new_data = join( "\x09", $h, $q, $t ) . "\x0A" . $new_data;
            }
            elsif ( $time - $t <= $this->{'latency'} ) {
                $new_data .= join( "\x09", $h, $q, $t ) . "\x0A";
            }
        }
        if ( !$done && $lock ) {
            $new_data = join( "\x09", $host, 1, $time ) . "\x0A" . $new_data;
            $this->{'counted_user'}{$email} = 1;
        }
        if ($new_data) {
            use bytes;
            $new_data = bytes::substr( $new_data, 0, 960 )
                if bytes::length $new_data > 960;
            $locked{ $this->encode_utf8($email) }
                = $this->encode_utf8($new_data);
        }
        elsif ( defined $locked{ $this->encode_utf8($email) } ) {
            delete $locked{ $this->encode_utf8($email) };
        }
    }
    untie %locked;
    if ( $is_locked && !$this->{'locked_msg_added'} ) {
        push(
            @{ $this->{'error_dialog'} },
            'パスワードをロックしました。<strong id="pswd-countdown">'
                . ( $this->{'latency'} / 60 )
                . '分</strong>以上待ってからもう一度お試し下さい。'
                . '<script type="text/javascript" language="JavaScript">'
                . "\x0A"
                . '<!-- <![CDATA[' . "\x0A"
                . 'var left_latency = '
                . $this->{'latency'}
                . '; var pswd_timer = setInterval(function () { left_latency--; document.getElementById(\'pswd-countdown\').innerHTML = ((left_latency >= 60) ? parseInt(left_latency / 60) + \'分\' : \'\') + parseInt(left_latency % 60) + \'秒\'; if (left_latency <= 0) { clearInterval(pswd_timer); } },1000);'
                . "\x0A"
                . '// ]]> -->' . "\x0A"
                . '</script>'
        );
        $this->{'locked_msg_added'} = 1;
    }
    return $is_locked;
}

######################################################################
#認証
sub is_master ($$;$) {
    my ( $this, $pswd, $fix_enabled ) = @_;
    my $is_master = 0;
    if (   defined $pswd
        && defined $this->{'master_password'}
        && $this->{'master_password'} ne '' )
    {
        $is_master
            = ( $this->{'master_password_encrypted'} )
            ? $this->is_identified( $pswd, $this->{'master_password'} )
            : ( ref $pswd && defined $pswd->{'encrypted_password'} )
            ? $this->is_identified( $this->{'master_password'},
            $pswd->{'encrypted_password'} )
            : (    !ref $pswd
                && $pswd ne ''
                && $pswd eq $this->{'master_password'} ) ? 1
            : 0;
        if (   $is_master
            && $fix_enabled
            && $this->{'fix_enabled'}
            && !ref $pswd )
        {
            $_[1] = {
                'encrypted_password' => (
                    ( $this->{'master_password_encrypted'} )
                    ? $this->{'master_password'}
                    : $this->encrypt($pswd)
                )
            };
        }
    }
    return $is_master;
}

######################################################################
#重複のチェック
sub is_unique ($$) {
    my ( $this, $pswd ) = @_;
    return 1 if !defined $this->{'check'} || !$this->{'check'};
    return 0 if $this->read_email($pswd);
    return 1;
}

######################################################################
#登録
sub register ($$;$$$) {
    my ( $this, $email, $subject, $message, $no_pending ) = @_;
    my ($pswd);
    $this->get_preference;

    #不正なアドレス
    return '' if $this->is_invalid_account($email);

    while (1) {
        $pswd = $this->rand_char;
        if ( !$this->is_master($pswd) && $this->is_unique($pswd) ) {
            last;
        }
    }
    return $this->password_register( $pswd, $email, 1, $subject, $message, 1,
        $no_pending );
}

sub is_invalid_account ($$) {
    my ( $this, $email ) = @_;
    if ( $this->is_invalid_email_address( $this->{'cgi_master'} ) ) {
        push(
            @{ $this->{'error_dialog'} },
            '登録に失敗しました。管理人の設定ミス！管理人メールアドレスを確認してください。'
        );
        return 1;
    }
    if ( $this->is_invalid_email_address($email) ) {
        push(
            @{ $this->{'error_dialog'} },
            '登録に失敗しました。メールアドレスを確認してください。'
        );
        return 1;
    }
    if (   ref $this->{'bad_addr'}
        && @{ $this->{'bad_addr'} } > 0
        && grep( eval { $email =~ /$_/ }, @{ $this->{'bad_addr'} } ) )
    {
        push(
            @{ $this->{'error_dialog'} },
            '登録に失敗しました。登録を制限されているメールアドレスです。'
        );
        return 1;
    }
    if (   ref $this->{'good_addr'}
        && @{ $this->{'good_addr'} } > 0
        && grep( eval { $email =~ /$_/ }, @{ $this->{'good_addr'} } ) )
    {
        push(
            @{ $this->{'error_dialog'} },
            '登録に失敗しました。登録対象外のメールアドレスです。'
        );
        return 1;
    }
    if (   ref $this->{'freemail'}
        && @{ $this->{'freemail'} } > 0
        && grep( eval { $email =~ /$_/ }, @{ $this->{'freemail'} } ) )
    {
        push(
            @{ $this->{'error_dialog'} },
            '登録に失敗しました。フリーメールは受け付けません。'
        );
        return 1;
    }
    return 0;
}

sub password_register ($$$;$$$$$) {
    no warnings qw(untie);
    my ( $this, $pswd, $email, $mailto_flag, $subject, $message,
        $fix_disabled, $no_pending )
        = @_;
    my ( $cryptpswd, %id, %bad, $tied_flag, $is_identifier );
    return '' if !defined $pswd || $pswd eq '';
    $tied_flag = 0;
    if ( $this->{'openid_enabled'} && $pswd =~ m!^(https?|xri)://!i ) {
        $mailto_flag   = 0;
        $cryptpswd     = $pswd;
        $is_identifier = 1;
    }
    else {
        $cryptpswd = $this->encrypt($pswd);
    }
    tie( %bad, $env::dbm_file, $this->{'dbm_path'} . '/baduser',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %bad && $bad{ $this->encode_utf8($email) } ) {
        push(
            @{ $this->{'error_dialog'} },
            '登録を拒否されました。管理人によってこのメールアドレスの登録は制限されています。'
        );
        untie %bad;
        return '';
    }
    untie %bad;
    tie(%id, $env::dbm_file,
        $this->{'dbm_path'} . '/users',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %id ) {
        $id{ $this->encode_utf8($email) } = $this->encode_utf8($cryptpswd);
        $tied_flag = 1;
    }
    untie %id;
    if ($tied_flag
        && (  !$mailto_flag
            || $this->mail_to_user( $pswd, $email, $subject, $message ) )
        )
    {
        $this->mtime( $email, ( ($no_pending) ? 0 : 1 ) );
        $this->delete_data( 'locked', $email );
        if ( !$fix_disabled && $this->{'fix_enabled'} ) {
            $_[1]
                = ($is_identifier)
                ? { 'claimed_id' => $pswd }
                : { 'encrypted_password' => $cryptpswd };
        }
        return $pswd;
    }
    return '';
}

sub add_id_info ($$) {
    my ( $this, $pswd, $hash ) = @_;
    return $pswd if defined $pswd && !ref $pswd;
    my %info = %$pswd;
    while ( my ( $k, $v ) = each %$hash ) {
        $info{$k} = $v;
    }
    return \%info;
}

######################################################################
#メールの送付
sub mail_to_user ($$$;$$) {
    my ( $this, $pswd, $email, $subject, $message ) = @_;
    unless ( ref $cgi::SendMail ) {
        require CMONOS::CGI_Lib::SendMail;
        $cgi::SendMail
            = SendMail->new( $this->{'sendmail'}, $this->{'cgi_master'} );
    }
    $subject
        = ( defined $this->{'mail_subject'} && $this->{'mail_subject'} ne '' )
        ? $this->{'mail_subject'}
        : '登録のお知らせ'
        if !defined $subject || $subject eq '';
    $message = $this->{'mail_msg'}
        if ( !defined $message || $message eq '' )
        && defined $this->{'mail_msg'}
        && $this->{'mail_msg'} ne '';

    if ( !defined $message || $message eq '' ) {
        my $cgiurl = $cgi::SendMail->my_url;
        $message = <<MSG;
ご登録ありがとうございます！

あなたのパスワードは、
-----------------------
$pswd
-----------------------
です。

パスワード記入の際には、登録に使用したメールアドレスと
上記パスワードを入力して下さい。

万一パスワードを忘れてしまったときは、登録をやり直して
新しいパスワードを取得して下さい。

あなたの参加を楽しみにしてお待ちしております。

-------------------------------------------------
$cgiurl
$this->{'cgi_master_name'} <$this->{'cgi_master'}>
-------------------------------------------------
MSG

    }
    elsif ( $message !~ /<password>/i ) {
        $message .= <<MSG;

-----------------------------------------
E-Mail   : <email>
Password : <password>
-----------------------------------------
MSG

    }
    else {
        $message =~ s/<password>/$pswd/gi;
    }
    $message =~ s/<email>/$email/gi;

    if ($cgi::SendMail->send_mail(
            'To'   => $email,
            'From' => "$this->{'cgi_master_name'} <$this->{'cgi_master'}>",
            'Reply-To' =>
                "$this->{'cgi_master_name'} <$this->{'cgi_master'}>",
            'Subject' => $subject,
            'message' => $message,
            'info'    => '0'
        )
        )
    {
        push(
            @{ $this->{'success_dialog'} },
            'メールアドレスにパスワードを送付しました。'
        );
        return 1;
    }
    else {
        push(
            @{ $this->{'error_dialog'} },
            'パスワード通知メールの送信に失敗しました。もう一度やりなおしてください。'
        );
        push( @{ $this->{'error_dialog'} }, $cgi::SendMail->error_dialog );
        return 0;
    }
}

########################################################################
#メールアドレス登録抹消
sub remove ($$$;$) {
    my ( $this, $pswd, $email, $skip_auth ) = @_;
    if ( !$skip_auth && !$this->is_registered( $pswd, $email ) ) {
        push(
            @{ $this->{'error_dialog'} },
            '登録したメールアドレス情報の削除に失敗しました。'
        );
        return 0;
    }
    if (   !$this->delete_data( 'users', $email )
        || !$this->delete_data( 'mtime',  $email )
        || !$this->delete_data( 'locked', $email )
        || !$this->delete_name($email) )
    {
        push(
            @{ $this->{'error_dialog'} },
            '登録したメールアドレス情報の削除に失敗しました。データベースとの接続に問題があります。'
        );
        return 0;
    }
    push(
        @{ $this->{'success_dialog'} },
        '登録したメールアドレス情報を削除しました。'
    );
    return 1;
}

#データの削除
sub delete_data ($$$) {
    my ( $this, $db, $key ) = @_;
    my ( %data, $check );
    $check = 0;
    tie(%data, $env::dbm_file,
        $this->{'dbm_path'} . '/' . $db,
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %data ) {
        utf8::encode($key) if utf8::is_utf8($key);
        delete $data{$key} if defined $data{$key};
        $check = 1 unless defined $data{$key};
    }
    untie %data;
    return $check;
}

#登録ネームの削除
sub delete_name ($$$) {
    my ( $this, $email ) = @_;
    my ( %name, %email, %user_name, $name, $check );
    $check = 0;
    tie(%name, $env::dbm_file,
        $this->{'dbm_path'} . '/email_to_name',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %name ) {
        $name = $this->decode_utf8( $name{ $this->encode_utf8($email) } );
        delete $name{ $this->encode_utf8($email) };
        $check = 1 unless defined $name{ $this->encode_utf8($email) };
    }
    untie %name;
    if ( defined $name && $name ne '' ) {
        tie(%email, $env::dbm_file,
            $this->{'dbm_path'} . '/name_to_email',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %email ) {
            delete $email{ $this->encode_utf8($name) };
            $check = 1 unless defined $email{ $this->encode_utf8($name) };
        }
        untie %email;
    }
    tie(%user_name, $env::dbm_file,
        $this->{'dbm_path'} . '/user_name',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %user_name ) {
        delete $user_name{ $this->encode_utf8($email) };
        $check = 1 unless defined $user_name{ $this->encode_utf8($email) };
    }
    untie %user_name;
    return $check;
}

########################################################################
#メールアドレス変更
sub move ($$$$$) {
    my ( $this, $old_pswd, $old_email, $new_pswd, $new_email, $skip_auth )
        = @_;
    if (!$skip_auth
        && (   !$this->is_registered( $old_pswd, $old_email )
            || !$this->is_registered( $new_pswd, $new_email ) )
        )
    {
        push(
            @{ $this->{'error_dialog'} },
            '登録情報の移行に失敗しました。移行先メールアドレスまたはパスワードに問題があります。'
        );
        return 0;
    }
    if (   !$this->delete_data( 'users', $old_email )
        || !$this->delete_data( 'mtime',  $old_email )
        || !$this->delete_data( 'locked', $old_email ) )
    {
        push(
            @{ $this->{'error_dialog'} },
            '以降元メールアドレス情報の削除に失敗しました。データベースとの接続に問題があります。'
        );
        return 0;
    }
    my ( %name, %email, %user_name, $name );
    utf8::encode($old_email) if utf8::is_utf8($old_email);
    utf8::encode($new_email) if utf8::is_utf8($new_email);
    tie(%name, $env::dbm_file,
        $this->{'dbm_path'} . '/email_to_name',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %name ) {
        if ( defined $name{$old_email} ) {
            $name = $name{$new_email} = $name{$old_email};
            delete $name{$old_email};
        }
    }
    untie %name;
    if ( defined $name && $name ne '' ) {
        tie(%email, $env::dbm_file,
            $this->{'dbm_path'} . '/name_to_email',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %email ) {
            $email{$name} = $new_email;
        }
        untie %email;
    }
    tie(%user_name, $env::dbm_file,
        $this->{'dbm_path'} . '/user_name',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %user_name ) {
        if ( defined $user_name{$old_email} ) {
            $name = $user_name{$new_email} = $user_name{$old_email};
            delete $user_name{$old_email};
        }
    }
    untie %user_name;
    push(
        @{ $this->{'success_dialog'} },
        '登録情報の移行に成功しました。'
    );
    return 1;
}

########################################################################
#メールアドレス登録抹消(一括)
sub remove_in_bulk ($$;$$$) {
    my ( $this, $email, $notice, $reason, $refuse ) = @_;
    my ($remove_msg,    $remove_subject, $error_flag, %name_to_email,
        %email_to_name, %user_name,      %users,      %mtime,
        %baduser,       %locked
    );
    $reason = '' if !defined $reason;
    if ( defined $notice && $notice ) {
        unless ( ref $cgi::SendMail ) {
            require CMONOS::CGI_Lib::SendMail;
            $cgi::SendMail
                = SendMail->new( $this->{'sendmail'}, $this->{'cgi_master'} );
        }

        $this->get_preference;
        $remove_subject
            = ( $this->{'remove_subject'} )
            ? $this->{'remove_subject'}
            : '【お知らせ】登録したメールアドレス情報を削除しました';
        $remove_msg = $this->{'remove_msg'};

        if ( defined $remove_msg && $remove_msg ne '' ) {
            $remove_msg =~ s/<reason>/$reason/igs;

        }
        else {
            my $cgiurl = $cgi::SendMail->my_url;

            $remove_msg = <<MSG;
${reason}管理人によってあなたの登録情報は削除されました。

ご利用ありがとうございました。

-------------------------------------------------
$cgiurl
$this->{'cgi_master_name'} <$this->{'cgi_master'}>
-------------------------------------------------
MSG
        }
    }

    tie(%name_to_email, $env::dbm_file,
        $this->{'dbm_path'} . '/name_to_email',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    tie(%email_to_name, $env::dbm_file,
        $this->{'dbm_path'} . '/email_to_name',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    tie(%user_name, $env::dbm_file,
        $this->{'dbm_path'} . '/user_name',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    tie(%users, $env::dbm_file,
        $this->{'dbm_path'} . '/users',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    tie(%baduser, $env::dbm_file,
        $this->{'dbm_path'} . '/baduser',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    ) if defined $refuse && $refuse;
    tie(%mtime, $env::dbm_file,
        $this->{'dbm_path'} . '/mtime',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    tie(%locked, $env::dbm_file,
        $this->{'dbm_path'} . '/locked',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );

    foreach my $i (@$email) {
        utf8::encode($i) if utf8::is_utf8($i);
        my ( $name, $check );
        $check = 0;
        if ( ref tied %users ) {
            if ( defined $users{$i} ) {
                delete $users{$i};
            }
            $check = 1 unless defined $users{$i};
        }
        if ( ref tied %mtime ) {
            if ( defined $mtime{$i} ) {
                delete $mtime{$i};
            }
            $check = 1 unless defined $mtime{$i};
        }
        if ( ref tied %locked ) {
            if ( defined $locked{$i} ) {
                delete $locked{$i};
            }
            $check = 1 unless defined $locked{$i};
        }
        if ( ref tied %email_to_name ) {
            if ( defined $email_to_name{$i} ) {
                $name = $this->encode_utf8( $email_to_name{$i} );
                delete $email_to_name{$i};
            }
            $check = 1 unless defined $email_to_name{$i};
        }
        else {
            $error_flag++;
            push(
                @{ $this->{'error_dialog'} },
                '名前の削除に失敗しました。データベースに接続できません。'
            );
        }
        if ( ref tied %name_to_email ) {
            if ( defined $name && $name ne '' ) {
                if ( defined $name_to_email{$name} ) {
                    delete $name_to_email{$name};
                }
                $check = 1 unless defined $name_to_email{$name};
            }
        }
        else {
            $error_flag++;
            push(
                @{ $this->{'error_dialog'} },
                '名前の削除に失敗しました。データベースに接続できません。'
            );
        }
        if ( ref tied %user_name ) {
            if ( defined $user_name{$i} ) {
                delete $user_name{$i};
            }
            $check = 1 unless defined $user_name{$i};
        }
        else {
            $error_flag++;
            push(
                @{ $this->{'error_dialog'} },
                'ユーザー名の削除に失敗しました。データベースに接続できません。'
            );
        }
        if ( !$check ) {
            push(
                @{ $this->{'error_dialog'} },
                '登録したメールアドレス情報を削除できませんでした（'
                    . $this->decode_utf8($i)
                    . '）。'
            );
            $error_flag++;
            next;
        }

        #登録制限指定
        if ( defined $refuse && $refuse ) {
            if ( ref tied %baduser ) {
                $baduser{$i} = time;
            }
            else {
                $error_flag++;
                push(
                    @{ $this->{'error_dialog'} },
                    '登録制限指定に失敗しました（'
                        . $i
                        . '）。データベースとの接続に問題があります。'
                );
            }
        }

        #登録抹消通知メール
        if ( defined $remove_msg && $remove_msg ) {
            unless (
                $cgi::SendMail->send_mail(
                    'To' => $this->decode_utf8($i),
                    'From' =>
                        "$this->{'cgi_master_name'} <$this->{'cgi_master'}>",
                    'Reply-To' =>
                        "$this->{'cgi_master_name'} <$this->{'cgi_master'}>",
                    'Subject' => $remove_subject,
                    'message' => $remove_msg,
                    'info'    => '0'
                )
                )
            {
                push(
                    @{ $this->{'success_dialog'} },
                    '登録抹消通知メールを送信できませんでした。'
                );
            }
        }
    }

    untie %locked;
    untie %mtime;
    untie %users;
    untie %user_name;
    untie %email_to_name;
    untie %name_to_email;
    untie %baduser if defined $refuse && $refuse;
    return ($error_flag) ? 0 : 1;
}

########################################################################
#メールアドレス登録制限対象解除(一括)
sub cancel_in_bulk ($$) {
    my ( $this, $email ) = @_;
    my (%bad);
    tie(%bad, $env::dbm_file,
        $this->{'dbm_path'} . '/baduser',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    push(
        @{ $this->{'error_dialog'} },
        '登録制限解除に失敗しました。データベースと接続できませんでした。'
    ) unless ref tied %bad;
    foreach my $m (@$email) {
        delete $bad{ $this->encode_utf8($m) };
        push(
            @{ $this->{'error_dialog'} },
            '登録制限解除に失敗しました（' . $m . '）。'
        ) if defined $bad{ $this->encode_utf8($m) };
    }
    untie %bad;
}

########################################################################
#一定時間利用の無い登録データを削除
sub gc ($$;$) {
    my ( $this, $time, $pending ) = @_;
    my ( $count, $mcount, @inactive_users, %mtime );
    $time = 1 if !$time;
    $time = time - $time * 86400;
    tie( %mtime, $env::dbm_file, $this->{'dbm_path'} . '/mtime',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %mtime ) {
        while ( my ( $i, $t ) = each %mtime ) {
            $count++;
            if ((      ( $pending && $t =~ /\.0$/ )
                    || ( !$pending && $t !~ /\.0$/ )
                )
                && $time > $t
                )
            {
                $mcount++;
                push( @inactive_users, $i );
            }
        }
    }
    untie %mtime;
    $this->remove_in_bulk( \@inactive_users ) if $mcount;
    return ( $mcount, $count );
}

########################################################################
#メールアドレス登録フォーム
sub print_email_register_form ($;$$$) {
    my ( $this, $email, $input, $script_name ) = @_;
    $script_name = $ENV{'SCRIPT_NAME'}
        if !defined $script_name || $script_name eq '';

    print <<FORM;
<form action="$script_name" method="post">
$input
<label for="email">メールアドレス</label><input name="email" type="text" size="30" value="$email" /><input TYPE="submit" VALUE="登録" />
</form>
FORM
}

########################################################################
#名前チェック
sub is_available_name ($$$$;$$) {
    my ( $this, $pswd, $email, $name, $is_registered, $nofix ) = @_;
    return 1 if !defined $name || $name eq '';
    my ( %email, $registered_email, $is_available, $available_name,
        $registered_name );
    tie( %email, $env::dbm_file, $this->{'dbm_path'} . '/name_to_email',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %email ) {
        if ( defined $email{ $this->encode_utf8($name) } ) {
            $registered_email
                = $this->decode_utf8( $email{ $this->encode_utf8($name) } );
            my $d = 1;
            $d++ while defined $email{ $this->encode_utf8( $name . $d ) };
            $available_name = $name . $d;
        }
        else {
            $is_available = 1;
        }
    }
    else {
        $is_available = 1;
    }
    untie %email;
    my $temp_name_enabled = 0;
    if (   defined $email
        && $email ne ''
        && ( $is_registered || $this->is_registered( $pswd, $email, 1 ) ) )
    {
        my (%name);
        if ( defined $is_available && $is_available ) {
            my ($old_name);
            tie(%name, $env::dbm_file,
                $this->{'dbm_path'} . '/email_to_name',
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );
            if ( ref tied %name ) {
                $old_name
                    = $this->decode_utf8(
                    $name{ $this->encode_utf8($email) } )
                    if defined $name{ $this->encode_utf8($email) };
                $name{ $this->encode_utf8($email) }
                    = $this->encode_utf8($name);
            }
            untie %name;
            tie(%email, $env::dbm_file,
                $this->{'dbm_path'} . '/name_to_email',
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );
            if ( ref tied %email ) {
                delete $email{ $this->encode_utf8($old_name) }
                    if defined $old_name;
                $email{ $this->encode_utf8($name) }
                    = $this->encode_utf8($email);
            }
            untie %email;
            push(
                @{ $this->{'success_dialog'} },
                '「'
                    . $name
                    . '」を名前として登録しました。他の利用者が同じ名前を使うことはありません。'
            );

        }
        elsif ( $email eq $registered_email ) {
            $is_available = 1;

        }
        else {
            tie(%name, $env::dbm_file,
                $this->{'dbm_path'} . '/email_to_name',
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );
            if ( ref tied %name ) {
                if ( defined $name{ $this->encode_utf8($email) }
                    && $name{ $this->encode_utf8($email) } ne '' )
                {
                    eval {
                        $_[3] = $this->decode_utf8(
                            $name{ $this->encode_utf8($email) } );
                    };
                    $registered_name = $this->decode_utf8(
                        $name{ $this->encode_utf8($email) } );
                }
                else {
                    eval { $_[3] = $available_name };
                    $name{ $this->encode_utf8($email) }
                        = $this->encode_utf8($available_name);
                    $temp_name_enabled = 1;
                }
            }
            else {
                eval { $_[3] = $available_name };
            }
            untie %name;
            if ($temp_name_enabled) {
                tie(%email, $env::dbm_file,
                    $this->{'dbm_path'} . '/name_to_email',
                    O_RDWR | O_CREAT,
                    $env::dbm_mode
                );
                if ( ref tied %email ) {
                    $email{ $this->encode_utf8($available_name) }
                        = $this->encode_utf8($email);
                }
                untie %email;
            }
        }

    }
    elsif ( !defined $is_available || !$is_available ) {
        eval { $_[3] = $available_name };
    }

    if ( defined $is_available && $is_available ) {
        return 1;
    }
    else {
        push(
            @{ $this->{'error_dialog'} },
            '「' . $name . '」は既に使われています。'
        );
        if ( $name ne $_[3] ) {
            if ( defined $registered_name && $registered_name ne '' ) {
                push(
                    @{ $this->{'error_dialog'} },
                    '既に名前として登録されている「'
                        . $registered_name
                        . '」を読み込みました。'
                );
            }
            elsif (!$nofix
                && defined $available_name
                && $available_name ne '' )
            {
                push(
                    @{ $this->{'error_dialog'} },
                    '現在利用できる名前「'
                        . $available_name
                        . '」に置き換えました。'
                );
                push(
                    @{ $this->{'error_dialog'} },
                    '「'
                        . $available_name
                        . '」を名前として登録しました。他の利用者が同じ名前を使うことはありません。'
                ) if $temp_name_enabled;
            }
        }
        return 0;
    }
}

sub name_to_email ($$$;$) {
    my ( $this, $pswd, $name, $is_registered ) = @_;
    return '' if !defined $name || $name eq '';
    my ( %email, $email );
    tie( %email, $env::dbm_file, $this->{'dbm_path'} . '/name_to_email',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %email ) {
        $email = $this->decode_utf8( $email{ $this->encode_utf8($name) } );
    }
    untie %email;
    if ( ( defined $is_registered && $is_registered )
        || $this->is_registered( $pswd, $email ) )
    {
        return $email;
    }
    else {
        return '';
    }
}

sub email_to_name ($$$;$) {
    my ( $this, $pswd, $email, $is_registered ) = @_;
    my ( %name, $name );
    if ( !defined $email || $email eq '' ) {
        return '';
    }
    elsif ( $is_registered || $this->is_registered( $pswd, $email ) ) {
        tie( %name, $env::dbm_file, $this->{'dbm_path'} . '/email_to_name',
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %name ) {
            $name = $this->decode_utf8( $name{ $this->encode_utf8($email) } );
        }
        untie %name;
        return $name;
    }
    else {
        return '';
    }
}

sub name_and_email ($$$;$) {
    my ( $this, $pswd, $email, $is_registered ) = @_;
    return '' if !defined $email || $email eq '';
    my $name = $this->email_to_name( $pswd, $email, $is_registered );
    $name =~ s/[<>\x0A\x0D,]//sg;
    return ( defined $name && $name ne '' )
        ? $name . ' <' . $email . '>'
        : $email;
}

sub email_to_name_and_email ($$) {
    my ( $this, $email ) = @_;
    my $addr = '';
    my @emails = ( ref $email ) ? @$email : split( /\s*,\s*/, $email );
    foreach my $m (@emails) {
        $addr
            .= ( ($addr) ? ', ' : '' )
            . ( ( $m !~ /[^<>]+<[^<>]+\@[^<>]+>/ )
            ? $this->name_and_email( '', $email, 1 )
            : $m );
    }
    return $addr;
}

########################################################################
# ユーザー名登録
sub user_name_register ($$$$;$) {
    my ( $this, $pswd, $email, $user_name, $is_registered ) = @_;
    return 1
        if !defined $email
        || !defined $user_name
        || $email eq ''
        || $user_name eq '';
    if ( $is_registered || $this->is_registered( $pswd, $email ) ) {
        my ( %user_name, $tied_flag );
        tie(%user_name, $env::dbm_file,
            $this->{'dbm_path'} . '/user_name',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %user_name ) {
            $user_name{ $this->encode_utf8($email) }
                = $this->encode_utf8($user_name);
            $tied_flag = 1;
        }
        untie %user_name;
        return ($tied_flag) ? 1 : 0;
    }
}

sub email_to_user_name ($$$;$) {
    my ( $this, $pswd, $email, $is_registered ) = @_;
    my ( %user_name, $user_name );
    if ( !defined $email || $email eq '' ) {
        return '';
    }
    elsif ( $is_registered || $this->is_registered( $pswd, $email ) ) {
        tie( %user_name, $env::dbm_file, $this->{'dbm_path'} . '/user_name',
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %user_name ) {
            $user_name = $this->decode_utf8(
                $user_name{ $this->encode_utf8($email) } );
        }
        untie %user_name;
        return $user_name;
    }
    else {
        return '';
    }
}

########################################################################
# メールアドレスと名前のハッシュを取得
sub get_name_hash_from_emails ($@) {
    my ( $this, @emails ) = @_;
    my ( %user_name, %name, %hash );
    return () if !@emails;
    %hash = ();
    tie( %name, $env::dbm_file, $this->{'dbm_path'} . '/email_to_name',
        O_RDONLY, $env::dbm_mode );
    tie( %user_name, $env::dbm_file, $this->{'dbm_path'} . '/user_name',
        O_RDONLY, $env::dbm_mode );
    foreach my $email (@emails) {
        if ( !$email ) {
            next;
        }
        elsif ( ref tied %name
            && defined $name{ $this->encode_utf8($email) } )
        {
            $hash{$email}
                = $this->decode_utf8( $name{ $this->encode_utf8($email) } );
        }
        elsif ( ref tied %user_name
            && defined $user_name{ $this->encode_utf8($email) } )
        {
            $hash{$email} = $this->decode_utf8(
                $user_name{ $this->encode_utf8($email) } );
        }
    }
    untie %name;
    untie %user_name;
    return %hash;
}

sub get_name_from_email ($$) {
    my ( $this, $email ) = @_;
    my ( %user_name, %name, $name );
    return () if !$email;
    $name = '';
    tie( %name, $env::dbm_file, $this->{'dbm_path'} . '/email_to_name',
        O_RDONLY, $env::dbm_mode );
    tie( %user_name, $env::dbm_file, $this->{'dbm_path'} . '/user_name',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %name && defined $name{ $this->encode_utf8($email) } ) {
        $name = $this->decode_utf8( $name{ $this->encode_utf8($email) } );
    }
    elsif ( ref tied %user_name
        && defined $user_name{ $this->encode_utf8($email) } )
    {
        $name
            = $this->decode_utf8( $user_name{ $this->encode_utf8($email) } );
    }
    untie %name;
    untie %user_name;
    return $name;
}

########################################################################
# メールアドレス書式チェック
sub is_invalid_email_address ($$) {
    my ( $this, $address ) = @_;
    return 0
        if defined $address
        && $address
        =~ m!^[0-9a-zA-Z_\-\.\+]+\@[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-\.]+$!i;
    return 1;
}

########################################################################
#メッセージ
sub error_dialog ($) {
    my $this = shift;
    my @error_dialog
        = ( ref $this->{'error_dialog'} ) ? @{ $this->{'error_dialog'} } : ();
    delete $this->{'error_dialog'};
    delete $this->{'locked_msg_added'};
    return grep( defined $_ && $_ ne '', @error_dialog );
}

sub success_dialog ($) {
    my $this = shift;
    my @success_dialog
        = ( ref $this->{'success_dialog'} )
        ? @{ $this->{'success_dialog'} }
        : ();
    delete $this->{'success_dialog'};
    return grep( defined $_ && $_ ne '', @success_dialog );
}

######################################################################
1;
