package Table;
use strict;
use utf8;
######################################################################
# テーブルタグモジュール
# Table.pm 1.1.7
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-08-31
######################################################################
#
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	(0)-0 オブジェクトを作る。
#		require CMONOS::CGI_Lib::Table;
#		obj = Table->new();
#
#	(0)-1 テーブルタグオプションを設定する。
#		obj->set(
#			'bg' => '1',									# 1=テーブルの背面を反映させる
#			'fixed' => '1',									# 1=読み込んだところから表示する
#			'summary' => 'summary',							# summary
#			'caption' => 'caption',							# caption
#			'cellpadding' => '2',							# cellpadding
#			'cellspacing' => '2',							# cellspacing
#			'border' => '1',								# border
#			'bordercolor' => '#000000',						# bordercolor
#			'background' => './bg.gif',						# background
#			'tablealign' => 'center',						# テーブル全体のalign
#			'tablewidth' => '500',							# テーブル全体のwidth
#			'tableheight' => '500',							# テーブル全体のheight
#			'tablehspace' => '10',							# hspace
#			'tablevspace' => '10',							# テーブル全体のvspace
#			'tableclass' => 'class1'						# テーブル全体のclass
#			'tableid' => 'table1',							# テーブル全体のid
#			'tablestyle' => 'style',						# テーブル全体のstyle
#			'tableoption' => '',							# 上記以外のテーブル全体オプション
#			'width' => '50,50,50',							# @width（繰り返す）
#			'height' => '10,10,10',							# @height（繰り返す）
#			'nowrap0' => '1,0,0',							# @nowrap（繰り返す）
#			'nowrap1' => '1,0,0',							# @nowrap（繰り返す）
#			'align0' => 'right,right,right',				# @align（繰り返す）
#			'align1' => 'right,right,right',				# @align（繰り返す）
#			'valign0' => 'top,top,middle',					# @valign（繰り返す）
#			'valign1' => 'top,top,middle',					# @valign（繰り返す）
#			'thenabled' => 'true',
#			'bgcolorF' => '#000000,#333333',				# 一列目のbgcolor（繰り返す）
#			'bgcolor0' => '#000000,#333333',				# @bgcolor0（繰り返す）
#			'bgcolor1' => '#333333,#000000',				# @bgcolor1（繰り返す）
#			'background0' => './bg0.gif,./bg1.gif',			# @background（繰り返す）
#			'background1' => './bg1.gif,./bg0.gif',			# @background（繰り返す）
#			'fontcolorH' => '#cccccc,#ffffff',				# 一行目のフォント色（繰り返す）
#			'fontcolorF' => '#cccccc,#ffffff',				# 一列目のフォント色（繰り返す）
#			'fontcolor0' => '#ffffff,#cccccc',				# @fontcolor0（繰り返す）
#			'fontcolor1' => '#ffffff,#cccccc',				# @fontcolor1（繰り返す）
#			'classH' => 'class1,class2',					# 一行目のクラス（繰り返す）
#			'classF' => 'class1,class2',					# 一列目のクラス（繰り返す）
#			'class0' => 'class1,class2',					# @class0（繰り返す）
#			'class1' => 'class1,class2',					# @class1（繰り返す）
#			'trclass' => 'class1,class2',					# TRのクラス（繰り返す）
#			'br' => 1										# 1=改行を<br />に変換する
#			'thenabled' => 'auto|col|row|false'					# thタグを使う（true|row=必ず一行目をthにする/col=必ず一列目をthにする/false=thを使わない/auto=必要なときのみ1行目をthにする）
#		);
#
#	(0)-3 設定を取得する
#		obj->get_setting(\$table,type,op);
#			table		設定情報を付加するテキストのリファレンス
#						<!--csv ...setting...-->table<!--/csv-->という形式になります。
#			type		csv or tsv
#			op			"no"=設定情報を付加しない/"tag"=<!--csv--><!--/csv-->タグのみ付加
#
#	(0)-4 設定をリセットする
#		obj->reset_all;
#
#	(0)-5 漢字コード/改行コードをセットする
#		obj->code_info([\$text]);
#			text		変換作業前のテキスト
#
# ---------------------------------------------------------------------------------------
#	(1) TABLEの作成
#		(1)-0-1	obj->table;
#		(1)-0-1	obj->_table;
#		(1)-1-1	obj->tr;
#		(1)-1-1	obj->_tr;
#		(1)-2-1	obj->td(data,\%option) obj->th(data,\%option)
#					%option		'colspan','rowspan','align','valign','bgcolor','background',
#								'fontcolor','fontsize','fontface','fontstyle'
#		(1)-2-2	obj->trtd(data,end,\%option);
#					obj->limitで設定した列制限値を超えると自動的に改行します。
#					end=1 でその行の列を列制限値まで増やして改行します。
#		(1)-2-3	obj->limit(collimit);
#					collimit		列制限値
#
# ---------------------------------------------------------------------------------------
#	(2)-0 obj->sv_to_table(\$sv[,info[,type[,option]]]);
#		csv/tsvの分解->table
#			sv			csv/tsvデータのリファレンス
#			type		csv or tsv
#			info		"no"=設定情報を付加しない/"tag"=<!--csv--><!--/csv-->タグのみ付加
#			option		分解後の処理を担当するオブジェクト
#							※obj2を設定すると、TABLE作成以外の処理を施すことができます。
#							※obj2側に(1)と同じ名前のサブルーチンを用意する必要が有ります。
#						テーブルオプション
#							※そのテーブルだけに適用されるオプションを指定できます。
#
#	(2)-1 text = obj->text_to_table(text[,info[,option[,\%obj]]]);
#		テキストのcsv/tsv部分をテーブルに変換し、変換したテキストを返します。
#			text		テキスト
#			info		"no"=設定情報を付加しない/"tag"=<!--_csv--><!--/_csv-->タグのみ付加
#			option		"1"=<!--tsv--><!--/tsv-->タグが無くてもtsvをテーブル化
#						"2"=<!--csv--><!--/csv-->タグが無くてもtsv/csvをテーブル化
#			\%obj		テーブルオブジェクトを収めたハッシュ（key=id名/value=オブジェクト）
#						のリファレンス
#
# ---------------------------------------------------------------------------------------
#	(3)-0-0 obj->array_to_sv(type,@array);
#		ARRAY->csv/tsv 1ライン
#			type		csv or tsv
#			array		配列
#
#	(3)-1 obj->text_to_sv(text[,info[,type]]);
#		TABLE->csv/tsv
#			text		テキスト
#			info		"no"=設定情報を付加しない/"tag"=<!--csv--><!--/csv-->タグのみ付加
#			type		csv or tsv
#						※タイプ指定がない場合、タグのないテーブルは変換しません。
#						※大文字にすると、タグによる指定よりtypeを優先します。
#
######################################################################
#	■利用規約
#	このスクリプトの著作権はCMONOSが保有しています。
#	このサブルーチン集は、以下の条件に従うかぎり自由に再配布できます。
#		・著作権表示を変更してはいけません。
#		・修正を加えた場合は、修正者名・修正者の連絡先・修正個所・修正日
#		　時を明記して下さい。
#	このスクリプトを利用したことによって生じた、いかなる損害にも
#	CMONOSは責任を負いません。
######################################################################
#オブジェクト作成
use Jcode;

sub new ($@) {
    my ( $pkg, @setting ) = @_;
    my $this = {};
    bless $this;
    $this->set(@setting) if @setting;
    $this;
}

sub jcode_convert($$$$;$) {
    my ( $this, $str, $ocode, $icode, $was_entity ) = @_;
    my $str_ref = ( ref $str ) ? $str : \$str;
    $ocode = 'utf8' if !$ocode;
    if ( !$icode || $ocode ne $icode ) {
        if ( $ocode eq 'utf8' ) {
            $$str_ref = $cgi::Jcode->set( $str_ref, $icode )->utf8;
        }
        elsif ( $ocode eq 'euc' ) {
            $$str_ref = $cgi::Jcode->set( $str_ref, $icode )->euc;
        }
        elsif ( $ocode eq 'jis' ) {
            $$str_ref = $cgi::Jcode->set( $str_ref, $icode )->jis;
        }
        elsif ( $ocode eq 'sjis' ) {
            $$str_ref = $cgi::Jcode->set( $str_ref, $icode )->sjis;
        }
        elsif ( $ocode eq 'ucs2' ) {
            $$str_ref = $cgi::Jcode->set( $str_ref, $icode )->ucs2;
        }
        else {
            Jcode::convert( $str_ref, $ocode, $icode );
        }
    }
    if ( $ocode eq 'utf8' ) {
        utf8::decode($$str_ref) if !utf8::is_utf8($$str_ref);
    }
    elsif ( utf8::is_utf8($$str_ref) ) {
        utf8::encode($$str_ref);
    }
    $$str_ref
        =~ s/\&\#([0-9]{4,}|x[a-fA-F0-9]+);/$this->_entity_to_jcode($1,$ocode)/seg
        if !$was_entity && $ocode ne 'ucs2';
    return $$str_ref;
}

sub _entity_to_jcode ($$$) {
    my ( $this, $code, $ocode ) = @_;
    utf8::encode($code) if utf8::is_utf8($code);
    $code = sprintf( "%X", $code ) unless $code =~ s/^x//;
    return $this->jcode_convert( pack( "U", hex($code) ), $ocode, 'utf8', 1 );
}

# --------------------------------------------------------------------
#	(0)-1 初期設定
sub set ($@) {
    my ( $this, @setting ) = @_;

    #文字列設定の処理
    if ( @setting == 1 ) {
        my $setting = $setting[0];
        @setting = ();
        while ( $setting
            =~ /[\x09\x0A\x0C\x0D\x20]*([0-9a-zA-Z_]+)[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]*)"/sg
            )
        {
            push( @setting, $1, $this->unescape_lite($2) );
        }
    }

    while ( my ( $k, $v ) = splice( @setting, 0, 2 ) ) {
        next if $k eq '' || !defined $v;
        $k = 'nowrap0' if $k eq 'nowrap';
        if ( $k =~ /^(width|height|trclass)$/ || $k =~ /[FH]$/ ) {
            @{ $this->{$k} } = split( /,/, $v );
        }
        elsif ( $k =~ /^([^0-9]+)([0-9]+)$/ ) {
            my ( $attribute, $num ) = ( $1, $2 );
            @{ $this->{$attribute} } = ()
                if !ref $this->{$attribute} eq 'ARRAY';
            if (   $attribute eq 'style'
                && ref $this->{$attribute}->[$num] eq 'ARRAY'
                && @{ $this->{$attribute}->[$num] } )
            {
                my ( @temp, $n );
                @temp = split( /,/, $v, -1 );
                $n = 0;
                foreach (@temp) {
                    $this->{$attribute}->[$num]->[$n] .= ';'
                        if defined $this->{$attribute}->[$num]->[$n]
                        && $this->{$attribute}->[$num]->[$n] !~ /;$/;
                    $this->{$attribute}->[$num]->[ $n++ ] .= $_;
                }
            }
            else {
                @{ $this->{$attribute}->[$num] } = split( /,/, $v, -1 );
            }
        }
        elsif ( $k eq 'thenabled' ) {
            $this->{$k} = $v;
        }
        elsif ( defined $this->{$k} && $this->{$k} =~ /[^0-9]/ ) {
            $this->{$k} .= ' ' . $v;
        }
        else {
            $this->{$k} = $v;
        }
    }

    #テーブル全体のオプション
    $this->{'option'}     = '';
    $this->{'thead_flag'} = 0;
    while ( my ( $k, $v ) = each %{$this} ) {
        if ( ref $v ) {
            $this->{'thead_flag'} = 1
                if $k =~ /H$/
                && @{ $this->{$k} } > 0
                && ( !defined $this->{'thenabled'}
                || $this->{'thenabled'} eq 'auto'
                || $this->{'thenabled'} eq 'true'
                || $this->{'thenabled'} eq 'row' );
        }
        elsif ( $k =~ s/^table// || $k eq 'cellspacing' || $k eq 'summary' ) {
            $this->{'option'}
                .= ' ' . $k . '="' . $this->escape_lite($v) . '"';
        }
    }
    $this->{'thead_flag'} = 1
        if defined $this->{'thenabled'}
        && ( $this->{'thenabled'} eq 'true'
        || $this->{'thenabled'} eq 'row' );

    #設定調整
    if ((      !defined $this->{'cellpadding'}
            || $this->{'cellpadding'} eq ''
            || $this->{'cellpadding'} > 0
        )
        && defined $this->{'border'}
        && $this->{'border'} > 0
        && defined $this->{'bordercolor'}
        && $this->{'bordercolor'} ne ''
        )
    {

        #テーブルの背面を有効にする
        if ( defined $this->{'bg'} && $this->{'bg'} ) {
            $this->{'TD'} = '<table cellspacing="0"';
            $this->{'TD'} .= ' cellpadding="' . $this->{'cellpadding'} . '"'
                if defined $this->{'cellpadding'}
                && $this->{'cellpadding'} ne '';
            $this->{'TD'} .= ' border="0"><tbody><tr><td>';
            $this->{'/TD'} = '</td></tr></tbody></table>';

            $this->{'option'} .= ' cellpadding="0"';
            $this->{'option'}
                .= ' background="' . $this->{'background'} . '"'
                if defined $this->{'background'}
                && $this->{'background'} ne '';
            $this->{'option'} .= ' border="' . $this->{'border'} . '"'
                if defined $this->{'border'} && $this->{'border'} ne '';
            $this->{'option'}
                .= ' bordercolor="' . $this->{'bordercolor'} . '"'
                if defined $this->{'bordercolor'}
                && $this->{'bordercolor'} ne '';
            $this->{'option'}
                .= ' bordercolorlight="' . $this->{'bordercolorlight'} . '"'
                if defined $this->{'bordercolorlight'}
                && $this->{'bordercolorlight'} ne '';
            $this->{'option'}
                .= ' bordercolordark="' . $this->{'bordercolordark'} . '"'
                if defined $this->{'bordercolordark'}
                && $this->{'bordercolordark'} ne '';

            $this->{'TABLE'}  = '';
            $this->{'/TABLE'} = '';

            #テーブルの背面を無視する
        }
        else {
            $this->{'TABLE'}
                = '<tbody><tr><td bgcolor="' . $this->{'bordercolor'} . '">';
            $this->{'TABLE'} .= '<table width="100%" border="0"';
            $this->{'TABLE'} .= ' style="table-layout: fixed"'
                if defined $this->{'fixed'}
                && $this->{'fixed'}
                && defined $this->{'tablewidth'}
                && $this->{'tablewidth'} > 0;
            $this->{'TABLE'}
                .= ' cellpadding="' . $this->{'cellpadding'} . '"'
                if defined $this->{'cellpadding'}
                && $this->{'cellpadding'} ne '';
            $this->{'TABLE'} .= ' background="' . $this->{'background'} . '"'
                if defined $this->{'background'}
                && $this->{'background'} ne '';
            $this->{'TABLE'} .= ' cellspacing="' . $this->{'border'} . '">';
            $this->{'/TABLE'} = '</tbody></table></td></tr>';

            @{ $this->{'bgcolor'}->[0] } = ('#ffffff')
                if !ref $this->{'bgcolor'} || !@{ $this->{'bgcolor'} };
            $this->{'option'} .= ' cellpadding="0" border="0"';

            $this->{'TD'}  = '';
            $this->{'/TD'} = '';
        }

    }
    else {
        $this->{'option'} .= ' cellpadding="' . $this->{'cellpadding'} . '"'
            if defined $this->{'cellpadding'} && $this->{'cellpadding'} ne '';
        $this->{'option'} .= ' border="' . $this->{'border'} . '"'
            if defined $this->{'border'} && $this->{'border'} ne '';
        $this->{'option'} .= ' bordercolor="' . $this->{'bordercolor'} . '"'
            if defined $this->{'bordercolor'} && $this->{'bordercolor'} ne '';
        $this->{'option'}
            .= ' bordercolorlight="' . $this->{'bordercolorlight'} . '"'
            if defined $this->{'bordercolorlight'}
            && $this->{'bordercolorlight'} ne '';
        $this->{'option'}
            .= ' bordercolordark="' . $this->{'bordercolordark'} . '"'
            if defined $this->{'bordercolordark'}
            && $this->{'bordercolordark'} ne '';
        $this->{'option'} .= ' background="' . $this->{'background'} . '"'
            if defined $this->{'background'} && $this->{'background'} ne '';

        $this->{'TD'}     = '';
        $this->{'/TD'}    = '';
        $this->{'TABLE'}  = '';
        $this->{'/TABLE'} = '';
    }
    $this->{'option'} .= ' style="table-layout: fixed"'
        if defined $this->{'fixed'}
        && defined $this->{'tablewidth'}
        && $this->{'fixed'}
        && $this->{'tablewidth'} > 0;
    return $this;
}

# --------------------------------------------------------------------
#	(0)-2 設定の取得
sub get_setting ($;$$$$$) {
    my ( $this, $table, $type, $info, $op, $setting ) = @_;
    my ($option);
    return if $info eq 'no';
    $option = '';
    if ( $type && $type =~ /^tsv$/i ) {
        $type = 'tsv';
    }
    else {
        $type = 'csv';
    }
    $type = '_' . $type if defined $op && $op;
    if ( $info ne 'tag' && $this->{'id'} eq '' ) {
        while ( my ( $k, $v ) = each %{$this} ) {
            next
                if $k eq 'option'
                || $k eq 'TD'
                || $k eq '/TD'
                || $k eq 'TABLE'
                || $k eq '/TABLE'
                || $k eq 'row'
                || $k eq 'col'
                || $k eq 'collimit'
                || $k eq '\n'
                || $k eq 'jcode'
                || $k eq 'myjcode'
                || $v eq '';
            if ( ref $v eq 'ARRAY' ) {
                if ( grep( ref $_ eq 'ARRAY', @$v ) ) {
                    for ( my $i = 0; $i < @$v; $i++ ) {
                        $option
                            .= ' '
                            . $k
                            . $i . '="'
                            . $this->escape_lite(
                            join( ',', @{ ${$v}[$i] } ) )
                            . '"';
                    }
                }
                else {
                    $option
                        .= ' '
                        . $k . '="'
                        . $this->escape_lite( join( ',', @$v ) ) . '"';
                }
            }
            else {
                $option .= ' ' . $k . '="' . $this->escape_lite($v) . '"';
            }
        }
    }
    elsif ( defined $this->{'id'} && $this->{'id'} ne '' ) {
        $option = ' id="' . $this->{'id'} . '"';
        $setting
            =~ s/[\x09\x0A\x0C\x0D\x20]*id[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"[^"]+"//sg;
    }
    $option
        .= ( ( $setting !~ /^[\x09\x0A\x0C\x0D\x20]/ ) ? ' ' : '' )
        . $setting
        if defined $setting && $setting;
    $$table
        = '<!--'
        . $type
        . $option . '-->' . "\x0A"
        . $$table . "\x0A" . '<!--/'
        . $type . '-->'
        if ref $table;
    return $option;
}

#エスケープ
sub escape_lite {
    my ( $this, $str ) = @_;
    $str
        =~ s/\&(?!(?:#[0-9]{2,6}|#x[a-fA-F0-9]+|[a-zA-Z0-9]{2,10});)/\&amp;/g;
    $str =~ s/"/&quot;/g;
    $str =~ s/</&lt;/g;
    $str =~ s/>/&gt;/g;
    return $str;
}

sub unescape_lite ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/\&quot;/"/g;
    $str =~ s/\&lt;/</g;
    $str =~ s/\&gt;/>/g;
    $str =~ s/\&amp;/&/g;
    return $str;
}

# --------------------------------------------------------------------
#	(0)-3 設定のリセット
sub reset_all ($) {
    my $this = shift;
    undef %{$this};
    return $this;
}

# --------------------------------------------------------------------
#	(0)-4 漢字コード/改行コード情報
sub set_encoding ($;$) {
    my ( $this, $text ) = @_;
    my ($agent);
    $this->{'icode'}
        = ( ref $text ) ? $this->ename_to_jname( Jcode->new($text)->icode )
        : ($text) ? $text
        :           'utf8';
    $agent = $ENV{'HTTP_USER_AGENT'};
    if ( $agent =~ /win/i ) {
        $this->{'jcode'} = 'sjis';
        $this->{'\n'}    = "\x0D\x0A";
    }
    elsif ( $agent =~ /mac/i ) {
        $this->{'jcode'} = 'sjis';
        $this->{'\n'}    = "\x0D";
    }
    else {
        $this->{'jcode'} = '';
        $this->{'\n'}    = "\x0A";
    }
    return $this;
}

sub ename_to_jname ($$) {
    my ( $this, $ename ) = @_;
    my %ename2j = (
        'shiftjis'    => 'sjis',
        'euc-jp'      => 'euc',
        '7bit-jis'    => 'jis',
        'iso-2022-jp' => 'iso_2022_jp',
        'UTF-16BE'    => 'ucs2',
    );
    if ( $ename2j{$ename} ) {
        return $ename2j{$ename};
    }
    elsif ($ename) {
        $ename =~ tr/-/_/;
        return $ename;
    }
    else {
        return 'utf8';
    }
}

######################################################################
#	(1)-0-1 obj->table;
sub table ($;@) {
    my ( $this, @option ) = @_;
    my ( $option, $k, $v, $caption, $caption_flag );
    $option = $caption = '';
    while ( ( $k, $v ) = splice( @option, 0, 2 ) ) {
        next if $k eq '';
        if ( $k eq 'caption' ) {
            $caption = '<caption>' . $v . '</caption>'
                if defined $v && $v ne '';
            $caption_flag = 1;
        }
        else {
            $option .= ' ' . $k . '="' . $this->escape_lite($v) . '"';
        }
    }
    $this->{'row'} = 0;
    $this->{'col'} = 0;
    $caption       = '<caption>' . $this->{'caption'} . '</caption>'
        if !$caption_flag
        && defined $this->{'caption'}
        && $this->{'caption'} ne '';
    return
          '<table'
        . ( ( $this->{'option'} ) ? $this->{'option'} : '' )
        . $option . '>'
        . $caption
        . ( ( $this->{'TABLE'} ) ? $this->{'TABLE'} : '' )
        . ( ( defined $this->{'thead_flag'} && $this->{'thead_flag'} )
        ? ''
        : '<tbody>' );
}

#	(1)-0-2 obj->_table;
sub _table ($) {
    my $this = shift;
    delete $this->{'row'};
    delete $this->{'col'};
    delete $this->{'rowspan'};
    return ( ( $this->{'/TABLE'} ) ? $this->{'/TABLE'} : '' )
        . '</tbody></table>';
}

# --------------------------------------------------------------------
#	(1)-2-1 obj->tr;
sub tr ($$) {
    my ( $this, $attributes_hash ) = @_;
    my ( $attributes, $k, $v, $class_flag );
    $attributes = '';
    $this->{'col'} = 0;
    if ( ref $this->{'trclass'} && @{ $this->{'trclass'} } ) {
        my $r = $this->{'row'} % @{ $this->{'trclass'} };
        if ( defined $this->{'trclass'}->[$r]
            && $this->{'trclass'}->[$r] ne '' )
        {
            $attributes_hash->{'class'} = $this->{'trclass'}->[$r]
                if !$attributes_hash->{'class'};
        }
    }
    if ( $attributes_hash->{'class+'} ) {
        $attributes_hash->{'class'}
            = ( ( $attributes_hash->{'class'} ) ? ' ' : '' )
            . $attributes_hash->{'class+'};
        delete $attributes_hash->{'class+'};
    }
    if ( ref $attributes_hash eq 'HASH' ) {
        while ( ( $k, $v ) = each %$attributes_hash ) {
            next if !defined $k || $k eq '';
            $class_flag = 1 if $k eq 'class';
            $attributes .= ' ' . $k . '="' . $this->escape_lite($v) . '"';
        }
    }
    return
          ( ( !$this->{'row'} && $this->{'thead_flag'} ) ? '<thead>' : '' )
        . '<tr'
        . $attributes . '>';
}

#	(1)-2-2 obj->_tr;
sub _tr ($$) {
    my ( $this, $nocount ) = @_;
    $this->{'row'}++ if !defined $nocount || !$nocount;
    $this->{'col'} = 0;
    return '</tr>'
        . (
        (          !$nocount
                && defined $this->{'row'}
                && $this->{'row'} == 1
                && $this->{'thead_flag'}
        ) ? '</thead><tbody>' : ''
        );
}

# --------------------------------------------------------------------
#	(1)-3-1 obj->td;
sub td ($;$$$) {
    my ( $this, $data, $option, $sv ) = @_;
    my $is_first
        = ( !$this->{'col'}
            && ( !ref $this->{'rowspan'} || !$this->{'rowspan'}{'0'} ) )
        ? 1
        : 0;
    my ( $td, $font, $fontend ) = $this->_td_setting( \$data, $option, $sv );
    if (( !$this->{'row'} && $this->{'thead_flag'} )
        || (   $is_first
            && defined $this->{'thenabled'}
            && $this->{'thenabled'} eq 'col' )
        )
    {
        return
              '<th'
            . $td . '>'
            . ( ( $this->{'TD'} ) ? $this->{'TD'} : '' )
            . $font
            . ( ( defined $data ) ? $data : '' )
            . $fontend
            . ( ( $this->{'/TD'} ) ? $this->{'/TD'} : '' ) . '</th>';
    }
    else {
        return
              '<td'
            . $td . '>'
            . ( ( $this->{'TD'} ) ? $this->{'TD'} : '' )
            . $font
            . ( ( defined $data ) ? $data : '' )
            . $fontend
            . ( ( $this->{'/TD'} ) ? $this->{'/TD'} : '' ) . '</td>';
    }
}

sub th ($;$$$) {
    my ( $this, $data, $option, $sv ) = @_;
    my ( $td, $font, $fontend ) = $this->_td_setting( \$data, $option, $sv );
    return
          '<th'
        . $td . '>'
        . ( ( $this->{'TD'} ) ? $this->{'TD'} : '' )
        . $font
        . ( ( defined $data ) ? $data : '' )
        . $fontend
        . ( ( $this->{'/TD'} ) ? $this->{'/TD'} : '' ) . '</th>';
}

sub _td_setting ($;$$) {
    my ( $this, $data, $option, $sv ) = @_;
    my ( $td, $font, $fontend, $k, $width, $height );
    my %option = ( ref $option eq 'HASH' ) ? %$option : ();
    $td = $font = $fontend = '';
    $$data = '' if !defined $$data;
    $$data = '&nbsp;' if !$this->{'tableclass'} && $$data eq '';

    if ( defined $this->{'br'} && $this->{'br'} ) {
        $$data =~ s/\x0D\x0A/<br \/>/gs;
        $$data =~ s/\x0D/<br \/>/gs;
        $$data =~ s/\x0A/<br \/>/gs;
        $$data =~ s/\x0B/<br \/>/gs;
    }

    #rowspan調整
    while ( ref $this->{'rowspan'} && $this->{'rowspan'}{ $this->{'col'} } ) {
        $this->{'rowspan'}{ $this->{'col'} }--;
        delete $this->{'rowspan'}{ $this->{'col'} }
            if !$this->{'rowspan'}{ $this->{'col'} };
        $this->{'col'}++;
    }

    #width/height
    if ( defined $option{'width'} && $option{'width'} ne '' ) {
        $width = $option{'width'};
    }
    elsif (!$this->{'row'}
        && ref $this->{'width'} eq 'ARRAY'
        && @{ $this->{'width'} } > 0 )
    {
        my $c
            = ( @{ $this->{'width'} } > 0 )
            ? $this->{'col'} % @{ $this->{'width'} }
            : 0;
        $width = $this->{'width'}->[$c]
            if defined $this->{'width'}->[$c] && $this->{'width'}->[$c] ne '';
    }
    $td .= ' width="' . $width . '"' if $width;
    if ( defined $option{'height'} && $option{'height'} ne '' ) {
        $height = $option{'height'};
    }
    elsif (!$this->{'col'}
        && ref $this->{'height'} eq 'ARRAY'
        && @{ $this->{'height'} } > 0 )
    {
        my $r
            = ( @{ $this->{'height'} } > 0 )
            ? $this->{'row'} % @{ $this->{'height'} }
            : 0;
        $height = $this->{'height'}->[$r]
            if defined $this->{'height'}->[$r]
            && $this->{'height'}->[$r] ne '';
    }
    $td .= ' height="' . $height . '"' if $height;

    $font = '';
    foreach $k (
        'nowrap',      'align',      'valign',     'bgcolor',
        'background',  'style',      'class',      'title',
        'id',          'fontcolor',  'fontsize',   'fontface',
        'char',        'charoff',    'abbr',       'axis',
        'headers',     'scope',      'lang',       'dir',
        'xml:lang',    'onclick',    'ondblclick', 'onmouseover',
        'onmousedown', 'onmouseout', 'onmouseup'
        )
    {
        my ( $attribute, $setting, $value );
        next
            unless defined $option{$k}
            || defined $this->{$k}
            || defined $this->{ $k . 'H' }
            || defined $this->{ $k . 'F' };
        if ( $k =~ /^font/ ) {
            $attribute = \$font;
        }
        else {
            $attribute = \$td;
        }
        if ( $k ne 'style' && defined $option{$k} && $option{$k} ne '' ) {
            $value = $option{$k};
        }
        elsif (!$this->{'row'}
            && ref $this->{ $k . 'H' } eq 'ARRAY'
            && @{ $this->{ $k . 'H' } } > 0 )
        {
            $setting = \@{ $this->{ $k . 'H' } };
        }
        elsif (!$this->{'col'}
            && ref $this->{ $k . 'F' } eq 'ARRAY'
            && @{ $this->{ $k . 'F' } } > 0 )
        {
            my $r = $this->{'row'} % @{ $this->{ $k . 'F' } };
            $value
                = ( defined $this->{ $k . 'F' }->[$r]
                    && $this->{ $k . 'F' }->[$r] ne '' )
                ? $this->{ $k . 'F' }->[$r]
                : '';
        }
        elsif ( ref $this->{$k} eq 'ARRAY' && @{ $this->{$k} } > 0 ) {
            my $r = $this->{'row'} % @{ $this->{$k} };
            $setting = \@{ $this->{$k}->[$r] };
        }
        else {
            next;
        }
        if ( !defined $value ) {
            my $c
                = ( ref $setting eq 'ARRAY' && @$setting > 0 )
                ? $this->{'col'} % @$setting
                : 0;
            $value = $this->escape_lite( $$setting[$c] )
                if ref $setting eq 'ARRAY'
                && defined $$setting[$c]
                && $$setting[$c] ne '';
        }
        if ( $k eq 'style' ) {
            $value .= ';' if $value && $value !~ /;/;
            $value .= 'width: ' . $width . 'px;'   if $width;
            $value .= 'height: ' . $height . 'px;' if $height;
            $value .= $option{$k} if defined $option{$k} && $option{$k} ne '';
            undef $width;
            undef $height;
        }
        elsif ( $k eq 'class' ) {
            $value .= $option{'class+'}
                if defined $option{'class+'} && $option{'class+'} ne '';
        }
        $$attribute .= ' ' . $k . '="' . $value . '"'
            if defined $value && $value ne '';
    }
    if ( $width || $height ) {
        $td
            .= ' style="'
            . ( ($width)  ? 'width:' . $width . 'px;'   : '' )
            . ( ($height) ? 'height:' . $height . 'px;' : '' ) . '"';
    }

    if ( $font ne '' ) {
        $font =~ s/font//g;
        $font    = '<font  ' . $font . '>';
        $fontend = '</font  >';
    }

    #colspan/rowspan
    if ( $sv && $$data =~ s/^([0-9]+)(x([0-9]+)|)\}//s ) {
        $option{'colspan'} = $1 + 0 if $1;
        $option{'rowspan'} = $3 + 0 if $3;
    }
    if ( defined $option{'rowspan'} && $option{'rowspan'} > 1 ) {
        $td .= ' rowspan="' . $option{'rowspan'} . '"';
        if ( defined $option{'colspan'} && $option{'colspan'} > 1 ) {
            foreach my $c ( 0 .. ( $option{'colspan'} - 1 ) ) {
                $this->{'rowspan'}{ ( $this->{'col'} + $c ) }
                    = $option{'rowspan'} - 1;
            }
        }
        else {
            $this->{'rowspan'}{ $this->{'col'} } = $option{'rowspan'} - 1;
        }
    }
    if ( defined $option{'colspan'} && $option{'colspan'} > 1 ) {
        $this->{'col'} += $option{'colspan'};
        $td .= ' colspan="' . $option{'colspan'} . '"';
    }
    else {
        $this->{'col'}++;
    }
    return ( $td, $font, $fontend );
}

# --------------------------------------------------------------------
#	(1)-3-2 obj->trtd;
sub trtd ($$;$$) {
    my ( $this, $data, $end, $option ) = @_;
    my $out = '';
    $this->{'collimit'} = 4
        if !defined $this->{'collimit'} || !$this->{'collimit'};
    return '' if ( !defined $data || $data eq '' ) && $end && !$this->{'col'};
    $out = $this->tr if !$this->{'col'};
    $out .= $this->td( $data, $option )
        unless ( !defined $data || $data eq '' ) && $end;
    if (   $this->{'row'}
        && $end
        && defined $this->{'col'}
        && $this->{'col'} < $this->{'collimit'} )
    {
        $out .= $this->td('') while $this->{'col'} < $this->{'collimit'};
    }
    $out .= $this->_tr
        if $this->{'col'}
        && ( ( $end && !$this->{'row'} )
        || $this->{'col'} >= $this->{'collimit'} );
    return $out;
}

# --------------------------------------------------------------------
#	(1)-3-3 obj->limit;
sub limit ($$) {
    my ( $this, $limit ) = @_;
    $this->{'collimit'} = $limit;
    return $this;
}

######################################################################
#	(2)-0 obj->sv_to_table;
sub sv_to_table ($$;$$$$) {
    my ( $this, $xsv, $info, $type, $object, $attributes ) = @_;
    my ( $obj, $sv, $pat, $table, $col, $count, $v, $s, $undef, $temp );
    if ( ref $xsv ) {
        $sv = $xsv;
    }
    else {
        $sv = \$xsv;
    }

    if ( !defined $object || $object eq '' ) {
        if ( defined $attributes && $attributes ne '' ) {
            $obj  = Table->new;
            %$obj = %$this;
            $obj->set($attributes);
            $info  = 'tag';
            $undef = 1;
        }
        else {
            $obj = $this;
        }
    }
    elsif ( ref $object ) {
        if ( defined $attributes && $attributes ne '' ) {
            $obj  = Table->new;
            %$obj = %$object;
            $obj->set($attributes);
            $info  = 'tag';
            $undef = 1;
        }
        else {
            $obj  = $object;
            $info = 'tag';
        }
    }
    else {
        $obj  = Table->new($object);
        $info = '';
    }

    $$sv =~ s/[\x0D\x0A]+$//;

    $type = 'tsv' if ( !defined $type || $type eq '' ) && $$sv =~ /\x09/;

    if ( $type && $type eq 'tsv' ) {
        $$sv =~ s/\x20{3}/\x09/g if $$sv !~ /\x09/;
        $pat = '([^\x09\x0D\x0A]*)(\x09|\x0D\x0A|\x0D|\x0A|\Z)';
    }
    else {
        $pat = '([^,\x0D\x0A]*)(,|\x0D\x0A|\x0D|\x0A|\Z)';
    }

    $table .= $obj->table;

    $count = $col = 0;
    while ( $$sv =~ /\G($pat)/gs ) {
        next if $1 eq '' || $obj->{'exit'};
        ( $v, $s ) = ( $2, $3 );
        if ( $type && $type ne 'tsv' ) {
            $count += ( $v =~ tr/"/"/ )
                if ( defined $v && $v =~ /^"/ )
                || ( defined $temp && $temp =~ /^"/ );
            if ( $count && $count % 2 > 0 ) {
                $temp .= $v . $s;
                next;
            }
            else {
                $v = $temp . $v if defined $temp;
                if ( $v =~ s/^"// ) {
                    $v =~ s/"$//;
                    $v =~ s/""/"/g;
                }
                undef $temp;
                $count = 0;
            }
        }
        else {
            $v =~ s/\x0B/\x0A/gs;
            $v =~ s/\&#09;/\x09/gs;
        }
        $v =~ s/\x0D\x0A/\x0A/gs;
        $v =~ s/\x0D/\x0A/gs;
        $table .= $obj->tr if $col == 0;
        $table .= $obj->td( $v, {}, 1 );
        if ( $s eq '' || $s =~ /^(\x0D\x0A|\x0D|\x0A)$/ ) {
            $table .= $obj->_tr;
            $col = 0;
        }
        else {
            $col++;
        }
    }

    if ($col) {

        #	$table .= $obj->td;
        $table .= $obj->_tr;
    }

    $table .= $obj->_table;
    eval { $obj->get_setting( \$table, $type, $info, 1, $attributes ) };

    undef %$obj if defined $undef && $undef;
    return $table;
}

# --------------------------------------------------------------------
#	(2)-1 text = obj->text_to_table(text[,info[,option[,\%obj]]]);
#		csv/tsvの分解->table
sub text_to_table ($$;$$$) {
    my ( $this, $text, $info, $option, $obj ) = @_;
    my ( $pat, $txt );
    $txt = '';
    foreach my $tag (
        'script',   'style',  'pre',    'code',  'table',   'ul',
        'ol',       'dl',     'object', 'embed', 'applet',  'select',
        'textarea', 'button', 'h[1-6]', 'area',  'bgsound', 'embed',
        'object',   'layer',  'video',  'audio', 'source',  'canvas'
        )
    {
        $text
            =~ s/(<$tag(?![0-9a-zA-Z_])[^>]*>)(.*?)(<\/$tag(?![0-9a-zA-Z_])[^>]*>)/$1 . $this->tab_to_entity($2) . $3/igse;
    }
    $text =~ s/(<!--)(.*?)(-->)/$1 . $this->tab_to_entity($2) . $3/igse;

    while ( my ( $k, $v ) = each %$obj ) {
        $v->{'id'} = $k if !defined $v->{'id'} || $v->{'id'} eq '';
    }

    if ($option) {
        $pat
            = '(?:[^\x0D\x0A\x09]+\x09[^\x0D\x0A]*(?:\x0D\x0A|\x0A|\x0D|\Z))+';
        $pat .= '|(?:[^\x0D\x0A]*,[^\x0D\x0A]*(?:\x0D\x0A|\x0A|\x0D|\Z))+'
            if $option > 1;
    }

    while ( $text
        =~ /\G(.*?)(?:<\!--([ct]sv)([^>]*)-->[\x0D\x0A ]*(.*?)[\x0D\x0A ]*<\!--\/[ct]sv-->|\Z)/gs
        )
    {
        my ( $sv1, $sv2, $xsv, $type, $object, $attributes );
        ( $sv1, $type, $attributes, $xsv )
            = ( $1, ( ($2) ? lc $2 : '' ), $3, $4 );
        $object = $obj->{$1}
            if defined $attributes
            && ref $obj
            && $attributes
            =~ /[\x09\x0A\x0C\x0D\x20]*id="([^"]+)"[\x09\x0A\x0C\x0D\x20]*/i
            && ref $obj->{$1};
        $sv2
            = ( defined $xsv && $xsv ne '' )
            ? $this->sv_to_table( $xsv, $info, $type, $object, $attributes )
            : '';
        $sv1 =~ s/([\x0D\x0A]+|^)($pat)/$1 . $this->sv_to_table($2,$info)/sge
            if $option;
        $txt .= $sv1 . $sv2;
    }
    return $this->entity_to_tab($txt);
}

#タブ→スペース
sub tab_to_entity ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/\x09/&#09;/gs;
    return $str;
}

sub entity_to_tab ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/&#09;/\x09/gs;
    return $str;
}

######################################################################
#	(3)-0-0 obj->array_to_sv(type,\@array);
#		ARRAY->csv/tsv 1ライン
sub array_to_sv ($$@) {
    my ( $this, $type, @array ) = @_;
    $this->{'\n'} = "\x0A" if !defined $this->{'\n'} || $this->{'\n'} eq '';
    map {
        $_ =~ s/\A<font  [^>]*>//s;
        $_ =~ s/<\/font[\x20]{1,}>\Z//s;
    } @array;
    if ( defined $type && $type =~ /^tsv$/i ) {
        $type = "\x09";
        map {
            $_ =~ s/\t/&#09;/gs;
            $_ =~ s/(\x0D\x0A|\x0D|\x0A|<br \/>)/\x0B/gs;
        } @array;
    }
    else {
        $type = ",";
        map {
            $_ =~ s/(\x0D\x0A|\x0D|\x0A|<br \/>)/$this->{'\n'}/gs;
            if ( $_ =~ /[,"\x0D\x0A]/ ) {
                $_ =~ s/"/""/g;
                $_ = '"' . $_ . '"';
            }
        } @array;
    }
    return $this->jcode_convert( join( $type, @array ),
        $this->{'jcode'}, $this->{'icode'} )
        . $this->{'\n'};
}

# --------------------------------------------------------------------
#	(3)-1-0 obj->table_to_sv(table[,info[,type]]);
#		TABLE->csv/tsv
sub table_to_sv ($$;$$$$) {
    my ( $this, $table, $info, $type, $tag, $option ) = @_;
    my ($sv);

    #書き出し形式の判定
    if ( defined $type && $type eq 'tsv' ) {
        $type = 'tsv';
    }
    elsif ( defined $type && $type eq 'csv' ) {
        $type = 'csv';
    }
    elsif ( defined $tag && $tag =~ /^(csv|tsv)$/i ) {
        $type = lc $1;
    }
    else {
        $type = 'csv';
    }

    #caption 抽出
    if ( $table =~ /<caption(?![0-9a-zA-Z_])[^>]*>(.*?)<\/caption>/s ) {
        $option
            .= ( ($option) ? ' ' : '' )
            . 'caption="'
            . $this->escape_lite($1) . '"';
    }

    $table
        =~ s/[\x09\x0A\x0C\x0D\x20]*(<thead(?![0-9a-zA-Z_])[^>]*>|<tbody(?![0-9a-zA-Z_])[^>]*>)[\x09\x0A\x0C\x0D\x20]*//igs;
    $table
        =~ s![\x09\x0A\x0C\x0D\x20]*(</thead>|</tbody>)[\x09\x0A\x0C\x0D\x20]*!!igs;
    $table
        =~ s/[\x09\x0A\x0C\x0D\x20]*<table(?![0-9a-zA-Z_])[^>]*>[\x09\x0A\x0C\x0D\x20]*<tr(?![0-9a-zA-Z_])[^>]*>[\x09\x0A\x0C\x0D\x20]*<td(?![0-9a-zA-Z_])[^>]*>[\x09\x0A\x0C\x0D\x20]*<table(?![0-9a-zA-Z_])[^>]*>//igs
        || $table
        =~ s/[\x09\x0A\x0C\x0D\x20]*<table(?![0-9a-zA-Z_])[^>]*>[\x09\x0A\x0C\x0D\x20]*//igs;
    $table
        =~ s![\x09\x0A\x0C\x0D\x20]*</table>[\x09\x0A\x0C\x0D\x20]*</td>[\x09\x0A\x0C\x0D\x20]*</tr>[\x09\x0A\x0C\x0D\x20]*</table>[\x09\x0A\x0C\x0D\x20]*!!igs
        || $table
        =~ s![\x09\x0A\x0C\x0D\x20]*</table>[\x09\x0A\x0C\x0D\x20]*!!igs;

    while ( $table =~ /<tr(?![0-9a-zA-Z_])[^>]*>(.*?)<\/tr>/igs ) {
        my $tr   = $1;
        my @cols = ();
        while ( $tr =~ /<(t[dh])(?![0-9a-zA-Z_])([^>]*)>(.*?)<\/\1>/igs ) {
            my $setting = $2;
            my $data    = $3;
            my $span;
            $span = $1 + 0 if $setting =~ / colspan="([0-9]+)"/i && $1 > 1;
            $span .= ( ($span) ? 'x' : '1x' ) . $1
                if $setting =~ / rowspan="([0-9]+)"/i && $1 > 1;
            push( @cols, ( ($span) ? $span . '}' : '' ) . $data );
        }
        $sv .= $this->array_to_sv( $type, @cols );
    }
    $sv =~ s/(\x0D\x0A|\x0D|\x0A)$//;

    if ( defined $option && $option ne '' ) {
        $sv
            = '<!--'
            . $type
            . $option . '-->' . "\x0A"
            . $sv . "\x0A" . '<!--/'
            . $type . '-->';
    }
    elsif ( $info ne 'no' ) {
        $this->get_setting( \$sv, $type, $info );
    }

    return $sv;
}

# --------------------------------------------------------------------
#	(3)-1 obj->text_to_sv(text[,info[,type]]);
#		TEXT->csv/tsv
sub text_to_sv ($$;$$) {
    my ( $this, $text, $info, $type ) = @_;
    foreach my $tag ( 'tsv', 'csv', 'table' ) {
        $text
            =~ s/<\!--_$tag([^>]*)-->(.*?)<\!--\/_$tag-->/$this->table_to_sv($2,$info,$type,$tag,$1)/egs;
        $text =~ s/\x0A{0,2}(<!--$tag[^>]*-->)/\x0A\x0A$1/sg;
        $text =~ s/(<!--\/$tag[^>]*-->)\x0A{0,2}/$1\x0A\x0A/sg;
    }
    if ( $type && $type =~ /^(csv|tsv)$/i ) {
        my ( $s, $e, $t, $txt, $tbl, $temp, $table );
        $s = $e = $t = 0;
        while ( $text =~ /\G(.*?)(<\/?table[^>]*>)/igs ) {
            ( $txt, $tbl ) = ( $1, $2 );
            $t++;
            if ( $tbl =~ /^<table[^>]*>$/i ) {
                $s++;
            }
            elsif ( $tbl =~ /^<\/table>$/i ) {
                $e++;
            }
            if ( $t == 1 ) {
                $temp  .= $txt;
                $table .= $tbl;
            }
            else {
                $table .= $txt . $tbl;
            }
            if ( $s <= $e ) {
                $s = 0;
                $e = 0;
                $t = 0;
                $temp .= $this->table_to_sv( $table, $info, $type );
                $table = '';
            }
        }
        $text = $temp if defined $temp && $temp;
    }
    return $text;
}

1;
