package MarkerLib;
require CMONOS::CGI_Lib::Tools::MapLib;
@ISA = qw(MapLib);
use strict;
use utf8;
######################################################################
# マーカーライブラリ編集モジュール
# MarkerLib.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2008-12-02
######################################################################
# 素材ライブラリCSSチェック
sub make_image_css ($$$;$$) {
    my ( $this, $image_dir, $dir, $is_image_lib, $setting ) = @_;
    my @files = $this->get_image_file_list( $image_dir, $dir );
    my $css = '';
    foreach my $name (@files) {
        if ( $name =~ /^([0-9a-zA-Z_]+)\.(gif|png|jpg)$/ ) {
            my $id = $1;
            $css .= "\x0A\x0A"
                . $this->mk_image_css_for_marker( $dir . '-' . $id,
                $dir, $id, $is_image_lib,
                $this->get_marker_info( $dir, $id, $is_image_lib ) );
        }
    }
    return $css;
}

sub mk_image_css_for_marker ($$$$$$$$$$) {
    my ($this,         $class_name, $dir,      $id,
        $is_image_lib, $anchor_x,   $anchor_y, $width,
        $height,       $name
    ) = @_;
    my ( $image_path, $css, $top, $left );
    $image_path = (
        ($is_image_lib) ? '.' : $this->base_change(
            $this->{'deco_obj'}->{'image_dir'} . '/' . $dir,
            $this->{'image_css'}
        )
        )
        . '/'
        . $name;
    $css = '/* ' . $class_name . ' start */' . "\x0A";
    $css .= 'div.imap-canvas div.' . $class_name . ' {' . "\x0A";
    $css .= '  width: ' . $width . 'px;' . "\x0A";
    $css .= '  height: ' . $height . 'px;' . "\x0A";
    if ( $anchor_x eq 'left' ) {
        $left = '0';
    }
    elsif ( $anchor_x eq 'right' ) {
        $left = '-' . $width . 'px';
    }
    else {
        $left = '-' . int( $width / 2 ) . 'px';
    }
    if ( $anchor_y eq 'top' ) {
        $top = '0';
    }
    elsif ( $anchor_y eq 'bottom' ) {
        $top = '-' . $height . 'px';
    }
    else {
        $top = '-' . int( $height / 2 ) . 'px';
    }
    $css
        .= '  margin: '
        . $top . ' '
        . $left . ' '
        . $top . ' '
        . $left
        . ' !important;' . "\x0A";
    $css .= '  padding: 0 !important;' . "\x0A";
    $css .= '  position: absolute;' . "\x0A";
    $css .= '  background-image: url("' . $image_path . '");' . "\x0A";
    $css .= '  cursor: move;' . "\x0A";
    $css .= '}' . "\x0A";
    $css .= 'div.imap-canvas div.' . $class_name . ' p {' . "\x0A";
    $css .= '  display: none;' . "\x0A";
    $css .= '}' . "\x0A";
    $css .= '/* ' . $class_name . ' end */' . "\x0A";
    return $css;
}

######################################################################
# 画像設定登録
sub set_image ($) {
    my $this       = shift;
    my $class_name = $main::dir . '-' . $main::id;
    $this->replace_image_css(
        $class_name,
        $this->mk_image_css_for_marker(
            $class_name,
            $main::dir,
            $main::id,
            0,
            $this->get_marker_info(
                $main::dir, $main::id,
                0,          $main::setting{'anchorX'},
                $main::setting{'anchorY'}
            )
        )
    );
}

sub get_marker_info ($$$;$$$) {
    my ( $this, $dir, $id, $is_image_lib, $anchor_x, $anchor_y ) = @_;
    my ( $width, $height, $name )
        = $this->{'deco_obj'}->image_info( $dir, $id, $is_image_lib );
    if ( !$is_image_lib && ( !$anchor_x || !$anchor_y ) ) {
        $anchor_x = 'center' if !$anchor_x;
        $anchor_y = 'bottom' if !$anchor_y;
        my $class_name = $dir . '-' . $id;
        my $image_css = $this->read_file( $this->{'image_css'}, 1 );
        if ( $image_css
            =~ m!/\* $class_name start \*/(.*?)/\* $class_name end \*/\x0A!s )
        {
            $image_css = $1;
            if ( $image_css =~ /div\.imap-canvas div\.$class_name {(.*?)}/s )
            {
                $image_css = $1;
                my ( $w, $h ) = ( $width, $height );
                $w = $1
                    if $image_css
                    =~ /[\x09\x0A\x0C\x0D\x20]+width: ([0-9]+)px/;
                $h = $1
                    if $image_css
                    =~ /[\x09\x0A\x0C\x0D\x20]+height: ([0-9]+)px/;
                if ( $image_css
                    =~ /[\x09\x0A\x0C\x0D\x20]+margin: -?([0-9]+)[px]* -?([0-9]+)[px]*/
                    )
                {
                    my ( $top, $left ) = ( $1, $2 );
                    if ( $left == 0 ) {
                        $anchor_x = 'left';
                    }
                    elsif ( $left == $w ) {
                        $anchor_x = 'right';
                    }
                    else {
                        $anchor_x = 'center';
                    }
                    if ( $top == 0 ) {
                        $anchor_y = 'top';
                    }
                    elsif ( $top == $h ) {
                        $anchor_y = 'bottom';
                    }
                    else {
                        $anchor_y = 'middle';
                    }
                }
            }
        }
    }
    return ( $anchor_x, $anchor_y, $width, $height, $name );
}

######################################################################
# 設定フォーム
sub print_setting_form ($) {
    my $this = shift;
    ( $main::setting{'anchorX'}, $main::setting{'anchorY'} )
        = $this->get_marker_info( $main::dir, $main::id )
        if !%main::setting && $main::dir && $main::id;
    print '<fieldset><legend>マーカー・ポイント</legend>';
    print $this->mk_select_menu(
        'setting:anchorX', '', '', $main::setting{'anchorX'},
        ''       => '',
        'left'   => '左',
        'center' => '中央',
        'right'  => '右'
    );

    print $this->mk_select_menu(
        'setting:anchorY', '', '', $main::setting{'anchorY'},
        ''       => '',
        'top'    => '上',
        'middle' => '中央',
        'bottom' => '下'
    );
    print
        '<div class="submit"><input type="submit" value="設定を保存" /></div>'
        if defined $main::id && $main::id ne '';
    print '</fieldset>';
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    #クラス登録
    $ImageLib::modules{'marker'} = 'MarkerLib'
        if !defined $ImageLib::modules{'marker'};
}

1;
