package UrlDecode;
use strict;
{ use bytes; }
use utf8;
use FileHandle;
STDOUT->autoflush(1);
######################################################################
# フォームから送られてきたデータをデコードするモジュール
# UrlDecode.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# オブジェクトを作る。
sub new ($) {
    my $pkg  = shift;
    my $this = {};
    bless $this;
    $this->fix_env;
    $this;
}

sub fix_env ($) {
    my $this = shift;
    $env::server_name = $ENV{'SERVER_NAME'} if !$env::server_name;
    $env::ssl
        = ( $ENV{'HTTPS'}
            || ( $ENV{'SERVER_PORT'} && $ENV{'SERVER_PORT'} == 443 ) )
        ? 1
        : 0
        if !defined $env::ssl || $env::ssl eq '';
    $env::file_mode = 0644 if !defined $env::file_mode || !$env::file_mode;
    $env::dir_mode  = 0755 if !defined $env::dir_mode  || !$env::dir_mode;
    $env::cgi_file_mode = 0755
        if !defined $env::cgi_file_mode || !$env::cgi_file_mode;
    return $this;
}

# URLエンコード
sub url_encode ($$) {
    my ( $this, $text ) = @_;
    utf8::encode($text) if utf8::is_utf8($text);
    $text
        =~ s/([\x00-\x1F\x21-\x29\x2B\x2C\x2F\x3A-\x3F\x5B-\x5E\x60\x7B-\xFF])/'%' . sprintf("%.2X", unpack("C", $1))/eg;
    $text =~ s/\x20/+/g;
    return $text;
}

# URLデコード
sub decode ($;$) {
    my ( $this, $p ) = @_;
    $p = caller if !defined $p || !$p;
    $this->url_decode( $p, $ENV{'QUERY_STRING'} );
}

# デコード
sub url_decode ($$;$) {
    my ( $this, $p, $query ) = @_;
    no strict "refs";
    $p = caller if !defined $p || !$p;
    $query =~ s/\+/ /g;
    while ( $query =~ /\G([^=\&]*)=([^\&]*)[\&]?/sgo ) {
        my ( $k, $v ) = ( $1, $2 );
        next if $k =~ /[^0-9a-zA-Z_]/ || $k =~ /^[A-Z_]+$/;
        utf8::encode($k) if utf8::is_utf8($k);
        utf8::encode($v) if utf8::is_utf8($v);
        $k =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
        $v =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
        ${ $p . '::' . $k } = $v;
    }
}

# クッキーデータの取得
sub read_cookie_direct ($$) {
    my ( $this, $cname ) = @_;
    return '' if !defined $ENV{'HTTP_COOKIE'};
    foreach ( split( /;[\x09\x0A\x0C\x0D\x20]*/, $ENV{'HTTP_COOKIE'} ) ) {
        my ( $k, $v ) = split( /=/, $_, 2 );
        return $v if $k eq $cname;
    }
}

#IPアドレス->HOSTネーム
sub to_host ($$) {
    my ( $this, $addr ) = @_;
    my ($host_name);
    return $addr if $addr =~ /[^0-9\.]/;
    $host_name = gethostbyaddr( pack( "C4", split( /\./, $addr ) ), 2 );
    return $addr if !defined $host_name || $host_name eq '';
    return $host_name;
}

#ホームディレクトリまでのサーバ内パス･URLパスを求める
sub home_location ($) {
    my ( $mypath, $d, @dir, $document_root );
    my $this = shift;
    if ( $ENV{'SCRIPT_FILENAME'} ) {
        $document_root = $ENV{'SCRIPT_FILENAME'};
        $document_root =~ s!/[^/]*$!!;
    }
    else {
        chdir;
        use Cwd;
        $document_root = cwd() || getcwd;
    }
    $mypath = $ENV{'SCRIPT_NAME'};
    $mypath =~ s!/[^/]*$!!;
    @dir = split( '/', $mypath );
    while ( $d = pop(@dir) ) {
        last unless $document_root =~ s!/$d$!!;
    }
    push( @dir, $d );
    return
          ( ($env::ssl) ? 'https://' : 'http://' )
        . $env::server_name
        . ( ( $ENV{'SERVER_PORT'} != 80 && $ENV{'SERVER_PORT'} != 443 )
        ? ':' . $ENV{'SERVER_PORT'}
        : '' )
        . join( '/', @dir );
}

# 日付ID (dateID,hourID,wday) = obj->date_id; ex) 20010725,130105,3
sub date_id ($;$) {
    my ( $this, $time ) = @_;
    $time = time if !defined $time || $time eq '';
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday ) = localtime($time);
    $year += 1900;
    $mon++;
    return (
        $year . sprintf( "%.2d", $mon ) . sprintf( "%.2d", $mday ),
        sprintf( "%.2d", $hour )
            . sprintf( "%.2d", $min )
            . sprintf( "%.2d", $sec ),
        $wday
    );
}

#ジャンプ
sub location ($$;$) {
    my ( $this, $url, $cookie ) = @_;
    binmode STDOUT, ":utf8";
    if ( $env::location eq 'no' ) {
        print 'Content-Type: text/html', "\x0A";
        print $cookie, "\x0A" if defined $cookie && $cookie;
        print "\x0A";
        print '<html><head><meta http-equiv="Refresh" content="0; URL=',
            $url, '" /></head><body></body></html>';
    }
    else {
        # IISサーバ(PerlIS)対応
        if ( $ENV{'PERLXS'} eq "PerlIS" ) {
            print "HTTP/1.0 302 Temporary Redirection\r\n";
            print "Content-type: text/html\n";
        }
        print 'Location: ', $url, "\x0A";
        print $cookie, "\x0A" if defined $cookie && $cookie;
        print "\x0A";
    }
    binmode STDOUT;
    exit;
}

# 透過GIF
sub invisible_gif ($$) {
    my ( $this, $cookie ) = @_;
    binmode STDOUT;
    print 'Content-type: image/gif', "\x0A";
    print 'Content-length: 43',      "\x0A";
    print 'Cache-Control: no-cache', "\x0A";
    print 'Pragma: no-cache',        "\x0A";
    print $cookie, "\x0A" if defined $cookie && $cookie;
    print "\x0A";
    print
        "GIF89a\x01\x00\x01\x00\x80\x00\x00\xFF\xFF\xFF\x00\x00\x00\x21\xF9\x04\x05\x14\x00\x00\x00\x2C\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00\x3B";
    exit;
}

# Join
sub csv ($$@) {
    my ( $this, $option, @array ) = @_;
    map {
        $_ = '' if !defined $_;
        $_ =~ s/[\x0D\x0A]/ /gs if defined $option && $option;
        if ( $_ =~ /[,"\x09\x0A\x0C\x0D\x20]/ ) {
            $_ =~ s/"/""/g;
            $_ = '"' . $_ . '"';
        }
    } @array;
    return join( ',', @array );
}

sub encode_utf8 ($$) {
    my ( $this, $str ) = @_;
    utf8::encode($str) if defined $str && utf8::is_utf8($str);
    return $str;
}

sub decode_utf8 ($$) {
    my ( $this, $str ) = @_;
    utf8::decode($str) if defined $str && !utf8::is_utf8($str);
    return $str;
}

######################################################################
#sub import {
BEGIN {
}

1;
