package CalList;
require CMONOS::Calendar::CalList;
@ISA = qw(CalCommon);
use strict;
use utf8;
######################################################################
# カレンダーイベント検索フォーム表示モジュール
# CalSearch.pm ver1.0.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-05-26
######################################################################
# オブジェクト作成
sub new ($) {
    my $pkg  = shift;
    my $this = {};
    bless $this, $pkg;
    return $this;
}

######################################################################
#イベントリスト
sub print_events ($) {
    my $this = shift;
    if ( $field::for{'category'} ) {
        $this->import_db( $default::preference_dir . '/event_category',
            \%setting::event_category, 1 )
            unless %setting::event_category
            && ref $setting::event_category{'ORDER'}
            && %{ $setting::event_category{'ORDER'} };
    }
    my $title_id = $this->get_title_id;
    my $template = $setting::page{$setting::key}{'template'};
    $template
        =~ s/<\!--monthname(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/monthname(?![0-9a-zA-Z_])[^>]*-->/$this->monthname($1,$2)/iges;
    $template
        =~ s/<\!--yearname(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/yearname(?![0-9a-zA-Z_])[^>]*-->/$this->yearname($1,$2)/iges;
    $template
        =~ s/<\!--monthlist(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/monthlist(?![0-9a-zA-Z_])[^>]*-->/$this->monthlist($1,$2)/iges;
    my $event_hash = $this->get_target_events_hash(
        [   (          $setting::page{$setting::key}{'listUnit'}
                    && $setting::page{$setting::key}{'listUnit'} eq 'monthly'
            )
            ? $this->get_target_month_dates($main::date)
            : $this->get_target_year_dates($main::date)
        ],
        $setting::page{$setting::key}{'listTarget'},
        $this->{'boundary'}
    );
    my $prefix
        = ( $setting::page{$setting::key}{'listUnit'} || 'yearly' ) . '-';
    require CMONOS::CGI_Lib::Template;
    my $template_obj = Template->new;
    $template_obj->set(
        'field_style_disabled' => $setting::field_style_disabled,
        'field_tag_enabled'    => $setting::field_tag_enabled
    );
    $template = $template_obj->deco($template);
    $template = '<!--list-->' . $template . '<!--/list-->'
        if $template !~ /<!--list(?!\w)[^<>]*?-->/is;

    while ( $template
        =~ /\G(.*?)(<!--list(?!\w)([^<>]*?)-->(.*?)<!--\/list(?!\w)[^<>]*?-->|\Z)/isg
        )
    {
        my ( $html, $attributes, $content ) = ( $1, $3, $4 );
        print $this->event_info_replace(
            $this->page_item_replace(
                $html, 0, '', $setting::page{$setting::key}{'title'}
            )
            ),
            my ( $sort, $header, $footer );
        ( $header, $footer, $attributes, $content )
            = $this->get_header_and_footer( $attributes, $content );
        $sort
            = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+sort="([^"]+)"/i )
            ? $this->unescape($1)
            : ( $setting::page{$setting::key}{'listSort'} || 'ascend' );
        my @events = $this->sort_target_events( $event_hash, $sort );
        if (@events) {
            print $header;
            $template_obj->template_parser( \$content );
            foreach my $n (@events) {
                print $this->get_event_data( $template_obj, $content, $n,
                    $title_id, $prefix )
                    if $n !~ /\D/;
            }
            print $footer;
        }
    }
    $this->untie_all_dbs;
}

sub prev_calendar_date ($$) {
    my ( $this, $date ) = @_;
    if ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        my ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
        $y--;
        $d = $this->max_mday( $y, $m ) if $d > $this->max_mday( $y, $m );
        return $y . sprintf( "%.2d", $m ) . sprintf( "%.2d", $d );
    }
    else {
        return $date;
    }
}

sub next_calendar_date ($$) {
    my ( $this, $date ) = @_;
    if ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        my ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
        $y++;
        $d = $this->max_mday( $y, $m ) if $d > $this->max_mday( $y, $m );
        return $y . sprintf( "%.2d", $m ) . sprintf( "%.2d", $d );
    }
    else {
        return $date;
    }
}

sub page_title ($$) {
    my ( $this, $date ) = @_;
    if ( $setting::page{$setting::key}{'listTarget'} ) {
        my ( $ns, $nmin, $nh, $nd, $nm, $ny ) = localtime(time);
        $ny += 1900;
        $nm++;
        $nd = $this->max_mday( $ny, $nm )
            if $nd > $this->max_mday( $ny, $nm );
        $this->{'boundary'}
            = $ny
            . sprintf( "%.2d", $nm )
            . sprintf( "%.2d", $nd )
            . sprintf( "%.2d", $nh )
            . sprintf( "%.2d", $nmin )
            . sprintf( "%.2d", $ns );
        my $boundary_date
            = $ny . sprintf( "%.2d", $nm ) . sprintf( "%.2d", $nd );
        $main::date = $date = $boundary_date
            if ( $setting::page{$setting::key}{'listTarget'} eq 'past'
            && $boundary_date < $main::date )
            || ( $setting::page{$setting::key}{'listTarget'} eq 'future'
            && $boundary_date > $main::date );
    }
    if ( $date =~ /^([0-9]{4})([0-9]{2})/ ) {
        my ( $y, $m ) = ( $1, $2 + 0 );
        if (   $setting::page{$setting::key}{'listUnit'}
            && $setting::page{$setting::key}{'listUnit'} eq 'monthly' )
        {
            return $y . '年' . $m . '月';
        }
        elsif ($setting::page{$setting::key}{'startMonth'}
            && $setting::page{$setting::key}{'startMonth'} > 1 )
        {
            return ( ( $setting::page{$setting::key}{'startMonth'} > $m )
                ? $y - 1
                : $y )
                . '年度';
        }
        else {
            return $y . '年';
        }
    }
    return '';
}

sub calendar_type ($) {
    my $this = shift;
    return 'list';
}

sub have_timetable ($) {
    my $this = shift;
    return 0;
}

sub get_target_year_dates ($$) {
    my ( $this, $date ) = @_;
    if ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        my ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
        $setting::page{$setting::key}{'startMonth'} = 1
            if !$setting::page{$setting::key}{'startMonth'};
        $y-- if $setting::page{$setting::key}{'startMonth'} > $m;
        $m = $setting::page{$setting::key}{'startMonth'};
        $d = 1;
        my @dates = ();
        foreach my $c ( 0 .. 11 ) {
            my $last = $this->get_last_mday( $y, $m );
            for ( my $i = 1; $i <= $last; $i++ ) {
                push( @dates,
                    $y . sprintf( "%.2d", $m ) . sprintf( "%.2d", $i ) );
            }
            $m++;
            if ( $m == 13 ) {
                $m = 1;
                $y++;
            }
        }
        return @dates;
    }
    else {
        return ();
    }
}

1;
