package EventCategory;
require CMONOS::CGI_Lib::Field;
@ISA = qw(Field);
use strict;
use utf8;
######################################################################
# フィールドモジュール（イベントカテゴリー）
# EventCategory.pm 1.0.4
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-01-27
######################################################################
sub _set ($$) {
    my ( $this, $setting ) = @_;

    #イベントカテゴリー情報
    $this->import_db( $default::preference_dir . '/event_category',
        \%setting::event_category, 1 )
        unless %setting::event_category
        && ref $setting::event_category{'ORDER'}
        && %{ $setting::event_category{'ORDER'} };
    $this->{'disabled'} = 1
        if !%setting::event_category
        || !ref $setting::event_category{'ORDER'}
        || !%{ $setting::event_category{'ORDER'} };
    $this->{'field_for'}{'category'} = $this->{'id'};
    return;
}

######################################################################
#	(0)-3 フィールドDB
sub db ($) {
    my $this = shift;
    return 'category';
}

######################################################################
#	(1)-0 obj->match_enabled;
sub match_enabled ($) {
    my $this = shift;
    return 0 if $this->{'disabled'};
    return (
        (   defined $main::search{ $this->{'id'} }{'str'}
                && $main::search{ $this->{'id'} }{'str'} ne ''
        )
            || ( ref $main::search{ $this->{'id'} }{'str'}
            && @{ $main::search{ $this->{'id'} }{'str'} } )
    ) ? 1 : 0;
}

#	(1) obj->match(str);
sub match ($;$) {
    my ( $this, $str ) = @_;
    return 1 if !$this->match_enabled;
    $this->scalar_to_array( \$main::search{ $this->{'id'} }{'str'} )
        if !ref $main::search{ $this->{'id'} }{'str'};
    foreach my $id ( @{ $main::search{ $this->{'id'} }{'str'} } ) {
        if ( defined $id && $id ne '' ) {
            if ( $id eq 'nodata' ) {
                return 1
                    if !defined $$str
                    || $$str eq ''
                    || !ref $setting::event_category{$$str};
            }
            elsif ( defined $$str && $$str eq $id ) {
                return 1;
            }
        }
    }
    return 0;
}

######################################################################
#	(2) obj->input_enabled;
sub input_enabled ($) {
    my $this = shift;
    return ( $this->{'disabled'} ) ? 0 : 1;
}

sub form {
    my ( $this, $status, $original ) = @_;
    return ''
        if $this->{'disabled'}
        || ( $this->{'required'}
        && values %{ $setting::event_category{'ORDER'} } == 1 );
    my @data = ( $main::data{ $this->{'id'} }{'str'} );
    my $form = $this->field_header( '', 'data:' . $this->{'id'} . ':str' );
    $form .= $this->event_category_menu( 'data:' . $this->{'id'} . ':str',
        0, \@data, $this->{'is_invalid_field'}{'str'} );
    $form .= $this->field_footer;
    return $form;
}

sub event_category_menu ($$$$;$) {
    my ( $this, $name, $is_multiple, $selected, $is_invalid_field ) = @_;
    my ($form);
    if ( $this->{'radiobutton'} ) {
        $form .= '<ol class="checkbox">';
    }
    else {
        $form
            .= '<select name="'
            . $name
            . '" id="'
            . $name . '"'
            . ( ($is_multiple)      ? ' multiple="multiple"'   : '' )
            . ( ($is_invalid_field) ? $this->set_invalid_class : '' ) . '>';
        $form .= '<option value=""></option>' if !$is_multiple;
    }
    foreach my $c ( sort { $a <=> $b }
        keys %{ $setting::event_category{'ORDER'} } )
    {
        my $id = $setting::event_category{'ORDER'}{$c};
        if (   !defined $id
            || !ref $setting::event_category{$id}
            || !$setting::event_category{$id}{'closed'}
            || $this->is_secure( 'setClosedEvent', 0, 'closed' ) )
        {
            my $is_selected
                = (    ref $selected eq 'ARRAY'
                    && @$selected
                    && grep( defined $_ && $_ ne '' && $id eq $_, @$selected )
                ) ? 1 : 0;
            if ( $this->{'radiobutton'} ) {
                $form
                    .= '<li><label><input type="'
                    . ( ($is_multiple) ? 'checkbox' : 'radio' )
                    . '" name="'
                    . $name
                    . '" value="'
                    . $id . '"'
                    . ( ($is_selected)      ? ' checked="checked"'     : '' )
                    . ( ($is_invalid_field) ? $this->set_invalid_class : '' )
                    . ' /> '
                    . $setting::event_category{$id}{'name'}
                    . '</label></li>';
            }
            else {
                $form
                    .= '<option value="'
                    . $id . '"'
                    . ( ($is_selected) ? ' selected="selected"' : '' ) . '>'
                    . $setting::event_category{$id}{'name'}
                    . '</option>';
            }
        }
    }
    if ( $is_multiple
        && ( $this->{'nodataOption'} || $this->{'searchOption'} ) )
    {
        my $id = 'nodata';
        my $is_selected
            = (    ref $selected eq 'ARRAY'
                && @$selected
                && grep( defined $_ && $_ ne '' && $id eq $_, @$selected ) )
            ? 1
            : 0;
        if ( $this->{'radiobutton'} ) {
            $form
                .= '<li><label><input type="'
                . ( ($is_multiple) ? 'checkbox' : 'radio' )
                . '" name="'
                . $name
                . '" value="'
                . $id . '"'
                . ( ($is_selected)      ? ' checked="checked"'     : '' )
                . ( ($is_invalid_field) ? $this->set_invalid_class : '' )
                . ' /> 〈未選択〉</label></li>';
        }
        else {
            $form
                .= '<option value="'
                . $id . '"'
                . ( ($is_selected) ? ' selected="selected"' : '' )
                . '>〈未選択〉</option>';
        }
    }
    if ( $this->{'radiobutton'} ) {
        $form .= '</ol>';
    }
    else {
        $form .= '</select>';
    }
    return $form;
}

######################################################################
#	(3) obj->is_valid_data;
sub is_valid_data ($;$) {
    my $this = shift;
    delete $this->{'is_invalid_field'} if ref $this->{'is_invalid_field'};
    if ( $this->{'disabled'} ) {
        delete $main::data{ $this->{'id'} }{'str'};
        return 1;
    }
    if ( $this->{'required'} ) {
        my @categories = values %{ $setting::event_category{'ORDER'} };
        $main::data{ $this->{'id'} }{'str'} = $categories[0]
            if @categories == 1;
    }
    my $id = $main::data{ $this->{'id'} }{'str'};
    if ( !defined $id || !$id ) {
        if ( $this->{'required'} ) {
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : 'カテゴリー' )
                    . 'を選択してください'
            ) if ref $cgi::action;
            $main::data{ $this->{'id'} }{'str'} = '';
            $this->{'is_invalid_field'}{'str'} = 1;
            return 0;
        }
    }
    elsif ( !defined $setting::event_category{$id}{'name'}
        || $setting::event_category{$id}{'name'} eq '' )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '指定された'
                . (
                ( $this->{'name'} ) ? $this->{'name'} : 'カテゴリー' )
                . 'は存在しません。'
        ) if ref $cgi::action;
        $main::data{ $this->{'id'} }{'str'} = '';
        $this->{'is_invalid_field'}{'str'} = 1;
        return 0;
    }
    elsif ( $setting::event_category{$id}{'closed'}
        && !$this->is_secure( 'setClosedEvent', 0, 'closed' ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '指定された'
                . (
                ( $this->{'name'} ) ? $this->{'name'} : 'カテゴリー' )
                . 'がみつかりませんでした。'
        ) if ref $cgi::action;
        $main::data{ $this->{'id'} }{'str'} = '';
        $this->{'is_invalid_field'}{'str'} = 1;
        return 0;
    }
    return 1;
}

######################################################################
#	(4) obj->usual_format(str);
sub usual_format_enabled ($) {
    my $this = shift;
    return ( $this->{'disabled'} ) ? 0 : 1;
}

sub usual_format {
    my ( $this, $str, $path, $c, $n ) = @_;
    return '' if defined $this->{'disabled'} && $this->{'disabled'};
    $str = $$str{'str'} if ref $str eq 'HASH';
    return ''
        if !defined $str || !$str || !$setting::event_category{$str}{'name'};
    return $this->field_tag(
        $this->style(
            $this->escape( $setting::event_category{$str}{'name'} ),
            $this->{'style'},     $this->{'fontsize'},
            $this->{'fontcolor'}, $this->{'fontface'}
        )
    );
}

sub has_content {
    my ( $this, $str ) = @_;
    return 0 if defined $this->{'disabled'} && $this->{'disabled'};
    $str = $$str{'str'} if ref $str eq 'HASH';
    return ( defined $str && $str && $setting::event_category{$str}{'name'} )
        ? 1
        : 0;
}

#---------------------------------------------------------------------
#	(4) obj->format_for_mail(str);
sub format_for_mail_enabled ($) {
    my $this = shift;
    return ( $this->{'disabled'} ) ? 0 : 1;
}

sub format_for_mail {
    my ( $this, $str, $path, $c, $n ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return ''
        if !defined $str || !$str || !$setting::event_category{$str}{'name'};
    return
          '['
        . $this->{'name'} . "]\x0A"
        . $setting::event_category{$str}{'name'}
        . "\x0A\x0A";
}

#	(4)-3 obj->format_for_text(str);
sub format_for_text_enabled ($) {
    my $this = shift;
    return ( $this->{'disabled'} ) ? 0 : 1;
}

sub format_for_text {
    my ( $this, $str, $path, $c, $n ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return ''
        if !defined $str || !$str || !$setting::event_category{$str}{'name'};
    return $setting::event_category{$str}{'name'} . "\x0A\x0A";
}

sub format_for_detail_enabled ($) {
    my $this = shift;
    return ( $this->{'disabled'} ) ? 0 : 1;
}

#---------------------------------------------------------------------
#	(4)-5 obj->confirmation(str);
sub confirmation_enabled ($) {
    my $this = shift;
    return ( $this->{'disabled'} ) ? 1 : 0;
}

sub confirmation ($$) {
    my ( $this, $str ) = @_;
    return '';
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my ( $this, $html_disabled ) = @_;

    #名前
    my $name = 'data:' . $this->{'id'};
    $this->print_field_info_form(
        $name,
        $this->{'disabled'},
        (   ( $this->{'disabled'} )
            ? '使用不可：イベントカテゴリー設定がみつかりません。'
            : ''
        )
    );

    print '<fieldset>';
    print
        '<legend>カテゴリー選択肢の表示</legend><ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_radiobutton" id="'
        . $name
        . '_radiobutton" value="1"'
        . ( ( defined $this->{'radiobutton'} && $this->{'radiobutton'} )
        ? ' checked="checked"'
        : '' )
        . ' /> ラジオボタンで表示する。</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_nodataOption" id="'
        . $name
        . '_nodataOption" value="1"'
        . ( ( defined $this->{'nodataOption'} && $this->{'nodataOption'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 検索時の選択肢に〈未選択〉を追加する。</label></li>';
    print '</ol></fieldset>';

    $this->print_font_setting_form($name);
    $this->print_setting_save_button;
    return;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this = shift;
    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };
    $main::data{ $this->{'id'} . '_radiobutton' } = ''
        if !$main::data{ $this->{'id'} . '_radiobutton' };
    $main::data{ $this->{'id'} . '_nodataOption' } = ''
        if !$main::data{ $this->{'id'} . '_nodataOption' };
    return $this->is_valid_font_setting;
}

######################################################################
#	(7) obj->search_form;
sub search_form ($) {
    my $this = shift;
    return '' if defined $this->{'disabled'} && $this->{'disabled'};
    $this->scalar_to_array( \$main::search{ $this->{'id'} }{'str'} );
    my $form
        = '<tr><td class="field-name"><label for="search:'
        . $this->{'id'}
        . ':str">'
        . $this->{'name'}
        . '</label></td><td class="field">';
    $form .= $this->event_category_menu( 'search:' . $this->{'id'} . ':str',
        1, $main::search{ $this->{'id'} }{'str'} );
    $form .= '</td></tr>';
    return $form;
}

######################################################################
#sub import {
BEGIN {
    Field::class_register( 'xcal_category', 'EventCategory',
        'イベントカテゴリー' );
}

1;
