package ShoppingCart;
require CMONOS::Calendar::Field::CalUserGeneratedField;
require CMONOS::CGI_Lib::Field::ShoppingCartForDB;
@ISA = qw(CalUserGeneratedField ShoppingCartForDB);
use strict;
use utf8;
######################################################################
# フィールド定義モジュール（ショッピングカート）
# ShoppingCart.pm 1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
sub set_default_value ($$) {
    my ( $this, $data ) = @_;
    $data->{'canceltimelimit'}     = $this->{'defaultcanceltimelimit'};
    $data->{'canceltimelimitunit'} = $this->{'defaultcanceltimelimitunit'};
}

sub print_period_setting_form ($) {
    my ( $this, $name ) = @_;
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">イベント連動</span></td>';
    print '<td class="field"><label>イベント開始の ';
    print '<input type="number" size="3" maxlength="3" name="'
        . $name
        . '_periodEnd" value="'
        . $this->escape( $this->{'periodEnd'} ) . '" />';
    print $this->mk_select_menu(
        '' . $name . '_periodEndUnit', '', '',
        ( $this->{'periodEndUnit'} || 'day' ),
        'min'  => '分前',
        'hour' => '時間前',
        'day'  => '日前',
        'week' => '週間前',
        'year' => '年前',
    );
    print ' まで</label>';
    print ' (<label><input type="checkbox" name="'
        . $name
        . '_noPeriodForDC" value="1"'
        . (
        (   defined $this->{'noPeriodForDC'}
                && $this->{'digitalContentMenuEnabled'}
        ) ? ' checked="checked"' : ''
        ) . ' /> デジタルコンテンツを除く</label>)';
    print '</td>';
    print '</tr>';
}

sub cancel_deadline_field ($$;$$$) {
    my ( $this, $name, $type, $data, $invalid_check ) = @_;
    my $form
        = '<br /><label><input type="number" size="3" maxlength="3" name="'
        . $name
        . 'canceltimelimit" id="'
        . $name
        . 'canceltimelimit" value="'
        . $this->escape( $data->{ $type . 'canceltimelimit' } ) . '"'
        . (
        (          $invalid_check
                && $this->{'is_invalid_field'}{ $type . 'canceltimelimit' }
        ) ? $this->set_invalid_class : ''
        ) . ' /></label><label>';
    $form .= $this->mk_select_menu(
        '' . $name . 'canceltimelimitunit', '', '',
        ( $data->{ $type . 'canceltimelimitunit' } || '86400' ),
        '60'     => '分前',
        '3600'   => '時間前',
        '86400'  => '日前',
        '604800' => '週間前',
    );
    $form .= ' まで</label>';
    return $form;
}

sub _is_valid_data ($) {
    my $this     = shift;
    my $is_valid = 1;
    if (   defined $main::data{ $this->{'id'} }{'canceltimelimit'}
        && $main::data{ $this->{'id'} }{'canceltimelimit'} ne ''
        && $this->is_nan( $main::data{ $this->{'id'} }{'canceltimelimit'} ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'キャンセル期限を確認してください。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} }{'canceltimelimit'};
        $this->{'is_invalid_field'}{'canceltimelimit'} = 1;
        $is_valid = 0;
    }
    return $is_valid;
}

sub _sc_period_is_ok ($$$$) {
    my ( $this, $data, $c, $n ) = @_;
    return 1
        if !defined $this->{'periodEnd'}
        || $this->{'periodEnd'} eq ''
        || ( $this->{'noPeriodForDC'} && $this->is_digital_content($data) );
    use Time::Local;
    my %db;
    my $start;
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    tie( %db, $env::dbm_file, $default::db . '/dt', O_RDONLY,
        $env::dbm_mode );

    if (   ref tied %db
        && $db{ $this->encode_utf8($n) }
        && $this->decode_utf8( $db{ $this->encode_utf8($n) } )
        =~ /^([0-9]{0,4})([0-9]{0,2})([0-9]{0,2})T?([0-9]{0,2})([0-9]{0,2})([0-9]{0,2})/
        )
    {
        $start = ($1)
            ? timelocal(
            ( $6 || 0 ),
            ( $5 || 0 ),
            ( $4 || 0 ),
            ( $3 || 1 ),
            ( ($2) ? $2 - 1 : 0 ), $1
            )
            : 0;
    }
    untie %db;
    return 0 if !$start;
    $this->{'periodEndUnit'} = 'day' if !$this->{'periodEndUnit'};
    my $gap
        = $this->{'periodEnd'}
        * ( ( $this->{'periodEndUnit'} eq 'min' ) ? 60
        : ( $this->{'periodEndUnit'} eq 'hour' ) ? 3600
        : ( $this->{'periodEndUnit'} eq 'day' )  ? 86400
        : ( $this->{'periodEndUnit'} eq 'week' ) ? 604800
        : ( $this->{'periodEndUnit'} eq 'year' ) ? 31536000
        :                                          86400 );
    return ( time <= $start - $gap ) ? 1 : 0;
}

sub is_valid_sub_setting ($) {
    my $this     = shift;
    my $is_valid = 1;
    if (   defined $main::data{ $this->{'id'} . '_periodEnd' }
        && $main::data{ $this->{'id'} . '_periodEnd' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_periodEnd' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '販売期間のイベント連動設定を確認してください。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_periodEnd' };
        $is_valid = 0;
    }
    if (   defined $main::data{ $this->{'id'} . '_defaultcanceltimelimit' }
        && $main::data{ $this->{'id'} . '_defaultcanceltimelimit' } ne ''
        && $this->is_nan(
            $main::data{ $this->{'id'} . '_defaultcanceltimelimit' }
        )
        )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'デフォルトのキャンセル期限を確認してください。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_defaultcanceltimelimit' };
        $is_valid = 0;
    }
    return $is_valid;
}

######################################################################
#sub import {
BEGIN {
    no warnings qw(once);
    if ( $env::checkout && -e $env::checkout ) {
        Field::class_register( 'ysc_cart', 'ShoppingCart',
            'ショッピングカート' );
    }
}

1;
