package GeneralSetting;
require CMONOS::CGI_Lib::Tools::General;
@ISA = qw(General);
use strict;
use utf8;
######################################################################
# 一般設定モジュール
# GeneralSetting.pm ver1.0.5
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#  最終更新日：2014-01-17
######################################################################
# 設定チェック項目取得
sub get_sub_setting_items ($) {
    my $this = shift;

    #チェックボックス
    push( @check::checkbox,
        'skipConfirmation', 'siteRSSEnabled', 'RSSEnabled',
        'freqDisabled',     'dateDisabled',   'endDateDisabled',
        'pendingTime' );
    push( @check::checkbox, 'masterRSSEnabled' ) if $default::subdivided;

    #タグを除去する
    push( @check::notag, 'description' );

    #数字チェック
    $check::num{'boundsStart'} = '日付と時刻の範囲(開始)';
    $check::num{'boundsEnd'}   = '日付と時刻の範囲(終了)';

    #アイコン画像
    $check::imagefile{'og'} = 'og:image';
    $check::ogimage{'og'}   = 1;
    if ($default::subdivided) {
        if ( $master::general{'subicon'} ) {
            $check::imagefile{'icon'}
                = 'カテゴリーアイコン';    #画像の名前
            $check::maxsize{'icon'}
                = ( $master::general{'iconMaxSize'} )
                ? $master::general{'iconMaxSize'}
                : 30000;    #ファイルサイズ制限
            $check::widthlimit{'icon'} = $master::general{'iconWidthLimit'}
                ;           #ファイル横幅制限
            $check::heightlimit{'icon'} = $master::general{'iconHeightLimit'}
                ;           #ファイル高さ制限
        }
        push( @check::checkbox, 'p' );
    }
    return;
}

######################################################################
# 設定環境2
sub is_database_cgi ($) {
    my $this = shift;
    return 1;
}

sub sub_set_preferences ($) {
    my $this = shift;
    $cgi::enctype_multipart = 1;

    # カテゴリーがない場合
    if ( !$default::subdivided ) {
        $default::file = '.'
            if !defined $default::file || $default::file eq '';
        $preference::filedir{'og'}
            = $default::file;    #画像保存ディレクトリ

        # カテゴリーがある場合
    }
    elsif ( $default::subdivided && defined $main::c && $main::c ne '' ) {
        if ( defined $master::general{'subicon'}
            && $master::general{'subicon'} )
        {
            $default::file = '.'
                if !defined $default::file || $default::file eq '';
            $preference::filedir{'icon'}
                = $default::file;    #画像保存ディレクトリ
        }
        $preference::filedir{'og'}
            = $default::file;        #画像保存ディレクトリ
    }
}

######################################################################
# サブ設定パネル
sub print_sub_preference_panel ($$) {
    my ( $this, $master ) = @_;

    # カテゴリーがある場合
    if ($default::subdivided) {
        print '<fieldset><legend>カテゴリー設定</legend>';
        if ( $master::general{'subicon'} ) {
            print '<fieldset><legend>カテゴリーアイコン</legend>';
            $this->print_image_upload_form( 'icon', 'icon', \%main::data );
            print '</fieldset>';
        }
        print '<fieldset><legend>og:image</legend>';
        $this->print_image_upload_form( 'og', 'og', \%main::data );
        print '</fieldset>';
        print
            '<fieldset><legend><label for="data:description">説明</label></legend>';
        print
            '<textarea name="data:description" id="data:description" cols="56" rows="6" wrap="soft" title="インデックスに表示するカレンダーの説明を設定します。">'
            . ( ( defined $main::data{'description'} )
            ? $main::data{'description'}
            : '' )
            . '</textarea>';
        print
            '<div class="extra-field"><label><input type="checkbox" name="data:p" value="1"'
            . ( ( $main::data{'p'} ) ? 'checked="checked"' : '' )
            . ' /> 改行の連続を &lt;p&gt; に変換する</label></div>';
        print
            '<div class="note">インデックスに表示するカレンダーの説明を設定します。</div></fieldset>';
        print '</fieldset>';
    }
    else {
        print '<fieldset><legend>メタデータ</legend>';
        print '<fieldset><legend>og:image</legend>';
        $this->print_image_upload_form( 'og', 'og', \%main::data );
        print '</fieldset>';
        print
            '<fieldset><legend><label for="data:description">説明</label></legend>';
        print
            '<textarea name="data:description" id="data:description" cols="56" rows="6" wrap="soft" title="メタデータに埋め込む説明を設定します。">'
            . ( ( defined $main::data{'description'} )
            ? $main::data{'description'}
            : '' )
            . '</textarea>';
        print
            '<div class="note">メタデータに埋め込む説明を設定します。</div></fieldset>';
        print '</fieldset>';
    }

    print '<fieldset><legend>予定編集設定</legend>';
    print '<table class="field-table"><tbody>';

    print
        '<tr><td class="field-name"><span class="datatitle">内容の確認</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="data:skipConfirmation" id="data:skipConfirmation" value="1"'
        . ( ( $main::data{'skipConfirmation'} ) ? 'checked="checked"' : '' )
        . ' /> 予定内容確認を省略する</label></td>';
    print '</tr>';

    print '<tr>';
    print
        '<td class="field-name"><label for="data:notice">新規予定通知</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        'data:notice',
        '新規予定を管理人・共同オーナー・スタッフに通知するかどうか設定します。',
        '', $main::data{'notice'},
        ''     => '新規予定通知機能停止',
        'on'   => 'メールで通知（内容を含まない）',
        'data' => 'メールで通知（内容を含む）'
    );
    print '</td>';
    print '</tr>';

    print '</tbody></table>';
    print '</fieldset>';

    print '<fieldset><legend>設定範囲</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:freqDisabled" value="1"'
        . ( ( $main::data{'freqDisabled'} ) ? ' checked="checked"' : '' )
        . ' /> 「繰り返し」設定の使用を中止する</label></li>';
    print
        '<li><label><input type="checkbox" name="data:dateDisabled" value="1"'
        . ( ( $main::data{'dateDisabled'} ) ? ' checked="checked"' : '' )
        . ' /> 「終日」設定の使用を中止する</label>';
    print
        ' (<label><input type="checkbox" name="data:endDateDisabled" value="1"'
        . ( ( $main::data{'endDateDisabled'} ) ? ' checked="checked"' : '' )
        . ' /> 日付をまたいだ時間指定を禁止する</label>)</li>';
    print '</ol>';
    print '<fieldset><legend>日付と時刻</legend>';
    print
        '<input type="number" size="3" maxlength="3" name="data:boundsStart" value="'
        . $this->escape( $main::data{'boundsStart'} ) . '" />';
    print $this->mk_select_menu(
        'data:boundsStartUnit', '', '',
        ( $main::data{'boundsStartUnit'} || 'day' ),
        'min'   => '分',
        'hour'  => '時間',
        'day'   => '日',
        'week'  => '週',
        'month' => 'ヶ月',
        'year'  => '年',
    );
    print $this->mk_select_menu(
        'data:boundsStartDirection', '', '',
        $main::data{'boundsStartDirection'},
        'before' => '前',
        'after'  => '後',
    );
    print
        'から <input type="number" size="3" maxlength="3" name="data:boundsEnd" value="'
        . $this->escape( $main::data{'boundsEnd'} ) . '" />';
    print $this->mk_select_menu(
        'data:boundsEndUnit', '', '',
        ( $main::data{'boundsEndUnit'} || 'day' ),
        'min'   => '分',
        'hour'  => '時間',
        'day'   => '日',
        'week'  => '週',
        'month' => 'ヶ月',
        'year'  => '年',
    );
    print $this->mk_select_menu(
        'data:boundsEndDirection', '', '', $main::data{'boundsEndDirection'},
        'before' => '前',
        'after'  => '後',
    );
    print 'まで';
    print '</fieldset>';
    print '<fieldset><legend>イベントの長さ</legend>';
    print
        '<input type="number" size="3" maxlength="3" name="data:minLength" value="'
        . $this->escape( $main::data{'minLength'} ) . '" />';
    print $this->mk_select_menu(
        'data:minLengthUnit', '', '',
        ( $main::data{'minLengthUnit'} || 'min' ),
        'min'   => '分',
        'hour'  => '時間',
        'day'   => '日',
        'week'  => '週',
        'month' => 'ヶ月',
        'year'  => '年',
    );
    print
        '以上 <input type="number" size="3" maxlength="3" name="data:maxLength" value="'
        . $this->escape( $main::data{'maxLength'} ) . '" />';
    print $this->mk_select_menu(
        'data:maxLengthUnit', '', '',
        ( $main::data{'maxLengthUnit'} || 'min' ),
        'min'   => '分',
        'hour'  => '時間',
        'day'   => '日',
        'week'  => '週',
        'month' => 'ヶ月',
        'year'  => '年',
    );
    print '以下';
    print '</fieldset>';
    print '<fieldset><legend>「分」指定</legend>';
    print $this->mk_select_menu(
        'data:minScale', '', '', $main::data{'minScale'},
        ''   => '1分きざみ',
        '5'  => '5分きざみ',
        '10' => '10分きざみ',
        '15' => '15分きざみ',
        '20' => '20分きざみ',
        '30' => '30分きざみ',
    );
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset><legend>保留時間</legend>';
    print '<ol class="checkbox">';
    print '<li><label>登録から';
    print $this->mk_select_menu(
        'data:pendingTime', '', '', $main::data{'pendingTime'},
        ''      => '',
        1800    => '30分',
        3600    => '1時間',
        10800   => '3時間',
        21600   => '6時間',
        43200   => '12時間',
        86400   => '1日',
        259200  => '3日',
        604800  => '1週間',
        2592000 => '30日',
    );
    print
        '以内の予定を、「イベント修正」「イベント入力」「イベント削除」のいずれの権限も持たないユーザーには表示しない。</label></li>';
    print '<li><label>開始時が';
    print $this->mk_select_menu(
        'data:previewTime', '', '', $main::data{'previewTime'},
        ''       => '',
        604800   => '1週間後',
        1209600  => '2週間後',
        1814400  => '3週間後',
        2419200  => '4週間後',
        3024000  => '5週間後',
        3628800  => '6週間後',
        4233600  => '7週間後',
        4838400  => '8週間後',
        5443200  => '9週間後',
        6048000  => '10週間後',
        6652800  => '11週間後',
        7257600  => '12週間後',
        7862400  => '13週間後',
        8467200  => '14週間後',
        9072000  => '15週間後',
        9676800  => '16週間後',
        15768000 => '半年後',
        31536000 => '1年後',
    );
    print
        '以降の予定を、「イベント修正」「イベント入力」「イベント削除」のいずれの権限も持たないユーザーには表示しない。</label></li>';
    print
        '</ol><div class="warning">RSS には表示されます。</div></fieldset>';

    #RSS更新設定
    print '<fieldset><legend>RSS更新設定</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:siteRSSEnabled" value="1"'
        . ( ( $main::data{'siteRSSEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> '
        . ( ( -f '../../index.xml' )
        ? '<a href="../../index.xml" target="_blank">サイト全体の新着情報(RSS)</a>'
        : 'サイト全体の新着情報(RSS)' )
        . 'を更新する</label></li>';
    print
        '<li><label><input type="checkbox" name="data:masterRSSEnabled" value="1"'
        . ( ( $main::data{'masterRSSEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> '
        . (
        (   $default::master_file_dir
                && -f $default::master_file_dir . '/index.xml'
        )
        ? '<a href="'
            . $default::master_file_dir
            . '/index.xml'
            . '" target="_blank">カレンダー全体の新着情報(RSS)</a>'
        : 'カレンダー全体の新着情報(RSS)'
        )
        . 'を更新する</label></li>'
        if $default::subdivided;
    print '<li><label><input type="checkbox" name="data:RSSEnabled" value="1"'
        . ( ( $main::data{'RSSEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> '
        . ( ( $default::file && -f $default::file . '/index.xml' )
        ? '<a href="'
            . $default::file
            . '/index.xml'
            . '" target="_blank">このカレンダーの新着情報(RSS)</a>'
        : 'このカレンダーの新着情報(RSS)' )
        . 'を更新する</label></li>';
    print '</ol>';
    print '</fieldset>';

}

######################################################################
# サブ設定チェック
sub _is_valid_setting ($) {
    my $this  = shift;
    my $start = $this->bounds_to_datetime(
        $main::data{'boundsStart'},
        $main::data{'boundsStartUnit'},
        $main::data{'boundsStartDirection'}
    );
    my $end = $this->bounds_to_datetime(
        $main::data{'boundsEnd'},
        $main::data{'boundsEndUnit'},
        $main::data{'boundsEndDirection'}
    );
    if ( $start && $end && $start > $end ) {
        push(
            @{ $cgi::action->{'dialog'} },
            '日付と時刻の設定範囲を確認してください。開始日時が終了日時より後になっています。'
        ) if ref $cgi::action;
        delete $main::data{'boundsStart'};
        delete $main::data{'boundsStartUnit'};
        delete $main::data{'boundsStartDirection'};
        delete $main::data{'boundsEnd'};
        delete $main::data{'boundsEndUnit'};
        delete $main::data{'boundsEndDirection'};
    }
    my $min = $this->timeunit_to_sec( $main::data{'minLength'},
        $main::data{'minLengthUnit'} );
    my $max = $this->timeunit_to_sec( $main::data{'maxLength'},
        $main::data{'maxLengthUnit'} );
    $main::data{'minLengthUnit'} = 'min' if !$main::data{'minLengthUnit'};
    $main::data{'maxLengthUnit'} = 'max' if !$main::data{'maxLengthUnit'};
    if ( $min && $max && $min > $max ) {
        push(
            @{ $cgi::action->{'dialog'} },
            'イベントの長さの設定範囲を確認してください。最短時間が最長時間より長くなっています。'
        ) if ref $cgi::action;
        delete $main::data{'minLength'};
        delete $main::data{'minLengthUnit'};
        delete $main::data{'maxLength'};
        delete $main::data{'maxLengthUnit'};
    }
    if ( $main::data{'minScale'}
        && !grep( $_ eq $main::data{'minScale'}, 5, 10, 15, 20, 30 ) )
    {
        $main::data{'minScale'} = '';
    }
    return;
}

sub bounds_to_datetime ($$$$) {
    my ( $this, $bounds, $bounds_unit, $bounds_direction ) = @_;
    if ( defined $bounds && $bounds ne '' ) {
        $bounds = $this->timeunit_to_sec( $bounds, $bounds_unit );
        if ( $bounds_direction && $bounds_direction eq 'before' ) {
            return time - $bounds;
        }
        else {
            return time + $bounds;
        }
    }
    return 0;
}

sub timeunit_to_sec ($$$) {
    my ( $this, $t, $unit ) = @_;
    if ( defined $t && $t ne '' ) {
        $unit = 'day' if !$unit;
        if ( $unit eq 'min' ) {
            $t *= 60;
        }
        elsif ( $unit eq 'hour' ) {
            $t *= 3600;
        }
        elsif ( $unit eq 'month' ) {
            $t *= 2592000;
        }
        elsif ( $unit eq 'year' ) {
            $t *= 31536000;
        }
        elsif ( $unit eq 'week' ) {
            $t *= 604800;
        }
        else {
            $t *= 86400;
        }
        return $t;
    }
    return 0;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    unless ( $default::subdivided && ( !defined $main::c || $main::c eq '' ) )
    {
        #クラス登録
        Common::class_register( '0000', 'GeneralSetting' );

        #ツール登録
        $setting::jump_menu{'0000'} = '一般設定';
    }
}

1;
