package InputField;
require CMONOS::CGI_Lib::Tools::FieldEditor;
@ISA = qw(FieldEditor);
use strict;
use utf8;
######################################################################
# 入力フィールド編集モジュール
# InputField.pm ver1.0.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-12-07
######################################################################
# フィールド読み込み設定
sub set_field ($) {
    my $this = shift;
    $setting::field_db_name = 'inputfield';
    $setting::field         = 2;
    push( @check::checkbox,
        'protected', 'Hidden', 'detailTable', 'searchEnabled' );
    $this->{'foldable'} = 1;
}

######################################################################
# 新規フィールドDB
sub mk_field ($$) {
    my ( $this, $id ) = @_;
    return if $field::obj{$id}->db ne $id;
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';

    #データベース作成
    if ( defined $default::db && $this->set_dir($default::db) ) {
        my ( %new, %mtime );
        tie(%new, $env::dbm_file,
            $default::db . '/' . $id,
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %new ) {
            tie( %mtime, $env::dbm_file, $default::db . '/mtime',
                O_RDONLY, $env::dbm_mode );
            if ( ref tied %mtime ) {
                while ( my ($n) = each %mtime ) {
                    $new{$n} = '' if !defined $new{$n} || $new{$n} eq '';
                }
            }
            untie(%mtime);
        }
        untie(%new);
    }
}

######################################################################
# 設定項目名
sub print_setting_item_name ($$) {
    my ( $this, $table ) = @_;
    print $table->td('保護');
    print $table->td('公開');
    print $table->td('隠す');
    print $table->td('詳細表示時<br />一覧項目');
    print $table->td('検索対象');
}

sub note ($) {
    my $this = shift;
    return
        '<div class="warning">画像等アップロードされたファイルはアクセス可能な位置に保存されます。「非公開」または「隠す」に設定しても直接アクセスすることは可能です。</div>';
}

######################################################################
# 設定項目
sub print_setting_items ($$$$;$) {
    my ( $this, $table, $field, $name, $id ) = @_;
    $field->{'Open'} = '1'
        if !defined $field->{'Open'} || $field->{'Open'} eq '';

    if ( $field->{'class'} !~ /^user_/ ) {
        print $table->td( '<input type="checkbox" name="'
                . $name
                . 'protected" value="1"'
                . ( ( $field->{'protected'} ) ? ' checked="checked"' : '' )
                . ' />' );
    }
    else {
        print $table->td('');
    }

    print $table->td(
        $this->mk_select_menu(
            $name . 'Open', '', '', $field->{'Open'},
            '1' => '一般公開',
            '0' => '非公開'
        )
    );

    print $table->td( '<input type="checkbox" name="'
            . $name
            . 'Hidden" value="1"'
            . ( ( $field->{'Hidden'} ) ? ' checked="checked"' : '' )
            . ' />' );

    if ( $field->format_for_detail_enabled ) {
        print $table->td( '<input type="checkbox" name="'
                . $name
                . 'detailTable" value="1"'
                . ( ( $field->{'detailTable'} ) ? ' checked="checked"' : '' )
                . ' onclick="cmonos.form.item(this.form).checkboxSync(this.checked,true,\''
                . $name
                . 'Hidden\',false)" />' );
    }
    else {
        print $table->td('');
    }

    if ( $field->search_form_enabled ) {
        print $table->td( '<input type="checkbox" name="'
                . $name
                . 'searchEnabled" value="1"'
                . (
                ( $field->{'searchEnabled'} ) ? ' checked="checked"' : '' )
                . ' />' );
    }
    else {
        print $table->td('');
    }
}

######################################################################
#サブ設定
sub print_sub_setting_items ($) {
    my $this = shift;
    return if !%field::obj;

    print
        '<fieldset><legend>特殊フィールド</legend><table class="field-table"><tbody>';

    print
        '<tr><td class="field-name"><label for="data:for_title">タイトルフィールド</label></td>';
    print
        '<td class="field"><select name="data:for_title" id="data:for_title">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->{'Open'}
            && !$field::obj{ $field::ORDER{$i} }->{'Hidden'}
            && $field::obj{ $field::ORDER{$i} }->sort_enabled
            && $field::obj{ $field::ORDER{$i} }->{'class'}
            =~ /^(?:text|addr_pref|time_date|time_span)/;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'title'}
                    && $field::for{'title'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_location">場所フィールド</label></td>';
    print
        '<td class="field"><select name="data:for_location" id="data:for_location">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->{'Open'}
            && !$field::obj{ $field::ORDER{$i} }->{'Hidden'}
            && $field::obj{ $field::ORDER{$i} }->sort_enabled
            && $field::obj{ $field::ORDER{$i} }->{'class'}
            =~ /^(text|text_url|select_menu|select_box|addr_[0-9a-zA-Z_])$/;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'location'}
                    && $field::for{'location'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_content">本文フィールド</label></td>';
    print
        '<td class="field"><select name="data:for_content" id="data:for_content">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->{'Open'}
            && !$field::obj{ $field::ORDER{$i} }->{'Hidden'}
            && $field::obj{ $field::ORDER{$i} }->{'class'} =~ /^text/;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'content'}
                    && $field::for{'content'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_name">名前フィールド</label></td>';
    print
        '<td class="field"><select name="data:for_name" id="data:for_name">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->{'class'} =~ /^text/
            && $field::obj{ $field::ORDER{$i} }->{'Open'}
            && !$field::obj{ $field::ORDER{$i} }->{'Hidden'}
            && $field::obj{ $field::ORDER{$i} }->sort_enabled;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'name'}
                    && $field::for{'name'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="data:NAMECHECK" value="1"'
        . ( ($field::NAMECHECK) ? ' checked="checked"' : '' )
        . ' /> 名前を重複チェックデータベースに登録する（登録済みの名前と重複していないかチェックする）</label></li>';
    print
        '<li><label><input type="checkbox" name="data:NAMEREGISTER" value="1"'
        . ( ($field::NAMEREGISTER) ? ' checked="checked"' : '' )
        . ' /> 名前をユーザー名データベースに登録する（他の CGI で利用）</label></li>';
    print '</ol>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_mail">代表メールフィールド</label></td>';
    print
        '<td class="field"><select name="data:for_mail" id="data:for_mail">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->{'e-mail'}
            && $field::obj{ $field::ORDER{$i} }->{'class'} =~ /email/;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'mail'}
                    && $field::for{'mail'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_media">メディアフィールド</label></td>';
    print
        '<td class="field"><select name="data:for_media" id="data:for_media">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && ( $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_media'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq
            'file_shipainter'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_epub' );
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'media'}
                    && $field::for{'media'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_media2">メディアフィールド(予備1)</label></td>';
    print
        '<td class="field"><select name="data:for_media2" id="data:for_media2">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && ( $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_media'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq
            'file_shipainter'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_epub' );
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'media2'}
                    && $field::for{'media2'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_media3">メディアフィールド(予備2)</label></td>';
    print
        '<td class="field"><select name="data:for_media3" id="data:for_media3">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && ( $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_media'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq
            'file_shipainter'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_epub' );
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'media3'}
                    && $field::for{'media3'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_coordinates">座標フィールド</label></td>';
    print
        '<td class="field"><select name="data:for_coordinates" id="data:for_coordinates" title="Googleマップにインデックスを表示する際座標とするフィールドを指定します。">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        my $id = $field::ORDER{$i};
        next
            unless ref $field::obj{$id}
            && $field::obj{$id}->{'class'} eq 'webapi_gmap';
        print '<option value="'
            . $id . '"'
            . (
            (   defined $field::for{'coordinates'}
                    && $field::for{'coordinates'} eq $id
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{$id}->name, 32, 'I', 1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print '</tbody><tbody>';

    print
        '<tr><td class="field-name"><label for="data:for_top">詳細表示ページのトップ項目</label></td>';
    print '<td class="field"><select name="data:for_top" id="data:for_top">';
    print '<option value=""></option>';
    foreach ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            if !ref $field::obj{ $field::ORDER{$_} }
            || $field::obj{ $field::ORDER{$_} }->{'Hidden'}
            || !$field::obj{ $field::ORDER{$_} }->format_for_detail_enabled;
        print '<option value="'
            . $field::ORDER{$_} . '"'
            . (
            (   defined $field::for{'top'}
                    && $field::for{'top'} eq $field::ORDER{$_}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$_} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print '</tbody></table>';
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
}

sub field_general_setting_check ($) {
    my $this = shift;
    $main::data{'NAMECHECK'} = ''
        if !defined $main::data{'NAMECHECK'} || !$main::data{'NAMECHECK'};
    $main::data{'NAMEREGISTER'} = ''
        if !defined $main::data{'NAMEREGISTER'}
        || !$main::data{'NAMEREGISTER'};
    return;
}

######################################################################
# データベース削除
sub delete_field ($@) {
    my ( $this, @delete ) = @_;
    my ( $dir, $file, $delete_all );
    local ( *NO, *DIR );

    #フィールドが全部削除された場合
    $delete_all = 1 if keys %main::order == 0;

    #データベース削除
    opendir( DIR, $default::db );
    foreach $file ( readdir(DIR) ) {
        next unless $file !~ /^\./ && -f "$default::db/$file";
        if ( $delete_all || grep( $file =~ /^$_(\.|_|$)/, @delete ) > 0 ) {
            unlink "$default::db/$file";
        }
    }
    closedir(DIR);

    #添付ファイル削除
    opendir( DIR, $default::file );
    foreach $dir ( readdir(DIR) ) {
        next unless $dir !~ /^\./ && -d "$default::file/$dir";
        opendir( NO, "$default::file/$dir" );
        foreach $file ( readdir(NO) ) {
            next unless $file !~ /^\./ && -f "$default::file/$dir/$file";
            if ( grep( $file =~ /^$_(\.|$)/, @delete ) > 0 ) {
                unlink "$default::file/$dir/$file";
            }
        }
        closedir(NO);
    }
    closedir(DIR);
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {

        #クラス登録
        Common::class_register( 'c100', 'InputField' );

        #ツール登録
        $setting::jump_menu{'c100'} = '入力フィールド設定'
            if !defined $setting::jump_menu{'c100'};
    }
}

1;
