package ViewerPage;
require CMONOS::CGI_Lib::Tools::Page;
@ISA = qw(Page);
use strict;
use utf8;
######################################################################
# イベント表示ページ設定モジュール
# ViewerPage.pm ver1.1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-05-26
######################################################################
# 設定項目取得
sub get_sub_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #チェックボックス
    push( @check::checkbox,
        'viewFixed',                  'weekRow',
        'monOmitted',                 'toolButtonsDisabled',
        'pswdIcon',                   'icsLink',
        'defaultSearchOparatorFixed', 'customColorEnabled' );

    #カラー書式チェック
    $check::color{'cellBgcolor'}             = 'セル背景色';
    $check::color{'cellDateColor'}           = 'セル日付文字色';
    $check::color{'cellDateBgcolor'}         = 'セル日付背景色';
    $check::color{'cellDateBorderColor'}     = 'セル日付下辺色';
    $check::color{'todayBgcolor'}            = '今日背景色';
    $check::color{'todayDateColor'}          = '今日日付文字色';
    $check::color{'todayDateBgcolor'}        = '今日日付背景色';
    $check::color{'todayDateBorderColor'}    = '今日日付下辺色';
    $check::color{'saturdayBgcolor'}         = '土曜日背景色';
    $check::color{'saturdayDateColor'}       = '土曜日日付文字色';
    $check::color{'saturdayDateBgcolor'}     = '土曜日日付背景色';
    $check::color{'saturdayDateBorderColor'} = '土曜日日付下辺色';
    $check::color{'sundayBgcolor'}           = '日曜日背景色';
    $check::color{'sundayDateColor'}         = '日曜日日付文字色';
    $check::color{'sundayDateBgcolor'}       = '日曜日日付背景色';
    $check::color{'sundayDateBorderColor'}   = '日曜日日付下辺色';
    $check::color{'holidayBgcolor'}          = '祝休日背景色';
    $check::color{'holidayDateColor'}        = '祝休日日付文字色';
    $check::color{'holidayDateBgcolor'}      = '祝休日日付背景色';
    $check::color{'holidayDateBorderColor'}  = '祝休日日付下辺色';
    $check::color{'extraBgcolor'}            = '前後月背景色';
    $check::color{'extraDateColor'}          = '前後月日付文字色';
    $check::color{'extraDateBgcolor'}        = '前後月日付背景色';
    $check::color{'extraDateBorderColor'}    = '前後月日付下辺色';
    $check::color{'timeColor'}               = '時刻文字色';
    $check::color{'cellBorderColor'}         = '罫線色';

    #数字チェック
    $check::num{'startTime'} = 'タイムテーブルの開始位置';

    #URL書式チェック
    $check::url{'returnUrl'} = '戻る先URL' if !$default::subdivided;
    return;
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    $preference::id = 'viewer';    # データキー
    $preference::name
        = 'ビューワページレイアウト';    # 設定パネル名
    $preference::title
        = 'ビューワページのレイアウトについて設定します。';
    require CMONOS::CGI_Lib::Deco;
    $this->{'icon_obj'} = Deco->new( $env::image_dir . '/calendar',
        $env::image_lib . '/calendar' );
}

######################################################################
# 設定パネル
sub _print_header_tool ($) {
    my $this = shift;
    return
        '<option value="&lt;!--monthname nav=&quot;enabled&quot; format=&quot;&quot;--&gt;&lt;!--/monthname--&gt;">月 (&lt;!--monthname--&gt;)</option>'
        . '<option value="&lt;!--yearname nav=&quot;enabled&quot; format=&quot;&quot;--&gt;&lt;!--/yearname--&gt;">年 (&lt;!--yearname--&gt;)</option>'
        . '<option value="&lt;!--monthlist start=&quot;&quot; format=&quot;&quot;--&gt;&lt;!--/monthlist--&gt;">1年分の月リスト (&lt;!--monthlist--&gt;)</option>'
        . '<option value="&lt;!--linktolist--&gt;&lt;!--/linktolist--&gt;">リスト表示へのリンク (&lt;!--linktolist--&gt;)</option>'
        . '<option value="&lt;!--if view=&quot;monthly,daily,weekly,list,search&quot;--&gt;&lt;!--else--&gt;&lt;!--/if--&gt;">条件分岐 (&lt;!--if view=""--&gt;)</option>'
        . '<option value="&lt;!--list--&gt;&lt;!--/list--&gt;">予定リスト (&lt;!--list--&gt;)</option>';
}

sub print_sub_preference_panel ($;$$) {
    my ( $this, $id, $key ) = @_;
    print
        '<fieldset><legend><label for="data:viewer_defaultView">デフォルトの表示</label></legend>';
    print $this->mk_select_menu(
        'data:viewer_defaultView',
        'イベント表示のデフォルト形式を設定します。',
        '', $main::data{'viewer_defaultView'},
        'w' => '週',
        'm' => '月',
        'd' => '日',
    );
    print
        ' <label for="data:viewer_viewFixed"><input type="checkbox" name="data:viewer_viewFixed" id="data:viewer_viewFixed"  value="1"'
        . (
        (   defined $main::data{'viewer_viewFixed'}
                && $main::data{'viewer_viewFixed'}
        ) ? ' checked="checked"' : ''
        ) . ' /> 固定する</label>';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:viewer_wdayFormat">曜日名</label></legend>';
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday )
        = localtime(time);
    $year += 1900;
    $mon++;
    print $this->mk_select_menu(
        'data:viewer_wdayFormat',
        '曜日名表示形式を設定します。', '',
        $main::data{'viewer_wdayFormat'},
        'j' => $this->day_of_the_week( $year, $mon, $mday, 'j' ),
        'j' => $this->day_of_the_week( $year, $mon, $mday, 'J' ),
        'e' => $this->day_of_the_week( $year, $mon, $mday, 'e' ),
        'E' => $this->day_of_the_week( $year, $mon, $mday, 'E' ),
    );
    print '</fieldset>';

    print '<fieldset><legend>月表示設定</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label for="data:viewer_weekRow"><input type="checkbox" name="data:viewer_weekRow" id="data:viewer_weekRow"  value="1"'
        . (
        (   defined $main::data{'viewer_weekRow'}
                && $main::data{'viewer_weekRow'}
        ) ? ' checked="checked"' : ''
        ) . ' /> 曜日名行を表示する</label></li>';
    print
        '<li><label for="data:viewer_monOmitted"><input type="checkbox" name="data:viewer_monOmitted" id="data:viewer_monOmitted"  value="1"'
        . (
        (   defined $main::data{'viewer_monOmitted'}
                && $main::data{'viewer_monOmitted'}
        ) ? ' checked="checked"' : ''
        ) . ' /> 日付表示で月を省略する</label></li>';
    print '</ol>';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:viewer_startTime">タイムテーブルの開始位置</label></legend>';
    print '<select name="data:viewer_startTime">';
    print '<option value=""'
        . (
        (   !defined $main::data{'viewer_startTime'}
                || $main::data{'viewer_startTime'} eq ''
        ) ? ' selected="selected"' : ''
        ) . '>現在時</option>';

    foreach my $h ( 0 .. 14 ) {
        print '<option value="', $h, '"';
        print(
            (          defined $main::data{'viewer_startTime'}
                    && $main::data{'viewer_startTime'} ne ''
                    && $main::data{'viewer_startTime'} == $h
            ) ? ' selected="selected"' : ''
        );
        print '>', $h, '時</option>';
    }
    print '</select>';
    print '</fieldset>';

    print '<fieldset class="foldable">';
    print
        '<legend><label><input type="checkbox" name="data:viewer_customColorEnabled" value="1"'
        . (
        (   defined $main::data{'viewer_customColorEnabled'}
                && $main::data{'viewer_customColorEnabled'}
        ) ? ' checked="checked"' : ''
        )
        . ' onclick="cmonos.element.item(\'custom-color-settings\').disappear(!this.checked);" /> カスタムカラー設定</label></legend>';
    print '<table class="field-table" id="custom-color-settings">';
    $this->print_custom_color_settings( 'cell',     'セル' );
    $this->print_custom_color_settings( 'today',    '今日' );
    $this->print_custom_color_settings( 'saturday', '土曜日' );
    $this->print_custom_color_settings( 'sunday',   '日曜日' );
    $this->print_custom_color_settings( 'holiday',  '祝休日' );
    $this->print_custom_color_settings( 'extra',    '前後月' );
    print '<tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:viewer_timeColor">時刻文字色</label></td>';
    print '<td class="q-field">';
    print
        '<input type="text" name="data:viewer_timeColor" id="data:viewer_timeColor" value="'
        . $this->escape( $main::data{'viewer_timeColor'} )
        . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:viewer_timeColor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
        . ( ( $main::data{'viewer_timeColor'} )
        ? ' style="background-color: '
            . $this->escape( $main::data{'viewer_timeColor'} ) . '"'
        : '' )
        . ' />';
    print
        ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:viewer_timeColor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:viewer_timeColor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    print
        ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:viewer_timeColor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:viewer_timeColor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
    print '</td>';
    print
        '<td class="field-name"><label for="data:viewer_cellBorderColor">罫線色</label></td>';
    print '<td class="q-field">';
    print
        '<input type="text" name="data:viewer_cellBorderColor" id="data:viewer_cellBorderColor" value="'
        . $this->escape( $main::data{'viewer_cellBorderColor'} )
        . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:viewer_cellBorderColor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
        . ( ( $main::data{'viewer_cellBorderColor'} )
        ? ' style="background-color: '
            . $this->escape( $main::data{'viewer_cellBorderColor'} ) . '"'
        : '' )
        . ' />';
    print
        ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:viewer_cellBorderColor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:viewer_cellBorderColor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    print
        ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:viewer_cellBorderColor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:viewer_cellBorderColor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
    print '</td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</fieldset>';
    print $this->javascript(
        'cmonos.element.item(\'custom-color-settings\').disappear(true);')
        if !$main::data{'viewer_customColorEnabled'};

    print '<fieldset><legend>リスト表示設定</legend>';
    print
        '<fieldset><legend><label for="data:viewer_listUnit">リスト対象</label></legend>';
    print $this->mk_select_menu(
        'data:viewer_listUnit',
        'リストをまとめる単位を設定します。', '',
        $main::data{'viewer_listUnit'},
        'monthly' => '月単位',
        'yearly'  => '年単位',
    );
    print $this->mk_select_menu(
        'data:viewer_listTarget', 'リスト対象を設定します。', '',
        $main::data{'viewer_listTarget'},
        ''       => 'すべて',
        'future' => '未来のみ',
        'past'   => '過去のみ',
    );
    print '</fieldset>';
    print
        '<fieldset><legend><label for="data:viewer_startMonth">年単位リストの開始月</label></legend>';
    print $this->mk_select_menu(
        'data:viewer_startMonth',
        '年単位リストの開始月を設定します。', '',
        $main::data{'viewer_startMonth'},
        '1'  => '1月開始',
        '2'  => '2月開始',
        '3'  => '3月開始',
        '4'  => '4月開始',
        '5'  => '5月開始',
        '6'  => '6月開始',
        '7'  => '7月開始',
        '8'  => '8月開始',
        '9'  => '9月開始',
        '10' => '10月開始',
        '11' => '11月開始',
        '12' => '12月開始',
    );
    print '</fieldset>';
    print
        '<fieldset><legend><label for="data:viewer_listSort">ソート</label></legend>';
    print $this->mk_select_menu(
        'data:viewer_listSort',
        'リストをまとめる単位を設定します。', '',
        $main::data{'viewer_listSort'},
        'ascend'  => '古い順',
        'descend' => '新しい順',
    );
    print '</fieldset>';
    $this->use_field( 'field', $default::field_pref_dir . '/inputfield', 2 );
    require CMONOS::CGI_Lib::Template;
    my $this_template = Template->new( \%field::obj );
    $this_template->print_template_form(
        'data:' . $id . 'template',
        'リスト表示テンプレート',
        \$main::data{ $key . 'template' },
        '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:'
            . $id
            . 'template\').quotationByMenu(this)" title="これらのタグは、書き出し時に適切なタグに変換されます。">'
            . '<option value="">▼独自タグを挿入する</option>'
            . '<option value="&lt;!--dt--&gt;&lt;!--/dt--&gt;">予定の日時 (&lt;!--dt--&gt;)</option>'
            . '<option value="&lt;!--title--&gt;&lt;!--/title--&gt;">詳細表示リンクつきタイトル (&lt;!--title--&gt;)</option>'
            . '<option value="&lt;!--readmore--&gt;&lt;!--/readmore--&gt;">もっとくわしく… (&lt;!--readmore--&gt;)</option>'
            . '<option value="&lt;!--list--&gt;&lt;!--/list--&gt;">予定リスト (&lt;!--list--&gt;)</option>'
            . '<option value="&lt;!--editbutton--&gt;&lt;!--/editbutton--&gt;">編集ボタン (&lt;!--editbutton--&gt;)</option>'
            . '</select>'
    );
    print '</fieldset>';

    print '<fieldset><legend>ボタン類</legend>';
    print
        '<label><input type="checkbox" name="data:viewer_toolButtonsDisabled" value="1"'
        . (
        (   defined $main::data{'viewer_toolButtonsDisabled'}
                && $main::data{'viewer_toolButtonsDisabled'}
        ) ? ' checked="checked"' : ''
        )
        . ' onclick="cmonos.element.item(\'tool-button-settings\').disappear(this.checked);" /> 非表示</label>';
    print '</fieldset>';
    print '<div id="tool-button-settings">';

    print
        '<fieldset><legend>iCalendar 形式ファイルダウンロードリンク</legend>';
    print '<label><input type="checkbox" name="data:viewer_icsLink" value="1"'
        . ( ( $main::data{'viewer_icsLink'} ) ? ' checked="checked"' : '' )
        . ' /> 使用する</label>';
    print '</fieldset>';

    print '<fieldset><legend>ナビゲーションボタン設定</legend>';
    $this->print_button_menu( '前へ ボタン', 'calprev' );
    $this->print_button_menu( '検索条件をリセット ボタン',
        'calreset' );
    $this->print_button_menu( 'カレンダーインデックス ボタン',
        'calindex' )
        if $default::subdivided;
    $this->print_button_menu( '今日 ボタン', 'caltoday' );
    $this->print_button_menu( '次へ ボタン', 'calnext' );
    print '</fieldset>';

    print '<fieldset><legend>表示切り替えボタン設定</legend>';
    $this->print_button_menu( '日表示ボタン',        'daily' );
    $this->print_button_menu( '週表示ボタン',        'weekly' );
    $this->print_button_menu( '月表示ボタン',        'monthly' );
    $this->print_button_menu( '検索ページ ボタン', 'calsearch' );
    print '</fieldset>';

    print '<fieldset><legend>ツールパネルボタン設定</legend>';
    $this->print_button_menu( 'カレンダーパネル ボタン',
        'calendarpanel' );
    $this->print_button_menu( '表示コントロールパネル ボタン',
        'togglepanel' );
    print '</fieldset>';

    print '<fieldset><legend>ツールボタン設定</legend>';
    $this->print_button_menu( '新規予定ボタン',           'calinput' );
    $this->print_button_menu( '環境設定を開く ボタン', 'caltool' );
    print
        '<label for="data:viewer_pswdIcon"><input type="checkbox" name="data:viewer_pswdIcon" id="data:viewer_pswdIcon"  value="1"'
        . (
        (   defined $main::data{'viewer_pswdIcon'}
                && $main::data{'viewer_pswdIcon'}
        ) ? ' checked="checked"' : ''
        ) . ' /> ログインボタンを表示する</label>';
    $this->print_button_menu( 'ログインボタン', 'callogin' );
    print '</fieldset>';

    print '</fieldset>';

    print '<fieldset><legend>予定編集ボタン設定</legend>';
    $this->print_button_menu( '予定編集ボタン', 'eventedit' );
    $this->print_button_menu( '予定削除ボタン', 'eventdelete' );
    $this->print_button_menu( '予定追加ボタン', 'eventadd' );
    print '</fieldset>';
    print '</div>';

    print '<fieldset>';
    print '<legend>戻る ボタン</legend>';
    print '<fieldset>';
    print $cgi::Deco->button_menu( 'data:returnButton', 'return', 120,
        $main::data{'returnButton'} );
    print '</fieldset>';
    print
        '<div class="additional-field"><label for="data:returnUrl">戻る先URL</label><input type="text" size="50" maxlength="100" name="data:returnUrl" id="data:returnUrl" value="'
        . $this->escape( $main::data{'returnUrl'} )
        . '" /></div>'
        if !$default::subdivided;
    print '</fieldset>';

    print $this->javascript(
        'cmonos.element.item(\'tool-button-settings\').disappear(true);')
        if $main::data{'viewer_toolButtonsDisabled'};

    print '<fieldset>';
    print
        '<legend><label for="data:viewer_defaultSearchOparator">デフォルトの検索方式</label></legend>';
    print $this->mk_select_menu(
        'data:viewer_defaultSearchOparator',
        '', '',
        (   ( defined $main::data{'viewer_defaultSearchOparator'} )
            ? $main::data{'viewer_defaultSearchOparator'}
            : 'OR'
        ),
        'AND' => '全ての条件に一致',
        'OR'  => 'いずれかの条件に一致',
    );
    print
        ' <label><input type="checkbox" name="data:viewer_defaultSearchOparatorFixed" id="data:viewer_defaultSearchOparatorFixed" value="1"'
        . ( ( $main::data{'viewer_defaultSearchOparatorFixed'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 固定する</label>';
    print '</fieldset>';

}

sub print_custom_color_settings ($$$) {
    my ( $this, $prefix, $name ) = @_;
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="data:viewer_'
        . $prefix
        . 'Bgcolor">'
        . $name
        . '背景色</label></td>';
    print '<td class="q-field">';
    print '<input type="text" name="data:viewer_'
        . $prefix
        . 'Bgcolor" id="data:viewer_'
        . $prefix
        . 'Bgcolor" value="'
        . $this->escape( $main::data{ 'viewer_' . $prefix . 'Bgcolor' } )
        . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'Bgcolor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
        . ( ( $main::data{ 'viewer_' . $prefix . 'Bgcolor' } )
        ? ' style="background-color: '
            . $this->escape( $main::data{ 'viewer_' . $prefix . 'Bgcolor' } )
            . '"'
        : '' )
        . ' />';
    print ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'Bgcolor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'Bgcolor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    print ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:viewer_'
        . $prefix
        . 'Bgcolor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:viewer_'
        . $prefix
        . 'Bgcolor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
    print '</td>';
    print '<td class="field-name"><label for="data:viewer_'
        . $prefix
        . 'DateColor">'
        . $name
        . '日付文字色</label></td>';
    print '<td class="q-field">';
    print '<input type="text" name="data:viewer_'
        . $prefix
        . 'DateColor" id="data:viewer_'
        . $prefix
        . 'DateColor" value="'
        . $this->escape( $main::data{ 'viewer_' . $prefix . 'DateColor' } )
        . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'DateColor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
        . ( ( $main::data{ 'viewer_' . $prefix . 'DateColor' } )
        ? ' style="background-color: '
            . $this->escape(
            $main::data{ 'viewer_' . $prefix . 'DateColor' } )
            . '"'
        : '' )
        . ' />';
    print ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'DateColor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'DateColor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    print ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:viewer_'
        . $prefix
        . 'DateColor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:viewer_'
        . $prefix
        . 'DateColor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
    print '</td>';
    print '</tr>';
    print '<tr class"tbody-end">';
    print '<td class="field-name"><label for="data:viewer_'
        . $prefix
        . 'DateBgcolor">'
        . $name
        . '日付背景色</label></td>';
    print '<td class="q-field">';
    print '<input type="text" name="data:viewer_'
        . $prefix
        . 'DateBgcolor" id="data:viewer_'
        . $prefix
        . 'DateBgcolor" value="'
        . $this->escape( $main::data{ 'viewer_' . $prefix . 'DateBgcolor' } )
        . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'DateBgcolor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
        . (
        ( $main::data{ 'viewer_' . $prefix . 'DateBgcolor' } )
        ? ' style="background-color: '
            . $this->escape(
            $main::data{ 'viewer_' . $prefix . 'DateBgcolor' }
            )
            . '"'
        : ''
        ) . ' />';
    print ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'DateBgcolor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'DateBgcolor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    print ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:viewer_'
        . $prefix
        . 'DateBgcolor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:viewer_'
        . $prefix
        . 'DateBgcolor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
    print '</td>';
    print '<td class="field-name"><label for="data:viewer_'
        . $prefix
        . 'DateBorderColor">'
        . $name
        . '日付下辺色</label></td>';
    print '<td class="q-field">';
    print '<input type="text" name="data:viewer_'
        . $prefix
        . 'DateBorderColor" id="data:viewer_'
        . $prefix
        . 'DateBorderColor" value="'
        . $this->escape(
        $main::data{ 'viewer_' . $prefix . 'DateBorderColor' } )
        . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'DateBorderColor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
        . (
        ( $main::data{ 'viewer_' . $prefix . 'DateBorderColor' } )
        ? ' style="background-color: '
            . $this->escape(
            $main::data{ 'viewer_' . $prefix . 'DateBorderColor' }
            )
            . '"'
        : ''
        ) . ' />';
    print ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'DateBorderColor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:viewer_'
        . $prefix
        . 'DateBorderColor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    print ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:viewer_'
        . $prefix
        . 'DateBorderColor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:viewer_'
        . $prefix
        . 'DateBorderColor\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
    print '</td>';
    print '</tr>';
    print '</tbody>';
}

sub print_button_menu ($$$) {
    my ( $this, $title, $name ) = @_;
    print '<fieldset><legend>' . $title . '</legend>';
    print $this->{'icon_obj'}->button_menu( 'data:viewer_' . $name,
        $name, 120, $main::data{ 'viewer_' . $name } );
    print '</fieldset>';
}

sub day_of_the_week ($$$$$) {
    my ( $this, $year, $mon, $mday, $type ) = @_;
    return ( $type && $type eq 'e' )
        ? $this->wday_name( $year, $mon, $mday, 1 )
        : ( $type && $type eq 'E' ) ? $this->wday_name( $year, $mon, $mday )
        : ( $type && $type eq 'J' )
        ? $this->wday_jname( $year, $mon, $mday ) . '曜日'
        : $this->wday_jname( $year, $mon, $mday );
}

######################################################################
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {

        #クラス登録
        Common::class_register( 'c010', 'ViewerPage' );

        #ツール登録
        $setting::jump_menu{'c010'}
            = 'ページレイアウト';    #これがデフォルト
        $setting::sub_jump_menu{'page'}{'c010'} = 'ビューワページ';
        $setting::selected_jump_menu{'c010'} = 'c010'
            ; #サブメニューがあるときメインメニューのデフォルトキー

        $preference::page{'c010'} = 'viewer';

    }
}

1;
