package Viewer;
require CMONOS::Calendar::CalCommon;
@ISA = qw(CalCommon);
use strict;
use utf8;
######################################################################
# カレンダーイベント表示モジュール
# Viewer.pm ver1.1.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-06-27
######################################################################
# $this->{'event'} = {
#	title		=>	タイトル,
#	start_date	=>	開始日(20070505),
#	start_time	=>	開始時(1010),
#	end_date	=>	終了日(20070505),
#	end_time	=>	終了時(1110),
#	category	=>	イベントカテゴリーID,
#	rrurl		=>	RRULE ハッシュリファレンス
# }
# $this->{'date'} = {
#	start_min	=>	開始分(0-1440),
#	end_min		=>	終了分(0-1440),
#	prev		=>	○日後まで続く,
#	next		=>	○日前から続く,
# }
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my (%db,           $title_field, $sc_field, $title_db,
        %date,         %event,       @dates,    %match,
        $pending_time, $preview_time
    );

    #承認待ち項目警告
    $this->set_pending_item_alert
        if !defined $main::v && $this->is_secure('tools');

    #メインターゲット
    if (   $main::today
        || !$main::date
        || $this->is_nan($main::date)
        || $main::date < 19700101
        || $main::date > 20360101 )
    {
        $main::date = $this->time_to_dtstamp( time, 1 );
    }
    else {
        $main::date = $this->dt_fix( $main::date, 1 );
    }

    if (   $main::event_category
        && $field::for{'category'}
        && ref $field::obj{ $field::for{'category'} }
        && $field::obj{ $field::for{'category'} }->db eq 'category' )
    {
        $main::search{ $field::for{'category'} }{'str'}
            = ( $main::event_category =~ /,/ )
            ? [ split( /,/, $main::event_category ) ]
            : $main::event_category;
        undef $main::event_category;
    }

    #検索条件リセット
    if ($main::reset) {
        undef %main::search;
        undef $main::and;
    }
    elsif (%main::search) {
        my %delete;
        my @delete_ids;
        while ( my ( $id, $hash ) = each %main::search ) {
            my $count = 0;
            if ( $id eq 'freq' ) {
                $count++ if $main::search{$id};
            }
            else {
                while ( my ( $k, $v ) = each %$hash ) {
                    if ( defined $v
                        && ( ( ref $v && @$v ) || ( !ref $v && $v ne '' ) ) )
                    {
                        $count++;
                    }
                    else {
                        $delete{$id} = $k;
                    }
                }
                if ( $count && $count == 1 && $main::search{$id}{'mode'} ) {
                    $delete{$id} = 'mode';
                    $count = 0;
                }
            }
            push( @delete_ids, $id ) if !$count;
        }
        while ( my ( $id, $name ) = each %delete ) {
            delete $main::search{$id}{$name};
        }
        foreach my $id (@delete_ids) {
            delete $main::search{$id};
        }
        undef %main::search if !%main::search;
    }
    @setting::form = grep( $_ ne 'search' && $_ ne 'and', @setting::form )
        if !%main::search;

    #デフォルト表示形式
    if ( $setting::page{'viewer'}{'viewFixed'}
        && ( !$main::v || $main::v ne 'l' ) )
    {
        $main::v = $setting::page{'viewer'}{'defaultView'} || 'w';
    }
    elsif ( !$main::v ) {
        $main::v
            = ( ( $setting::page{'viewer'}{'defaultView'} )
            ? $setting::page{'viewer'}{'defaultView'}
            : 'w' );
    }
    $this->set_og_url(
        $this->my_url
            . (
            ( defined $main::c && $main::c ne '' ) ? '?c=' . $main::c : ''
            )
    );
    $this->set_og_cgi_summary;
    $this->set_last_modified( $setting::general{'mtime'} )
        if !$main::srch && $setting::general{'lastModifiedHeaderEnabled'};

    #ビューオブジェクト
    if ($main::srch) {
        require CMONOS::Calendar::CalSearch;
        $this->{'view'} = CalSearch->new;
        $this->print_calendar;
        return;
    }
    elsif ( $main::v eq 'l' && $setting::page{$setting::key}{'template'} ) {
        require CMONOS::Calendar::CalList;
        $this->{'view'} = CalList->new;
        if ($main::next) {
            $main::date = $this->{'view'}->next_calendar_date($main::date);
        }
        elsif ($main::prev) {
            $main::date = $this->{'view'}->prev_calendar_date($main::date);
        }
        $this->print_calendar;
        return;
    }
    elsif ( $main::v eq 'm' ) {
        require CMONOS::Calendar::CalMonthlyView;
        $this->{'view'} = CalMonthlyView->new;
    }
    elsif ( $main::v eq 'd' ) {
        require CMONOS::Calendar::CalDailyView;
        $this->{'view'} = CalDailyView->new;
    }
    else {
        $main::v = 'w';
        require CMONOS::Calendar::CalWeeklyView;
        $this->{'view'} = CalWeeklyView->new;
    }
    if ($main::next) {
        $main::date = $this->{'view'}->next_calendar_date($main::date);
    }
    elsif ($main::prev) {
        $main::date = $this->{'view'}->prev_calendar_date($main::date);
    }

    #イベントカテゴリー読み込み
    if ( $field::for{'category'} ) {
        $this->import_db( $default::preference_dir . '/event_category',
            \%setting::event_category, 1 )
            unless %setting::event_category
            && ref $setting::event_category{'ORDER'}
            && %{ $setting::event_category{'ORDER'} };
    }

    #表示対象となる日付
    @dates = $this->{'view'}->get_target_dates($main::date);

    use Fcntl qw(:DEFAULT :flock);
    eval 'use ' . $env::dbm_file . ';';
    tie( %{ $db{'category'} },
        $env::dbm_file, "$default::db/category", O_RDONLY, $env::dbm_mode )
        if $field::for{'category'};
    tie(%{ $db{'dt'} }, $env::dbm_file, "$default::db/dt",
        O_RDONLY,       $env::dbm_mode
    );
    tie(%{ $db{'date'} }, $env::dbm_file, "$default::db/date",
        O_RDONLY,         $env::dbm_mode
    );
    if (   $setting::general{'pendingTime'}
        && !$this->is_secure('input')
        && !$this->is_secure('delete')
        && !$this->is_secure('revise') )
    {
        tie(%{ $db{'ctime'} }, $env::dbm_file, "$default::db/ctime",
            O_RDONLY,          $env::dbm_mode
        );
        $pending_time = time - $setting::general{'pendingTime'};
    }
    if (   $setting::general{'previewTime'}
        && !$this->is_secure('input')
        && !$this->is_secure('delete')
        && !$this->is_secure('revise') )
    {
        $preview_time
            = $this->time_id( time + $setting::general{'previewTime'} );
    }
    $title_field
        = ( $field::for{'title'} ) ? $field::for{'title'}
        : ( $field::for{'key'} )  ? $field::for{'key'}
        : ( $field::for{'name'} ) ? $field::for{'name'}
        :                           "";
    if ($title_field) {
        $title_db = $field::obj{$title_field}->db;
        tie( %{ $db{$title_db} },
            $env::dbm_file, "$default::db/$title_db", O_RDONLY,
            $env::dbm_mode );
    }
    if (   $field::for{'showcase'}
        && ref $field::obj{ $field::for{'showcase'} }
        && $field::obj{ $field::for{'showcase'} }->db eq 'showcase' )
    {
        $sc_field = $field::obj{ $field::for{'showcase'} };
        tie( %{ $db{'showcase'} },
            $env::dbm_file, "$default::db/showcase", O_RDONLY,
            $env::dbm_mode );
    }
    require CMONOS::CGI_Lib::Holiday;
    my $holiday = Holiday->new;

    #表示対象となるイベントを読み込み
    my @search_fields = (%main::search) ? keys %main::search : ();
DATE:
    foreach my $ymd (@dates) {
        my ( $y, $m, $d, $md );
        $date{$ymd} = {};

        #関連する日付
        my @dt = ( 'DAILY', $ymd );
        if ( $ymd =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
            $md = $2 . $3;
            ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
            $date{$ymd}{'wday'} = $this->get_wday( $y, $m, $d );
            my $holiday_name = $holiday->jp($y)->name( $m, $d );
            $date{$ymd}{'holiday'} = $holiday_name if $holiday_name;
            my $w = $this->number_to_wday( $date{$ymd}{'wday'} );
            my $wn = int( $d / 7 ) + ( ( $d % 7 ) ? 1 : 0 ) . $w;
            push( @dt, $w, $wn, $d, $md );
        }

        #関連する日付のイベント
    DATEID:
        foreach my $dt (@dt) {
            if ( $db{'date'}->{ $this->encode_utf8($dt) } ) {
                my @n = split(
                    /,/,
                    $this->decode_utf8(
                        $db{'date'}->{ $this->encode_utf8($dt) }
                    )
                );
            EVENT:
                foreach my $n (@n) {
                    my $category      = '';
                    my $category_name = '';
                    my $prev = ( $n =~ s/^([\-]?[0-9]+)<// ) ? eval($1) : 0;
                    my $next = ( $n =~ s/>([\-]?[0-9]+)$// ) ? eval($1) : 0;
                    next DATEID
                        if $pending_time
                        && $db{'ctime'}->{ $this->encode_utf8($n) }
                        && $db{'ctime'}->{ $this->encode_utf8($n) }
                        > $pending_time;

                    #カテゴリーチェック
                    if (   $field::for{'category'}
                        && %setting::event_category
                        && ref $setting::event_category{'ORDER'}
                        && %{ $setting::event_category{'ORDER'} }
                        && $db{'category'}->{ $this->encode_utf8($n) } )
                    {
                        $category = $this->decode_utf8(
                            $db{'category'}->{ $this->encode_utf8($n) } );
                        if ( !ref $setting::event_category{$category} ) {
                            undef $category;
                        }
                        elsif ( $setting::event_category{$category}{'closed'}
                            && !$this->is_secure('closed') )
                        {
                            next EVENT;
                        }
                        $category_name
                            = $setting::event_category{$category}{'name'};
                    }

                    #検索
                    if (@search_fields) {
                        my ( $id, $is_matched, $search_enabled );
                    SEARCH:
                        foreach my $id (@search_fields) {
                            if ( $id eq 'freq' ) {
                                if ( $main::search{'freq'} ) {
                                    $search_enabled = 1;
                                    my $freq = 0;
                                    if ($db{'dt'}->{ $this->encode_utf8($n) }
                                        && $this->decode_utf8(
                                            $db{'dt'}
                                                ->{ $this->encode_utf8($n) }
                                        ) =~ /FREQ=\w+/
                                        )
                                    {
                                        $freq = 1;
                                    }
                                    if ((      $main::search{'freq'} eq '1'
                                            && $freq
                                        )
                                        || ( $main::search{'freq'} eq '-1'
                                            && !$freq )
                                        )
                                    {
                                        $is_matched = 1;
                                    }
                                    elsif ($main::and) {
                                        next EVENT;
                                    }
                                }
                            }
                            elsif (
                                   ref $field::obj{$id}
                                && $field::obj{$id}->{'searchEnabled'}
                                && (   $id eq $title_field
                                    || $field::obj{$id}->{'class'} eq
                                    'xcal_category'
                                    || (defined $setting::security{'detail'}
                                        {'passwordCheck'}
                                        && $setting::security{'detail'}
                                        {'passwordCheck'} eq 'nocheck' )
                                    || $this->is_one_of_owners
                                )
                                && !$field::obj{$id}->{'Hidden'}
                                && $field::obj{$id}->match_enabled
                                && (   $field::obj{$id}->{'Open'}
                                    || $this->is_secure('secret') )
                                )
                            {
                                $search_enabled = 1;
                                my $db = $field::obj{$id}->db;
                                next SEARCH if !defined $db || !$db;
                                tie( %{ $db{$db} },
                                    $env::dbm_file, "$default::db/$db",
                                    O_RDONLY, $env::dbm_mode )
                                    if !ref tied %{ $db{$db} };
                                my $dat = $this->decode_utf8(
                                    $db{$db}->{ $this->encode_utf8($n) } );
                                if ( $field::obj{$id}->match( \$dat ) ) {
                                    $is_matched = 1;
                                }
                                elsif ($main::and) {
                                    next EVENT;
                                }
                            }
                        }
                        next EVENT if $search_enabled && !$is_matched;
                        if ( !$search_enabled ) {
                            undef %main::search;
                            undef @search_fields;
                        }
                    }

                    if ( ref $event{$n} ) {
                        next EVENT
                            if !$this->is_required_date(
                            $dt,
                            $event{$n}->{'rrule'},
                            $event{$n}->{'start_date'},
                            $event{$n}->{'end_date'},
                            $ymd,
                            $y,
                            $m,
                            $d,
                            $md,
                            \$prev,
                            \$next
                            );
                    }
                    elsif (
                        $db{'dt'}->{ $this->encode_utf8($n) }
                        && $this->decode_utf8(
                            $db{'dt'}->{ $this->encode_utf8($n) }
                        )
                        =~ /^([0-9]{8})T?([0-9]{6}|)-([0-9]{8})T?([0-9]{6}|)\x09([^\x09\x0A\x0C\x0D\x20]*)/
                        )
                    {
                        my ($start_date, $start_time, $end_date,
                            $end_time,   $rrule
                        ) = ( $1, $2, $3, $4, $5 );
                        next EVENT
                            if $preview_time
                            && $preview_time
                            < $start_date . ( $start_time || '000000' );
                        my $rrule_hash;
                        if ($rrule) {
                            while ( $rrule
                                =~ /\G([0-9a-zA-Z_]+)=([0-9a-zA-Z_,]+);?/g )
                            {
                                my ( $k, $v ) = ( $1, $2 );
                                if ( $v =~ /,/ ) {
                                    $rrule_hash->{$k} = [ split( /,/, $v ) ];
                                }
                                else {
                                    $rrule_hash->{$k} = $v;
                                }
                            }
                            $this->scalar_to_array( \$rrule_hash->{'BYDAY'} )
                                if $rrule_hash->{'BYDAY'};
                            next EVENT
                                if !$this->is_required_date( $dt,
                                $rrule_hash, $start_date, $end_date, $ymd,
                                $y, $m, $d, $md, \$prev, \$next );
                            $event{$n}->{'rrule'} = $rrule_hash;
                        }
                        $event{$n}->{'title'} = $this->detag_text(
                            $field::obj{$title_field}->usual_format(
                                $this->decode_utf8(
                                    $db{$title_db}->{ $this->encode_utf8($n) }
                                ),
                                $default::file . '/' . $n,
                                $main::c, $n
                            )
                        ) if $title_field;
                        $event{$n}->{'title'} = $category_name
                            if ( !defined $event{$n}->{'title'}
                            || $event{$n}->{'title'} eq '' )
                            && $category_name;
                        $event{$n}->{'start_date'} = $start_date;
                        $event{$n}->{'start_time'} = $start_time
                            if $start_time ne '';
                        $event{$n}->{'end_date'} = $end_date;
                        $event{$n}->{'end_time'} = $end_time
                            if $end_time ne '';
                        $event{$n}->{'category'} = $category if $category;

                        if ( defined $sc_field && ref $sc_field ) {
                            my %data = $sc_field->elements(
                                $this->decode_utf8(
                                    $db{'showcase'}
                                        ->{ $this->encode_utf8($n) }
                                )
                            );
                            if ( !$sc_field->sc_stock_is_ok( \%data, 1 ) ) {
                                $event{$n}->{'sellout'} = 1;
                            }
                            elsif (
                                defined $sc_field->{
                                    'limitedAvailabilityBounds'}
                                && $sc_field->{'limitedAvailabilityBounds'}
                                ne ''
                                && $data{'stock'}
                                && $data{'stock'}
                                <= $sc_field->{'limitedAvailabilityBounds'} )
                            {
                                $event{$n}->{'remaining'} = 1;
                            }
                        }
                    }
                    else {
                        next EVENT;
                    }
                    my @available_wday;
                    if ($category
                        && defined $setting::event_category{$category}
                        {'availableWday'}
                        && (ref $setting::event_category{$category}
                            {'availableWday'}
                            || $setting::event_category{$category}
                            {'availableWday'} ne '' )
                        )
                    {
                        $this->scalar_to_array(
                            \$setting::event_category{$category}
                                {'availableWday'} );
                        @available_wday
                            = @{ $setting::event_category{$category}
                                {'availableWday'} };
                        my $is_holiday
                            = (    $date{$ymd}{'wday'} != 0
                                && $date{$ymd}{'holiday'}
                                && grep( $_ == 7, @available_wday ) ) ? 1 : 0;
                        unless (
                            (   grep( $_ == $date{$ymd}{'wday'},
                                    @available_wday )
                                && !$is_holiday
                            )
                            || (!grep( $_ == $date{$ymd}{'wday'},
                                    @available_wday )
                                && $is_holiday
                            )
                            )
                        {
                            if (   !$event{$n}->{'start_time'}
                                && !$setting::event_category{$category}
                                {'overlapDisabled'} )
                            {
                                $date{$ymd}{'date'}{$n}{'prev'}   = $prev;
                                $date{$ymd}{'date'}{$n}{'next'}   = $next;
                                $date{$ymd}{'date'}{$n}{'span'}   = 0;
                                $date{$ymd}{'date'}{$n}{'hidden'} = 1;
                            }
                            next EVENT;
                        }
                    }
                    if ( $category
                        && defined $setting::event_category{$category}
                        {'availableTime'}
                        && $setting::event_category{$category}
                        {'availableTime'} ne '' )
                    {
                        if (   $event{$n}->{'start_time'}
                            || $setting::event_category{$category}
                            {'overlapDisabled'} )
                        {
                            my $start      = "0000";
                            my $start_time = "0000";
                            my $end_time   = "2400";
                            my $start_min  = 0;
                            my $end_min    = 1440;
                            my $count      = 0;
                            my $this_event;
                            if (  !$prev
                                && $event{$n}->{'start_time'}
                                && $event{$n}->{'start_time'}
                                =~ /^([0-9]{2})([0-9]{2})/ )
                            {
                                $start_time = sprintf( "%.2d", $1 )
                                    . sprintf( "%.2d", $2 );
                                $start = sprintf( "%.2d", $1 )
                                    . sprintf( "%.2d",
                                    ( ( $2 >= 30 ) ? 30 : 0 ) );
                                $start_min = $1 * 60 + $2;
                            }
                            if (  !$next
                                && $event{$n}->{'end_time'}
                                && $event{$n}->{'end_time'}
                                =~ /^([0-9]{2})([0-9]{2})/ )
                            {
                                $end_time = sprintf( "%.2d", $1 )
                                    . sprintf( "%.2d", $2 );
                                $end_min = $1 * 60 + $2;
                            }
                        SAMEEVENT:
                            foreach my $span (
                                split(
                                    /[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,
                                    $setting::event_category{$category}
                                        {'availableTime'}
                                )
                                )
                            {
                                if ( $span
                                    =~ /^([0-9]{1,2}):([0-9]{2})-([0-9]{1,2}):([0-9]{2})/
                                    )
                                {
                                    my ( $f, $t )
                                        = ( ( $1 * 60 + $2 ),
                                        ( $3 * 60 + $4 ) );
                                    my $s = sprintf( "%.2d", $1 )
                                        . sprintf( "%.2d",
                                        ( ( $2 >= 30 ) ? 30 : 0 ) );
                                    my $st = sprintf( "%.2d", $1 )
                                        . sprintf( "%.2d", $2 );
                                    my $et = sprintf( "%.2d", $3 )
                                        . sprintf( "%.2d", $4 );
                                    if ( $start_min < $t && $end_min > $f ) {
                                        my $num
                                            = $n
                                            . (
                                            ($count) ? '.' . $count : '' );
                                        $start = $s if $f > $start_min;
                                        $this_event
                                            = $date{$ymd}{'time'}{$start}
                                            {$num} = {};
                                        $this_event->{'start_time'}
                                            = ( $f < $start_min )
                                            ? $start_time
                                            : $st;
                                        $this_event->{'end_time'}
                                            = ( $end_min <= $t )
                                            ? $end_time
                                            : $et;
                                        $this_event->{'start_min'}
                                            = ( $f < $start_min )
                                            ? $start_min
                                            : $f;
                                        $this_event->{'end_min'}
                                            = ( $end_min <= $t )
                                            ? $end_min
                                            : $t;
                                        $this_event->{'prev_time'}
                                            = ($count) ? 1 : 0;
                                        $this_event->{'next_time'}
                                            = ( $end_min > $t ) ? 1 : 0;
                                        $this_event->{'prev'} = $prev;
                                        $this_event->{'next'} = $next;
                                        $event{$num}          = $event{$n}
                                            if $num ne $n;
                                        last SAMEEVENT if $end_min <= $t;
                                        $start_min = $t;
                                        $count++;
                                    }
                                }
                            }
                            $this_event->{'next_time'} = ($next) ? 1 : 0
                                if ref $this_event;
                        }
                        else {
                            $date{$ymd}{'date'}{$n}{'prev'} = $prev;
                            $date{$ymd}{'date'}{$n}{'next'} = $next;
                            $date{$ymd}{'date'}{$n}{'span'}
                                = $this->date_span( $y, $m, $d, $next,
                                \@available_wday, $holiday )
                                if @available_wday;
                        }

                    }
                    elsif ( $event{$n}->{'start_time'} ) {
                        my $start     = "0000";
                        my $start_min = 0;
                        my $end_min   = 1440;
                        if (  !$prev
                            && $event{$n}->{'start_time'}
                            =~ /^([0-9]{2})([0-9]{2})/ )
                        {
                            my ( $h, $min ) = ( $1 + 0, $2 + 0 );
                            $start
                                = sprintf( "%.2d", $h )
                                . sprintf( "%.2d",
                                ( ( $min >= 30 ) ? 30 : 0 ) );
                            $start_min = $h * 60 + $min;
                        }
                        if (  !$next
                            && $event{$n}->{'end_time'}
                            =~ /^([0-9]{2})([0-9]{2})/ )
                        {
                            $end_min = $1 * 60 + $2;
                        }
                        $date{$ymd}{'time'}{$start}{$n}{'start_min'}
                            = $start_min;
                        $date{$ymd}{'time'}{$start}{$n}{'end_min'} = $end_min;
                        $date{$ymd}{'time'}{$start}{$n}{'prev'}    = $prev;
                        $date{$ymd}{'time'}{$start}{$n}{'next'}    = $next;
                    }
                    else {
                        $date{$ymd}{'date'}{$n}{'prev'} = $prev;
                        $date{$ymd}{'date'}{$n}{'next'} = $next;
                        $date{$ymd}{'date'}{$n}{'span'}
                            = $this->date_span( $y, $m, $d, $next,
                            \@available_wday, $holiday )
                            if @available_wday;
                    }
                }
            }
        }
    }
    undef @search_fields;

    #DBの接続を解除する
    while ( my ( $k, $db ) = each %db ) {
        untie(%$db);
    }
    undef %db;

    #---------------------------------------------------------------------
    #書き出し
    $this->{'event'} = \%event;
    $this->{'view'}->set_event_data( \%date, \%event );
    $this->print_calendar;
}

sub date_span ($$$$$) {
    my ( $this, $y, $m, $d, $next, $available_wday_array, $holiday ) = @_;
    my $span = 1;
    while ( $span <= $next ) {
        last
            if !$this->is_available_wday( $available_wday_array,
            $this->get_next_date_array( $y, $m, $d, $span ), $holiday );
        $span++;
    }
    return ( $span - 1 );
}

######################################################################
# 出力ひな型
sub print_calendar ($) {
    my $this = shift;
    $this->use_icon('calendar');
    $this->use_deco();

    #ヘッダ
    $this->set_style_sheet_link("$env::css/calendar-common.css");
    if ( $main::v eq 'l' ) {
        $this->push_title( $this->{'view'}->page_title($main::date) );
    }
    elsif ( !$main::srch ) {
        $this->set_style_sheet_link("$default::file/event-category.css");
        if ( $setting::page{'viewer'}{'customColorEnabled'} ) {
            my $css = '';
            $css
                .= 'table#daytable, table#daytable td, div#timetable-box, table#timetable-grid, table#timetable-grid td, div.before-half, div.after-half {  border-color: '
                . $setting::page{'viewer'}{'cellBorderColor'}
                . ";  }\x0A"
                if $setting::page{'viewer'}{'cellBorderColor'};
            $css
                .= 'table#daytable td, div#timetable-box table td {  background-color: '
                . $setting::page{'viewer'}{'cellBgcolor'}
                . ";  }\x0A"
                if $setting::page{'viewer'}{'cellBgcolor'};
            $css
                .= 'div.cal-date {'
                . ( ( $setting::page{'viewer'}{'cellDateBgcolor'} )
                ? '  background-color: '
                    . $setting::page{'viewer'}{'cellDateBgcolor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'cellDateColor'} )
                ? '  color: '
                    . $setting::page{'viewer'}{'cellDateColor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'cellDateBorderColor'} )
                ? '  border-color: '
                    . $setting::page{'viewer'}{'cellDateBorderColor'} . ';'
                : '' )
                . "  }\x0A"
                if $setting::page{'viewer'}{'cellDateBgcolor'}
                || $setting::page{'viewer'}{'cellDateColor'}
                || $setting::page{'viewer'}{'cellDateBorderColor'};
            $css
                .= 'div.cal-date a {  color: '
                . $setting::page{'viewer'}{'cellDateColor'}
                . ";  }\x0A"
                if $setting::page{'viewer'}{'cellDateColor'};
            $css
                .= 'td.calsaturday {  background-color: '
                . $setting::page{'viewer'}{'saturdayBgcolor'}
                . " !important;  }\x0A"
                if $setting::page{'viewer'}{'saturdayBgcolor'};
            $css
                .= 'td.calsaturday div.cal-date {'
                . ( ( $setting::page{'viewer'}{'saturdayDateBgcolor'} )
                ? '  background-color: '
                    . $setting::page{'viewer'}{'saturdayDateBgcolor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'saturdayDateColor'} )
                ? '  color: '
                    . $setting::page{'viewer'}{'saturdayDateColor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'saturdayDateBorderColor'} )
                ? '  border-color: '
                    . $setting::page{'viewer'}{'saturdayDateBorderColor'}
                    . ';'
                : '' )
                . "  }\x0A"
                if $setting::page{'viewer'}{'saturdayDateBgcolor'}
                || $setting::page{'viewer'}{'saturdayDateColor'}
                || $setting::page{'viewer'}{'saturdayDateBorderColor'};
            $css
                .= 'td.calsaturday div.cal-date a {  color: '
                . $setting::page{'viewer'}{'saturdayDateColor'}
                . ";  }\x0A"
                if $setting::page{'viewer'}{'saturdayDateColor'};
            $css
                .= 'td.calsunday {  background-color: '
                . $setting::page{'viewer'}{'sundayBgcolor'}
                . " !important;  }\x0A"
                if $setting::page{'viewer'}{'sundayBgcolor'};
            $css
                .= 'td.calsunday div.cal-date {'
                . ( ( $setting::page{'viewer'}{'sundayDateBgcolor'} )
                ? '  background-color: '
                    . $setting::page{'viewer'}{'sundayDateBgcolor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'sundayDateColor'} )
                ? '  color: '
                    . $setting::page{'viewer'}{'sundayDateColor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'sundayDateBorderColor'} )
                ? '  border-color: '
                    . $setting::page{'viewer'}{'sundayDateBorderColor'} . ';'
                : '' )
                . "  }\x0A"
                if $setting::page{'viewer'}{'sundayDateBgcolor'}
                || $setting::page{'viewer'}{'sundayDateColor'}
                || $setting::page{'viewer'}{'sundayDateBorderColor'};
            $css
                .= 'td.calsunday div.cal-date a {  color: '
                . $setting::page{'viewer'}{'sundayDateColor'}
                . ";  }\x0A"
                if $setting::page{'viewer'}{'sundayDateColor'};
            $css
                .= 'td.calholiday {  background-color: '
                . $setting::page{'viewer'}{'holidayBgcolor'}
                . " !important;  }\x0A"
                if $setting::page{'viewer'}{'holidayBgcolor'};
            $css
                .= 'td.calholiday div.cal-date {'
                . ( ( $setting::page{'viewer'}{'holidayDateBgcolor'} )
                ? '  background-color: '
                    . $setting::page{'viewer'}{'holidayDateBgcolor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'holidayDateColor'} )
                ? '  color: '
                    . $setting::page{'viewer'}{'holidayDateColor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'holidayDateBorderColor'} )
                ? '  border-color: '
                    . $setting::page{'viewer'}{'holidayDateBorderColor'} . ';'
                : '' )
                . "  }\x0A"
                if $setting::page{'viewer'}{'holidayDateBgcolor'}
                || $setting::page{'viewer'}{'holidayDateColor'}
                || $setting::page{'viewer'}{'holidayDateBorderColor'};
            $css
                .= 'td.calholiday div.cal-date a {  color: '
                . $setting::page{'viewer'}{'holidayDateColor'}
                . ";  }\x0A"
                if $setting::page{'viewer'}{'holidayDateColor'};
            $css
                .= 'td.caltoday {  background-color: '
                . $setting::page{'viewer'}{'todayBgcolor'}
                . " !important;  }\x0A"
                if $setting::page{'viewer'}{'todayBgcolor'};
            $css
                .= 'td.caltoday div.cal-date {'
                . ( ( $setting::page{'viewer'}{'todayDateBgcolor'} )
                ? '  background-color: '
                    . $setting::page{'viewer'}{'todayDateBgcolor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'todayDateColor'} )
                ? '  color: '
                    . $setting::page{'viewer'}{'todayDateColor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'todayDateBorderColor'} )
                ? '  border-color: '
                    . $setting::page{'viewer'}{'todayDateBorderColor'} . ';'
                : '' )
                . "  }\x0A"
                if $setting::page{'viewer'}{'todayDateBgcolor'}
                || $setting::page{'viewer'}{'todayDateColor'}
                || $setting::page{'viewer'}{'todayDateBorderColor'};
            $css
                .= 'td.caltoday div.cal-date a {  color: '
                . $setting::page{'viewer'}{'todayDateColor'}
                . ";  }\x0A"
                if $setting::page{'viewer'}{'todayDateColor'};
            $css
                .= 'div.monthly-calendar table#daytable td.calextra {  background-color: '
                . $setting::page{'viewer'}{'extraBgcolor'}
                . " !important;  }\x0A"
                if $setting::page{'viewer'}{'extraBgcolor'};
            $css
                .= 'div.monthly-calendar table#daytable td.calextra div.cal-date {'
                . ( ( $setting::page{'viewer'}{'extraDateBgcolor'} )
                ? '  background-color: '
                    . $setting::page{'viewer'}{'extraDateBgcolor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'extraDateColor'} )
                ? '  color: '
                    . $setting::page{'viewer'}{'extraDateColor'} . ';'
                : '' )
                . ( ( $setting::page{'viewer'}{'extraDateBorderColor'} )
                ? '  border-color: '
                    . $setting::page{'viewer'}{'extraDateBorderColor'} . ';'
                : '' )
                . "  }\x0A"
                if $setting::page{'viewer'}{'extraDateBgcolor'}
                || $setting::page{'viewer'}{'extraDateColor'}
                || $setting::page{'viewer'}{'extraDateBorderColor'};
            $css
                .= 'div.monthly-calendar table#daytable td.calextra div.cal-date a {  color: '
                . $setting::page{'viewer'}{'extraDateColor'}
                . ";  }\x0A"
                if $setting::page{'viewer'}{'extraDateColor'};
            $css
                .= 'table#timetable-grid td.time-scale div.hourly-list {  color: '
                . $setting::page{'viewer'}{'timeColor'}
                . ";  }\x0A"
                if $setting::page{'viewer'}{'timeColor'};
            $this->set_css($css);
        }
        my ( $y, $m )
            = ( $main::date =~ /^([0-9]{4})([0-9]{2})/ )
            ? ( $1 + 0, $2 + 0 )
            : ( 'null', 'null' );
        my $event_category_json = '';
        if (   %setting::event_category
            && ref $setting::event_category{'ORDER'}
            && %{ $setting::event_category{'ORDER'} } )
        {
            my @selected_category = ();
            if (   $field::for{'category'}
                && %main::search
                && ref $main::search{ $field::for{'category'} }
                && defined $main::search{ $field::for{'category'} }{'str'} )
            {
                $this->scalar_to_array(
                    \$main::search{ $field::for{'category'} }{'str'} )
                    if !ref $main::search{ $field::for{'category'} }{'str'};
                @selected_category = grep( $_ ne '',
                    @{ $main::search{ $field::for{'category'} }{'str'} } );
            }
            foreach my $o ( sort { $a <=> $b }
                keys %{ $setting::event_category{'ORDER'} } )
            {
                my $id = $setting::event_category{'ORDER'}{$o};
                next
                    if ( $setting::event_category{$id}{'closed'}
                    && !$this->is_secure('closed') )
                    || ( @selected_category
                    && !grep( $_ eq $id, @selected_category ) );
                $event_category_json .= ', ' if $event_category_json;
                $event_category_json
                    .= "{id : '"
                    . $id
                    . "', name : '"
                    . (
                    (   defined $setting::event_category{$id}{'name'}
                            && $setting::event_category{$id}{'name'} ne ''
                    )
                    ? $setting::event_category{$id}{'name'}
                    : 'カテゴリー' . $id
                    ) . "'}";
            }
        }
        else {
            $event_category_json = "{id : '1', name : '全イベント'}";
        }
        my $option
            = (%main::search)
            ? $this->set_query( 'url', [ 'search', 'and' ], 1 )
            : '';
        my $c = ( $default::subdivided && $main::c ) ? $main::c : '';
        my $fixed
            = ( $setting::page{$setting::key}{'viewFixed'} )
            ? 'true'
            : 'false';
        my $javascript = <<JAVASCRIPT;
cmonos.tool.calendar.current = {
	year : $y,
	mon : $m,
	view : '$main::v',
	fixed : $fixed,
	option : '$option',
	calendarCategory : '$c',
	category : [$event_category_json]
};
JAVASCRIPT
        $this->set_javascript($javascript);
        $this->set_onload(
                  'cmonos.tool.calendar.restore(\''
                . $cgi::path
                . '?m=Viewer'
                . (
                ( $default::subdivided && $main::c )
                ? '&amp;c=' . $main::c
                : ''
                )
                . '\');'
        );
        $this->set_onload(
            'cmonos.tool.calendar.setUp('
                . (
                ( defined $setting::page{$setting::key}{'startTime'} )
                ? $setting::page{$setting::key}{'startTime'}
                : ''
                )
                . ');'
        ) if $this->{'view'}->have_timetable;
        $this->push_title( $this->{'view'}->page_title($main::date) );
        $this->push_title(
            ( ($default::english) ? 'Search Results' : '検索結果' ) )
            if %main::search && !$main::srch;
    }
    $this->set_javascript_link("$env::javascript/cal.js");
    $this->print_page_header( $setting::key, undef, undef, undef, undef,
        undef, undef, undef,
        'cal-' . $this->{'view'}->calendar_type . '-view' );

    #アイコン
    $this->print_tool_button('top')
        if !$setting::page{$setting::key}{'toolButtonsDisabled'};

    #イベントリスト
    $this->{'view'}->print_events;

    #戻る
    $this->print_return_button;

    #フッタ
    $this->print_page_footer($setting::key);
}

sub push_title ($$) {
    my ( $this, $title ) = @_;
    return if !defined $title || $title eq '';
    $this->push_window_title($title);
    if ( defined $setting::page{$setting::key}{'titleText'}
        && $setting::page{$setting::key}{'titleText'} ne '' )
    {
        my $tag = ( $setting::page{$setting::key}{'titleText'}
                =~ s/((<\/[^>]+>)+)$// ) ? $1 : '';
        $setting::page{$setting::key}{'titleText'}
            .= ' (' . $title . ')' . $tag;
    }
}

######################################################################
# 表示切り替えボタン
sub view_switch_button ($$;$$$$$$$) {
    my ($this,     $icon,    $title,  $url, $query,
        $li_class, $onclick, $target, $is_selected
    ) = @_;
    my $button;
    $button
        = '<li'
        . ( ( defined $li_class && $li_class ne '' )
        ? ' class="' . $li_class . '"'
        : '' )
        . '>';
    if ( defined $url && $url ne '' ) {
        $url .= $this->amp_escape( $this->add_query( 'url', $query ) )
            if ref $query;
        $button .= $cgi::icon->button(
            $icon,
            $setting::page{'viewer'}{$icon}
                . ( ($is_selected) ? '-selected' : '' ),
            $url,
            $title,
            $target,
            ( ($onclick) ? { 'onclick' => $onclick } : '' )
        );
    }
    else {
        if ($target) {
            if ( $onclick =~ /^return/ ) {
                $onclick =~ s/;$/ \&\& /;
                $onclick .= '(this.form.target=\'' . $target . '\');';
            }
            else {
                $onclick .= ';' if $onclick;
                $onclick .= 'this.form.target=\'' . $target . '\';';
            }
        }
        $button .= $cgi::icon->submit_button(
            $icon,
            $setting::page{'viewer'}{$icon}
                . ( ($is_selected) ? '-selected' : '' ),
            $title,
            ( ($onclick) ? { 'onclick' => $onclick } : '' ),
            $query
        );
    }
    $button .= '</li>';
    return $button;
}

######################################################################
# アイコンの表示
sub print_tool_button ($;$) {
    my ( $this, $name ) = @_;
    my ( $url, $viewform_flag, $query );
    my @form = @setting::form;
    push( @form, 'date' );
    if ( $setting::general{'postMethod'} ) {
        unshift( @form, 'n' ) if $main::v eq 'listevent' && $main::n;
        print $this->form_start(
            'key'      => 'browse',
            'name'     => 'view_' . $name,
            'required' => \@form
        );
        $viewform_flag = 1;
    }
    else {
        $url = $cgi::path
            . $this->amp_escape( $this->set_query( 'url', \@form ) );
    }
    undef @form;
    print '<ul class="tools panel" id="view-selector-' . $name . '">';

    # ICSファイル
    if ( $setting::page{'viewer'}{'icsLink'} ) {
        print '<li class="group-end"><a href="'
            . $cgi::path
            . '?m=Ics'
            . (
            ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
            . '" id="ics-download" title="iCalendar 形式でダウンロード">';
        print '<img src="'
            . $env::icon_dir
            . '/ics_16.png" width="16" height="16" border="0" title="iCalendar 形式でダウンロード" alt="iCalendar 形式でダウンロード" />';
        print '</a></li>';
    }

    # 前へ/インデックス/検索条件をクリア/今日/次へ
    print $this->view_switch_button(
        'calprev', '前のカレンダー',
        $url, { 'v' => $main::v, 'prev' => 1 }
    );
    print $this->view_switch_button( 'calindex', 'インデックス',
        $cgi::path )
        if $default::subdivided;
    print $this->view_switch_button(
        'calreset', '検索条件をリセット',
        $url, { 'v' => $main::v, 'reset' => 1 }
    ) if %main::search;
    print $this->view_switch_button( 'caltoday', '今日', $url,
        { 'v' => $main::v, 'today' => 1 } );
    print $this->view_switch_button(
        'calnext', '次のカレンダー',
        $url, { 'v' => $main::v, 'next' => 1 }
    );

    # 日/週/月
    if ( !$setting::page{'viewer'}{'viewFixed'} ) {
        print $this->view_switch_button( 'daily', '日', $url, { 'v' => 'd' },
            'group-start', '', '',
            ( ( !$main::srch && $main::v eq 'd' ) ? 1 : 0 ) );
        print $this->view_switch_button( 'weekly', '週', $url,
            { 'v' => 'w' },
            '', '', '', ( ( !$main::srch && $main::v eq 'w' ) ? 1 : 0 ) );
        print $this->view_switch_button( 'monthly', '月', $url,
            { 'v' => 'm' },
            '', '', '', ( ( !$main::srch && $main::v eq 'm' ) ? 1 : 0 ) );
    }

    # //検索//カレンダーパネル//トグルパネル
    print $this->view_switch_button( 'calsearch', '検索', $url,
        { 'v' => $main::v, 'srch' => 1 },
        'group-start', '', '', ( ($main::srch) ? 1 : 0 ) );
    if (   $main::v ne 's'
        && $main::v ne 'l'
        && (  !$ENV{'HTTP_USER_AGENT'}
            || $ENV{'HTTP_USER_AGENT'} !~ /mac/i
            || $ENV{'HTTP_USER_AGENT'} !~ /MSIE/i )
        )
    {
        print $this->view_switch_button(
            'calendarpanel',
            'カレンダーパネル',
            'javascript:void(0)',
            '',
            'group-start',
            'cmonos.tool.calendar.calendarPalette.open(\''
                . $cgi::path
                . '?m=Viewer'
                . (
                ( $default::subdivided && $main::c )
                ? '&amp;c=' . $main::c
                : ''
                )
                . '\');'
        );
        print $this->view_switch_button(
            'togglepanel',
            '表示コントロールパネル',
            'javascript:void(0)',
            '',
            'group-start',
            'cmonos.tool.calendar.togglePanel.open();'
        );
    }

    # 新規投稿
    if ( $this->is_secure('input') ) {
        print $this->view_switch_button(
            'calinput',
            '新しい予定を入力する',
            $cgi::path
                . '?m=Editor&amp;k=editor'
                . (
                ( $default::subdivided && $main::c )
                ? '&amp;c=' . $main::c
                : ''
                )
                . '&amp;v='
                . $main::v
                . '&amp;dtstart='
                . $main::date,
            '',
            'group-start',
            $this->detail_viewer_opener,
            'detailViewer',
        );
    }

    # ツール
    if ( $this->is_secure('tools') ) {
        print $this->view_switch_button(
            'caltool',
            '環境設定を開く',
            $cgi::path
                . '?m=Tools'
                . ( ( defined $main::c ) ? '&amp;c=' . $main::c : '' ),
            '',
            'group-start'
        );
    }

    # パスワード操作
    if ( defined $setting::page{$setting::key}{'pswdIcon'}
        && $setting::page{$setting::key}{'pswdIcon'} )
    {
        print $this->view_switch_button(
            'callogin',
            'ログイン',
            $env::register_cgi . '?k=pswd&amp;m=Password',
            '',
            'group-start',
            (     ($env::register_in_same_window)
                ? ()
                : ( "cmonos.window.open('LoginWindow',450,300)",
                    'LoginWindow',
                )
            ),
        );
    }

    print '</ul>';
    print '</form>' if defined $viewform_flag && $viewform_flag;
}

######################################################################
# 戻るボタン
sub print_return_button ($) {
    my $this = shift;
    my ($url);
    if ( defined $setting::page{'returnUrl'} && $setting::page{'returnUrl'} )
    {
        $url = $setting::page{'returnUrl'};
    }
    elsif ( defined $default::subdivided && $default::subdivided ) {
        $url = $cgi::path;
    }
    else {
        return;
    }
    print '<div id="return-to" class="panel">';
    print $cgi::Deco->button( 'return', $setting::page{'returnButton'},
        $url, ( ($default::english) ? 'Return' : '戻る' ) );
    print '</div>';
}

######################################################################
# ヘッダ・フッタ
sub _page_item_replace ($$;$$$$) {
    my ( $this, $str, $file_flag, $output, $title, $info ) = @_;
    $$str
        =~ s/<!--(if|unless)([\x09\x0A\x0C\x0D\x20].*?-->.*?<!--\/\1(?![0-9a-zA-Z_]).*?-->)/$this->if_replace($1,'<!--' . $1 . $2)/iges;
    $$str
        =~ s/<\!--monthname(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/monthname(?![0-9a-zA-Z_])[^>]*-->/$this->monthname($1,$2)/iges;
    $$str
        =~ s/<\!--yearname(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/yearname(?![0-9a-zA-Z_])[^>]*-->/$this->yearname($1,$2)/iges;
    $$str
        =~ s/<\!--monthlist(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/monthlist(?![0-9a-zA-Z_])[^>]*-->/$this->monthlist($1,$2)/iges;
    $$str
        =~ s/<\!--linktolist(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/linktolist(?![0-9a-zA-Z_])[^>]*-->/$this->linktolist($1,$2)/iges;
    $$str
        =~ s/<\!--linktocalendar(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/linktocalendar(?![0-9a-zA-Z_])[^>]*-->/$this->linktocalendar($1,$2)/iges;
    $$str
        =~ s/<\!--list(?!\w)([^<>]*?)-->(.*?)<!--\/list(?!\w)[^<>]*?-->/$this->event_list($1,$2,$this->{'event'})/iges;
    $$str
        =~ s/<\!--return(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/return(?![0-9a-zA-Z_])[^>]*-->/$this->return_link($1,$2)/iges;
}

sub return_link ($$$) {
    my ( $this, $attributes, $content ) = @_;
    $content
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+text="([^"]+)"/i )
        ? $1
        : ''
        if !$content;
    my $class
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+class="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    my $id
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+id="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    return
          '<a href="'
        . $cgi::path
        . '?m=Viewer'
        . ( ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
        . '&amp;date='
        . $main::date
        . $this->amp_escape(
        $this->set_query( 'url', [ 'and', 'search' ], 1 ) )
        . '"'
        . ( ($id)    ? ' id="' . $this->escape($id) . '"'       : '' )
        . ( ($class) ? ' class="' . $this->escape($class) . '"' : '' ) . '>'
        . ( $content || '戻る' ) . '</a>';
}

sub if_replace ($$$) {
    my ( $this, $tag, $str ) = @_;
    my $is_matched = 0;
    $str
        =~ s/<!--(if|unless|else?if|else?unless|else)(?=[\x09\x0A\x0C\x0D\x20\-])(.*?)-->(.*?.?)(?=<!--(?:\/\1|else?if|else?unless|else|\/$tag)(?![0-9a-zA-Z_]).*?-->)/$this->_if_replace($1,$2,$3,\$is_matched)/iges;
    return $str;
}

sub _if_replace ($$$$$) {
    my ( $this, $tag, $attributes, $value, $is_matched_ref ) = @_;
    $value = $this->unescape($1)
        if $attributes =~ s/[\x09\x0A\x0C\x0D\x20]value="(.*?)"//si;
    $tag = lc $tag;
    if ($$is_matched_ref) {
        $value = '';
    }
    else {
        my $is_matched = 0;
        if ( $tag eq 'else' ) {
            $is_matched = 1 if !$$is_matched_ref;
        }
        elsif ( !$is_matched
            && $attributes =~ /[\x09\x0A\x0C\x0D\x20]view="(.*?)"/i )
        {
            my @view = map { substr( $_, 0, 1 ) }
                split( /\W/, lc $this->unescape($1) );
            my $v
                = ( ref $this->{'view'} )
                ? substr( $this->{'view'}->calendar_type, 0, 1 )
                : 'w';
            $is_matched = ( grep( $_ eq $v, @view ) ) ? 1 : 0;
            $is_matched = ($is_matched) ? 0 : 1 if $tag =~ /unless/;
        }
        if ($is_matched) {
            $$is_matched_ref = 1;
        }
        else {
            $value = '';
        }
    }
    return $value;
}

sub linktolist ($$$) {
    my ( $this, $attributes, $content ) = @_;
    $content
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+text="([^"]+)"/i )
        ? $this->unescape($1)
        : ''
        if !$content;
    my $class
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+class="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    my $id
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+id="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    return
          '<a href="'
        . $cgi::path
        . '?m=Viewer&amp;v=l'
        . ( ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
        . '&amp;date='
        . $main::date
        . $this->amp_escape(
        $this->set_query( 'url', [ 'and', 'search' ], 1 ) )
        . '"'
        . ( ($id)    ? ' id="' . $this->escape($id) . '"'       : '' )
        . ( ($class) ? ' class="' . $this->escape($class) . '"' : '' ) . '>'
        . ( $content || 'リスト' ) . '</a>';
}

sub linktocalendar ($$$) {
    my ( $this, $attributes, $content ) = @_;
    $content
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+text="([^"]+)"/i )
        ? $this->unescape($1)
        : ''
        if !$content;
    my $class
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+class="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    my $id
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+id="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    return
          '<a href="'
        . $cgi::path
        . '?m=Viewer'
        . ( ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
        . '&amp;date='
        . $main::date
        . $this->amp_escape(
        $this->set_query( 'url', [ 'and', 'search' ], 1 ) )
        . '"'
        . ( ($id)    ? ' id="' . $this->escape($id) . '"'       : '' )
        . ( ($class) ? ' class="' . $this->escape($class) . '"' : '' ) . '">'
        . ( $content || 'カレンダーに戻る' ) . '</a>';
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {

        #設定DB読み込み設定
        $setting::general  = 1;    #一般設定読み込み
        $setting::deco     = 1;    #テキスト装飾設定読み込み
        $setting::security = 1;    #アクセス権設定読み込み
        $setting::table    = 1;    #テーブル設定読み込み
        $setting::page     = 1;    #ページ設定読み込み

        #クッキー
        $cookie::id_enabled = 1;    #ID情報クッキー読み込み

        #フィールド読み込み設定
        $setting::field_db_name = 'inputfield';    #フィールド設定DB
        $setting::field         = 2
            ; #フィールド設定読み込み（2=オリジナルも読み込む）
        $setting::field_style_disabled = 0
            ; #2=フォント設定スタイル設定を無視/1=フォント設定のみ無視
        $setting::field_tag_enabled = 'no'
            ;   #tag=タグのみ/all=全ての設定/no=タグをつけない

        #必須フォーム
        @setting::form = ( 'm', 'k', 'and', 'search' ) if !@setting::form;

        #デフォルト画面の調整
        if ( $default::subdivided && defined $main::c && $main::c ne '' ) {
            unshift( @setting::form, 'c' );
        }

        #イメージCSS
        $setting::image_css = 'calendar-image.css';

        #各種キー設定
        $setting::security_key = 'browse'
            if !defined $setting::security_key
            || $setting::security_key eq
            '';    #デフォルトセキュリティキー

        #ページ設定キー
        $setting::key = 'viewer'
            if !defined $setting::key || $setting::key eq '';

        #デフォルトキー
        $main::k = 'viewer' if !defined $main::k || !$main::k;

        #クラス登録
        Common::class_register( 'viewer', 'Viewer' );

    }
    else {
        $main::m = $default::m = 'Index';
        $main::k = $default::k = 'index';
        require CMONOS::Calendar::Index;
    }
}

1;
