package DayCounter;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# 日数カウントモジュール
# DayCounter.pm ver1.0.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-02-17
######################################################################
# メイン
sub run ($;$) {
    my ( $this, $mkfile_flag ) = @_;
    my ($mday, $mon,  $year,     %specialday,
        %temp, %cell, %date,     $id,
        $cal,  $m,    $calendar, %counter
    );
    local (*OUT);
    $calendar = '';

    #ページヘッダ
    if ( defined $mkfile_flag && $mkfile_flag ) {
        if ( defined $setting::security{'dcount'}{'passwordCheck'}
            && $setting::security{'dcount'}{'passwordCheck'} eq 'nocheck' )
        {
            if ( open( OUT, ">>$default::file" ) ) {
                binmode OUT, ":utf8";
                flock( OUT, LOCK_EX );
                truncate( OUT, 0 );
                seek( OUT, 0, 0 );
                $main::mode = 'ssi';
            }
            else {
                $this->print_invisible_gif;
                return;
            }
        }
        else {
            $this->write_file( $default::file, '', 1 );
            $this->write_file( $default::file, '', 1 )
                if $default::file =~ s/\.ssi$/.js/;
            $this->print_invisible_gif;
            return;
        }
    }
    elsif ( defined $main::mode && $main::mode eq 'ssi' ) {
        binmode STDOUT;
        $this->print_http_header('text/html');
        *OUT = *STDOUT;
    }
    elsif ( defined $main::mode && $main::mode eq 'js' ) {
        binmode STDOUT;
        $this->print_http_header('text/javascript');
        *OUT = *STDOUT;
    }

    $this->import_db( $default::preference_dir . '/counter', \%counter, 1 )
        if $default::preference_dir;
    ( $mday, $mon, $year )
        = ( localtime( time + $default::advance * 3600 ) )[ 3 .. 5 ];
    $mon++;
    $year += 1900;

    #カレンダーオブジェクト
    require CMONOS::CGI_Lib::Calendar;
    require CMONOS::CGI_Lib::Holiday;
    $cal = Calendar->new(
        'jcode' => 'utf8',                               # utf8/euc/jis/sjis
        'css'   => $setting::page{'calendar'}{'css'},    # 1=css
        'bg'    => $setting::page{'calendar'}{'bg'}
        ,    # 1=テーブルの背面を反映させる
        'cellpadding' => $setting::page{'calendar'}{'cellpadding'}
        ,    # cellpadding
        'cellspacing' => $setting::page{'calendar'}{'cellspacing'}
        ,    # cellspacing
        'border'      => $setting::page{'calendar'}{'border'},       # border
        'bordercolor' => $setting::page{'calendar'}{'bordercolor'}
        ,    # bordercolor
        'cellwidth'  => $setting::page{'calendar'}{'cellwidth'},   # セル幅
        'cellheight' => $setting::page{'calendar'}{'cellheight'}
        ,    # セル高さ
        'cellalign'  => $setting::page{'calendar'}{'cellalign'}, # セルalign
        'cellvalign' => $setting::page{'calendar'}{'cellvalign'}
        ,    # セルvalign
        'bgcolor'   => $setting::page{'calendar'}{'bgcolor'},  # 標準bgcolor
        'fontcolor' => $setting::page{'calendar'}{'fontcolor'}
        ,    # 標準文字色
        'fontsize' => $setting::page{'calendar'}{'fontsize'}
        ,    # 標準文字サイズ
        'style' => $setting::page{'calendar'}{'style'}
        ,    # 標準文字スタイル
        'bold' => $setting::page{'calendar'}{'bold'},    # 標準文字太字
        'headerfontcolor' => $setting::page{'calendar'}{'headerfontcolor'}
        ,                                                # 月名文字色
        'headerfontsize' => $setting::page{'calendar'}{'headerfontsize'}
        ,    # 月名文字サイズ
        'headerstyle' => $setting::page{'calendar'}{'headerstyle'}
        ,    # 月名文字スタイル
        'headerbold' => $setting::page{'calendar'}{'headerbold'}
        ,    # 月名文字太字
        'thbgcolor' => $setting::page{'calendar'}{'thbgcolor'}
        ,    # 曜日bgcolor
        'thfontcolor' => $setting::page{'calendar'}{'thfontcolor'}
        ,    # 曜日文字色
        'thfontsize' => $setting::page{'calendar'}{'thfontsize'}
        ,    # 曜日文字サイズ
        'thstyle' => $setting::page{'calendar'}{'thstyle'}
        ,    # 曜日文字スタイル
        'thbold' => $setting::page{'calendar'}{'thbold'}, # 曜日文字太字
        'linkstyle' => $setting::page{'calendar'}{'linkstyle'}
        ,    # リンク文字スタイル
        'linkbold' => $setting::page{'calendar'}{'linkbold'}
        ,    # リンク文字太字
        'weekformat' => $setting::page{'calendar'}{'weekformat'},    # 曜日
          #（j=日本語/e=英語/f=フル/u=ucfirst/U=大文字/l=小文字）
        'monthformat' => $setting::page{'calendar'}{'monthformat'},    # 月
          #（j=日本語/J=陰暦月名/e=英語/f=フル/u=ucfirst/U=大文字/l=小文字）
        'yearformat' => $setting::page{'calendar'}{'yearformat'},    # 年
             #（J=年号/w=西暦/j=西暦日本語表記/k=漢字表記）
        'saturdaybgcolor' => $setting::page{'calendar'}{'saturdaybgcolor'}
        ,    # 土曜日bgcolor
        'sundaybgcolor' => $setting::page{'calendar'}{'sundaybgcolor'}
        ,    # 日曜日bgcolor
        'holidaybgcolor' => $setting::page{'calendar'}{'holidaybgcolor'}
        ,    # 休日bgcolor
        'specialdaybgcolor' => $setting::page{'calendar'}{'specialdaybgcolor'}
        ,    # 特別な日bgcolor
        'specialwdaybgcolor' =>
            $setting::page{'calendar'}{'specialwdaybgcolor'}
        ,    # 特別な曜日bgcolor
        'specialwday' => $setting::page{'calendar'}{'specialwday'}
        ,    # 特別な曜日
        'specialwdayexception' =>
            $setting::page{'calendar'}{'specialwdayexception'}
        ,    # 特別な曜日例外
        'specialwdayextra' => $setting::page{'calendar'}
            {'specialwdayextra'}    # 特別な曜日追加
    );

    #表示する場合のヘッダ
    if ( !defined $main::mode || $main::mode eq '' ) {
        $this->set_og_url( $this->my_url );
        $this->set_css( $cal->mk_css )
            if !$setting::page{'calendar'}{'calcssunembedded'};
        $this->print_page_header('dcount');
        *OUT = *STDOUT;
    }
    elsif ( !$setting::page{'calendar'}{'calcssunembedded'} ) {
        $calendar = '<style type="text/css">' . $cal->mk_css . '</style>';
    }
    if ( defined $setting::page{'calendar'}{'specialday'}
        && $setting::page{'calendar'}{'specialday'} )
    {
        while ( $setting::page{'calendar'}{'specialday'}
            =~ /^([0-9]{1,2})[\x20]*\/[\x20]*([0-9]{1,2})[\x20]*=[\x20]*(.*?)$/mg
            )
        {
            $specialday{ $1 + 0 }{ $2 + 0 } = $3;
        }
    }
    if ( defined $setting::page{'calendar'}{'addition'}
        && $setting::page{'calendar'}{'addition'} )
    {
        while ( $setting::page{'calendar'}{'addition'}
            =~ /^([0-9]{1,2})[\x20]*\/[\x20]*([0-9]{1,2})[\x20]*=[\x20]*(.*?)$/mg
            )
        {
            $cell{ $1 + 0 }{ $2 + 0 } = $3;
        }
    }

    while ( ($id) = each %counter ) {
        if ( !ref $counter{$id}{'target'} || !@{ $counter{$id}{'target'} } ) {
            my $temp = $counter{$id}{'target'};
            delete $counter{$id}{'target'};
            if ( defined $temp && $temp ) {
                @{ $counter{$id}{'target'} } = ($temp);
            }
            else {
                @{ $counter{$id}{'target'} } = ( 'exception', 'other' );
            }
        }
        $date{$id} = $this->day_count(
            $counter{$id}{'day'},     $counter{$id}{'title'},
            $counter{$id}{'bgcolor'}, \@{ $counter{$id}{'target'} },
            $cal,                     $mday,
            $mon,                     $year,
            \%specialday,             \%temp
        );
    }
    foreach $m ( keys %temp ) {
        my $d;
        foreach $d ( keys %{ $temp{$m} } ) {
            if ( defined $specialday{$m}{$d} && $specialday{$m}{$d} ne '' ) {
                $specialday{$m}{$d} =~ s/:.*?$//;
                $specialday{$m}{$d} .= '/';
            }
            $specialday{$m}{$d} .= $temp{$m}{$d};
        }
    }
    $calendar .= $cal->calendar( $year, $mon, \%specialday, \%cell );
    if ( $mon == 12 ) {
        $mon = 1;
        $year++;
    }
    else {
        $mon++;
    }
    $calendar .= $cal->calendar( $year, $mon, \%specialday, \%cell );

    if ( ( $main::mode && defined $setting::page{'calendar'}{'template'} )
        || defined $setting::page{'dcount'}{'template'} )
    {
        my $template
            = ($main::mode)
            ? $setting::page{'calendar'}{'template'}
            : $setting::page{'dcount'}{'template'};
        $template = $this->page_item_replace( $template, 0, '',
            ( ($main::mode) ? '' : $setting::page{'dcount'}{'title'} ) );
        if ( $template =~ /<!--frame(?![0-9a-zA-Z_])([^>]*)-->/ ) {
            require CMONOS::CGI_Lib::Frame;
            my $frame = Frame->new;
            $template = $frame->text_to_frame( $template, 'no',
                $frame::obj{'frame'}, '', ( ($mkfile_flag) ? 'abs' : '' ) );
        }
        $template
            =~ s/<\!--daycounter(?![0-9a-zA-Z_])[^<>]*?[\x09\x0A\x0C\x0D\x20]+id[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([0-9]+)"[^<\->]*-->/((defined $date{$1}) ? $date{$1} : '')/ge;
        $template
            =~ s/<\!--daycountertitle(?![0-9a-zA-Z_])[^<>]*?[\x09\x0A\x0C\x0D\x20]+id[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([0-9]+)"[^<\->]*-->/((defined $counter{$1}{'title'}) ? (defined $counter{$1}{'bgcolor'}) ? '<span style="background-color: ' . $counter{$1}{'bgcolor'} . '">' . $counter{$1}{'title'} . '<\/span>' : $counter{$1}{'title'} : '')/ge;
        $template =~ s/<\!--calendar(?![0-9a-zA-Z_])[^<\->]*-->/$calendar/g;
        if ($mkfile_flag) {
            my $mkcal = $ENV{'SCRIPT_NAME'};
            $mkcal =~ s/start.cgi$/mkcal.cgi/;
            $template .= '<img src="' . $mkcal . '" width="1" height="1" />'
                if $template !~ /<img[^>]+src="$mkcal"[^>]*>/i;
        }
        print OUT ( ( $main::mode && $main::mode eq 'js' )
            ? $this->html_to_javascript($template)
            : $template );
    }

    #ページフッタ
    if ($mkfile_flag) {
        close(OUT);
        $this->mk_external_javascript($default::file);
        $this->print_invisible_gif;
    }
    elsif ( !defined $main::mode || $main::mode eq '' ) {
        $this->print_page_footer('dcount');
    }
}

sub day_count ($$$$$$$$$$$) {
    my ($this, $day, $title, $bgcolor,    $target, $cal,
        $mday, $mon, $year,  $specialday, $temp
    ) = @_;

    #@$target = ('exception','other') if !@$target;
    my @month = ( 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
    $month[2]++ if $cal->is_leap_year($year);
    my $holiday_flag     = ( grep( $_ eq 'holiday',     @$target ) ) ? 1 : 0;
    my $specialday_flag  = ( grep( $_ eq 'specialday',  @$target ) ) ? 1 : 0;
    my $specialwday_flag = ( grep( $_ eq 'specialwday', @$target ) ) ? 1 : 0;
    my $exception_flag   = ( grep( $_ eq 'exception',   @$target ) ) ? 1 : 0;
    my $other_flag       = ( grep( $_ eq 'other',       @$target ) ) ? 1 : 0;
    my $count            = 0;
    $day = 0 if !$day;
    $mday--;

    while ( $count <= $day ) {
        $mday++;
        if ( $mday > $month[$mon] ) {
            $mday = 1;
            $mon++;
            if ( $mon > 12 ) {
                $mon = 1;
                $year++;
                @month
                    = ( 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
                $month[2]++ if $cal->is_leap_year($year);
            }
        }
        my $holiday = Holiday->new($year);
        unless ( $exception_flag
            && $cal->is_special_wday_exception( $mon, $mday ) )
        {
            my $checked_flag = 0;
            if ( $holiday->is_holiday( $mon, $mday ) ) {
                $checked_flag = 1;
                next if !$holiday_flag;
            }
            if ( $cal->is_special_wday( $year, $mon, $mday ) ) {
                $checked_flag = 1;
                next if !$specialwday_flag;
            }
            if (   ref $specialday
                && ref $$specialday{$mon}
                && $$specialday{$mon}{$mday} )
            {
                $checked_flag = 1;
                next if !$specialday_flag;
            }
            next if !$other_flag && !$checked_flag;
        }
        $count++;
    }
    $$temp{$mon}{$mday} .= $title if defined $title && $title;
    $$temp{$mon}{$mday} .= ':' . $bgcolor if defined $bgcolor && $bgcolor;
    return $cal->date( $year, $mon, $mday );
}

######################################################################
sub print_invisible_gif ($) {
    my $this = shift;
    binmode STDOUT;
    print 'Content-type: image/gif', "\x0A";
    print 'Content-length: 43',      "\x0A";

    #print 'Cache-Control: no-cache',"\x0A";
    #print 'Pragma: no-cache',"\x0A";
    print "\x0A";
    print
        "GIF89a\x01\x00\x01\x00\x80\x00\x00\xFF\xFF\xFF\x00\x00\x00\x21\xF9\x04\x05\x14\x00\x00\x00\x2C\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00\x3B";
}

######################################################################
sub unauthorized_alert ($) {
    my $this = shift;
    if ($main::m) {
        if ( $main::mode && ( $main::mode eq 'ssi' || $main::mode eq 'js' ) )
        {
            $this->print_http_header('text/javascript');
        }
        else {
            $this->print_password_check_form($setting::key);
        }
    }
    else {
        $this->write_file( $default::file, '', 1 );
        $this->write_file( $default::file, '', 1 )
            if $default::file =~ s/\.ssi$/.js/;
        $this->print_invisible_gif;
    }
    exit;
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #設定DB読み込み設定
    $setting::general = 1 if !$setting::general;    #一般設定読み込み
    $setting::deco = 1
        if !$setting::deco;    #テキスト装飾設定読み込み
    $setting::security = 1
        if !$setting::security;    #アクセス権設定読み込み
    $setting::page = 1 if !$setting::page;    #ページ設定読み込み

    #設定種キー設定
    $setting::security_key = 'dcount'
        if !defined $setting::security_key
        || $setting::security_key ne
        'dcount';    #デフォルトセキュリティキー
    $setting::key    = 'dcount';
    $setting::subkey = 'calendar'
        if !defined $setting::subkey || $setting::subkey ne 'calendar';

    #クッキー
    $cookie::id_enabled = 1
        if !$cookie::id_enabled;    #ID情報クッキー読み込み

    #デフォルトキー
    $main::k = 'dcount' if !defined $main::k || !$main::k;

    #必須フォーム
    @setting::form = ( 'm', 'k' ) if !@setting::form;

    #クラス登録
    Common::class_register( 'dcount', 'DayCounter' );
}

1;
