package CounterEditor;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# 日数カウント設定モジュール
# CounterEditor.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this         = shift;
    my $counter_pref = $default::preference_dir . '/counter';
    my %counter;

    #全体編集書き込み
    if ( defined $main::a && $main::a eq 'e' ) {
        my ($k);

        #不正なデータの除去
        while ( ($k) = each %main::data ) {
            if ( $k =~ /^([0-9]+)_bgcolor$/ ) {
                my $id = $1;
                if ( !defined $main::data{$k} || $main::data{$k} eq '' ) {
                    push(
                        @{ $this->{'dialog'} },
                        'ID:'
                            . $id
                            . ' を更新できませんでした。背景色が設定されていません。'
                    );
                    delete $main::data{$k};
                }
                elsif ( $this->is_invalid_color( $main::data{$k} ) ) {
                    push(
                        @{ $this->{'dialog'} },
                        'ID:'
                            . $id
                            . ' を更新できませんでした。背景色指定を確認してください。'
                    );
                    delete $main::data{$k};
                }
            }
            if ( $k =~ /^([0-9]+)_title$/ ) {
                my $id = $1;
                if ( !defined $main::data{$k} || $main::data{$k} eq '' ) {
                    push(
                        @{ $this->{'dialog'} },
                        'ID:'
                            . $id
                            . ' を更新できませんでした。タイトルが設定されていません。'
                    );
                    delete $main::data{$k};
                }
            }
        }

        #データ削除
        push( @main::delete, $main::delete )
            if defined $main::delete && $main::delete;
        @main::delete
            = (@main::delete)
            ? grep( defined $_ && $_ ne '', @main::delete )
            : ();

        #新規作成
        if ( defined $main::new{'title'} && $main::new{'title'} ne '' ) {
            if ( !defined $main::new{'bgcolor'}
                || $main::new{'bgcolor'} eq '' )
            {
                push(
                    @{ $this->{'dialog'} },
                    '新規設定を追加できませんでした。背景色が設定されていません。'
                );
                delete $main::new{'bgcolor'};
            }
            elsif ( $this->is_invalid_color( $main::new{'bgcolor'} ) ) {
                push(
                    @{ $this->{'dialog'} },
                    '新規設定を追加できませんでした。背景色指定を確認してください。'
                );
                delete $main::new{'bgcolor'};
            }
            else {
                my ( $k, $v, $n );
                while (1) {
                    $n++;
                    last
                        if !defined $main::data{ $n . '_day' }
                        || $main::data{ $n . '_day' } eq '';
                }
                while ( ( $k, $v ) = each %main::new ) {
                    if ( ref $v eq 'ARRAY' ) {
                        @{ $main::data{ $n . '_' . $k } } = @$v;
                    }
                    else {
                        $main::data{ $n . '_' . $k } = $v;
                    }
                }
                undef(%main::new);
            }
        }

        $this->write_db( $counter_pref, \%main::data, '', \@main::delete,
            \%counter, 1 );

    }
    else {
        $this->import_db( $counter_pref, \%counter, 1 );

    }
    undef(%main::data);

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');
    print '<h1>日数カウント設定</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="e" />';
    print '<fieldset><legend>カウンターリスト</legend>';

    #テーブル設定
    require CMONOS::CGI_Lib::Table;
    my $table = Table->new;
    $table->set(
        'tableclass' => 'tooltable',
        'trclass'    => 'odd,even',
        'class0'     => 'odd,even',
        'classF'     => 'caution',
        'thenabled'  => 'true'
    );

    #表
    print $table->table;

    #ヘッダ
    print $table->tr;
    print $table->td('削除');
    print $table->td('ID');
    print $table->td('タイトル');
    print $table->td('カウント');
    print $table->td('カウント対象');
    print $table->td('セル背景色');
    print $table->_tr;

    #設定項目
    my $id;
    foreach $id ( sort { $a <=> $b } keys %counter ) {
        $this->scalar_to_array( \$counter{$id}{'target'} );
        print $table->tr;
        print $table->td( '<input type="checkbox" name="delete" value="'
                . $id
                . '" onclick="cmonos.form.field(this).checkboxConfirm(\''
                . $this->escape( $counter{$id}{'title'} )
                . ' に関連する全てのデータが削除されます。本当に削除しますか？\')" />'
        );
        print $table->td($id);
        print $table->td( '<input type="text" name="data:'
                . $id
                . '_title" value="'
                . $this->escape( $counter{$id}{'title'} )
                . '" size="20" />' );
        print $table->td(
            $this->day_count_menu(
                'data:' . $id . '_day',
                $counter{$id}{'day'}
            )
        );
        print $table->td(
            $this->count_target_menu(
                'data:' . $id . '_target',
                $counter{$id}{'target'}
            )
        );
        print $table->td(
            $this->cell_bgcolor(
                'data:' . $id . '_bgcolor',
                $counter{$id}{'bgcolor'}
            )
        );
        print $table->_tr;
    }

    #新規作成
    $this->scalar_to_array( \$main::new{'target'} );
    print $table->tr;
    print $table->td('');
    print $table->td('新規');
    print $table->td( '<input type="text" name="new:title" value="'
            . $this->escape( $main::new{'title'} )
            . '" size="20" />' );
    print $table->td( $this->day_count_menu( 'new:day', $main::new{'day'} ) );
    print $table->td(
        $this->count_target_menu( 'new:target', $main::new{'target'} ) );
    print $table->td(
        $this->cell_bgcolor( 'new:bgcolor', $main::new{'bgcolor'} ) );
    print $table->_tr;

    print $table->_table;
    undef %$table;
    undef $table;

    print
        '<div class="hints">日数カウントを追加するには新規タイトルを記入します。</div></fieldset>';
    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
    print '</form>';
    if ($main::a) {
        my $mkcal = $ENV{'SCRIPT_NAME'};
        $mkcal =~ s/start.cgi$/mkcal.cgi/;
        print '<img src="' . $mkcal
            . '?refresh" width="1" height="1" alt="" />';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

sub day_count_menu ($$$) {
    my ( $this, $name, $default ) = @_;
    my ($menu);
    $menu = '<select name="' . $name . '" id="' . $name . '">';
    foreach ( 0 .. 31 ) {
        $menu
            .= '<option value="'
            . $_ . '"'
            . ( ( defined $default && $default == $_ )
            ? ' selected="selected"'
            : '' )
            . '>'
            . $_
            . '日後</option>';
    }
    $menu .= '</select>';
    return $menu;
}

sub count_target_menu ($$$) {
    my ( $this, $name, $default ) = @_;
    my ($menu);
    @$default = ( 'exception', 'other' ) if !ref $default || @$default == 0;
    $menu
        = '<select name="'
        . $name
        . '" id="'
        . $name
        . '" multiple="multiple" size="5">';
    $menu
        .= '<option value="holiday"'
        . (
        ( grep( $_ eq 'holiday', @$default ) ) ? ' selected="selected"' : '' )
        . '>祝祭日</option>';
    $menu
        .= '<option value="specialday"'
        . ( ( grep( $_ eq 'specialday', @$default ) )
        ? ' selected="selected"'
        : '' )
        . '>特別な日</option>';
    $menu
        .= '<option value="specialwday"'
        . ( ( grep( $_ eq 'specialwday', @$default ) )
        ? ' selected="selected"'
        : '' )
        . '>特別な曜日</option>';
    $menu
        .= '<option value="exception"'
        . ( ( grep( $_ eq 'exception', @$default ) )
        ? ' selected="selected"'
        : '' )
        . '>特別な曜日の例外</option>';
    $menu
        .= '<option value="other"'
        . (
        ( grep( $_ eq 'other', @$default ) ) ? ' selected="selected"' : '' )
        . '>上記以外の日</option>';
    $menu .= '</select>';
}

sub cell_bgcolor ($$$) {
    my ( $this, $name, $default ) = @_;
    my $form
        = '<input type="text" name="'
        . $name
        . '" id="'
        . $name
        . '" value="'
        . ( ( defined $default ) ? $default : '' )
        . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\''
        . $name
        . '\',\'dom\')" onchange="cmonos.validate.field(this).asColor()"'
        . ( ( defined $default )
        ? ' style="background-color: ' . $default . '"'
        : '' )
        . ' title="カレンダーのセル背景色を設定します。" />';
    $form
        .= ' <a href="JavaScript:cmonos.tool.colorPalette.open(\''
        . $name
        . '\')" ondblclick="cmonos.tool.colorPalette.open(\''
        . $name
        . '\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    $form
        .= ' <a href="JavaScript:cmonos.tool.colorSlider.open(\''
        . $name
        . '\')" ondblclick="cmonos.tool.colorSlider.open(\''
        . $name
        . '\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
    return $form;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 'd100', 'CounterEditor' );

    #ツール登録
    $setting::jump_menu{'d100'} = '日数カウント設定';

}

1;
