package FeedMixer;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:flock);
use Time::Local;
######################################################################
# フィードミキサーモジュール
# FeedMixer.pm ver1.0.4
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-05-30
######################################################################
# メイン
sub run ($;$) {
    my ( $this, $mkfile_flag ) = @_;
    my ($cache_file,       $list,  $template,
        $article_template, @feeds, %required,
        %article,          $count, $max
    );
    local (*OUT);

    #ページヘッダ
    if ($mkfile_flag) {
        if ( defined $setting::security{'feed'}{'passwordCheck'}
            && $setting::security{'feed'}{'passwordCheck'} eq 'nocheck' )
        {
            if ( open( OUT, ">>$default::file" ) ) {
                binmode OUT, ":utf8";
                flock( OUT, LOCK_EX );
                truncate( OUT, 0 );
                seek( OUT, 0, 0 );
                $cache_file = $default::file;
            }
            else {
                $this->print_invisible_gif;
                return;
            }
        }
        else {
            $this->write_file( $default::file, '', 1 );
            $this->write_file( $default::file, '', 1 )
                if $default::file =~ s/\.ssi$/.js/;
            $this->print_invisible_gif;
            return;
        }
    }
    else {
        binmode STDOUT, ":utf8";
        if ( defined $main::mode && $main::mode eq 'ssi' ) {
            $this->print_http_header('text/html');
            $cache_file = $default::preference_dir . '/ssi_cache.html';
        }
        elsif ( defined $main::mode && $main::mode eq 'js' ) {
            $this->print_http_header('text/javascript');
            $cache_file = $default::preference_dir . '/ssi_cache.js';
        }
        else {
            $this->print_http_header('text/html');
            $cache_file = $default::preference_dir . '/cache.html';
        }
        if ( !$main::refresh && -s $cache_file ) {
            my $ltime = time - $default::interval * 3600;
            my $mtime = ( stat($cache_file) )[9];
            if ( $mtime && $ltime && $mtime > $ltime ) {
                $this->print_cache($cache_file);
                return;
            }
        }
        $cache_file = $default::preference_dir . '/ssi_cache.html'
            if defined $main::mode && $main::mode eq 'js';
        if ( open( OUT, ">>$cache_file" ) ) {
            binmode OUT, ":utf8";
            flock( OUT, LOCK_EX );
            truncate( OUT, 0 );
            seek( OUT, 0, 0 );
            $this->print_page_header( $setting::key, \*OUT ) if !$main::mode;
        }
        else {
            print '';
            return;
        }
    }

    #フィルタ
    my ( @required_words, @ignore_words );
    if ( $setting::general{'requiredWords'} ) {
        @required_words = split( /\s+/, $setting::general{'requiredWords'} );
    }
    if ( $setting::general{'ignoreWords'} ) {
        @ignore_words = split( /\s+/, $setting::general{'ignoreWords'} );
    }

    if ( !$setting::page{$setting::key}{'template'} ) {
        $template = <<TEMP;
<!--mtime--><!--/mtime--> 更新
<ul class="index">
<!--article--><li><!--title--><!--/title--><!--site head=" (" foot=")"--><!--/site--></li><!--/article-->
</ul>
TEMP
    }
    else {
        $template = $setting::page{$setting::key}{'template'};
    }
    $template = $this->page_item_replace( $template, 0, '',
        $setting::page{$setting::key}{'title'} );
    if ( $template =~ /<!--frame(?![0-9a-zA-Z_])([^>]*)-->/ ) {
        require CMONOS::CGI_Lib::Frame;
        my $frame = Frame->new;
        $template
            = $frame->text_to_frame( $template, 'no', $frame::obj{'frame'},
            '', ( ($mkfile_flag) ? 'abs' : '' ) );
    }
    $article_template
        = ( $template
            =~ s/<!--article(?![0-9a-zA-Z_])([^>]*)-->(.*?)(<!--\/article-->)/<!--article--><!--\/article-->/sg
        )
        ? $this->join_header_and_footer( $1, $2 )
        : '<li><!--title--><!--/title--><!--site head=" (" foot=")"--><!--/site--></li>';
    foreach my $ele (
        'title',  'description', 'content', 'site',
        'author', 'link',        'mtime',   'date'
        )
    {
        $required{$ele} = 1
            if $article_template =~ /<!--$ele(?![0-9a-zA-Z_])[^>]*-->/s;
    }
    $template
        =~ s/<\!--mtime(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/mtime-->/$this->date_replace(time,$1,$2,0,(($main::mode) ? () : ($cgi::path . '?refresh=1','更新する')))/sge;
    $template
        =~ s/<\!--date(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/date-->/$this->date_replace(time,$1,$2,1)/sge;
    require CMONOS::CGI_Lib::HTTP_Client;
    @feeds = split( /[\x0A\x0D]+/, $setting::general{'feeds'} );
FEED:
    foreach my $url (@feeds) {
        my ($hash);
        if ( $url =~ m!^/! ) {
            $hash = HTTP_Client->new->get_hash_from_xml(
                $this->read_file( $this->path_change($url), 1 ) );
        }
        elsif ( $url =~ m!^https?://!i ) {
            my $res = HTTP_Client->new->request( 'GET' => $url );
            $hash = $res->hash('utf8') if $res->is_success;
        }
        if ( ref $hash ) {
            my ( $site, $site_url, $limit, $description_length );
            $limit = $setting::page{$setting::key}{'limit'} || 4;
            $description_length
                = $setting::page{$setting::key}{'descriptionLength'} || 128;
            if ( ref $hash->{'rss'} ) {
                $site = $hash->{'rss'}->{'channel'}->{'title'}
                    if defined $hash->{'rss'}->{'channel'}->{'title'};
                $site_url = $hash->{'rss'}->{'channel'}->{'link'}
                    if defined $hash->{'rss'}->{'channel'}->{'link'};
                if ( ref $hash->{'rss'}->{'channel'}->{'item'} ) {
                    my $n = 0;
                    $hash->{'rss'}->{'channel'}->{'item'}
                        = [ $hash->{'rss'}->{'channel'}->{'item'} ]
                        if ref $hash->{'rss'}->{'channel'}->{'item'} eq
                        'HASH';
                ITEM:
                    foreach my $item (
                        @{ $hash->{'rss'}->{'channel'}->{'item'} } )
                    {
                        last ITEM if $n >= $limit;
                        my $id
                            = ( ref $item->{'guid'}
                                && defined $item->{'guid'}->{'value'} )
                            ? $item->{'guid'}->{'value'}
                            : ( defined $item->{'guid'} ) ? $item->{'guid'}
                            : ( defined $item->{'dc:identifier'} )
                            ? $item->{'dc:identifier'}
                            : ( defined $item->{'link'} ) ? $item->{'link'}
                            :                               $url . '#' . $n;
                        $article{$id}{'mtime'}
                            = $this->date_to_gmt(
                              ( $item->{'pubDate'} ) ? $item->{'pubDate'}
                            : ( $item->{'dc:date'} ) ? $item->{'dc:date'}
                            :                          time );
                        $article{$id}{'link'}
                            = $item->{'link'}
                            || $item->{'guid'}
                            || $site_url
                            || $url;
                        $article{$id}{'site'} = $this->detag($site)
                            if defined $site;
                        $article{$id}{'site_url'} = $site_url
                            if defined $site_url;
                        $article{$id}{'title'}
                            = $this->detag( $item->{'title'} )
                            if $required{'title'} && defined $item->{'title'};
                        $article{$id}{'description'} = $this->cut_string(
                            ( defined $item->{'description'} )
                            ? $this->escape_except_html_entity(
                                $item->{'description'}
                                )
                            : $this->detag(
                                ( defined $item->{'content'} )
                                ? $item->{'content'}
                                : ( defined $item->{'content:encoded'} )
                                ? $item->{'content:encoded'}
                                : ''
                            ),
                            $description_length
                        ) if $required{'description'};
                        $article{$id}{'content'}
                            = $this->tag_fix( ( defined $item->{'content'} )
                            ? $item->{'content'}
                            : ( defined $item->{'content:encoded'} )
                            ? $item->{'content:encoded'}
                            : ( defined $item->{'description'} )
                            ? $item->{'description'}
                            : '' )
                            if $required{'content'};
                        $article{$id}{'author'}
                            = $this->detag(
                            ( defined $item->{'author'} ) ? $item->{'author'}
                            : ( defined $item->{'dc:creator'} )
                            ? $item->{'dc:creator'}
                            : ( defined $item->{'dc:publisher'} )
                            ? $item->{'dc:publisher'}
                            : '' )
                            if $required{'author'};

                        if ($this->is_not_required(
                                $article{$id}, \@required_words,
                                \@ignore_words
                            )
                            )
                        {
                            delete $article{$id};
                        }
                        else {
                            $n++;
                        }
                    }
                }
            }
            elsif ( ref $hash->{'rdf:RDF'} ) {
                $site = $hash->{'rdf:RDF'}->{'channel'}->{'title'}
                    if defined $hash->{'rdf:RDF'}->{'channel'}->{'title'};
                $site_url = $hash->{'rdf:RDF'}->{'channel'}->{'link'}
                    if defined $hash->{'rdf:RDF'}->{'channel'}->{'link'};
                if ( ref $hash->{'rdf:RDF'}->{'item'} ) {
                    my $n = 0;
                    $hash->{'rdf:RDF'}->{'item'}
                        = [ $hash->{'rdf:RDF'}->{'item'} ]
                        if ref $hash->{'rdf:RDF'}->{'item'} eq 'HASH';
                ITEM:
                    foreach my $item ( @{ $hash->{'rdf:RDF'}->{'item'} } ) {
                        last ITEM if $n >= $limit;
                        my $id
                            = ( ref $item->{'guid'}
                                && defined $item->{'guid'}->{'value'} )
                            ? $item->{'guid'}->{'value'}
                            : ( defined $item->{'guid'} ) ? $item->{'guid'}
                            : ( defined $item->{'dc:identifier'} )
                            ? $item->{'dc:identifier'}
                            : ( defined $item->{'link'} ) ? $item->{'link'}
                            :                               $url . '#' . $n;
                        $article{$id}{'mtime'}
                            = $this->date_to_gmt(
                              ( $item->{'pubDate'} ) ? $item->{'pubDate'}
                            : ( $item->{'dc:date'} ) ? $item->{'dc:date'}
                            :                          time );
                        $article{$id}{'link'}
                            = $item->{'link'}
                            || $item->{'guid'}
                            || $site_url
                            || $url;
                        $article{$id}{'site'} = $this->detag($site)
                            if defined $site;
                        $article{$id}{'site_url'} = $site_url
                            if defined $site_url;
                        $article{$id}{'title'}
                            = $this->detag( $item->{'title'} )
                            if $required{'title'} && defined $item->{'title'};
                        $article{$id}{'description'} = $this->cut_string(
                            ( defined $item->{'description'} )
                            ? $this->escape_except_html_entity(
                                $item->{'description'}
                                )
                            : $this->detag(
                                ( defined $item->{'content'} )
                                ? $item->{'content'}
                                : ( defined $item->{'content:encoded'} )
                                ? $item->{'content:encoded'}
                                : ''
                            ),
                            $description_length
                        ) if $required{'description'};
                        $article{$id}{'content'}
                            = $this->tag_fix( ( defined $item->{'content'} )
                            ? $item->{'content'}
                            : ( defined $item->{'content:encoded'} )
                            ? $item->{'content:encoded'}
                            : ( defined $item->{'description'} )
                            ? $item->{'description'}
                            : '' )
                            if $required{'content'};
                        $article{$id}{'author'}
                            = $this->detag(
                            ( defined $item->{'author'} ) ? $item->{'author'}
                            : ( defined $item->{'dc:creator'} )
                            ? $item->{'dc:creator'}
                            : ( defined $item->{'dc:publisher'} )
                            ? $item->{'dc:publisher'}
                            : '' )
                            if $required{'author'};

                        if ($this->is_not_required(
                                $article{$id}, \@required_words,
                                \@ignore_words
                            )
                            )
                        {
                            delete $article{$id};
                        }
                        else {
                            $n++;
                        }
                    }
                }
            }
            elsif ( ref $hash->{'feed'} ) {
                $site
                    = ( ref $hash->{'feed'}->{'title'} )
                    ? $hash->{'feed'}->{'title'}->{'value'}
                    : $hash->{'feed'}->{'title'}
                    if defined $hash->{'feed'}->{'title'};
                $site_url = $this->get_atom_value( $hash->{'feed'}->{'link'},
                    'html', 'href' )
                    if defined $hash->{'feed'}->{'link'};
                if ( ref $hash->{'feed'}->{'entry'} ) {
                    my $n = 0;
                    $hash->{'feed'}->{'entry'}
                        = [ $hash->{'feed'}->{'entry'} ]
                        if ref $hash->{'feed'}->{'entry'} eq 'HASH';
                ITEM:
                    foreach my $item ( @{ $hash->{'feed'}->{'entry'} } ) {
                        last if $n >= $limit;
                        my $id
                            = ( defined $item->{'id'} ) ? $item->{'id'}
                            : ( ref $item->{'link'} )
                            ? $this->get_atom_value( $item->{'link'}, 'html',
                            'href' )
                            : $url . '#' . $n;
                        $article{$id}{'mtime'}
                            = $this->date_to_gmt(
                              ( $item->{'updated'} )   ? $item->{'updated'}
                            : ( $item->{'modified'} )  ? $item->{'modified'}
                            : ( $item->{'published'} ) ? $item->{'published'}
                            : ( $item->{'issued'} )    ? $item->{'issued'}
                            :                            time );
                        $article{$id}{'link'}
                            = $this->get_atom_value( $item->{'link'}, 'html',
                            'href' )
                            || $this->get_atom_value( $item->{'link'}, '',
                            'href' )
                            || $site_url
                            || $url;
                        $article{$id}{'site'} = $this->detag($site)
                            if defined $site;
                        $article{$id}{'site_url'} = $site_url
                            if defined $site_url;
                        $article{$id}{'title'}
                            = $this->detag( ( ref $item->{'title'} )
                            ? $item->{'title'}->{'value'}
                            : $item->{'title'} )
                            if $required{'title'} && defined $item->{'title'};
                        $article{$id}{'description'} = $this->cut_string(
                            $this->detag(
                                ( ref $item->{'summary'} )
                                ? $item->{'summary'}->{'value'}
                                : ( defined $item->{'summary'} )
                                ? $item->{'summary'}
                                : $this->tag_fix(
                                    $this->get_atom_value(
                                        $item->{'content'}, 'html',
                                        'value'
                                    )
                                )
                            ),
                            $description_length
                        ) if $required{'description'};
                        $article{$id}{'content'} = $this->tag_fix(
                            $this->get_atom_value(
                                $item->{'content'}, 'html', 'value'
                            )
                            )
                            if $required{'content'}
                            && defined $item->{'content'};
                        $article{$id}{'author'}
                            = $this->detag( $item->{'author'}->{'name'} )
                            if $required{'author'}
                            && ref $item->{'author'} eq 'HASH'
                            && defined $item->{'author'}->{'name'};

                        if ($this->is_not_required(
                                $article{$id}, \@required_words,
                                \@ignore_words
                            )
                            )
                        {
                            delete $article{$id};
                        }
                        else {
                            $n++;
                        }
                    }
                }
            }
        }
    }
    $list  = '';
    $count = 0;
    $max
        = ( $setting::page{$setting::key}{'max'} )
        ? $setting::page{$setting::key}{'max'}
        : 0;
    foreach my $id (
        sort { $article{$b}{'mtime'} <=> $article{$a}{'mtime'} || $a cmp $b }
        keys %article )
    {
        last if $max && ++$count > $max;
        my $temp      = $article_template;
        my $is_linked = 0;
        $article{$id}{'link'}
            =~ s/[\?\&]utm_source=[^\&=]*&utm_medium=[^\&=]*//i;
        if ( $required{'site'} ) {
            my $site
                = ( defined $article{$id}{'site'}
                    && $article{$id}{'site'} ne '' )
                ? ( defined $article{$id}{'site_url'}
                    && $article{$id}{'site_url'} ne '' )
                    ? '<a href="'
                    . $this->escape( $article{$id}{'site_url'} ) . '"'
                    . ( ( $setting::general{'nofollow'} )
                        ? ' rel="nofollow"'
                        : '' )
                    . ( ( $setting::page{$setting::key}{'window'} )
                        ? ' target="_blank"'
                        : '' )
                    . '>'
                    . $article{$id}{'site'} . '</a>'
                    : $article{$id}{'site'}
                : '';
            $temp
                =~ s/<\!--site(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/site(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('site',$1,$2,$site)/ges;
        }
        if ( $required{'title'} ) {
            my $title
                = ( defined $article{$id}{'title'}
                    && $article{$id}{'title'} ne '' )
                ? '<a href="'
                . $this->escape( $article{$id}{'link'} ) . '"'
                . (
                ( $setting::general{'nofollow'} ) ? ' rel="nofollow"' : '' )
                . ( ( $setting::page{$setting::key}{'window'} )
                ? ' target="_blank"'
                : '' )
                . ( ( $article{$id}{'site'} )
                ? ' title="'
                    . $this->escape_except_html_entity(
                    $article{$id}{'site'} )
                    . '"'
                : '' )
                . '>'
                . $article{$id}{'title'} . '</a>'
                : '';
            $temp
                =~ s/<\!--title(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/title(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('title',$1,$2,$title)/ges;
            $is_linked = 1;
        }
        if ( $required{'link'} ) {
            my $link
                = ( defined $article{$id}{'link'}
                    && $article{$id}{'link'} ne '' )
                ? '<a href="'
                . $this->escape( $article{$id}{'link'} ) . '"'
                . (
                ( $setting::general{'nofollow'} ) ? ' rel="nofollow"' : '' )
                . ( ( $setting::page{$setting::key}{'window'} )
                ? ' target="_blank"'
                : '' )
                . ( ( $article{$id}{'site'} )
                ? ' title="'
                    . $this->escape_except_html_entity(
                    $article{$id}{'site'} )
                    . '"'
                : '' )
                . '>'
                . $this->escape( $article{$id}{'link'} ) . '</a>'
                : '';
            $temp
                =~ s/<\!--link(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/link(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('link',$1,$2,$link)/ges;
            $is_linked = 1;
        }
        $temp
            =~ s/<\!--mtime(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/mtime-->/$this->date_replace($article{$id}{'mtime'},$1,$2,0,(($is_linked || !$article{$id}{'link'}) ? () : ($article{$id}{'link'},(($article{$id}{'site'}) ? $article{$id}{'site'} : ''),$setting::page{$setting::key}{'window'},$setting::general{'nofollow'})))/sge;
        $temp
            =~ s/<\!--date(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/date-->/$this->date_replace($article{$id}{'mtime'},$1,$2,1,(($is_linked || !$article{$id}{'link'}) ? () : ($article{$id}{'link'},(($article{$id}{'site'}) ? $article{$id}{'site'} : ''),$setting::page{$setting::key}{'window'},$setting::general{'nofollow'})))/sge;
        $temp
            =~ s/<\!--author(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/author(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('author',$1,$2,$article{$id}{'author'})/ges
            if $required{'author'};
        if ( $required{'description'} ) {
            my $description
                = $this->autolink( $article{$id}{'description'}, 1, 1 );
            if (   $article{$id}{'site_url'}
                && $article{$id}{'site'}
                =~ m!Twitter[\x09\x0A\x0C\x0D\x20]*/[\x09\x0A\x0C\x0D\x20]*(.*)$!i
                )
            {
                my $name = $1;
                my $link_attributes
                    = ( ( $setting::general{'nofollow'} ) ? ' rel="nofollow"'
                    : '' )
                    . ( ( $setting::page{$setting::key}{'window'} )
                    ? ' target="_blank"'
                    : '' );
                my $site
                    = '<strong><a href="'
                    . $this->escape( $article{$id}{'site_url'} ) . '"'
                    . $link_attributes . '>'
                    . $name
                    . '</a></strong>';
                $description =~ s/^$name:/$site/;
                $description
                    =~ s!\@([0-9a-zA-Z_]+)!\@<a href="http://twitter.com/$1"$link_attributes>$1</a>!g;
                $description
                    =~ s!\#([0-9a-zA-Z_]+)(?=(?:[\x09\x0A\x0C\x0D\x20]|$))!<a href="http://search.twitter.com/search?q=%23$1"$link_attributes>#$1</a>!g;
                $description
                    =~ s!\#([^\x09\x0A\x0C\x0D\x20]+_)(?=(?:[\x09\x0A\x0C\x0D\x20]|$))!'<a href="http://search.twitter.com/search?q=%23' . $this->url_encode($1) . '"' . $link_attributes . '>#' . $1 . '</a>'!eg;
            }
            $temp
                =~ s/<\!--description(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/description(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('description',$1,$2,$description)/ges;
        }
        $temp
            =~ s/<\!--content(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/content(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('content',$1,$2,$article{$id}{'content'})/ges
            if $required{'content'};
        $list .= $temp;
    }
    $template
        =~ s/<\!--article--><\!--\/article-->/$this->item_tag_replace('article','','',$list)/ges;
    if ($mkfile_flag) {
        my $mklist = $ENV{'SCRIPT_NAME'};
        $mklist =~ s/start.cgi$/mklist.cgi/;
        $template .= '<img src="' . $mklist . '" width="1" height="1" />'
            if $template !~ /<img[^>]+src="$mklist"[^>]*>/i;
    }
    print OUT $template;
    $this->print_page_footer( $setting::key, \*OUT ) if !$main::mode;
    close(OUT);
    $this->mk_external_javascript($cache_file);

    #ページフッタ
    if ($mkfile_flag) {
        $this->print_invisible_gif;
    }
    else {
        $cache_file = $default::preference_dir . '/ssi_cache.js'
            if defined $main::mode && $main::mode eq 'js';
        $this->print_cache($cache_file);
        binmode STDOUT;
    }
}

sub is_not_required($$$$) {
    my ( $this, $article, $required_words, $ignore_words ) = @_;
    return 0 if !@$required_words && !@$ignore_words;
    my @data = map( $this->z_to_h( $_, 'utf8' ),
        (   $article->{'title'},   $article->{'description'},
            $article->{'content'}, $article->{'author'}
        ) );
    my $is_matched = 0;
    if (@$required_words) {
        foreach my $w (@$required_words) {
            if ( grep( eval { $_ =~ /$w/i }, @data ) ) {
                if (   $setting::general{'requiredWordsMethod'}
                    && $setting::general{'requiredWordsMethod'} eq 'and' )
                {
                    return 1;
                }
                else {
                    $is_matched = 1;
                }
            }
        }
        return 1 if !$is_matched;
    }
    if (@$ignore_words) {
        $is_matched = 1;
        foreach my $w (@$ignore_words) {
            if ( grep( eval { $_ =~ /$w/i }, @data ) ) {
                if (   $setting::general{'ignoreWordsMethod'}
                    && $setting::general{'ignoreWordsMethod'} ne 'and' )
                {
                    return 1;
                }
            }
            else {
                $is_matched = 0;
            }
        }
        return $is_matched;
    }
    else {
        return 0;
    }
}

sub get_atom_value ($$$$) {
    my ( $this, $element, $type, $name ) = @_;
    return '' if !defined $element;
    $element = [$element] if !ref $element || ref $element ne 'ARRAY';
    foreach my $this_element (@$element) {
        if ((   (   defined $this_element->{'type'}
                    && $this_element->{'type'} =~ /$type/
                )
                || !$type
            )
            && defined $this_element->{$name}
            )
        {
            return $this_element->{$name};
        }
    }
    return '';
}

######################################################################
sub date_replace ($$;$$$$$) {
    my ($this, $gmt,   $attributes, $content, $date,
        $link, $title, $newwindow,  $nofollow
    ) = @_;
    my $type
        = (    $attributes
            && $attributes =~ /[\x09\x0A\x0C\x0D\x20]+type="([^"]+)"/s )
        ? $1
        : $setting::general{'datetype'};
    my $format = '';
    if ( $content
        =~ s/[\x09\x0A\x0C\x0D\x20]*<\!--format(?![0-9a-zA-Z_])[^>]*-->(.*?)<\!--\/format(?![0-9a-zA-Z_])[^>]*-->//si
        )
    {
        $format = $1;
        $attributes
            .= ' format="'
            . $this->lf_to_entity( $this->escape($format) ) . '"'
            unless $attributes
            =~ s/([\x09\x0A\x0C\x0D\x20]+format[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*")[^"]*(")/$1 . $this->lf_to_entity($this->escape($format)) . $2/gesi;
    }
    elsif ( $attributes
        =~ /[\x09\x0A\x0C\x0D\x20]+format[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/si
        )
    {
        $format = $this->unescape( $this->entity_to_lf($1) );
    }
    my ( $header, $footer )
        = $this->get_header_and_footer( $attributes, $content );
    $type .= '>d' if $date;
    $type = 'gmt' . $type if $type !~ /gmt/;
    return $header
        . (
        ($link)
        ? '<a href="'
            . $this->escape($link) . '"'
            . (
            ($title)
            ? ' title="' . $this->escape_except_html_entity($title) . '"'
            : ''
            )
            . ( ($nofollow)  ? ' rel="nofollow"'  : '' )
            . ( ($newwindow) ? ' target="_blank"' : '' ) . '>'
        : ''
        )
        . $this->date( $gmt, 0, $type, $format )
        . ( ($link) ? '</a>' : '' )
        . $footer;
}

######################################################################
sub print_invisible_gif ($) {
    my $this = shift;
    binmode STDOUT;
    print 'Content-type: image/gif', "\x0A";
    print 'Content-length: 43',      "\x0A";

    #print 'Cache-Control: no-cache',"\x0A";
    #print 'Pragma: no-cache',"\x0A";
    print "\x0A";
    print
        "GIF89a\x01\x00\x01\x00\x80\x00\x00\xFF\xFF\xFF\x00\x00\x00\x21\xF9\x04\x05\x14\x00\x00\x00\x2C\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00\x3B";
}

######################################################################
sub print_cache ($$) {
    my ( $this, $path ) = @_;
    local (*FILE);
    if ( open( FILE, $path ) ) {
        binmode FILE, ":utf8";
        flock( FILE, LOCK_SH );
        binmode STDOUT, ":utf8";
        while (<FILE>) {
            print;
        }
        close(FILE);
        binmode STDOUT;
    }
}

######################################################################
sub unauthorized_alert ($) {
    my $this = shift;
    if ($main::m) {
        binmode STDOUT, ":utf8";
        if ( $main::mode && ( $main::mode eq 'ssi' || $main::mode eq 'js' ) )
        {
            $this->print_http_header('text/javascript');
        }
        else {
            $this->print_password_check_form($setting::key);
        }
    }
    else {
        $this->write_file( $default::file, '', 1 );
        $this->write_file( $default::file, '', 1 )
            if $default::file =~ s/\.ssi$/.js/;
        $this->print_invisible_gif;
    }
    exit;
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #設定DB読み込み設定
    $setting::general = 1 if !$setting::general;    #一般設定読み込み
    $setting::deco = 1
        if !$setting::deco;    #テキスト装飾設定読み込み
    $setting::security = 1
        if !$setting::security;    #アクセス権設定読み込み
    $setting::page = 1 if !$setting::page;    #ページ設定読み込み

    #設定種キー設定
    undef $main::mode
        if !$main::mode || ( $main::mode ne 'ssi' && $main::mode ne 'js' );
    $setting::security_key = 'feed';  #デフォルトセキュリティキー
    if ($main::mode) {
        $setting::key = 'ssi';
    }
    else {
        $setting::key = 'feed';
    }

    #クッキー
    $cookie::id_enabled = 1
        if !$cookie::id_enabled;      #ID情報クッキー読み込み

    #デフォルトキー
    $main::k = 'feed' if !defined $main::k || !$main::k;

    #必須フォーム
    @setting::form = ( 'm', 'k' ) if !@setting::form;

    #クラス登録
    Common::class_register( 'feed', 'FeedMixer' );
}

1;
