package GateKeeper;
require CMONOS::GateKeeper::Tools::DSelector;
@ISA = qw(DSelector);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# アクセス制限管理モジュール
# GateKeeper.pm ver1.0.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-12-20
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ($auth_enabled,         $auth_dbm_enabled,
        $auto_disabled,        $auto_register_enabled,
        $auto_reissue_enabled, %name,
        @user,                 @options,
        @search,               @account_list,
        $current_directory,    $htaccess,
        $htpasswd,             $htcomment,
        $htauthdb,             $htmaildb,
        $comment_data,         @search_list
    );
    $main::a = '' if !defined $main::a;

#選択直後→一般設定にデフォルトディレクトリとして登録
    if ( $main::a eq 'ds' ) {
        my (%general);
        $general{'current'} = $main::current = $this->current_directory;
        $this->write_db( $default::preference_dir . '/general', \%general );
        $main::a = '';
    }
    $current_directory = $this->path_conv( $main::current, 'path' );
    $htaccess = $current_directory . '/.htaccess';

    #保存形式のチェック
    $auto_disabled = 1 if $ENV{'SCRIPT_NAME'} =~ /^$main::current/;
    ( $auth_dbm_enabled, $auto_register_enabled, $auto_reissue_enabled )
        = $this->get_auth_settings($htaccess);
    $auto_register_enabled = ( $main::reg eq 'on' )  ? 1 : 0 if $main::reg;
    $auto_reissue_enabled  = ( $main::reis eq 'on' ) ? 1 : 0 if $main::reis;
    $auto_register_enabled = $auto_reissue_enabled = $main::reg = $main::reis
        = 0
        if $auto_disabled;
    if ($auth_dbm_enabled) {
        use AnyDBM_File;
        $htauthdb = $current_directory . '/.htauthdb';
        $htmaildb = $current_directory . '/.htmaildb';
    }
    else {
        $htpasswd  = $current_directory . '/.htpasswd';
        $htcomment = $current_directory . '/.htcomment';
    }

    #アクセス制限解除
    if ( $main::a eq 'cancel' ) {
        if ((   (      $auth_dbm_enabled
                    && $this->delete_auth_dbm($current_directory)
                )
                || (!$auth_dbm_enabled
                    && ( !-f $current_directory . '/.htgroup'
                        || unlink( $current_directory . '/.htgroup' ) )
                    && ( !-f $htpasswd  || unlink($htpasswd) )
                    && ( !-f $htcomment || unlink($htcomment) )
                )
            )
            && $this->auth_cancel( $htaccess, $current_directory )
            )
        {
            undef %main::data;
            $main::a = 'new';
            push(
                @{ $this->{'dialog'} },
                'アクセス制限を解除しました。'
            );
            (   $auth_dbm_enabled, $auto_register_enabled,
                $auto_reissue_enabled
            ) = $this->get_auth_settings($htaccess);
            $auto_register_enabled = $auto_reissue_enabled = $main::reg
                = $main::reis = 0
                if $auto_disabled;
            if ($auth_dbm_enabled) {
                use AnyDBM_File;
                $htauthdb = $current_directory . '/.htauthdb';
                $htmaildb = $current_directory . '/.htmaildb';
            }
            else {
                $htpasswd  = $current_directory . '/.htpasswd';
                $htcomment = $current_directory . '/.htcomment';
            }
        }
        else {
            push(
                @{ $this->{'dialog'} },
                'アクセス制限の解除に失敗しました。'
            );
            $main::a = '';
        }

        #登録アカウントリスト
        #} elsif ($main::a eq 'list') {
        #	undef %main::data;
        #	$main::a = '';
    }
    push( @main::delete, $main::delete )
        if defined $main::delete && $main::delete;
    @main::delete = grep( $_ ne '', @main::delete );

    #入力のチェック
    #.htpasswd / .htcomment 更新アカウントのみ抽出
    my (%dialog, %new_account, %new_passwd,
        %mailto, %expiretime,  @new_data
    );
    if (%main::data) {
        no warnings qw(untie);
        my ( $acc, @new_account, @new_addr, @all_addr, %htauthdb, %htmaildb );
        if ($auth_dbm_enabled) {
            tie( %htauthdb, 'AnyDBM_File', $htauthdb, O_RDONLY,
                $env::dbm_mode );
            tie( %htmaildb, 'AnyDBM_File', $htmaildb, O_RDONLY,
                $env::dbm_mode );
        }
        elsif ( $setting::general{'uniqaddr'} ) {
            if ( open( CMT, $htcomment ) ) {
                binmode CMT, ":utf8";
                flock( CMT, LOCK_SH );
                my ($line);
                while ( $line = <CMT> ) {
                    chomp $line;
                    my (@data) = split( /\t/, $line );
                    push( @all_addr, $data[2] )
                        if $data[2] && !$main::data{'acc'}{ $data[0] };
                }
                close(CMT);
            }
        }
        while ( ($acc) = each %{ $main::data{'acc'} } ) {
            my ($msg);

            #削除するアカウント
            next if @main::delete && grep( $_ eq $acc, @main::delete );

            #調整
            $main::data{'name'}{$acc} =~ s/[\x09\x0A\x0C\x0D\x20]+/ /sg
                if defined $main::data{'name'}{$acc};
            $main::data{'cmt'}{$acc} =~ s/[\x09\x0A\x0C\x0D\x20]+/ /sg
                if defined $main::data{'cmt'}{$acc};
            if ($auth_dbm_enabled) {
                $main::data{'name'}{$acc} =~ s/:/=/g
                    if defined $main::data{'name'}{$acc};
                $main::data{'cmt'}{$acc} =~ s/:/=/g
                    if defined $main::data{'cmt'}{$acc};
            }

            #アカウントチェック
            if (defined $main::data{'acc'}{$acc}
                && ( $main::data{'acc'}{$acc} ne $acc
                    || (   $acc =~ /^_[0-9]+$/
                        && $main::data{'acc'}{$acc} ne '' ) )
                )
            {
                if ((   $auth_dbm_enabled
                        && $htauthdb{ $this->encode_utf8(
                                $main::data{'acc'}{$acc} ) }
                    )
                    || ( !$auth_dbm_enabled
                        && grep( $_ eq $main::data{'acc'}{$acc},
                            @new_account ) )
                    )
                {
                    $dialog{$acc}
                        .= 'このアカウント ('
                        . $main::data{'acc'}{$acc}
                        . ') は既に使用されています。';
                }
                elsif ( $main::data{'acc'}{$acc}
                    =~ /^[0-9a-zA-Z]{1}[0-9a-zA-Z_]{1,31}$/ )
                {
                    $new_account{$acc} = $main::data{'acc'}{$acc};
                    push( @new_account, $main::data{'acc'}{$acc} )
                        if !$auth_dbm_enabled;
                }
                elsif ($acc !~ /^_[0-9]+$/
                    || $main::data{'acc'}{$acc} ne '' )
                {
                    $dialog{$acc}
                        .= 'アカウントは2文字以上32文字以内の半角英数字にしてください。';
                }
            }

            #アドレスチェック
            if ( defined $main::data{'addr'}{$acc}
                && $main::data{'addr'}{$acc} ne '' )
            {
                if ($setting::general{'uniqaddr'}
                    && ((   $auth_dbm_enabled
                            && $htmaildb{
                                $this->encode_utf8(
                                    $main::data{'addr'}{$acc}
                                )
                            }
                            && !grep( $_ eq $acc,
                                split(
                                    /,/,
                                    $this->decode_utf8(
                                        $htmaildb{
                                            $this->encode_utf8(
                                                $main::data{'addr'}{$acc}
                                            )
                                        }
                                    )
                                ) )
                        )
                        || (!$auth_dbm_enabled
                            && (grep( $_ eq $main::data{'addr'}{$acc},
                                    @new_addr )
                                || grep( $_ eq $main::data{'addr'}{$acc},
                                    @all_addr )
                            )
                        )
                    )
                    )
                {
                    $dialog{$acc}
                        .= 'このメールアドレス ('
                        . $main::data{'addr'}{$acc}
                        . ') は既に登録されています。';
                }
                elsif (
                    $this->is_invalid_email_address(
                        $main::data{'addr'}{$acc}
                    )
                    )
                {
                    $dialog{$acc}
                        .= 'メールアドレスではありません。';
                }
                elsif ( !$auth_dbm_enabled ) {
                    push( @new_addr, $main::data{'addr'}{$acc} );
                }
            }

            #期限チェック→期限確定
            if ( defined $main::data{'expire'}{$acc}
                && $main::data{'expire'}{$acc} ne '' )
            {
                my ( $year, $mon, $mday )
                    = split( /[^0-9]/, $main::data{'expire'}{$acc} );
                if (   $year
                    && $year > 2000
                    && $mon
                    && $mday
                    && !$this->is_impossible_mday( $year, $mon, $mday ) )
                {
                    use Time::Local;
                    $main::data{'exptime'}{$acc}
                        = timelocal( 0, 0, 0, $mday + 0, $mon - 1, $year );
                }
                else {
                    $dialog{$acc}
                        .= '有効期限の日付を正常に取得できませんでした。';
                }
            }
            elsif ( defined $main::data{'lifetime'}{$acc}
                && $main::data{'lifetime'}{$acc} ne '' )
            {
                if ( $main::data{'lifetime'}{$acc} eq 'ever' ) {
                    $main::data{'exptime'}{$acc} = '';
                }
                elsif ( $main::data{'lifetime'}{$acc} =~ /[^0-9]/ ) {
                    $dialog{$acc}
                        .= '有効期限を正常に取得できませんでした。';
                }
                else {
                    $main::data{'exptime'}{$acc}
                        = time + $main::data{'lifetime'}{$acc};
                }
            }
            if (   defined $main::data{'exptime'}{$acc}
                && $main::data{'exptime'}{$acc} ne ''
                && $main::data{'exptime'}{$acc} < time )
            {
                $dialog{$acc}
                    .= '有効期限設定が正常に設定できませんでした。';
            }

            #パスワードチェック
            if (   defined $main::data{'passwd'}{$acc}
                && $main::data{'passwd'}{$acc} ne ''
                && $this->is_invalid_password(
                    $main::data{'passwd'}{$acc},
                    4, 8, 1
                )
                )
            {
                $dialog{$acc}
                    .= 'パスワードは4文字以上8文字以内の英字で始まる半角英数字にしてください。';
            }
            elsif ( defined $main::data{'passwd'}{$acc}
                && $main::data{'passwd'}{$acc} ne '' )
            {
                $new_passwd{$acc} = $main::data{'passwd'}{$acc};
            }

            #パスワード発行
            if ( defined $main::data{'mail'}{$acc}
                && $main::data{'mail'}{$acc} )
            {
                if (!defined $main::data{'addr'}{$acc}
                    || $this->is_invalid_email_address(
                        $main::data{'addr'}{$acc}
                    )
                    )
                {
                    $dialog{$acc}
                        .= 'メールアドレスが登録されていないためメールを配信できません。';
                }
                elsif ( !defined $dialog{$acc} || $dialog{$acc} eq '' ) {
                    $mailto{$acc} = $main::data{'addr'}{$acc};
                    $name{$acc}   = $main::data{'name'}{$acc}
                        if defined $main::data{'name'}{$acc};
                    $expiretime{$acc} = $main::data{'exptime'}{$acc}
                        if defined $main::data{'exptime'}{$acc};
                    if ( !defined $main::data{'passwd'}{$acc}
                        || $main::data{'passwd'}{$acc} eq '' )
                    {
                        $new_passwd{$acc} = $this->rand_char;
                    }
                }
            }

            #エラー
            if ($dialog{$acc}
                || ( $acc =~ /^_[0-9]+$/
                    && ( !$new_account{$acc} || !$new_passwd{$acc} ) )
                )
            {
                $dialog{$acc}
                    .= 'アカウントとパスワードは必ず設定して下さい。'
                    if $acc =~ /^_[0-9]+$/
                    && (
                    ( !$new_account{$acc} && $new_passwd{$acc} )
                    || (   $new_account{$acc}
                        && !$new_passwd{$acc}
                        && !$main::data{'mail'}{$acc} )
                    );
                delete $new_account{$acc}
                    if defined $new_account{$acc} && $new_account{$acc};
                delete $new_passwd{$acc}
                    if defined $new_passwd{$acc} && $new_passwd{$acc};
            }
            else {
                push( @new_data, $acc )
                    if ( $auth_dbm_enabled
                    && defined $main::data{'acc'}{$acc}
                    && $main::data{'acc'}{$acc} ne '' )
                    || ( !$auth_dbm_enabled
                    && ( $new_account{$acc} || $new_passwd{$acc} ) );
                delete $main::data{'passwd'}{$acc};
                delete $main::data{'mail'}{$acc};
            }
        }
        if ($auth_dbm_enabled) {
            untie(%htauthdb);
            untie(%htmaildb);
        }
    }

    #DBM
    if ($auth_dbm_enabled) {
        my ( %htauthdb, %htmaildb );
        @search = grep( $_ ne '', split( /\s+/, $main::search ) )
            if $main::a ne 'new'
            && defined $main::search
            && $main::search ne '';
        $main::a = 'new' if !@search && !$main::expired_only;

        #登録/更新
        if (   @new_data
            || @main::delete
            || (   $main::gc
                && $this->fileglob( $current_directory, '^\.htauthdb', 'f' ) )
            )
        {
            my ( $acc, @delete );
            tie( %htauthdb, 'AnyDBM_File', $htauthdb, O_RDWR | O_CREAT,
                $env::dbm_mode );
            tie( %htmaildb, 'AnyDBM_File', $htmaildb, O_RDWR | O_CREAT,
                $env::dbm_mode );
            foreach $acc (@new_data) {

                #新規登録
                if ( $acc =~ /^_[0-9]+$/ ) {
                    if ( $new_account{$acc} && $new_passwd{$acc} ) {
                        $htauthdb{ $this->encode_utf8( $new_account{$acc} ) }
                            = $this->encode_utf8(
                            join(
                                ':',
                                crypt( $new_passwd{$acc}, $this->salt ),
                                'member',
                                $main::data{'name'}{$acc},
                                $main::data{'addr'}{$acc},
                                $main::data{'cmt'}{$acc},
                                (   ( defined $main::data{'exptime'}{$acc} )
                                    ? $main::data{'exptime'}{$acc}
                                    : ''
                                )
                            )
                            );
                        $htmaildb{ $this->encode_utf8(
                                $main::data{'addr'}{$acc} ) }
                            = $this->encode_utf8(
                            $this->push_account(
                                $this->decode_utf8(
                                    $htmaildb{
                                        $this->encode_utf8(
                                            $main::data{'addr'}{$acc}
                                        )
                                    }
                                ),
                                $new_account{$acc}
                            )
                            );
                    }

                    #更新
                }
                elsif ( $htauthdb{ $this->encode_utf8($acc) } ) {
                    my @data = split(
                        /:/,
                        $this->decode_utf8(
                            $htauthdb{ $this->encode_utf8($acc) }
                        )
                    );
                    my $mail_is_changed = 0;
                    if ( $new_account{$acc} ) {
                        delete $htauthdb{ $this->encode_utf8($acc) };
                        $htauthdb{ $this->encode_utf8( $new_account{$acc} ) }
                            = $this->encode_utf8(
                            join(
                                ':',
                                (   ( $new_passwd{$acc} )
                                    ? crypt( $new_passwd{$acc}, $this->salt )
                                    : $data[0]
                                ),
                                $data[1],
                                $main::data{'name'}{$acc},
                                $main::data{'addr'}{$acc},
                                $main::data{'cmt'}{$acc},
                                (   ( defined $main::data{'exptime'}{$acc} )
                                    ? $main::data{'exptime'}{$acc}
                                    : $data[5]
                                )
                            )
                            );
                        $htmaildb{ $this->encode_utf8(
                                $main::data{'addr'}{$acc} ) }
                            = $this->encode_utf8(
                            $this->push_account(
                                $this->decode_utf8(
                                    $htmaildb{
                                        $this->encode_utf8(
                                            $main::data{'addr'}{$acc}
                                        )
                                    }
                                ),
                                $new_account{$acc}
                            )
                            );
                        $mail_is_changed = 1;
                    }
                    else {
                        $htauthdb{ $this->encode_utf8($acc) }
                            = $this->encode_utf8(
                            join(
                                ':',
                                (   ( $new_passwd{$acc} )
                                    ? crypt( $new_passwd{$acc}, $this->salt )
                                    : $data[0]
                                ),
                                $data[1],
                                $main::data{'name'}{$acc},
                                $main::data{'addr'}{$acc},
                                $main::data{'cmt'}{$acc},
                                (   ( defined $main::data{'exptime'}{$acc} )
                                    ? $main::data{'exptime'}{$acc}
                                    : $data[5]
                                )
                            )
                            );
                        if ( $data[3] ne $main::data{'addr'}{$acc} ) {
                            $htmaildb{
                                $this->encode_utf8(
                                    $main::data{'addr'}{$acc}
                                )
                                }
                                = $this->encode_utf8(
                                $this->push_account(
                                    $this->decode_utf8(
                                        $htmaildb{
                                            $this->encode_utf8(
                                                $main::data{'addr'}{$acc}
                                            )
                                        }
                                    ),
                                    $acc
                                )
                                );
                            $mail_is_changed = 1;
                        }
                    }
                    if ($mail_is_changed) {
                        my $old_acc = $this->shift_account(
                            $this->decode_utf8(
                                $htmaildb{ $this->encode_utf8( $data[3] ) }
                            ),
                            $acc
                        );
                        if ($old_acc) {
                            $htmaildb{ $this->encode_utf8( $data[3] ) }
                                = $this->encode_utf8($old_acc);
                        }
                        else {
                            delete $htmaildb{ $this->encode_utf8( $data[3] )
                            };
                        }
                    }
                }
                push( @delete, $acc );
            }
            foreach $acc (@main::delete) {
                my @data = split(
                    /:/,
                    $this->decode_utf8(
                        $htauthdb{ $this->encode_utf8($acc) }
                    )
                );
                delete $htauthdb{ $this->encode_utf8($acc) };
                my $old_acc = $this->shift_account(
                    $this->decode_utf8(
                        $htmaildb{ $this->encode_utf8( $data[3] ) }
                    ),
                    $acc
                );
                if ($old_acc) {
                    $htmaildb{ $this->encode_utf8( $data[3] ) }
                        = $this->encode_utf8($old_acc);
                }
                else {
                    delete $htmaildb{ $this->encode_utf8( $data[3] ) };
                }
            }
            if ($main::gc) {
                my ( $acc, $list, $time, $count );
                $time  = time;
                $count = 0;
                while ( ( $acc, $list ) = each %htauthdb ) {
                    my @data
                        = split( /:/, ( ( defined $list ) ? $list : '' ) );
                    if ( $data[5] && $data[5] < $time ) {
                        delete $htauthdb{ $this->encode_utf8($acc) };
                        my $old_acc = $this->shift_account(
                            $this->decode_utf8(
                                $htmaildb{ $this->encode_utf8( $data[3] ) }
                            ),
                            $acc
                        );
                        if ($old_acc) {
                            $htmaildb{ $this->encode_utf8( $data[3] ) }
                                = $this->encode_utf8($old_acc);
                        }
                        else {
                            delete $htmaildb{ $this->encode_utf8( $data[3] )
                            };
                        }
                        $count++;
                    }
                }
                push(
                    @{ $this->{'dialog'} },
                    $count
                        . ' の期限切れアカウントを削除しました。'
                ) if $count;
            }
            untie(%htauthdb);
            untie(%htmaildb);
            foreach (@delete) {
                $this->delete_data($_);
            }
        }

        #設定書き換え
        if ( @new_data || $main::reg || $main::reis ) {

            #.htaccess チェック
            if ($this->htaccess_is_prepared(
                    $htaccess,         $current_directory,
                    $auth_dbm_enabled, $auto_register_enabled,
                    $auto_reissue_enabled
                )
                )
            {

                #メールによるパスワードの通知
                $this->mail_to_user( \%mailto, \%new_account, \%new_passwd,
                    \%name, \%expiretime )
                    if %mailto;

            }
            else {
                push(
                    @{ $this->{'dialog'} },
                    '必要な設定ファイルを作成できませんでした。パーミッションを調整して下さい。'
                );
            }
        }

        #データベースのバックアップ
        if ($main::backup) {
            my $htauthdb_bak = $htauthdb . '.bak';
            my $htmaildb_bak = $htmaildb . '.bak';
            if ( $main::backup eq 'make' ) {
                if (   $this->auth_dbm_backup( $htauthdb, $htauthdb_bak )
                    && $this->auth_dbm_backup( $htmaildb, $htmaildb_bak ) )
                {
                    push(
                        @{ $this->{'dialog'} },
                        'バックアップを作成しました。'
                    );
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        'データベース・設定 のバックアップに失敗しました。'
                    );
                }
            }
            elsif ( $main::backup eq 'restore' ) {
                if (   $this->auth_dbm_restore( $htauthdb, $htauthdb_bak )
                    && $this->auth_dbm_restore( $htmaildb, $htmaildb_bak ) )
                {
                    push(
                        @{ $this->{'dialog'} },
                        'バックアップを復元しました。'
                    );
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        'バックアップの復元に失敗しました。'
                    );
                }
            }
            elsif ( $main::backup eq 'remove' ) {
                if (   ( !-f $htauthdb_bak || unlink $htauthdb_bak )
                    && ( !-f $htmaildb_bak || unlink $htmaildb_bak ) )
                {
                    push(
                        @{ $this->{'dialog'} },
                        'バックアップを削除しました。'
                    );
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        'バックアップの削除に失敗しました。'
                    );
                }
            }
        }

        #text
    }
    else {
        my ( $passwd_data, $new_comment, @gc, $gc_done );
        local ( *PASS, *CMT );

        #全アカウント名
        if ( open( PASS, $htpasswd ) ) {
            binmode PASS, ":utf8";
            flock( PASS, LOCK_SH );
            my ( $get_data, $line );
            $get_data = 1 if @new_data || @main::delete || $main::gc;
            while ( $line = <PASS> ) {
                my ($acc);
                $acc = $1 if $line =~ /^([^:]+)/;
                next if grep( $_ eq $acc, @main::delete );
                push( @account_list, $acc );
                $passwd_data .= $line if $get_data;
            }
            close(PASS);
            chmod( $env::file_mode, $htpasswd );
        }

        #アカウント重複チェック
        if (%new_account) {
            my @new_account = keys %new_account;
            foreach my $acc (@new_account) {
                utf8::encode($acc) if utf8::is_utf8($acc);
                if (grep( $_ eq $this->decode_utf8( $new_account{$acc} ),
                        @account_list )
                    )
                {
                    $dialog{$acc}
                        .= 'このアカウント ('
                        . $this->decode_utf8( $new_account{$acc} )
                        . ') は既に使用されています。';
                    @new_data
                        = grep( $_ ne $this->decode_utf8($acc), @new_data );
                    delete $new_account{$acc};
                    delete $new_passwd{$acc};
                    delete $mailto{$acc};
                    delete $name{$acc};
                    delete $expiretime{$acc};
                }
                else {
                    @account_list = grep( $_ ne $acc, @account_list );
                    push( @account_list,
                        $this->decode_utf8( $new_account{$acc} ) );
                }
            }
        }
        @account_list = sort @account_list;
        $main::a = 'new' if !@account_list;

        #検索条件
        if (   $main::a ne 'new'
            && defined $main::search
            && $main::search ne '' )
        {
            @search = grep( $_ ne '', split( /\s+/, $main::search ) );
            map( $_ = quotemeta($_), @search )
                if !defined $main::regexp || !$main::regexp;
        }

        #.htpasswd 更新
        if (@new_data) {
            my ($acc);
            foreach $acc (@new_data) {

                #新規登録
                if ( $acc =~ /^_[0-9]+$/ ) {
                    $passwd_data .= $new_account{$acc} . ':'
                        . crypt( $new_passwd{$acc}, $this->salt ) . "\x0A";

                    #更新
                }
                else {
                    unless ( $passwd_data
                        =~ s/^$acc:(.*?)$/(($new_account{$acc}) ? $new_account{$acc} : $acc) . ':' . (($new_passwd{$acc}) ? crypt($new_passwd{$acc}, $this->salt) : $1)/me
                        )
                    {
                        $passwd_data
                            .= $new_account{$acc} . ':'
                            . crypt( $new_passwd{$acc}, $this->salt ) . "\x0A"
                            if $new_account{$acc} && $new_passwd{$acc};
                    }
                }
            }
        }

        #コメント読み込み/更新
        if ( open( CMT, $htcomment ) ) {
            binmode CMT, ":utf8";
            flock( CMT, LOCK_SH );
            my ($line);
            my $t = time;
            while ( $line = <CMT> ) {
                chomp $line;
                my (@data) = split( /\t/, $line, 5 );
                if ( grep( $_ eq $data[0], @main::delete ) ) {
                    $this->delete_data( $data[0] );
                    $new_comment = 1;
                    next;
                }
                if ($main::gc
                    && ((      defined $main::data{'exptime'}{ $data[0] }
                            && $main::data{'exptime'}{ $data[0] }
                            && $main::data{'exptime'}{ $data[0] } < time
                        )
                        || ( $data[4] && $data[4] < time )
                    )
                    )
                {
                    push( @gc, $data[0] );
                    $this->delete_data( $data[0] );
                    $new_comment = 1;
                    next;
                }
                if (   defined $main::data{'acc'}{ $data[0] }
                    && $main::data{'acc'}{ $data[0] } ne ''
                    && !$dialog{ $data[0] } )
                {
                    if (   $main::data{'acc'}{ $data[0] } ne $data[0]
                        || $main::data{'name'}{ $data[0] } ne $data[1]
                        || $main::data{'addr'}{ $data[0] } ne $data[2]
                        || $main::data{'cmt'}{ $data[0] } ne $data[3]
                        || defined $main::data{'exptime'}{ $data[0] } )
                    {
                        $line = join(
                            "\x09",
                            $main::data{'acc'}{ $data[0] },
                            $main::data{'name'}{ $data[0] },
                            $main::data{'addr'}{ $data[0] },
                            $main::data{'cmt'}{ $data[0] },
                            (   (   defined $main::data{'exptime'}{ $data[0] }
                                )
                                ? $main::data{'exptime'}{ $data[0] }
                                : $data[4]
                            )
                        );
                        $new_comment = 1;
                    }
                    $this->delete_data( $data[0] );
                }
                if (@search) {
                    my $dat = join( "\t", @data[ 0 .. 3 ] );
                    push(
                        @search_list,
                        (   (   defined $main::data{'acc'}{ $data[0] }
                                    && $main::data{'acc'}{ $data[0] } ne ''
                            ) ? $main::data{'acc'}{ $data[0] } : $data[0]
                        )
                        )
                        if ( !$main::expired_only
                        || ( $data[4] && $data[4] < $t ) )
                        && (
                        (   !$main::and
                            && grep( eval { $line =~ /$_/ }, @search )
                        )
                        || ( $main::and
                            && grep( eval { $line !~ /$_/ }, @search ) == 0 )
                        );
                }
                elsif ($main::expired_only) {
                    push(
                        @search_list,
                        (   (   defined $main::data{'acc'}{ $data[0] }
                                    && $main::data{'acc'}{ $data[0] } ne ''
                            ) ? $main::data{'acc'}{ $data[0] } : $data[0]
                        )
                    ) if $data[4] && $data[4] < $t;
                }
                $comment_data .= $line . "\x0A";
            }
            close(CMT);
            chmod( $env::file_mode, $htcomment );
        }

        #.htpasswd から期限切れアカウントを削除
        if ( $main::gc && @gc && $passwd_data ) {
            my $count = 0;
            my $gc;
            foreach $gc (@gc) {
                $count++ if $passwd_data =~ s/^$gc:.*?\x0A//m;
            }
            @account_list = grep { my $acc = $_; !grep( $_ eq $acc, @gc ) }
                @account_list;
            $main::a = 'new' if !@account_list;
            if ($count) {
                $gc_done = 1;
                push(
                    @{ $this->{'dialog'} },
                    $count
                        . ' の期限切れアカウントを削除しました。'
                );
            }
        }

        #新規登録
        if (%main::data) {
            my ( $acc, @delete );
            while ( ($acc) = each %{ $main::data{'acc'} } ) {
                if (   defined $main::data{'acc'}{$acc}
                    && $main::data{'acc'}{$acc} ne ''
                    && !$dialog{$acc} )
                {
                    $comment_data .= join(
                        "\x09",
                        $main::data{'acc'}{$acc},
                        $main::data{'name'}{$acc},
                        $main::data{'addr'}{$acc},
                        $main::data{'cmt'}{$acc},
                        (   ( defined $main::data{'exptime'}{$acc} )
                            ? $main::data{'exptime'}{$acc}
                            : ''
                        )
                    ) . "\x0A";
                    $new_comment = 1;
                    push( @delete, $acc );
                }
            }
            foreach (@delete) {
                $this->delete_data($_);
            }
        }

        #書き換え
        if ( $passwd_data || $new_comment ) {

            #.htaccess チェック
            if ($this->htaccess_is_prepared(
                    $htaccess,         $current_directory,
                    $auth_dbm_enabled, $auto_register_enabled,
                    $auto_reissue_enabled
                )
                &&

                #.htpasswd 更新
                (     !$passwd_data
                    || $this->write_file( $htpasswd, \$passwd_data, 1 )
                )
                &&

                #.htcomment 更新
                (     !$new_comment
                    || $this->write_file( $htcomment, \$comment_data, 1 )
                )
                )
            {

                #メールによるパスワードの通知
                $this->mail_to_user( \%mailto, \%new_account, \%new_passwd,
                    \%name, \%expiretime )
                    if %mailto;

            }
            else {
                push(
                    @{ $this->{'dialog'} },
                    '必要な設定ファイルを作成できませんでした。パーミッションを調整して下さい。'
                );
            }
        }
        elsif ( $main::reg || $main::reis ) {
            push(
                @{ $this->{'dialog'} },
                '設定変更に失敗しました。'
                )
                if !$this->htaccess_is_prepared( $htaccess,
                $current_directory, $auth_dbm_enabled,
                $auto_register_enabled, $auto_reissue_enabled );
        }
    }

    #エラー
    my ($error_account);
    if (%dialog) {
        my $acc;
        foreach $acc ( sort keys %dialog ) {
            push(
                @{ $this->{'dialog'} },
                (   ( $acc =~ /^_([0-9]+)$/ )
                    ? '新規アカウント ' . ( $1 + 0 ) . ' の登録'
                    : 'アカウント ' . $acc . ' の更新'
                    )
                    . 'に失敗しました。'
                    . $dialog{$acc}
            );
        }
        $error_account = 1;
    }
    undef %dialog;

    #ページ数
    my ( $lastpage, $pageoption );
    $main::u = 10 if $this->is_nan($main::u) || $main::u > 100;
    $main::p = 1 if $this->is_nan($main::p);

    #編集対象抽出
    my (@target);

    #エラーがある場合
    if ( defined $error_account && $error_account ) {
        @target  = sort keys %{ $main::data{'acc'} };
        $main::u = @target if $main::u < @target;
        $main::a = ( defined $main::ref ) ? $main::ref : '';
        if ( $main::a ne 'new' ) {
            if ($auth_dbm_enabled) {
                my (%htauthdb);
                tie( %htauthdb, 'AnyDBM_File', $htauthdb, O_RDONLY,
                    $env::dbm_mode );
                if ( ref tied %htauthdb ) {
                    foreach my $n (@target) {
                        my @data = split(
                            /:/,
                            $this->decode_utf8(
                                $htauthdb{ $this->encode_utf8($n) }
                            )
                        );
                        $main::data{'exptime'}{$n} = $data[5];
                    }
                }
                untie(%htauthdb);
            }
            else {
                if ( open( CMT, $htcomment ) ) {
                    binmode CMT, ":utf8";
                    flock( CMT, LOCK_SH );
                    my ($line);
                    while ( $line = <CMT> ) {
                        if ( grep( $line =~ /^$_\t/, @target ) ) {
                            chomp $line;
                            my @data = split( /\t/, $line );
                            $main::data{'exptime'}{ $data[0] } = $data[4];
                        }
                    }
                    close(CMT);
                }
            }
        }

        #新規作成
    }
    elsif ( $main::a eq 'new' ) {
        foreach ( 1 .. $main::u ) {
            push( @target, '_' . sprintf( "%.2d", $_ ) );
        }

        #エラーなし
    }
    else {

        #CSV書き出し
        if ( $main::a eq 'csv' ) {
            my $agent
                = ( $ENV{'HTTP_USER_AGENT'} ) ? $ENV{'HTTP_USER_AGENT'} : '';
            my $jcode = '';
            my $r     = "\x0A";
            if ( $agent =~ /win/i ) {
                $jcode = 'sjis';
                $r     = "\x0D\x0A";
            }
            elsif ( $agent =~ /mac/i ) {
                $jcode = 'sjis';
                $r     = "\x0D";
            }
            binmode STDOUT;

            #$this->print_http_header('text/comma-separated-values');
            $this->print_http_header('application/csv');
            if ($auth_dbm_enabled) {
                my (%htauthdb);
                tie( %htauthdb, 'AnyDBM_File', $htauthdb, O_RDONLY,
                    $env::dbm_mode );
                if ( ref tied %htauthdb ) {
                    my ( $k, $v );
                    while ( ( $k, $v ) = each %htauthdb ) {
                        my @data = split( /:/, $v );
                        @data = splice( @data, 2, 3 );
                        my $value = $this->svjoin( ',', $k, @data );
                        $this->jcode_convert( \$value, $jcode, 'utf8' )
                            if $jcode;
                        utf8::encode($value) if utf8::is_utf8($value);
                        print( $value, $r );
                    }
                }
                untie(%htauthdb);
            }
            else {
                if ( open( CMT, $htcomment ) ) {
                    binmode CMT, ":utf8";
                    flock( CMT, LOCK_SH );
                    while (<CMT>) {
                        $_ =~ s/\x09[0-9]*\x0A//;
                        my $v = $this->svjoin( ',', split( /\t/, $_ ) );
                        $this->jcode_convert( \$v, $jcode, 'utf8' ) if $jcode;
                        utf8::encode($v) if utf8::is_utf8($v);
                        print( $v, $r );
                    }
                    close(CMT);
                }
            }
            exit;
        }

        no warnings qw(untie);
        my ( $account_list, $acc, $count, $from, $to, %htauthdb );
        undef %main::data;
        if ($auth_dbm_enabled) {
            @search_list = ();
            if ( @search || $main::expired_only ) {
                tie( %htauthdb, 'AnyDBM_File', $htauthdb, O_RDONLY,
                    $env::dbm_mode );
                if (@search) {
                    if ( $main::field && $main::field eq 'addr' ) {
                        my (%htmaildb);
                        tie( %htmaildb, 'AnyDBM_File', $htmaildb, O_RDONLY,
                            $env::dbm_mode );
                        if ( ref tied %htmaildb ) {
                            foreach my $n (@search) {
                                push(
                                    @search_list,
                                    split(
                                        /,/,
                                        $this->decode_utf8(
                                            $htmaildb{ $this->encode_utf8($n)
                                            }
                                        )
                                    )
                                ) if $htmaildb{ $this->encode_utf8($n) };
                            }
                        }
                        untie(%htmaildb);
                    }
                    else {
                        @search_list = grep( $htauthdb{$_}, @search )
                            if ref tied %htauthdb;
                    }
                    if ( @search_list && $main::expired_only ) {
                        my @temp_list = ();
                        my $t         = time;
                        foreach my $n (@search_list) {
                            push( @temp_list, $n )
                                if $this->decode_utf8(
                                $htauthdb{ $this->encode_utf8($n) } )
                                =~ /([0-9]+)$/ && $1 < $t;
                        }
                        @search_list = @temp_list;
                    }
                }
                else {
                    my ( $k, $v );
                    my $t = time;
                    while ( ( $k, $v ) = each %htauthdb ) {
                        push( @search_list, $k )
                            if $v =~ /([0-9]+)$/ && $1 < $t;
                    }
                }
            }
            @search_list = sort @search_list if @search_list;
            $account_list = \@search_list;
        }
        elsif ( @search || $main::expired_only ) {
            @search_list  = sort @search_list;
            $account_list = \@search_list;
        }
        else {
            $account_list = \@account_list;
        }

        #ページ数判定
        $lastpage
            = ($main::u)
            ? ( ( @$account_list % $main::u )
            ? int( @$account_list / $main::u ) + 1
            : @$account_list / $main::u )
            : 1;
        if ( $main::p < 1 ) {
            $main::p = 1;
        }
        elsif ( $main::p > $lastpage ) {
            $main::p = $lastpage;
        }
        ( $from, $to )
            = ( ( $main::u * ( $main::p - 1 ) ), ( $main::u * $main::p ) )
            if $main::u > 0;
        foreach $acc (@$account_list) {
            $count++;

            #ページ
            if ( $main::u > 0 ) {
                if ( $from >= $count ) {
                    $pageoption = 1;
                    next;
                }
                elsif ( $to < $count ) {
                    $pageoption = 1;
                    last;
                }
            }
            push( @target, $acc );
        }

        #データ抽出
        foreach $acc (@target) {
            $main::data{'acc'}{$acc} = $acc;
            if ($auth_dbm_enabled) {
                my ( $passwd, $group );
                (   $passwd,                   $group,
                    $main::data{'name'}{$acc}, $main::data{'addr'}{$acc},
                    $main::data{'cmt'}{$acc},  $main::data{'exptime'}{$acc}
                    )
                    = split(
                    /:/,
                    $this->decode_utf8(
                        $htauthdb{ $this->encode_utf8($acc) }
                    )
                    );
            }
            elsif ( $comment_data =~ /^$acc\t(.*)$/m ) {
                (   $main::data{'name'}{$acc}, $main::data{'addr'}{$acc},
                    $main::data{'cmt'}{$acc},  $main::data{'exptime'}{$acc}
                ) = split( /\t/, $1 );
            }
        }
        untie(%htauthdb) if $auth_dbm_enabled;
    }
    undef $comment_data if !$auth_dbm_enabled;
    $lastpage = 1 if !defined $lastpage || !$lastpage;

    $auth_enabled = 1
        if ( $auth_dbm_enabled
        && $this->fileglob( $current_directory, '^\.htauthdb', 'f' ) )
        || ( !$auth_dbm_enabled && -f $htpasswd );

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print
        '<h1 title="アクセス制限について設定します。">アクセス制限設定 ('
        . ( ($main::current) ? $main::current : '/' ) . ')'
        . ( ($auto_register_enabled)
        ? '<span style="background-color: #f66">[自動登録 ON]</span>'
        : '' )
        . ''
        . ( ($auto_reissue_enabled)
        ? '<span style="background-color: #ff6">[自動再発行 ON]</span>'
        : '' )
        . '</h1>';

    #フォーム開始
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => [ @setting::form, 'current' ]
    );
    print '<input type="hidden" name="ref" value="' . $main::a . '" />';
    print '<fieldset class="submit">';
    if ($auth_dbm_enabled) {
        print '<select name="field">';
        print '<option value="acc">アカウントが</option>';
        print '<option value="addr"'
            . ( ( $main::field && $main::field eq 'addr' )
            ? ' selected="selected"'
            : '' )
            . '>メールアドレスが</option>';
        print '</select>';
        print
            '<input type="search" placeholder="検索" results="5" autosave="'
            . ( ($default::id) ? $default::id : 'gatekeeper' )
            . '" size="30" name="search" value="'
            . $this->escape($main::search)
            . '" title="検索文字列を含むアカウントのみ抽出します。" />';
        print 'のいずれかに一致するユーザーを';
        print '<input type="submit" value="検索" />';
    }
    else {
        print
            '<input type="search" placeholder="検索" results="5" autosave="'
            . ( ($default::id) ? $default::id : 'gatekeeper' )
            . '" size="30" name="search" value="'
            . $this->escape($main::search)
            . '" title="検索文字列を含むアカウントのみ抽出します。" />';
        print '<select name="regexp">';
        print '<option value="">正規表現不使用</option>';
        print '<option value="1"'
            . ( ($main::regexp) ? ' selected="selected"' : '' )
            . '>正規表現使用</option>';
        print '</select>';
        print '<select name="and">';
        print '<option value="">いくつかに一致</option>';
        print '<option value="1"'
            . ( ($main::and) ? ' selected="selected"' : '' )
            . '>全てに一致</option>';
        print '</select>';
        print '<input type="submit" value="検索" />';
    }
    print ' <label><input type="checkbox" name="expired_only" value="1"'
        . ( ($main::expired_only) ? ' checked="checked"' : '' )
        . ' /> 期限切れアカウントのみ検索</label>';
    print '</fieldset>';

    print '<fieldset><legend>アカウント一覧</legend>';
    if (@target) {

        #ユーザーカテゴリーメニュー
        @options = split( /\x0A/, $setting::general{'options'} )
            if defined $setting::general{'options'}
            && $setting::general{'options'};

        print ' (' . $main::p . '/' . $lastpage . 'ページ)';

        #解析結果テーブル
        require CMONOS::CGI_Lib::Table;
        my $table = Table->new;
        $table->set(
            'tableclass' => 'tooltable',
            'trclass'    => 'odd,even',
            'class0'     => 'odd,even',
            'classF'     => ( ( $main::a ne 'new' ) ? 'caution' : '' ),
            'thenabled'  => 'true'
        );

        print $table->table;
        print $table->tr;
        print $table->td('削除') if $main::a ne 'new';
        print $table->td('アカウント');
        print $table->td('名前');
        print $table->td('メールアドレス');
        print $table->td('コメント');
        print $table->td(
            '有効期限' . ( ( $main::a ne 'new' ) ? '更新' : '' ) );
        print $table->td(
            '有効期限' . ( ( $main::a ne 'new' ) ? '更新' : '' ) );
        print $table->td('パスワード');
        print $table->td('通知');
        print $table->_tr;

        my ($acc);
        foreach $acc (@target) {
            print $table->tr;
            print $table->td( '<input type="checkbox" name="delete" value="'
                    . $acc
                    . '" />' )
                if $main::a ne 'new';
            print $table->td( '<input type="text" name="data:acc:'
                    . $acc
                    . '" value="'
                    . $this->escape( $main::data{'acc'}{$acc} )
                    . '" size="10" />' );
            print $table->td( '<input type="text" name="data:name:'
                    . $acc
                    . '" value="'
                    . $this->escape( $main::data{'name'}{$acc} )
                    . '" size="10" />' );
            print $table->td( '<input type="email" name="data:addr:'
                    . $acc
                    . '" value="'
                    . $this->escape( $main::data{'addr'}{$acc} )
                    . '" size="24" />' );
            print $table->td( $this->account_comment( $acc, \@options ) );
            print $table->td(
                $this->mk_select_menu(
                    'data:lifetime:' . $acc,
                    'パスワードの有効期限を選択します。',
                    '',
                    $main::data{'lifetime'}{$acc},
                    '' => (
                        (   $main::a eq 'new' || !$main::data{'exptime'}{$acc}
                        ) ? '無期限' : $this->date(
                            $main::data{'exptime'}{$acc}, 0,
                            'gmt' . $setting::general{'datetype'}
                            )
                            . (
                            ( $main::data{'exptime'}{$acc} < time )
                            ? '[!]'
                            : ''
                            )
                    ),
                    86400    => '1日',
                    604800   => '1週間',
                    1209600  => '2週間',
                    2678400  => '1ヶ月',
                    5270400  => '2ヶ月',
                    8208000  => '3ヶ月',
                    10540800 => '4ヶ月',
                    13219200 => '5ヶ月',
                    15811200 => '6ヶ月',
                    18489600 => '7ヶ月',
                    21081600 => '8ヶ月',
                    23760000 => '9ヶ月',
                    26352000 => '10ヶ月',
                    29030400 => '11ヶ月',
                    31536000 => '1年',
                    47347200 => '1年6ヶ月',
                    63072000 => '2年',
                    94608000 => '3年',
                    (   ( $main::a ne 'new' ) ? ( 'ever' => '無期限' ) : ()
                    )
                )
            );
            print $table->td( '<input type="text" name="data:expire:'
                    . $acc
                    . '" value="'
                    . $this->escape( $main::data{'expire'}{$acc} )
                    . '" size="10" maxlength="10" title="YYYY/MM/DD" />' );
            print $table->td( '<input type="text" name="data:passwd:'
                    . $acc
                    . '" value="'
                    . $this->escape( $main::data{'passwd'}{$acc} )
                    . '" size="8" maxlength="8" />' );
            print $table->td(
                      '<input type="checkbox" name="data:mail:'
                    . $acc
                    . '" value="1" '
                    . (
                    ( $main::data{'mail'}{$acc} ) ? 'checked="checked"' : ''
                    )
                    . ' />'
            );
            print $table->_tr;
        }
        print $table->_table;
    }
    else {
        print '<div class="warning">'
            . ( ( @search || $main::expired_only )
            ? '検索条件に一致する'
            : '' )
            . 'ユーザーが存在しません。</div>';
    }
    print '</fieldset>';

    print '<fieldset class="submit">';
    print
        '<label for="u" class="datatitle">表示数</label><input type="number" name="u" id="u" value="'
        . $main::u
        . '" size="3" maxlength="3" /> ';
    print '<input type="button" value="'
        . ( ($default::english) ? 'Reset' : 'リセット' )
        . '" onclick="cmonos.form.item(this.form).resetAll(\'data:\')" /> '
        if $main::a eq 'new';
    if ( $main::a eq 'new' ) {
        print '<input type="submit" value="登録→登録済アカウント'
            . ( ($auth_dbm_enabled) ? '検索' : '編集' ) . '" /> ';
    }
    else {
        print
            '<input type="submit" name="a=new" value="更新→新規登録" /> ';
        if ( defined $pageoption && $pageoption ) {
            print '<input type="submit" name="p='
                . ( $main::p - 1 )
                . '&amp;a='
                . $main::a
                . '" value="前のページ" />'
                if $main::p > 1;
            print '<input type="submit" name="p='
                . ( $main::p + 1 )
                . '&amp;a='
                . $main::a
                . '" value="次のページ" />'
                if $main::p < $lastpage;
        }
    }
    if ( $main::a eq 'new' ) {
        print ' <input type="submit" name="p=' . $main::p
            . '&amp;a=new" value="登録" class="save" />';
    }
    else {
        print ' <input type="submit" name="p=' . $main::p
            . '" value="更新" class="save" />';
    }

    print '</fieldset>';
    if ($auth_enabled) {
        print '<fieldset class="submit">';
        print '<ul class="tools"><li>';
        print '</li>';
        if ( !$auto_disabled ) {
            print( ($auto_register_enabled)
                ? '<li class="group-start"><input type="submit" name="reg=off&amp;a='
                    . $main::a
                    . '" value="自動登録 OFF" /></li>'
                : '<li class="group-start"><input type="submit" name="reg=on&amp;a='
                    . $main::a
                    . '" value="自動登録 ON" /></li>' );
            print( ($auto_reissue_enabled)
                ? '<li><input type="submit" name="reis=off&amp;a='
                    . $main::a
                    . '" value="自動再発行 OFF" /></li>'
                : '<li><input type="submit" name="reis=on&amp;a='
                    . $main::a
                    . '" value="自動再発行 ON" /></li>' );
        }
        if ($auth_dbm_enabled) {
            print
                '<li class="group-start"><input type="submit" name="backup=make&amp;a='
                . $main::a
                . '" value="バックアップ作成" onclick="return confirm(\'アカウント情報をテキストファイルに書き出します。この処理には数分かかることがあります。\')" /></li>';
            if ( -f $htauthdb . '.bak' ) {
                print '<li><input type="submit" name="backup=remove&amp;a='
                    . $main::a
                    . '" value="バックアップ削除" onclick="return confirm(\'バックアップファイルを削除します。この作業は取り消しできません。本当に削除しますか？\')" /></li>';
                print '<li><input type="submit" name="backup=restore&amp;a='
                    . $main::a
                    . '" value="バックアップ復元" onclick="return confirm(\'バックアップを復元すると現在のデータが上書きされます。この作業は取り消しできません。本当に復元しますか？\')" /></li>';
            }
        }
        print
            '<li class="group-start"><input type="submit" name="a=csv" value="CSV 書き出し" onclick="cmonos.form.item(this.form).appendDownloadPath(\'register.csv\')" /></li>';
        print '</ul>';
        print '</fieldset>';

        print
            '<fieldset class="delete"><input type="submit" name="gc=1&amp;a='
            . $main::a
            . '" value="期限切れアカウントを削除する" onclick="return confirm(\'期限切れアカウントの情報が全て失われます。本当に解除しますか？\')" /></fieldset>';
        print
            '<fieldset class="delete"><input type="submit" name="a=cancel" value="アクセス制限解除" onclick="return confirm(\'全てのアカウント情報が失われます。本当に解除しますか？\')" /></fieldset>';
    }
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# 入力データの消去
sub delete_data ($$) {
    my ( $this, $acc ) = @_;
    my (@name) = keys %main::data;
    foreach (@name) {
        delete $main::data{$_}{$acc};
    }
}

######################################################################
# アカウントのコメント
sub account_comment ($$;$) {
    my ( $this, $acc, $options ) = @_;
    my ( $menu, $selected );
    if ( ref $options && @$options ) {
        $menu = '<select name="data:cmt:' . $acc . '">';
        foreach (@$options) {
            $menu .= '<option value="' . $_ . '"';
            if ( $_ eq $main::data{'cmt'}{$acc} ) {
                $menu .= ' selected="selected"';
                $selected = 1;
            }
            $menu .= '>' . $_ . '</option>';
        }
        if ( !defined $selected || !$selected ) {
            $menu
                .= '<option value="'
                . $main::data{'cmt'}{$acc}
                . '" selected="selected">'
                . $main::data{'cmt'}{$acc}
                . '</option>';
        }
        $menu .= '</select>';
    }
    else {
        $menu
            = '<input type="text" name="data:cmt:'
            . $acc
            . '" value="'
            . $this->escape( $main::data{'cmt'}{$acc} )
            . '" size="16" />';
    }
    return $menu;
}

######################################################################
# メールアドレス=>アカウント
sub push_account ($$$) {
    my ( $this, $data, $acc ) = @_;
    my @data = split( /,/, ( ( defined $data ) ? $data : '' ) );
    return ( grep( $_ eq $acc, @data ) ) ? $data : join( ',', @data, $acc );
}

sub shift_account ($$$) {
    my ( $this, $data, $acc ) = @_;
    my @data = split( /,/, ( ( defined $data ) ? $data : '' ) );
    @data = grep( $_ ne $acc, @data );
    return join( ',', @data );
}

######################################################################
# Auth DBM チェック
sub get_auth_settings ($$) {
    my ( $this, $htaccess ) = @_;
    my $data = $this->read_file( $htaccess, 1 );
    return (
        (   (   defined $data
                    && $data =~ /^AuthDBMUserFile[\x09\x0A\x0C\x0D\x20]/mi
            ) ? 1
            : ( defined $data
                    && $data =~ /^AuthUserFile[\x09\x0A\x0C\x0D\x20]/mi ) ? 0
            : (        $setting::general{'authmod'}
                    && $setting::general{'authmod'} eq 'dbm' ) ? 1
            : 0
        ),
        (   ( defined $data && $data =~ /^# auto_register_enabled = true$/m )
            ? 1
            : 0
        ),
        (   ( defined $data && $data =~ /^# auto_reissue_enabled = true$/m )
            ? 1
            : 0
        ),
    );
}

######################################################################
# Auth DBM バックアップ
sub auth_dbm_backup ($$$) {
    my ( $this, $db_path, $bak_path ) = @_;
    my ( %db, $done );
    $done = 0;
    tie( %db, 'AnyDBM_File', $db_path, O_RDONLY, $env::dbm_mode );
    if ( ref tied %db && open( BAK, ">>$bak_path" ) ) {
        binmode BAK, ":utf8";
        flock( BAK, LOCK_EX );
        truncate( BAK, 0 );
        seek( BAK, 0, 0 );
        my ( $k, $v );
        while ( ( $k, $v ) = each %db ) {
            print BAK $k . "\x09" . $v . "\x0A" if $k && $v;
        }
        close(BAK);
        chmod( $env::file_mode, $bak_path );
        $done = 1;
    }
    untie(%db);
    return $done;
}

sub auth_dbm_restore ($$$) {
    my ( $this, $db_path, $bak_path ) = @_;
    my ( %db, $done );
    $done = 0;
    tie( %db, 'AnyDBM_File', $db_path, O_RDWR | O_CREAT, $env::dbm_mode );
    if ( ref tied %db && open( BAK, $bak_path ) ) {
        binmode BAK, ":utf8";
        flock( BAK, LOCK_SH );
        while (<BAK>) {
            chomp $_;
            next if !$_;
            my ( $k, $v ) = split( /\x09/, $_, 2 );
            utf8::encode($k) if utf8::is_utf8($k);
            utf8::encode($v) if utf8::is_utf8($v);
            $db{$k} = $v if $k && $v;
        }
        close(BAK);
        chmod( $env::file_mode, $bak_path );
        $done = 1;
    }
    untie(%db);
    return $done;
}

######################################################################
# .htaccess 調整+.htgroup 作成
sub htaccess_is_prepared ($$$;$$$) {
    my ( $this, $htaccess, $current_directory, $auth_dbm_enabled,
        $auto_register_enabled, $auto_reissue_enabled )
        = @_;
    my ( $data, $temp, $htgroup, $setting );
    $htgroup = $current_directory . '/.htgroup';
    $data    = $this->read_file( $htaccess, 1 );
    $temp    = $data;

    $setting = "\x0A# Gate Keeper - start -\x0A";
    $setting .= '# auto_register_enabled = '
        . ( ($auto_register_enabled) ? 'true' : 'false' ) . "\x0A";
    $setting .= '# auto_reissue_enabled = '
        . ( ($auto_reissue_enabled) ? 'true' : 'false' ) . "\x0A";
    if ( $auto_register_enabled || $auto_reissue_enabled ) {
        $setting
            .= 'ErrorDocument 401 '
            . $ENV{'SCRIPT_NAME'}
            . '?m=AutoRegister&refresh=1&path='
            . $this->url_encode($main::current) . "\x0A";
    }
    $setting .= 'AuthType Basic' . "\x0A";
    $setting .= 'AuthName "Member Site"' . "\x0A";
    $setting .= 'Require valid-user' . "\x0A";

    if ($auth_dbm_enabled) {
        $setting .= <<SETTING;
AuthDBMUserFile  $current_directory/.htauthdb
AuthDBMGroupFile $current_directory/.htauthdb

<Files .htauthdb>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htauthdb.pag>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htauthdb.dir>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htauthdb.db>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htauthdb.bak>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htmaildb>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htmaildb.pag>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htmaildb.dir>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htmaildb.db>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htmaildb.bak>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files ~ "^\.[Hh][Tt]">
  Order allow,deny
  Deny from all
  Satisfy All
</Files>

SETTING

    }
    else {
        $setting .= <<SETTING;
AuthUserFile  $current_directory/.htpasswd
AuthGroupFile $current_directory/.htgroup

<Files .htaccess>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htpasswd>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htgroup>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files .htcomment>
  Order allow,deny
  Deny from all
  Satisfy All
</Files>
<Files ~ "^\.[Hh][Tt]">
  Order allow,deny
  Deny from all
  Satisfy All
</Files>

SETTING
    }
    $setting .= "# Gate Keeper - end -\x0A";
    $data .= $setting
        unless $data
        =~ s/\x0A# Gate Keeper - start -\x0A.*?\x0A# Gate Keeper - end -\x0A/$setting/s;

    #.htaccess 書き込み
    unless ( bytes::length $temp == bytes::length $data
        || $this->write_file( $htaccess, \$data, 1 ) )
    {
        return 0;
    }

    #.htgroup 作成
    undef $temp;
    $data = '';
    if (   $auth_dbm_enabled
        || -f $htgroup
        || $this->write_file( $htgroup, \$data, 1 ) )
    {
        return 1;
    }
    else {
        return 0;
    }
}

######################################################################
# .htaccess アクセス制限解除
sub auth_cancel ($$$) {
    my ( $this, $htaccess, $current_directory ) = @_;
    my ($data);
    local (*FH);
    return 1 unless -f $htaccess;

    $data = $this->read_file( $htaccess, 1 );
    $data
        =~ s/\x0A# Gate Keeper - start -\x0A.*?\x0A# Gate Keeper - end -\x0A//s;
    $data =~ s/[\x0A]+/\x0A/sg;
    if ( !$data || $data =~ /^[\x09\x0A\x0C\x0D\x20]+$/s ) {
        return unlink($htaccess);
    }
    else {
        return $this->write_file( $htaccess, \$data, 1 );
    }
}

sub delete_auth_dbm ($$) {
    my ( $this, $current_directory ) = @_;
    my @db = $this->fileglob( $current_directory, '^\.(htauthdb|htmaildb)',
        'f' );
    return ( !@db || unlink @db ) ? 1 : 0;
}

######################################################################
# メール送信
sub mail_to_user ($$$$$$) {
    my ( $this, $mailto, $new_account, $new_passwd, $name, $expiretime ) = @_;
    my ( $acc, $url );
    $url = $this->http_my_domain . $main::current;

    if (   !defined $setting::general{'msg'}
        || $setting::general{'msg'} !~ /<account>/i
        || $setting::general{'msg'} !~ /<password>/i )
    {
        $setting::general{'msg'} .= <<MSG;
$url
以下の階層にアクセスするときは、下記アカウントおよびパスワードを
入力してください。

────────────────────────────────
アカウント：<account>
パスワード：<password>
────────────────────────────────

MSG
    }
    else {
        $setting::general{'msg'} =~ s/<url>/$url/ig;
    }
    if (   !defined $setting::general{'subject'}
        || !$setting::general{'subject'} )
    {
        $setting::general{'subject'} = 'お知らせ';
    }
    else {
        $setting::general{'subject'} =~ s/<url>/$url/ig;
    }

    unless ( ref $cgi::SendMail ) {
        require CMONOS::CGI_Lib::SendMail;
        $cgi::SendMail = SendMail->new(
            ( ( defined $env::sendmail )     ? $env::sendmail     : '' ),
            ( ( defined $env::master_email ) ? $env::master_email : '' ),
            '',
            ( ( defined $cgi::input ) ? $cgi::input : '' ),
            1
        );
    }

    my $from = (
        ( $setting::general{'master'} )
        ? ( ( $setting::general{'mastername'} )
            ? $setting::general{'mastername'} . ' <'
                . $setting::general{'master'} . '>'
            : $setting::general{'master'} )
        : $env::master_email
    );

    while ( ($acc) = each %$mailto ) {
        my $account
            = ( defined $$new_account{$acc} && $$new_account{$acc} ne '' )
            ? $$new_account{$acc}
            : $acc;
        my $password
            = ( defined $$new_passwd{$acc} && $$new_passwd{$acc} ne '' )
            ? $$new_passwd{$acc}
            : 'これまでと同じパスワード';
        my $name
            = ( defined $$name{$acc} && $$name{$acc} ne '' )
            ? $$name{$acc}
            : $account;
        my $expire
            = ( defined $$expiretime{$acc} && $$expiretime{$acc} )
            ? $this->date( $$expiretime{$acc}, 0,
            'gmt' . $setting::general{'datetype'} )
            : '無期限';

        my $msg = $setting::general{'msg'};
        $msg =~ s/<name>/$name/ig;
        $msg =~ s/<account>/$account/ig;
        $msg =~ s/<password>/$password/ig;
        $msg =~ s/<expire>/$expire/ig;

        my $subject = $setting::general{'subject'};
        $subject =~ s/<name>/$name/ig;
        $subject =~ s/<account>/$account/ig;
        $subject =~ s/<password>/$password/ig;
        $subject =~ s/<expire>/$expire/ig;

        unless (
            $cgi::SendMail->send_mail(
                'To' => (
                    ( $name ne '' )
                    ? $name . ' <' . $$mailto{$acc} . '>'
                    : $$mailto{$acc}
                ),
                'From'    => $from,
                'Subject' => $subject,
                'message' => $msg,
                'info'    => '0'
            )
            )
        {
            push(
                @{ $this->{'dialog'} },
                'メールを送信できませんでした（'
                    . $account
                    . '）。'
            );
            push( @{ $this->{'dialog'} }, $cgi::SendMail->dialog );
        }
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 'k201', 'GateKeeper' );

    #デフォルトメニュー登録
    $setting::selected_jump_menu{'k201'}
        = 'k200';    #メインメニューのデフォルトキー

}

1;
