package Publish;
require CMONOS::HomePageMaker::Editor;
@ISA = qw(Editor);
use strict;
use utf8;
######################################################################
# 自動公開モジュール
# Publish.pm ver1.0.1
# Copyright (C) 2010-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新：2016-07-05
######################################################################
# メイン
sub run ($) {
    my $this = shift;
    $main::debug = 0;
    my $log   = '';
    my $count = 0;
    my $today = $this->date_id;
    if ( !$setting::general{'autoPublishingEnabled'} ) {
        push(
            @{ $this->{'dialog'} },
            '公開予約機能が許可されていません。'
        );
        $this->print_response;
    }
    elsif ($setting::general{'lastAutoPublishedDate'}
        && $setting::general{'lastAutoPublishedDate'} == $today )
    {
        if ($main::debug) {
            push(
                @{ $this->{'dialog'} },
                '今日公開予定の記事は既に公開済みです。'
            );
        }
        else {
            $this->print_response;
        }
    }

    #必要なモジュール
    use Fcntl qw(:DEFAULT :flock);
    eval 'use ' . $env::dbm_file . ';';
    use File::Path;
    require CMONOS::CGI_Lib::Template;

    #サブグループがある場合のホームディレクトリ
    $master::home = $default::home;
    if ( defined $default::subdivided && $default::subdivided ) {
        if (   !defined $default::home
            || $default::home eq ''
            || !defined $main::c
            || $main::c eq '' )
        {
            push(
                @{ $this->{'dialog'} },
                'フォルダが指定されていません。'
            );
            $this->print_response;
        }
        elsif (defined $main::c
            && $main::c ne ''
            && -d $default::preference_dir )
        {
            $default::home .= '/' . $main::c;
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '指定されたフォルダがみつかりませんでした。'
            );
            $this->print_response;
        }
    }
    else {
        undef $main::c;
    }

    #記事ファイル拡張子
    $default::ext = ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html';

    #日付管理モード設定値調整
    if ( defined $default::date && $default::date ) {

        #カレンダーディレクトリ
        $calendar::path = $default::home . '/calendar';
        $this->set_dir($calendar::path);

        #履歴ディレクトリ
        $history::path = $default::home . '/history';
        $this->set_dir($history::path);

        #カレンダーオブジェクト作成
        $this->mk_calendar_obj;
    }

    #総合インデックス情報読み込み
    if (   defined $default::subdivided
        && defined $main::c
        && $default::subdivided
        && $main::c ne '' )
    {
        $this->import_db( $default::master_preference_dir . '/page',
            \%setting::page, 1, 'masterindex', 'metanavi', '!' )
            if $main::w
            || ( defined $main::delete && $main::delete ne '' )
            || $main::a eq 'refresh';
    }

    my @pending = ();
    if ( opendir( DIR, $default::home ) ) {
        no warnings qw(numeric);
        @pending
            = sort { $a <=> $b || $a cmp $b }
            grep( $_ =~ /^[0-9a-zA-Z_]+\.pending$/
                && -d "$default::home/$_"
                && -f "$default::home/$_/publish.dat"
                && $this->read_file( "$default::home/$_/publish.dat", 1 )
                <= $today,
            readdir(DIR) );
        closedir(DIR);
    }
    else {
        push(
            @{ $this->{'dialog'} },
            'ホームディレクトリの設定を確認してください。'
        );
        $this->print_response;
    }
    if ($main::debug) {
        push(
            @{ $this->{'dialog'} },
            '公開予定日を過ぎた記事が存在しません。'
        ) if !@pending;
    }

#-----------------------------------------------------------------------------
#非公開|公開　切り替え
    foreach my $dir (@pending) {
        local $main::n = $dir;
        $main::n =~ s/\.pending$//;
        if ( defined $default::date && $default::date ) {
            $this->d_article_scan($main::n);
        }
        else {
            $this->article_scan($main::n);
        }
        if ( -d "$default::home/$main::n.pending" ) {
            unlink "$default::home/$main::n.pending/.htaccess";
            unlink "$default::home/$main::n.pending/publish.dat";
            rename(
                "$default::home/$main::n.pending",
                "$default::home/$main::n"
            );
            if ( opendir( DIR, $default::home . '/' . $main::n ) ) {
                no warnings qw(numeric);
                local ( $article::title, $article::description );
                my (%counter);
                my $public_path
                    = $this->path_change( "$default::home/$main::n",
                    $ENV{'SCRIPT_NAME'}, 'abs' );
                my $pending_path = $public_path . '.pending/';
                my $public_url   = $this->full_path($public_path);
                my $pending_url  = $public_url . '.pending/';
                $public_path .= '/';
                $public_url  .= '/';
                my $public_url_encoded  = $this->url_encode($public_url);
                my $pending_url_encoded = $this->url_encode($pending_url);
                my $public_fla_encoded  = $this->url_encode(
                    $this->base_change(
                        "$default::home/$main::n",
                        $env::javascript . '/player'
                    )
                );
                my $pending_fla_encoded = $this->url_encode(
                    $this->base_change(
                        "$default::home/$main::n.pending",
                        $env::javascript . '/player'
                    )
                );
                $this->article_path_replace(
                    "$default::home/$main::n",
                    $pending_path,
                    $public_path,
                    [   $pending_url         => $public_url,
                        $pending_url_encoded => $public_url_encoded,
                        $pending_fla_encoded => $public_fla_encoded,
                    ]
                );
                @article::pending = grep( $_ ne $main::n, @article::pending )
                    if $default::date && @article::pending;
                $article::count++;
                $this->use_field( 'info',
                    $default::field_pref_dir . '/infofield',
                    2, 0, $setting::field_tag_enabled, 'info' )
                    if !%info::obj;
                @page::list
                    = sort { $a <=> $b || $a cmp $b }
                    grep( -f "$default::home/$dir/$_"
                        && $_ =~ s/^([0-9]+)\.(?:shtml|html|htm|shtm)$/$1/i
                        && !$counter{$_}++,
                    readdir(DIR) );
                closedir(DIR);

                foreach my $p (@page::list) {
                    my ( $output, $article, $new_template, %link );
                    local ( %main::data, $main::p, $main::chap, $main::sect,
                        $cgi::output, $cgi::output_option );
                    $cgi::output = $output
                        = $default::home . '/'
                        . $main::n . '/'
                        . $p . '.'
                        . $default::ext;
                    $article = $this->read_html_file($output);
                    next
                        if $article !~ /<!--chapter(?![0-9a-zA-Z_])[^>]*-->/s;
                    $article::title = $this->get_article_title( \$article )
                        if !defined $article::title || $article::title eq '';
                    $article::title
                        = ($default::date)
                        ? $this->date( substr( $main::n, 0, 8 ),
                        0, $setting::general{'datetype'} )
                        : $main::n
                        if !defined $article::title || $article::title eq '';
                    $article::description
                        = $this->get_article_description( \$article )
                        if !defined $article::description
                        || $article::description eq '';
                    $main::p = 'i';
                    $this->mk_template_obj;
                    $new_template = $this->{'template'}->refresh(
                        \$article,
                        \$article,
                        $default::home . '/' . $main::n . '/i',
                        'detail',
                        $output,
                        '',
                        $this,
                        \&Editor::index_and_rss_refresh,
                        $main::c,
                        $main::n
                    );
                    $article::title
                        = ($default::date)
                        ? $this->date( substr( $main::n, 0, 8 ),
                        0, $setting::general{'datetype'} )
                        : $main::n
                        if !defined $article::title || $article::title eq '';
                    $main::p = $p;
                    $this->mk_template_obj;
                    $article = $this->{'template'}->refresh(
                        \$article,
                        \$new_template,
                        $default::home . '/' . $main::n . '/' . $main::p,
                        'detail',
                        $output,
                        '',
                        $this,
                        \&Editor::index_and_rss_refresh,
                        $main::c,
                        $main::n
                    );
                    ( $article::prev_title, $article::next_title )
                        = $this->a_neighbor_link_replace(
                        \$article,
                        (   ($article::prev)
                            ? "$default::home/$article::prev"
                            : ''
                        ),
                        (   ($article::next)
                            ? "$default::home/$article::next"
                            : ''
                        ),
                        $output, '',
                        $article::title,
                        \%link,
                        $article::prev_title,
                        $article::next_title
                        );

                    if ( $p eq $page::list[0] ) {
                        $this->replace_link_rel( \$article,
                            { 'prev' => $link{'prev'} } )
                            if ref $link{'prev'};
                    }
                    elsif ( $p eq $page::list[$#page::list] ) {
                        $this->replace_link_rel( \$article,
                            { 'next' => $link{'next'} } )
                            if ref $link{'next'};
                    }
                    $this->write_html_file(
                        $output,
                        $this->tag_replace(
                            $article,               \$article::title,
                            \$article::description, $main::p,
                            $cgi::output,           $cgi::output_option,
                        ),
                        1,
                        'article',
                    );
                }
                $this->mk_calendar($main::n) if $default::date;
                $log
                    .= '<li>「<a href="'
                    . $this->full_path($default::home) . '/'
                    . $main::n . '/1.'
                    . $default::ext
                    . '" title="'
                    . $this->escape($article::description) . '">'
                    . $article::title
                    . '</a>」を公開しました。</li>';
                $count++;
            }
        }
    }

    my %general;
    tie( %general, $env::dbm_file, $default::preference_dir . '/general',
        O_RDWR, $env::dbm_mode );
    if ( ref tied %general ) {
        $general{'lastAutoPublishedDate'} = $this->encode_utf8($today);
    }
    untie %general;
    $this->print_response( '<ol>' . $log . '</ol>', $count );
}

sub print_response ($$$) {
    my ( $this, $log, $count ) = @_;
    $main::res = 'rss' if !$main::res;
    if ( $main::res eq 'img' ) {
        binmode STDOUT;
        print 'Content-type: image/gif', "\x0A";
        print 'Content-length: 43',      "\x0A";
        print 'Cache-Control: no-cache', "\x0A";
        print 'Pragma: no-cache',        "\x0A";
        print "\x0A";
        print
            "GIF89a\x01\x00\x01\x00\x80\x00\x00\xFF\xFF\xFF\x00\x00\x00\x21\xF9\x04\x05\x14\x00\x00\x00\x2C\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00\x3B";
    }
    elsif ( $main::res eq 'js' ) {
        binmode STDOUT;
        print 'Content-type: text/javascript', "\x0A";
        print 'Content-length: 1',             "\x0A";
        print 'Cache-Control: no-cache',       "\x0A";
        print 'Pragma: no-cache',              "\x0A";
        print "\x0A\x0A";
    }
    else {
        my $link
            = $this->full_path( $default::home . '/index.' . $default::ext );
        my $gmt = gmtime(time);
        $gmt
            =~ s/^([a-zA-Z]+)[\x09\x0A\x0C\x0D\x20]+([a-zA-Z]+)[\x09\x0A\x0C\x0D\x20]+([0-9]{1,2})[\x09\x0A\x0C\x0D\x20]+([0-9]{2}:[0-9]{2}:[0-9]{2})[\x09\x0A\x0C\x0D\x20]+([0-9]{4})/$1, $3 $2 $5 $4/;
        $gmt .= ' GMT';
        binmode STDOUT, ":utf8";
        print 'Content-type: text/xml', "\x0A";
        print "\x0A";
        print '<?xml version="1.0" encoding="UTF-8"?>', "\x0A";
        print
            '<rss version="2.0" xmlns:content="http://purl.org/rss/1.0/modules/content/">',
            "\x0A";
        print '<channel>',     "\x0A";
        print '<description>', $setting::general{'cginame'} . '更新情報',
            '</description>', "\x0A";
        print '<docs>http://blogs.law.harvard.edu/tech/rss</docs>', "\x0A";
        print '<generator>http://cmonos.jp/</generator>',           "\x0A";
        print '<language>ja-JP</language>',                         "\x0A";
        print '<lastBuildDate>', $gmt, '</lastBuildDate>', "\x0A";
        print '<link>',    $link, '</link>',    "\x0A";
        print '<pubDate>', $gmt,  '</pubDate>', "\x0A";
        print '<title>', $setting::general{'cginame'} . '更新情報',
            '</title>', "\x0A";

        if ( ref $this->{'dialog'} && @{ $this->{'dialog'} } ) {
            my $count = 0;
            my $description
                = '<ol><li>'
                . join( '</li><li>', @{ $this->{'dialog'} } )
                . '</li></ol>';
            print '<item>',                     "\x0A";
            print '<content:encoded><![CDATA[', $description,
                ']]></content:encoded>', "\x0A";
            print '<description>', $this->detag($description),
                '</description>', "\x0A";
            print '<guid isPermaLink="true">', $link, '?error', time,
                '</guid>', "\x0A";
            print '<link>',    $link, '</link>',    "\x0A";
            print '<pubDate>', $gmt,  '</pubDate>', "\x0A";
            print '<title>',
                $setting::general{'cginame'} . '更新情報(エラー)',
                '</title>', "\x0A";
            print '</item>', "\x0A";
        }
        if ($count) {
            print '<item>',                     "\x0A";
            print '<content:encoded><![CDATA[', $log,
                ']]></content:encoded>', "\x0A";
            print '<description>', $this->detag($log), '</description>',
                "\x0A";
            print '<guid isPermaLink="true">', $link, '?', time, '</guid>',
                "\x0A";
            print '<link>',    $link, '</link>',    "\x0A";
            print '<pubDate>', $gmt,  '</pubDate>', "\x0A";
            print '<title>',
                  $setting::general{'cginame'}
                . '更新情報('
                . $count
                . '件)', '</title>', "\x0A";
            print '</item>', "\x0A";
        }
        print '</channel>', "\x0A";
        print '</rss>',     "\x0A";
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ( !$default::subdivided || $main::c ) {
        no warnings qw(once);

        #設定DB読み込み設定
        $setting::general  = 1;    #一般設定読み込み
        $setting::deco     = 1;    #テキスト装飾設定読み込み
        $setting::security = 1;    #アクセス権設定読み込み
        $setting::table    = 1;    #テーブル設定読み込み
        $setting::page     = 1;    #ページ設定読み込み

        $setting::field_db_name = 'inputfield';    #フィールド設定DB
        $setting::field         = 2
            ; #フィールド設定読み込み（2=オリジナルも読み込む）
        $setting::field_style_disabled = 0
            ; #2=フォント設定スタイル設定を無視/1=フォント設定のみ無視
        $setting::field_tag_enabled = 'tag'
            ;   #tag=タグのみ/all=全ての設定/no=タグをつけない

        #設定種キー設定
        $setting::security_key
            = 'publish';    #デフォルトセキュリティキー
        $setting::security{'publish'}{'passwordCheck'} = 'nocheck';

        #デフォルトキー
        $main::k = 'publish';

        #クラス登録
        Common::class_register( 'publish', 'Publish' );

    }
}

1;
