package ArticleSweeper;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# 記事整理モジュール
# ArticleSweeper.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( $error, @folder, %year );

    #必要なモジュール
    use File::Path;

    #サブグループがある場合のホームディレクトリ
    if ( defined $default::subdivided && $default::subdivided ) {
        $default::home = '.'
            if !defined $default::home || $default::home eq '';
        if (   defined $main::c
            && $main::c ne ''
            && -d "$default::home/$main::c" )
        {
            $default::home .= '/' . $main::c;
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '指定されたフォルダがみつかりませんでした。'
            );
            $error++;
        }
    }
    $error++ if !defined $default::date || !$default::date;

    #記事リスト
    if ( opendir( DIR, $default::home ) ) {
        @folder
            = grep( -d "$default::home/$_"
                && $_ =~ /^[0-9]{10}(?:\.pending)?$/,
            readdir(DIR) );
        closedir(DIR);
    }
    else {
        push(
            @{ $this->{'dialog'} },
            'ホームディレクトリの設定を確認してください。'
        );
        $error++;
    }

    #一括削除
    if ( defined $main::delete && $main::delete ) {
        my ( @delete, $delerror, @temp, $oldest );
        no warnings qw(numeric);
        foreach ( sort { $a <=> $b } @folder ) {
            if ( $_ =~ /^([0-9]{4})[0-9]{6}(?:\.pending)?$/ ) {
                if ( $1 <= $main::delete ) {
                    push( @delete, $_ );
                }
                else {
                    $oldest = $_ if !defined $oldest || !$oldest;
                    push( @temp, $_ );
                }
            }
        }
        @folder = @temp;

        #記事削除
        foreach (@delete) {
            $delerror++ unless rmtree "$default::home/$_";
        }

        #後記事からのリンクを修正
        $this->rm_link( $default::home . '/' . $oldest, 'prev' ) if $oldest;

        #履歴とカレンダーの削除
        if ( defined $default::date && $default::date ) {

            #履歴の削除
            $this->remove( "$default::home/history", $main::delete, '',
                'prevpage' );

            #カレンダーの削除
            $this->remove( "$default::home/calendar", $main::delete - 1,
                $main::delete );

        }

        if ( defined $delerror && $delerror ) {
            push(
                @{ $this->{'dialog'} },
                '記事の削除に失敗しました（'
                    . $delerror
                    . '）。'
            );
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '指定された記事を削除しました。'
            );
        }
    }

    #年
    foreach (@folder) {
        $year{ substr( $_, 0, 4 ) }++;
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    if ( defined $error && $error ) {
        print 'エラー！';
        $this->print_page_footer('tools');
        exit;
    }

    print
        '<h1>記事の整理</h1><p>指定された年以前の記事を一括削除します。履歴は削除されますが、インデックスからのリンクは削除されません。</p>';

    if (%year) {
        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'input',
            'method'   => 'post',
            'required' => \@setting::form
        );
        print '<fieldset class="delete">';
        print
            '<select name="delete" title="一括削除する記事を指定します。（）は削除対象となる記事の総数です。"><option value="">↓削除する範囲を選択</option>';

        my ($count);
        foreach ( sort { $a <=> $b } keys %year ) {
            $count += $year{$_};
            print '<option value="'
                . $_ . '">'
                . $_
                . ' 年以前の記事（ '
                . $count
                . ' 件）</option>';
        }

        print
            '</select> を <input type="submit" value="一括削除する" onclick="return confirm(\'この操作は取り消せません。本当に削除しますか？\');" />';
        print '</fieldset></form>';
    }
    else {
        print '<p>記事が見つかりません。</p>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# 履歴/カレンダーの削除
sub remove ($$$;$$) {
    my ( $this, $dir, $delete, $del, $link ) = @_;
    my ( $oldest, @folder );
    if ( opendir( DIR, $dir ) ) {
        no warnings qw(numeric);
        @folder = sort { $a <=> $b }
            grep( -f "$dir/$_" && $_ =~ /^[0-9]{6}/, readdir(DIR) );
        closedir(DIR);
    }
    else {
        return;
    }

    #削除
    foreach (@folder) {
        if ( $_ =~ /^([0-9]{4})[0-9]{2,4}\./ ) {
            if ( $1 <= $delete ) {
                unlink "$dir/$_";
            }
            elsif ( $del && $1 <= $del ) {
                unlink "$dir/$_";
                if ( open( OUT, ">>$dir/$_" ) ) {
                    binmode OUT;
                    flock( OUT, LOCK_EX );
                    truncate( OUT, 0 );
                    seek( OUT, 0, 0 );
                    print OUT '';
                    close(OUT);
                }
            }
            elsif ( !defined $oldest || $oldest eq '' ) {
                $oldest = $_;
            }
            else {
                last;
            }
        }
    }

    #リンクを修正
    $this->_rm_link( $dir . '/' . $oldest, $link ) if $oldest && $link;
}

######################################################################
# リンクの削除
sub rm_link ($$$) {
    my ( $this, $dir, $link ) = @_;
    my (@folder);
    if ( opendir( DIR, $dir ) ) {
        @folder = grep( -f "$dir/$_" && $_ =~ /^[0-9]+/, readdir(DIR) );
        closedir(DIR);
    }
    else {
        return;
    }

    #削除
    foreach (@folder) {
        $this->_rm_link( "$dir/$_", $link );
    }
}

sub _rm_link ($$) {
    my ( $this, $path, $link ) = @_;
    my ($data);
    local (*FILE);
    local (*OUT);

    if ( open( FILE, $path ) ) {
        binmode FILE, ":utf8";
        flock( FILE, LOCK_EX );
        $data = join( '', <FILE> );
        close(FILE);
    }
    else {
        return 0;
    }

    $data =~ s/<\!--$link(?![0-9a-zA-Z_])[^>]*-->.*?<\!--\/$link-->//sg;

    return if !defined $data || $data eq '';
    if ( open( OUT, ">>$path" ) ) {
        binmode OUT, ":utf8";
        flock( OUT, LOCK_EX );
        truncate( OUT, 0 );
        seek( OUT, 0, 0 );
        print OUT $data;
        close(OUT);
        return 1;
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ($default::date
        && ( !$default::subdivided
            || ( $default::subdivided && defined $main::c && $main::c ne '' )
        )
        )
    {

        #クラス登録
        Common::class_register( 'H200', 'ArticleSweeper' );

        #メニュー登録
        $setting::jump_menu{'H200'} = '記事の整理'
            if !defined $setting::jump_menu{'H200'};

    }
}

1;
