package Exporter;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# 記事書き出しモジュール
# Exporter.pm ver1.0.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-03-31
######################################################################
# 実行
sub run ($) {
    my $this = shift;

    if ( defined $main::type && $main::type ne '' ) {

        #テキストをデータベースに変換
        require CMONOS::CGI_Lib::Table;
        my $export = Table->new;

        $export->set_encoding();
        binmode STDOUT;
        if ($main::for_excel) {
            $this->print_http_header('application/vnd.ms-excel');
        }
        elsif ( $main::type eq 'tsv' ) {
            $this->print_http_header('application/tab-separated-values');
        }
        else {
            $this->print_http_header('application/csv');
        }

        $this->use_field( 'field', $default::field_pref_dir . '/inputfield',
            2 )
            if !%field::obj;
        require CMONOS::CGI_Lib::Template;
        my $template = Template->new( \%field::obj, \%field::for,
            $frame::obj{'frame'} );
        $template->set(
            'field_style_disabled' => 0,
            'field_tag_name'       => 'field',
            'chapter'              => 'chapter',
            'section'              => 'section',
        );
        my $article = $this->read_file($default::home);

    CHAPTER:
        while ( $article
            =~ /<!--chapter(?![0-9a-zA-Z_])[^>]*-->(.*?)<!--\/chapter-->/sg )
        {
            my $chapter = $1;

        SECTION:
            while ( $chapter
                =~ /<!--section(?![0-9a-zA-Z_])[^>]*-->(.*?)<!--\/section-->/sg
                )
            {
                my $section = $1;
                my @data    = ();

            FIELD:
                while ( $section
                    =~ /<!--field(?![0-9a-zA-Z_])[^>]*[\x09\x0A\x0C\x0D\x20]+id="([0-9]+)"[^>]*-->(.*?)<!--\/field-->/sg
                    )
                {
                    my ( $id, $data ) = ( $1, $2 );
                    my %data
                        = $template->tag_to_data( \$data, $field::obj{$id} );
                    my $str = '';
                    if ( $field::obj{$id}->{'elements'} ) {
                        while ( my ( $k, $v ) = each %data ) {
                            next
                                if $k eq 'str'
                                || $k eq 'comment'
                                || !defined $v;
                            $v = join( "\0", @$v )
                                if ref $v && ref $v eq 'ARRAY';
                            $v =~ s/\x0A/\&#10;/sg;
                            $str .= $k . ':' . $v . "\x0A";
                        }
                        if ( defined $data{'comment'}
                            && $data{'comment'} ne '' )
                        {
                            $str .= "\x0A" if !defined $str || $str eq '';
                            $str .= "\x0A";
                            $str .= $data{'comment'};
                        }
                    }
                    elsif ( defined $data{'str'} ) {
                        if ( defined $field::obj{$id}->{'color'}
                            && $field::obj{$id}->{'color'} )
                        {
                            my $fontcolor = $field::obj{$id}
                                ->text_color( \$data{'str'} );
                            $str = $data{'str'}
                                . ( ($fontcolor) ? '<>' . $fontcolor : '' );
                        }
                        elsif (ref $data{'str'}
                            && ref $data{'str'} eq 'ARRAY' )
                        {
                            $str = join( "\0", @{ $data{'str'} } );
                        }
                        else {
                            $str = $data{'str'};
                        }
                    }
                    push( @data, $str );
                }
                print $export->array_to_sv( $main::type, @data );
            }
        }
        exit;
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>ファイル書き出し</h1>';

    my $filename = $default::id || 'exported';
    $filename =~ s/^_//;
    $filename .= '_data';
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'path'     => '/.dl/' . $filename . '.csv',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print
        '<fieldset class="submit"><input type="submit" name="type=csv" value="カンマ区切りテキスト（Excel形式）" /> <label><input type="checkbox" name="for_excel" value="1"'
        . ( ($main::for_excel) ? ' checked="checked"' : '' )
        . ' /> Excel ファイルとして書き出す</label></fieldset></form>';
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'path'     => '/.dl/' . $filename . '.tab',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print
        '<fieldset class="submit"><input type="submit" name="type=tsv" value="タブ区切りテキスト（FileMaker形式）" /></fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');

}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 'h211', 'Exporter' );

    #メニュー登録
    $setting::jump_menu{'h211'} = '書き出し'
        if !defined $setting::jump_menu{'h210'};
}

1;
