package NaviLayout;
require CMONOS::CGI_Lib::Tools::Preference;
@ISA = qw(Preference);
use strict;
use utf8;
######################################################################
# ナビゲーションレイアウトモジュール
# NaviLayout.pm ver1.0.5
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 更新日：2013-12-18
######################################################################
# 設定チェック項目
sub get_setting_items ($) {
    my $this = shift;

    #チェックボックス
    push( @check::checkbox, 'accestat', 'icontable' );

    if ( defined $env::inline_navi && $env::inline_navi ) {
        push( @check::checkbox,
            'gsearchenabled', 'gsearchfixed',    'fontweight',
            'textshadow',     'sitetitleshadow', 'colortabbgrepeatx',
            'currentdisabled' );
        push( @check::notag, 'navi_gsearchtitle', 'navi_gsearchlabel' );

        $check::color{'bgcolor'} = 'アイコン背景色'
            if !defined $check::color{'bgcolor'};
        $check::color{'tabbgcolor'} = 'タブ背景色'
            if !defined $check::color{'tabbgcolor'};
        $check::color{'sitetitlecolor'} = 'サイトタイトル色'
            if !defined $check::color{'sitetitlecolor'};
        $check::color{'sitesubtitlecolor'} = 'サイトの説明 文字色'
            if !defined $check::color{'sitesubtitlecolor'};

        $check::num{'iconbgwidth'} = 'アイコン背景幅'
            if !defined $check::num{'iconbgwidth'};
        $check::num{'tabbgwidth'} = 'タブ背景幅'
            if !defined $check::num{'tabbgwidth'};
        $check::num{'tabwidth'} = 'タブ領域幅'
            if !defined $check::num{'tabwidth'};
        $check::num{'size'} = 'ナビゲーション高さ'
            if !defined $check::num{'size'};
        $check::num{'tabmargintop'} = 'タブ全体上余白'
            if !defined $check::num{'tabmargintop'};
        $check::num{'tabmarginleft'} = 'タブ全体左右余白'
            if !defined $check::num{'tabmarginleft'};
        $check::num{'tabpaddingtop'} = 'タブ文字列上余白'
            if !defined $check::num{'tabpaddingtop'};
        $check::num{'tabpaddingbottom'} = 'タブ文字列下余白'
            if !defined $check::num{'tabpaddingbottom'};
        $check::num{'tabpaddingleft'} = 'タブ文字列左余白'
            if !defined $check::num{'tabpaddingleft'};
        $check::num{'tabpaddingright'} = 'タブ文字列右余白'
            if !defined $check::num{'tabpaddingright'};
        $check::num{'tabspace'} = 'タブ間隔'
            if !defined $check::num{'tabspace'};
        $check::num{'fontsize'} = 'タブフォントサイズ'
            if !defined $check::num{'fontsize'};
        $check::num{'sitetitlesize'}
            = 'サイトタイトルフォントサイズ'
            if !defined $check::num{'sitetitlesize'};
        $check::num{'titlewidth'} = 'タイトル領域幅'
            if !defined $check::num{'titlewidth'};

        $check::color{'menuhovercolor'}
            = 'カテゴリーメニュー・マウスオーバー時背景色'
            if !defined $check::color{'menuhovercolor'};
        $check::color{'menuactivecolor'}
            = 'カテゴリーメニュー・マウスクリック時背景色'
            if !defined $check::color{'menuactivecolor'};
    }
    else {
        push( @check::checkbox, 'scroll' );

        $check::num{'size'} = 'フレーム幅'
            if !defined $check::num{'size'};
    }

    #数字チェック
    $check::num{'space'} = 'アイコン間隔'
        if !defined $check::num{'space'};
}

######################################################################
# 設定チェック
sub is_valid_setting ($) {
    my $this = shift;
    if (   $env::inline_navi
        && defined $main::data{'navi_sitetitleurl'}
        && $main::data{'navi_sitetitleurl'} ne ''
        && $main::data{'navi_sitetitleurl'} !~ m!^/!
        && $this->is_invalid_url( $main::data{'navi_sitetitleurl'} ) )
    {
        push(
            @{ $this->{'dialog'} },
            'サイトタイトルのリンク先 URL を更新できませんでした。リンク先 URL は絶対パスで指定してください。'
        );
        delete $main::data{'navi_sitetitleurl'};
    }
    return;
}

######################################################################
# 設定環境
sub set_preferences ($) {
    my $this = shift;
    $preference::db = 'page'
        if !defined $preference::db
        || $preference::db ne 'page';    # 記録用データベース名
    $preference::submenukey = 'page'
        if !defined $preference::submenukey
        || $preference::submenukey ne 'page';    # サブメニューキー

    $preference::id = 'navi'
        if !defined $preference::id
        || $preference::id ne 'navi';            # データキー
    $preference::name = 'ナビゲーションレイアウト'
        if !defined $preference::name;           # 設定パネル名
    $preference::title
        = 'ナビゲーションのレイアウトについて設定します。'
        if !defined $preference::title;

    unless ( ref $cgi::Deco ) {
        require CMONOS::CGI_Lib::Deco;
        $cgi::Deco = Deco->new;
    }
    my $js = <<SCRIPT;
function toggleNaviType () {
	var input_list = document.getElementsByTagName('input');
	var navi_type = '';
	for (var i=0; i<input_list.length; i++) {
		if (input_list[i].type != null && input_list[i].type == 'radio' && input_list[i].name == 'data:navi_type' && input_list[i].checked) {
			navi_type = input_list[i].value;
			break;
		}
	}
	var type_list = document.getElementsByTagName('div');
	for (var i=0; i<type_list.length; i++) {
		if (type_list[i].className != null && type_list[i].className == 'type-settings') {
			if (type_list[i].id == 'type-' + navi_type) {
				type_list[i].style.display = 'block';
			} else {
				type_list[i].style.display = 'none';
			}
		}
	}
	if (navi_type == 'tab' || navi_type == 'vicuna') {
		document.getElementById('title-url').style.display = 'block';
	} else {
		document.getElementById('title-url').style.display = 'none';
	}
};
SCRIPT
    $this->set_javascript($js);
    $this->set_onload('toggleNaviType();');
}

######################################################################
# 設定パネル
sub print_preference_panel ($) {
    my $this = shift;
    require CMONOS::CGI_Lib::StyleSheetHelper;
    my $helper = StyleSheetHelper->new;

    print '<input type="hidden" name="id" value="navi" />';

    #フレーム方式
    if ( !defined $env::inline_navi || !$env::inline_navi ) {
        print '<fieldset>';
        print '<legend>ナビゲーションフレーム設定</legend>';
        print '<fieldset>';
        print '<legend>フレーム分割方式</legend>';
        print '<ol class="checkbox">';
        print
            '<li><label><input type="radio" name="data:navi_frametype" value="ht"'
            . (
            (   defined $main::data{'navi_frametype'}
                    && $main::data{'navi_frametype'} eq 'ht'
            ) ? ' checked="checked"' : ''
            )
            . ' /> 上下分割（上がナビゲーション）</label></li>';
        print
            '<li><label><input type="radio" name="data:navi_frametype" value="hb"'
            . (
            (   defined $main::data{'navi_frametype'}
                    && $main::data{'navi_frametype'} eq 'hb'
            ) ? ' checked="checked"' : ''
            )
            . ' /> 上下分割（下がナビゲーション）</label></li>';
        print
            '<li><label><input type="radio" name="data:navi_frametype" value="vl"'
            . (
            (   defined $main::data{'navi_frametype'}
                    && $main::data{'navi_frametype'} eq 'vl'
            ) ? ' checked="checked"' : ''
            )
            . ' /> 左右分割（左がナビゲーション）</label></li>';
        print
            '<li><label><input type="radio" name="data:navi_frametype" value="vr"'
            . (
            (   defined $main::data{'navi_frametype'}
                    && $main::data{'navi_frametype'} eq 'vr'
            ) ? ' checked="checked"' : ''
            )
            . ' /> 左右分割（右がナビゲーション）</label></li>';
        print '</ol>';
        print '</fieldset>';
        print '<fieldset>';
        print
            '<legend><label for="data:navi_size">フレーム幅</label></legend>';
        print
            '<input type="text" name="data:navi_size" id="data:navi_size" size="7" value="'
            . $this->escape( $main::data{'navi_size'} )
            . '" /> <span class="unit">ピクセル</span>';
        print '</fieldset>';
        print '<ol class="checkbox">';
        print
            '<li><label><input type="checkbox" name="data:navi_scroll" value="no"'
            . (
            (   defined $main::data{'navi_scroll'}
                    && $main::data{'navi_scroll'} eq 'no'
            ) ? ' checked="checked"' : ''
            )
            . ' /> ナビゲーションフレームのスクロールバーを表示しない</label></li>';
        print
            '<li><label><input type="checkbox" name="data:navi_accestat" value="1"'
            . (
            (   defined $main::data{'navi_accestat'}
                    && $main::data{'navi_accestat'}
            ) ? ' checked="checked"' : ''
            ) . ' /> アクセス解析タグを埋め込む</label></li>';
        print '</ol>';
        print '</fieldset>';
    }
    else {
        $this->use_icon( 'navi', 'navi_icon' );
        print '<fieldset>';
        print '<legend>アクセス解析</legend>';
        print
            '<label><input type="checkbox" name="data:navi_accestat" value="1"'
            . (
            (   defined $main::data{'navi_accestat'}
                    && $main::data{'navi_accestat'}
            ) ? ' checked="checked"' : ''
            ) . ' /> アクセス解析タグを埋め込む</label>';
        print '</fieldset>';

        $main::data{'navi_gsearchtitle'} = 'サイト検索'
            if !$main::data{'navi_gsearchtitle'};
        $main::data{'navi_gsearchlabel'} = 'このサイト'
            if !$main::data{'navi_gsearchlabel'};
        print '<fieldset>';
        print
            '<legend><label><input type="checkbox" name="data:navi_gsearchenabled" value="1"'
            . (
            (   defined $main::data{'navi_gsearchenabled'}
                    && $main::data{'navi_gsearchenabled'}
            ) ? ' checked="checked"' : ''
            )
            . ' onclick="cmonos.element.item(\'gsearch-settings\').toggle();" /> <a href="https://www.google.co.jp/cse/all" target="_blank">Google</a> によるサイト検索</label></legend>';
        print '<div id="gsearch-settings">';
        print '<table class="field-table"><tbody>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_gsearchtitle">検索欄のタイトル</label></td>';
        print
            '<td class="field"><input type="text" name="data:navi_gsearchtitle" id="data:navi_gsearchtitle" value="'
            . $this->escape( $main::data{'navi_gsearchtitle'} )
            . '" size="16" title="「サイト検索」等検索検索欄のタイトルを設定します。" /></td>';
        print '</tr>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_gsearchlabel">検索結果ラベル</label></td>';
        print
            '<td class="field"><input type="text" name="data:navi_gsearchlabel" id="data:navi_gsearchlabel" value="'
            . $this->escape( $main::data{'navi_gsearchlabel'} )
            . '" size="16" title="「このサイト」等検索結果に表示するラベルを設定します。" /></td>';
        print '</tr>';
        print '<tr>';
        print
            '<td class="field-name"><span class="datatitle">検索欄の位置</span></td>';
        print
            '<td class="field"><label><input type="radio" name="data:navi_gsearchalign" value="left"'
            . (
            (         !$main::data{'navi_gsearchalign'}
                    || $main::data{'navi_gsearchalign'} eq 'left'
            ) ? ' checked="checked"' : ''
            ) . ' />左上端</label>';
        print
            ' <label><input type="radio" name="data:navi_gsearchalign" value="right"'
            . (
            (          $main::data{'navi_gsearchalign'}
                    && $main::data{'navi_gsearchalign'} eq 'right'
            ) ? ' checked="checked"' : ''
            ) . ' />右上端</label>';
        print
            ' (<label><input type="checkbox" name="data:navi_gsearchfixed" value="right"'
            . (
            ( $main::data{'navi_gsearchfixed'} ) ? ' checked="checked"' : '' )
            . ' />固定</label>)</td>';
        print '</tr>';
        print '</tbody></table>';
        print '<fieldset>';
        print '<legend>検索欄表示ボタン</legend>';
        print $cgi::icon->button_menu( 'data:navi_gsearchbutton', 'search',
            120, $main::data{'navi_gsearchbutton'} );
        print '</fieldset>';
        print '</div></fieldset>';
        print $this->javascript(
            'cmonos.element.item(\'gsearch-settings\').disappear(true);')
            if !$main::data{'navi_gsearchenabled'};

        print '<fieldset>';
        print
            '<legend><label for="data:navi_naviheader">ヘッダ</label></legend>';
        print
            '<textarea name="data:navi_naviheader" id="data:navi_naviheader" cols="80" rows="5" wrap="off" class="content-editor-area">'
            . $this->escape( $main::data{'navi_naviheader'} )
            . '</textarea>';
        $this->print_html_helper('data:navi_naviheader');
        print '</fieldset>';
    }
    print
        '<fieldset id="title-url"><legend><label for="data:navi_sitetitleurl">サイトタイトルのリンク先 URL</label></legend>';
    print
        '<input type="text" name="data:navi_sitetitleurl" id="data:navi_sitetitleurl" value="'
        . $this->escape( $main::data{'navi_sitetitleurl'} )
        . '" size="50" title="絶対パスで指定します。" /></fieldset>';

    print '<fieldset>';
    print '<legend>サイトナビゲーション方式の選択</legend>';

    print '<fieldset>';
    print
        '<legend><label><input type="radio" name="data:navi_type" value="icon"'
        . (
        (   defined $main::data{'navi_type'}
                && $main::data{'navi_type'} eq 'icon'
        ) ? ' checked="checked"' : ''
        )
        . ' onclick="toggleNaviType()" />アイコン方式 : 指定されたアイコンを自動的に配置します。</label></legend>';
    print '<div id="type-icon" class="type-settings">';
    print '<table class="field-table"><tbody>';

    if ( defined $env::inline_navi && $env::inline_navi ) {
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_iconbgwidth">アイコン背景幅</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_iconbgwidth" id="data:navi_iconbgwidth" value="'
            . $this->escape( $main::data{'navi_iconbgwidth'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print
            '<td class="field-name"><label for="data:navi_iconbgalign">アイコン背景の位置</label></td>';
        print '<td class="q-field">';
        print $this->mk_select_menu(
            'data:navi_iconbgalign', '', '', $main::data{'navi_iconbgalign'},
            ''       => '',
            'left'   => '左寄せ',
            'center' => '中央寄せ',
            'right'  => '右寄せ'
        );
        print '</td>';
        print '</tr>';
    }
    print '<tr>';
    print
        '<td class="field-name"><label for="data:navi_cellvalign">アイコン位置揃え</label></td><td class="q-field">';
    print $this->mk_select_menu(
        'data:navi_cellvalign', '', '', $main::data{'navi_cellvalign'},
        ''       => '',
        'top'    => '上端揃え',
        'middle' => '中央揃え',
        'bottom' => '下端揃え'
    );
    print '</td>';
    print
        '<td class="field-name"><label for="data:navi_space">アイコン間隔</label></td>';
    print
        '<td class="q-field"><input type="text" name="data:navi_space" id="data:navi_space" value="'
        . $this->escape( $main::data{'navi_space'} )
        . '" size="7" title="アイコン間隔を設定します。" /> <span class="unit">ピクセル</span>';
    print '</td></tr>';

    if ( defined $env::inline_navi && $env::inline_navi ) {
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_align">ナビゲーション位置</label></td>';
        print '<td class="q-field">';
        print $this->mk_select_menu(
            'data:navi_align', '', '', $main::data{'navi_align'},
            ''       => '',
            'left'   => '左',
            'center' => '中央',
            'right'  => '右'
        );
        print '</td>';
        print
            '<td class="field-name"><label for="data:navi_size">ナビゲーション高さ</label></td>';
        print '<td class="q-field">';
        print
            '<input type="text" name="data:navi_size" id="data:navi_size" size="7" value="'
            . $this->escape( $main::data{'navi_size'} )
            . '" /> <span class="unit">ピクセル</span>';
        print '</td></tr>';
        print '<tr>';
        print
            '<td class="field-name"><span class="datatitle">表示方法</span></td>';
        print
            '<td class="field" colspan="3"><label><input type="checkbox" name="data:navi_icontable" value="1"'
            . (
            (   defined $main::data{'navi_icontable'}
                    && $main::data{'navi_icontable'}
            ) ? ' checked="checked"' : ''
            )
            . ' /> テーブルを使う（カテゴリーメニューを使えなくなります）。</label>';
        print '</td></tr></tbody></table>';

        print '<fieldset>';
        print '<legend>アイコン背景画像</legend>';
        print $cgi::Deco->background_menu( 'data:navi_background',
            'background', 100, $main::data{'navi_background'} );
        print '</fieldset>';
        print '<fieldset>';
        print
            '<legend><label for="data:navi_bgcolor">アイコン背景色</label></legend>';
        print
            '<input type="text" name="data:navi_bgcolor" id="data:navi_bgcolor" value="'
            . $this->escape( $main::data{'navi_bgcolor'} )
            . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:navi_bgcolor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()" style="background-color: '
            . $this->escape( $main::data{'navi_bgcolor'} ) . '" />';
        print
            ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:navi_bgcolor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_bgcolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
        print
            ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:navi_bgcolor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:navi_bgcolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
        print '</fieldset>';
        print '<fieldset>';
        print
            '<legend><label for="data:navi_iconcss">追加スタイルシート</label></legend>';
        print
            '<textarea name="data:navi_iconcss" id="data:navi_iconcss" cols="80" rows="5" wrap="off" class="content-editor-area" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_iconcss+\',\'dom\')">';
        print $this->escape( $main::data{'navi_iconcss'} );
        print '</textarea>';
        $helper->print_menu('data:navi_iconcss');
        print '</fieldset>';
        print '</div></fieldset>';

        print '<fieldset>';
        print
            '<legend><label><input type="radio" name="data:navi_type" value="tab"'
            . (
            (   defined $main::data{'navi_type'}
                    && $main::data{'navi_type'} eq 'tab'
            ) ? ' checked="checked"' : ''
            )
            . ' onclick="toggleNaviType()" />CSS タブ方式 : CSS を利用したタブによりナビゲーションを作成します。</label></legend>';
        print '<div id="type-tab" class="type-settings">';

        print '<table class="field-table"><tbody>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_tabbgcolor">タブ背景色</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_tabbgcolor" id="data:navi_tabbgcolor" value="'
            . $this->escape( $main::data{'navi_tabbgcolor'} )
            . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:navi_tabbgcolor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()" style="background-color: '
            . $this->escape( $main::data{'navi_tabbgcolor'} ) . '" />';
        print
            ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:navi_tabbgcolor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_tabbgcolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
        print
            ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:navi_tabbgcolor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:navi_tabbgcolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
        print '</td>';
        print
            '<td class="field-name"><label for="data:navi_tabbgwidth">タブ背景幅</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_tabbgwidth" id="data:navi_tabbgwidth" value="'
            . $this->escape( $main::data{'navi_tabbgwidth'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print '</tr>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_tabbgalign">タブ背景の位置</label></td>';
        print '<td class="q-field">';
        print $this->mk_select_menu(
            'data:navi_tabbgalign', '', '', $main::data{'navi_tabbgalign'},
            ''       => '',
            'left'   => '左寄せ',
            'center' => '中央寄せ',
            'right'  => '右寄せ'
        );
        print '</td>';
        print
            '<td class="field-name"><label for="data:navi_tabspace">タブ間隔</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_tabspace" id="data:navi_tabspace" value="'
            . $this->escape( $main::data{'navi_tabspace'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print '</tr>';
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><label for="data:navi_tabmargintop">タブ全体上余白</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_tabmargintop" id="data:navi_tabmargintop" value="'
            . $this->escape( $main::data{'navi_tabmargintop'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print
            '<td class="field-name"><label for="data:navi_tabmarginleft">タブ全体左右余白</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_tabmarginleft" id="data:navi_tabmarginleft" value="'
            . $this->escape( $main::data{'navi_tabmarginleft'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print '</tr>';
        print '</tbody>';
        print '<tbody>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_tabpaddingtop">タブ文字列上余白</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_tabpaddingtop" id="data:navi_tabpaddingtop" value="'
            . $this->escape( $main::data{'navi_tabpaddingtop'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print
            '<td class="field-name"><label for="data:navi_tabpaddingbottom">タブ文字列下余白</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_tabpaddingbottom" id="data:navi_tabpaddingbottom" value="'
            . $this->escape( $main::data{'navi_tabpaddingbottom'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print '</tr>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_tabpaddingleft">タブ文字列左余白</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_tabpaddingleft" id="data:navi_tabpaddingleft" value="'
            . $this->escape( $main::data{'navi_tabpaddingleft'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print
            '<td class="field-name"><label for="data:navi_tabpaddingright">タブ文字列右余白</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_tabpaddingright" id="data:navi_tabpaddingright" value="'
            . $this->escape( $main::data{'navi_tabpaddingright'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print '</tr>';
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><span class="datatitle">タブ文字装飾</span></td>';
        print '<td class="q-field">';
        print
            '<label><input type="checkbox" name="data:navi_fontweight" id="data:navi_fontweight" value="bold"'
            . (
            (   defined $main::data{'navi_fontweight'}
                    && $main::data{'navi_fontweight'} eq 'bold'
            ) ? ' checked="checked"' : ''
            ) . ' /> 太字</label>';
        print
            ' <label><input type="checkbox" name="data:navi_textshadow" id="data:navi_textshadow" value="1"'
            . (
            (   defined $main::data{'navi_textshadow'}
                    && $main::data{'navi_textshadow'}
            ) ? ' checked="checked"' : ''
            ) . ' /> 影付き</label>';
        print '</td>';
        print
            '<td class="field-name"><label for="data:navi_fontsize">タブフォントサイズ</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_fontsize" id="data:navi_fontsize" value="'
            . $this->escape( $main::data{'navi_fontsize'} )
            . '" size="4" /> <span class="unit">%</span></td>';
        print '</tr>';
        print '</tbody>';
        print '<tbody>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_sitetitle">サイトタイトル</label></td>';
        print '<td class="field" colspan="3">';
        print $this->mk_select_menu(
            'data:navi_sitetitle', '', '', $main::data{'navi_sitetitle'},
            ''  => '表示しない',
            '1' => 'サイトタイトルを表示する',
            '2' =>
                'サイトタイトルとサイトの説明を表示する',
        );
        print '</td></tr>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_sitetitlecolor">サイトタイトル色</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_sitetitlecolor" id="data:navi_sitetitlecolor" value="'
            . $this->escape( $main::data{'navi_sitetitlecolor'} )
            . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:navi_sitetitlecolor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()" style="background-color: '
            . $this->escape( $main::data{'navi_sitetitlecolor'} ) . '" />';
        print
            ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:navi_sitetitlecolor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_sitetitlecolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
        print
            ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:navi_sitetitlecolor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:navi_sitetitlecolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
        print '</td>';
        print
            '<td class="field-name"><label for="data:navi_sitetitlesize">サイトタイトルフォントサイズ</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_sitetitlesize" id="data:navi_sitetitlesize" value="'
            . $this->escape( $main::data{'navi_sitetitlesize'} )
            . '" size="4" /> <span class="unit">%</span></td>';
        print '</tr>';
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><label for="data:navi_sitesubtitlecolor">サイトの説明 文字色</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_sitesubtitlecolor" id="data:navi_sitesubtitlecolor" value="'
            . $this->escape( $main::data{'navi_sitesubtitlecolor'} )
            . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:navi_sitesubtitlecolor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()" style="background-color: '
            . $this->escape( $main::data{'navi_sitesubtitlecolor'} ) . '" />';
        print
            ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:navi_sitesubtitlecolor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_sitesubtitlecolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
        print
            ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:navi_sitesubtitlecolor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:navi_sitesubtitlecolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
        print '</td>';
        print
            '<td class="field-name"><span class="datatitle">サイトタイトル文字装飾</span></td>';
        print
            '<td class="q-field"><label><input type="checkbox" name="data:navi_sitetitleshadow" id="data:navi_sitetitleshadow" value="1"'
            . (
            (   defined $main::data{'navi_sitetitleshadow'}
                    && $main::data{'navi_sitetitleshadow'}
            ) ? ' checked="checked"' : ''
            ) . ' /> 影付き</label></td>';
        print '</tr>';
        print '</tbody></table>';

        print '<fieldset>';
        print '<legend>ロゴ画像</legend>';
        print $cgi::icon->image_menu( 'data:navi_logo', 'logo', 100,
            $main::data{'navi_logo'} );
        print '</fieldset>';

        print '<fieldset>';
        print
            '<legend title="「ナビゲーション作成」ページでタブの色を設定します。"><label><input type="radio" name="data:navi_tabtype" value="color"'
            . (
            (   defined $main::data{'navi_tabtype'}
                    && $main::data{'navi_tabtype'} eq 'color'
            ) ? ' checked="checked"' : ''
            )
            . ' />カラータブ : CSS によって色つきタブを作成します。</label></legend>';

        print '<table class="field-table"><tbody>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_titlewidth">タイトル領域幅</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_titlewidth" id="data:navi_titlewidth" value="'
            . $this->escape( $main::data{'navi_titlewidth'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print
            '<td class="field-name"><label for="data:navi_colortabalign">タブ位置揃え</label></td>';
        print '<td class="q-field">';
        print $this->mk_select_menu(
            'data:navi_colortabalign', '', '',
            $main::data{'navi_colortabalign'},
            ''       => '',
            'left'   => '左寄せ',
            'center' => '中央寄せ',
            'right'  => '右寄せ'
        );
        print '</td>';
        print '</tr>';
        print '</tbody></table>';

        print '<fieldset>';
        print
            '<legend><label for="data:navi_beltwidth">帯幅</label></legend>';
        print
            '<input type="text" name="data:navi_beltwidth" id="data:navi_beltwidth" value="'
            . $this->escape( $main::data{'navi_beltwidth'} )
            . '" size="4" /> <span class="unit">ピクセル</span>';
        print '</fieldset>';
        print '<fieldset>';
        print '<legend>背景画像</legend>';
        print $cgi::Deco->background_menu( 'data:navi_colortabbg',
            'background', 100, $main::data{'navi_colortabbg'} );
        print
            '<label><input type="checkbox" name="data:navi_colortabbgrepeatx" id="data:navi_colortabbgrepeatx" value="1"'
            . (
            (   defined $main::data{'navi_colortabbgrepeatx'}
                    && $main::data{'navi_colortabbgrepeatx'}
            ) ? ' checked="checked"' : ''
            ) . ' /> 下端に固定する</label>';
        print '</fieldset>';
        print '<fieldset>';
        print
            '<legend><label for="data:navi_colortabcss">追加スタイルシート</label></legend>';
        print
            '<textarea name="data:navi_colortabcss" id="data:navi_colortabcss" cols="80" rows="5" wrap="off" class="content-editor-area" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_colortabcss+\',\'dom\')">';
        print $this->escape( $main::data{'navi_colortabcss'} );
        print '</textarea>';
        $helper->print_menu('data:navi_colortabcss');
        print '</fieldset>';
        print '</fieldset>';

        print '<fieldset>';
        print
            '<legend><label><input type="radio" name="data:navi_tabtype" value="image"'
            . (
            (   defined $main::data{'navi_tabtype'}
                    && $main::data{'navi_tabtype'} eq 'image'
            ) ? ' checked="checked"' : ''
            )
            . ' />イメージタブ : CSS と画像によってタブを作成します。</label></legend>';

        print '<table class="field-table"><tbody>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_tabwidth">タブ領域幅</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_tabwidth" id="data:navi_tabwidth" value="'
            . $this->escape( $main::data{'navi_tabwidth'} )
            . '" size="4" /> <span class="unit">ピクセル</span></td>';
        print
            '<td class="field-name"><label for="data:navi_tabalign">タブ領域の位置</label></td>';
        print '<td class="q-field">';
        print $this->mk_select_menu(
            'data:navi_tabalign', '', '', $main::data{'navi_tabalign'},
            ''       => '',
            'left'   => '左寄せ',
            'center' => '中央寄せ',
            'right'  => '右寄せ'
        );
        print '</td>';
        print '</tr>';
        print '</tbody></table>';

        print '<fieldset>';
        print '<legend>タブ背景画像</legend>';
        print $cgi::icon->background_menu( 'data:navi_tabbg', 'tabbg', 100,
            $main::data{'navi_tabbg'},
            0, 0, 0, 1 );
        print '</fieldset>';

        print '<fieldset>';
        print '<legend>タブ画像</legend>';
        print $cgi::icon->tab_menu( 'data:navi_offtab', 'tab',
            $main::data{'navi_offtab'} );
        print '</fieldset>';

        print '<fieldset>';
        print '<legend>選択済みタブ画像</legend>';
        print $cgi::icon->tab_menu( 'data:navi_ontab', 'tab',
            $main::data{'navi_ontab'} );
        print '</fieldset>';

        print '<fieldset>';
        print
            '<legend><label for="data:navi_tabcss">追加スタイルシート</label></legend>';
        print
            '<textarea name="data:navi_tabcss" id="data:navi_tabcss" cols="80" rows="5" wrap="off" class="content-editor-area" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_tabcss+\',\'dom\')">';
        print $this->escape( $main::data{'navi_tabcss'} );
        print '</textarea>';
        $helper->print_menu('data:navi_tabcss');
        print '</fieldset>';
        print '</fieldset>';

        print '</div></fieldset>';

        print '<fieldset>';
        print
            '<legend><label><input type="radio" name="data:navi_type" value="vicuna"'
            . (
            (   defined $main::data{'navi_type'}
                    && $main::data{'navi_type'} eq 'vicuna'
            ) ? ' checked="checked"' : ''
            )
            . ' onclick="toggleNaviType()" />Vicuna 互換ヘッダ方式 : Vicuna CMS 互換のナビゲーションを作成します。</label></legend>';
        print
            '<div id="type-vicuna" class="type-settings"><p class="note">CSSファイル編集で、Vicuna CMS 用のスタイルシートを読み込む必要があります。</p>';
        print '<fieldset>';
        print '<legend>Vicuna CMS スキン調整</legend>';
        print
            '<label><input type="checkbox" name="data:navi_currentdisabled" id="data:navi_currentdisabled" value="1"'
            . ( ( $main::data{'navi_currentdisabled'} )
            ? ' checked="checked"'
            : '' )
            . ' /> Vicuna CMS スキンから現在のページを強調するスタイルシートを<strong>読み込まない</strong></label>';
        print '</fieldset>';
        print '<fieldset>';
        print
            '<legend><label for="data:navi_vicunacss">追加スタイルシート</label></legend>';
        print
            '<textarea name="data:navi_vicunacss" id="data:navi_vicunacss" cols="80" rows="5" wrap="off" class="content-editor-area" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_vicunacss+\',\'dom\')">';
        print $this->escape( $main::data{'navi_vicunacss'} );
        print '</textarea>';
        $helper->print_menu( 'data:navi_vicunacss', 0,
            '<option value="/*CURRENT_LI*/">現在のページに対応する &lt;li&gt;</option><option value="/*CURRENT_A*/">現在のページに対応する &lt;a&gt;</option>'
        );
        print '</fieldset>';
        print '</div></fieldset>';

    }
    else {
        print '</tbody></table></div></fieldset>';
    }

    print '<fieldset>';
    print
        '<legend><label><input type="radio" name="data:navi_type" value="template"'
        . (
        (   defined $main::data{'navi_type'}
                && $main::data{'navi_type'} eq 'template'
        ) ? ' checked="checked"' : ''
        )
        . ' onclick="toggleNaviType()" />テンプレート方式 : テンプレートに基づきナビゲーションを作成します。</label></legend>';
    print '<div id="type-template" class="type-settings">';

    require CMONOS::CGI_Lib::Template;
    my $this_template = Template->new;

    #入力支援ツール
    my $tool
        = '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:navi_template\').quotationByMenu(this)" title="これらのタグは、書き出し時に適切なタグに変換されます。">';
    $tool .= '<option value="">▼特殊タグを挿入する</option>';
    $tool
        .= '<option value="&lt;!--sitenavi type=&quot;icon&quot;--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/sitenavi--&gt;">サイトナビゲーションアイコン</option>';
    $tool
        .= '<option value="&lt;!--sitenavi type=&quot;link&quot;--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/sitenavi--&gt;">サイトナビゲーションリンク(&lt;a&gt;)</option>';
    $tool
        .= '<option value="&lt;!--sitenavi type=&quot;list&quot;--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/sitenavi--&gt;">サイトナビゲーションリスト(&lt;li&gt;)</option>';
    $tool .= '</select>';
    $this_template->print_template_form(
        'data:navi_template',
        'ナビゲーションテンプレート',
        \$main::data{'navi_template'}, \$tool
    );

    print '<fieldset>';
    print
        '<legend><label for="data:navi_navicss">ナビゲーションスタイルシート</label></legend>';
    print
        '<textarea name="data:navi_navicss" id="data:navi_navicss" cols="80" rows="20" wrap="off" class="content-editor-area" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_navicss+\',\'dom\')">';
    print $this->escape( $main::data{'navi_navicss'} );
    print '</textarea>';
    $helper->print_menu('data:navi_navicss');
    print '</fieldset>';
    print '</div></fieldset>';

    print '</fieldset>';
    print '</fieldset>';

    if ( defined $env::inline_navi && $env::inline_navi ) {
        print '<fieldset>';
        print '<legend>カテゴリーメニュー設定</legend>';
        print '<table class="field-table"><tbody>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_menuhovercolor">マウスオーバー時背景色</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_menuhovercolor" id="data:navi_menuhovercolor" value="'
            . $this->escape( $main::data{'navi_menuhovercolor'} )
            . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:navi_menuhovercolor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()" style="background-color: '
            . $this->escape( $main::data{'navi_menuhovercolor'} ) . '" />';
        print
            ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:navi_menuhovercolor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_menuhovercolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
        print
            ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:navi_menuhovercolor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:navi_menuhovercolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
        print '</td>';
        print '</tr>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:navi_menuactivecolor">マウスクリック時背景色</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:navi_menuactivecolor" id="data:navi_menuactivecolor" value="'
            . $this->escape( $main::data{'navi_menuactivecolor'} )
            . '" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open(\'data:navi_menuactivecolor\',\'dom\')" onchange="cmonos.validate.field(this).asColor()" style="background-color: '
            . $this->escape( $main::data{'navi_menuactivecolor'} ) . '" />';
        print
            ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:navi_menuactivecolor\')" ondblclick="cmonos.tool.colorPalette.open(\'data:navi_menuactivecolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
        print
            ' <a href="JavaScript:cmonos.tool.colorSlider.open(\'data:navi_menuactivecolor\')" ondblclick="cmonos.tool.colorSlider.open(\'data:navi_menuactivecolor\',\'window\')"><img src="'
            . $env::image_dir
            . '/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>';
        print '</td>';
        print '</tr>';
        print '</tbody>';
        print '</table>';
        print '</fieldset>';

        print '<fieldset>';
        print
            '<legend><label for="data:navi_navifooter">フッタ</label></legend>';
        print
            '<textarea name="data:navi_navifooter" id="data:navi_navifooter" cols="80" rows="5" wrap="off" class="content-editor-area">'
            . $this->escape( $main::data{'navi_navifooter'} )
            . '</textarea>';
        $this->print_html_helper('data:navi_navifooter');
        print '</fieldset>';
    }
    print
        '<fieldset class="submit"><span class="warning">変更は次回「ナビゲーション書き出し」時に反映されます。</span> <input type="submit" value="保存" class="save" /></fieldset>';
}

sub print_html_helper ($$) {
    my ( $this, $name ) = @_;
    print '<div class="toolbox">';
    require CMONOS::CGI_Lib::HTMLHelper;
    print HTMLHelper->new->html_helper_menu($name);
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\''
        . $name
        . '\').setAutoFit(this.checked);" />フィット</label>';
    print '</div>';
}

######################################################################
# 登録
#sub import {
BEGIN {

    #クラス登録
    Common::class_register( 'n000', 'NaviLayout' );

    #ツール登録
    $setting::jump_menu{'n000'} = 'ページレイアウト'
        if !defined $setting::jump_menu{'n000'};
    $setting::sub_jump_menu{'page'}{'n000'}
        = 'ナビゲーションレイアウト'
        if !defined $setting::sub_jump_menu{'page'}{'n000'};
}

1;
