package DataEditor;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# 書き出しモジュール
# DataEditor.pm ver1.0.3
# Copyright (C) 1999-2022 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2022-10-07
######################################################################
# 実行
sub run ($) {
    no warnings qw(numeric);
    my $this = shift;
    my ( $file_name, @datafiles );
    @main::target = ($main::target) if !@main::target && $main::target;
    $main::a = '' if !defined $main::a;
    $main::code
        = ( $ENV{'HTTP_USER_AGENT'} && $ENV{'HTTP_USER_AGENT'} =~ /Win/i )
        ? 'utf8bom'
        : 'utf8'
        if !defined $main::code;
    $main::rcode
        = ( $ENV{'HTTP_USER_AGENT'} && $ENV{'HTTP_USER_AGENT'} =~ /Win/i )
        ? 'crlf'
        : ( $ENV{'HTTP_USER_AGENT'} =~ /Mac/i ) ? 'cr'
        : 'lf'
        if !defined $main::rcode;

    #自動処理
    if ( defined $main::remote && $main::remote && !@main::target ) {
        local (*DATA);
        opendir( DATA, $default::data );
        @datafiles = readdir(DATA);
        closedir(DATA);
        foreach (@datafiles) {
            if ( $_ =~ /([0-9]+)\.tsv$/ ) {
                if ( $main::a eq 'delete' ) {
                    push( @main::target, $1 )
                        if -d $default::data . '/' . $1 . '.checked';
                }
                elsif ( $main::a eq 'v' ) {
                    push( @main::target, $1 )
                        if !-d $default::data . '/' . $1 . '.checked';
                }
            }
        }
    }

    #データ表示
    if ( $main::a eq 'v' && @main::target ) {
        my ($file_no);
        @main::target = sort { $a <=> $b || $a cmp $b } @main::target;
        binmode STDOUT;
        if ($main::for_excel) {
            $this->print_http_header( 'application/vnd.ms-excel', 0, 1 );
        }
        elsif ( defined $main::type && $main::type eq 'csv' ) {
            $this->print_http_header( 'text/comma-separated-values', 0, 1 );
        }
        else {
            $this->print_http_header( 'text/tab-separated-values', 0, 1 );
        }
        foreach $file_no (@main::target) {
            next if $file_no !~ /^[0-9]{14}$/;
            local (*DATA);
            if ( open( DATA, $default::data . '/' . $file_no . '.tsv' ) ) {
                binmode DATA, ":utf8";
                flock( DATA, LOCK_EX );
                my ($data);
                while ( $data = <DATA> ) {
                    if ( defined $main::type && $main::type eq 'csv' ) {
                        chomp $data;
                        $data =~ s/\x0B/\x0A/g;
                        my @data = split( /\x09/, $data );
                        map {
                            if ( $_ =~ /[,"\x0A]/ ) {
                                $_ =~ s/"/""/g;
                                $_ = '"' . $_ . '"';
                            }
                        } @data;
                        $data = join( ',', @data ) . "\x0A";
                    }
                    if ( $main::rcode eq 'crlf' ) {
                        $data =~ s/\x0A/\x0D\x0A/g;
                    }
                    elsif ( $main::rcode eq 'cr' ) {
                        $data =~ s/\x0A/\x0D/g;
                    }
                    $this->jcode_convert( \$data, $main::code, 'utf8' )
                        if $main::code eq 'sjis'
                        || $main::code eq 'euc'
                        || $main::code eq 'jis';
                    utf8::encode($data) if utf8::is_utf8($data);
                    print "\xEF\xBB\xBF" if $main::code eq 'utf8bom';
                    print $data;
                }
                close(DATA);
                if ( !-d $default::data . '/' . $file_no . '.checked' ) {
                    mkdir( $default::data . '/' . $file_no . '.checked',
                        0700 );
                }
            }
        }
        exit;
    }
    elsif ( $main::a eq 'v' ) {
        push(
            @{ $this->{'dialog'} },
            'ファイルを指定してください。'
        );
    }

    #データ削除
    if ( $main::a eq 'delete' && @main::target ) {
        my ( $file_no, $del_count, $error_count );
        foreach $file_no (@main::target) {
            my ($check_dir);
            $check_dir = $default::data . '/' . $file_no . '.checked';
            if ( !-d $check_dir ) {
                push(
                    @{ $this->{'dialog'} },
                    '未読最新データは削除できません。'
                );
                next;
            }
            if ( unlink( $default::data . '/' . $file_no . '.tsv' )
                && rmdir $check_dir )
            {
                $del_count++;
            }
            else {
                $error_count++;
            }
        }
        push(
            @{ $this->{'dialog'} },
            $del_count . '個のファイルを削除しました。'
        ) if defined $del_count && $del_count;
        push(
            @{ $this->{'dialog'} },
            $error_count
                . '個のファイルを削除できませんでした。'
        ) if defined $error_count && $error_count;
        undef @main::target;
    }
    elsif ( $main::a eq 'delete' ) {
        push(
            @{ $this->{'dialog'} },
            '削除するファイルを指定してください。'
        );
    }

    #自動処理
    if ( defined $main::remote && $main::remote ) {
        binmode STDOUT;
        $this->print_http_header('text/plain');
        my ( $msg, $r );
        $main::code = 'sjis' if !defined $main::code || !$main::code;
        if ( $main::rcode eq 'lf' ) {
            $r = "\x0A";
        }
        elsif ( $main::rcode eq 'cr' ) {
            $r = "\x0D";
        }
        else {
            $r = "\x0D\x0A";
        }
        foreach $msg ( @{ $this->{'dialog'} } ) {
            $this->jcode_convert( \$msg, $main::code, 'utf8' )
                if $main::code eq 'sjis'
                || $main::code eq 'jis'
                || $main::code eq 'euc';
            utf8::encode($msg) if utf8::is_utf8($msg);
            print $msg;
            print $r;
        }
        exit;
    }

    #データ編集
    local (*DATA);
    opendir( DATA, $default::data );
    @datafiles
        = sort { $b <=> $a || $b cmp $a }
        grep( $_ =~ /[0-9]+\.tsv$/ && -f $default::data . '/' . $_,
        readdir(DATA) );
    closedir(DATA);

    my $downloadname = $default::id || 'exported';
    $downloadname =~ s/^_//;
    $downloadname .= '_data';
    my $javascript = <<JAVASCRIPT;
cmonos.tool.questionnaireDataExporter = function (delete_data) {
	if (document.getElementById('target') && document.getElementById('target').selectedIndex == -1) {
		alert('ファイルを選択してください。');
		return false;
	}
	if (delete_data) {
		cmonos.form.name('input').appendDownloadPath()
	} else {
		cmonos.form.name('input').field(\'type\').appendDownloadPathByValue({'csv':'$downloadname.csv','tsv':'$downloadname.tsv'});
	}
	return true;
}
JAVASCRIPT
    $this->set_javascript($javascript);

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>データ書き出し/削除</h1>';

    if ( !@datafiles ) {
        print
            '<fieldset>データファイルがみつかりません。</fieldset>';
        $this->print_page_footer('tools');
        exit;
    }

    #フォーム開始
    $this->fill( \$main::code );
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'id'       => 'input',
        'method'   => 'post',
        'required' => \@setting::form
    );
    print '<fieldset><table class="field-table"><tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="type">形式</label></td>';
    print '<td class="field"><select name="type" id="type">';
    print '<option value="csv"'
        . ( ( defined $main::type && $main::type eq 'csv' )
        ? ' selected="selected"'
        : '' )
        . '>カンマ区切りテキスト（csv）</option>';
    print '<option value="tsv">タブ区切りテキスト</option>';
    print '</select>';
    print '<label><input type="checkbox" name="for_excel" value="1"'
        . ( ($main::for_excel) ? ' checked="checked"' : '' )
        . ' /> Excel ファイルとして書き出す</label>';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="code">文字コード</label></td>';
    print '<td class="field"><select name="code" id="code">';
    print '<option value="utf8bom"'
        . ( ( $main::code eq 'utf8bom' ) ? ' selected="selected"' : '' )
        . '>UTF-8(BOM付き)</option>';
    print '<option value="utf8"'
        . ( ( $main::code eq 'utf8' ) ? ' selected="selected"' : '' )
        . '>UTF-8</option>';
    print '<option value="sjis"'
        . ( ( $main::code eq 'sjis' ) ? ' selected="selected"' : '' )
        . '>ShiftJIS</option>';
    print '<option value="euc"'
        . ( ( $main::code eq 'euc' ) ? ' selected="selected"' : '' )
        . '>EUC</option>';
    print '<option value="jis"'
        . ( ( $main::code eq 'jis' ) ? ' selected="selected"' : '' )
        . '>JIS</option>';
    print '</select>';
    print '<select name="rcode">';
    print '<option value="crlf"'
        . ( ( $main::rcode eq 'crlf' ) ? ' selected="selected"' : '' )
        . '>CR+LF(Windows)</option>';
    print '<option value="cr"'
        . ( ( $main::rcode eq 'cr' ) ? ' selected="selected"' : '' )
        . '>CR(Mac)</option>';
    print '<option value="lf"'
        . ( ( $main::rcode eq 'lf' ) ? ' selected="selected"' : '' )
        . '>LF(UNIX)</option>';
    print '</select>';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="target">ファイル</label></td>';
    print
        '<td class="field"><select name="target" id="target" multiple="multiple" size="'
        . scalar(@datafiles) . '">';

    foreach $file_name (@datafiles) {
        my ( $file_no, $mtime );
        $file_no = $1 if $file_name =~ /^([0-9]+)\.tsv$/;
        $mtime = ( stat( $default::data . '/' . $file_name ) )[9];
        print '<option value="' . $file_no . '">';
        print $this->date( $file_no, 0, '/24' );
        print '-';
        print $this->date( $mtime, 0, 'gmt/24' );
        print '【済】' if -d $default::data . '/' . $file_no . '.checked';
        print '</option>';
    }
    print '</select>';
    print '</td>';
    print '</tr>';
    print '</tbody></table></fieldset>';
    print '<fieldset class="submit">';
    print
        '<input type="submit" name="a=delete" value="削除する" onclick="return cmonos.tool.questionnaireDataExporter(true) && confirm(\'この操作は取り消せません。本当に削除しますか？\') &amp;&amp; cmonos.form.item(this.form).isNotSubmitted();" class="delete"/>';
    print
        '<input type="submit" name="a=v" value="書き出す" class="export" onclick="return cmonos.tool.questionnaireDataExporter()" />';
    print '</fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 'q200', 'DataEditor' );

    #メニュー登録
    $setting::jump_menu{'q200'} = 'データ書き出し/削除';

}

1;
