package ImageCSSBuilder;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# 画像CSS再構築モジュール
# ImageCSSBuilder.pm ver1.1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-06-03
######################################################################
# 実行
sub run ($) {
    my $this = shift;

    #再構築
    $this->update($main::target)
        if defined $main::target && $main::target ne '';

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>画像ライブラリ CSS 再構築</h1>';

    print $this->form_start(
        'key'    => 'tools',
        'name'   => 'input',
        'method' => 'post',
        'onsubmit' =>
            'return confirm(\'この操作は取り消せません。本当に変更しますか？\') &amp;&amp; cmonos.form.item(this).isNotSubmitted();',
        'required' => \@setting::form
    );
    if ( $this->{'rebuild_log'} ) {
        print '<fieldset><legend>CSS 再構築ログ</legend><ol>';
        print $this->{'rebuild_log'};
        print '</ol></fieldset>';
    }
    print '<fieldset class="delete"><select name="target" id="target">';
    print '<option value="imagedir"'
        . ( ( defined $main::target && $main::target eq 'imagedir' )
        ? ' selected="selected"'
        : '' )
        . '>画像ライブラリ</option>';
    print '<option value="imagelib"'
        . ( ( defined $main::target && $main::target eq 'imagelib' )
        ? ' selected="selected"'
        : '' )
        . '>画像素材ライブラリ</option>';
    print '</select>';
    print
        '<input type="submit" value="CSS を再構築する" /></fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

sub update ($$;$) {
    my ( $this, $target, $dir ) = @_;
    my ( $env_image_dir, $env_image_lib, $env_css, $image_dir, $image_css,
        $deco_obj, %setting );

    if ($dir) {
        if ( -d $dir . '/cgi/imageLib' ) {
            $env_image_lib = $dir . '/cgi/imageLib';
        }
        elsif ( $target ne 'imagedir' ) {
            return;
        }
        else {
            $env_image_lib = $env::image_lib;
            $target        = 'imagedir';
        }
        $env_image_dir = $dir . '/image';
        $env_css       = $dir . '/lib/css';
        return if !-f $env_css . '/common.css';
    }
    else {
        $env_image_dir = $env::image_dir;
        $env_css       = $env::css;
        $env_image_lib = $env::image_lib;
    }
    require CMONOS::CGI_Lib::Deco;

    #全体ライブラリ
    $image_dir = ( $target eq 'imagedir' ) ? $env_image_dir : $env_image_lib;
    $image_css = ( $target eq 'imagedir' ) ? $env_css . '/image.css' : '';
    $deco_obj = Deco->new( $env_image_dir, $env_image_lib );
    $this->image_css_rebuild(
        $image_dir,
        $image_css,
        $deco_obj,
        \%setting,
        [   'check',  'cart',   'detail', 'email', 'index', 'jump',
            'next',   'prev',   'return', 'close', 'vote',  'comment',
            'submit', 'cancel', 'edit',   'tools'
        ],
        [ 'bargraph', 'bargraphbg', 'misc', 'title', 'new' ],
        ['frame'],
        ['map'],
        ['marker'],
        ['toggler'],
    );
    $this->preference_rebuild( $env::preference_dir . '/deco', \%setting )
        if $target eq 'imagedir';
    undef $deco_obj;
    undef %setting;

    #Baobabライブラリ
    $image_dir
        = ( $target eq 'imagedir' )
        ? $env_image_dir . '/baobab'
        : $env_image_lib . '/baobab';
    $image_css
        = ( $target eq 'imagedir' ) ? $env_css . '/baobab-image.css' : '';
    $this->use_icon( 'baobab', 'bb_icon', '', 1 );
    $this->image_css_rebuild(
        $image_dir,
        $image_css,
        $cgi::icon,
        \%setting,
        [   'input',         'inputdelete',
            'inputedit',     'inputres',
            'inputrmthread', 'list',
            'listarticle',   'listsearch',
            'listthread',    'listtree',
            'tool',          'toolpassword',
            'viewarticle',   'viewthread',
            'viewtree',      'listreset'
        ],
        [   'statusnew', 'statusoperational', 'statussolved',
            'threadcategory'
        ]
    );
    $this->preference_rebuild( $cgi::icon->preference_db, \%setting )
        if $target eq 'imagedir';
    undef $cgi::icon;
    undef %setting::icon;
    undef %setting;

    #naviライブラリ
    local %setting::deco_option = (
        'bbs'       => 'save_all',
        'db'        => 'save_all',
        'page'      => 'save_all',
        'equipment' => 'save_all'
    );
    $image_dir
        = ( $target eq 'imagedir' )
        ? $env_image_dir . '/navi'
        : $env_image_lib . '/navi';
    $image_css
        = ( $target eq 'imagedir' ) ? $env_css . '/navi-image.css' : '';
    $this->use_icon( 'navi', 'navi_icon', '', 1 );
    $this->image_css_rebuild( $image_dir, $image_css, $cgi::icon, \%setting,
        [ 'bbs', 'db', 'page', 'equipment', 'search' ] );
    $this->preference_rebuild( $cgi::icon->preference_db, \%setting )
        if $target eq 'imagedir';
    undef $cgi::icon;
    undef %setting::icon;
    undef %setting;

    #Calendarライブラリ
    $image_dir
        = ( $target eq 'imagedir' )
        ? $env_image_dir . '/calendar'
        : $env_image_lib . '/calendar';
    $image_css
        = ( $target eq 'imagedir' ) ? $env_css . '/calendar-image.css' : '';
    $deco_obj = Deco->new( $env_image_dir . '/calendar',
        $env_image_lib . '/calendar' );
    $this->image_css_rebuild(
        $image_dir,
        $image_css,
        $deco_obj,
        \%setting,
        [   'calendarpanel', 'calindex',  'calinput', 'callogin',
            'calnext',       'calprev',   'calreset', 'calsearch',
            'caltoday',      'caltool',   'daily',    'eventadd',
            'eventdelete',   'eventedit', 'monthly',  'togglepanel',
            'weekly'
        ],
    );
    undef $deco_obj;
    undef %setting;

    #TableMakerライブラリ
    $image_dir
        = ( $target eq 'imagedir' )
        ? $env_image_dir . '/tablemaker'
        : $env_image_lib . '/tablemaker';
    $image_css
        = ( $target eq 'imagedir' ) ? $env_css . '/tablemaker-image.css' : '';
    $deco_obj = Deco->new( $env_image_dir . '/tablemaker',
        $env_image_lib . '/tablemaker' );
    $this->image_css_rebuild(
        $image_dir,
        $image_css,
        $deco_obj,
        \%setting,
        [   'tableview', 'gmapview',     'tileview',   'tmmailer',
            'tmsearch',  'tminput',      'tmlist',     'tmtool',
            'tmlogin',   'detaildelete', 'detailedit', 'tmdelete',
            'tmedit',    'tmfavorites',  'tmreset'
        ]
    );
    undef $deco_obj;
    undef %setting;

    #orderライブラリ
    $image_dir
        = ( $target eq 'imagedir' )
        ? $env_image_dir . '/order'
        : $env_image_lib . '/order';
    $image_css
        = ( $target eq 'imagedir' ) ? $env_css . '/order-image.css' : '';
    $deco_obj
        = Deco->new( $env_image_dir . '/order', $env_image_lib . '/order' );
    $this->image_css_rebuild(
        $image_dir,
        $image_css,
        $deco_obj,
        \%setting,
        [   'trash',    'sum',          'checkout', 'empty',
            'estimate', 'confirmation', 'order'
        ]
    );
    undef $deco_obj;
    undef %setting;
}

#CSS再構築
sub image_css_rebuild ($$$$$$$$$$$) {
    my ($this,    $image_dir, $image_css, $deco_obj,
        $setting, $button,    $image,     $frame,
        $map,     $marker,    $toggler
    ) = @_;
    my $css            = '';
    my $image_dir_name = $image_dir . '/';
    $image_dir_name =~ s/\.\.\///g;

    #ボタン
    if ( ref $button && @$button ) {
        my $this_button = ImageLib->new( 'button', $deco_obj, $image_css );
        foreach my $dir (@$button) {
            $css .= $this_button->make_image_css( $image_dir, $dir,
                ( ( !$image_css ) ? 1 : 0 ), $setting );
            if ( !$image_css && $css ) {
                $this->write_file( $image_dir . '/' . $dir . '/image.css',
                    \$css );
                $css = '';
            }
            $this->{'rebuild_log'}
                .= '<li>' . $image_dir_name . $dir . '</li>';
        }
    }

    #画像
    if ( ref $image && @$image ) {
        my $this_image = ImageLib->new( 'image', $deco_obj, $image_css );
        foreach my $dir (@$image) {
            my @images = $this_image->get_image_file_list( $image_dir, $dir );
            my $img;
            foreach $img (@images) {
                my %image_info = ();
                my $id;
                $id = $1 if $img =~ /^([^\.])+/;
                my $class_name = $dir . '-' . $id;
                $image_info{'name'} = $img;
                ( $image_info{'width'}, $image_info{'height'} )
                    = $cgi::Image->info(
                    $image_dir . '/' . $dir . '/' . $img );
                if ( $dir eq 'bargraph' || $dir eq 'bargraphbg' ) {
                    $css .= "\x0A\x0A"
                        . $this_image->mk_image_css_for_bargraph( $class_name,
                        \%image_info, $dir, $id )
                        if $image_css;
                    delete $image_info{'width'};
                    delete $image_info{'name'};
                }
                elsif ( $dir eq 'title' ) {
                    $css .= "\x0A\x0A"
                        . $this_image->mk_image_css_for_title( $class_name,
                        \%image_info, $dir, $id )
                        if $image_css;
                    undef %image_info;
                }
                $this_image->add_setting_hash( $setting, \%image_info, $dir,
                    $id )
                    if $image_css && %image_info;
            }
            if ( !$image_css && $css ) {
                $this->write_file( $image_dir . '/' . $dir . '/image.css',
                    \$css, 1 );
                $css = '';
            }
            $this->{'rebuild_log'}
                .= '<li>' . $image_dir_name . $dir . '</li>';
        }
    }

    #フレーム
    if ( ref $frame && @$frame ) {
        my $this_frame = ImageLib->new( 'frame', $deco_obj, $image_css );
        foreach my $dir (@$frame) {
            $css .= $this_frame->make_image_css( $image_dir, $dir,
                ( ( !$image_css ) ? 1 : 0 ), $setting );
            if ( !$image_css && $css ) {
                $this->write_file( $image_dir . '/' . $dir . '/image.css',
                    \$css );
                $css = '';
            }
            $this->{'rebuild_log'}
                .= '<li>' . $image_dir_name . $dir . '</li>';
        }
    }

    #地図画像
    if ( ref $map && @$map && $image_css ) {
        my $this_map = ImageLib->new( 'map', $deco_obj, $image_css );
        foreach my $dir (@$map) {
            my @files = $this_map->get_image_file_list( $image_dir, $dir );
            foreach my $name (@files) {
                if ( $name =~ /^([0-9a-zA-Z_]+)\.(gif|png|jpg)$/ ) {
                    my $id = $1;
                    $css .= "\x0A\x0A"
                        . $this_map->mk_image_css_for_map( $dir . '-' . $id,
                        $dir, $id, 0,
                        $this_map->{'deco_obj'}->image_info( $dir, $id ) );
                }
            }
            $this->{'rebuild_log'}
                .= '<li>' . $image_dir_name . $dir . '</li>';
        }
    }

    #地図マーカー
    if ( ref $marker && @$marker && $image_css ) {
        my $this_marker = ImageLib->new( 'marker', $deco_obj, $image_css );
        foreach my $dir (@$marker) {
            my @files = $this_marker->get_image_file_list( $image_dir, $dir );
            foreach my $name (@files) {
                if ( $name =~ /^([0-9a-zA-Z_]+)\.(gif|png|jpg)$/ ) {
                    my $id = $1;
                    $css .= "\x0A\x0A"
                        . $this_marker->mk_image_css_for_marker(
                        $dir . '-' . $id,
                        $dir, $id, 0,
                        $this_marker->get_marker_info( $dir, $id ) );
                }
            }
            $this->{'rebuild_log'}
                .= '<li>' . $image_dir_name . $dir . '</li>';
        }
    }

    #トグルボタン
    if ( ref $toggler && @$toggler ) {
        my $this_toggler = ImageLib->new( 'toggler', $deco_obj, $image_css );
        foreach my $dir (@$toggler) {
            $css .= $this_toggler->make_image_css( $image_dir, $dir,
                ( ( !$image_css ) ? 1 : 0 ), $setting );
            if ( !$image_css && $css ) {
                $this->write_file( $image_dir . '/' . $dir . '/image.css',
                    \$css );
                $css = '';
            }
            $this->{'rebuild_log'}
                .= '<li>' . $image_dir_name . $dir . '</li>';
        }
    }

    $this->write_file( $image_css, \$css, 1 ) if $image_css;
}

#装飾設定書き込み
sub preference_rebuild ($$;$) {
    my ( $this, $preference_db, $setting ) = @_;
    my (%db);
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    tie( %db, $env::dbm_file, $preference_db, O_RDWR, $env::dbm_mode );
    if ( ref tied %db ) {
        my (@delete);
        while ( my ( $k, $v ) = each %db ) {
            utf8::decode($k) if !utf8::is_utf8($k);
            utf8::decode($v) if !utf8::is_utf8($v);
            my ( $dir, $id, $name ) = split( /_/, $k );
            if (

                #背景ライブラリ
                ( $dir eq 'background' ) ||

                #画像ライブラリ
                (   grep( $_ eq $dir,
                        (   'misc',              'statusnew',
                            'statusoperational', 'statussolved',
                            'new'
                            ) )
                    && $name ne 'width'
                    && $name ne 'height'
                    && $name ne 'name'
                    && $name ne 'alt'
                )
                || ( $dir eq 'title' && $name ne 'alt' )
                || ( grep( $_ eq $dir, ( 'bargraph', 'bargraphbg' ) )
                    && $name ne 'height' )
                || (   $dir eq 'threadcategory'
                    && $name ne 'width'
                    && $name ne 'height'
                    && $name ne 'name' )
                ||

                #ボタンライブラリ
                (   grep( $_ eq $dir,
                        (   'check',         'cart',
                            'detail',        'email',
                            'index',         'jump',
                            'next',          'prev',
                            'return',        'close',
                            'vote',          'comment',
                            'submit',        'cancel',
                            'edit',          'tools',
                            'input',         'inputdelete',
                            'inputedit',     'inputres',
                            'inputrmthread', 'list',
                            'listarticle',   'listsearch',
                            'listthread',    'listtree',
                            'tool',          'toolpassword',
                            'viewarticle',   'viewthread',
                            'viewtree',      'calendarpanel',
                            'calindex',      'calinput',
                            'callogin',      'calnext',
                            'calprev',       'calreset',
                            'calsearch',     'caltoday',
                            'caltool',       'daily',
                            'eventadd',      'eventdelete',
                            'eventedit',     'monthly',
                            'togglepanel',   'weekly',
                            'bbs',           'db',
                            'page',          'equipment',
                            'search'
                        ) )
                )
                ||

                #フレームライブラリ
                ( $dir eq 'frame' && $name ne 'type' )

                )
            {
                push( @delete, $k );
            }
        }
        foreach my $k (@delete) {
            delete $db{$k};
        }
        while ( my ( $k, $v ) = each %$setting ) {
            next if !defined $k || $k eq '';
            $db{$k} = $v;
        }
    }
    else {
        push(
            @{ $this->{'dialog'} },
            'データベース書き込みに失敗しました。'
        );
    }
    untie %db;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 's302', 'ImageCSSBuilder' );

    #メニュー登録
    $setting::jump_menu{'s302'} = '画像ライブラリ CSS 再構築';

}

1;
