package MasterInfo;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# 管理情報変更モジュール
# MasterInfo.pm ver1.0.14
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-07-11
######################################################################
# 実行
sub run ($;$) {
    my ( $this, $install ) = @_;
    my ($is_valid,                  $is_document_root,
        $error_flag,                $doctype,
        $master_password_encrypted, $env_doctype,
        @title_and_description,     $mobile_mode_disabled
    );
    $is_document_root = 1 if $install || !$cgi::input->home_directory(1);
    $this->get_setup_env if !$install;

    #入力チェック
    if ( defined $main::password && $main::password ne '' ) {
        if ( $main::password =~ /^[0-9a-zA-Z_]{6,}$/ ) {
            if ( $main::password eq $main::password2 ) {
                if ($env::session_enabled) {
                    $master_password_encrypted = 1;
                    $main::password
                        = $cgi::Register->encrypt($main::password);
                }
                $is_valid = 1;
            }
            else {
                push(
                    @{ $this->{'dialog'} },
                    '確認のためのパスワードが一致しません。'
                );
                $error_flag = 1;
            }
        }
        else {
            push(
                @{ $this->{'dialog'} },
                'パスワードは6文字以上の半角英数字にしてください。'
            );
            $error_flag = 1;
        }
    }
    elsif ( $main::a && $install ) {
        push(
            @{ $this->{'dialog'} },
            'マスターパスワードは必ず設定してください。'
        );
        $error_flag = 1;
    }
    if ( defined $main::emailaddr && $main::emailaddr ne '' ) {
        $main::emailaddr = $this->z_to_h($main::emailaddr);
        if ( !$install && $main::emailaddr eq $env::master_email ) {
            undef $main::emailaddr;
        }
        elsif ( $this->is_invalid_email_address($main::emailaddr) ) {
            push(
                @{ $this->{'dialog'} },
                '管理人メールアドレスを確認してください。'
            );
            $error_flag = 1;
        }
        else {
            $is_valid = 1;
        }
    }
    elsif ( $main::a && $install ) {
        push(
            @{ $this->{'dialog'} },
            '管理人メールアドレスは必ず設定してください。'
        );
        $error_flag = 1;
    }
    if ( defined $main::title && $main::title ne '' ) {
        if ( !$install && $main::title eq $env::title ) {
            undef $main::title;
        }
        else {
            $this->env_escape( \$main::title );
            $is_valid = 1;
        }
    }
    elsif ( $main::a && $install ) {
        push(
            @{ $this->{'dialog'} },
            'サイトタイトルは必ず設定してください。'
        );
        $error_flag = 1;
    }
    if ( defined $main::copyright && $main::copyright ne $env::copyright ) {
        $this->env_escape( \$main::copyright );
        $is_valid = 1;
    }
    else {
        undef $main::copyright;
    }
    if ( defined $main::description
        && $main::description ne $env::description )
    {
        $this->env_escape( \$main::description );
        $is_valid = 1;
    }
    else {
        undef $main::description;
    }
    if ( defined $main::rss_marker && $main::rss_marker ne '' ) {
        $main::rss_marker = '0' if $main::rss_marker ne '1';
        if ( $main::rss_marker eq $env::rss_marker ) {
            undef $main::rss_marker;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::doctype
        && $main::doctype
        =~ /^(html5|(?:xhtml1|html401)(?:strict|transitional|loose))$/ )
    {
        if ( $main::doctype eq 'html5' ) {
            $doctype = '<!DOCTYPE html>';
        }
        elsif ( $main::doctype eq 'xhtml1strict' ) {
            $doctype
                = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
        }
        elsif ( $main::doctype eq 'xhtml1transitional' ) {
            $doctype
                = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';
        }
        elsif ( $main::doctype eq 'html401strict' ) {
            $doctype
                = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">';
        }
        elsif ( $main::doctype eq 'html401loose' ) {
            $doctype
                = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">';
        }
        else {
            $doctype
                = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">';
        }
        if ( $doctype eq $env::doctype ) {
            undef $doctype;
            undef $main::doctype;
        }
        else {
            $is_valid = 1;
        }
    }
    else {
        undef $main::doctype;
    }
    if ( defined $main::html_header
        && ( !$env::html_header || $main::html_header ne $env::html_header ) )
    {
        if (   $main::html_header eq ''
            && defined $env::html_header
            && $env::html_header eq '' )
        {
            undef $main::html_header;
        }
        elsif ($main::html_header eq ''
            || $main::html_header
            =~ /<(?:meta|link|script|style)(?!\w)[^>]+>/si )
        {
            $main::html_header =~ s/<!--#(?!include virtual=")[^><]+?-->//sg;
            $main::html_header =~ s/^\s*//s;
            $main::html_header =~ s/\s*$//s;
            $main::html_header =~ s/^;+//sg;
            $main::html_header =~ s/\$(?=[a-z])/&#036;/sg;
            $is_valid = 1;
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '共通 HTML ヘッダを確認してください。ヘッダに適切なタグが存在しません。'
            );
            $error_flag = 1;
        }
    }
    else {
        undef $main::html_header;
    }
    if ( defined $main::inline_navi && $main::inline_navi ne '' ) {
        if ( $main::inline_navi eq $env::inline_navi ) {
            undef $main::inline_navi;
        }
        else {
            $main::inline_navi = '0' if $main::inline_navi ne '1';
            $is_valid = 1;
        }
    }
    if ( defined $main::window_title_format
        && $main::window_title_format ne '' )
    {
        $main::window_title_format
            = $this->escape_except_html_entity($main::window_title_format);
        $main::window_title_format =~ s/'//g;
        if ( $main::window_title_format eq $env::window_title_format ) {
            undef $main::window_title_format;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::session_enabled && $main::session_enabled ne '' ) {
        $main::session_enabled = '0'
            if $main::session_enabled ne '1' && $main::session_enabled ne '2';
        if ( $main::session_enabled eq $env::session_enabled ) {
            undef $main::session_enabled;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::session_security && $main::session_security ne '' ) {
        $main::session_security = 'ip,agent'
            if $main::session_security ne 'ip'
            && $main::session_security ne 'agent'
            && $main::session_security ne 'none';
        if ( $main::session_security eq $env::session_security ) {
            undef $main::session_security;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::openid_enabled && $main::openid_enabled ne '' ) {
        $main::openid_enabled = '0' if $main::openid_enabled ne '1';
        if ( $main::openid_enabled eq $env::openid_enabled ) {
            undef $main::openid_enabled;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::password_threshold ) {
        if (   $main::password_threshold ne ''
            && $this->is_nan($main::password_threshold) )
        {
            push(
                @{ $this->{'dialog'} },
                '許容するパスワード認証失敗回数を確認してください。'
            );
            $error_flag = 1;
        }
        elsif ( $main::password_threshold eq $env::password_threshold ) {
            undef $main::password_threshold;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::password_latency ) {
        if (   $main::password_latency ne ''
            && $this->is_nan($main::password_latency) )
        {
            push(
                @{ $this->{'dialog'} },
                'パスワードロック時の待ち時間を確認してください。数ではありません。'
            );
            $error_flag = 1;
        }
        elsif ( $main::password_latency eq $env::password_latency ) {
            undef $main::password_latency;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::user_id && $main::user_id ne '' ) {
        $main::user_id = 'individual' if $main::user_id ne 'shared';
        if (  !$env::home_dir_id
            || $is_document_root
            || (   $env::register_cgi
                && $env::register_cgi eq '../pswd/start.cgi'
                && $main::user_id eq 'individual' )
            || (   $env::register_cgi
                && $env::register_cgi ne '../pswd/start.cgi'
                && $main::user_id eq 'shared' )
            )
        {
            undef $main::user_id;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::rss_maxitem && $main::rss_maxitem ne '' ) {
        if ( $this->is_nan($main::rss_maxitem) ) {
            push(
                @{ $this->{'dialog'} },
                'RSS 最大記事数を確認してください。数ではありません。'
            );
            $error_flag = 1;
        }
        elsif ( $main::rss_maxitem eq $env::rss_maxitem ) {
            undef $main::rss_maxitem;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::rss_description_maxlength
        && $main::rss_description_maxlength ne '' )
    {
        if ( $this->is_nan($main::rss_description_maxlength) ) {
            push(
                @{ $this->{'dialog'} },
                'RSS 説明項目最大字数を確認してください。数ではありません。'
            );
            $error_flag = 1;
        }
        elsif ( $main::rss_description_maxlength eq
            $env::rss_description_maxlength )
        {
            undef $main::rss_description_maxlength;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::google_api_key
        && $main::google_api_key ne $env::google_api_key )
    {
        $main::google_api_key =~ s/[^0-9a-zA-Z_\-]//g;
        $is_valid = 1;
    }
    else {
        undef $main::google_api_key;
    }
    if ( defined $main::google_custom_search_cx
        && $main::google_custom_search_cx ne $env::google_custom_search_cx )
    {
        $main::google_custom_search_cx =~ s/[^0-9a-zA-Z_\-:]//g;
        $is_valid = 1;
    }
    else {
        undef $main::google_custom_search_cx;
    }
    if ( defined $main::google_analytics_uacct
        && $main::google_analytics_uacct ne $env::google_analytics_uacct )
    {
        $main::google_analytics_uacct =~ s/[^0-9a-zA-Z_\-]//g;
        $is_valid = 1;
    }
    else {
        undef $main::google_analytics_uacct;
    }
    if ( defined $main::google_analytics_embedded
        && $main::google_analytics_embedded ne '' )
    {
        $main::google_analytics_embedded = '0'
            if $main::google_analytics_embedded ne '1';
        if ( $main::google_analytics_embedded eq
            $env::google_analytics_embedded )
        {
            undef $main::google_analytics_embedded;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::amazon_access_key && $main::amazon_access_key ne '' )
    {
        $main::amazon_access_key =~ s/['\x09\x0A\x0C\x0D\x20]+//g;
        if ( $main::amazon_access_key eq $env::amazon_access_key ) {
            undef $main::amazon_access_key;
        }
        else {
            $is_valid = 1;
        }
    }
    elsif ( $env::amazon_access_key && $main::delete_amazon_access_key ) {
        $main::amazon_access_key = '';
    }
    else {
        undef $main::amazon_access_key;
    }
    if ( defined $main::amazon_secret_access_key
        && $main::amazon_secret_access_key ne '' )
    {
        $main::amazon_secret_access_key =~ s/['\x09\x0A\x0C\x0D\x20]+//g;
        if ( $main::amazon_secret_access_key eq
            $env::amazon_secret_access_key )
        {
            undef $main::amazon_secret_access_key;
        }
        else {
            $is_valid = 1;
        }
    }
    elsif ($env::amazon_secret_access_key
        && $main::delete_amazon_secret_access_key )
    {
        $main::amazon_secret_access_key = '';
    }
    else {
        undef $main::amazon_secret_access_key;
    }
    if ( defined $main::amazon_associates_id ) {
        $main::amazon_associates_id =~ s/['\x09\x0A\x0C\x0D\x20]+//g;
        if ( $main::amazon_associates_id eq $env::amazon_associates_id ) {
            undef $main::amazon_associates_id;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::fb_appid ) {
        $main::fb_appid =~ s/['\x09\x0A\x0C\x0D\x20]+//g;
        if ( $main::fb_appid eq $env::fb_appid ) {
            undef $main::fb_appid;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::og_type ) {
        if ( $main::og_type =~ /[^a-z]/ || $main::og_type eq $env::og_type ) {
            undef $main::og_type;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( defined $main::cookie_lifetime && $main::cookie_lifetime ne '' ) {
        if (   $this->is_nan($main::cookie_lifetime)
            || $main::cookie_lifetime < 3 )
        {
            push(
                @{ $this->{'dialog'} },
                'Cookie 保存日数を確認してください。3以上の数にしてください。'
            );
            $error_flag = 1;
        }
        elsif ( $main::cookie_lifetime == $env::cookie_lifetime ) {
            undef $main::cookie_lifetime;
        }
        else {
            $is_valid = 1;
        }
    }
    if ( !defined $mobile_mode_disabled ) {
        my $cmonos_js
            = $this->read_file( $env::javascript . '/cmonos.js', 1 );
        $mobile_mode_disabled
            = ( $cmonos_js =~ /mobile_mode_disabled\s*:\s*true\s*,/s )
            ? 1
            : 0;
        if ($main::a
            && (   ( !$main::mobile_mode_disabled && $mobile_mode_disabled )
                || ( $main::mobile_mode_disabled && !$mobile_mode_disabled ) )
            )
        {
            my $disabled = ($main::mobile_mode_disabled) ? 'true' : 'false';
            if ( $cmonos_js
                =~ s/mobile_mode_disabled\s*:\s*(?:false|true)\s*,/mobile_mode_disabled : $disabled,/sg
                )
            {
                $this->write_file( $env::javascript . '/cmonos.js',
                    \$cmonos_js, 1 );
                $mobile_mode_disabled = ($main::mobile_mode_disabled) ? 1 : 0;
                push(
                    @{ $this->{'dialog'} },
                    'スマートフォン用表示切り替えを'
                        . (
                        ( !$mobile_mode_disabled )
                        ? '有効に'
                        : '無効化'
                        )
                        . 'しました。'
                );
            }
        }
    }
    if ($main::a) {
        if ( !$main::master_email_required && $env::master_email_required ) {
            $main::master_email_required = 0;
            $is_valid                    = 1;
        }
        elsif ( $main::master_email_required && !$env::master_email_required )
        {
            $main::master_email_required = 1;
            $is_valid                    = 1;
        }
        else {
            undef $main::master_email_required;
        }
        if (  !$main::register_in_same_window
            && $env::register_in_same_window )
        {
            $main::register_in_same_window = 0;
            $is_valid                      = 1;
        }
        elsif ( $main::register_in_same_window
            && !$env::register_in_same_window )
        {
            $main::register_in_same_window = 1;
            $is_valid                      = 1;
        }
        else {
            undef $main::register_in_same_window;
        }
        if ( !$main::noindex && $env::noindex ) {
            $main::noindex = 0;
            $is_valid      = 1;
        }
        elsif ( $main::noindex && !$env::noindex ) {
            $main::noindex = 1;
            $is_valid      = 1;
        }
        else {
            undef $main::noindex;
        }
        if ( !$main::no_generator && $env::no_generator ) {
            $main::no_generator = 0;
            $is_valid           = 1;
        }
        elsif ( $main::no_generator && !$env::no_generator ) {
            $main::no_generator = 1;
            $is_valid           = 1;
        }
        else {
            undef $main::no_generator;
        }
    }
    my $new_ogimage = 0;
    if ($main::image) {
        unless ( ref $cgi::Image ) {
            require CMONOS::CGI_Lib::Image;
            $cgi::Image = Image->new;
        }
        my $is_valid_image = 1;
        my ( $width, $height, $ext )
            = $cgi::Image->input_info( $cgi::input, 'image', 0, 640, 640, 1 );

        #MIME-TYPEチェック
        if ( !defined $ext || $ext eq '' ) {
            push(
                @{ $this->{'dialog'} },
                'サイトのサムネイル画像(og:image)のファイルタイプを確認してください。'
            );
            $is_valid_image = 0;
        }

        #画像サイズチェック
        if ( $width && ( $width < 50 || $width > 640 ) ) {
            push(
                @{ $this->{'dialog'} },
                'サイトのサムネイル画像(og:image)の横幅を確認してください。50ピクセル以上640ピクセル以内にしてください。'
            );
            $is_valid_image = 0;
        }
        if ( $height && ( $height < 50 || $height > 640 ) ) {
            push(
                @{ $this->{'dialog'} },
                'サイトのサムネイル画像(og:image)の高さを確認してください。50ピクセル以上640ピクセル以内にしてください。'
            );
            $is_valid_image = 0;
        }

        if ($is_valid_image) {
            $cgi::input->save_file( 'image', '!og', '', '', '../../', );
            $new_ogimage = 1;
            $is_valid    = 1;
        }
    }

    #env.pl書き換え
    if ( $main::a && $is_valid && !$error_flag ) {
        local (*FILE);
        my ( $data, $env_path, $top_cgi_path, $cookie_path, $top_dir_fixed );
        $env_path = $default::cgi_data . '/lib/env.pl';
        if ( $install && $default::http_home_dir ) {
            $env::top_dir           = $default::http_home_dir;
            $default::http_home_dir = '';
            $top_dir_fixed          = 1;
        }
        $data = $this->read_file( $env_path, 1 );
        $data
            =~ s/^[\x20\x09]*\$master_password[\x20\x09\x0A]*=[^;]*;/\$master_password = '$main::password';/gm
            if defined $main::password && $main::password ne '';
        $data
            =~ s/^[\x20\x09]*\$master_password_encrypted[\x20\x09\x0A]*=[^;]*;/\$master_password_encrypted = 1;/gm
            if defined $main::password
            && $main::password ne ''
            && $master_password_encrypted;
        $data
            =~ s/^[\x20\x09]*\$master_email_required[\x20\x09\x0A]*=[^;]*;/\$master_email_required = $main::master_email_required;/gm
            if defined $main::master_email_required
            && $main::master_email_required ne '';
        $data
            =~ s/^[\x20\x09]*\$master_email[\x20\x09\x0A]*=[^;]*;/\$master_email = '$main::emailaddr';/gm
            if defined $main::emailaddr && $main::emailaddr ne '';
        $data
            =~ s/^[\x20\x09]*\$register_in_same_window[\x20\x09\x0A]*=[^;]*;/\$register_in_same_window = $main::register_in_same_window;/gm
            if defined $main::register_in_same_window
            && $main::register_in_same_window ne '';
        $data
            =~ s/^[\x20\x09]*\$title[\x20\x09\x0A]*=.*?';/\$title = '$main::title';/gm
            if defined $main::title && $main::title ne '';
        $data
            =~ s/^[\x20\x09]*\$copyright[\x20\x09\x0A]*=.*?';/\$copyright = '$main::copyright';/gm
            if defined $main::copyright;
        $data
            =~ s/^[\x20\x09]*\$description[\x20\x09\x0A]*=.*?';/\$description = '$main::description';/gm
            if defined $main::description;
        $data
            =~ s/^[\x20\x09]*\$rss_marker[\x20\x09\x0A]*=[^;]*;/\$rss_marker = $main::rss_marker;/gm
            if defined $main::rss_marker && $main::rss_marker ne '';
        $data
            =~ s/^[\x20\x09]*\$doctype[\x20\x09\x0A]*=[^;]*;/\$doctype = '$doctype';/gm
            if defined $doctype && $doctype ne '';

        if ( defined $main::html_header ) {
            my $html_header = $main::html_header;
            $html_header =~ s/\\/\\\\/sg;
            $html_header =~ s/'/\\'/sg;
            $data
                =~ s/[\x20\x09]*\$html_header[\x20\x09\x0A]*=.*?[^\\]';/\$html_header = '$html_header';/sg;
        }
        $data
            =~ s/^[\x20\x09]*\$inline_navi[\x20\x09\x0A]*=[^;]*;/\$inline_navi = $main::inline_navi;/gm
            if defined $main::inline_navi && $main::inline_navi ne '';
        $data
            =~ s/^[\x20\x09]*\$window_title_format[\x20\x09\x0A]*=.*?';/\$window_title_format = '$main::window_title_format';/gm
            if defined $main::window_title_format
            && $main::window_title_format ne '';
        $data
            =~ s/^[\x20\x09]*\$session_enabled[\x20\x09\x0A]*=[^;]*;/\$session_enabled = $main::session_enabled;/gm
            if defined $main::session_enabled && $main::session_enabled ne '';
        $data
            =~ s/^[\x20\x09]*\$session_security[\x20\x09\x0A]*=[^;]*;/\$session_security = '$main::session_security';/gm
            if defined $main::session_security
            && $main::session_security ne '';
        $data
            =~ s/^[\x20\x09]*\$openid_enabled[\x20\x09\x0A]*=[^;]*;/\$openid_enabled = $main::openid_enabled;/gm
            if defined $main::openid_enabled && $main::openid_enabled ne '';
        if ( defined $main::password_threshold ) {
            if ( $main::password_threshold ne '' ) {
                $data
                    =~ s/^[\x20\x09]*\$password_threshold[\x20\x09\x0A]*=[^;]*;/\$password_threshold = $main::password_threshold;/gm;
            }
            else {
                $data
                    =~ s/^[\x20\x09]*\$password_threshold[\x20\x09\x0A]*=[^;]*;/\$password_threshold = '';/gm;
            }
        }
        if ( defined $main::password_latency ) {
            if ( $main::password_latency ne '' ) {
                $data
                    =~ s/^[\x20\x09]*\$password_latency[\x20\x09\x0A]*=[^;]*;/\$password_latency = $main::password_latency;/gm;
            }
            else {
                $data
                    =~ s/^[\x20\x09]*\$password_latency[\x20\x09\x0A]*=[^;]*;/\$password_latency = '';/gm;
            }
        }
        $data
            =~ s/^[\x20\x09]*\$rss_maxitem[\x20\x09\x0A]*=[^;]*;/\$rss_maxitem = $main::rss_maxitem;/gm
            if defined $main::rss_maxitem && $main::rss_maxitem ne '';
        $data
            =~ s/^[\x20\x09]*\$rss_description_maxlength[\x20\x09\x0A]*=[^;]*;/\$rss_description_maxlength = $main::rss_description_maxlength;/gm
            if defined $main::rss_description_maxlength
            && $main::rss_description_maxlength ne '';
        $data
            =~ s/^[\x20\x09]*\$google_api_key[\x20\x09\x0A]*=[^;]*;/\$google_api_key = '$main::google_api_key';/gm
            if defined $main::google_api_key;
        $data
            =~ s/^[\x20\x09]*\$google_custom_search_cx[\x20\x09\x0A]*=[^;]*;/\$google_custom_search_cx = '$main::google_custom_search_cx';/gm
            if defined $main::google_custom_search_cx;
        $data
            =~ s/^[\x20\x09]*\$google_analytics_uacct[\x20\x09\x0A]*=[^;]*;/\$google_analytics_uacct = '$main::google_analytics_uacct';/gm
            if defined $main::google_analytics_uacct;
        $data
            =~ s/^[\x20\x09]*\$google_analytics_embedded[\x20\x09\x0A]*=[^;]*;/\$google_analytics_embedded = $main::google_analytics_embedded;/gm
            if defined $main::google_analytics_embedded
            && $main::google_analytics_embedded ne '';
        $data
            =~ s/^[\x20\x09]*\$amazon_access_key[\x20\x09\x0A]*=[^;]*;/\$amazon_access_key = '$main::amazon_access_key';/gm
            if defined $main::amazon_access_key;
        $data
            =~ s/^[\x20\x09]*\$amazon_secret_access_key[\x20\x09\x0A]*=[^;]*;/\$amazon_secret_access_key = '$main::amazon_secret_access_key';/gm
            if defined $main::amazon_secret_access_key;
        $data
            =~ s/^[\x20\x09]*\$amazon_associates_id[\x20\x09\x0A]*=[^;]*;/\$amazon_associates_id = '$main::amazon_associates_id';/gm
            if defined $main::amazon_associates_id;
        $data
            =~ s/^[\x20\x09]*\$fb_appid[\x20\x09\x0A]*=[^;]*;/\$fb_appid = '$main::fb_appid';/gm
            if defined $main::fb_appid;
        $data
            =~ s/^[\x20\x09]*\$og_type[\x20\x09\x0A]*=[^;]*;/\$og_type = '$main::og_type';/gm
            if defined $main::og_type;
        $data
            =~ s/^[\x20\x09]*\$cookie_lifetime[\x20\x09\x0A]*=[^;]*;/\$cookie_lifetime = $main::cookie_lifetime;/gm
            if defined $main::cookie_lifetime && $main::cookie_lifetime ne '';
        $data
            =~ s/^[\x20\x09]*\$noindex[\x20\x09\x0A]*=[^;]*;/\$noindex = $main::noindex;/gm
            if defined $main::noindex && $main::noindex ne '';
        $data
            =~ s/^[\x20\x09]*\$no_generator[\x20\x09\x0A]*=[^;]*;/\$no_generator = $main::no_generator;/gm
            if defined $main::no_generator && $main::no_generator ne '';

        if ($main::user_id) {
            if ( $main::user_id eq 'shared' ) {
                $cookie_path = $this->user_directory(1)
                    . ( ($env::top_dir) ? $env::top_dir : '' );
                $cookie_path = '/' if !$cookie_path;
                $top_cgi_path = $env::image_lib;
                $top_cgi_path =~ s/\/[^\/]+$//;
                $data
                    =~ s!^[\x20\x09]*\$session_store[\x20\x09\x0A]*=[^;]*;!\$session_store = '$default::cgi_data_root/data/session_store';!gm;
                $data
                    =~ s!^[\x20\x09]*\$register_cgi[\x20\x09\x0A]*=[^;]*;!\$register_cgi = '$top_cgi_path/pswd/start.cgi';!gm;
                $data
                    =~ s!^[\x20\x09]*\$register_db[\x20\x09\x0A]*=[^;]*;!\$register_db = '$default::cgi_data_root/preferences/register';!gm;
                $data
                    =~ s/^[\x20\x09]*\$cookie_path[\x20\x09\x0A]*=[^;]*;/\$cookie_path = '$cookie_path';/gm;
            }
            else {
                $cookie_path
                    = $this->user_directory(1)
                    . ( ($env::top_dir) ? $env::top_dir : '' )
                    . ( ($default::http_home_dir)
                    ? $default::http_home_dir
                    : '' );
                $cookie_path = '/' if !$cookie_path;
                $data
                    =~ s!^[\x20\x09]*\$session_store[\x20\x09\x0A]*=[^;]*;!\$session_store = '$default::cgi_data/data/session_store';!gm;
                $data
                    =~ s!^[\x20\x09]*\$register_cgi[\x20\x09\x0A]*=[^;]*;!\$register_cgi = '../pswd/start.cgi';!gm;
                $data
                    =~ s!^[\x20\x09]*\$register_db[\x20\x09\x0A]*=[^;]*;!\$register_db = '$default::cgi_data/preferences/register';!gm;
                $data
                    =~ s/^[\x20\x09]*\$cookie_path[\x20\x09\x0A]*=[^;]*;/\$cookie_path = '$cookie_path';/gm;
            }
        }
        $data
            =~ s/^[\x20\x09]*#?[\x20\x09]*\$top_dir[\x20\x09\x0A]*=[^;]*;/\$top_dir = '$env::top_dir';/gm
            if $top_dir_fixed;
        if ( open( FILE, ">>$env_path" ) ) {
            binmode FILE, ":utf8";
            flock( FILE, LOCK_EX );
            truncate( FILE, 0 );
            seek( FILE, 0, 0 );
            print FILE $data;
            close(FILE);
            push(
                @{ $this->{'dialog'} },
                '管理情報を更新しました。'
            ) if !$install;
            $this->rename_navi_file($main::inline_navi)
                if defined $main::inline_navi && $main::inline_navi ne '';

            if ($main::user_id) {
                if ( $main::user_id eq 'shared' ) {
                    $env::session_store
                        = "$default::cgi_data_root/data/session_store";
                    $env::register_cgi = "$top_cgi_path/pswd/start.cgi";
                    $env::register_db
                        = "$default::cgi_data_root/preferences/register";
                    $env::cookie_path = $cookie_path;
                }
                else {
                    $env::session_store
                        = "$default::cgi_data/data/session_store";
                    $env::register_cgi = '../pswd/start.cgi';
                    $env::register_db
                        = "$default::cgi_data/preferences/register";
                    $env::cookie_path = $cookie_path;
                }
                if ( $env::session_enabled && $this->use_session(1) ) {
                    $session::id = $this->read_cookie_direct(
                        $this->login_cookie_name(1) );
                    $session::is_changed = 1;
                }
                else {
                    $this->use_register(1);
                    $this->login_cookie_name(1);
                    $main::set_cookie_id = 1;
                }
            }
            if (   ( defined $main::password && $main::password ne '' )
                || ( defined $main::emailaddr && $main::emailaddr ne '' ) )
            {
                $env::master_password = $main::pswd = $main::password
                    if defined $main::password && $main::password ne '';
                $env::master_password_encrypted = 1
                    if defined $main::password
                    && $main::password ne ''
                    && $master_password_encrypted;
                $env::master_email = $main::email = $main::emailaddr
                    if defined $main::emailaddr && $main::emailaddr ne '';
                if ( $env::session_enabled && $this->use_session(1) ) {
                    $session::id
                        = $cgi::Session->set_user_info( $main::email,
                        { 'encrypted_password' => $main::pswd },
                        $session::id );
                    $session::is_changed = 1;
                }
                else {
                    $this->use_register(1);
                    $main::set_cookie_id = 1;
                }
            }
            $env::master_email_required = $main::master_email_required
                if defined $main::master_email_required
                && $main::master_email_required ne '';
            $env::register_in_same_window = $main::register_in_same_window
                if defined $main::register_in_same_window
                && $main::register_in_same_window ne '';
            @title_and_description = (
                $env::title,               $env::description,
                $env::window_title_format, $main::title,
                $main::description,        $main::window_title_format,
                $doctype,                  $env::html_header,
                $main::html_header,        $main::noindex,
                $main::fb_appid,           $main::no_generator,
                $new_ogimage
                )
                if !$install
                && ( defined $main::title
                || defined $main::description
                || defined $main::window_title_format
                || defined $doctype
                || defined $main::html_header
                || defined $main::noindex
                || defined $main::fb_appid
                || defined $main::no_generator
                || $new_ogimage );
            $env::title = $main::title
                if defined $main::title && $main::title ne '';
            $env::copyright = $main::copyright if defined $main::copyright;
            $env::description = $main::description
                if defined $main::description;
            $env::rss_marker = $main::rss_marker
                if defined $main::rss_marker && $main::rss_marker ne '';
            $env::doctype = $doctype if defined $doctype && $doctype ne '';
            $env::html_header = $main::html_header
                if defined $main::html_header;
            $env::inline_navi = $main::inline_navi
                if defined $main::inline_navi && $main::inline_navi ne '';
            $env::window_title_format = $main::window_title_format
                if defined $main::window_title_format
                && $main::window_title_format ne '';
            $env::session_enabled = $main::session_enabled
                if defined $main::session_enabled
                && $main::session_enabled ne '';
            $env::session_security = $main::session_security
                if defined $main::session_security
                && $main::session_security ne '';
            $env::openid_enabled = $main::openid_enabled
                if defined $main::openid_enabled
                && $main::openid_enabled ne '';
            $env::password_threshold = $main::password_threshold
                if defined $main::password_threshold;
            $env::password_latency = $main::password_latency
                if defined $main::password_latency;
            $env::rss_maxitem = $main::rss_maxitem
                if defined $main::rss_maxitem && $main::rss_maxitem ne '';
            $env::rss_description_maxlength
                = $main::rss_description_maxlength
                if defined $main::rss_description_maxlength
                && $main::rss_description_maxlength ne '';
            $env::google_api_key = $main::google_api_key
                if defined $main::google_api_key;
            $env::google_custom_search_cx = $main::google_custom_search_cx
                if defined $main::google_custom_search_cx;
            $env::google_analytics_uacct = $main::google_analytics_uacct
                if defined $main::google_analytics_uacct;
            $env::google_analytics_embedded
                = $main::google_analytics_embedded
                if defined $main::google_analytics_embedded
                && $main::google_analytics_embedded ne '';
            $env::amazon_access_key = $main::amazon_access_key
                if defined $main::amazon_access_key
                && $main::amazon_access_key ne '';
            $env::amazon_secret_access_key = $main::amazon_secret_access_key
                if defined $main::amazon_secret_access_key
                && $main::amazon_secret_access_key ne '';
            $env::amazon_associates_id = $main::amazon_associates_id
                if defined $main::amazon_associates_id;
            $env::fb_appid = $main::fb_appid if defined $main::fb_appid;
            $env::og_type  = $main::og_type  if defined $main::og_type;
            $env::cookie_lifetime = $main::cookie_lifetime
                if defined $main::cookie_lifetime
                && $main::cookie_lifetime ne '';
            $env::noindex = $main::noindex
                if defined $main::noindex && $main::noindex ne '';
            $env::no_generator = $main::no_generator
                if defined $main::no_generator && $main::no_generator ne '';
            undef $main::emailaddr;
            undef $main::password;
            undef $main::master_email_required;
            undef $main::register_in_same_window;
            undef $main::title;
            undef $main::description;
            undef $main::copyright;
            undef $main::rss_marker;
            undef $main::doctype;
            undef $main::html_header;
            undef $main::inline_navi;
            undef $main::window_title_format;
            undef $main::session_enabled;
            undef $main::session_security;
            undef $main::openid_enabled;
            undef $main::user_id;
            undef $main::password_threshold;
            undef $main::password_latency;
            undef $main::rss_maxitem;
            undef $main::rss_description_maxlength;
            undef $main::google_api_key;
            undef $main::google_custom_search_cx;
            undef $main::google_analytics_uacct;
            undef $main::google_analytics_embedded;
            undef $main::amazon_access_key;
            undef $main::amazon_secret_access_key;
            undef $main::delete_amazon_access_key;
            undef $main::delete_amazon_secret_access_key;
            undef $main::amazon_associates_id;
            undef $main::fb_appid;
            undef $main::og_type;
            undef $main::cookie_lifetime;
            undef $main::noindex;
            undef $main::no_generator;
            return 1 if $install;
        }
        else {
            push(
                @{ $this->{'dialog'} },
                (   ($install)
                    ? '管理情報の設定に失敗しました。'
                    : '管理情報の更新に失敗しました。'
                    )
                    . $env_path
            );
        }
    }
    elsif ( defined $error_flag && $error_flag ) {
        push(
            @{ $this->{'dialog'} },
            (   ($install)
                ? '管理情報の設定に失敗しました。'
                : '管理情報の更新に失敗しました。'
            )
        );
    }
    undef $doctype;

    #ページヘッダ
    if (@title_and_description) {
        require CMONOS::CGI_Lib::ProgressBar;
        $cgi::ProgressBar = ProgressBar->new;
        $this->set_javascript( $cgi::ProgressBar->javascript );
    }
    $this->jump_menu('tools') if !$install;
    $this->print_page_header('tools');

    if ($install) {
        print '<h1>管理情報設定→インストール</h1>';
        print
            '<p class="note">これら管理情報はインストール後変更することができます。</p>';
    }
    else {
        $cgi::ProgressBar->print_progress_bar('ファイル')
            if @title_and_description;
        print '<h1>管理情報変更</h1>';
    }
    print
        '<fieldset><legend>リフレッシュログ</legend><ol id="refresh-log"></ol></fieldset>'
        if @title_and_description && $main::log;
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'multi'    => 1,
        'onsubmit' => 'return confirm(\''
            . (
            ($install)
            ? 'この設定でよろしいですか？　これら管理情報はインストール後も、「セットアップアシスタント」で変更することができます。'
            : 'この操作は取り消せません。本当に変更しますか？'
            )
            . '\') &amp;&amp; cmonos.form.item(this).isNotSubmitted(true);',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="1" />';
    print '<fieldset><table class="field-table"><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="password">マスターパスワード</label></td>';
    print
        '<td class="field"><input type="password" name="password" id="password" size="50" /></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="password2">マスターパスワード（確認）</label></td>';
    print
        '<td class="field"><input type="password" name="password2" id="password2" size="50" />';
    print '<div class="hints">パスワードの例 "'
        . $this->rand_char_sample
        . '"</div>';
    print( ($install)
        ? '<div class="warning">必ず設定してください。</div>'
        : '<div class="hints">無記入の場合変更されません。</div>'
    );
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="emailaddr">管理人アドレス</label></td>';
    print
        '<td class="field"><input type="email" name="emailaddr" id="emailaddr" size="30" value="'
        . $this->escape(
        ( defined $main::emailaddr && $main::emailaddr ne '' )
        ? $main::emailaddr
        : $env::master_email )
        . '" />';
    print( ($install)
        ? '<div class="warning">必ず設定してください。</div>'
        : '' );
    print '</td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><span class="datatitle">マスターパスワード認証</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="master_email_required" value="1"'
        . (
        (   $main::master_email_required
                || ( !defined $main::master_email_required
                && $env::master_email_required )
        ) ? ' checked="checked"' : ''
        )
        . ' /> マスターパスワードを認証する際、管理人メールアドレスを要求する</label></td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="session_enabled">ユーザー情報の記憶</label></td>';
    print
        '<td class="field"><select name="session_enabled" id="session_enabled">';
    print '<option value="2"'
        . (
        (   (   defined $main::session_enabled
                    && $main::session_enabled eq '2'
            )
                || ( !defined $main::session_enabled
                && $env::session_enabled eq '2' )
        ) ? ' selected="selected"' : ''
        )
        . '>全てのユーザー情報をセッション管理によって保持する</option>';
    print '<option value="1"'
        . (
        (   (   defined $main::session_enabled
                    && $main::session_enabled eq '1'
            )
                || ( !defined $main::session_enabled
                && $env::session_enabled eq '1' )
        ) ? ' selected="selected"' : ''
        )
        . '>パスワード情報とショッピングカート情報をセッション管理によって保持する</option>';
    print '<option value="0"'
        . (
        (   (   defined $main::session_enabled
                    && $main::session_enabled eq '0'
            )
                || ( !defined $main::session_enabled
                && $env::session_enabled eq '0' )
        ) ? ' selected="selected"' : ''
        ) . '>Cookie に全てのユーザー情報を記憶する</option>';
    print '</select>';
    print
        '<div class="hints">セッション管理ではユーザーごとにユニークな ID が割り振られ ID に対応した<strong>ユーザー情報がサーバ上に保存</strong>されます。</div>';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="cookie_lifetime">Cookie 保存日数</label></td>';
    print
        '<td class="field"><input type="number" min="0" name="cookie_lifetime" id="cookie_lifetime" size="6" value="'
        . $this->escape(
        ( defined $main::cookie_lifetime && $main::cookie_lifetime ne '' )
        ? $main::cookie_lifetime
        : $env::cookie_lifetime )
        . '" /> 日';
    print '</td>';
    print '</tr>';

    if ($env::session_enabled) {
        print '<tr>';
        print
            '<td class="field-name"><label for="session_security">パスワードの再確認</label></td>';
        print
            '<td class="field"><select name="session_security" id="session_security">';
        print '<option value="ip,agent"'
            . (
            (   (   defined $main::session_security
                        && $main::session_security eq 'ip,agent'
                )
                    || ( !defined $main::session_security
                    && $env::session_security eq 'ip,agent' )
            ) ? ' selected="selected"' : ''
            )
            . '>前回認証時と同一IPかつ同一エージェントでない場合パスワードを再確認する</option>';
        print '<option value="ip"'
            . (
            (   (   defined $main::session_security
                        && $main::session_security eq 'ip'
                )
                    || ( !defined $main::session_security
                    && $env::session_security eq 'ip' )
            ) ? ' selected="selected"' : ''
            )
            . '>前回認証時と同一IPでない場合パスワードを再確認する</option>';
        print '<option value="agent"'
            . (
            (   (   defined $main::session_security
                        && $main::session_security eq 'agent'
                )
                    || ( !defined $main::session_security
                    && $env::session_security eq 'agent' )
            ) ? ' selected="selected"' : ''
            )
            . '>前回認証時と同一エージェントでない場合パスワードを再確認する</option>';
        print '<option value="none"'
            . (
            (   (   defined $main::session_security
                        && $main::session_security eq 'none'
                )
                    || ( !defined $main::session_security
                    && $env::session_security eq 'none' )
            ) ? ' selected="selected"' : ''
            )
            . '>前回認証時とセッションIDが一致すればパスワードを確認しない</option>';
        print '</select>';
        print '</td>';
        print '</tr>';

        if ( eval { require LWP::UserAgent } && eval 'use Crypt::SSLeay;1;' )
        {
            print '<tr>';
            print
                '<td class="field-name"><label for="openid_enabled">OpenID による認証</label></td>';
            print
                '<td class="field"><select name="openid_enabled" id="openid_enabled">';
            print '<option value="0"'
                . (
                (   (   defined $main::openid_enabled
                            && $main::openid_enabled eq '0'
                    )
                        || ( !defined $main::openid_enabled
                        && $env::openid_enabled eq '0' )
                ) ? ' selected="selected"' : ''
                ) . '>OpenID による認証を許可しない</option>';
            print '<option value="1"'
                . (
                (   (   defined $main::openid_enabled
                            && $main::openid_enabled eq '1'
                    )
                        || ( !defined $main::openid_enabled
                        && $env::openid_enabled eq '1' )
                ) ? ' selected="selected"' : ''
                ) . '>OpenID による認証を許可する</option>';
            print '</select>';
            print '</td>';
            print '</tr>';
        }
    }
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">パスワードロック</span></td>';
    print '<td class="field">';
    print
        '<label><input type="number" name="password_threshold" id="password_threshold" size="3" min="0" value="'
        . $this->escape( ( defined $main::password_threshold )
        ? $main::password_threshold
        : $env::password_threshold )
        . '" /> 回以上パスワード認証に失敗したとき、</label>';
    print
        '<label><input type="number" name="password_latency" id="password_latency" size="3" min="0" value="'
        . $this->escape( ( defined $main::password_latency )
        ? $main::password_latency
        : $env::password_latency )
        . '" /> 分間パスワード認証を拒絶する。</label>';
    print '</td>';
    print '</tr>';

    if ( $env::home_dir_id && !$is_document_root ) {
        my $user_id
            = (    $env::register_cgi
                && $env::register_cgi eq '../pswd/start.cgi' )
            ? 'individual'
            : 'shared';
        print '<tr>';
        print
            '<td class="field-name"><label for="user_id">ログイン情報の共有</label></td>';
        print '<td class="field"><select name="user_id" id="user_id">';
        print '<option value="shared"'
            . (
            (   ( defined $main::user_id && $main::user_id eq 'shared' )
                    || ( !defined $main::user_id && $user_id eq 'shared' )
            ) ? ' selected="selected"' : ''
            ) . '>ログイン情報を最上層と共有する</option>';
        print '<option value="individual"'
            . (
            (   ( defined $main::user_id && $main::user_id eq 'individual' )
                    || ( !defined $main::user_id && $user_id eq 'individual' )
            ) ? ' selected="selected"' : ''
            )
            . '>ログイン情報をこのディレクトリでのみ利用する</option>';
        print '</select>';
        print '</td>';
        print '</tr>';
    }
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><span class="datatitle">ログインページ</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="register_in_same_window" value="1"'
        . (
        (   $main::register_in_same_window
                || ( !defined $main::register_in_same_window
                && $env::register_in_same_window )
        ) ? ' checked="checked"' : ''
        )
        . ' /> ログインページを同じウィンドウで開く</label></td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="title">サイトタイトル</label></td>';
    print
        '<td class="field"><input type="text" name="title" id="title" size="30" value="'
        . $this->escape( ( defined $main::title && $main::title ne '' )
        ? $main::title
        : $env::title )
        . '" />';
    print( ($install)
        ? '<div class="warning">必ず設定してください。</div>'
        : '' );
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="copyright">サイトの著作権者</label></td>';
    print
        '<td class="field"><input type="text" name="copyright" id="copyright" size="30" value="'
        . $this->escape(
        ( defined $main::copyright && $main::copyright ne '' )
        ? $main::copyright
        : $env::copyright )
        . '" />';
    print
        '<div class="hints">RSSのサイトの著作権表示などに使用されます。</div>';
    print '</td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="description">サイトの説明</label></td>';
    print
        '<td class="field"><textarea name="description" id="description" cols="60" rows="2">'
        . $this->escape(
        ( defined $main::description && $main::description ne '' )
        ? $main::description
        : $env::description )
        . '</textarea>';
    print
        '<div class="hints">RSSのサイトの説明などに使用されます。</div>';
    print '</td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="doctype">DOCTYPE 宣言</label></td>';
    print '<td class="field"><select name="doctype" id="doctype">';

    if (   !defined $env::doctype
        || $env::doctype =~ / XHTML 1.0 Transitional/
        || $env::doctype eq '' )
    {
        $env_doctype = 'xhtml1transitional';
    }
    elsif ( $env::doctype =~ /<!DOCTYPE html>/i ) {
        $env_doctype = 'html5';
    }
    elsif ( $env::doctype =~ / XHTML 1.0 Strict/ ) {
        $env_doctype = 'xhtml1strict';
    }
    elsif ( $env::doctype =~ / HTML 4.01 Strict/ ) {
        $env_doctype = 'html401strict';
    }
    elsif ( $env::doctype =~ / HTML 4.01 Transitional/ ) {
        $env_doctype = 'html401transitional';
    }
    elsif ( $env::doctype =~ /html4\/loose\.dtd/ ) {
        $env_doctype = 'html401loose';
    }
    else {
        $env_doctype = '';
        print '<option value="">'
            . $this->escape($env::doctype)
            . '</option>';
    }
    print '<option value="html5"'
        . (
        (          ( defined $main::doctype && $main::doctype eq 'html5' )
                || ( !defined $main::doctype && $env_doctype eq 'html5' )
        ) ? ' selected="selected"' : ''
        ) . '>HTML5</option>';
    print '<option value="xhtml1strict"'
        . (
        (   ( defined $main::doctype && $main::doctype eq 'xhtml1strict' )
                || ( !defined $main::doctype
                && $env_doctype eq 'xhtml1strict' )
        ) ? ' selected="selected"' : ''
        ) . '>XHTML1.0</option>';
    print '<option value="xhtml1transitional"'
        . (
        (   (   defined $main::doctype
                    && $main::doctype eq 'xhtml1transitional'
            )
                || ( !defined $main::doctype
                && $env_doctype eq 'xhtml1transitional' )
        ) ? ' selected="selected"' : ''
        ) . '>XHTML1.0(互換モード)</option>';
    print '<option value="html401strict"'
        . (
        (   ( defined $main::doctype && $main::doctype eq 'html401strict' )
                || ( !defined $main::doctype
                && $env_doctype eq 'html401strict' )
        ) ? ' selected="selected"' : ''
        ) . '>HTML4.01</option>';
    print '<option value="html401transitional"'
        . (
        (   (   defined $main::doctype
                    && $main::doctype eq 'html401transitional'
            )
                || ( !defined $main::doctype
                && $env_doctype eq 'html401transitional' )
        ) ? ' selected="selected"' : ''
        ) . '>HTML4.01(旧 IE 互換)</option>';
    print '<option value="html401loose"'
        . (
        (   ( defined $main::doctype && $main::doctype eq 'html401loose' )
                || ( !defined $main::doctype
                && $env_doctype eq 'html401loose' )
        ) ? ' selected="selected"' : ''
        ) . '>HTML4.01(互換モード)</option>';
    print '</select>';
    print '</td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="html_header">共通 HTML ヘッダ</label></td>';
    print
        '<td class="field"><textarea name="html_header" id="html_header" cols="60" rows="6">'
        . $this->escape(
        ( defined $main::html_header && $main::html_header ne '' )
        ? $main::html_header
        : $env::html_header )
        . '</textarea>';
    print '<div class="toolbox">';
    print
        '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'html_header\').quotationByMenu(this)">';
    print '<option value="">▼挿入する</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;meta name=&quot;&quot; content=&quot;&quot; /&gt;')
        . '">&lt;meta name=&quot;&quot; content=&quot;&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;meta name=&quot;author&quot; content=&quot;&quot; /&gt;')
        . '">&lt;meta name=&quot;author&quot; content=&quot;著作者&quot; /&gt;</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;meta http-equiv=&quot;X-UA-Compatible&quot; content=&quot;IE=edge&quot; /&gt;'
        )
        . '">&lt;meta http-equiv=&quot;X-UA-Compatible&quot; content=&quot;IE=edge&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;meta name=&quot;viewport&quot; content=&quot;width=device-width, initial-scale=1.0&quot; /&gt;'
        )
        . '">&lt;meta name=&quot;viewport&quot; content=&quot;width=device-width, initial-scale=1.0&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;meta name=&quot;format-detection&quot; content=&quot;telephone=no&quot; /&gt;'
        )
        . '">&lt;meta name=&quot;format-detection&quot; content=&quot;telephone=no&quot; /&gt;</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;&quot; href=&quot;&quot; type=&quot;&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;&quot; href=&quot;&quot; type=&quot;&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;stylesheet&quot; type=&quot;text/css&quot; media=&quot;screen, print, tty&quot; href=&quot;&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;stylesheet&quot; href=&quot;&quot; type=&quot;text/css&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;contents&quot; href=&quot;&quot; title=&quot;目次&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;contents&quot; href=&quot;目次のURI&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;index&quot; href=&quot;&quot; title=&quot;索引&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;index&quot; href=&quot;索引のURI&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;copyright&quot; href=&quot;&quot; title=&quot;&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;copyright&quot; href=&quot;著作権文書のURI&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;appendix&quot; href=&quot;&quot; title=&quot;&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;appendix&quot; href=&quot;付録文書のURI&quot; /&gt;</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;script type=&quot;text/javascript&quot; language=&quot;JavaScript&quot; src=&quot;&quot; charset=&quot;UTF-8&quot;&gt;&lt;/script&gt;'
        )
        . '">&lt;script type=&quot;text/javascript&quot; src=&quot;&quot;&gt;&lt;/script&gt;</option>';
    print '<option value=""></option>';
    print '<option value="[[REPLACE]]">[置換]</option>';
    print '</select>';
    print
        ' <label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'html_header\').setAutoFit(this.checked);" />フィット</label>';
    print '</div></td>';
    print '</tr>';
    print '</tbody><tbody>';

    if ( !$install ) {
        print '<tr class="tbody-end">';
        print
            '<td class="field-name"><label for="inline_navi">ナビゲーション</label></td>';
        print
            '<td class="field"><select name="inline_navi" id="inline_navi">';
        print '<option value="1"'
            . (
            (   ( defined $main::inline_navi && $main::inline_navi eq '1' )
                    || ( !defined $main::inline_navi
                    && $env::inline_navi eq '1' )
            ) ? ' selected="selected"' : ''
            ) . '>埋め込み方式</option>';
        print '<option value="0"'
            . (
            (   ( defined $main::inline_navi && $main::inline_navi eq '0' )
                    || ( !defined $main::inline_navi
                    && $env::inline_navi eq '0' )
            ) ? ' selected="selected"' : ''
            )
            . '>フレーム分割方式【非推奨】(HTML5非準拠/非対応環境あり)</option>';
        print '</select>';
        print '</td>';
        print '</tr>';
    }
    print '</tbody><tbody>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="inline_navi">ウィンドウタイトルフォーマット</label></td>';
    print
        '<td class="field"><select name="window_title_format" id="window_title_format">';
    foreach my $wf ( ':,:', ' | , | ', ' | , | ,1', ' | , &lt; ,1', ' | ,,1',
        ' | ' )
    {
        print '<option value="'
            . $wf . '"'
            . (
            (   (   defined $main::window_title_format
                        && $main::window_title_format eq $wf
                )
                    || ( !defined $main::window_title_format
                    && $env::window_title_format eq $wf )
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->window_title( $wf, 'サイトタイトル',
            '大カテゴリー', '小カテゴリー',
            'ページタイトル' )
            . '</option>';
    }
    print '</select>';
    print( ($install)
        ? ''
        : '<div class="warning">カテゴリーを省略するフォーマットからカテゴリーを省略しないフォーマットへ変更する場合、静的書き出しページでは、記事を更新しない限りカテゴリーが省略されたままになります。</div>'
    );
    print '</td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><span class="datatitle">表示切り替え</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="mobile_mode_disabled" value="1"'
        . ( ( $main::mobile_mode_disabled || $mobile_mode_disabled )
        ? ' checked="checked"'
        : '' )
        . ' /> スマートフォンで スマートフォン用 CSS (mobile.css) と PC用 CSS (default.css 等) を切り替えるボタンを<strong>使用しない</strong></label></td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="rss_maxitem">RSS 最大記事数</label></td>';
    print
        '<td class="field"><input type="number" name="rss_maxitem" id="rss_maxitem" size="3" min="0" value="'
        . $this->escape(
        ( defined $main::rss_maxitem && $main::rss_maxitem ne '' )
        ? $main::rss_maxitem
        : $env::rss_maxitem )
        . '" /> 件';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="rss_description_maxlength">RSS 説明項目最大字数</label></td>';
    print
        '<td class="field"><input type="number" name="rss_description_maxlength" id="rss_description_maxlength" size="3" min="0" value="'
        . $this->escape(
        (   defined $main::rss_description_maxlength
                && $main::rss_description_maxlength ne ''
        )
        ? $main::rss_description_maxlength
        : $env::rss_description_maxlength
        ) . '" /> 字(半角)以内';
    print '</td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="description">RSS のリンク先 URL</label></td>';
    print '<td class="field"><select name="rss_marker" id="rss_marker">';
    print '<option value="0"'
        . (
        (          ( defined $main::rss_marker && $main::rss_marker eq '0' )
                || ( !defined $main::rss_marker && $env::rss_marker eq '0' )
        ) ? ' selected="selected"' : ''
        )
        . '>RSS からのアクセスであることを示す変数を追加しない</option>';
    print '<option value="1"'
        . (
        (          ( defined $main::rss_marker && $main::rss_marker eq '1' )
                || ( !defined $main::rss_marker && $env::rss_marker eq '1' )
        ) ? ' selected="selected"' : ''
        )
        . '>RSS からのアクセスであることを示す変数を追加する</option>';
    print '</select>';
    print '</td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="google_api_key"><a href="https://console.developers.google.com/" target="_blank">Google API キー</a></label></td>';
    print
        '<td class="field"><input type="text" name="google_api_key" id="google_api_key" size="50" value="'
        . $this->escape(
        ( defined $main::google_api_key && $main::google_api_key ne '' )
        ? $main::google_api_key
        : $env::google_api_key )
        . '" />';
    print '</td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="google_custom_search_cx"><a href="https://www.google.co.jp/cse/all" target="_blank">Google カスタム検索 検索エンジン ID (cx)</a></label></td>';
    print
        '<td class="field"><input type="text" name="google_custom_search_cx" id="google_custom_search_cx" size="50" value="'
        . $this->escape(
        (   defined $main::google_custom_search_cx
                && $main::google_custom_search_cx ne ''
        ) ? $main::google_custom_search_cx : $env::google_custom_search_cx
        ) . '" />';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="google_analytics_uacct"><a href="http://www.google.co.jp/intl/ja_ALL/analytics/" target="_blank">Google Analytics ユーザーアカウント</a></label></td>';
    print
        '<td class="field"><input type="text" name="google_analytics_uacct" id="google_analytics_uacct" size="30" value="'
        . $this->escape(
        (   defined $main::google_analytics_uacct
                && $main::google_analytics_uacct ne ''
        ) ? $main::google_analytics_uacct : $env::google_analytics_uacct
        ) . '" />';
    print '</td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="google_analytics_embedded">Google Analytics の利用</a></label></td>';
    print
        '<td class="field"><select name="google_analytics_embedded" id="google_analytics_embedded">';
    print '<option value="1"'
        . (
        (   (   defined $main::google_analytics_embedded
                    && $main::google_analytics_embedded eq '1'
            )
                || ( !defined $main::google_analytics_embedded
                && $env::google_analytics_embedded eq '1' )
        ) ? ' selected="selected"' : ''
        )
        . '>デフォルトでトラッキングコードを埋め込む</option>';
    print '<option value="0"'
        . (
        (   (   defined $main::google_analytics_embedded
                    && $main::google_analytics_embedded eq '0'
            )
                || ( !defined $main::google_analytics_embedded
                && $env::google_analytics_embedded eq '0' )
        ) ? ' selected="selected"' : ''
        )
        . '>デフォルトではトラッキングコードを埋め込まない</option>';
    print '</select>';
    print '</td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="amazon_access_key"><a href="https://aws-portal.amazon.com/gp/aws/developer/registration/index.html" target="_blank">Amazon アクセスキー</a></label></td>';
    print
        '<td class="field"><input type="text" name="amazon_access_key" id="amazon_access_key" size="30" value="'
        . $this->escape($main::amazon_access_key) . '" />';

    if ($env::amazon_access_key) {
        print
            ' <label><input type="checkbox" name="delete_amazon_access_key" id="delete_amazon_access_key" value="1"'
            . (
            ($main::delete_amazon_access_key) ? ' checked="checked"' : '' )
            . ' />削除';
        print '<div class="hints">現在の設定='
            . $this->escape( $this->blur_text($env::amazon_access_key) )
            . '</div>';
        print
            '<div class="hints">無記入の場合変更されません。</div>';
    }
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="amazon_secret_access_key">Amazon 秘密のアクセスキー</label></td>';
    print
        '<td class="field"><input type="text" name="amazon_secret_access_key" id="amazon_secret_access_key" size="50" value="'
        . $this->escape($main::amazon_secret_access_key) . '" />';
    if ($env::amazon_secret_access_key) {
        print
            ' <label><input type="checkbox" name="delete_amazon_secret_access_key" id="delete_amazon_secret_access_key" value="1"'
            . ( ($main::delete_amazon_secret_access_key)
            ? ' checked="checked"'
            : '' )
            . ' />削除';
        print '<div class="hints">現在の設定='
            . $this->escape(
            $this->blur_text($env::amazon_secret_access_key) )
            . '</div>';
        print
            '<div class="hints">無記入の場合変更されません。</div>';
    }
    print '</td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="amazon_associates_id">Amazon アソシエイト ID</label></td>';
    print
        '<td class="field"><input type="text" name="amazon_associates_id" id="amazon_associates_id" size="30" value="'
        . $this->escape(
        (   defined $main::amazon_associates_id
                && $main::amazon_associates_id ne ''
        ) ? $main::amazon_associates_id : $env::amazon_associates_id
        ) . '" /></td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="fb_appid">Facebook Platform アプリケーションID</label></td>';
    print
        '<td class="field"><input type="text" name="fb_appid" id="fb_appid" size="30" value="'
        . $this->escape( ( defined $main::fb_appid && $main::fb_appid ne '' )
        ? $main::fb_appid
        : $env::fb_appid )
        . '" /></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="og_type">サイトのタイプ(og:type)</label></td>';
    print '<td class="field">';
    require CMONOS::CGI_Lib::OGPTypeMenu;
    print OGPTypeMenu->new->menu(
        'og_type',
        (   ( defined $main::og_type && $main::og_type ne '' )
            ? $main::og_type
            : $env::og_type
        )
    );
    print '</td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="image">サイトのサムネイル画像(og:image)</label></td>';
    print '<td class="field"><input type="file" name="image" id="image" />';
    my $og_image = $this->image_file_whole_path('../../og');

    if ($og_image) {
        unless ( ref $cgi::Image ) {
            require CMONOS::CGI_Lib::Image;
            $cgi::Image = Image->new;
        }
        print '<div class="sample-image">';
        print $cgi::Image->tag( $og_image, 0,
            '現在のサイトのサムネイル画像(og:image)',
            '', 200, 200 );
        print
            '<div class="caption">現在のサイトのサムネイル画像(og:image)</div></div>';
    }
    print '</td>';
    print '</tr>';
    print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">検索サイトへの登録拒否</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="noindex" value="1"'
        . ( ( $main::noindex || ( !defined $main::noindex && $env::noindex ) )
        ? ' checked="checked"'
        : '' )
        . ' /> 検索サイトの検索結果にこのサイトを表示することを拒否する</label></td>';
    print '</tr>';
    print
        '<td class="field-name"><span class="datatitle">ジェネレータ</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="no_generator" value="1"'
        . (
        (   $main::no_generator
                || ( !defined $main::no_generator && $env::no_generator )
        ) ? ' checked="checked"' : ''
        )
        . ' /> ジェネレータ(&lt;meta name="generator"&gt;)をメタ情報に含めない</label></td>';
    print '</tr>';
    print '<tr>';
    print '</tbody></table></fieldset>';
    print '<fieldset class="submit">';
    print '<label><input type="checkbox" name="log" value="1"'
        . ( ($main::log) ? ' checked="checked"' : '' )
        . ' /> ログを表示する</label> '
        if !$install;
    print '<input type="submit" value="'
        . ( ($install)
        ? 'この管理情報で CMONOS.JP をインストールする'
        : '管理情報を変更する' )
        . '" /></fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');

    $this->refresh_title_and_description( @title_and_description, $og_image )
        if @title_and_description;
    exit if $install;
}

sub blur_text ($$) {
    my ( $this, $text ) = @_;
    return '' if !$text;
    $text =~ s/^(.{10}).*$/$1.../;
    return $text . '(後半部分を省略してあります)';
}

sub rename_navi_file($$) {
    my ( $this, $inline_navi_flag ) = @_;
    return if !-d '../top' || !-f '../top/start.cgi';
    my ( $top_page_path, $index_path, %db );
    $top_page_path = '../../top.' . ( ($env::ssi) ? $env::ssi_ext : 'html' );
    $index_path = '../../index.' . ( ($env::ssi) ? $env::ssi_ext : 'html' );
    eval 'use ' . $env::dbm_file . ';';
    if ( defined $inline_navi_flag && $inline_navi_flag ) {
        if ( -f $top_page_path ) {
            rename( '../../index.html', '../../frame.html' );
            rename( $top_page_path,     $index_path );
            $this->change_default_setting( '../top/default.pl', 'home',
                '\'../../index.\' . (($env::ssi) ? $env::ssi_ext : \'html\')'
            );
            $this->change_default_setting( '../navimaker/default.pl',
                'frame_path', '\'../../frame.html\'' );
        }
        tie(%db, $env::dbm_file,
            $default::preferences . '/navimaker/navi_db',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %db ) {
            $db{'top_uri'} = $this->encode_utf8(
                '../../index.' . ( ($env::ssi) ? $env::ssi_ext : 'html' ) );
        }
        untie %db;
    }
    else {
        if ( !-f $top_page_path ) {
            rename( $index_path,        $top_page_path );
            rename( '../../frame.html', '../../index.html' );
            $this->change_default_setting( '../top/default.pl', 'home',
                '\'../../top.\' . (($env::ssi) ? $env::ssi_ext : \'html\')' );
            $this->change_default_setting( '../navimaker/default.pl',
                'frame_path', '\'../../index.html\'' );
        }
        tie(%db, $env::dbm_file,
            $default::preferences . '/navimaker/navi_db',
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
        if ( ref tied %db ) {
            $db{'top_uri'} = $this->encode_utf8(
                '../../top.' . ( ($env::ssi) ? $env::ssi_ext : 'html' ) );
        }
        untie %db;
    }
}

sub change_default_setting ($$$$) {
    my ( $this, $path, $name, $value ) = @_;
    local (*FILE);
    my $data = $this->read_file( $path, 1 );
    $data =~ s/^[\x20\x09]*\$$name[\x20\x09\x0A]*=[^;]*;/\$$name = $value;/gm;
    $this->write_file( $path, \$data );
}

######################################################################
#タイトルと説明のリフレッシュ
sub refresh_title_and_description ($$$$$$$$$$$$$$$) {
    my ($this,             $old_title,   $old_description,
        $old_format,       $new_title,   $new_description,
        $new_format,       $new_doctype, $old_header,
        $new_header,       $new_noindex, $new_fb_appid,
        $new_no_generator, $new_ogimage, $ogimage
    ) = @_;
    my ( @cgi, @html_files, $count, $total, $dialog, $refresh_log );
    local (*DIR);
    return 0 unless opendir( DIR, '../' );
    @cgi
        = grep( $_ !~ /^\./ && -d '../' . $_ && -f '../' . $_ . '/default.pl',
        readdir(DIR) );
    closedir(DIR);
    foreach my $dir (@cgi) {
        my ( $setting, $lib, $path, $subdivided, $default_pl );
        $default_pl = '../' . $dir . '/default.pl';
        $setting = $this->read_file( $default_pl, 1 );
        if (   $setting
            && $setting
            =~ /^[\x20\x09]*\$lib[\x20\x09\x0A]*=[\x20\x09]*["']?([^"']+?)["']?;/m
            )
        {
            $lib = $1;
        }
        else {
            next;
        }
        if ( $setting
            =~ /^[\x20\x09]*\$subdivided[\x20\x09\x0A]*=[\x20\x09]*["']?([^"']+?)["']?;/m
            )
        {
            $subdivided = $1;
        }
        if ( $lib eq 'HomePageMaker' ) {
            $path = $this->un_base_change( eval($1), $default_pl )
                if $setting =~ /^[\x20\x09]*\$home[\x20\x09\x0A]*=(.*?);/m;
            if ( opendir( DIR, $path ) ) {
                my @articles;
                while ( my $file = readdir(DIR) ) {
                    next if $file =~ /^\./;
                    if ( -d $path . '/' . $file ) {
                        push( @articles, $path . '/' . $file );
                    }
                    elsif ( -f $path . '/' . $file
                        && $file =~ /\.(s?html?|ssi)$/i )
                    {
                        push( @html_files, $path . '/' . $file );
                    }
                }
                closedir(DIR);
                foreach my $dir (@articles) {
                    my @articles2;
                    if ( opendir( DIR, $dir ) ) {
                        while ( my $file = readdir(DIR) ) {
                            next if $file =~ /^\./;
                            if ($subdivided) {
                                if ( -d $dir . '/' . $file ) {
                                    push( @articles2, $dir . '/' . $file );
                                }
                                elsif ( -f $dir . '/' . $file
                                    && $file =~ /\.(s?html?|ssi)$/i )
                                {
                                    push( @html_files, $dir . '/' . $file );
                                }
                            }
                            elsif ( $file =~ /^([0-9]+|index)\.s?html?$/i
                                && -f $dir . '/' . $file )
                            {
                                push( @html_files, $dir . '/' . $file );
                            }
                        }
                        closedir(DIR);
                        if (@articles2) {
                            foreach my $dir2 (@articles2) {
                                if ( opendir( DIR, $dir2 ) ) {
                                    push(
                                        @html_files,
                                        map( $dir2 . '/' . $_,
                                            grep( $_
                                                    =~ /^([0-9]+|index)\.s?html?$/i
                                                    && -f $dir2 . '/' . $_,
                                                readdir(DIR) ) )
                                    );
                                    closedir(DIR);
                                }
                            }
                        }
                    }
                }
            }
        }
        elsif ( $lib eq 'HomePageMakerLite' ) {
            push( @html_files,
                $this->un_base_change( eval($1), $default_pl ) )
                if $setting =~ /^[\x20\x09]*\$home[\x20\x09\x0A]*=(.*?);/m;
        }
        elsif ( $lib eq 'Baobab' ) {
            $path = $this->un_base_change( eval($1), $default_pl )
                if $setting =~ /^[\x20\x09]*\$file[\x20\x09\x0A]*=(.*?);/m;
            if ($subdivided) {
                if ( opendir( DIR, $path ) ) {
                    push(
                        @html_files,
                        map( $path . '/' . $_ . '/index.html',
                            grep( $_ !~ /^\./
                                    && -d $path . '/' . $_
                                    && -f $path . '/' . $_ . '/index.html',
                                readdir(DIR) ) )
                    );
                    closedir(DIR);
                }
            }
            else {
                push( @html_files, $path . '/index.html' );
            }
        }
        else {
            next;
        }
    }
    push( @html_files, '../order/files/payment_info.shtml' )
        if -f '../order/files/payment_info.shtml';
    push( @html_files, '../order/files/payment_info.html' )
        if -f '../order/files/payment_info.html';
    push( @html_files, $env::externalfiles . '/metaindex.ssi' )
        if $env::externalfiles . '/metaindex.ssi';
    $total = @html_files + 3;
    if (@html_files) {
        my ( $site_title, $format, $metadata_hash, $ogdir );
        if (   ( defined $new_title && $new_title ne '' )
            || defined $new_fb_appid
            || $new_ogimage )
        {
            $metadata_hash = {};
            $metadata_hash->{'app_id'} = $new_fb_appid
                if defined $new_fb_appid;
            $metadata_hash->{'locale'} = $env::locale || 'ja_JP';
            $metadata_hash->{'site_name'} = $new_title
                if defined $new_title && $new_title ne '';
            $ogdir = $ogimage = $this->full_path($ogimage);
            $ogdir =~ s/\.(?:gif|jpg|png)$/./i;
        }
        if ( ( defined $new_title && $new_title ne '' ) || $new_format ) {
            $site_title
                = ( defined $new_title && $new_title ne '' )
                ? $new_title
                : $old_title;
            $format = {};
            (   $format->{'old'}->{'site_title_separator'},
                $format->{'old'}->{'category_separator'},
                $format->{'old'}->{'reverse'}
            ) = split( /,/, $old_format, 3 );
            $format->{'site_title_separator'}
                = ( defined $format->{'old'}->{'site_title_separator'} )
                ? quotemeta( $format->{'old'}->{'site_title_separator'} )
                : '';
            $format->{'category_separator'}
                = ( defined $format->{'old'}->{'category_separator'} )
                ? quotemeta( $format->{'old'}->{'category_separator'} )
                : '';
            if ($new_format) {
                (   $format->{'new'}->{'site_title_separator'},
                    $format->{'new'}->{'category_separator'},
                    $format->{'new'}->{'reverse'}
                ) = split( /,/, $new_format, 3 );
                $dialog
                    .= '<li>ウィンドウタイトルをカテゴリーを省略するフォーマットからカテゴリーを省略しないフォーマットへ変更しました。静的書き出しページでは、記事を更新しない限りカテゴリーが省略されたままになりますのでご注意ください。</li>'
                    if !$format->{'old'}->{'category_separator'}
                    && $format->{'new'}->{'category_separator'};
            }
        }
        $new_header .= "\x0A" if $new_header && $old_header;
        foreach my $path (@html_files) {
            $count++;
            my $changed = 0;
            my $html = $this->read_file( $path, 1 );
            if ($new_doctype) {
                $changed = 1
                    if $html =~ s/<!DOCTYPE\s[^>]*>/$new_doctype/is
                    || $html =~ s/^/$new_doctype/s;
            }
            if ( defined $new_header ) {
                if ($old_header) {
                    $changed = 1
                        if $html
                        =~ s/\Q$old_header\E\x0A?(?=.*<title(?!\w)[^>]*>.*<\/head>)/$new_header/is;
                }
                else {
                    $changed = 1
                        if $html
                        =~ s/([\x09\x0A\x0C\x0D\x20]*<title(?!\w)[^>]*>)(?=.*<\/head>)/\x0A$new_header$1/is;
                }
            }
            $changed = 1
                if ref $format
                && $html
                =~ s/(<title(?![0-9a-zA-Z_])[^>]*>)(.*?)(<\/title>)/$1 . $this->refresh_window_title($2,$old_title,$site_title,$format) . $3/iges;
            if ( defined $new_title && $new_title ne '' ) {
                my $old = quotemeta $old_title;
                $changed = 1
                    if $html
                    =~ s/<\!--(sitetitle)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/sitetitle(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$new_title,1)/iges;
                $changed = 1
                    if $html
                    =~ s/(<\!--topicpath(?![0-9a-zA-Z_])[^>]*-->.*?<!--data-->.*?<a(?![0-9a-zA-Z_])[^>]*>).*?(<\/a>)/$1$new_title$2/igs;
                $changed = 1
                    if $html
                    =~ s/(<link rel="home"[^>]* title=")$old("[^>]*>)/$1$new_title$2/igs;
                $changed = 1
                    if $html
                    =~ s/(<link rel="alternate" type="application\/rss\+xml"[^>]* title=")$old( - [^>"]*"[^>]*>)/$1$new_title$2/igs;
            }
            if ( defined $new_description ) {
                $changed = 1
                    if $html
                    =~ s/<\!--(sitedescription)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/sitedescription(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$new_description,1)/iges;
                if ($old_description) {
                    my $old = quotemeta $old_description;
                    $changed = 1
                        if $html
                        =~ s/(>[\x09\x0A\x0C\x0D\x20]*| title="| alt=")$old_description([\x09\x0A\x0C\x0D\x20]*<|")/$1$new_description$2/;
                }
            }
            if ( defined $new_noindex && $new_noindex ne '' ) {
                if ($new_noindex) {
                    $changed = 1
                        if $html
                        !~ /<meta[\x09\x0A\x0C\x0D\x20]+name="robots"[\x09\x0A\x0C\x0D\x20]+content="noindex,[\x09\x0A\x0C\x0D\x20]*nofollow"[\x09\x0A\x0C\x0D\x20]*\/?>[\x09\x0A\x0C\x0D\x20]*/i
                        && ( $html
                        =~ s/(<meta[\x09\x0A\x0C\x0D\x20]+http-equiv="Content-Style-Type"[\x09\x0A\x0C\x0D\x20]+content="text\/css"[\x09\x0A\x0C\x0D\x20]*\/?>[\x09\x0A\x0C\x0D\x20]*)/$1<meta name="robots" content="noindex,nofollow" \/>\x0A/is
                        || $html
                        =~ s/([\x09\x0A\x0C\x0D\x20]*<\/head>)/\x0A<meta name="robots" content="noindex,nofollow" \/>$1/is
                        );
                }
                else {
                    $changed = 1
                        if $html
                        =~ s/<meta[\x09\x0A\x0C\x0D\x20]+name="robots"[\x09\x0A\x0C\x0D\x20]+content="noindex,[\x09\x0A\x0C\x0D\x20]*nofollow"[\x09\x0A\x0C\x0D\x20]*\/?>[\x09\x0A\x0C\x0D\x20]*//gsi;
                }
            }
            if ( defined $new_no_generator && $new_no_generator ne '' ) {
                if ($new_no_generator) {
                    $changed = 1
                        if $html
                        =~ s/<meta name="generator" content="http:\/\/cmonos.jp\/" \/>[\x09\x0A\x0C\x0D\x20]*//gsi;
                }
                else {
                    $changed = 1
                        if $html
                        !~ /<meta name="generator" content="http:\/\/cmonos.jp\/" \/>[\x09\x0A\x0C\x0D\x20]*/i
                        && ( $html
                        =~ s/(<meta[\x09\x0A\x0C\x0D\x20]+http-equiv="Content-Style-Type"[\x09\x0A\x0C\x0D\x20]+content="text\/css"[\x09\x0A\x0C\x0D\x20]*\/?>[\x09\x0A\x0C\x0D\x20]*)/$1<meta name="generator" content="http:\/\/cmonos.jp\/" \/>\x0A/is
                        || $html
                        =~ s/([\x09\x0A\x0C\x0D\x20]*<\/head>)/\x0A<meta name="generator" content="http:\/\/cmonos.jp\/" \/>$1/is
                        );
                }
            }
            if (   ref $metadata_hash
                && $path =~ /\.s?html?$/i
                && $html =~ /<html[^>]*>/
                && $html =~ /<head[^>]*>/ )
            {
                $metadata_hash->{'image'}
                    = $this->get_metadata_by_name( \$html, 'og:image' );
                $metadata_hash->{'image'} = $ogimage
                    if !$metadata_hash->{'image'}
                    || $metadata_hash->{'image'}
                    =~ /^\Q$ogdir\E(?:gif|jpg|png)$/;
                $metadata_hash->{'type'}
                    = $this->get_metadata_by_name( \$html, 'og:type' );
                $metadata_hash->{'title'}
                    = $this->get_metadata_by_name( \$html, 'og:title' )
                    || $this->get_metadata_by_name( \$html, 'title' );
                $metadata_hash->{'description'}
                    = $this->get_metadata_by_name( \$html, 'og:description' )
                    || $this->get_metadata_by_name( \$html, 'description' );
                $this->replace_meta_data( \$html, $metadata_hash );
                $changed = 1;
            }
            if ($changed) {
                $this->write_file( $path, \$html, 1 );
                $this->mk_external_javascript($path) if $path =~ /\.ssi$/;
                $refresh_log
                    .= '<li>'
                    . $this->path_change( $path, $ENV{'SCRIPT_NAME'}, 'abs' )
                    . '</li>'
                    if $main::log;
                $refresh_log = ''
                    if $cgi::ProgressBar->update(
                    $count, $total, 20,
                    (   ($refresh_log)
                        ? 'document.getElementById(\'refresh-log\').innerHTML += \''
                            . $refresh_log . '\';'
                        : ''
                    )
                    );
            }
        }
    }
    if ( ( defined $new_title && $new_title ne '' )
        || defined $new_description )
    {
        my $navi_modified = 0;
        foreach my $path (
            $env::externalfiles . '/site-navi.ssi',
            $env::externalfiles . '/site-navi-sjis.ssi',
            $env::externalfiles . '/site-navi-euc.ssi'
            )
        {
            $count++;
            my $changed = 0;
            my $html = $this->read_file( $path, 1 );
            my $sitetitle
                = ( defined $old_title && $old_title ne '' )
                ? $old_title
                : '';
            my $sitedescription
                = ( defined $old_description && $old_description ne '' )
                ? $old_description
                : '<!--sitedescription--><!--/sitedescription-->';
            my $title
                = ( defined $new_title && $new_title ne '' )
                ? $new_title
                : '';
            my $description
                = ( defined $new_description && $new_description ne '' )
                ? $new_description
                : '<!--sitedescription--><!--/sitedescription-->';
            $sitetitle       = quotemeta $sitetitle;
            $sitedescription = quotemeta $sitedescription;
            $changed         = 1
                if $sitetitle ne ''
                && $title ne ''
                && $html
                =~ s/(>[\x09\x0A\x0C\x0D\x20]*| title="| alt=")$sitetitle([\x09\x0A\x0C\x0D\x20]*<|")/$1$title$2/;
            $changed = 1
                if defined $new_description
                && $sitedescription ne ''
                && $description ne ''
                && $html
                =~ s/(>[\x09\x0A\x0C\x0D\x20]*| title="| alt=")$sitedescription([\x09\x0A\x0C\x0D\x20]*<|")/$1$description$2/;

            if ( $path =~ /-(sjis|euc)\.ssi$/ ) {
                $this->jcode_convert( \$html, $1, 'utf8' );
            }
            if ($changed) {
                $navi_modified = 1;
                $this->write_file( $path, \$html, 1 );
                $this->mk_external_javascript($path);
                $refresh_log
                    .= '<li>'
                    . $this->path_change( $path, $ENV{'SCRIPT_NAME'}, 'abs' )
                    . '</li>'
                    if $main::log;
                $refresh_log = ''
                    if $cgi::ProgressBar->update(
                    $count, $total, 20,
                    (   ($refresh_log)
                        ? 'document.getElementById(\'refresh-log\').innerHTML += \''
                            . $refresh_log . '\';'
                        : ''
                    )
                    );
            }
        }
        if ($navi_modified) {
            $dialog
                .= '<li>ナビゲーションを更新しました。サイトタイトルなどが期待通りに更新されていない場合は、ナビメーカーの「ナビゲーション書き出し」を実行してください。なお、掲示板「バオバブ」のデフォルトページは、新しい書き込みがあるまでナビゲーション部分が更新されませんのでご注意ください。</li>';
        }
        else {
            $dialog
                .= '<li>ナビゲーションを更新できませんでした。サイトタイトルの変更などをナビゲーションに反映させるため、ナビメーカーの「ナビゲーション書き出し」を実行してください。なお、掲示板「バオバブ」のデフォルトページは、新しい書き込みがあるまでナビゲーション部分が更新されませんのでご注意ください。</li>';
        }
    }
    $dialog
        .= '<li>サイトのウィンドウタイトルを更新しました。</li>'
        if ( defined $new_title && $new_title ne '' ) || $new_format;
    $dialog
        .= '<li>サイトの検索サイト登録設定を更新しました。</li>'
        if defined $new_noindex && $new_noindex ne '';
    $dialog
        .= '<li>サイトの Open Graph メタ情報を更新しました。</li>'
        if ( defined $new_title && $new_title ne '' )
        || defined $new_fb_appid
        || $new_ogimage;
    $dialog
        .= '<li>サイトのジェネレータ情報を更新しました。</li>'
        if defined $new_no_generator && $new_no_generator ne '';
    $dialog .= '<li>サイトの DOCTYPE 宣言を更新しました。</li>'
        if $new_doctype;
    $dialog
        .= '<li>サイトの共通 HTML ヘッダを更新しました。</li>'
        if defined $new_header;
    $cgi::ProgressBar->update(
        $count, $total, 0,
        (   ($refresh_log)
            ? 'document.getElementById(\'refresh-log\').innerHTML += \''
                . $refresh_log . '\';'
            : ''
            )
            . 'document.getElementById(\'cgi-dialog-list\').innerHTML += \''
            . ( ($dialog) ? $dialog : '<li>エラー</li>' ) . '\';'
    );
    $cgi::ProgressBar->remove;
}

sub refresh_window_title ($$$$) {
    my ( $this, $title, $old_title, $site_title, $format ) = @_;
    $old_title = quotemeta $old_title;
    if ( ref $format->{'new'} ) {
        my @title_path = ();
        if ( $format->{'old'}->{'reverse'} ) {
            $title =~ s/$format->{'site_title_separator'}$old_title$//s
                if $format->{'site_title_separator'} ne '';
            @title_path
                = ( $format->{'category_separator'} ne '' )
                ? reverse(
                split( /$format->{'category_separator'}/, $title ) )
                : ($title);
        }
        else {
            $title =~ s/^$old_title$format->{'site_title_separator'}//s
                if $format->{'site_title_separator'} ne '';
            @title_path
                = ( $format->{'category_separator'} ne '' )
                ? split( /$format->{'category_separator'}/, $title )
                : ($title);
        }
        @title_path
            = grep( defined $_ && $_ ne '' && $_ =~ /[^\x09\x0A\x0C\x0D\x20]/,
            @title_path );
        return ( ( defined $site_title ) ? $site_title : '' ) if !@title_path;
        if ( defined $format->{'new'}->{'category_separator'}
            && $format->{'new'}->{'category_separator'} ne '' )
        {
            @title_path = reverse @title_path
                if $format->{'new'}->{'reverse'};
            $title = join( $format->{'new'}->{'category_separator'},
                @title_path );
        }
        else {
            $title = $title_path[$#title_path];
        }
        if ( defined $site_title && $site_title ne '' ) {
            if ( $format->{'new'}->{'reverse'} ) {
                $title .= $format->{'new'}->{'site_title_separator'}
                    . $site_title;
            }
            else {
                $title
                    = $site_title
                    . $format->{'new'}->{'site_title_separator'}
                    . $title;
            }
        }
    }
    elsif ( $format->{'site_title_separator'} ne '' ) {
        if ( $format->{'old'}->{'reverse'} ) {
            $title
                =~ s/$format->{'site_title_separator'}$old_title$/$format->{'old'}->{'site_title_separator'}$site_title/s;
        }
        else {
            $title
                =~ s/^$old_title$format->{'site_title_separator'}/$site_title$format->{'old'}->{'site_title_separator'}/s;
        }
    }
    return $this->escape_except_html_entity($title);
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$env::demo ) {

        #クラス登録
        Common::class_register( '00s0', 'MasterInfo' );

        #メニュー登録
        $setting::jump_menu{'00s0'} = '管理情報変更';
    }
}

1;
