package SiteChmod;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# パーミッション最適化モジュール
# SiteChmod.pm ver1.0.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-02-14
######################################################################
# CGI がユーザー権限で動いていない場合の CGI が読み書きするファイルのパーミッション
$SiteChmod::rw_mode = 666;

# CGI がユーザー権限で動いていない場合の CGI が読み書きするディレクトリのパーミッション
$SiteChmod::rwx_mode = 777;
######################################################################
# 実行
sub run ($) {
    my $this = shift;

    #パーミッション調整
    $this->site_chmod( $main::data{'s'}, $main::data{'u'}, $main::data{'p'} )
        if defined $main::a && $main::a eq 'opt';

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>パーミッション最適化</h1>';

    #フォーム開始
    print $this->form_start(
        'key'    => 'tools',
        'name'   => 'input',
        'method' => 'post',
        'onsubmit' =>
            'return confirm(\'この操作は取り消せません。本当に最適化しますか？\') &amp;&amp; cmonos.form.item(this).isNotSubmitted();',
        'required' => \@setting::form
    );

    if ( !$env::ownerpower ) {
        eval 'chomp($main::data{"s"} = `/bin/hostname`)'
            if !defined $main::data{'s'} || $main::data{'s'} eq '';
        print '<fieldset>';
        print
            '<div class="hints">FTP コマンドにより、このサイトのファイルおよびディレクトリのアクセス権限（パーミッション）を、CGI が動作するよう最適化します。</div>';
        print '<table class="field-table"><tbody>';
        print
            '<tr><td class="field-name"><label for="data:s">FTP サーバ</label></td><td class="field"><input type="text" size="40" name="data:s" id="data:s" value="'
            . $this->escape( $main::data{'s'} )
            . '" /></td></tr>';
        print
            '<tr><td class="field-name"><label for="data:u">ユーザー名</label></td><td class="field"><input type="text" size="10" name="data:u" id="data:u" value="'
            . $this->escape( $main::data{'u'} )
            . '" /></td></tr>';
        print
            '<tr><td class="field-name"><label for="data:p">パスワード</label></td><td class="field"><input type="password" size="10" name="data:p" id="data:p" /></td></tr>';
        print
            '<tr><td class="field-name"><span class="datatitle">最適化オプション</span></td><td class="field">';
        print
            '<label><input type="checkbox" name="option" id="option" value="1"'
            . ( ($main::option) ? ' checked="checked"' : '' )
            . ' /> CGI 所有のファイルあるいはディレクトリを無視しない</label>';
        print '</td></tr></tbody></table></fieldset>';
    }
    print '<fieldset class="submit">';
    print '<label><input type="checkbox" name="log" value="1"'
        . ( ($main::log) ? ' checked="checked"' : '' )
        . ' /> パーミッション最適化ログを表示する</label> ';
    print
        '<input type="submit" name="a=opt" value="パーミッションを最適化する" />';
    print '</fieldset></form>';

    if ( $this->{'chmod_done'} && $main::log ) {
        print
            '<fieldset><legend>パーミッション最適化ログ</legend>';
        print( ( $this->{'chmod_log'} )
            ? '<ol>' . $this->{'chmod_log'} . '</ol>'
            : '<div class="hints">パーミッションを最適化するべきファイルあるいはディレクトリが見つかりません。</div>'
        );
        print '</fieldset>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# パーミッション最適化
sub site_chmod ($$$$) {
    my ( $this, $hostname, $username, $passwd ) = @_;
    my (@rw_files,      @rwx_dirs,         @directories,
        @files,         @data_directories, @data_files,
        @cgi_files,     $dir_mode,         $file_mode,
        $cgi_file_mode, @rwx_dir
    );
    local (*DIR);
    $hostname = $default::hostname if !defined $hostname || $hostname eq '';
    $username = $default::username if !defined $username || $username eq '';
    $passwd   = $default::passwd   if !defined $passwd   || $passwd eq '';

    if ( !defined $env::ownerpower || !$env::ownerpower ) {
        if ( !defined $hostname || $hostname eq '' ) {
            push(
                @{ $this->{'dialog'} },
                'FTPサーバを入力して下さい。'
            );
            return 0;
        }
        if ( !defined $username || $username eq '' ) {
            push(
                @{ $this->{'dialog'} },
                'ユーザー名を入力して下さい。'
            );
            return 0;
        }
        if ( !defined $passwd || $passwd eq '' ) {
            push(
                @{ $this->{'dialog'} },
                'パスワードを入力して下さい。'
            );
            return 0;
        }
    }

    $dir_mode      = sprintf( "%o", $env::dir_mode );
    $file_mode     = sprintf( "%o", $env::file_mode );
    $cgi_file_mode = sprintf( "%o", $env::cgi_file_mode );

    @rw_files    = ();
    @rwx_dirs    = ();
    @cgi_files   = ();
    @files       = ();
    @directories = ();

    my $document_root = $this->document_root;
    $default::htdocs_ftp = $1
        if $default::htdocs_ftp eq '' && $document_root =~ /([^\/]+)$/;

    if ($env::ownerpower) {

        #関連データディレクトリのスキャン
        $this->mod_scan( \@rw_files, 600, \@cgi_files,
            600, \@files, \@rwx_dirs, 700, \@directories,
            $default::cgi_data, $default::cgi_data, 1 )
            if defined $default::cgi_data && $default::cgi_data;
        push( @data_directories, @directories );
        push( @data_files,       @files );
        push( @data_files,       @cgi_files );
        undef @directories;
        undef @files;
        undef @cgi_files;
        @cgi_files   = ();
        @files       = ();
        @directories = ();

    }
    else {

        #初期設定ディレクトリのスキャン
        $this->mod_scan(
            \@rw_files,            $cgi_file_mode,
            \@cgi_files,           $file_mode,
            \@files,               \@rwx_dirs,
            $dir_mode,             \@directories,
            $default::preferences, $default::preferences_ftp,
            1
        ) if $default::preferences && $default::preferences_ftp;

        #データディレクトリのスキャン
        $this->mod_scan(
            \@rw_files,         $cgi_file_mode, \@cgi_files,
            $file_mode,         \@files,        \@rwx_dirs,
            $dir_mode,          \@directories,  $default::data,
            $default::data_ftp, 1
        ) if $default::data && $default::data_ftp;

        #郵便番号ＤＢ
        $this->mod_scan(
            \@rw_files,            $cgi_file_mode, \@cgi_files,
            $file_mode,            \@files,        \@rwx_dirs,
            $dir_mode,             \@directories,  $env::zipcode,
            $default::zipcode_ftp, 1
        ) if $env::zipcode && $default::zipcode_ftp;

    }

    #CGI関連ディレクトリの抽出
    if ( opendir( DIR, '../' ) ) {
        my ( @cgi, $cgi_path, @files_dir, @tools );
        @cgi
            = grep( $_ !~ /\./
                && -d '../' . $_
                && -f '../' . $_ . '/default.pl',
            readdir(DIR) );
        closedir(DIR);
        @tools = (
            'accesscounter', 'accestat', 'gatekeeper', 'navimaker',
            'pswd',          'setup',    'station',    'top',
            'yahoosearch',   'zipcode'
        );
        $cgi_path = $this->path_change( '../', $ENV{'SCRIPT_NAME'}, 'abs' );
        foreach my $cgi (@cgi) {
            if ( grep( $_ eq $cgi, @tools ) ) {
                next;
            }
            elsif ( -d '../' . $cgi . '/files' ) {
                push(
                    @files_dir,
                    $default::htdocs_ftp
                        . $this->path_change(
                        '../' . $cgi . '/files', $ENV{'SCRIPT_NAME'},
                        'abs'
                        )
                );
            }
            else {
                my $default
                    = $this->read_file( '../' . $cgi . '/default.pl', 1 );
                my $lib
                    = ( $default
                        =~ /\$lib[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*'([0-9a-zA-Z_]+)';/s
                    ) ? $1 : '';
                if ( $lib eq 'HomePageMaker' ) {
                    if ( $default
                        =~ /\$home[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*(.*?);/s
                        )
                    {
                        push( @rwx_dir, eval($1) );
                    }
                }
                elsif ( $lib eq 'HomePageMakerLite' ) {
                    if ( $default
                        =~ /\$home[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*(.*?);/s
                        )
                    {
                        $this->add_rw_path( \@rw_files, $file_mode, \@files,
                            eval($1) );
                    }
                    if ( $default
                        =~ /\$file[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*(.*?);/s
                        )
                    {
                        push( @rwx_dir, eval($1) );
                    }
                }
            }
        }
        $this->mod_scan(
            \@rw_files,
            $cgi_file_mode,
            \@cgi_files,
            $file_mode,
            \@files,
            \@rwx_dirs,
            $dir_mode,
            \@directories,

  #		$document_root . $cgi_path,$default::htdocs_ftp . $cgi_path,0,\@files_dir
            $document_root, $default::htdocs_ftp, 0, \@files_dir
        );
    }
    $this->add_rw_path( \@rw_files, $file_mode, \@files,
        '../../index.' . ( ($env::ssi) ? $env::ssi_ext : 'html' ) );
    $this->add_rw_path( \@rw_files, $file_mode, \@files,
        '../../top.' . ( ($env::ssi) ? $env::ssi_ext : 'html' ) );
    $this->add_rw_path( \@rw_files, $file_mode, \@files, '../../navi.html' );
    $this->add_rw_path( \@rw_files, $file_mode, \@files, '../../frame.html' );

#画像ディレクトリ/CSSディレクトリ/外部ファイルディレクトリ
    push( @rwx_dir, $env::image_dir, $env::css, $env::externalfiles );

    #書き換え可能ディレクトリのスキャン
    if (@rwx_dir) {
        foreach my $dir (@rwx_dir) {
            $dir = $this->path_change( $dir, $ENV{'SCRIPT_NAME'}, 'abs' );
            $this->mod_scan(
                \@rw_files,            $cgi_file_mode,
                \@cgi_files,           $file_mode,
                \@files,               \@rwx_dirs,
                $dir_mode,             \@directories,
                $document_root . $dir, $default::htdocs_ftp . $dir,
                1
            );
        }
    }

    my $count = 0;

    #ユーザー権限で動作している場合
    if ( defined $env::ownerpower && $env::ownerpower ) {
        if (@files) {
            my @temp = ();
            foreach my $p (@files) {
                if ( $p =~ /\.dat$/ ) {
                    push( @data_files, $p );
                }
                else {
                    push( @temp, $p );
                }
            }
            @files = @temp;
        }
        $count += $this->cgi_site_chmod( \@directories, $env::dir_mode );
        $count += $this->cgi_site_chmod( \@files,       $env::file_mode );
        $count += $this->cgi_site_chmod( \@cgi_files,   $env::cgi_file_mode );
        $count += $this->cgi_site_chmod( \@data_directories, 0700 );
        $count += $this->cgi_site_chmod( \@data_files,       0600 );

        #FTP接続
    }
    else {
        local (*FTP);
        my ($code);
        require CMONOS::CGI_Lib::TcpClient;
        if (!TcpClient->new(
                'handle'   => \*FTP,
                'host'     => $hostname,
                'portname' => 'ftp',
            )->tcp_connect
            )
        {
            push(
                @{ $this->{'dialog'} },
                '接続先が FTP 接続を受け付けません。'
            );
            return 0;
        }
        if ( ( $code = $this->status ) != 220 ) {
            push(
                @{ $this->{'dialog'} },
                '接続先が新しいユーザーを受け付けません。('
                    . $code . ')'
            );
            print FTP 'QUIT', "\x0D\x0A";
            return 0;
        }

        print FTP 'USER ', $username, "\x0D\x0A";
        if ( ( $code = $this->status ) != 331 ) {
            push(
                @{ $this->{'dialog'} },
                'ユーザー名を確認してください。('
                    . $code . ')'
            );
            print FTP 'QUIT', "\x0D\x0A";
            return 0;
        }

        print FTP 'PASS ', $passwd, "\x0D\x0A";
        if ( ( $code = $this->status ) != 230 ) {
            push(
                @{ $this->{'dialog'} },
                'パスワードを確認してください。('
                    . $code . ')'
            );
            print FTP 'QUIT', "\x0D\x0A";
            return 0;
        }

        #SITE chmod
        $count += $this->ftp_site_chmod( \@rw_files, $SiteChmod::rw_mode );
        $count += $this->ftp_site_chmod( \@rwx_dirs, $SiteChmod::rwx_mode );
        $count += $this->ftp_site_chmod( \@directories, $dir_mode );
        $count += $this->ftp_site_chmod( \@files,       $file_mode );
        $count += $this->ftp_site_chmod( \@cgi_files,   $cgi_file_mode );

        print FTP 'QUIT', "\x0D\x0A";
        close(FTP);
    }
    $this->{'chmod_done'} = 1;
    push(
        @{ $this->{'dialog'} },
        $count
            . ' のファイルまたはディレクトリのパーミッションを最適化しました。'
    );
    return $count;
}

sub add_rw_path ($$$$$) {
    my ( $this, $rw_files, $file_mode, $files, $path ) = @_;
    return if !-f $path;
    my $fpath = $default::htdocs_ftp
        . $this->path_change( $path, $ENV{'SCRIPT_NAME'}, 'abs' );
    my @stat = stat($path);
    my $mod = sprintf( "%o", $stat[2] );
    $mod = $1 if $mod =~ /([0-9]{3})$/;
    if ($env::ownerpower) {
        push( @$files, $path ) if $mod ne $file_mode;
    }
    else {
        push( @$rw_files, $fpath ) if $mod ne $SiteChmod::rw_mode;
    }
}

sub mod_scan ($$$$$$$$$$$;$$) {
    my ($this,        $rw_files, $cgi_file_mode, $cgi_files,
        $file_mode,   $files,    $rwx_files,     $dir_mode,
        $directories, $dir,      $ftp_path,      $rw,
        $rwx_dir
    ) = @_;

    return unless opendir( DIR, $dir );
    my @file = readdir(DIR);
    closedir(DIR);

    my ($name);
    foreach $name (@file) {
        next if $name =~ /^[\.\/]+$/;
        my ( $path, $fpath, @stat, $mod );
        $path  = $dir . '/' . $name;
        $fpath = $ftp_path . '/' . $name;
        @stat  = stat($path);

        $mod = sprintf( "%o", $stat[2] );
        $mod = $1 if $mod =~ /([0-9]{3})$/;
        if ( -f $path ) {
            if ( $env::ownerpower && $name =~ /\.dat$/ && $mod eq '600' ) {
                next;
            }
            elsif (!$env::ownerpower
                && !$main::option
                && ( $stat[4] eq $< || $stat[4] eq $> ) )
            {
                next;
            }
            elsif (
                !$env::ownerpower
                && ( $rw
                    || ( ref $rwx_dir && grep( $_ eq $fpath, @$rwx_dir ) ) )
                )
            {
                push( @$rw_files, $fpath ) if $mod ne $SiteChmod::rw_mode;
            }
            elsif ( $name =~ /\.(pl|cgi)$/i ) {
                push( @$cgi_files, ( ($env::ownerpower) ? $path : $fpath ) )
                    if $mod ne $cgi_file_mode;
            }
            elsif ( $mod ne $file_mode ) {
                push( @$files, ( ($env::ownerpower) ? $path : $fpath ) );
            }
        }
        elsif ( -d $path ) {
            if (!$env::ownerpower
                && ( $rw
                    || ( ref $rwx_dir && grep( $_ eq $fpath, @$rwx_dir ) ) )
                )
            {
                push( @$rwx_files, $fpath )
                    if $mod ne $SiteChmod::rwx_mode
                    && ( $main::option
                    || ( $stat[4] ne $< && $stat[4] ne $> ) );
                $this->mod_scan(
                    $rw_files, $cgi_file_mode, $cgi_files, $file_mode,
                    $files,    $rwx_files,     $dir_mode,  $directories,
                    $path,     $fpath,         1
                );
            }
            else {
                push( @$directories, ( ($env::ownerpower) ? $path : $fpath ) )
                    if $mod ne $dir_mode
                    && ( $env::ownerpower
                    || $main::option
                    || ( $stat[4] ne $< && $stat[4] ne $> ) );
                $this->mod_scan(
                    $rw_files, $cgi_file_mode, $cgi_files, $file_mode,
                    $files,    $rwx_files,     $dir_mode,  $directories,
                    $path,     $fpath,         $rw
                );
            }
        }
    }
}

sub ftp_site_chmod ($$$) {
    my ( $this, $list, $mod ) = @_;
    my $count = 0;
    my $file;
    foreach $file (@$list) {
        print FTP 'SITE chmod ', $mod, ' ', $file, "\x0D\x0A";
        my $status = $this->status;
        $this->{'chmod_log'}
            .= '<li>CHMOD '
            . $mod . ' '
            . $file
            . ' =&gt; '
            . $status . '</li>'
            if defined $main::log && $main::log;
        $count++ if $status == 200;
    }
    return $count;
}

sub status ($$) {
    my $this = shift;
    my $status;
    while ( $status = <FTP> ) {
        return $1 if $status =~ /^([0-9]{3})/;
    }
}

sub cgi_site_chmod ($$$) {
    my ( $this, $list, $mod ) = @_;
    my $count = 0;
    my $file;
    foreach $file (@$list) {
        my ($status);
        if ( chmod( $mod, $file ) ) {
            $status = 'successful';
            $count++;
        }
        else {
            $status = 'failed';
        }
        if ( defined $main::log && $main::log ) {
            $file =~ s/^[\.\/]+//;
            $this->{'chmod_log'}
                .= '<li>CHMOD '
                . sprintf( "%o", $mod ) . ' '
                . $file
                . ' =&gt; '
                . $status . '</li>';
        }
    }
    return $count;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 's300', 'SiteChmod' );

    #メニュー登録
    $setting::jump_menu{'s300'} = 'パーミッション最適化';

}

1;
