package SiteTemplateExporter;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use Fcntl qw(:flock);
use File::Path;
use strict;
use utf8;
######################################################################
# サイトテンプレート書き出しモジュール
# SiteTemplateExporter.pm ver1.0.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-04-18
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( @site_template_list, $site_template_dir );
    $this->get_setup_env;
    unless ( eval 'use Archive::Tar;Archive::Tar->has_zlib_support;' ) {
        $this->jump_menu('tools');
        $this->print_page_header('tools');
        print '<h1>サイトテンプレート書き出し</h1>';
        print
            '<p>Archive::Tar モジュールがみつからないため、この機能は利用できません。「~/cgi-data/template/site」にサイトテンプレートが保存されています。FTP 等でサイトテンプレートフォルダをローカルディスクにダウンロードしてください。</p>';
        $this->print_page_footer('tools');
        return 1;
    }
    $site_template_dir
        = $env::user_site_template_dir || $env::template_dir . '/site';

    #サイトテンプレートスキャン
    local (*DIR);
    if ( opendir( DIR, $site_template_dir ) ) {
        @site_template_list
            = grep( $_ !~ /^\./ && -d $site_template_dir . '/' . $_,
            readdir(DIR) );
        closedir(DIR);
    }

    #作業フォルダ
    my $temp_dir
        = $env::clipboard
        . '/site_template_exporter'
        . ( $env::home_dir_id || '' );
    my $status_file = $temp_dir . '/status';
    my $target_info = $temp_dir . '/target';
    my $pending_time
        = ( !-f $status_file ) ? 0 : time - ( stat($status_file) )[9];
    if ( -d $temp_dir && -f $status_file && $pending_time > 600 ) {
        my ( $t, $n, $l ) = split( /\x09/, $this->read_file($target_info) );
        push(
            @{ $this->{'dialog'} },
            '前回サイトテンプレート「'
                . $this->site_template_title( $n, ( $l || $t ) )
                . '」の圧縮に失敗した可能性があります。作業フォルダを削除しました。サイトテンプレートは「~/cgi-data/template/site」に保存されています。FTP などでサイトテンプレートをフォルダごとダウンロードしてください。'
        );
        rmtree $temp_dir;
    }
    my $status = $this->read_file( $status_file, 1 ) || 0;
    my ( $tempid, $name, $label )
        = split( /\x09/, $this->read_file($target_info) );

    #書き出し処理
    if ( $status == 2 && $tempid ) {
        if ($main::cancel) {
            push(
                @{ $this->{'dialog'} },
                'サイトテンプレート「'
                    . $this->site_template_title(
                    $name, ( $label || $tempid )
                    )
                    . '」の書き出しを中止しました。'
            );
            rmtree $temp_dir;
            $status = 0;
        }
        elsif ( $ENV{'PATH_INFO'} ) {
            my $path = $temp_dir . '/' . $tempid . '.tgz';
            my $size = -s $path;
            binmode STDOUT;
            print "Content-Type: application/x-tar-gz\n";
            print "Content-Length: $size\n";
            print "\n";
            local (*FILE);
            if ( open( FILE, $path ) ) {
                binmode FILE;
                flock( FILE, LOCK_SH );
                my $buf;
                while ( read( FILE, $buf, $env::buffer_size ) ) {
                    print $buf;
                }
                close(FILE);
            }
            rmtree $temp_dir;
            return;
        }
    }
    elsif ($main::target) {
        if (   !@site_template_list
            || !grep( $_ eq $main::target, @site_template_list ) )
        {
            push(
                @{ $this->{'dialog'} },
                'サイトテンプレート書き出しに失敗しました。指定されたサイトテンプレートは存在しません。'
            );
        }
        else {
            my ( $target, $home_dir );
            $tempid = $label = $target = $main::target;
            if ( !$env::user_site_template_dir ) {
                ( $tempid, $home_dir )
                    = $this->get_sitetemplate_id_and_home_dir($target);
                if ($home_dir) {
                    if (   $default::home_dir
                        && $home_dir !~ /^\Q$default::home_dir\E(?:\/|$)/ )
                    {
                        undef $tempid;
                    }
                    else {
                        $label = $this->escape( $tempid . '@~' . $home_dir );
                    }
                }
            }
            if ( !$tempid ) {
                push(
                    @{ $this->{'dialog'} },
                    'サイトテンプレート書き出しに失敗しました。指定されたサイトテンプレートは存在しません。'
                );
            }
            else {
                ($name) = split(
                    /\x09/,
                    $this->read_file(
                        $site_template_dir . '/' . $target . '/name.txt'
                    )
                );
                $name = 'デフォルトのサイトテンプレート'
                    if !$name && $tempid eq 'default';
                my $cwd = $this->get_cwd;
                $this->set_dir($temp_dir);
                $this->write_file( $status_file, 1, 1 );
                $this->write_file( $target_info,
                    $tempid . "\x09" . $name . "\x09" . $label, 1 );
                chdir( $site_template_dir . '/' . $target );
                use File::Find qw(find);
                my $tar = Archive::Tar->new;
                my @files;
                find( sub { push( @files, $File::Find::name ); }, '.' );
                $tar->add_files(@files);
                $tar->write(
                    $this->path_change(
                        $temp_dir . '/' . $tempid . '.tgz',
                        $cwd . '/start.cgi',
                        'abs'
                    ),
                    9
                );
                chdir($cwd);

                if ( -f $temp_dir . '/' . $tempid . '.tgz' ) {
                    $this->write_file( $status_file, 2, 1 );
                    $status = 2;
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        'サイトテンプレート「'
                            . $this->site_template_title( $name,
                            ( $label || $tempid ) )
                            . '」の圧縮に失敗しました。'
                    );
                    rmtree $temp_dir;
                    $status = 0;
                }
            }
        }
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>サイトテンプレート書き出し</h1>';

    if ( $status == 1 ) {
        print '<p>サイトテンプレート「'
            . $this->site_template_title( $name, ( $label || $tempid ) )
            . '」を圧縮しています... (圧縮に失敗している場合、'
            . ( ($pending_time) ? 10 - int( $pending_time / 60 ) : 10 )
            . '分後に作業フォルダが削除されます)</p>';
    }
    elsif ( $status == 2 ) {
        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'input',
            'method'   => 'post',
            'path'     => '/' . $tempid . '.tgz',
            'required' => \@setting::form
        );
        print '<p>サイトテンプレート「'
            . $this->site_template_title( $name, ( $label || $tempid ) )
            . '」の準備が整いました。</p>';
        print
            '<fieldset class="submit" id="export"><input type="submit" value="書き出す" onclick="cmonos.tool.hideSubmitButton()" /></fieldset></form>';
        print $this->javascript(
            'cmonos.tool.hideSubmitButton = function () { document.getElementById("export").style.display = "none"; setTimeout(function () { location.href+="?m=Tools&k=s702"; } ,30000) }'
        );
    }
    elsif (@site_template_list) {
        no warnings qw(numeric);
        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'input',
            'method'   => 'post',
            'required' => \@setting::form
        );
        print '<input type="hidden" name="export" value="1" />';
        print
            '<fieldset><legend>サイトテンプレートを選択</legend><ol class="checkbox">';
        @site_template_list
            = sort { $a <=> $b || $a cmp $b } @site_template_list;
        foreach my $dir (@site_template_list) {
            my ( $tempid, $home_dir, $name );
            $tempid = $dir;
            if ( !$env::user_site_template_dir ) {
                ( $tempid, $home_dir )
                    = $this->get_sitetemplate_id_and_home_dir($dir);
                if ($home_dir) {
                    next
                        if $default::home_dir
                        && $home_dir !~ /^\Q$default::home_dir\E(?:\/|$)/;
                    $tempid = $this->escape( $tempid . '@~' . $home_dir );
                }
            }
            ($name) = split(
                /\x09/,
                $this->read_file(
                    $site_template_dir . '/' . $dir . '/name.txt'
                )
            );
            $name = 'デフォルトのサイトテンプレート'
                if !$name && $dir eq 'default';
            $name = '' if !defined $name;
            print '<li><label><input type="radio" name="target" value="'
                . $dir . '" /> '
                . $this->site_template_title( $name, $tempid )
                . '</lable></li>';
        }
        print '</ol></fieldset>';
        print
            '<fieldset class="submit"><input type="submit" value="書き出す" /></fieldset></form>';
    }
    else {
        print
            '<fieldset><div class="hints">書き出し可能なサイトテンプレートが存在しません。</div></fieldset>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 's702', 'SiteTemplateExporter' );

    #メニュー登録
    $setting::jump_menu{'s702'} = 'サイトテンプレート書き出し';

}

1;
