package CartViewer;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# カート内容表示モジュール
# CartViewer.pm ver1.0.6
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2017-08-17
######################################################################
sub unauthorized_alert ($) {
    my $this = shift;
    local @setting::form
        = ( 'iframe_disabled', 'quantity', 'cart', 'cancel_all' );
    if ( $this->{'access_forbidden'} ) {
        $this->error403(1);
    }
    else {
        $this->print_password_check_form($setting::key);
    }
}

######################################################################
# メイン
sub run ($) {
    my $this = shift;
    my ( %item_data, @item_order, $item_id, $this_icon, $url, $referer_url,
        $discount, $discount_ratio );
    $this->import_db( $default::preference_dir . '/terms',
        \%setting::payment, 1 )
        if !%setting::payment;
    require CMONOS::CGI_Lib::CartController;
    $cgi::CartController = CartController->new;
    if ( $ENV{'HTTP_REFERER'} ) {
        my $here = $this->my_url;
        $referer_url = $ENV{'HTTP_REFERER'}
            if $ENV{'HTTP_REFERER'} !~ /^\Q$here\E/;
    }
    $referer_url = $cgi::CartController->get_url_of_last_item
        if !$referer_url;
    if ($main::cancel_all) {
        $cgi::CartController->cancel_all;
    }
    elsif (%main::cart) {
        $cgi::CartController->add( \%main::cart, \%item_data, \@item_order );
    }
    elsif (%main::quantity) {
        $cgi::CartController->change( \%main::quantity, \%item_data,
            \@item_order );
    }
    else {
        $cgi::CartController->check( '', \%item_data, \@item_order );
    }
    $setting::page{$setting::key}{'bodyId'}
        = 'document' . ( ($default::id) ? $default::id : '' ) . '_cart';
    if (@item_order) {
        my $json  = '';
        my $total = 0;
        foreach my $item_id (@item_order) {
            my $price = (
                ( $item_data{$item_id}{'optionprice'} )
                ? ( $item_data{$item_id}{'price'}
                        + $item_data{$item_id}{'optionprice'} )
                : $item_data{$item_id}{'price'}
            );
            $json .= ",\x0A" if $json;
            $json
                .= '		{"price":'
                . $price
                . ',"quantity":'
                . $item_data{$item_id}{'quantity'}
                . ',"quantity_id":"quantity:'
                . $item_id
                . '","discountable":'
                . (
                (         !$setting::payment{'discountlimited'}
                        || $item_data{$item_id}{'discountable'}
                ) ? 'true' : 'false'
                )
                . ',"price_id":"sc-item-'
                . $item_id
                . '-price","unit_price_id":"sc-item-'
                . $item_id
                . '-unit-price"}';
            $total += $price * $item_data{$item_id}{'quantity'}
                if !$setting::payment{'discountlimited'}
                || $item_data{$item_id}{'discountable'};
        }
        $json = 'cmonos.tool.shoppingcart.data = {"item":[' . "\x0A"
            . $json . "\x0A";
        $json
            .= '	],"sc_sub_total_id": "sc-sub-total-value","sc_tax_id": "sc-tax-value","sc_total_id": "sc-total-value","tax_rate": '
            . ( ( defined $env::tax_rate ) ? ( $env::tax_rate || 0 ) : 0.08 )
            . ',"tax_frac":"'
            . ( ($env::tax_frac) ? $env::tax_frac : 'floor' ) . '"';
        if (   %setting::payment
            && ref $setting::payment{'discount'}
            && %{ $setting::payment{'discount'} } )
        {
            my $j = '';
            $discount = 0;
            foreach my $n ( sort { $b <=> $a || $b cmp $a }
                keys %{ $setting::payment{'discount'} } )
            {
                if (   $setting::payment{'discount'}{$n} > 0
                    && $setting::payment{'discount'}{$n} < 100 )
                {
                    $j
                        .= ( ($j) ? ',' : '' )
                        . '{"bound":'
                        . $n
                        . ',"ratio":'
                        . $setting::payment{'discount'}{$n} . '}';
                    if (  !$discount
                        && $n <= $total
                        && $setting::payment{'discount'}{$n} )
                    {
                        $discount = $setting::payment{'discount'}{$n};
                    }
                }
            }
            $discount_ratio = 1 - ( $discount / 100 );
            $json
                .= ',discount: ['
                . $j
                . '],rounddown:'
                . $setting::payment{'discountrounddown'}
                if $j;
        }
        $json .= "\x0A}\x0A";
        $json .= 'cmonos.tool.shoppingcart.ajaxDisabled = true' . "\x0A"
            if $setting::page{$setting::key}{'ajaxdisabled'};
        $json .= 'cmonos.tool.shoppingcart.iframeDisabled = true' . "\x0A"
            if $main::iframe_disabled;
        $this->set_javascript($json);
    }
    else {
        $this->set_onload(
            'cmonos.tool.shoppingcart.window.sizeCheck(640,480)');
        $this->print_page_header($setting::key);
        print '<p id="sc-empty-notice">'
            . ( $setting::page{'emptyCartMsg'}
                || 'カートに何も入っていません。' )
            . '</p>';
        print $this->close_button(
            $referer_url,
            $setting::page{$setting::key}{'closeButton'},
            $setting::page{$setting::key}{'backButton'},
            0, 1
        );
        $this->print_page_footer($setting::key);
        return;
    }
    require CMONOS::CGI_Lib::Deco;
    $this_icon
        = Deco->new( $env::image_dir . '/order', $env::image_lib . '/order' );

    #ページヘッダ
    $this->set_onload('cmonos.tool.shoppingcart.window.sizeCheck(640,480);');
    $this->print_page_header($setting::key);

    #フォーム
    print $this->form_start(
        'key'        => 'checkout',
        'name'       => 'cart',
        'method'     => 'post',
        'novalidate' => 'novalidate',
        'onsubmit'   => 'return cmonos.form.item(this).isNotSubmitted()',
        'required'   => \@setting::form
    );

    my $noshipping_only = 1;
    my $for_iframe = ( $setting::key eq 'iframe' ) ? 1 : 0;
    my ( $total, $count );
    if ($for_iframe) {
        print '<table id="sc-table-compact">';
        print '<tbody id="sc-item-tbody">';
    }
    else {
        print '<table id="sc-table">';
        print '<thead>';
        print '<tr class="odd">';
        print '<th class="sc-item-name">'
            . ( $setting::general{'itemname'} || '商品' ) . '</th>';
        print '<th class="sc-item-option">オプション</th>';
        print '<th class="sc-item-unit-price">単価</th>';
        print '<th class="sc-item-quantity">数量</th>';
        print '<th class="sc-item-price">価格</th>';
        print '<th class="sc-item-tool">操作</th>';
        print '</tr>';
        print '</thead>';
        print '<tbody id="sc-item-tbody">';
    }
    foreach my $item_id (@item_order) {
        $count++;
        my $price = (
            ( $item_data{$item_id}{'optionprice'} )
            ? ( $item_data{$item_id}{'price'}
                    + $item_data{$item_id}{'optionprice'} )
            : $item_data{$item_id}{'price'}
        );
        my $price_str = '&yen;' . $this->digit($price);
        if ($discount
            && (  !$setting::payment{'discountlimited'}
                || $item_data{$item_id}{'discountable'} )
            )
        {
            $price = int( $price * $discount_ratio );
            $price_str
                = '<del>'
                . $price_str
                . '</del><ins>(-'
                . $this->round_down( $discount,
                $setting::payment{'discountrounddown'}, 'floor' )
                . '%)</ins> &yen;'
                . $this->digit($price);
        }
        if ($for_iframe) {
            print '<tr class="'
                . ( ( $count % 2 ) ? 'odd' : 'even' )
                . '" id="sc-item-'
                . $item_id . '">';
            print '<td>';
            print '<div class="sc-item-name"><a href="'
                . $this->escape(
                (          $item_data{$item_id}{'referer'}
                        || $item_data{$item_id}{'path'}
                )
                . ( ( $item_data{$item_id}{'anchor'} )
                    ? $item_data{$item_id}{'anchor'}
                    : ''
                )
                )
                . '" target="_blank">'
                . $item_data{$item_id}{'name'}
                . (
                (          $item_data{$item_id}{'key'}
                        && $item_data{$item_id}{'key'} ne
                        $item_data{$item_id}{'name'}
                )
                ? '<span class="sc-item-key"> ['
                    . $this->detag_text( $item_data{$item_id}{'key'} )
                    . ']</span>'
                : ''
                )
                . '</a>'
                . ( ( $item_data{$item_id}{'digitalcontent'} )
                ? '<span class="sc-item-filename"> ('
                    . $this->escape( $item_data{$item_id}{'digitalcontent'} )
                    . ')</span>'
                : '' )
                . '</div>';
            print '<div class="sc-item-option">'
                . $this->escape( $item_data{$item_id}{'option'} )
                . '</div>'
                if $item_data{$item_id}{'option'};
            print '<div class="sc-item-info">';
            print '<span class="sc-item-unit-price" id="sc-item-'
                . $item_id
                . '-unit-price">'
                . $price_str
                . '</span>';
        }
        else {
            $url
                = (    $item_data{$item_id}{'referer'}
                    || $item_data{$item_id}{'path'} )
                . ( ( $item_data{$item_id}{'anchor'} )
                ? $item_data{$item_id}{'anchor'}
                : '' );
            print '<tr class="'
                . ( ( $count % 2 ) ? 'odd' : 'even' )
                . '" id="sc-item-'
                . $item_id . '">';
            print '<td class="sc-item-name"><a href="'
                . $this->escape($url)
                . '" target="_blank">'
                . $item_data{$item_id}{'name'}
                . (
                (          $item_data{$item_id}{'key'}
                        && $item_data{$item_id}{'key'} ne
                        $item_data{$item_id}{'name'}
                )
                ? '<span class="sc-item-key"> ['
                    . $this->detag_text( $item_data{$item_id}{'key'} )
                    . ']</span>'
                : ''
                )
                . '</a>'
                . ( ( $item_data{$item_id}{'digitalcontent'} )
                ? '<span class="sc-item-filename"> ('
                    . $this->escape( $item_data{$item_id}{'digitalcontent'} )
                    . ')</span>'
                : '' )
                . '</td>';
            print '<td class="sc-item-option">'
                . $this->escape( $item_data{$item_id}{'option'} ) . '</td>';
            print '<td class="sc-item-unit-price" id="sc-item-'
                . $item_id
                . '-unit-price">'
                . $price_str . '</td>';
        }
        if ($item_data{$item_id}{'digitalcontent'}
            || (   $item_data{$item_id}{'max'}
                && $item_data{$item_id}{'max'} == 1 )
            )
        {
            $item_data{$item_id}{'quantity'} = 1;
            if ($for_iframe) {
                print
                    '<span class="sc-item-quantity">&times;<input type="hidden" id="quantity:'
                    . $item_id
                    . '" name="quantity:'
                    . $item_id
                    . '" value="1" />1</span>';
            }
            else {
                print
                    '<td class="sc-item-quantity"><input type="hidden" id="quantity:'
                    . $item_id
                    . '" name="quantity:'
                    . $item_id
                    . '" value="1" />1</td>';
            }
        }
        else {
            if ($for_iframe) {
                print
                    '<span class="sc-item-quantity">&times;<input type="number" size="3" min="0"'
                    . ( ( $item_data{$item_id}{'max'} )
                    ? ' max="' . $item_data{$item_id}{'max'} . '"'
                    : '' )
                    . ' id="quantity:'
                    . $item_id
                    . '" name="quantity:'
                    . $item_id
                    . '" value="'
                    . ( ( defined $item_data{$item_id}{'quantity'} )
                    ? $item_data{$item_id}{'quantity'}
                    : '' )
                    . '" onchange="'
                    . ( ( $setting::page{$setting::key}{'ajaxdisabled'} )
                    ? 'cmonos.tool.shoppingcart.recalculate()'
                    : 'cmonos.tool.shoppingcart.changeItem(\''
                        . $item_id
                        . '\',this.value)' )
                    . '" /></span>';
            }
            else {
                print
                    '<td class="sc-item-quantity"><input type="number" size="3" min="0"'
                    . ( ( $item_data{$item_id}{'max'} )
                    ? ' max="' . $item_data{$item_id}{'max'} . '"'
                    : '' )
                    . ' id="quantity:'
                    . $item_id
                    . '" name="quantity:'
                    . $item_id
                    . '" value="'
                    . ( ( defined $item_data{$item_id}{'quantity'} )
                    ? $item_data{$item_id}{'quantity'}
                    : '' )
                    . '" onchange="'
                    . ( ( $setting::page{$setting::key}{'ajaxdisabled'} )
                    ? 'cmonos.tool.shoppingcart.recalculate()'
                    : 'cmonos.tool.shoppingcart.changeItem(\''
                        . $item_id
                        . '\',this.value)' )
                    . '" /></td>';
            }
            $noshipping_only = 0 if !$item_data{$item_id}{'noshipping'};
        }
        $price
            = ( ( $price && $item_data{$item_id}{'quantity'} )
            ? $price * $item_data{$item_id}{'quantity'}
            : 0 );
        if ($for_iframe) {
            print '=<span class="sc-item-price" id="sc-item-'
                . $item_id
                . '-price">&yen;'
                . $this->digit($price)
                . '</span>';
            print '<span class="sc-item-tool">';
            print $this_icon->button(
                'trash',
                $setting::page{$setting::key}{'trashButton'},
                '#', 'X', '',
                {   'onclick' => 'cmonos.tool.shoppingcart.deleteItem(this,\''
                        . $item_id
                        . '\');return false;'
                }
            );
            print '</span>';
            print '</div>';
            print '</td>';
            print '</tr>';
        }
        else {
            print '<td class="sc-item-price" id="sc-item-'
                . $item_id
                . '-price">&yen;'
                . $this->digit($price) . '</td>';
            print '<td class="sc-item-tool">';
            print $this_icon->button(
                'trash',
                $setting::page{$setting::key}{'trashButton'},
                '#', 'X', '',
                {   'onclick' => 'cmonos.tool.shoppingcart.deleteItem(this,\''
                        . $item_id
                        . '\');return false;'
                }
            );
            print '</td>';
            print '</tr>';
        }
        $total += $price;
    }
    if ($for_iframe) {
        print '</tbody>';
        print '<tbody id="sc-total-tbody">';
        print '<tr>';
        print '<td>';
        print '<div class="sc-tool">';
        print $this_icon->button(
            'empty',
            $setting::page{$setting::key}{'emptyButton'},
            './start.cgi?m=CartViewer&cancel_all=1',
            'カートを空にする'
        );
        print '</div>';
    }
    else {
        print '</tbody>';
        print '<tbody id="sc-total-tbody">';
    }
    if ($env::tax_rate) {
        if ($for_iframe) {
            print
                '<div id="sc-sub-total">小計:<span id="sc-sub-total-value">&yen;'
                . $this->digit($total)
                . '</span></div>';
        }
        else {
            print '<tr id="sc-sub-total" class="sc-total-start">';
            print '<td class="sc-name" colspan="4">小計</td>';
            print '<td class="sc-value" id="sc-sub-total-value">&yen;'
                . $this->digit($total) . '</td>';
            print '<td class="sc-tool empty"></td>';
            print '</tr>';
        }
        my $tax
            = ( $env::tax_frac && $env::tax_frac eq 'ceil' )
            ? int( ( $total * $env::tax_rate ) + 0.99999 )
            : ( $env::tax_frac && $env::tax_frac eq 'round' )
            ? int( ( $total * $env::tax_rate ) + 0.5 )
            : int( $total * $env::tax_rate );
        if ($for_iframe) {
            print '<div id="sc-tax">消費税:<span id="sc-tax-value">&yen;'
                . $this->digit($tax)
                . '</span></div>';
        }
        else {
            print '<tr id="sc-tax">';
            print '<td class="sc-name" colspan="4">消費税</td>';
            print '<td class="sc-value" id="sc-tax-value">&yen;'
                . $this->digit($tax) . '</td>';
            print '<td class="sc-tool empty"></td>';
            print '</tr>';
            print '<tr id="sc-total">';
        }
        $total += $tax;
    }
    elsif ( !$for_iframe ) {
        print '<tr id="sc-total" class="sc-total-start">';
    }
    if ($for_iframe) {
        print '<div id="sc-total">合計'
            . $this->carriage_and_fee_msg($noshipping_only)
            . ':<span id="sc-total-value">&yen;'
            . $this->digit($total)
            . '</span></div>';
        print '</td>';
        print '</tr>';
        print '</tbody>';
        print '</table>';
    }
    else {
        print '<td class="sc-name" colspan="4">合計'
            . $this->carriage_and_fee_msg($noshipping_only) . '</td>';
        print '<td class="sc-value" id="sc-total-value">&yen;'
            . $this->digit($total) . '</td>';
        print '<td class="sc-tool">';
        print $this_icon->button(
            'empty',
            $setting::page{$setting::key}{'emptyButton'},
            './start.cgi?m=CartViewer&cancel_all=1&iframe_disabled=1',
            (   ( $setting::page{$setting::key}{'emptyButtonName'} )
                ? $setting::page{$setting::key}{'emptyButtonName'}
                : 'カートを空にする'
            )
        );
        print '</td>';
        print '</tr>';
        print '</tbody>';
        print '</table>';
    }
    print '<div id="sc-update-box">';
    if ( !$noshipping_only ) {
        print '<div id="sc-update-sub-total">';
        print $this_icon->submit_button( 'sum',
            $setting::page{$setting::key}{'sumButton'},
            '再計算', '', { 'm' => 'CartViewer' } );
        print '</div>';
    }
    print '<div id="sc-proceed-to-checkout">';
    print $this_icon->submit_button(
        'checkout',
        $setting::page{$setting::key}{'checkoutButton'},
        (   ( $setting::page{$setting::key}{'checkoutButtonName'} )
            ? $setting::page{$setting::key}{'checkoutButtonName'}
            : 'レジに進む'
        ),
        {   'onclick' =>
                'return cmonos.tool.shoppingcart.setTarget(this.form)'
        },
        { 'm' => 'CheckOut' }
    );
    print '</div>';
    print $this->close_button(
        ( $referer_url || $url ),
        $setting::page{$setting::key}{'closeButton'},
        $setting::page{$setting::key}{'backButton'}
    );
    print '</div>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer($setting::key);
}

sub carriage_and_fee_msg ($$) {
    my ( $this, $noshipping_only ) = @_;
    my %payment_terms = ();
    $this->import_db( $default::preference_dir . '/terms',
        \%payment_terms, 1 );
    my $carriage = (
        (          $payment_terms{'carriage'}
                && $payment_terms{'carriage'} eq 'disabled'
        )
            || $noshipping_only
    ) ? 0 : 1;
    my $fee
        = ( !$payment_terms{'otherenabled'}
            && ( !$payment_terms{'codenabled'} || $noshipping_only ) )
        ? 0
        : 1;
    return (
        ( $carriage || $fee )
        ? '(<a href="'
            . $default::file
            . '/payment_info.'
            . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' )
            . '" target="_blank">'
            . (
              ( $carriage && $fee ) ? '手数料・送料'
            : ($fee) ? '手数料'
            :          '送料'
            )
            . '</a>別)'
        : ''
    );
}

# ウィンドウを閉じる
sub close_button ($;$$$$$) {
    my ( $this, $url, $close_image, $return_image, $closebutton_only,
        $normal )
        = @_;
    $this->use_deco();
    my $is_xhtml = (
        ( $default::doctype && $default::doctype =~ /XHTML/ )
            || ( !$default::doctype
            && $env::doctype
            && $env::doctype =~ /XHTML/ )
    ) ? 1 : 0;
    my $close_button
        = ( ($normal)
        ? '<div id="close-button">'
        : '<div id="sc-cartviewer-close-button">' );
    $close_button
        .= '<script type="text/javascript" language="JavaScript">' . "\x0A";
    $close_button .= '<!--' . ( ($is_xhtml) ? ' <![CDATA[' : '' ) . "\x0A";
    $close_button .= 'if (parent && parent != self) {' . "\x0A";
    $close_button
        .= '	if (parent.document.getElementById(\'shoppingcart\')) document.write(\''
        . $cgi::Deco->button(
        'close',
        $close_image,
        '#',
        ( ($default::english) ? 'Close.' : '閉じる' ),
        '',
        {   'onclick' =>
                'cmonos.tool.shoppingcart.window.close();return false;'
        }
        ) . '\');' . "\x0A";
    $close_button .= '} else {' . "\x0A";
    $close_button .= '	if (window.opener) {' . "\x0A";
    $close_button .= '		document.write(\''
        . $cgi::Deco->button( 'close', $close_image, '#',
        ( ($default::english) ? 'Close.' : '閉じる' ),
        '', { 'onclick' => 'window.close();return false;' } )
        . '\');' . "\x0A";
    $close_button .= '	} else {' . "\x0A";

    if ($url) {
        $close_button .= '		document.write(\''
            . $cgi::Deco->button( 'return', $return_image, $url,
            ( ($default::english) ? 'Return.' : '戻る' ) )
            . '\');' . "\x0A";
    }
    else {
        $close_button .= '		document.write(\''
            . $cgi::Deco->button(
            'return', $return_image,
            '#', ( ($default::english) ? 'Return.' : '戻る' ),
            '', { 'onclick' => 'history.back();return false;' }
            ) . '\');' . "\x0A";
    }
    $close_button .= '	}' . "\x0A";
    $close_button .= '}' . "\x0A";
    $close_button .= '// ' . ( ($is_xhtml) ? ']]> ' : '' ) . '-->' . "\x0A";
    $close_button .= '</script>' . "\x0A";
    $close_button
        .= '<noscript><a href="'
        . $this->escape($url) . '">'
        . ( ($default::english) ? 'Return.' : '戻る' )
        . '</a></noscript>' . "\x0A"
        if $url;
    $close_button .= '</div>';
    return $close_button;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ( $default::type && $default::type eq 'order' ) {
        no warnings qw(once);

        #設定DB読み込み設定
        $setting::general  = 1;    #一般設定読み込み
        $setting::deco     = 1;    #テキスト装飾設定読み込み
        $setting::security = 1;    #アクセス権設定読み込み
        $setting::table    = 1;    #テーブル設定読み込み
        $setting::page     = 1;    #ページ設定読み込み

        $setting::field_db_name = '';    #フィールド設定DB
        $setting::field         = 0
            ; #フィールド設定読み込み（2=オリジナルも読み込む）

        #設定種キー設定
        $setting::security_key = 'checkout'
            if !defined $setting::security_key
            || !$setting::security_key
            ;    #デフォルトセキュリティキー

        #クッキー
        $cookie::id_enabled = 1;    #ID情報クッキー読み込み

        #必須フォーム
        @setting::form = ('iframe_disabled');

        #イメージCSS
        $setting::image_css = 'order-image.css';

        #設定種キー設定
        $setting::key = (
            $env::checkout_window_type
                && ( $env::checkout_window_type eq 'iframe'
                || $env::checkout_window_type eq 'modal' )
                && !$main::iframe_disabled
        ) ? 'iframe' : 'cartviewer';    #ページ設定キー

        #デフォルトキー
        $main::k = 'cartviewer' if !defined $main::k || !$main::k;

        #クラス登録
        Common::class_register( 'cartviewer', 'CartViewer' );
    }
}

1;
