package PayPal;
use strict;
use utf8;
use Jcode;
######################################################################
# PayPal.pm ver.1.0.3
# CMONOS::TableMaker::CheckOutAPI::PayPal.pm ver1.0
# Copyright (C) 2009-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-04-11
######################################################################
# オブジェクトを作る。
sub new ($$$$;$$) {
    my ( $pkg, $username, $password, $signature, $currency_id, $sandbox,
        $ajax )
        = @_;
    my $this = {
        'Username'   => $username,
        'Password'   => $password,
        'Signature'  => $signature,
        'currencyID' => ( ($currency_id) ? $currency_id : 'JPY' ),
        'Version'    => '112.0',
        'sandbox'    => ( ($sandbox) ? 1 : 0 ),
        'ajax'       => ( ($ajax) ? 1 : 0 ),
    };
    $this->{'end_point'}
        = ($sandbox)
        ? 'https://api-3t.sandbox.paypal.com/2.0/'
        : 'https://api-3t.paypal.com/2.0/';
    $this->{'redirect_url'}
        = ( ($sandbox)
        ? 'https://www.sandbox.paypal.com'
        : 'https://www.paypal.com' )
        . '/cgi-bin/webscr?cmd=_express-checkout'
        . (
        (          $ENV{'HTTP_USER_AGENT'}
                && $ENV{'HTTP_USER_AGENT'} !~ m!https?://!i
                && $ENV{'HTTP_USER_AGENT'}
                =~ /(?:iPhone|iPod|Android|DoCoMo|KDDI|UP\.Browser|J-PHONE|Vodafone|SoftBank|MCONV)/
        ) ? '-mobile' : ''
        ) . '&token=';
    bless $this;
    return $this;
}

sub request ($$) {
    my ( $this, $specific_api_name, $request_xml ) = @_;
    my $xml = '<?xml version="1.0" encoding="UTF-8"?>' . "\x0A";
    $xml
        .= '<SOAP-ENV:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">'
        . "\x0A";
    $xml .= '<SOAP-ENV:Header>' . "\x0A";
    $xml .= '<RequesterCredentials xmlns="urn:ebay:api:PayPalAPI">' . "\x0A";
    $xml .= '<Credentials xmlns="urn:ebay:apis:eBLBaseComponents">' . "\x0A";
    $xml .= '<Username>' . $this->{'Username'} . '</Username>' . "\x0A";
    $xml .= '<Password>' . $this->{'Password'} . '</Password>' . "\x0A";
    $xml .= '<Signature>' . $this->{'Signature'} . '</Signature>' . "\x0A";
    $xml .= '</Credentials>' . "\x0A";
    $xml .= '</RequesterCredentials>' . "\x0A";
    $xml .= '</SOAP-ENV:Header>' . "\x0A";
    $xml .= '<SOAP-ENV:Body>' . "\x0A";
    $xml
        .= '<'
        . $specific_api_name
        . 'Req xmlns="urn:ebay:api:PayPalAPI">' . "\x0A";
    $xml .= '<' . $specific_api_name . 'Request>' . "\x0A";
    $xml
        .= '<Version xmlns=urn:ebay:apis:eBLBaseComponents">'
        . $this->{'Version'}
        . '</Version>' . "\x0A";
    $xml .= $request_xml if $request_xml;
    $xml .= '</' . $specific_api_name . 'Request>' . "\x0A";
    $xml .= '</' . $specific_api_name . 'Req>' . "\x0A";
    $xml .= '</SOAP-ENV:Body>' . "\x0A";
    $xml .= '</SOAP-ENV:Envelope>' . "\x0A";
    require CMONOS::CGI_Lib::HTTP_Client;
    my $res = HTTP_Client->new->request(
        'POST' => $this->{'end_point'},
        {   'Content-Type' => 'text/xml; charset=utf-8',
            'Accept'       => '*/*',
        },
        $xml,
        'utf8'
    );
    my $response = $res->hash( 'utf8', 1 );

    if ( $this->{'sandbox'} ) {
        my $reqxml = $xml;
        my $resxml = $res->content('utf8');
        if ( !$this->{'ajax'} ) {
            $reqxml = $this->escape($xml);
            $reqxml =~ s/&gt;[\x09\x0A\x0C\x0D\x20]*&lt;/&gt;<br \/>&lt;/gs;
            $resxml = $this->escape($resxml);
            $resxml =~ s/&gt;[\x09\x0A\x0C\x0D\x20]*&lt;/&gt;<br \/>&lt;/gs;
        }
        push( @{ $this->{'errors'} }, { 'LongMessage' => $reqxml } );
        push( @{ $this->{'errors'} }, { 'LongMessage' => $resxml } );
    }
    if ( $res->is_success ) {
        if (   ref $response
            && ref $response->{'SOAP-ENV:Envelope'}
            && ref $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
            && ref $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
            { $specific_api_name . 'Response' } )
        {
            if ( $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
                { $specific_api_name . 'Response' }{'Ack'}
                && $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
                { $specific_api_name . 'Response' }{'Ack'} =~ /Success/ )
            {
                return $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
                    { $specific_api_name . 'Response' };
            }
            else {
                if (ref $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
                    { $specific_api_name . 'Response' }{'Errors'} )
                {
                    push(
                        @{ $this->{'errors'} },
                        (   (   ref $response->{'SOAP-ENV:Envelope'}
                                    {'SOAP-ENV:Body'}
                                    { $specific_api_name . 'Response' }
                                    {'Errors'} eq 'ARRAY'
                            )
                            ? @{$response->{'SOAP-ENV:Envelope'}
                                    {'SOAP-ENV:Body'}
                                    { $specific_api_name . 'Response' }
                                    {'Errors'}
                                }
                            : $response->{'SOAP-ENV:Envelope'}
                                {'SOAP-ENV:Body'}
                                { $specific_api_name . 'Response' }{'Errors'}
                        )
                    );
                }
                else {
                    push(
                        @{ $this->{'errors'} },
                        { 'ShortMessage' => 'Error' }
                    );
                }
                return {};
            }
        }
        else {
            push(
                @{ $this->{'errors'} },
                { 'ShortMessage' => $res->message }
            );
            return {};
        }
    }
    else {
        if (   ref $response
            && ref $response->{'SOAP-ENV:Envelope'}
            && ref $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
            && ref $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
            {'SOAP-ENV:Fault'} )
        {
            push(
                @{ $this->{'errors'} },
                {   'ShortMessage' => (
                        (   $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
                                {'SOAP-ENV:Fault'}{'faultstring'}
                        )
                        ? $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
                            {'SOAP-ENV:Fault'}{'faultstring'}
                        : ''
                    ),
                    'ErrorCode' => (
                        (   $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
                                {'SOAP-ENV:Fault'}{'faultcode'}
                        )
                        ? $response->{'SOAP-ENV:Envelope'}{'SOAP-ENV:Body'}
                            {'SOAP-ENV:Fault'}{'faultcode'}
                        : ''
                    ),
                }
            );
        }
        push( @{ $this->{'errors'} }, { 'ShortMessage' => $res->message } );
        return {};
    }
}

sub field ($$$;$) {
    my ( $this, $name, $type, $data ) = @_;
    $data
        = ( defined $data )
        ? Jcode->new($data)->fallback(&Jcode::FB_XMLCREF)->utf8
        : $this->get($name);
    utf8::decode($data) if !utf8::is_utf8($data);
    my $is_amount = ( $type && $type =~ /:BasicAmountType$/ ) ? 1 : 0;
    return ( defined $data && $data ne '' )
        ? '<'
        . $name
        . (
        ($is_amount) ? ' currencyID="' . $this->{'currencyID'} . '"' : '' )
        . '>'
        . ( ($is_amount) ? sprintf( "%.2f", $data ) : $this->escape($data) )
        . '</'
        . $name . '>' . "\x0A"
        : '';

#return (defined $data && $data ne '') ? '<' . $name . ' xsi:type="' . (($type) ? $type : 'xs:string') . '"' . (($is_amount) ? ' currencyID="' . $this->{'currencyID'} . '"' : '') . '>' . (($is_amount) ? sprintf("%.2f", $data) : $this->escape($data)) . '</' . $name . '>' . "\x0A" : '';
}

sub set_express_checkout ($$) {
    my ( $this, $param ) = @_;
    $this->set_parameter($param);
    my $xml
        .= '<SetExpressCheckoutRequestDetails xmlns="urn:ebay:apis:eBLBaseComponents">'
        . "\x0A";
    $xml .= $this->field('ReturnURL');
    $xml .= $this->field('CancelURL');
    $xml .= $this->field( 'PaymentAction', 'ebl:PaymentActionCodeType' );
    $xml .= $this->field('LocaleCode');
    $xml .= $this->field( 'SolutionType', 'ebl:SolutionTypeType' );
    $xml .= $this->field( 'LandingPage', 'ebl:LandingPageType' );
    $xml .= $this->field('PageStyle');
    $xml .= $this->field('BrandName');
    $xml .= $this->field('AllowNote');
    $xml .= $this->get_payment_details;
    $xml .= $this->field('NoShipping');
    $xml .= '</SetExpressCheckoutRequestDetails>' . "\x0A";
    my $res = $this->request( 'SetExpressCheckout', $xml );

    if ( $res->{'Token'} ) {
        $this->set_parameter( { 'Token'   => $res->{'Token'} } );
        $this->set_parameter( { 'PayerID' => $res->{'PayerID'} } );
        return 1;
    }
    else {
        return 0;
    }
}

sub get_payment_details ($) {
    my $this = shift;
    my $xml  = '<PaymentDetails>' . "\x0A";
    if ( ref $this->{'payment_details'} ) {
        $xml .= $this->field( 'OrderTotal', 'cc:BasicAmountType',
            $this->{'payment_details'}{'OrderTotal'} );
        $xml .= $this->field( 'ItemTotal', 'cc:BasicAmountType',
            $this->{'payment_details'}{'ItemTotal'} );
        $xml .= $this->field( 'TaxTotal', 'cc:BasicAmountType',
            $this->{'payment_details'}{'TaxTotal'} );
        $xml .= $this->field( 'ShippingTotal', 'cc:BasicAmountType',
            $this->{'payment_details'}{'ShippingTotal'} );
        $xml .= $this->field( 'ShippingDiscount', 'cc:BasicAmountType',
            $this->{'payment_details'}{'ShippingDiscount'} );
        $xml .= $this->field( 'HandlingTotal', 'cc:BasicAmountType',
            $this->{'payment_details'}{'HandlingTotal'} );
        if ( ref $this->{'payment_details'}{'items'} ) {
            foreach my $item ( @{ $this->{'payment_details'}{'items'} } ) {
                $xml .= '<PaymentDetailsItem>' . "\x0A";
                $xml .= $this->field( 'Name', '', $item->{'Name'} );
                $xml .= $this->field( 'Description', '',
                    $item->{'Description'} );
                $xml .= $this->field( 'Amount', 'cc:BasicAmountType',
                    $item->{'Amount'} );
                $xml .= $this->field( 'Tax', 'cc:BasicAmountType',
                    $item->{'Tax'} );
                $xml .= $this->field( 'Quantity', 'xs:integer',
                    $item->{'Quantity'} );
                if ( $this->{'payment_details'}{'ItemCategory'} ) {
                    $xml
                        .= $this->field( 'ItemCategory',
                        'ebl:ItemCategoryType',
                        $this->{'payment_details'}{'ItemCategory'} );
                }
                else {
                    $xml .= $this->field( 'ItemCategory',
                        'ebl:ItemCategoryType', $item->{'ItemCategory'} );
                }
                $xml .= '</PaymentDetailsItem>' . "\x0A";
            }
        }
        $this->clear_payment_details;
    }
    else {
        $xml .= $this->field( 'OrderTotal', 'cc:BasicAmountType' );
        $xml .= $this->field('OrderDescription');
    }
    $xml .= '</PaymentDetails>' . "\x0A";
    return $xml;
}

sub set_payment_details ($$) {
    my ( $this, $hash ) = @_;
    $this->{'payment_details'} = {} if !ref $this->{'payment_details'};
    while ( my ( $k, $v ) = each %$hash ) {
        $this->{'payment_details'}{$k} = $v;
    }
}

sub add_payment_details_item ($$) {
    my ( $this, $hash ) = @_;
    $this->{'payment_details'} = {} if !ref $this->{'payment_details'};
    $this->{'payment_details'}{'items'} = []
        if !ref $this->{'payment_details'}{'items'};
    my $n = @{ $this->{'payment_details'}{'items'} };
    $this->{'payment_details'}{'items'}[$n] = {};
    while ( my ( $k, $v ) = each %$hash ) {
        $this->{'payment_details'}{'items'}[$n]{$k} = $v;
    }
}

sub clear_payment_details ($) {
    my $this = shift;
    delete $this->{'payment_details'};
}

sub get_express_checkout_details ($;$) {
    my ( $this, $token ) = @_;
    $this->set_parameter( { 'Token' => $token } ) if $token;
    my $res = $this->request( 'GetExpressCheckoutDetails',
        $this->field('Token') );
    if (   ref $res->{'GetExpressCheckoutDetailsResponseDetails'}
        && $res->{'GetExpressCheckoutDetailsResponseDetails'}{'Token'}
        && $this->get('Token') eq
        $res->{'GetExpressCheckoutDetailsResponseDetails'}{'Token'}
        && ref $res->{'GetExpressCheckoutDetailsResponseDetails'}{'PayerInfo'}
        && $res->{'GetExpressCheckoutDetailsResponseDetails'}{'PayerInfo'}
        {'PayerID'} )
    {
        $this->set_parameter(
            {   'PayerID' =>
                    $res->{'GetExpressCheckoutDetailsResponseDetails'}
                    {'PayerInfo'}{'PayerID'}
            }
        );
        return $res->{'GetExpressCheckoutDetailsResponseDetails'}
            {'PayerInfo'};
    }
    else {
        return 0;
    }
}

sub do_express_checkout_payment ($$) {
    my ( $this, $param ) = @_;
    $this->set_parameter($param);
    my $xml
        .= '<DoExpressCheckoutPaymentRequestDetails xmlns="urn:ebay:apis:eBLBaseComponents">'
        . "\x0A";
    $xml .= $this->field('Token');
    $xml .= $this->field( 'PaymentAction', 'ebl:PaymentActionCodeType' );
    $xml .= $this->field( 'PayerID', 'ebl:UserIDType' );
    $xml .= $this->get_payment_details;
    $xml .= '</DoExpressCheckoutPaymentRequestDetails>' . "\x0A";
    my $res = $this->request( 'DoExpressCheckoutPayment', $xml );

    if (   ref $res->{'DoExpressCheckoutPaymentResponseDetails'}
        && $res->{'DoExpressCheckoutPaymentResponseDetails'}{'Token'}
        && ref $res->{'DoExpressCheckoutPaymentResponseDetails'}
        {'PaymentInfo'}
        && $res->{'DoExpressCheckoutPaymentResponseDetails'}{'PaymentInfo'}
        {'TransactionID'} )
    {
        my $ptime = '';
        $ptime = time
            if $res->{'DoExpressCheckoutPaymentResponseDetails'}
            {'PaymentInfo'}{'PaymentStatus'}
            && $res->{'DoExpressCheckoutPaymentResponseDetails'}
            {'PaymentInfo'}{'PaymentStatus'} eq 'Completed';
        if ( $this->get('PaymentAction') eq 'Sale' && !$ptime ) {
            push(
                @{ $this->{'errors'} },
                {   'ShortMessage' =>
                        '支払いを完了できませんでした。',
                    'LongMessage' => $this->payment_status(
                        $res->{'DoExpressCheckoutPaymentResponseDetails'}
                            {'PaymentInfo'}{'PaymentStatus'}
                        )
                        . $this->pending_reason(
                        $res->{'DoExpressCheckoutPaymentResponseDetails'}
                            {'PaymentInfo'}{'PendingReason'}
                        ),
                    (   (   $res->{'DoExpressCheckoutPaymentResponseDetails'}
                                {'PaymentInfo'}{'ReasonCode'}
                        )
                        ? ( 'ErrorCode' => $res->{
                                'DoExpressCheckoutPaymentResponseDetails'}
                                {'PaymentInfo'}{'ReasonCode'} )
                        : ()
                    )
                }
            );
        }
        return (
            $res->{'DoExpressCheckoutPaymentResponseDetails'}{'PaymentInfo'}
                {'TransactionID'},
            $ptime
        );
    }
    else {
        return ( '', '' );
    }
}

sub do_capture ($;$$$) {
    my ( $this, $authorization_id, $amount, $complete_type ) = @_;
    $this->set_parameter(
        {   (     ($authorization_id)
                ? ( 'AuthorizationID' => $authorization_id )
                : ()
            ),
            ( ($amount)        ? ( 'Amount'       => $amount )        : () ),
            ( ($complete_type) ? ( 'CompleteType' => $complete_type ) : () ),
        }
    );
    $this->{'parameter'}{'CompleteType'} = 'Complete'
        if !$this->{'parameter'}{'CompleteType'};
    my $xml .= $this->field('AuthorizationID');
    $xml .= $this->field( 'Amount',       'cc:BasicAmountType' );
    $xml .= $this->field( 'CompleteType', 'ebl:CompleteCodeType' );
    my $res = $this->request( 'DoCapture', $xml );
    if (   ref $res->{'DoCaptureResponseDetails'}
        && $res->{'DoCaptureResponseDetails'}{'AuthorizationID'}
        && $res->{'DoCaptureResponseDetails'}{'AuthorizationID'} eq
        $this->get('AuthorizationID')
        && ref $res->{'DoCaptureResponseDetails'}{'PaymentInfo'}
        && $res->{'DoCaptureResponseDetails'}{'PaymentInfo'}{'TransactionID'}
        )
    {

        if ($res->{'DoCaptureResponseDetails'}{'PaymentInfo'}{'PaymentStatus'}
            && $res->{'DoCaptureResponseDetails'}{'PaymentInfo'}
            {'PaymentStatus'} eq 'Completed' )
        {
            my $ptime = time;
            return (
                $res->{'DoCaptureResponseDetails'}{'PaymentInfo'}
                    {'TransactionID'},
                $ptime
            );
        }
        else {
            push(
                @{ $this->{'errors'} },
                {   'ShortMessage' =>
                        '支払いを完了できませんでした。',
                    'LongMessage' => $this->payment_status(
                        $res->{'DoCaptureResponseDetails'}{'PaymentInfo'}
                            {'PaymentStatus'}
                        )
                        . $this->pending_reason(
                        $res->{'DoCaptureResponseDetails'}{'PaymentInfo'}
                            {'PendingReason'}
                        ),
                }
            );
            return ();
        }
    }
    else {
        return ();
    }
}

sub do_void ($;$) {
    my ( $this, $authorization_id ) = @_;
    $this->set_parameter( { 'AuthorizationID' => $authorization_id } )
        if $authorization_id;
    my $res = $this->request( 'DoVoid', $this->field('AuthorizationID') );
    if (   $res->{'AuthorizationID'}
        && $res->{'AuthorizationID'} eq $this->get('AuthorizationID') )
    {
        return 1;
    }
    else {
        push(
            @{ $this->{'errors'} },
            {   'ShortMessage' =>
                    '注文の取り消しに失敗しました。'
            }
        );
        return 0;
    }
}

sub refund_transaction_request ($$;$) {
    my ( $this, $transaction_id, $amount ) = @_;
    $this->set_parameter(
        {   'TransactionID' => $transaction_id,
            'RefundType'    => ( ($amount) ? 'Partial' : 'Full' ),
            ( ($amount) ? ( 'Amount' => $amount ) : () ),
        }
    );
    my $xml .= $this->field('TransactionID');
    $xml .= $this->field( 'Amount',     'cc:BasicAmountType' );
    $xml .= $this->field( 'RefundType', 'ebl:CompleteCodeType' );
    my $res = $this->request( 'RefundTransaction', $xml );
    if ( ref $res && $res->{'RefundTransactionID'} ) {
        if ( $res->{'GrossRefundAmount'} ) {
            return ( $res->{'GrossRefundAmount'}, time );
        }
        else {
            push(
                @{ $this->{'errors'} },
                {   'ShortMessage' =>
                        '払い戻しを完了できませんでした。',
                    'LongMessage' =>
                        '払い戻しを完了できませんでした。',
                }
            );
            return ();
        }
    }
    else {
        return ();
    }
}

sub redirect ($) {
    my $this = shift;
    binmode STDOUT, ":utf8";
    if ($env::location_header_disabled) {
        print 'Content-Type: text/html' . "\x0A\x0A";
        print '<html><head><meta http-equiv="Refresh" content="0; URL='
            . $this->escape( $this->{'redirect_url'} . $this->get('Token') )
            . '" /></head><body></body></html>';
    }
    else {
        print 'Location: '
            . $this->{'redirect_url'}
            . $this->get('Token')
            . "\x0A\x0A";
    }
    binmode STDOUT;
    exit;
}

sub get ($) {
    my ( $this, $name ) = @_;
    return ( defined $this->{'parameter'}{$name} )
        ? $this->{'parameter'}{$name}
        : '';
}

sub set_parameter ($$) {
    my ( $this, $param ) = @_;
    while ( my ( $k, $v ) = each %$param ) {
        $this->{'parameter'}{$k} = $v;
    }
}

sub errors ($$) {
    my ( $this, $type ) = @_;
    my @msg;
    $type = 'all' if !$type;
    foreach my $error ( @{ $this->{'errors'} } ) {
        next if !ref $error;
        push( @msg, $error->{'ShortMessage'} )
            if $error->{'ShortMessage'}
            && ( $type eq 'short' || $type eq 'all' );
        push( @msg, $error->{'LongMessage'} )
            if $error->{'LongMessage'}
            && ( $type eq 'short' || $type eq 'all' );
        push(
            @msg,
            $error->{'ErrorCode'}
                . (
                ( $error->{'SeverityCode'} )
                ? ':' . $error->{'SeverityCode'}
                : ''
                )
            )
            if $error->{'ErrorCode'}
            && ( $type eq 'code'
            || ( $this->{'sandbox'} && $type eq 'all' ) );
    }
    delete $this->{'errors'};
    return @msg;
}

sub escape ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/&/&amp;/gs;
    $str =~ s/"/&quot;/g;
    $str =~ s/</&lt;/g;
    $str =~ s/>/&gt;/g;
    return $str;
}

sub payment_status ($$) {
    my ( $this, $code ) = @_;
    return (
          ( !$code || $code eq 'None' ) ? ''
        : ( $code eq 'Canceled-Reversal' )
        ? '支払い取り消しがキャンセル済みです。'
        : ( $code eq 'Completed' ) ? '支払いが完了しました。'
        : ( $code eq 'Denied' )
        ? 'お客様が支払いを拒否しました。'
        : ( $code eq 'Expired' )
        ? 'この支払いの承認期限を過ぎています。 '
        : ( $code eq 'Failed' )  ? '支払いが失敗しました。'
        : ( $code eq 'Pending' ) ? '支払いが未決済です。 '
        : ( $code eq 'Refunded' )
        ? 'お客様が支払いを払い戻しました。'
        : ( $code eq 'Reversed' )  ? '支払いが取り消されました。'
        : ( $code eq 'Processed' ) ? '支払いが受諾されました。'
        : ( $code eq 'Voided' )
        ? 'この取引の承認が無効になっています。'
        : ''
    );
}

sub pending_reason ($$) {
    my ( $this, $code ) = @_;
    return (
          ( !$code || $code eq 'none' ) ? ''
        : ( $code eq 'address' )
        ? '支払いが未決済である理由は、顧客が確認済み配送先住所を持たず、お客様の支払い受領設定で、このような支払いについて手動で受諾または拒否するように設定されているためです。設定を変更するには、[個人設定]に移動してください。'
        : ( $code eq 'echeck' )
        ? '支払いが未決済である理由は、支払いに使用されたeCheckが決済されていないためです。'
        : ( $code eq 'intl' )
        ? '支払いが未決済である理由は、お客様が米国以外に口座を持ち、振り替えることができないからです。お客様のアカウント概要で、手動で受諾または拒否する必要があります。'
        : ( $code eq 'multi-currency' )
        ? '送信した通貨の口座残高がなく、支払い受領設定で、この支払いを自動的に変換して受諾するように設定されていない場合、手動で受諾または拒否する必要があります。'
        : ( $code eq 'verify' )
        ? '支払いが未決済である理由は、お客様がまだ確認済みになっていないからです。この支払いを受諾する前にアカウントの認証を実行する必要があります。'
        : ( $code eq 'other' )
        ? '支払いが未決済である理由が、上記以外の場合。詳細については、PayPalカスタマーサービスにお問い合わせください。'
        : ''
    );
}

1;
