package KMLViewer;
require CMONOS::TableMaker::Viewer;
@ISA = qw(Viewer);
use strict;
use utf8;
######################################################################
# KML ビューワモジュール
# KMLViewer.pm ver1.1.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-01-21
######################################################################
# 座標範囲を座標フィールドの検索としてセット
sub set_viewer ($) {
    my $this = shift;
    return
           if $setting::page{$setting::key}{'refreshDisabled'}
        || !defined $main::gmap_sw
        || $main::gmap_sw eq ''
        || !defined $main::gmap_ne
        || $main::gmap_ne eq '';
    my $has_coordinates
        = (    defined $field::for{'coordinates'}
            && ref $field::obj{ $field::for{'coordinates'} }
            && $field::obj{ $field::for{'coordinates'} }->{'class'}
            && $field::obj{ $field::for{'coordinates'} }->{'class'} eq
            'webapi_gmap' ) ? 1 : 0;
    while ( my ( $id, $field ) = each %field::obj ) {
        next
            unless ref $field
            && !$field->{'Hidden'}
            && ( $field->{'Open'} || $this->is_secure('secret') )
            && $field->{'class'} eq 'webapi_gmap';
        if ( !$has_coordinates ) {
            $field::for{'coordinates'} = $id;
            $has_coordinates = 1;
        }
        if ( $id eq $field::for{'coordinates'} ) {
            $main::search{$id}{'sw'}  = $main::gmap_sw;
            $main::search{$id}{'ne'}  = $main::gmap_ne;
            $field->{'searchEnabled'} = 1;
        }
        else {
            delete $main::search{$id}{'sw'};
            delete $main::search{$id}{'ne'};
        }
    }
    return;
}

######################################################################
# 必須検索項目
sub has_to_satisfy_queries ($) {
    my $this = shift;
    return ( $field::for{'coordinates'} );
}

######################################################################
# ペンディング情報を非表示
sub set_pending_item_alert ($) {
    my $this = shift;
}

######################################################################
# 表示フィールド取得
sub get_required_fields ($) {
    my $this = shift;
    return
        if ref $this->{'required_fields'} && @{ $this->{'required_fields'} };
    $this->get_required_fields_for_template;
    push( @{ $this->{'required_fields'} }, $field::for{'coordinates'} )
        if $field::for{'coordinates'}
        && !grep( $_ eq $field::for{'coordinates'},
        @{ $this->{'required_fields'} } );
    push( @{ $this->{'required_fields'} }, $field::for{'title'} )
        if $field::for{'title'}
        && !
        grep( $_ eq $field::for{'title'}, @{ $this->{'required_fields'} } );
    push( @{ $this->{'required_fields'} }, $field::for{'key'} )
        if $field::for{'key'}
        && !grep( $_ eq $field::for{'key'}, @{ $this->{'required_fields'} } );
    push( @{ $this->{'required_fields'} }, $field::for{'name'} )
        if $field::for{'name'}
        && !
        grep( $_ eq $field::for{'name'}, @{ $this->{'required_fields'} } );
}

######################################################################
# ページヘッダ
sub print_viewer_header ($) {
    my $this = shift;
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    $setting::page{$setting::key}{'limit'} = 100
        if !defined $setting::page{$setting::key}{'limit'}
        || $setting::page{$setting::key}{'limit'} eq '';
    $main::uni = $main::u    = $setting::page{$setting::key}{'limit'};
    $main::p   = $main::pnum = 0;
    binmode STDOUT, ":utf8";
    $this->print_http_header('application/vnd.google-earth.kml+xml');

    #$this->print_http_header('text/xml');
}

######################################################################
# コンテンツヘッダ
sub print_contents_header ($) {
    my $this = shift;
    print '<?xml version="1.0" encoding="UTF-8"?>', "\x0A";
    print
        '<kml xmlns="http://earth.google.com/kml/2.2" xmlns:atom="http://www.w3.org/2005/Atom">',
        "\x0A";
    print '<Document>', "\x0A";
    print '  <name>'
        . $this->clean_up( $setting::general{'cginame'}, 1 )
        . "</name>\x0A"
        if defined $setting::general{'cginame'}
        && $setting::general{'cginame'} ne '';
    if ( ref $field::obj{ $field::for{'coordinates'} }
        && $field::obj{ $field::for{'coordinates'} }->{'defaultMarker'} )
    {
        $this->{'style_id'} = 'defaultId';
        my $gmap_field = $field::obj{ $field::for{'coordinates'} };
        print '  <Style id="defaultIcon">', "\x0A";
        print '    <IconStyle>',            "\x0A";
        print '      <Icon>',               "\x0A";
        print '        <href>' . $this->set_protocol($gmap_field->{'defaultMarker'}) . '</href>',
            "\x0A";
        print '        <width>'
            . $gmap_field->{'defaultMarkerWidth'}
            . '</width>', "\x0A";
        print '        <height>'
            . $gmap_field->{'defaultMarkerHeight'}
            . '</height>', "\x0A";
        print '        <anchorx>'
            . $gmap_field->{'defaultMarkerAnchorX'}
            . '</anchorx>', "\x0A";
        print '        <anchory>'
            . $gmap_field->{'defaultMarkerAnchorY'}
            . '</anchory>', "\x0A";
        print '      </Icon>',    "\x0A";
        print '    </IconStyle>', "\x0A";
        print '  </Style>',       "\x0A";
    }
    else {
        $this->{'style_id'} = 'temporaryIcon';
        my $href
            = ( $setting::page{$setting::key}{'markerUrl'} )
            ? $setting::page{$setting::key}{'markerUrl'}
            : '//maps.google.com/mapfiles/ms/micons/red-dot.png';
        require CMONOS::CGI_Lib::GMapMarkerList;
        my $shadow = GMapMarkerList->get_shadow_by_marker($href);
        if ( !$shadow ) {
            $shadow = $href;
            $shadow =~ s/\.png$/.shadow.png/;
        }
        print '  <Style id="temporaryIcon">',  "\x0A";
        print '    <IconStyle>',               "\x0A";
        print '      <Icon>',                  "\x0A";
        print '        <href>',                $this->set_protocol($href), '</href>', "\x0A";
        print '        <width>32</width>',     "\x0A";
        print '        <height>32</height>',   "\x0A";
        print '        <anchorx>16</anchorx>', "\x0A";
        print '        <anchory>32</anchory>', "\x0A";
        print '        <shadow>',              $this->set_protocol($shadow), '</shadow>', "\x0A";
        print '        <swidth>59</swidth>',   "\x0A";
        print '        <sheight>32</sheight>', "\x0A";
        print '      </Icon>',                 "\x0A";
        print '    </IconStyle>',              "\x0A";
        print '  </Style>',                    "\x0A";
    }
}

sub set_protocol ($$) {
    my($this,$url) = @_;
    my $p = ($env::ssl) ? 'https:' : 'http:';
    $url = $p . $url unless $url =~ s/https?:/$p/is;
    return $url;
}

######################################################################
# コンテンツフッタ
sub print_contents_footer ($) {
    my $this = shift;
    if ( !$this->{'mcount'} ) {
        require CMONOS::CGI_Lib::Deco;
        my $icon = Deco->new(
            $env::image_dir . '/tablemaker',
            $env::image_lib . '/tablemaker'
        );
        my $image = $icon->image_path( 'nomatch',
            $setting::page{$setting::key}{'nomatch'} );
        $image = $this->full_path(
            ($image) ? $image : $env::image_dir . '/tablemaker/nomatch.png' );
        print '  <ScreenOverlay>', "\x0A";
        print
            '    <name>条件に一致するレコードが存在しません。</name>',
            "\x0A";
        print '    <visibility>0</visibility>', "\x0A";
        print
            '    <description>条件に一致するレコードが存在しません。</description>',
            "\x0A";
        print '    <Icon>', "\x0A";
        print '      <href>' . $image . '</href>', "\x0A";
        print '    </Icon>', "\x0A";
        print
            '    <overlayXY x="0.5" y="0.5" xunits="fraction" yunits="fraction"/>',
            "\x0A";
        print
            '    <screenXY x="0.5" y="0.5" xunits="fraction" yunits="fraction"/>',
            "\x0A";
        print '    <size width="'
            . ( ( $setting::page{$setting::key}{'nomatchWidth'} )
            ? $setting::page{$setting::key}{'nomatchWidth'}
            : '280' )
            . '" height="'
            . ( ( $setting::page{$setting::key}{'nomatchHeight'} )
            ? $setting::page{$setting::key}{'nomatchHeight'}
            : '90' )
            . '" x="-1" y="-1" xunits="fraction" yunits="fraction"/>', "\x0A";
        print '  </ScreenOverlay>', "\x0A";
    }
    elsif ( $setting::page{$setting::key}{'limit'} < $this->{'mcount'} ) {
        require CMONOS::CGI_Lib::Deco;
        my $icon = Deco->new(
            $env::image_dir . '/tablemaker',
            $env::image_lib . '/tablemaker'
        );
        my $image = $icon->image_path( 'toomuch',
            $setting::page{$setting::key}{'toomuch'} );
        $image = $this->full_path(
            ($image) ? $image : $env::image_dir . '/tablemaker/toomuch.png' );
        print '  <ScreenOverlay>', "\x0A";
        print
            '    <name>全てのレコードを表示できませんでした。</name>',
            "\x0A";
        print '    <visibility>0</visibility>', "\x0A";
        print
            '    <description>ヒット数が多過ぎるため、全てのレコードを表示できませんでした。</description>',
            "\x0A";
        print '    <Icon>', "\x0A";
        print '      <href>' . $image . '</href>', "\x0A";
        print '    </Icon>', "\x0A";
        print
            '    <overlayXY x="0.5" y="0.5" xunits="fraction" yunits="fraction"/>',
            "\x0A";
        print
            '    <screenXY x="0.5" y="0.5" xunits="fraction" yunits="fraction"/>',
            "\x0A";
        print '    <size width="'
            . ( ( $setting::page{$setting::key}{'toomuchWidth'} )
            ? $setting::page{$setting::key}{'toomuchWidth'}
            : '280' )
            . '" height="'
            . ( ( $setting::page{$setting::key}{'toomuchHeight'} )
            ? $setting::page{$setting::key}{'toomuchHeight'}
            : '90' )
            . '" x="-1" y="-1" xunits="fraction" yunits="fraction"/>', "\x0A";
        print '  </ScreenOverlay>', "\x0A";
    }
    print '</Document>', "\x0A";
    print '</kml>',      "\x0A";
}

######################################################################
# ページフッタ
sub print_viewer_footer ($) {
    my $this = shift;
    binmode STDOUT;
}

######################################################################
# レコード
sub print_record ($$$) {
    my ( $this, $c, $n ) = @_;
    my ( %temp, %icon, $file_dir, $name, $title, $key, $description,
        $coordinates );
    $this->set_title_link( $c, $n );
    $file_dir = "$default::file/$c/$n";
    foreach my $id ( @{ $this->{'required_fields'} } ) {
        next
            if !ref $field::obj{$id}
            || !$field::obj{$id}->usual_format_enabled
            || $field::obj{$id}->{'class'} eq 'select_xmap';
        my $db = $field::obj{$id}->db;
        unless ( !$db || ref tied %{ $this->{'DB'}{$c}{$db} } ) {
            tie( %{ $this->{'DB'}{$c}{$db} },
                $env::dbm_file, "$default::db/$c/$db", O_RDONLY,
                $env::dbm_mode );
        }
        my $data
            = ($db)
            ? $this->decode_utf8(
            $this->{'DB'}{$c}{$db}{ $this->encode_utf8($n) } )
            : '';
        if ( $id eq $field::for{'coordinates'} ) {
            %icon = $field::obj{$id}->elements($data);
            $coordinates
                = ( defined $icon{'m'} && $icon{'m'} ne '' && $icon{'c'} )
                ? $icon{'c'}
                : ( $icon{'kml'} && $icon{'p'} ) ? $icon{'p'}
                : ( $icon{'c'} ) ? $icon{'c'}
                :                  '';
            $coordinates
                = ( $coordinates =~ /^(-?[0-9\.]+),(-?[0-9\.]+)$/ )
                ? $2 . ',' . $1
                : '';
        }
        next if $field::obj{$id}->{'class'} eq 'webapi_gmap';
        if ( $id eq $field::for{'title'} ) {
            $title = $field::obj{$id}
                ->format_for_text( $data, $file_dir, $c, $n );
        }
        elsif ( $id eq $field::for{'key'} ) {
            $key = $field::obj{$id}
                ->format_for_text( $data, $file_dir, $c, $n );
        }
        elsif ( $id eq $field::for{'name'} ) {
            $name = $field::obj{$id}
                ->format_for_text( $data, $file_dir, $c, $n );
        }
        if ( $this->{'has_template'} ) {
            $temp{$id} = $data;
        }
        else {
            local $main::m   = 'GMapViewer';
            local $main::k   = 'gmap';
            local $main::mod = 'GMapViewer';
            $data = $this->clean_up(
                $field::obj{$id}->usual_format( $data, $file_dir, $c, $n ) );
            $description
                .= '<div>'
                . (
                ( defined $field::for{'key'} && $id eq $field::for{'key'} )
                ? '<strong>'
                : '' )
                . $data
                . (
                ( defined $field::for{'key'} && $id eq $field::for{'key'} )
                ? '</strong>'
                : '' )
                . '</div>'
                if $data ne '';
        }
    }
    if ( $this->{'has_template'} ) {
        local $main::m   = 'GMapViewer';
        local $main::k   = 'gmap';
        local $main::mod = 'GMapViewer';
        $description
            = $this->{'template'}
            ->replace( $setting::page{$setting::key}{'template'},
            \%temp, '', $file_dir, $c, $n );
    }
    else {
        $description
            = '<div style="font-size: 14px;">' . $description . '</div>';
    }
    $this->clear_title_link;
    return
           if !defined $coordinates
        || $coordinates eq ''
        || !defined $description
        || $description eq '';
    $name = ($title) ? $title : ($key) ? $key : ($name) ? $name : '';
    my $info = '';
    $info .= '    <name>' . $this->clean_up( $name, 1 ) . "</name>\x0A"
        if defined $name && $name ne '';
    if ( defined $description && $description ne '' ) {
        my $my_url = $this->my_url;
        if ( $description
            =~ /<a[\x09\x0A\x0C\x0D\x20]+href="(?:\Q$my_url\E|\Q$cgi::path\E)(\?m=DetailViewer(?![0-9a-zA-Z_])[^"]*(?:&amp;c=$c&amp;n=$n|&amp;record_id=$n)(?![0-9a-zA-Z_])[^"]*)"[^>]*>/s
            )
        {
            $info .= '    <atom:link href="' . $my_url . $1 . "\" />\x0A";
            $info
                .= '    <atom:id>'
                . $my_url
                . '?m=DetailViewer&amp;c='
                . $c
                . '&amp;n='
                . $n
                . "</atom:id>\x0A";
        }
        $info
            .= '    <description><![CDATA['
            . $this->clean_up($description)
            . "]]></description>\x0A";
    }
    if ( %icon && $field::obj{ $field::for{'coordinates'} }->{'showKml'} ) {
        if ( $icon{'kml'} && !$icon{'url'} ) {
            my $kml_path
                = $default::file . '/'
                . $c . '/'
                . $n . '/'
                . $field::for{'coordinates'}
                . ( ( $icon{'kml'} eq 'kmz' ) ? '.kmz' : '.kml' );
            $icon{'url'} = $this->full_path($kml_path);
        }
        if ( $icon{'url'} ) {
            print '  <NetworkLink>' . "\x0A";
            print $info;
            print '    <Link>' . "\x0A";
            print '      <href>'
                . $this->escape( $icon{'url'} )
                . '</href>' . "\x0A";
            print '    </Link>' . "\x0A";
            print '  </NetworkLink>' . "\x0A";
        }
    }
    print '  <Placemark>', "\x0A";
    print $info;
    if ( %icon && $icon{'mk'} && $icon{'width'} ) {
        $icon{'mk'} = $this->full_path($file_dir) . '/' . $icon{'mk'}
            if $icon{'mk'} && $icon{'mk'} !~ /^https?:\/\//i;
        print '    <Style>',       "\x0A";
        print '      <IconStyle>', "\x0A";
        print '        <Icon>',    "\x0A";
        print '          <href>' . $icon{'mk'} . '</href>',            "\x0A";
        print '          <width>' . $icon{'width'} . '</width>',       "\x0A";
        print '          <height>' . $icon{'height'} . '</height>',    "\x0A";
        print '          <anchorx>' . $icon{'anchorx'} . '</anchorx>', "\x0A";
        print '          <anchory>' . $icon{'anchory'} . '</anchory>', "\x0A";
        print '        </Icon>',    "\x0A";
        print '      </IconStyle>', "\x0A";
        print '    </Style>',       "\x0A";
    }
    else {
        print '    <styleUrl>#' . $this->{'style_id'} . '</styleUrl>', "\x0A";
    }
    print '    <Point>', "\x0A";
    print '      <coordinates>' . $coordinates . '</coordinates>', "\x0A";
    print '    </Point>',   "\x0A";
    print '  </Placemark>', "\x0A";
}

sub clean_up ($$;$) {
    my ( $this, $str, $detag ) = @_;
    $str =~ s/<!--[^>]*?-->//gs;
    $str
        =~ s/\&(?!(?:#[0-9]{2,6}|#x[a-fA-F0-9]+|[a-zA-Z0-9]{2,10});)/\&amp;/g;
    if ($detag) {
        $str =~ s/<(.*?)>/(($1 =~ m! alt="(.*?)"!i) ? $1 : '')/egs;
        $str =~ s/"/&quot;/g;
        $str =~ s/</&lt;/g;
        $str =~ s/>/&gt;/g;
    }
    else {
        $str
            =~ s/(<[^<>]*[\x09\x0A\x0C\x0D\x20]+(?:href|src|background|action)=["']?)(?![0-9a-zA-Z_]+:)([^"\x09\x0A\x0C\x0D\x20]+)([^<>]*>)/$1 . $this->full_path($2) . $3/seig;
        $str
            =~ s/(<param[\x09\x0A\x0C\x0D\x20]+name=["']?(?:src|movie)["']?[\x09\x0A\x0C\x0D\x20]+value=["']?)(?![0-9a-zA-Z_]+:)([^"\x09\x0A\x0C\x0D\x20]+)([^<>]*>)/$1 . $this->full_path($2) . $3/seig;
        $str
            =~ s/(<video(?![0-9a-zA-Z_])[^<>]*[\x09\x0A\x0C\x0D\x20]+preview=["']?)(?![0-9a-zA-Z_]+:)([^"\x09\x0A\x0C\x0D\x20]+)([^<>]*>)/$1 . $this->full_path($2) . $3/seig;
        $str =~ s/<script(?![0-9a-zA-Z_])[^>]*>.*?<\/script>//sig;
    }
    $str =~ s/[\x0D\x0A]+//gs;
    return $str;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    $setting::field_tag_enabled
        = 'no'; #tag=タグのみ/all=全ての設定/no=タグをつけない
    $setting::field_style_disabled = 2
        ; #2=フォント設定スタイル設定を無視/1=フォント設定のみ無視

    #設定種キー設定
    $setting::key = 'gmap'
        if !defined $setting::key || !$setting::key;    #ページ設定キー

    #デフォルトキー
    $main::k = 'kml' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'kml', 'KMLViewer' );
    $main::mod = $main::m;
}

1;
