package GMapPage;
require CMONOS::CGI_Lib::Tools::Page;
@ISA = qw(Page);
use strict;
use utf8;
######################################################################
# Googleマップページ設定モジュール
# GMapPage.pm ver1.1.6
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-12-04
######################################################################
# 設定項目取得
sub get_sub_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #数字チェック
    $check::num{'limit'}        = '最大表示数';
    $check::num{'gmapMaxWidth'} = '情報ウィンドウ最大幅';

    #チェックボックス
    push( @check::checkbox,
        'toolboxTop',       'toolboxBottom',
        'gmapOverView',     'gmapStreetView',
        'gmapSearch',       'refreshDisabled',
        'placeListEnabled', 'floatingSearchFormEnabled' );

    return;
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    $preference::id = 'gmap';    # データキー
    $preference::name = 'Googleマップ表示ページレイアウト'
        ;                        # 設定パネル名
    $preference::title
        = 'Googleマップ表示ページのレイアウトについて設定します。';
    require CMONOS::CGI_Lib::GMapMarkerList;
    $this->set_css( GMapMarkerList->new->marker_selector_css );
    require CMONOS::CGI_Lib::Deco;
    $this->{'icon_obj'} = Deco->new( $env::image_dir . '/tablemaker',
        $env::image_lib . '/tablemaker' );
    $setting::image_css = 'tablemaker-image.css';
}

######################################################################
# 設定パネル
sub _print_header_tool ($) {
    my $this = shift;
    return
        '<option value="&lt;!--categorylist--&gt;&lt;!--/categorylist--&gt;">カテゴリーリスト(&lt;!--categorylist--&gt;)</option><option value="&lt;!--search--&gt;&lt;!--/search--&gt;">検索フォーム (&lt;!--search--&gt;)</option>';
}

sub print_sub_preference_panel ($$$) {
    my ( $this, $id, $key ) = @_;
    print '<fieldset><legend>ツールボックス</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:gmap_toolboxTop" id="data:gmap_toolboxTop" value="1"'
        . (
        (   defined $main::data{ $key . 'toolboxTop' }
                && $main::data{ $key . 'toolboxTop' }
        ) ? ' checked="checked"' : ''
        )
        . ' /> ツールボックスをページ上部に表示する</label></li>';
    print
        '<li><label><input type="checkbox" name="data:gmap_toolboxBottom" id="data:gmap_toolboxBottom" value="1"'
        . (
        (   defined $main::data{ $key . 'toolboxBottom' }
                && $main::data{ $key . 'toolboxBottom' }
        ) ? ' checked="checked"' : ''
        )
        . ' /> ツールボックスをページ下部に表示する</label></li>';
    print '</ol>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>レコード読み込み設定</legend>';
    print
        '<label><input type="checkbox" name="data:gmap_refreshDisabled" id="data:gmap_refreshDisabled" value="1"'
        . (
        ( $main::data{'gmap_refreshDisabled'} ) ? ' checked="checked"' : '' )
        . ' /> 地図の移動・拡大縮小時に表示対象レコードを取得せず、最初に全てのレコードを読み込む</label>';
    print '</fieldset>';

    print '<fieldset>';
    print
        '<legend><label for="data:gmap_limit">最大表示数</label></legend>';
    print
        '<input type="text" name="data:gmap_limit" id="data:gmap_limit" size="8" value="'
        . $this->escape( $main::data{'gmap_limit'} )
        . '" /> 件';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>Googleマップ表示領域設定</legend>';

    print '<fieldset>';
    print '<legend>Googleマップ表示設定</legend>';
    print
        '<label>幅 <input type="text" size="6" name="data:gmap_gmapWidth" id="data:gmap_gmapWidth" value="'
        . $this->escape( $main::data{'gmap_gmapWidth'} )
        . '" /> px</label> &times; ';
    print
        '<label>高さ <input type="text" size="6" name="data:gmap_gmapHeight" id="data:gmap_gmapHeight" value="'
        . $this->escape( $main::data{'gmap_gmapHeight'} )
        . '" /> px</label>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:gmap_gmapOverView" id="data:gmap_gmapOverView" value="1"'
        . ( ( $main::data{'gmap_gmapOverView'} ) ? ' checked="checked"' : '' )
        . ' /> 概観地図を表示する</label></li>';
    print
        '<li><label><input type="checkbox" name="data:gmap_gmapStreetView" id="data:gmap_gmapStreetView" value="1"'
        . (
        ( $main::data{'gmap_gmapStreetView'} ) ? ' checked="checked"' : '' )
        . ' /> ストリートビューツールを表示する</label></li>';
    print
        '<li><label><input type="checkbox" name="data:gmap_gmapSearch" id="data:gmap_gmapSearch" value="1"'
        . ( ( $main::data{'gmap_gmapSearch'} ) ? ' checked="checked"' : '' )
        . ' /> 住所検索ツールを表示する</label></li>';
    print '</ol>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>デフォルトの地図</legend>';
    my $url = $this->escape(
              $env::javascript
            . '/gmap.html?'
            . $this->url_encoded_query(
            {   (   ($env::google_api_key) ? ( 'key' => $env::google_api_key )
                    : ()
                ),
                'z' =>
                    ( ( $main::data{'gmap_z'} ) ? $main::data{'gmap_z'} : 4 ),
                't' => (
                    ( $main::data{'gmap_t'} ) ? $main::data{'gmap_t'} : ''
                ),
                'c' => (
                    ( $main::data{'gmap_c'} ) ? $main::data{'gmap_c'}
                    : '36.315125,137.504883'
                ),
                'z_field' => 'data:gmap_z',
                't_field' => 'data:gmap_t',
                'c_field' => 'data:gmap_c',
                ( ( $main::data{'gmap_gmapOverView'} ) ? ( 'om' => 1 ) : () ),
                (     ( $main::data{'gmap_gmapStreetView'} ) ? ( 'sv' => 1 )
                    : ()
                ),
                ( ( $main::data{'gmap_gmapSearch'} ) ? ( 's' => 1 ) : () ),
                (     ( $main::data{'gmap_gmapLinkTarget'} )
                    ? ( 'tr' => $main::data{'gmap_gmapLinkTarget'} )
                    : ( 'tr' => 'd' )
                ),
                (     ( $main::data{'gmap_gmapMaxWidth'} )
                    ? ( 'mw' => $main::data{'gmap_gmapMaxWidth'} )
                    : ()
                ),
            }
            )
    );
    print '<iframe class="gmap-frame" id="default-gmap-viewer"';
    print ' src="' . $url . '"';
    print ' width="'
        . ( ( $main::data{'gmap_gmapWidth'} )
        ? $main::data{'gmap_gmapWidth'}
        : 400 )
        . '"';
    print ' height="'
        . ( ( $main::data{'gmap_gmapHeight'} )
        ? $main::data{'gmap_gmapHeight'}
        : 400 )
        . '"';
    print ' scrolling="no" marginwidth="0" marginheight="0" frameborder="0">';
    print '<a href="' . $url
        . '" target="_blank">新しいウィンドウで地図を開く</a>';
    print '</iframe>';
    print '</fieldset>';
    print '<input type="hidden" name="data:gmap_z" id="data:gmap_z" value="'
        . $this->escape( $main::data{'gmap_z'} ) . '" />';
    print '<input type="hidden" name="data:gmap_t" id="data:gmap_t" value="'
        . $this->escape( $main::data{'gmap_t'} ) . '" />';
    print '<input type="hidden" name="data:gmap_c" id="data:gmap_c" value="'
        . $this->escape( $main::data{'gmap_c'} ) . '" />';

    $this->use_field( 'field', $default::field_pref_dir . '/inputfield', 2 )
        if !%field::obj;
    print '<fieldset>';
    print
        '<legend><label for="data:gmap_template">情報ウィンドウ テンプレート</label></legend>';
    print
        '<textarea name="data:gmap_template" id="data:gmap_template" cols="72" rows="10" wrap="off" class="content-editor-area">'
        . $this->escape( $main::data{'gmap_template'} )
        . '</textarea>';
    print '<div class="toolbox">';
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'data:gmap_template\').setAutoFit(this.checked);" />フィット</label> ';
    print
        '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:gmap_template\').quotationByMenu(this)" title="これらのタグは、対応するフィールドデータに変換されます。">';
    print
        '<option value="">▼入力フィールドタグを挿入する</option>';

    foreach my $n ( sort { $a <=> $b } keys %field::ORDER ) {
        my $fid = $field::ORDER{$n};
        next
            if !$fid
            || $fid < 1
            || !ref $field::obj{$fid}
            || $field::obj{$fid}->{'class'} eq 'webapi_gmap'
            || $field::obj{$fid}->{'class'} eq 'select_xmap'
            || $field::obj{$fid}->{'class'} =~ /^ug_/;
        print '<option value="&lt;!--field id=&quot;'
            . $fid
            . '&quot;--&gt;&lt;!--/field--&gt;">'
            . $fid . ' : '
            . $field::obj{$fid}->{'name'}
            . '</option>';
    }
    print '<option value=""></option>';
    print
        '<option value="&lt;!--head--&gt;&lt;!--/head--&gt;">フィールドヘッダ</option>';
    print
        '<option value="&lt;!--foot--&gt;&lt;!--/foot--&gt;">フィールドフッタ</option>';
    print
        '<option value="&lt;!--name--&gt;&lt;!--/name--&gt;">フィールド名</option>';
    print
        '<option value="&lt;!--groupname--&gt;&lt;!--/groupname--&gt;">フィールドグループ名</option>';
    print '<option value=""></option>';
    print
        '<option value="&lt;!--counter id=&quot;&quot;--&gt;&lt;!--/counter--&gt;">カウンター</option>';
    print '<option value=""></option>';
    print
        '<option value="&lt;!--if id=&quot;&quot;--&gt;&lt;!--else--&gt;&lt;!--/if--&gt;">id に値があるとき〜</option>';
    print
        '<option value="&lt;!--unless id=&quot;&quot;--&gt;&lt;!--else--&gt;&lt;!--/unless--&gt;">id に値がないとき〜</option>';
    print '</select>';
    require CMONOS::CGI_Lib::HTMLHelper;
    print HTMLHelper->new->html_helper_menu('data:gmap_template');
    print '</div>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>情報ウィンドウ設定</legend>';
    print '<label>情報ウィンドウ内リンクを';
    print $this->mk_select_menu(
        'data:gmap_gmapLinkTarget', '', '',
        $main::data{'gmap_gmapLinkTarget'},
        'd' => '詳細表示ウィンドウで開く',
        'p' => '同じウィンドウで開く',
        'b' => '新しいウィンドウで開く',
    );
    print '</label>';
    print
        ' (<label for="data:gmap_gmapMaxWidth">情報ウィンドウ最大幅</label>';
    print
        '<input type="number" size="4" name="data:gmap_gmapMaxWidth" id="data:gmap_gmapMaxWidth" value="'
        . $this->escape( $main::data{'gmap_gmapMaxWidth'} )
        . '" /> px)';
    print '</fieldset>';

    print GMapMarkerList->new->marker_selector( 'data:gmap_markerUrl',
        $main::data{'gmap_markerUrl'} );

    print
        '<fieldset><legend>条件に一致するデータが存在しないときの警告画像</legend>';
    print $this->{'icon_obj'}
        ->image_menu( 'data:gmap_nomatch', 'nomatch', 120,
        $main::data{'gmap_nomatch'} );
    print '</fieldset>';

    print
        '<fieldset><legend>ヒット数が多過ぎて全てのデータを表示できないときの警告画像</legend>';
    print $this->{'icon_obj'}
        ->image_menu( 'data:gmap_toomuch', 'toomuch', 120,
        $main::data{'gmap_toomuch'} );
    print '</fieldset>';

    print '</fieldset>';

    print '<fieldset>';
    print '<legend>表示レコードリスト</legend>';
    print
        '<label><input type="checkbox" name="data:gmap_placeListEnabled" id="data:gmap_placeListEnabled" value="1"'
        . (
        ( $main::data{'gmap_placeListEnabled'} ) ? ' checked="checked"' : '' )
        . ' /> 表示レコードリストを表示する</label>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>検索欄設定</legend>';
    print
        '<label><input type="checkbox" name="data:gmap_floatingSearchFormEnabled" id="data:gmap_floatingSearchFormEnabled" value="1"'
        . ( ( $main::data{'gmap_floatingSearchFormEnabled'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 検索欄をドラッグ可能にする</label>';
    print '</fieldset>';
}

######################################################################
# 設定チェック
sub is_valid_setting ($) {
    my $this = shift;
    $this->check_gmapsize( 'gmap_gmapWidth',  'Googleマップ横幅' );
    $this->check_gmapsize( 'gmap_gmapHeight', 'Googleマップ高さ' );
    ( $main::data{'gmap_nomatchWidth'}, $main::data{'gmap_nomatchHeight'} )
        = ( $main::data{'gmap_nomatch'} )
        ? $this->{'icon_obj'}
        ->image_info( 'nomatch', $main::data{'gmap_nomatch'} )
        : ( '', '' );
    ( $main::data{'gmap_toomuchWidth'}, $main::data{'gmap_toomuchheight'} )
        = ( $main::data{'gmap_toomuch'} )
        ? $this->{'icon_obj'}
        ->image_info( 'toomuch', $main::data{'gmap_toomuch'} )
        : ( '', '' );
    return;
}

sub check_gmapsize ($$$) {
    my ( $this, $key, $name ) = @_;
    if ( defined $main::data{$key} && $main::data{$key} ne '' ) {
        $main::data{$key} =~ s/px$//si;
        my $is_percent = ( $main::data{$key} =~ s/[\%％]$//si ) ? 1 : 0;
        if ( $this->is_nan( $main::data{$key} ) ) {
            push(
                @{ $this->{'dialog'} },
                $name
                    . 'を更新できませんでした。半角数字(+%)にしてください。'
            );
            delete $main::data{$key};
            return 0;
        }
        $main::data{$key} .= '%' if $is_percent;
    }
    return 1;
}

######################################################################
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 't003', 'GMapPage' );

    #ツール登録
    $setting::sub_jump_menu{'page'}{'t003'}
        = 'Googleマップ表示ページ';
    $setting::selected_jump_menu{'t003'} = 't000'
        ; #サブメニューがあるときメインメニューのデフォルトキー

    $preference::page{'t003'} = 'gmap';
}

1;
