package TeaserLayout;
require CMONOS::CGI_Lib::Tools::Page;
require CMONOS::TableMaker::TMCommon;
@ISA = qw(Page TMCommon);
use strict;
use utf8;
no warnings qw(once);
######################################################################
# 詳細ページ設定モジュール
# TeaserLayout.pm ver1.2.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-05-04
######################################################################
# 設定項目取得
sub get_sub_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #数字チェック
    $check::num{'tableWidth'} = 'テーブル幅';
    $check::num{'tileWidth'}  = 'タイル横幅';
    $check::num{'tileHeight'} = 'タイル高さ';
    $check::num{'tableCols'}  = 'テーブル列数';
    $check::num{'tableRows'}  = 'テーブル行数';
    $check::num{'interval'}   = '更新間隔';

    #チェックボックス
    push( @check::checkbox,
        'keepInterval', 'linkEnabled', 'tileTableEnabled' );

    #タグ修正
    push( @check::tagfix, 'linkText' );

    return;
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    $this->{'teaser_no'}
        = ( defined $main::k && $main::k =~ /_([0-9]+)$/ ) ? $1 : '';
    $preference::id = 'teaser' . $this->{'teaser_no'};    # データキー
    $preference::name
        = '埋め込み用リスト'
        . $this->{'teaser_no'}
        . 'レイアウト';                              # 設定パネル名
    $preference::title
        = '埋め込み用リスト'
        . $this->{'teaser_no'}
        . 'のレイアウトについて設定します。';
}

sub is_valid_setting ($) {
    my $this = shift;
    if ( defined $main::u && $main::u ne '' && $this->is_nan($main::u) ) {
        push(
            @{ $this->{'dialog'} },
            'テーブル表示時の表示件数を確認してください。半角数字にしてください。'
        );
        undef $main::u;
    }

    #検索条件を URL エンコード
    $main::data{ $preference::id . '_query' } = $this->set_query( 'url',
        [ 'c', 'f', 'u', 's', 'w', 'and', 'search' ] );
    $main::data{ $preference::id . '_query' } =~ s/^\?//s;
    return;
}

######################################################################
# 設定パネル
sub print_preference_panel ($) {
    my $this = shift;
    my ( $id, $key );
    $id = ( defined $preference::id ) ? $preference::id : '';
    $id .= '_' if $id ne '';

    #設定のインポート
    $this->use_field( 'field', $default::field_pref_dir . '/inputfield', 2 );
    my @skip_fields
        = ( 'xtm_favorites', 'xtm_pending', 'ug_sbm', 'xtm_checker' );
    if ( defined $main::import && $main::import ne '' && %preference::page ) {
        $key = $main::import . '_';
    }
    else {
        $key = $id;
    }

    print '<input type="hidden" name="id" value="' . $preference::id . '" />';

    print '<fieldset><legend><label for="data:' . $id
        . 'interval">更新間隔</label></legend>';
    print $this->mk_select_menu(
        'data:' . $id . 'interval', '', '', $main::data{ $key . 'interval' },
        '3600'   => '1時間',
        '10800'  => '3時間',
        '21600'  => '6時間',
        '43200'  => '半日',
        '86400'  => '1日',
        '259200' => '3日',
        '604800' => '1週間',
    );
    print ' <label><input type="checkbox" name="data:'
        . $id
        . 'keepInterval" id="data:'
        . $id
        . 'keepInterval" value="1"'
        . (
        (   defined $main::data{ $key . 'keepInterval' }
                && $main::data{ $key . 'keepInterval' }
        ) ? ' checked="checked"' : ''
        )
        . ' /> データベースが更新されていても更新間隔を優先する</label>';
    print '</fieldset>';

    #クエリーをデコード
    if ( $main::data{ $key . 'query' } ) {
        my %in = ();
        $this->url_decode( $main::data{ $key . 'query' }, \%in, 1 );
        $this->variable( \%in, 0, 'main' );
        push( @main::f, $main::f )
            if !@main::f && defined $main::f && $main::f;
        push( @main::c, $main::c )
            if !@main::c && defined $main::c && $main::c;
        undef $main::f;
        undef $main::c;
    }

    print '<fieldset><legend>ソート設定</legend>';

    #カテゴリー選択
    print $this->category_selector(
        '<table class="field-table"><tbody>',
        '<tr><td class="field-name"><label for="c">カテゴリー</label></td><td class="field">',
        '</td></tr>',
        1,
        1
    );

    #ソート
    print '<tr>';
    print '<td class="field-name"><label for="s">ソート</label></td>';
    print '<td class="field">';

    print '<select id="s" name="s">';

    $main::s
        = ( $field::for{'sort'} ) ? $field::for{'sort'}
        : ( $field::for{'key'} )   ? $field::for{'key'}
        : ( $field::for{'title'} ) ? $field::for{'title'}
        : ''
        if !defined $main::s || $main::s eq '';
    foreach ( sort { $a <=> $b } keys %field::ORDER ) {
        my $fid = $field::ORDER{$_};
        next
            if !ref $field::obj{$fid}
            || $field::obj{$fid}->{'Hidden'}
            || !$field::obj{$fid}->{'Open'}
            || !$field::obj{$fid}->sort_enabled
            || !$field::obj{$fid}->{'sortEnabled'}
            || grep( $_ eq $field::obj{$fid}->{'class'}, @skip_fields );
        print '<option value="'
            . $fid . '"'
            . ( ( defined $main::s && $main::s eq $fid )
            ? ' selected="selected"'
            : '' )
            . '>'
            . $this->jfold( $field::obj{$fid}->name, 32, 'I', 1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    #ソート順
    print '<tr>';
    print '<td class="field-name"><label for="w">'
        . ( ($default::english) ? 'Sort By' : 'ソート順' )
        . '</label></td>';
    print '<td class="field">';

    $main::w = ($field::SORTORDER) ? $field::SORTORDER : 'ascend'
        if !defined $main::w || $main::w eq '';
    print '<select id="w" name="w">';
    print '<option value="ascend"'
        . ( ( defined $main::w && $main::w eq 'ascend' )
        ? ' selected="selected"'
        : '' )
        . '>'
        . ( ($default::english) ? 'ascending' : '昇順' )
        . '</option>';
    print '<option value="descend"'
        . ( ( defined $main::w && $main::w eq 'descend' )
        ? ' selected="selected"'
        : '' )
        . '>'
        . ( ($default::english) ? 'descending' : '降順' )
        . '</option>';
    print '</select>';
    print '</td></tr>';
    print '<tr>';
    print '<td class="field-name"><label for="u">表示件数</label></td>';
    print
        '<td class="field"><input type="number" size="3" maxlength="3" name="u" id="u" value="'
        . $this->escape($main::u)
        . '" /> 件</td>';
    print '</tr>';

    print '</tbody></table></fieldset>';

    my ( $search_form, $search );
    foreach ( sort { $a <=> $b } keys %field::ORDER ) {
        my $fid = $field::ORDER{$_};
        next
            if !ref $field::obj{$fid}
            || $field::obj{$fid}->{'Hidden'}
            || !$field::obj{$fid}->{'Open'}
            || !$field::obj{$fid}->{'searchEnabled'}
            || !$field::obj{$fid}->search_form_enabled
            || grep( $_ eq $field::obj{$fid}->{'class'}, @skip_fields );
        $search_form .= $field::obj{$fid}->search_form;
        $search++;
    }

    if ( defined $search && $search ) {
        print '<fieldset><legend>検索設定</legend>';
        print '<table class="field-table"><tbody>';
        print $search_form;
        print '</tbody>';
        if ( $search > 1 ) {
            print '<tbody>';
            print '<tr class="tbody-start">';
            print
                '<td class="field-name"><label for="and">検索方式</label></td>';
            print '<td class="field">';
            print '<select name="and">';
            print '<option value="1">全ての条件に一致</option>';
            print '<option value="0"'
                . ( ( !$main::and ) ? ' selected="selected"' : '' )
                . '>いずれかの条件に一致</option>';
            print '</select>';
            print '</td></tr>';
            print '</tbody>';
        }
        print '</table></fieldset>';
    }

    print '<fieldset><legend><label><input type="radio" name="data:'
        . $id
        . 'mode" value="tile"'
        . (
        (          $main::data{ $key . 'mode' }
                && $main::data{ $key . 'mode' } eq 'tile'
        ) ? ' checked="checked"' : ''
        )
        . ' onclick="cmonos.element.item(\'table-settings\').disappear(this.checked);cmonos.element.item(\'tile-settings\').disappear(!this.checked);" />タイル表示</label></legend>';
    print '<div id="tile-settings">';
    print '<fieldset><legend>タイル表示設定</legend>';
    print '<table class="field-table"><tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="data:' . $id
        . 'tileWidth">タイル横幅</label></td>';
    print '<td class="field"><input type="text" name="data:'
        . $id
        . 'tileWidth" id="data:'
        . $id
        . 'tileWidth" value="'
        . $this->escape( $main::data{ $key . 'tileWidth' } )
        . '" size="7" maxlength="7" /> <span class="">ピクセル</span></td>';
    print '<td class="field-name"><label for="data:' . $id
        . 'tileHeight">タイル高さ</label></td>';
    print '<td class="field"><input type="text" name="data:'
        . $id
        . 'tileHeight" id="data:'
        . $id
        . 'tileHeight" value="'
        . $this->escape( $main::data{ $key . 'tileHeight' } )
        . '" size="7" maxlength="7" /> <span class="">ピクセル</span></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="data:' . $id
        . 'tileAlign">行揃え</label></td>';
    print '<td class="q-field">';
    print $this->mk_select_menu(
        'data:' . $id . 'tileAlign', '', '',
        $main::data{ $key . 'tileAlign' },
        ''       => '',
        'left'   => '左寄せ',
        'center' => '中央揃え',
        'right'  => '右寄せ'
    );
    print '</td>';
    print '<td class="field-name"><label for="data:' . $id
        . 'tileValign">垂直位置揃え</label></td>';
    print '<td class="q-field">';
    print $this->mk_select_menu(
        'data:' . $id . 'tileValign', '', '',
        $main::data{ $key . 'tileValign' },
        ''       => '',
        'top'    => '上寄せ',
        'middle' => '中央揃え',
        'bottom' => '下寄せ'
    );
    print '</td>';
    print '</tr>';
    print '</tbody></table>';
    print '<fieldset><legend><label><input type="checkbox" name="data:'
        . $id
        . 'tileTableEnabled" value="1"'
        . (
        (   !defined $main::data{ $key . 'tileTableEnabled' }
                || $main::data{ $key . 'tileTableEnabled' }
        ) ? ' checked="checked"' : ''
        )
        . ' onclick="cmonos.element.item(\'tile-table-settings\').disappear(!this.checked);" />テーブルでタイル状に表示する</label></legend>';
    print '<div id="tile-table-settings"><table class="field-table"><tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="data:' . $id
        . 'tileTable">タイルテーブルタイプ</label></td>';
    print '<td class="q-field">';
    print $this->table_menu(
        'data:' . $id . 'tileTable',
        (   ( $key eq 'tile_' )
            ? $main::data{'tile_table'}
            : $main::data{ $key . 'tileTable' }
        )
    );
    print '</td>';
    print '<td class="field-name"><label for="data:' . $id
        . 'tableCols">タイル列数</label></td>';
    print '<td class="q-field"><input type="text" name="data:'
        . $id
        . 'tableCols" id="data:'
        . $id
        . 'tableCols" value="'
        . $this->escape( $main::data{ $key . 'tableCols' } )
        . '" size="7" maxlength="7" /></td>';
    print '</tr>';
    print '</tbody></table></div>';
    print '</fieldset>';
    print $this->javascript(
        "cmonos.element.item('tile-table-settings').disappear(true);")
        if defined $main::data{ $key . 'tileTableEnabled' }
        && !$main::data{ $key . 'tileTableEnabled' };
    print '<fieldset><legend>タイルフレーム</legend>';
    print $cgi::Deco->frame_menu( 'data:' . $id . 'tileFrame',
        'frame', $main::data{ $key . 'tileFrame' } );
    print '</fieldset>';
    print '</fieldset>';

    require CMONOS::CGI_Lib::Template;
    my $this_template = Template->new( \%field::obj );
    $this_template->print_template_form(
        'data:' . $id . 'template',
        'タイルテンプレート（上記タイル表示設定よりテンプレートが優先されます）',
        \$main::data{ $key . 'template' },
        '',
        '一部使用できないフィールドタイプがあります。',
        \@skip_fields
    );

    print '</div>';
    print '</fieldset>';

    print '<fieldset><legend><label><input type="radio" name="data:'
        . $id
        . 'mode" value="table"'
        . (
        (         !$main::data{ $key . 'mode' }
                || $main::data{ $key . 'mode' } eq 'table'
        ) ? ' checked="checked"' : ''
        )
        . ' onclick="cmonos.element.item(\'tile-settings\').disappear(this.checked);cmonos.element.item(\'table-settings\').disappear(!this.checked);" />テーブル表示</label></legend>';
    print '<fieldset id="table-settings">';
    print '<legend>テーブル表示設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="f">'
        . ( ($default::english) ? 'Field' : '表示項目' )
        . '</label></td>';
    print '<td class="field">';
    my $size = keys %field::obj;
    $size = 8 if $size > 8;
    $size = 1 if $size < 1;
    print '<select id="f" name="f" size="' . $size . '" multiple="multiple">';

    foreach ( sort { $a <=> $b } keys %field::ORDER ) {
        my $fid = $field::ORDER{$_};
        next
            if !ref $field::obj{$fid}
            || $field::obj{$fid}->{'Hidden'}
            || !$field::obj{$fid}->{'Open'}
            || grep( $_ eq $field::obj{$fid}->{'class'}, @skip_fields );
        print '<option value="'
            . $fid . '"'
            . (
            (          ( !@main::f && $field::obj{$fid}->{'Default'} )
                    || ( @main::f && grep( $_ eq $fid, @main::f ) )
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{$fid}->name, 32, 'I', 1 )
            . '</option>';
    }
    print( ( $setting::page{'index'}{'checkboxf'} ) ? '</ol>' : '</select>' );
    print '</td></tr>';
    print '<tr>';
    print '<td class="field-name"><label for="data:' . $id
        . 'table">テーブルタイプ</label></td>';
    print '<td class="field">';
    print $this->table_menu(
        'data:' . $id . 'table',
        (   ( $key eq 'tile_' )
            ? $main::data{ $id . 'table' }
            : $main::data{ $key . 'table' }
        )
    );
    print '</td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</fieldset>';
    print '</fieldset>';
    print $this->javascript(
        "cmonos.element.item('"
            . (
            (          $main::data{ $key . 'mode' }
                    && $main::data{ $key . 'mode' } eq 'tile'
            ) ? 'table-settings' : 'tile-settings'
            )
            . "').disappear(true);"
    );

    print '<fieldset><legend><label><input type="checkbox" name="data:'
        . $id
        . 'linkEnabled" value="1"'
        . (
        ( $main::data{ $key . 'linkEnabled' } ) ? ' checked="checked"' : '' )
        . ' />「もっと見る」リンク</label></legend>';
    print '<label>リンクテキスト <input type="text" name="data:'
        . $id
        . 'linkText" value="'
        . $this->escape( $main::data{ $key . 'linkText' } )
        . '" size="56" /></label>';
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
    $this->print_import_menu;
    print '</form>';

    print '<form name="sample-codes" id="sample-codes" action="#">';
    my $target
        = $this->path_change( $default::file, $ENV{'SCRIPT_NAME'}, 'abs' )
        . '/teaser'
        . $this->{'teaser_no'};
    if ( defined $env::ssi && $env::ssi ) {
        print
            '<fieldset class="sample"><legend><label for="sample-ssi">埋め込みタグ(静的 SSI)</label></legend>';
        print '<textarea cols="70" rows="3">&lt;!--#include virtual=&quot;'
            . $target
            . '.ssi&quot;--&gt;</textarea>';
        print '</fieldset>';
    }
    $target = $this->http_my_domain . $target;
    print
        '<fieldset class="sample"><legend><label for="sample-js">埋め込みタグ(静的 JavaScript)</label></legend>';
    print
        '<textarea cols="70" rows="3">&lt;script type=&quot;text/javascript&quot; language=&quot;JavaScript&quot; src=&quot;'
        . $target
        . '.js&quot;&gt;&lt;/script&gt;</textarea>';

    print '</fieldset>';
    print '<img src="'
        . $cgi::path
        . '?m=Teaser&amp;tn='
        . $this->{'teaser_no'}
        . '&amp;refresh=1&amp;nocache='
        . time
        . '" width="1" height="1" alt="" />'
        if $main::id;
    return;
}

######################################################################
#sub import {
BEGIN {
    if ( ( !$default::type || $default::type ne 'order' )
        && $default::teaser_variation )
    {
        no warnings qw(once);

        #クラス登録
        Common::class_register( 't040', 'TeaserLayout' );

        #ツール登録
        $setting::sub_jump_menu{'page'}{'t040'} = '埋め込み用リスト';
        $setting::selected_jump_menu{'t040'} = 't000'
            ; #サブメニューがあるときメインメニューのデフォルトキー
        $preference::page{'t040'} = 'teaser';

        if ( $default::teaser_variation >= 2 ) {
            foreach my $n ( 2 .. $default::teaser_variation ) {
                my $k = 't040_' . $n;
                Common::class_register( $k, 'TeaserLayout' );
                $setting::sub_jump_menu{'page'}{$k}
                    = '埋め込み用リスト' . $n;
                $setting::selected_jump_menu{$k} = 't000'
                    ; #サブメニューがあるときメインメニューのデフォルトキー
                $preference::page{$k} = 'teaser' . $n;
            }
        }
    }
}

1;
