package ViewerPage;
require CMONOS::CGI_Lib::Tools::Page;
@ISA = qw(Page);
use strict;
use utf8;
######################################################################
# テーブル表示ページ設定モジュール
# ViewerPage.pm ver1.1.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-06-04
######################################################################
# 設定項目取得
sub get_sub_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #チェックボックス
    push( @check::checkbox, 'toolboxTop', 'toolboxBottom' );

    return;
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    $preference::id = 'viewer';    # データキー
    $preference::name
        = 'テーブル表示ページレイアウト';   # 設定パネル名
    $preference::title
        = 'テーブル表示ページのレイアウトについて設定します。';
    require CMONOS::CGI_Lib::Deco;
    $this->{'icon_obj'} = Deco->new( $env::image_dir . '/tablemaker',
        $env::image_lib . '/tablemaker' );
    $setting::image_css = 'tablemaker-image.css';
}

######################################################################
# 設定チェック
sub is_valid_setting ($) {
    my $this = shift;
    $main::data{'uStatic'} = ''
        if !defined $main::data{'uStatic'} || $main::data{'uStatic'} eq '';
    if (   defined $main::data{'u'}
        && $main::data{'u'} ne ''
        && $this->is_nan( $main::data{'u'} ) )
    {
        push(
            @{ $this->{'dialog'} },
            '１ページ表示件数を更新できませんでした。半角数字にしてください。'
        );
        delete $main::data{'u'};
    }
    if (   defined $main::data{'uLimit'}
        && $main::data{'uLimit'} ne ''
        && $this->is_nan( $main::data{'uLimit'} ) )
    {
        push(
            @{ $this->{'dialog'} },
            '１ページ表示件数最大値を更新できませんでした。半角数字にしてください。'
        );
        delete $main::data{'uLimit'};
    }
    $main::data{'uLimit'} = $main::data{'u'}
        if $main::data{'u'}
        && $main::data{'uLimit'}
        && $main::data{'u'} > $main::data{'uLimit'};
    return;
}

######################################################################
# 設定パネル
sub _print_header_tool ($) {
    my $this = shift;
    return
        '<option value="&lt;!--categorylist--&gt;&lt;!--/categorylist--&gt;">カテゴリーリスト(&lt;!--categorylist--&gt;)</option><option value="&lt;!--search--&gt;&lt;!--/search--&gt;">検索フォーム (&lt;!--search--&gt;)</option><option value="&lt;!--results--&gt;&lt;!--/results--&gt;">条件にマッチしたレコードの数 (&lt;!--results--&gt;)</option>';
}

sub print_sub_preference_panel ($$$) {
    my ( $this, $id, $key ) = @_;
    $main::data{'u'} = 20 if !defined $main::data{'u'} || !$main::data{'u'};

    print '<input type="hidden" name="id" value="" />';
    print '<fieldset>';
    print '<legend>１ページ表示件数</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="data:u">デフォルトの１ページ表示件数</label></td>';
    print
        '<td class="field"><input type="number" size="3" maxlength="3" name="data:u" id="data:u" value="'
        . $this->escape( $main::data{'u'} )
        . '" /> 件 ';
    print '(<label><input type="checkbox" name="data:uStatic" value="1"'
        . ( ( defined $main::data{'uStatic'} && $main::data{'uStatic'} )
        ? ' checked="checked"'
        : '' )
        . ' />固定</label>)';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:uLimit">１ページ表示件数最大値</label></td>';
    print
        '<td class="field"><input type="number" size="3" maxlength="3" name="data:uLimit" id="data:uLimit" value="'
        . $this->escape( $main::data{'uLimit'} )
        . '" /> 件';
    print '</td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</fieldset>';

    print '<fieldset><legend>ツールボックス</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="data:'
        . $id
        . 'toolboxTop" id="data:'
        . $id
        . 'toolboxTop" value="1"'
        . (
        (   defined $main::data{ $key . 'toolboxTop' }
                && $main::data{ $key . 'toolboxTop' }
        ) ? ' checked="checked"' : ''
        )
        . ' /> ツールボックスをページ上部に表示する</label></li>';
    print '<li><label><input type="checkbox" name="data:'
        . $id
        . 'toolboxBottom" id="data:'
        . $id
        . 'toolboxBottom" value="1"'
        . (
        (   defined $main::data{ $key . 'toolboxBottom' }
                && $main::data{ $key . 'toolboxBottom' }
        ) ? ' checked="checked"' : ''
        )
        . ' /> ツールボックスをページ下部に表示する</label></li>';
    print '</ol>';
    print '</fieldset>';

    print '<fieldset><legend><label for="data:' . $id
        . 'table">テーブルタイプ</label></legend>';
    print $this->table_menu( 'data:' . $id . 'table',
        $main::data{ $key . 'table' } );
    print '</fieldset>';

    $this->print_sub_sub_preference_panel( $id, $key );
}

sub print_sub_sub_preference_panel ($$$) {
    my ( $this, $id, $key ) = @_;
    print '<fieldset><legend>レコード編集ボタン設定</legend>';
    print '<fieldset><legend>レコード編集ボタン</legend>';
    print $this->{'icon_obj'}
        ->button_menu( 'data:tmedit', 'tmedit', 120, $main::data{'tmedit'} );
    print '</fieldset>';
    print '<fieldset><legend>レコード削除ボタン</legend>';
    print $this->{'icon_obj'}->button_menu( 'data:tmdelete', 'tmdelete', 120,
        $main::data{'tmdelete'} );
    print '</fieldset>';
    print '</fieldset>';
}

######################################################################
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 't001', 'ViewerPage' );

    #ツール登録
    $setting::sub_jump_menu{'page'}{'t001'} = 'テーブル表示ページ';
    $setting::selected_jump_menu{'t001'} = 't000'
        ; #サブメニューがあるときメインメニューのデフォルトキー

    $preference::page{'t001'} = 'viewer';
}

1;
