package AjaxZipCodeSearch;
require CMONOS::CGI_Lib::AjaxSetUp;
@ISA = qw(AjaxSetUp);
use strict;
use utf8;
######################################################################
# Ajax 郵便番号検索モジュール
# ZipCodeSearch.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2008-05-28
######################################################################
# メイン
sub run ($) {
    my $this = shift;
    $this->print_response( { 'status' => 'error' } )
        if !defined $env::zipcode || !$env::zipcode;

    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';

    #---------------------------------------------------------------------
    #検索
    $this->fill( \$main::zip3, \$main::zip4, \$main::zip );
    $main::zip3 = $this->z_to_h($main::zip3) if $main::zip3 =~ /[^0-9]/;
    $main::zip4 = $this->z_to_h($main::zip4) if $main::zip4 =~ /[^0-9]/;
    $main::zip  = $this->z_to_h($main::zip)  if $main::zip =~ /[^0-9]/;
    $main::zip =~ s/[^0-9]//g;
    $main::zip3 =~ s/[^0-9]//g;
    $main::zip4 =~ s/[^0-9]//g;
    $main::zip = $main::zip3 . $main::zip4
        if $main::zip eq '' && $main::zip3 ne '' && $main::zip4 ne '';

    if ( $main::zip =~ /^[0-9]{7}$/ ) {
        my @addr_list = ();
        my %zipcode;
        tie( %zipcode, $env::dbm_file, $env::zipcode . '/zipcode',
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %zipcode ) {
            $this->mk_address_list_except_zipcode(
                $this->decode_utf8(
                    $zipcode{ $this->encode_utf8($main::zip) }
                ),
                \@addr_list
            );
        }
        else {
            $this->print_response( { 'status' => 'dbm error' } );
        }
        untie(%zipcode);
        if (@addr_list) {
            my @addr = split( /\x20/, $addr_list[0] );
            $this->print_response(
                {   'status' => 'success',
                    'pref'   => ( ( defined $addr[0] ) ? $addr[0] : '' ),
                    'city'   => ( ( defined $addr[1] ) ? $addr[1] : '' ),
                    'street' => ( ( defined $addr[2] ) ? $addr[2] : '' )
                        . ( ( defined $addr[3] ) ? $addr[3] : '' ),
                    'building' => ( ( defined $addr[4] ) ? $addr[4] : '' )
                }
            );
        }
        else {
            $this->print_response( { 'status' => 'unknown zipcode' } );
        }
    }
    else {
        $this->print_response( { 'status' => 'query error' } );
    }
}

######################################################################
#検索にマッチした住所リスト作成
sub mk_address_list_except_zipcode ($$$) {
    my ( $this, $addr, $addr_list ) = @_;
    $addr = '' if !defined $addr;
    if ( $addr =~ /^(.*?)\(([^\(\)]+)\)(.*)$/ ) {
        my $addr1 = $1;
        my $addr3 = $3;
        my @addr  = split( /\|/, $2, -1 );
        my ($addr2);
        foreach $addr2 (@addr) {
            $this->mk_address_list_except_zipcode( $addr1 . $addr2 . $addr3,
                $addr_list );
        }
    }
    elsif ( $addr =~ /\|/ ) {
        my ( @addr, $addr1 );
        $addr =~ s/[\(\)]//g;
        @addr = split( /\|/, $addr, -1 );
        foreach $addr1 (@addr) {
            $this->mk_address_list_except_zipcode( $addr1, $addr_list );
        }
    }
    elsif ( $addr =~ /^(.*?)\[([0-9]+)\-([0-9]+)\](.*)$/ ) {
        my $addr1 = $1;
        my $d1    = $2;
        my $d2    = $3;
        my $addr2 = $4;
        if ( $d2 - $d1 > 50 ) {
            $this->mk_address_list_except_zipcode( $addr1, $addr_list );
        }
        else {
            my $d;
            foreach $d ( $d1 .. $d2 ) {
                $this->mk_address_list_except_zipcode( $addr1 . $d . $addr2,
                    $addr_list );
            }
        }
    }
    else {
        $addr =~ s/[\x0A\x0D]//g;
        push( @$addr_list, $addr ) unless grep( $_ eq $addr, @$addr_list );
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #設定DB読み込み設定
    $setting::general  = 1;    #一般設定読み込み
    $setting::security = 1;    #アクセス権設定読み込み

    #設定種キー設定
    $setting::security_key
        = 'search';            #デフォルトセキュリティキー
    $setting::key = 'search';  #ページ設定キー

    #クッキー
    $cookie::id_enabled = 1;    #ID情報クッキー読み込み

    #デフォルトキー
    $main::k = 'asearch' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'asearch', 'AjaxZipCodeSearch' );
}

1;
