package Common;
use Fcntl qw(:flock);
use strict;
{ use bytes; }
use utf8;
use Jcode;
use File::Path;
$Jcode::FALLBACK = &Jcode::FB_XMLCREF if defined $Jcode::FALLBACK;
use FileHandle;
STDOUT->autoflush(1);
######################################################################
# フォームから送られてきたデータをデコード・整形・修正するモジュール
# Common.pm ver1.9.13
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
# 注意：Jcode.pm[Copyright 1999-2005 Dan Kogai <dankogai@dan.co.jp>]が必要です。
# 参照：RFC1867（フォームアップロードの仕様書）
# 最終更新：2020-09-16
######################################################################
#
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	(0)-1 obj = Common->new(key);
#		オブジェクトを作ります。
#		オブジェクトを作り、(0)-2で登録されたキーワード(key)に対応するクラスに結合します。
#			require CMONOS::CGI_Lib::Common;
#			obj = Common->new(key);
#
#	(0)-2 Common::class_register('key','子クラスのパッケージ名')
#		クラスを登録します。子クラス側でキーワードとクラス名を登録することで、子クラスにオ
#		ブジェクトを結合させることができます。
#			例）子クラスのimportで、ClassRegisterを呼び出します。
#			sub import {
#				Common::class_register('key','子クラスのパッケージ名');
#			}
#
#	(0)-3 Common::use_modules(dir,\@use,\@ignore,\required);
#		指定されたディレクトリ（dir）内にあるモジュールをuseします。
#			dir			指定ディレクトリの@INCからのパス
#			read		use するモジュールの配列のリファレンス
#			ignore		無視するモジュールの配列のリファレンス
#			required	最低限のモジュールのHASHのリファレンス
#						※上記三つの条件が無指定の場合全てのモジュールをuseします）
#
# ---------------------------------------------------------------------------------------
#	(1) $text = obj->url_encode(text,option);
#		textにURLエンコーディングを施します。
#		optionを '1' にすると、&と=をエンコードしません。
#
#	(1)-2 obj->set_query(type,\@required)
#			type			url = URLエンコード/post = hiddenタグ
#			required		必須変数
#
#	(1)-3 obj->add_query(type,\%hash)
#			%hash		変数名と値の組み
#
#	(1)-4 obj->url_encoded_query(\%hash)
#			%hash		変数名と値の組み
#
# ---------------------------------------------------------------------------------------
#	(2) @query = obj->get_query;
#		GETメソッドで送信された要求をデコードします。
#		例）query.cgi?test1&test2&test3 -> @data = (test1,test2,test3)
#
# ---------------------------------------------------------------------------------------
#	(3) %in|@order = obj->decode([option[,jcode[,r[,ignore]]]]);
#		データをデコードし、keyが変数名、valueが送られてきた値のハッシュを返します。
#		ファイルのアップロードの場合、アップロードされたファイルの名前のみ格納します。
#		同じ変数名に二つ以上の値があるときは"\0"で区切られます。
#		@{obj->{'query_order'} に変数が送られてきた順に変数名を収めます。
#
#			option	=1	データをデコードし、コールしたパッケージにおいて、シンボリック名を名前
#						にした変数に、送信されたデータを格納します。（例：obj->decode(1)）
#						変数名に二つ以上の値があるときは配列に格納されます。
#						変数名に二つ以上の値があるときは配列に格納されます。
#						@{obj->{'query_order'} は作成されません。
#
#						例）	<input type="hidden" name="name" value="value" />
#								->$name = value / @name = (values)
#					=0	変数の順序（@order）を返します。
#
#			jcode		utf8/jis/sjis/euc
#			r			lf/cr/crlf
#			ignore		無視するシンボリック名を指定します。（optionが"1"のときのみ有効）
#
#			例）obj->decode('1','master_password','master');
#			この例では'master_password'および'master'という名前で送られてきた値を無視します。
#
#			変数名を":"で区切ると構造体ハッシュに値を格納します。
#			例）"name:key1:key2"->$name{'key1'}{'key2'}
#
#			変数名を"^"で区切ると多次元配列に値を格納します。
#			例）"name^0^1"->$name[0][1]
#
#			変数名を"="で区切ると、"="の前を変数名、後ろを値として処理します。本来の値は無
#			視されます。また、本来の値が空の場合、この処理は行われません。また、"&"で複数
#			の変数を設定することができます。（optionが"1"のときのみ有効）
#			例）"name1=value1&name2=value2"->$name1 = "value1"/$name2 = "value2"
#
#			※ ":"と"^"を同時に使用することはできません。
#			※ 変数名に含まれる、上記オプションと英数字+"_"以外の文字は"_"に変換されます。
#			※ mainパッケージから呼び出されたときに限り、大文字のみの変数名をスキップします。
#				特殊変数の書き換えを防止するための制限です。
#
# ---------------------------------------------------------------------------------------
#	(4) obj->save_all_files(name,\@mtype,limit,dir);
#		アップロードされた全てのファイルを指定されたディレクトリに名前を付けて保存します。
#			例）
#			obj->save_all_files(
#				"pic",
#				["image/gif:gif","text/plain:txt","image/jpeg:jpg"],
#				50000,
#				"./user/dir/",
#			);
#
#		obj->save_file(vname,name,\@mtype,limit,dir);
#		アップロードされた全てのファイルを指定されたディレクトリに名前を付けて保存します。
#			例）
#			obj->save_file(
#				'upload',
#				"pic",
#				["image/gif:gif","text/plain:txt","image/jpeg:jpg"],
#				50000,
#				"./user/dir/",
#			);
#
#		vname		保存するファイルのシンボリックネーム。
#					上の例だと<input name="upload" type="file" />。
#		limit		ファイルの大きさの上限を設定します（byte）。
#					必ず1000以上にして下さい。'i'で制限なし。
#		dir		ファイルを保存するディレクトリを設定します。ディレクトリが存在しない場合、
#					指定された名前でパーミッション777のディレクトリを自動作成し、その中にファ
#					イルを保存します。ディレクトリは相対パスで指定して下さい。それ以外の場合
#					は、@{obj->{'dialog'}}にエラーメッセージを格納します。ファイルの保存は
#					実行されません。
#		name		"<client>" -->送られてきたファイル名に準じて名前を付けます。同じ名前があ
#					　　　　　　　れば上書きします。
#					"<vname>" -->シンボリックネームと同じ名前を付けます。同じ名前があれば上書
#					　　　　　　 きします。
#					"<time>" -->現在時刻に基づいたファイル名を付けます。現在時刻の後にファイ
#					　　　　　　ルの重複を避けるため、2桁の数字を追加します。
#					　　　　　　ファイル名から最後の2桁を取り除くと保存時刻が取り出せます。
#					　　　　　　拡張子の指定がない場合、拡張子はオリジナルファイルに準じます。
#					　　　　　　例）2000年12月15日10時31分23秒に送信されたファイル
#					　　　　　　　　->2000121510312300.txt
#					"name" -->指定ディレクトリ内に同じ名前のファイルができないように番号をつ
#					　　　　　けます。例）name2.txt
#					"!name" -->頭に「!」をつけると、指定ディレクトリ内に同じ名前のファイルが
#					　　　　　 あれば上書きします。例）name.txt
#		mtype		受け付けるファイルのMIME-TYPEを指定します。どんなものも受け付けるときは、
#					指定しないで下さい。
#					MIME-TYPEの後に拡張子も指定してください。
#					------------------------------------見本----------------------------------
#					'text/plain:txt','text/html:html','image/gif:gif','image/jpeg:jpg',
#					'image/pjpeg:jpg','audio/x-midi:mid',
#					'video/mpeg:mpg','video/quicktime:mov','video/x-msvideo:avi',
#					'application/x-shockwave-flash:swf','application/x-director:dcr'
#					------------------------------------見本----------------------------------
#					拡張子が複数ある時はどれか一つにするか指定しないで下さい。
#					指定がないときはアップロードファイルの拡張子に準じます。
#					'image/pjpeg:jpg'は、プログレッシブJPEGです。
#
#					■text/plain,text/htmlのオプション：MIME-TYPE:拡張子:jcode:lf
#					-->'text/plain:txt:utf8:lf','text/html:html'
#					jcodeを指定すると指定された漢字コードに変換して保存します。（utf8,jis,sjis,euc）
#					"lf"をつけると改行コードをLFに変換して保存します。
#
#		▼その他の機能
#		エラーがあった場合、@{obj->{'dialog'}}にエラーメッセージを出力します。
#
#		▼マックからのアップロードについて
#		マックバイナリで送られてきたファイルは制限容量以内ならどんなものでも受け付けます。
#		その場合、ファイルの種類をオリジナルの拡張子から判断します。
#
# ---------------------------------------------------------------------------------------
#	(5) obj->tag_fix(html[,br[,noembed[,nostyle[,p[,nofollow]]]]]);
#		クライアントによって入力されたHTML文書のタグの不正を修正します。
#		$htmlにタグの閉じ忘れがあった場合、不足している開始タグ終了タグを追加ます。
#			html		リファレンス => オリジナルを修正します。
#						変数 => 修正文字列を返します。
#			br			1 => 改行を<br />に変換します。
#						2 => 改行を<br />に変換し、改行コードを全て削除します。
#			noembed		1 => IMGやEMBED等外部ファイルを埋め込むタグを消去
#			nostyle		1 => CSSを消去
#			p			1 => 改行の連続を<p>に変換する
#			nofollow	1 => リンク先を検索エンジンに無視させる
#
# ---------------------------------------------------------------------------------------
#	(6) obj->tag_fix_multi(br,noembed,nostyle,p,nofollow,@html);
#		複数のhtmlテキストのタグを修正します。
#			@html		リファレンス => オリジナルを変換します。
#						変数 => 変換文字列を納めた配列を返します。
#
# ---------------------------------------------------------------------------------------
#	(7) obj->detag(@html);
#		htmlのタグを除去します。
#			@html		リファレンス => オリジナルを変換します。
#						変数 => 変換文字列を納めた配列を返します。
#
# ---------------------------------------------------------------------------------------
#	(8)-1-1 obj->escape(@html);
#		$htmlのタグにエスケープを施します。
#			@html		リファレンス => オリジナルを変換します。
#						変数 => 変換文字列を納めた配列を返します。
#
#	(8)-1-2 obj->escape_except_html_entity(@html);
#		$htmlのタグにエスケープを施します。HTMLエンティティはエスケープしません
#			@html		リファレンス => オリジナルを変換します。
#						変数 => 変換文字列を納めた配列を返します。
#
#	(8)-2 obj->unescape(@html);
#		$htmlのタグにエスケープを施します。
#			@html		リファレンス => オリジナルを変換します。
#						変数 => 変換文字列を納めた配列を返します。
#
# ---------------------------------------------------------------------------------------
#	(9)-1-1 obj->autolink_multi(\@html,nofollow,protect,getTitle,target,target2);
#		URLおよびメールアドレスを自動判別し、リンクを張ります。
#		例）<a href="url" target="_blank">url</a>
#
#			@html		リファレンス => オリジナルを変換します。
#			protect		1=メールアドレスを JavaScript + HTMLエンティティ で保護する。
#			nofollow	1=検索エンジンにリンクをたどらせないよう指示する。
#			getTitle	1=ページタイトルを取得する。
#			target		表示ウィンドウを指定します。無指定（''）の場合、_blankになります。
#							'_blank'-->新しいウィンドウに表示
#							'_self'-->現在のウィンドウ・フレームに表示
#							'_parent'-->親フレームに表示
#							'_top'-->現在のウィンドウ全体に表示
#			target2		ユーザーページ内HTMLへのリンクの表示ウィンドウを指定します。
#
#	(9)-1-2 obj->autolink(html,nofollow_flag,target,target2);
#		URLおよびメールアドレスを自動判別し、リンクを張ります。
#		例）<a href="url" target="_blank">url</a>
#
#			html		リファレンス => オリジナルを変換します。
#						変数 => 変換文字列を返します。
#
#	(9)-1-3 obj->delink(html);
#		自動リンクされた URL とメールアドレスを元に戻します。
#			html		リファレンス => オリジナルを変換します。
#						変数 => 変換文字列を返します。
#
#	(9)-2-1 obj->autolist_multi(@html);
#		リストに変換します。
#			@html		リファレンス => オリジナルを変換します。
#						変数 => 変換文字列を納めた配列を返します。
#
#	(9)-2-2 obj->autolist(html); #TagFix より前にやること
#		リストに変換します。
#			html		リファレンス => オリジナルを変換します。
#						変数 => 変換文字列を返します。
#
#	(9)-2-3 obj->delist(html);
#		リストに変換されたテキストを元に戻します。
#			html		リファレンス => オリジナルを変換します。
#						変数 => 変換文字列を返します。
#
# ---------------------------------------------------------------------------------------
#	(10) obj->is_invalid_url($url);
#		$urlがURLの書式通りかどうか判定します。書式通りでない場合、「1」を返します。
#
# ---------------------------------------------------------------------------------------
#	(11)-1 obj->is_invalid_email_address($mail_address);
#		$mail_addressがメールアドレスの書式通りかどうか判定します。
#		書式通りでない場合、「1」を返します。
#
#	(11)-2 obj->is_invalid_email_address_header($mail_address);
#		$mail_addressがメールアドレスの書式（名前 <name@domain>,...）通りかどうか判定します。
#		書式通りでない場合、「1」を返します。
#
#	(11)-3 obj->get_address(@addr);
#		@addrからメールアドレスのみ取り出します。
#
# ---------------------------------------------------------------------------------------
#	(12) obj->is_invalid_image($image,option);
#		$imageが画像のURLであるかどうか判定します。末尾が「.jpg」か「.gif」でない場合、
#		「1」を返します。optionを'full'としておくと、画像のURLが書式通りに絶対パスで指定さ
#		れているかどうか判定します。
#
# ---------------------------------------------------------------------------------------
#	(13) obj->client_ip_is_matched(@ip);
#		クライアントのIPアドレスが指定されたIPアドレス（@ip）とマッチするかどうかを判定しま
#		す。指定されたIPアドレスにマッチするとき"1"を返します。
#
# ---------------------------------------------------------------------------------------
#	(14) obj->referer_is_matched(referer,jump_to,op);
#		referer以下の階層からの接続でない場合、jump_toに移動させます。
#			referer		直前URL（設定の無い場合はユーザー領域のトップページ）
#			jump_to		移動先URL（設定の無い場合はユーザー領域のトップページ）
#							error		Error404メッセージを表示
#							no			判定のみ（"0"を返して何もしません）
#			op				"1"=直前のURLを取得できない場合接続を許可
#
# ---------------------------------------------------------------------------------------
#	(15) obj->is_invalid_color($color);
#		$colorが色指定の書式通りかどうか判定します。書式通りでない場合、「1」を返します。
#
# ---------------------------------------------------------------------------------------
#	(16)-1 obj->is_nan(num);
#		numが半角数字かどうか判定します。半角数字でない場合、"1"を返します。
#		全角数字は半角数字に変換されます。
#
#	(16)-2 obj->is_not_word(text);
#		textが半角英数字かどうか判定します。半角数字でない場合、"1"を返します。
#		全角英数字は半角数字に変換されます。
#
#	(16)-3 obj->is_invalid_password(text[,offset,limit,option]);
#		textが半角英数字かどうか判定します。半角英数字でない場合、"1"を返します。
#		全角英数字は半角数字に変換されます。
#			offset		パスワードに必要な文字数
#			limit		パスワード文字数制限
#			option		1=英字で始まる半角英数字
#
#	(16)-4 obj->is_not_hiragana(text);
#		textがひらがなかどうか判定します。ひらがなでない場合、"1"を返します。
#
#	(16)-5 obj->is_not_katakana(text);
#		textがカタカナかどうか判定します。カタカナでない場合、"1"を返します。
#		半角カタカナは全角に変換されます。
#
# ---------------------------------------------------------------------------------------
#	(17) obj->jump($url,option);
#		$urlに自動ジャンプさせCGIプログラムを終了します。obj->set_cookie()が施してある場合、
#		Set-Cookieヘッダも出力します。optionを'no'にすると、プログラムを終了しません。
#
# ---------------------------------------------------------------------------------------
#	(18) obj->print_http_header(type);
#		ファイルヘッダ部分をprintします。
#		事前にobj->set_cookie()が施してある場合、Set-Cookieヘッダも作成します。
#			type		Content-Typeの設定（設定の無い場合は"text/html"になります。）
#
#	(18)-2 obj->print_html_header(handle);
#		ファイルヘッダ(PrintHTTPHeader)とHTMLヘッダを出力します。
#		事前にobj->set_cookie()が施してある場合、Set-Cookieヘッダも作成します。
#		(16)-3..(16)-7の設定が反映されます。
#			handle	出力先ハンドル（設定の無い場合はSTDOUTになります。）
#
#	(18)-3 obj->set_alert(@alert);
#		JavaScript:alertの設定をします。
#
#	(18)-4 obj->set_title(title);
#		<title>の設定をします。
#
#	(18)-5 obj->set_body(
#				'text' => '#000000',
#				'link' => '#0000ff',
#				'alink' => '#00ffff',
#				'vlink' => '#000099',
#				'bgcolor' => '#ffffff',
#				'background' => 'background.gif',
#				'bgproperties' => 'fixed',
#				'topmargin' => '0',
#				'leftmargin' => '0'
#			 );
#		<body>の設定をします。
#
#	(18)-6 obj->set_javascript(javascript);
#		JavaScriptを設定します。
#
#	(18)-7 obj->set_css(CSS[,link,alink,vlink,hover,dec]);
#		CSSを設定します。
#			css		スタイルシート
#			link		リンク色
#			alink		アクティブリンク色
#			vlink		訪問済みリンク色
#			hover		マウスオーバーリンク色
#			dec		no=下線なしリンク
#
#	(18)-8 obj->set_style_sheet_link(path);
#			path		スタイルシートファイルのパス
#
#	(18)-9 obj->set_base(href);
#			href		ベースURLの設定
#
#	(18)-10 obj->SetHTMLHeder(header);
#			header	上記以外の追加ヘッダ
#
#	(18)-11 obj->ResetHTMLHeder;
#			ヘッダのクリア
#
# ---------------------------------------------------------------------------------------
#	(19)-1 presenttime = obj->present_time(op,type,time);
#		op		時刻表示の範囲（year/mon/mday/wday/hour/min/sec）例）'year-min'
#		type	時刻表示のタイプ
#				'/' -> '2000/12/25/(月) 10:23:55'
#				'c' -> '2000年12月25日(月) 10:23:55'
#		time	設定されたtime（GMT）の時刻を返します。
#
#		日本語表記の現在時刻を返します。
#		現在時刻は、year年mon月mday日(wday) hour : min
#
#	(19)-2 timeID = obj->time_id(time);
#		time	設定されたtime（GMT）を元に時間IDをを作成します。
#
#		時間IDは、年・月・日・時・分・秒を四桁と二桁の数字に変換した数になります。
#		例）1999年1月1日9時30分58秒　-->    $time_ID = '19990101093058'
#
#	(19)-3 day = obj->date([year,mon,mday[,hour,min,sec]],limit,type,format);
#	       day = obj->date('YearMonMday[HourMinSec]',limit,type,format);
#		現在からlimit日以内の時間を「あせって」「あす」「きょう」「きのう」「おとつい」
#		「○日後」「○日前」と表示します。配列の他、時間IDからも相対日時を割り出すことができ
#		ます。
#
#		type	時刻表示のタイプ
#				'-'		->	'2005-11-25 13:23:55'
#				'v'		->	'2005/15/25T13:23:55Z'
#				'/'		->	'2005/11/25/(金) 1:23:55 p.m.'
#				'/24'	->	'2005/11/25/(金) 13:23:55'
#				'c'		->	'2005年11月25日(金) 午後1時23分55秒'
#				'c24'	->	'2005年11月25日(金) 13時23分55秒'
#				'cj'	->	'平成17年11月25日(金) 午後1時23分55秒'
#				'c24j'	->	'平成17年11月25日(金) 13時23分55秒'
#				'cjk'	->	'平成一七年一一月二五日(金) 午後一時二三分五五秒'
#				'c24jk'	->	'平成一七年一一月二五日(金) 一三時二三分五五秒'
#				'cJk'	->	'平成一七年霜月二五日(金) 午後一時二三分五五秒'
#				'c24Jk'	->	'平成一七年霜月二五日(金) 一三時二三分五五秒'
#
#	(19)-4 obj->is_impossible_mday(year,mon,mday);
#		入力された年月日が存在するかどうかチェックします。存在しない場合1を返します。
#			year	年（4桁）
#			mon		月[1-12]
#			mday	日
#
#	(19)-5 obj->kansuuji(option,@num);
#		数字を漢字に変換します。
#			option	k=一二三/K=壱弐参
#			num		数字
#
#	(19)-5 obj->nengo(year,mon,mday);
#		西暦を明治から平成までの年号に変換します。
#			year	年（西暦4桁）
#			mon		月（0-11）
#			mday	日（01-31）
#
#	(19)-5 obj->tsuki(@mon);
#		月を陰暦月名に変換します。
#			mon		月（0-11）
#
# ---------------------------------------------------------------------------------------
#	(20)-1 obj->set_cookie(limit,cname,domain,path,\%data);
#		Set-Cookieヘッダを返します。このcookieはlimit日後に消去されます。
#		obj->{'Set_Cookie'} にSet-Cookieヘッダを格納。
#
#		limit		Cookie期限（日数単位）／設定無し=そのセッションのみで消える）
#		cname		Cookieの名前
#		domain	Cookieを設定するドメイン
#		path		Cookieを設定するパス（相対パスでもOK）
#		%data		Cookieに記憶させるデータのハッシュ
#
#	(20)-2 $expires = obj->get_expires(limit);
#		クッキー消去期日（limit日後に消去する場合の値）を返します。
#
# ---------------------------------------------------------------------------------------
#	(21)@data = obj->read_cookie(cname);
#		URLエンコード形式で送られたクッキーデータをmainパッケージで設定します。
#		既にmainパッケージに同名の変数が存在する場合、Cookieデータは無視されます。
#		Cookieが読み込まれたときは、$cookie::read{'cname'} = 1 となります。
#
# ---------------------------------------------------------------------------------------
#	(22) obj->my_url;
#		CGIまでのURLを返します。例）http://www.xxx.ne.jp/cgi-bin/xxx/xxx.cgi
#
# ---------------------------------------------------------------------------------------
#	(23) host_name = obj->to_host(ip);  /  ip = obj->to_ip(host_name);
#		host_name = obj->to_host(ip);　　IPアドレスをHOSTネームに変換します。
#		ip = obj->to_ip(host_name);　　HOSTネームをIPアドレスに変換します。
#		ただし、常に変換に成功するとは限りません。
#
# ---------------------------------------------------------------------------------------
#	(24) $url = obj->path_conv(path[,op]);  サーバ内パスとURLパスを変換します。
#		また、$cgi::user_directory にホームディレクトリまでのURLパスを、
#		$cgi::document_root にホームディレクトリまでのサーバ内パスを、格納します。
#
#		path		変換するパス
#		op			'url'=必ずURLに変換／'path'=必ずPATHに変換
#
#		例）"/~nobody/test/test.html"→"/user/home/public_html/test/test.html"
#		　　"/user/home/public_html/test/test.html"→"/~nobody/test/test.html"
#		　　$cgi::user_directory = "/~nobody"
#		　　$cgi::document_root = "/user/home/public_html"
#
# ---------------------------------------------------------------------------------------
#	(25)-1 $url = obj->path_change($path,baseurl[,op]);
#		baseurlから見て相対パスと絶対パスを変換します。
#		baseurlは絶対パスで指定されなければなりません。
#		baseurlが設定されていないときはSCRIPT_NAMEを軸に変換を行います。
#
#		path		変換するパス
#		baseurl		起点
#		op			'abs'=必ず絶対パスに変換／'rel'=必ず相対パスに変換
#
#
#		例）"/a/b/c/test.html"をbaseurlにした場合
#		　　"/a/d/e/test2.html"→"../../d/e/test2.html"
#		　　"../f/g/test2.html"→"/a/b/f/g/test2.html"
#
#	(25)-2 $url = obj->base_change(path,baseurl);
#		CGIからの相対パス（path）をbaseurlから見た相対パスに変換します。
#
#	(25)-3 $url = obj->un_base_change(path,baseurl);
#		baseurlからの相対パス（path）をCGIから見た相対パスに戻します。
#
# ---------------------------------------------------------------------------------------
#	(26) $text = obj->z_to_h($text,jcode);
#		全角英数字を半角に変換します。jcodeオプション(utf8/jis/sjis/euc)で漢字コードの種類を指
#		定してください。例）$text = obj->z_to_h($text,'utf8');
#
# ---------------------------------------------------------------------------------------
#	(27)-1 $text = obj->jfold(text,width[,lf,row,punc,quot]);
#		指定された文字数（半角換算）に桁揃えします。
#
#		text		桁揃えするテキスト
#		width		半角換算桁数（設定無しの場合は"70"に設定されます）
#		lf			改行を残すか残さないか
#					（i=残さない/r=改行の後空白で埋めない/設定無し=改行を残し空白で埋める）
#		row			行数制限（指定された行数を超えるテキストを削除します）
#		punc		区切り文字（設定無しの場合は"\x0A"に設定されます）
#		quot		引用符（設定された引用符を行頭に追加します）
#
#	(27)-2 $text = obj->cut_string(text,length,tag);
#		指定された文字数（半角換算）で文字列を切ります。
#
#		text		対象となるテキスト
#		length		文字数
#		tag			HTML タグの扱い（no=タグを除去する/i=タグを無視する）
#
#	(27)-3 $text = obj->quotation_color(text,@color);
#		引用部分を着色します。
#
#		text		引用部分を着色するテキスト
#		@color		色指定（Res[1],Res[2],Res[3]...）
#
# ---------------------------------------------------------------------------------------
#	(28) csv等を分割・結合します。
#
#		@array = obj->svsplit(w,text);
#			w		区切り文字
#			text	一行分のcsvデータ等（""で囲まれた区切り文字はエスケープされます）
#
#		$text = obj->svjoin(w,@array);
#			w			区切り文字
#			@array	区切り文字で結合したい文字（区切り文字のある要素は""で囲まれます）
#
#		例）csvの場合：@array = obj->svsplit(',',text);　$text = obj->svjoin(',',@array);
#		例）tsvの場合：@array = obj->svsplit("\t",text);　$text = obj->svjoin("\t",@array);
#
# ---------------------------------------------------------------------------------------
#	(31) obj->fileglob(dir,pattern,type,find,unmatched,skipdir);
#		ディレクトリをスキャンしてパターンと一致するファイル/ディレクトリのリストを返します。
#			dir			ディレクトリ
#			pattern		検索パターン
#			type		d=ディレクトリ/f=ファイル
#			find		1=ディレクトリを下る
#			unmatched	1=パターンに一致しないリストを返させる
#			skipdir		ディレクトリを下る際スキップするディレクトリ名を指定します。
#
# ---------------------------------------------------------------------------------------
#	(31) obj->chmod_dir(mode,path);
#		パスの親ディレクトリのパーミッションを設定します。
#
# ---------------------------------------------------------------------------------------
#	(32)-1 obj->file_copy(file,copy);
#		ファイル（file=オリジナルのパス）を複製（copy=複製先パス）します。
#
#	(32)-2 obj->directory_copy(dir,copy,find,\@ignore,maxsize);
#		ディレクトリ（dir=オリジナルのパス）を複製（copy=複製先パス）します。
#			find		1=dirに含まれるディレクトリも複製する
#			@ignore		複製しない条件（正規表現可）
#			maxsize		ファイルサイズがこれ以上大きいときはコピーしない
#
# ---------------------------------------------------------------------------------------
#	(33)-1 obj->minimum(\%hash);
#		最小値のキーを返します。
#
#	(33)-2 obj->maximum(\%hash);
#		最大値のキーを返します。
#
# ---------------------------------------------------------------------------------------
#	(34) obj->new_number(\%hash[,start]);
#		数値をキーにしたハッシュの新規キーを返します。
#
#	(34)-2 obj->new_number(\@array[,start]);
#		数値の配列の空いている数値を返します。
#
# ---------------------------------------------------------------------------------------
#	(35) obj->sort_by(\$a,\$b,sort_order);
#		ソートに使います。
#			sort_order		ascend=昇順/descend=降順
#
#		例）
#		foreach (
#			sort {$this->sort_by(\$hash{$a},\$hash{$b},'ascend') || $this->sort_by(\$a,\$b,'ascend')} @array
#		) {...
#
# ---------------------------------------------------------------------------------------
#	(36) obj->digit(str[,s,n]);
#		str 中の数列をn桁ごとに、sで位取りします。
#
# ---------------------------------------------------------------------------------------
#	(37) obj->error404;
#		404 Not Found エラーメッセージを出力しプログラムを終了します。optionを'no'にすると
#		プログラムを終了しません。不正なアクセスに対してCGIの存在をカモフラージュするのに
#		便利です。
#
# ---------------------------------------------------------------------------------------
#	(38) @msg = obj->dialog;
#		Common.pmの処理で発生したエラーのエラーメッセージを返します。
#		JavaScriptのalertを意識し改行は'\n'となっています。
#
######################################################################
#	■利用規約
#	このスクリプトの著作権はCMONOSが保有しています。
#	このサブルーチン集は、以下の条件に従うかぎり自由に再配布できます。
#		・著作権表示を変更してはいけません。
#		・修正を加えた場合は、修正者名・修正者の連絡先・修正個所・修正日
#		　時を明記して下さい。
#	このスクリプトを利用したことによって生じた、いかなる損害にも
#	CMONOSは責任を負いません。
######################################################################
if ( !ref $cgi::Jcode ) {
    $cgi::Jcode = Jcode->new;
}

#オブジェクト作成
sub new ($;$) {
    my ( $pkg, $key ) = @_;
    my $this = {};
    if (   defined $key
        && defined $Common::modules{$key}
        && $Common::modules{$key} ne '' )
    {
        bless $this, $Common::modules{$key};
    }
    else {
        bless $this;
    }
    $this->fix_env;
    $this;
}

sub fix_env ($) {
    my $this = shift;
    $env::server_name = $ENV{'SERVER_NAME'} || $ENV{'HTTP_HOST'}
        if !$env::server_name;
    $env::ssl
        = ( $ENV{'HTTPS'}
            || ( $ENV{'SERVER_PORT'} && $ENV{'SERVER_PORT'} == 443 ) )
        ? 1
        : 0
        if !defined $env::ssl || $env::ssl eq '';
    $env::file_mode     = 0644                   if !$env::file_mode;
    $env::dir_mode      = 0755                   if !$env::dir_mode;
    $env::cgi_file_mode = 0755                   if !$env::cgi_file_mode;
    $env::css           = '../../lib/css'        if !$env::css;
    $env::javascript    = '../../lib/javascript' if !$env::javascript;
    $env::home_dir_id      = ''            if !defined $env::home_dir_id;
    $env::clipboard        = './clipboard' if !$env::clipboard;
    $env::public_clipboard = './clipboard' if !$env::public_clipboard;
    $env::buffer_size      = 8192          if !$env::buffer_size;
    $env::post_max_size    = 100000000     if !$env::post_max_size;
    return $this;
}

#クラス登録
sub class_register ($$) {
    my ( $key, $pkg ) = @_;
    $Common::module_count++
        if !defined $Common::modules{$key} || $Common::modules{$key} eq '';
    $Common::modules{$key} = $pkg;
}

#指定ディレクトリ内のモジュールをuse
sub use_modules ($;$$$) {
    my ( $dir, $use_m, $ignore, $required ) = @_;
    return 0 if !defined $dir || $dir eq '';
    my ( @modules, $lib, $open, $m );
    local (*DIR);
    foreach $lib (@INC) {
        if ( $lib && opendir( DIR, "$lib/CMONOS/$dir" ) ) {
            $open++;
            last;
        }
    }
    return 0 if !$open;
    @modules = grep( $_ =~ s/\.pm$//, readdir(DIR) );
    closedir(DIR);
    return 0 if !@modules;
    $dir .= '/';
    $dir =~ s!/!::!g;
    foreach $m (@modules) {
        next
            if ( ref $use_m && @$use_m && !grep( $_ eq $m, @$use_m ) )
            || ( ref $ignore && @$ignore && grep( $_ eq $m, @$ignore ) )
            || ( ref $required && %$required && !$$required{$m} );
        eval 'require CMONOS::' . $dir . $m;
    }
    return 1;
}

#初期化
sub fill ($;@) {
    my $this = shift;
    foreach (@_) {
        next if !ref $_;
        $$_ = '' if !defined $$_;
    }
}

sub dfill ($;@) {
    my $this = shift;
    foreach (@_) {
        next if !ref $_;
        $$_ = 0 if !defined $$_ || $$_ eq "";
    }
}

sub scalar_to_array ($$) {
    my ( $this, $scalar ) = @_;
    if ( ref $scalar && ( !ref $$scalar || ref $$scalar ne 'ARRAY' ) ) {
        my $temp = ( defined $$scalar ) ? $$scalar : undef;
        undef $$scalar;
        $$scalar = [];
        push( @{$$scalar}, $temp ) if defined $temp;
    }
}

sub ename_to_jname ($$) {
    my ( $this, $ename ) = @_;
    return 'utf8' if !$ename;
    $ename = lc $ename;
    my %ename2j = (
        'shiftjis'    => 'sjis',
        'x-sjis'      => 'sjis',
        'shift_jis'   => 'sjis',
        'euc-jp'      => 'euc',
        'x-euc-jp'    => 'euc',
        '7bit-jis'    => 'jis',
        'x-jis'       => 'jis',
        'iso-2022-jp' => 'jis',
        'utf-16be'    => 'ucs2',
        'utf-8'       => 'utf8',
    );
    if ( $ename2j{$ename} ) {
        return $ename2j{$ename};
    }
    else {
        $ename =~ tr/-/_/;
        return $ename;
    }
}

######################################################################
#エンコード
sub url_encode ($$;$) {
    my ( $this, $text, $op ) = @_;
    return '' if !defined $text || $text eq '';
    utf8::encode($text) if utf8::is_utf8($text);
    if ( defined $op && $op ) {
        $text
            =~ s/([\x00-\x1F\x21-\x25\x27-\x29\x2B\x2C\x2F\x3A-\x3C\x3E\x3F\x5B-\x5E\x60\x7B-\xFF])/'%' . sprintf("%.2X", unpack("C", $1))/eg;
    }
    else {
        $text
            =~ s/([\x00-\x1F\x21-\x29\x2B\x2C\x2F\x3A-\x40\x5B-\x5E\x60\x7B-\xFF])/'%' . sprintf("%.2X", unpack("C", $1))/eg;
    }
    $text =~ s/\x20/+/g;
    return $text;
}

#	(1)-2 obj->set_query
sub set_query ($$$;$) {
    my ( $this, $type, $required, $additional ) = @_;
    return '' if !ref $required || !@$required;
    my ( $key, $query, $has_hint );
    no strict "refs";
    no warnings qw(numeric);
    $query = '';
    foreach $key (@$required) {
        if ( $key eq '_' ) {
            $has_hint = 1;
            next;
        }
        if ( @{ 'main::' . $key } ) {
            foreach
                my $v ( sort { $a <=> $b || $a cmp $b } @{ 'main::' . $key } )
            {
                $query .= $this->make_query( $type, $key, $v );
            }
        }
        if ( %{ 'main::' . $key } ) {
            foreach my $k ( sort { $a <=> $b || $a cmp $b }
                keys %{ 'main::' . $key } )
            {
                $query .= $this->make_query(
                    $type,
                    $key . ':' . $k,
                    ${ 'main::' . $key }{$k}
                );
            }
        }
        if ( defined ${ 'main::' . $key } ) {
            $query .= $this->make_query( $type, $key, ${ 'main::' . $key } );
        }
    }
    if ( $type =~ /^url/i ) {
        $query = '&_=%EF%BC%BF' . $query
            if !$has_hint && $query =~ /\%[8-9a-fA-F][0-9a-fA-F]/;
        $query =~ s/^\&/\?/ if $type =~ /^url$/i && !$additional;
    }
    return $query;
}

sub make_query ($$$$) {
    my ( $this, $type, $key, $value ) = @_;
    no warnings qw(numeric);
    my $query = '';
    if ( ref $value eq 'ARRAY' ) {
        foreach my $v ( sort { $a <=> $b || $a cmp $b } @$value ) {
            $query .= $this->make_query( $type, $key, $v );
        }
    }
    elsif ( ref $value eq 'HASH' ) {
        foreach my $k ( sort { $a <=> $b || $a cmp $b } keys %$value ) {
            $query
                .= $this->make_query( $type, $key . ':' . $k, $value->{$k} );
        }
    }
    elsif ( ref $value eq 'SCALAR' ) {
        return '';
    }
    elsif ( $type =~ /^url/i ) {
        $query
            .= '&'
            . $this->url_encode($key) . '='
            . $this->url_encode($value);
    }
    else {
        $query
            .= '<input type="hidden" name="'
            . $this->escape($key)
            . '" value="'
            . $this->escape($value) . '" />';
    }
    return $query;
}

#	(1)-3 obj->add_query
sub add_query ($$$) {
    my ( $this, $type, $hash ) = @_;
    my $query = '';
    no warnings qw(numeric);
    foreach my $k ( sort { $a <=> $b || $a cmp $b } keys %$hash ) {
        $query .= $this->make_query( $type, $k, $hash->{$k} );
    }
    return $query;
}

#	(1)-4 obj->url_encoded_query
sub url_encoded_query ($$;$) {
    my ( $this, $hash, $add_hint ) = @_;
    my ($query);
    no warnings qw(numeric);
    foreach my $k ( sort { $a <=> $b || $a cmp $b } keys %$hash ) {
        $query .= $this->make_query( 'url', $k, $hash->{$k} );
    }
    if ( defined $query ) {
        $query = '&_=%EF%BC%BF' . $query
            if $add_hint && $query =~ /\%[8-9a-fA-F][0-9a-fA-F]/;
        $query =~ s/^&//;
        return $query;
    }
    else {
        return '';
    }
}

######################################################################
# 要求のデコード
sub get_query {
    my $this = shift;
    my @data = split( /&/, $ENV{'QUERY_STRING'} );
    map( $_ =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge, @data );
    return @data;
}

######################################################################
# 不正な要求の排除
sub referer_filter ($;$$$) {
    my ( $this, $method, $allow_ref, $strict ) = @_;
    if ( ( !$method || $ENV{'REQUEST_METHOD'} eq $method )
        && !$this->referer_is_me($strict) )
    {
        $this->jump( $this->my_url . $this->set_query( 'url', $allow_ref ) );
    }
}

sub referer_is_me ($;$) {
    my ( $this, $strict ) = @_;
    return ($strict) ? 0 : 1 if !$ENV{'HTTP_REFERER'};
    my $script_location  = quotemeta( $this->my_url );
    my $script_addr      = quotemeta( $this->my_url(1) );
    my $script_location2 = quotemeta(
          ( ($env::ssl) ? 'https://' : 'http://' )
        . ( $ENV{'SERVER_NAME'} || $ENV{'HTTP_HOST'} )
            . (
            ( $ENV{'SERVER_PORT'} != 80 && $ENV{'SERVER_PORT'} != 443 )
            ? ':' . $ENV{'SERVER_PORT'}
            : ''
            )
            . $ENV{'SCRIPT_NAME'}
    );
    my $script_addr2 = quotemeta(
        ( ($env::ssl) ? 'https://' : 'http://' )
        . ( ( $ENV{'SERVER_ADDR'} ) ? $ENV{'SERVER_ADDR'}
            : $this->to_ip( $ENV{'SERVER_NAME'} || $ENV{'HTTP_HOST'} )
            )
            . (
            ( $ENV{'SERVER_PORT'} != 80 && $ENV{'SERVER_PORT'} != 443 )
            ? ':' . $ENV{'SERVER_PORT'}
            : ''
            )
            . $ENV{'SCRIPT_NAME'}
    );
    return ( $ENV{'HTTP_REFERER'}
            =~ /^(?:$script_location|$script_location2|$script_addr|$script_addr2)/i
    ) ? 1 : 0;
}

sub query_filter ($;$$) {
    my ( $this, $allow_ref, $hash_ref ) = @_;
    $this->referer_filter( 'POST', $allow_ref );
    if ( $ENV{'QUERY_STRING'} ) {
        my $query = $ENV{'QUERY_STRING'};
        if ($query) {
            no strict "refs";
            while ( $query =~ /\G&?([^&=]+)=[^&=]+/g ) {
                my $k = $1;
                utf8::encode($k);
                $k =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
                $k = $1 if $k =~ /^([^=]+)=/;
                next
                    if ref $allow_ref
                    && @$allow_ref
                    && grep( $_ eq $k, @$allow_ref );
                if ( $k =~ /^([0-9a-zA-Z_]+):/ ) {
                    if ( ref $hash_ref ) {
                        delete $hash_ref->{$1};
                    }
                    else {
                        undef %{ 'main::' . $1 };
                    }
                }
                elsif ( $k =~ /^([0-9a-zA-Z_]+)\^/ ) {
                    if ( ref $hash_ref ) {
                        delete $hash_ref->{$1};
                    }
                    else {
                        undef @{ 'main::' . $1 };
                    }
                }
                else {
                    if ( ref $hash_ref ) {
                        delete $hash_ref->{$k};
                    }
                    else {
                        undef @{ 'main::' . $k };
                        undef ${ 'main::' . $k };
                    }
                }
            }
        }
    }
}

sub secure_query_for_tools_ref ($) {
    my $this = shift;
    return [ 'm', 'k', 'c', 'target', 'dir', 'id' ];
}

######################################################################
#デコード
sub decode ($;$@) {
    my ( $this, $option, $jcode, $r, @ignore ) = @_;
    my ( $stdin, %in, $input, $end, $input2, $end2, $k, $v, $boundary,
        $sub_boundary, $icode );
    $this->{'is_decoder'} = 1;
    undef @{ $this->{'query_order'} };
    if ( defined $ENV{'CONTENT_TYPE'}
        && $ENV{'CONTENT_TYPE'} =~ /charset="?([0-9a-zA-Z_\-]+)/i )
    {
        $icode = $this->ename_to_jname($1);
        undef $icode
            if $icode ne 'sjis'
            && $icode ne 'euc'
            && $icode ne 'jis'
            && $icode ne 'utf8';
    }
    if ( defined $ENV{'QUERY_STRING'} && $ENV{'QUERY_STRING'} ne '' ) {
        my $query = $ENV{'QUERY_STRING'};
        $query =~ s/(^|\&)utm_source=[^\&#=]+\&utm_medium=[^\&#=]+//g;
        $this->url_decode( $query, \%in, $option, $jcode, $r, $icode, 1 );
    }
    if ( $ENV{'REQUEST_METHOD'} eq "POST" ) {
        binmode STDIN;
        if ( $ENV{'CONTENT_TYPE'} =~ m!multipart/form-data! ) {
            my $boundary
                = ( $ENV{'CONTENT_TYPE'}
                    =~ /boundary(?:="|="\-+|=\-+|=)([^\-\x0D\x0A]*)/ )
                ? quotemeta $1
                : '';
            my $master_boundary = $boundary;
            my $buffer          = '';
            my $total_length    = 0;
            my $param_temp      = '';
            my $param_ref;
            my $file_opened;
            my $k;
            my $is_attachment;
            my $macbinary            = 0;
            my $file_upload_disabled = 0;
            my $check_key            = $this->read_cookie_direct('UPLOADKEY');

            if ( $env::post_max_size < $ENV{'CONTENT_LENGTH'} ) {
                $file_upload_disabled = 1;
                push(
                    @{  ( ( ref $cgi::action ) ? $cgi::action : $this )
                            ->{'dialog'}
                    },
                    'アップロード容量が制限値を超えたため、アップロードされたファイルはすべて無視されました。'
                );
            }
            if ( $check_key && $env::clipboard ) {
                my $id = $default::id;
                if ( !$id ) {
                    $id = $ENV{'SCRIPT_NAME'};
                    $id =~ s!/[^/]+$!!;
                    $id =~ s!/!_!g;
                }
                $this->{'upload_check_key'}
                    = $id . '_'
                    . $this->encrypted_unique_id(
                    $check_key . ( $ENV{'REMOTE_ADDR'} || '' ), 'uL' );
                $this->write_file(
                    $env::clipboard . '/' . $this->{'upload_check_key'},
                    '0/' . $ENV{'CONTENT_LENGTH'}, 1 );
            }
            local *TMP;

        READ:
            while (1) {
                last READ if $total_length >= $ENV{'CONTENT_LENGTH'};
                my $read_length = read(
                    STDIN, $buffer,
                    (   ( $total_length + $env::buffer_size )
                        > $ENV{'CONTENT_LENGTH'}
                        ? $ENV{'CONTENT_LENGTH'} - $total_length
                        : $env::buffer_size
                    ),
                    bytes::length $buffer
                );
                last READ if !bytes::length $buffer;
                if (  !$total_length
                    && $buffer
                    =~ s/(?:\A|\x0D\x0A|\x0D|\x0A)\-+([^\-\x0D\x0A]*)(?:\x0D\x0A|\x0D|\x0A)//s
                    && !$boundary
                    && !$master_boundary )
                {
                    $master_boundary = $boundary = quotemeta $1;
                }
                $total_length += $read_length;
                if ( $this->{'upload_check_key'} && $total_length % 100 == 0 )
                {
                    $this->write_file(
                        $env::clipboard . '/' . $this->{'upload_check_key'},
                        $total_length . '/' . $ENV{'CONTENT_LENGTH'}, 1 );
                }

            BUF:
                while (1) {

                    # Mac Binary 削除
                    if ($macbinary) {
                        if ( $macbinary > bytes::length $buffer) {
                            $buffer = bytes::substr( $buffer, $macbinary );
                            $macbinary = 0;
                        }
                        else {
                            $macbinary -= bytes::length $buffer;
                            $buffer = '';
                        }
                    }

                    #データ開始
                    if ( $buffer
                        =~ s/^(content-disposition:.*?)(?:\x0D\x0A\x0D\x0A|\x0D\x0D|\x0A\x0A)//is
                        )
                    {
                        my $header = $1;
                        if ( $header
                            =~ /^content-disposition:\x20*form-data/i )
                        {
                            $k
                                = ( $header
                                    =~ /[;\x09\x0A\x0C\x0D\x20]name="([^"\x0D\x0A]*)"/i
                                ) ? $1 : '';
                            $this->unify( [ \$k ], $jcode, $r, $icode );
                            push( @{ $this->{'query_order'} }, $k )
                                unless $option
                                || grep( $_ eq $k,
                                @{ $this->{'query_order'} } );
                            if ( $header
                                =~ /^Content-type:\x20*multipart\/mixed,\x20*boundary=([^\x0D\x0A]+)/mi
                                )
                            {
                                $boundary      = quotemeta $1;
                                $is_attachment = 1;
                                next BUF;
                            }
                        }

                        #アップロード
                        if ( $header
                            =~ /[;\x09\x0A\x0C\x0D\x20]filename="([^"\x0D\x0A]*)"/i
                            )
                        {
                            my $file_name = $1;
                            if ($file_name) {
                                if ($file_upload_disabled) {
                                    $file_opened = 1;
                                }
                                else {
                                    $file_name
                                        =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
                                    $this->unify( [ \$file_name ],
                                        $jcode, $r, $icode );
                                    my $file_type
                                        = ( $header
                                            =~ /^Content-Type:\x20*([^\x0D\x0A]+)\x20*$/im
                                        ) ? lc $1 : '';
                                    if ( $file_type
                                        =~ m!^application/(x-|)macbinary$! )
                                    {
                                        $macbinary = 128;
                                        $file_type
                                            = (
                                            $file_name =~ /\.([0-9a-zA-Z_]+)$/
                                                && $Common::mime_type{ lc $1 }
                                            )
                                            ? $Common::mime_type{ lc $1 }
                                            : 'application/octet-stream';
                                    }
                                    $in{$k} .= "\0" if defined( $in{$k} );
                                    $in{$k} .= $file_name;
                                    my $path = $this->set_clipboard($k);
                                    $this->set_posted_file_properties(
                                        $path,
                                        {   'name'     => $k,
                                            'filename' => $file_name,
                                            'type'     => $file_type
                                        }
                                    );
                                    if ( open( FILE, ">>$path/data" ) ) {
                                        binmode FILE;
                                        flock( FILE, LOCK_EX );
                                        truncate( FILE, 0 );
                                        seek( FILE, 0, 0 );
                                        $file_opened = 1;
                                    }
                                    else {
                                        push(
                                            @{  (   ( ref $cgi::action )
                                                    ? $cgi::action
                                                    : $this
                                                )->{'dialog'}
                                            },
                                            '添付ファイルの保存に失敗しました('
                                                . $file_name
                                                . ')。使用可能な空き容量が不足している可能性があります。'
                                        );
                                    }
                                }
                            }

                            #データ
                        }
                        else {
                            $param_ref = \$in{$k};
                        }

                        #データ終端
                    }
                    elsif ( $buffer
                        =~ s/^(.*?)(?:\x0D\x0A|\x0D|\x0A|)\-+$boundary(--|)(?:\x0D\x0A|\x0D|\x0A|)//s
                        )
                    {
                        if ( ref $param_ref ) {
                            $param_temp .= $1;
                            if ( !$icode && $k eq '_' ) {
                                $icode = $this->get_charset_from_hint(
                                    $param_temp);
                                delete $in{'_'};
                            }
                            else {
                                $this->unify( [ \$param_temp ],
                                    $jcode, $r, $icode );
                                $$param_ref .= "\0" if defined($$param_ref);
                                $$param_ref .= $param_temp;
                            }
                            undef $param_ref;
                            undef $param_temp;
                        }
                        elsif ($file_opened) {
                            if ( !$file_upload_disabled ) {
                                print FILE $1;
                                close(FILE);
                            }
                            undef $file_opened;
                        }
                        if ($3) {
                            if ($is_attachment) {
                                $boundary = $master_boundary;
                                undef $is_attachment;
                            }
                            else {
                                last READ;
                            }
                        }

                    }
                    else {
                        last BUF;
                    }
                }

                if (   bytes::length $buffer > 0
                    && ( ref $param_ref || $file_opened )
                    && bytes::length $buffer >= $env::buffer_size * 2 )
                {
                    my $buf = bytes::substr( $buffer, 0, $env::buffer_size );
                    $buffer = bytes::substr( $buffer, $env::buffer_size );
                    if ( ref $param_ref ) {
                        $param_temp .= $buf;
                    }
                    elsif ( $file_opened && !$file_upload_disabled ) {
                        print FILE $buf;
                    }
                }
                if ( !$read_length ) {
                    if ( ref $param_ref ) {
                        $param_temp .= $buffer;
                        $this->unify( [ \$param_temp ], $jcode, $r, $icode );
                        $$param_ref .= "\0" if defined($$param_ref);
                        $$param_ref .= $param_temp;
                        undef $param_ref;
                        undef $param_temp;
                    }
                    elsif ($file_opened) {
                        if ( !$file_upload_disabled ) {
                            print FILE $buffer;
                            close(FILE);
                        }
                        undef $file_opened;
                    }
                    last READ;
                }
            }
            if ($file_opened) {
                if ( !$file_upload_disabled ) {
                    print FILE $1;
                    close(FILE);
                }
                undef $file_opened;
            }
            if ( $this->{'upload_check_key'} ) {
                unlink $env::clipboard . '/' . $this->{'upload_check_key'};
                delete $this->{'upload_check_key'};
            }

        }
        else {
            my $stdin = '';
            read( STDIN, $stdin, $ENV{'CONTENT_LENGTH'} );
            if (   $in{'HTTP_RAW_POST_DATA'}
                || $stdin
                =~ s/(\&|^)HTTP_RAW_POST_DATA=1(\&|$)/(($2) ? $1 : '')/e )
            {
                delete $in{'HTTP_RAW_POST_DATA'};
                $cgi::HTTP_RAW_POST_DATA = $stdin;
            }
            else {
                $this->url_decode( $stdin, \%in, $option, $jcode, $r, $icode,
                    1 );
            }
        }
    }

    # 変数をコール側で設定
    if ( defined $option && $option ) {
        if ( $in{'openid.ns'} && $in{'openid.mode'} ) {
            foreach my $k ( keys %in ) {
                if ( $k =~ /^openid\.[0-9a-zA-Z_\.]+$/ ) {
                    $cgi::openid{$k}
                        = ( ref $in{$k} ) ? $in{$k}
                        : ( $in{$k} && $in{$k} =~ /\0/ )
                        ? [ split( /\0/, $in{$k} ) ]
                        : $in{$k};
                    delete $in{$k};
                }
            }
        }
        my $p = caller;
        $this->variable( \%in, 0, $p, $jcode, $r, $icode, \@ignore );
    }
    return %in;
}

sub url_decode ($$;$$$$$$) {
    my ( $this, $query_string, $hash, $option, $jcode, $r, $icode,
        $char_hint )
        = @_;
    my ($package_name);
    if (  !$icode
        && $char_hint
        && $query_string
        =~ s/(\&|^)(?:_|\%5F)=([^\&=]*)(\&|$)/(($2) ? $1 : '')/ei )
    {
        $icode = $this->get_charset_from_hint($2);
        $query_string
            =~ s/(\&|^)(?:_|\%5F)=([^\&=]*)(\&|$)/(($2) ? $1 : '')/eig;
    }
    $package_name = $hash if defined $hash && !ref $hash && $hash ne '';
    $hash = {} if !defined $hash || !ref $hash;
    $query_string =~ s/^[\?\&]//;
    $query_string =~ s/\+/ /g;
    while ( $query_string =~ /\G([^=\&]*)=([^\&]*)[\&]?/sg ) {
        my ( $k, $v ) = ( $1, $2 );
        next if !defined $k || $k eq '';
        utf8::encode($k) if utf8::is_utf8($k);
        utf8::encode($v) if utf8::is_utf8($v);
        $k =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
        $v =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
        $this->unify( [ \$k, \$v ], $jcode, $r, $icode );
        $hash->{$k} .= "\0" if defined( $hash->{$k} );
        $hash->{$k} .= $v;
        push( @{ $this->{'query_order'} }, $k )
            unless $option || grep( $_ eq $k, @{ $this->{'query_order'} } );
    }
    return ($package_name)
        ? $this->variable( $hash, 1, $package_name, $jcode, $r, $icode )
        : %$hash;
}

sub get_charset_from_hint ($$) {
    my ( $this, $c ) = @_;
    utf8::encode($c) if utf8::is_utf8($c);
    $c =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
    return
          ( $c eq "\xFF\x3F" )                         ? "ucs2"
        : ( $c eq "\xEF\xBC\xBF" )                     ? "utf8"
        : ( $c eq "\xA1\xB2" )                         ? "euc"
        : ( $c eq "\x81\x51" )                         ? "sjis"
        : ( $c eq "\x1B\x24\x42\x21\x32\x1B\x28\x42" ) ? "jis"
        :                                                "";
}

sub clipboard_sweeper ($) {
    my $this = shift;
    local (*DIR);
    my @expired = ();
    if ( opendir( DIR, $env::clipboard ) ) {
        push(
            @expired,
            map( $env::clipboard . '/' . $_,
                grep( $_ !~ /^\./
                        && $_ !~ /\.html$/
                        && -M $env::clipboard . '/' . $_ > 1,
                    readdir(DIR) ) )
        );
        closedir(DIR);
    }
    if (   $env::public_clipboard
        && $env::public_clipboard ne $env::clipboard
        && opendir( DIR, $env::public_clipboard ) )
    {
        push(
            @expired,
            map( $env::public_clipboard . '/' . $_,
                grep( $_ !~ /^\./
                        && $_ !~ /\.html$/
                        && -M $env::public_clipboard . '/' . $_ > 1,
                    readdir(DIR) ) )
        );
        closedir(DIR);
    }
    foreach my $path (@expired) {
        if ( -f $path ) {
            unlink $path;
        }
        elsif ( -d $path ) {
            rmtree $path;
        }
    }
    return scalar(@expired);
}

sub set_clipboard ($;$$$) {
    my ( $this, $param, $n, $name ) = @_;
    if ( !$this->{'clipboard_id'} ) {
        if ( $this->set_dir($env::clipboard) ) {
            $this->write_file(
                $env::clipboard . '/.htaccess',
                'Satisfy all' . "\x0A"
                    . 'order deny,allow' . "\x0A"
                    . 'deny from all' . "\x0A",
                1
            );
            if (   $env::public_clipboard
                && $env::public_clipboard ne $env::clipboard
                && $this->set_dir($env::public_clipboard) )
            {
                $this->write_file(
                    $env::public_clipboard . '/.htaccess',
                    'Satisfy all' . "\x0A"
                        . 'order deny,allow' . "\x0A"
                        . 'deny from all' . "\x0A",
                    1
                );
            }
            $this->clipboard_sweeper;
            $this->{'clipboard_id'} = $this->unique_id;
            $this->set_dir( $env::clipboard . '/' . $this->{'clipboard_id'} );
            if ($this->set_dir(
                    $env::public_clipboard . '/' . $this->{'clipboard_id'}
                )
                )
            {
                my $ip = $ENV{'REMOTE_HOST'}
                    || $this->to_host( $ENV{'REMOTE_ADDR'} );
                $this->write_file(
                    $env::public_clipboard . '/'
                        . $this->{'clipboard_id'}
                        . '/.htaccess',
                    'Satisfy all' . "\x0A"
                        . 'order deny,allow' . "\x0A"
                        . 'deny from all' . "\x0A"
                        . 'allow from '
                        . $ip . "\x0A",
                    1
                );
            }
        }
    }
    my $clipboard = $env::clipboard . '/' . $this->{'clipboard_id'};
    return $clipboard if !$param;
    $param = $this->parameter_directory_name($param);
    if ( !defined $n ) {
        $n = 0;
        while ( -d $clipboard . '/' . $param . '/' . $n ) {
            $n++;
        }
    }
    $this->set_dir( $clipboard . '/' . $param . '/' . $n );
    return $clipboard . '/' . $param . '/' . $n
        . ( ($name) ? '/' . $name : '' );
}

sub parameter_directory_name ($$) {
    my ( $this, $param ) = @_;
    utf8::encode($param) if utf8::is_utf8($param);
    $param =~ s/([\x00-\xFF])/sprintf("%.2X", unpack("C", $1))/eg;
    return $param;
}

sub get_public_clipboard ($) {
    my $this = shift;
    return ( $this->{'clipboard_id'} )
        ? $env::public_clipboard . '/' . $this->{'clipboard_id'}
        : '';
}

sub get_clipboard ($$;$$) {
    my ( $this, $param, $n ) = @_;
    if ( $this->{'clipboard_id'}
        && -d $env::clipboard . '/' . $this->{'clipboard_id'} )
    {
        my $clipboard = $env::clipboard . '/' . $this->{'clipboard_id'};
        $param = $this->parameter_directory_name($param);
        if ( -d $clipboard . '/' . $param ) {
            if ( defined $n && $n ne '' ) {
                return $clipboard . '/' . $param . '/' . $n
                    if -d $clipboard . '/' . $param . '/' . $n;
            }
            else {
                return $clipboard . '/' . $param;
            }
        }
    }
    return '';
}

sub post_clipboard($$$$;$) {
    my ( $this, $param, $n, $data, $properties ) = @_;
    my $path = $this->set_clipboard( $param, $n );
    $this->write_file( $path . '/data', \$data, 1 );
    delete $properties->{'savedir'};
    delete $properties->{'savename'};
    $this->set_posted_file_properties( $path, $properties )
        if ref $properties;
}

sub clone_posted_file($$$;$) {
    my ( $this, $param, $clone, $n ) = @_;
    $n = 0 if !$n;
    my %properties
        = $this->get_posted_file_properties_from_param( $param, $n );
    if (%properties) {
        $properties{'name'} = $clone;
        $param              = $this->parameter_directory_name($param);
        $clone              = $this->parameter_directory_name($clone);
        if ($properties{'filename'}
            && $this->set_dir(
                      $env::clipboard . '/'
                    . $this->{'clipboard_id'} . '/'
                    . $clone . '/'
                    . $n
            )
            )
        {
            $this->file_copy(
                (   ( $properties{'savedir'} && $properties{'savename'} )
                    ? $properties{'savedir'} . '/' . $properties{'savename'}
                    : $env::clipboard . '/'
                        . $this->{'clipboard_id'} . '/'
                        . $param . '/'
                        . $n . '/data'
                ),
                $env::clipboard . '/'
                    . $this->{'clipboard_id'} . '/'
                    . $clone . '/'
                    . $n . '/data'
            );
            delete $properties{'savedir'};
            delete $properties{'savename'};
            $this->set_posted_file_properties_from_param( $properties{'name'},
                $n, \%properties );
        }
    }
    return %properties;
}

sub get_posted_file_data($$;$) {
    my ( $this, $param, $n ) = @_;
    $n = 0 if !$n;
    my %properties
        = $this->get_posted_file_properties_from_param( $param, $n );
    if (%properties) {
        if ( $properties{'filename'} ) {
            return $this->read_file(
                (   ( $properties{'savedir'} && $properties{'savename'} )
                    ? $properties{'savedir'} . '/' . $properties{'savename'}
                    : $env::clipboard . '/'
                        . $this->{'clipboard_id'} . '/'
                        . $this->parameter_directory_name($param) . '/'
                        . $n . '/data'
                ),
                1
            );
        }
    }
    return '';
}

sub remove_posted_file($$;$) {
    my ( $this, $param, $n ) = @_;
    if ( $this->{'clipboard_id'} ) {
        my $dir
            = $env::clipboard . '/'
            . $this->{'clipboard_id'} . '/'
            . $this->parameter_directory_name($param)
            . ( ( defined $n ) ? '/' . $n : '' );
        rmtree $dir if -d $dir;
    }
}

sub get_posted_file_numbers($$) {
    my ( $this, $param ) = @_;
    return () if !defined $this->{'clipboard_id'};
    local *DIR;
    my @nums = ();
    my $clipboard
        = $env::clipboard . '/'
        . $this->{'clipboard_id'} . '/'
        . $this->parameter_directory_name($param);
    if ( opendir( DIR, $clipboard ) ) {
        @nums = grep( $_ !~ /^\./ && -d $clipboard . '/' . $_, readdir(DIR) );
        closedir(DIR);
    }
    return @nums;
}

sub get_posted_file_properties ($$) {
    my ( $this, $path ) = @_;
    return () if !$path;
    my $data = $this->read_file( $path . '/properties', 1 );
    my %properties = ();
    while ( $data =~ s/^([0-9a-zA-Z_]+)\:([^\x0A\x0D]*)\x0A//s ) {
        $properties{$1} = $2;
    }
    return %properties;
}

sub get_posted_file_properties_from_param ($$;$) {
    my ( $this, $param, $n ) = @_;
    return $this->get_posted_file_properties(
        $this->get_clipboard( $param, ( ($n) ? $n : 0 ) ) );
}

sub get_posted_file_path_from_param ($$;$) {
    my ( $this, $param, $n ) = @_;
    my %properties = $this->get_posted_file_properties(
        $this->get_clipboard( $param, ( ($n) ? $n : 0 ) ) );
    if (   $properties{'savedir'}
        && $properties{'savename'}
        && -f $properties{'savedir'} . '/' . $properties{'savename'} )
    {
        return $properties{'savedir'} . '/' . $properties{'savename'};
    }
    else {
        return '';
    }
}

sub get_posted_file_size ($$) {
    my ( $this, $path ) = @_;
    if ( $path && $path !~ m!^/! && -d $path && -f $path . '/properties' ) {
        my %properties = $this->get_posted_file_properties($path);
        if ( $properties{'savedir'} && $properties{'savename'} ) {
            return -s $properties{'savedir'} . '/' . $properties{'savename'}
                || 0;
        }
        else {
            return -s $path . '/data' || 0;
        }
    }
    return 0;
}

sub get_posted_file_size_from_param ($$;$) {
    my ( $this, $param, $n ) = @_;
    return $this->get_posted_file_size(
        $this->get_clipboard( $param, ( ($n) ? $n : 0 ) ) );
}

sub set_posted_file_properties ($$$) {
    my ( $this, $path, $properties ) = @_;
    return 0 if !$path;
    $this->set_dir($path);
    my $data = '';
    while ( my ( $k, $v ) = each %$properties ) {
        $v =~ s/[\x0A\x0D]/ /g;
        $data .= $k . ':' . $v . "\x0A";
    }
    return $this->write_file( $path . '/properties', \$data, 1 );
}

sub set_posted_file_properties_from_param ($$;$) {
    my ( $this, $param, $n, $properties ) = @_;
    return $this->set_posted_file_properties(
        $this->set_clipboard( $param, ( ($n) ? $n : 0 ) ), $properties );
}

sub inherit_posted_file_properties ($$$) {
    my ( $this, $param, $n ) = @_;
    if (   $main::clipboard_id
        && $main::clipboard_id !~ /\.\./
        && -d $env::clipboard . '/' . $main::clipboard_id )
    {
        $param = $this->parameter_directory_name($param);
        if ( defined $n && $n ne '' ) {
            if (  -d $env::clipboard . '/'
                . $main::clipboard_id . '/'
                . $param . '/'
                . $n )
            {
                my $clipboard = $this->set_clipboard . '/' . $param;
                if ( $this->set_dir($clipboard) ) {
                    rename(
                        $env::clipboard . '/'
                            . $main::clipboard_id . '/'
                            . $param . '/'
                            . $n,
                        $clipboard . '/' . $n
                    );
                    my %properties = $this->get_posted_file_properties(
                        $clipboard . '/' . $n );
                    if ( $properties{'savedir'} && $properties{'savename'} ) {
                        if ( -s $properties{'savedir'} . '/'
                            . $properties{'savename'} )
                        {
                            rename(
                                $properties{'savedir'} . '/'
                                    . $properties{'savename'},
                                $env::public_clipboard . '/'
                                    . $this->{'clipboard_id'} . '/'
                                    . $properties{'savename'}
                            );
                            $properties{'savedir'}
                                = $env::public_clipboard . '/'
                                . $this->{'clipboard_id'};
                        }
                        elsif ( -s $clipboard . '/' . $n . '/data' ) {
                            delete $properties{'savedir'};
                            delete $properties{'savename'};
                        }
                        else {
                            rmtree $clipboard . '/' . $n;
                            return 1;
                        }
                        $this->set_posted_file_properties(
                            $clipboard . '/' . $n,
                            \%properties );
                        return 1;
                    }
                    elsif ( !-s $clipboard . '/' . $n . '/data' ) {
                        rmtree $clipboard . '/' . $n;
                        return 1;
                    }
                }
            }
        }
        elsif (
            -d $env::clipboard . '/' . $main::clipboard_id . '/' . $param )
        {
            local *DIR;
            my $clipboard = $this->set_clipboard . '/' . $param;
            rename(
                $env::clipboard . '/' . $main::clipboard_id . '/' . $param,
                $clipboard );
            if ( opendir( DIR, $clipboard ) ) {
                my @num = grep( $_ !~ /^\./ && -d $clipboard . '/' . $_,
                    readdir(DIR) );
                close(DIR);
                foreach my $n (@num) {
                    my %properties = $this->get_posted_file_properties(
                        $clipboard . '/' . $n );
                    if ( $properties{'savedir'} && $properties{'savename'} ) {
                        if ( -s $properties{'savedir'} . '/'
                            . $properties{'savename'} )
                        {
                            rename(
                                $properties{'savedir'} . '/'
                                    . $properties{'savename'},
                                $env::public_clipboard . '/'
                                    . $this->{'clipboard_id'} . '/'
                                    . $properties{'savename'}
                            );
                            $properties{'savedir'}
                                = $env::public_clipboard . '/'
                                . $this->{'clipboard_id'};
                        }
                        elsif ( -s $clipboard . '/' . $n . '/data' ) {
                            delete $properties{'savedir'};
                            delete $properties{'savename'};
                        }
                        else {
                            rmtree $clipboard . '/' . $n;
                            next;
                        }
                        $this->set_posted_file_properties(
                            $clipboard . '/' . $n,
                            \%properties );
                    }
                    elsif ( !-s $clipboard . '/' . $n . '/data' ) {
                        rmtree $clipboard . '/' . $n;
                    }
                }
                return 1;
            }
        }
    }
    return 0;
}

sub clear_clipboard ($;$$) {
    my $this = shift;
    $this->{'clipboard_expired'} = 1;
}

sub unique_id ($) {
    my $this = shift;
    return (
        $ENV{'UNIQUE_ID'}
            && ( !$main::clipboard_id
            || $ENV{'UNIQUE_ID'} ne $main::clipboard_id )
    ) ? $ENV{'UNIQUE_ID'} : $this->encrypted_unique_id;
}

sub encrypted_unique_id ($$;$) {
    my ( $this, $id, $salt ) = @_;
    if ($id) {
        $id = $this->encrypt( $id, $salt, ( ($salt) ? 1 : 0 ) );
    }
    else {
        $id = $this->encrypt( time . $$ . $ENV{'REMOTE_ADDR'} );
    }
    $id =~ s/\$1\$//g;
    $id =~ s/\$//g;
    $id =~ s/\./\-/g;
    $id =~ s/\//\_/g;
    return $id;
}

######################################################################
#乱文字発生
sub rand_char ($$) {
    my ( $this, $num ) = @_;
    my (@charset) = ( 'a' .. 'z', 'A' .. 'Z', '0' .. '9' );
    my ($char);
    $num = 8 if !$num;
    $num--;
    foreach ( 0 .. $num ) {
        $char .= $charset[ int( rand(62) ) ];
    }
    return $char;
}

######################################################################
#暗号化
sub salt ($) {
    my $this = shift;
    my (@saltset) = ( 'a' .. 'z', 'A' .. 'Z', '0' .. '9', '.', '/' );
    my ( $p1, $p2, $now, $mday, $salt );
    ( $p1, $p2 ) = unpack( "C2", $this->rand_char );
    $now  = time;
    $mday = $now / ( 60 * 60 * 24 ) + $p1 + $p2;
    $salt = $saltset[ $mday % 64 ] . $saltset[ $now % 64 ];
    return $salt;
}

sub encrypt ($$;$$) {
    my ( $this, $text, $salt, $fixed ) = @_;
    my ( $crypt, @salts, $no );
    if ($fixed) {
        $salt
            = ( defined $salt && $salt =~ /^([a-zA-Z0-9\/\.]{2})/ )
            ? $1
            : $this->salt;
    }
    else {
        if ( defined $salt && $salt =~ s/^\$1\$// ) {
            @salts = split( /\$1\$/, $salt );
            map( $_ = substr( $_, 0, 2 ), @salts );
        }
        elsif ( defined $salt && $salt ne '' ) {
            @salts = split( /([a-zA-Z0-9\/\.]{2})(?:[a-zA-Z0-9\/\.]{11})/,
                $salt );
            @salts = grep( $_ =~ /^[a-zA-Z0-9\/\.]{2}$/, @salts );
        }
    }
    $no = 0;
    while ( $text =~ /([a-zA-Z0-9\/\.]{1,8})/g ) {
        $crypt .= crypt(
            $1,
            (     ($fixed) ? $salt
                : ( @salts && defined $salts[$no] ) ? $salts[$no]
                :                                     $this->salt
            )
        );
        $no++;
    }
    return $crypt;
}

######################################################################
#文字コードと改行コードを統一
sub unify ($$$$;$) {
    my ( $this, $refs, $jcode, $r, $icode ) = @_;
    return if !$jcode && !$r;
    foreach my $str (@$refs) {
        $$str = '' if !defined $$str;
        if ( $$str =~ /\0/ ) {
            my @temp = split( /\0/, $$str );
            $$str = '';
            foreach my $v (@temp) {
                $this->unify( [ \$v ], $jcode, $r, $icode );
                $$str .= "\0" if $$str;
                $$str .= $v;
            }
            next;
        }
        else {
            $this->jcode_convert( $str, $jcode, $icode ) if $jcode;
            if ( !$r ) {
                next;
            }
            elsif ( $r eq 'lf' ) {
                $$str =~ s/\x0D\x0A/\x0A/sg;
                $$str =~ s/\x0D/\x0A/sg;
            }
            elsif ( $r eq 'cr' ) {
                $$str =~ s/\x0D\x0A/\x0D/sg;
                $$str =~ s/\x0A/\x0D/sg;
            }
            elsif ( $r eq 'crlf' ) {
                $$str =~ s/(?:\x0D\x0A|\x0D|\x0A)/\x0D\x0A/sg;
            }
        }
    }
}

sub jcode_convert($$$$;$) {
    my ( $this, $str, $ocode, $icode, $was_entity ) = @_;
    my $str_ref = ( ref $str ) ? $str : \$str;
    $ocode = 'utf8' if !$ocode;
    if ( !$icode || $ocode ne $icode ) {
        if ( $ocode eq 'utf8' ) {
            $$str_ref = $cgi::Jcode->set( $str_ref, $icode )->utf8;
        }
        elsif ( $ocode eq 'euc' ) {
            $$str_ref = $cgi::Jcode->set( $str_ref, $icode )->euc;
        }
        elsif ( $ocode eq 'jis' ) {
            $$str_ref = $cgi::Jcode->set( $str_ref, $icode )->jis;
        }
        elsif ( $ocode eq 'sjis' ) {
            $$str_ref = $cgi::Jcode->set( $str_ref, $icode )->sjis;
        }
        elsif ( $ocode eq 'ucs2' ) {
            $$str_ref = $cgi::Jcode->set( $str_ref, $icode )->ucs2;
        }
        else {
            Jcode::convert( $str_ref, $ocode, $icode );
        }
    }
    if ( $ocode eq 'utf8' ) {
        utf8::decode($$str_ref) if !utf8::is_utf8($$str_ref);
    }
    elsif ( utf8::is_utf8($$str_ref) ) {
        utf8::encode($$str_ref);
    }
    $$str_ref
        =~ s/\&\#([0-9]{4,}|x[a-fA-F0-9]+);/$this->_entity_to_jcode($1,$ocode)/seg
        if !$was_entity && $ocode ne 'ucs2';
    return $$str_ref;
}

sub _entity_to_jcode ($$$) {
    my ( $this, $code, $ocode ) = @_;
    utf8::encode($code) if utf8::is_utf8($code);
    $code = sprintf( "%X", $code ) unless $code =~ s/^x//;
    return $this->jcode_convert( pack( "U", hex($code) ), $ocode, 'utf8', 1 );
}

sub encode_utf8 ($$) {
    my ( $this, $str ) = @_;
    utf8::encode($str) if defined $str && !ref $str && utf8::is_utf8($str);
    return $str;
}

sub decode_utf8 ($$) {
    my ( $this, $str ) = @_;
    utf8::decode($str) if defined $str && !ref $str && !utf8::is_utf8($str);
    return $str;
}

######################################################################
#変数をコール側で設定
sub variable ($$;$$$$$$) {
    my ( $this, $in, $overwrite_disabled, $p, $jcode, $r, $icode, $ignore )
        = @_;
    my ( $k, $v, $check, $main );
    $check = 0;
    no strict "refs";
    $p    = caller if !defined $p || !$p;
    $main = 1      if $p eq ''    || $p eq 'main';
    $p .= '::';
    while ( ( $k, $v ) = each %$in ) {
        my (%temp);

        # 変数名を分解
        if ( $k =~ /=/ ) {
            next if $k =~ /\.y$/s;
            $k =~ s/\.[xy]$//;
            while ( $k =~ /\G([^=&]+)=([^&]*)(&|$)/sg ) {
                my ( $nk, $nv ) = ( $1, $2 );
                if ( $nv eq '%03' ) {
                    $nv = $v;
                }
                else {
                    my $decoded = 0;
                    utf8::encode($nk) if utf8::is_utf8($nk);
                    utf8::encode($nv) if utf8::is_utf8($nv);
                    $decoded = 1
                        if $nk
                        =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
                    $decoded = 1
                        if $nv
                        =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
                    $this->unify( [ \$nk, \$nv ], $jcode, $r, $icode )
                        if $decoded;
                }
                $temp{$nk} .= "\0" if defined $temp{$nk};
                $temp{$nk} .= $nv;
            }
        }
        else {
            %temp = ( $k => $v );
        }

        while ( ( $k, $v ) = each %temp ) {
            next if ref $ignore && grep( $_ eq $k, @$ignore );

            # 多次元配列/構造体
            $k =~ s/[^0-9a-zA-Z_:\^]/_/g;
            $k =~ s/:([^:\^]*)/\{'$1'\}/g;
            $k
                =~ s/\^([^\{\}\^]*)/'[' . (($1 =~ m![^0-9]!) ? 0 : $1+0) . ']'/eg;

            # 特殊変数の疑いがあるものをスキップ
            if ( $main && $k =~ /^[A-Z_]+(\{|\[|$)/ ) {
                next;

                # コール側で設定
            }
            elsif ( $v =~ /\0/ ) {
                if ( $k =~ /[\[\{]/ ) {
                    $k = '{$' . $p . $k . '}';
                }
                else {
                    $k = $p . $k;
                }
                next if $overwrite_disabled && eval '@' . $k;
                eval '@' . $k . ' = split(/\0/,$v);';
            }
            else {
                next if $overwrite_disabled && eval 'defined $' . $p . $k;
                eval '$' . $p . $k . ' = $v;';
            }
            $check = 1;
        }
        undef %temp;
    }
    return $check;
}

sub dename ($$;$$$$) {
    my ( $this, $hash, $jcode, $r, $keys, $icode ) = @_;
    my ( %temp, $k, $v );
    while ( ( $k, $v ) = each %$hash ) {
        if ( $k =~ /=/ ) {
            @$keys = grep( $_ ne $k, @$keys ) if ref $keys eq 'ARRAY';
            next if $k =~ /\.y$/s;
            $k =~ s/\.[xy]$//;
            while ( $k =~ /\G([^=&]+)=([^&]*)(&|$)/sg ) {
                my ( $nk, $nv ) = ( $1, $2 );
                utf8::encode($nk) if utf8::is_utf8($nk);
                utf8::encode($nv) if utf8::is_utf8($nv);
                $nk =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
                $nv =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
                $this->unify( [ \$nk, \$nv ], $jcode, $r, $icode );
                $temp{$nk} .= "\0" if defined $temp{$nk};
                $temp{$nk} .= $nv;
                push( @$keys, $nk )
                    if ref $keys eq 'ARRAY' && !grep( $_ eq $nk, @$keys );
            }
        }
        else {
            $temp{$k} = $v;
        }
    }
    return %temp;
}

######################################################################
#全ファイルの保存
sub save_all_files ($$$;$$) {
    my ( $this, $name, $mtype, $limit, $dir ) = @_;
    return ()
        unless $this->{'clipboard_id'}
        && opendir( DIR, $env::clipboard . '/' . $this->{'clipboard_id'} );
    my @parameters
        = grep( $_ !~ /^\./
            && -d $env::clipboard . '/' . $this->{'clipboard_id'} . '/' . $_,
        readdir(DIR) );
    closedir(DIR);
    foreach my $k (@parameters) {
        utf8::encode($k) if utf8::is_utf8($k);
        $k =~ s/([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
        $this->save_file( $k, $name, $mtype, $limit, $dir );
    }
    $this->clear_clipboard;
    return ( ref $this->{'posted_file'} eq 'ARRAY' )
        ? @{ $this->{'posted_file'} }
        : ();
}

######################################################################
#ファイルの保存
sub save_file ($$;$$$$$$$) {
    my ( $this, $k, $filename, $mtype, $limit, $dir, $exclusivename, $delete )
        = @_;
    return 0 if !$this->{'clipboard_id'};
    my $clipboard = $this->get_clipboard($k);
    $dir = $this->get_public_clipboard if !$dir;

    #環境チェック
    if ( $limit && ( $limit =~ /[^0-9]/ || $limit < 1000 ) ) {
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            'ファイルのアップロードに失敗しました。スクリプトエラー！ファイルサイズ許容値の設定を確認してください。'
        );
        return;
    }
    if ( !$this->set_dir($dir) ) {
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            'ファイルのアップロードに失敗しました。スクリプトエラー！ファイル保存先の設定を確認してください。'
        );
        return;
    }

# 許可されているmime-typeと拡張子の対応をハッシュに納める
    my ( %mime_type, %jcode );
    if ( ref $mtype ) {
        foreach my $m (@$mtype) {
            my ( $key, $value, $jcode );
            ( $key, $value, $jcode ) = split( /:/, $m, 3 );
            $mime_type{$key} = $value;
            $jcode{$key} = $jcode if defined $jcode && $jcode ne '';
        }
    }

    #clipboard
    return 0 unless opendir( DIR, $clipboard );
    my @attachments = map( $clipboard . '/' . $_,
        grep( $_ !~ /^\./ && -d $clipboard . '/' . $_, readdir(DIR) ) );
    closedir(DIR);

    my $is_dat = ( $filename && $filename =~ /^!.*\.dat$/ ) ? 1 : 0;
    foreach my $path (@attachments) {
        my ( %properties, $file_path, $ext, $name, $cname );
        %properties = $this->get_posted_file_properties($path);
        $file_path
            = ( $properties{'savedir'} && $properties{'savename'} )
            ? $properties{'savedir'} . '/' . $properties{'savename'}
            : $path . '/data';
        if (   !defined $properties{'name'}
            || $properties{'name'} eq ''
            || !defined $properties{'filename'}
            || $properties{'filename'} eq ''
            || ( !$is_dat && !-s $file_path ) )
        {
            $properties{'delete'} = $exclusivename if $delete;
            unlink(
                $this->fileglob(
                    $dir, '^' . $properties{'delete'} . '\\.[0-9a-zA-Z_]+$',
                    'f'
                )
                )
                if defined $properties{'delete'}
                && $properties{'delete'} ne '';
            $this->set_posted_file_properties( $path, \%properties );
            next;
        }

        #入力のチェック
        if ( $limit && $limit < -s $file_path ) {
            push(
                @{  ( ( ref $cgi::action ) ? $cgi::action : $this )
                        ->{'dialog'}
                },
                'ファイル「'
                    . $properties{'name'}
                    . '」のアップロードに失敗しました。ファイルサイズが大きすぎます（'
                    . $limit
                    . 'バイト以内）。'
            );
            next;
        }

        #MIME-TYPEチェック〜拡張子取得
        if ( defined $mime_type{ $properties{'type'} } ) {
            $ext = '.' . $mime_type{ $properties{'type'} }
                if defined $mime_type{ $properties{'type'} }
                && $mime_type{ $properties{'type'} } ne '';
        }
        elsif ( ref $mtype && @$mtype ) {
            push(
                @{  ( ( ref $cgi::action ) ? $cgi::action : $this )
                        ->{'dialog'}
                },
                'ファイル「'
                    . $properties{'filename'}
                    . '」のアップロードに失敗しました。あなたの送信したファイルのMIME-TYPEは、「'
                    . $properties{'type'}
                    . '」です。アップロードが許可されていない種類のファイルです。'
            );
            next;
        }

        #オリジナルの拡張子
        if ( $properties{'filename'} =~ /([^\/\\:]*)(\.[^\.]+)$/ ) {
            $cname = $1;
            $ext = lc $2 if !$ext;
        }

        #保存名の拡張子
        $name = ($filename) ? $filename : '';
        $ext = lc $2
            if $name
            && $name =~ s/^(.*)(\.[0-9a-zA-Z_]+)$/$1/
            && ( lc($2) eq '.dat' || $ext eq '' );
        $ext = '.jpg' if $ext eq '.jpeg';
        $ext = '.txt'
            if $ext eq ''
            || $ext =~ /^\.(?:pl|cgi|pm|vbs|asp|exe|app|php|rb)$/i;

        #ファイル名
        delete $properties{'basename'};
        delete $properties{'exclusivename'};
        $properties{'exclusivename'} = $exclusivename if $exclusivename;
        if ( $name eq '<client>' ) {
            $cname = $properties{'filename'}
                if !defined $cname || $cname eq '';
            $name = $cname . $ext;
        }
        elsif ( $name eq '<vname>' ) {
            $name = $k . $ext;
        }
        elsif ( $name eq '<time>' ) {
            $properties{'basename'} = $name;
            $name = $this->name_by_time( $ext, $dir );
        }
        elsif ( $name =~ /^\!(.*)$/ ) {
            $name = $1;
            $name =~ s/^\.+//g;
            $name = 'file' if !defined $name || $name eq '';
            $properties{'exclusivename'} = $name if !$exclusivename;
            $name = $name . $ext;
        }
        else {
            $name = 'file' if !defined $name || $name eq '';
            $properties{'basename'} = $name;
            $name = $this->name_by_number( $name, $ext, $dir );
        }
        $properties{'savedir'}  = $dir;
        $properties{'savename'} = $name;
        my $save_path = $dir . '/' . $name;
        unlink(
            grep( $_ ne $save_path,
                $this->fileglob(
                    $dir,
                    '^' . $properties{'exclusivename'} . '\\.[0-9a-zA-Z_]+$',
                    'f'
                ) )
            )
            if defined $properties{'exclusivename'}
            && $properties{'exclusivename'} ne '';

        #テキストの処理
        if (   $ext =~ m!^\.(?:txt|html|htm)$!
            && defined $jcode{ $properties{'type'} }
            && $jcode{ $properties{'type'} } ne '' )
        {
            my $data = $this->read_file( $file_path, 1 );
            my ( $jcode, $r ) = split( /:/, $jcode{ $properties{'type'} } );
            if ( $r eq 'lf' ) {
                $data =~ s/\x0D\x0A/\x0A/gs;
                $data =~ s/\x0D/\x0A/gs;
            }
            elsif ( $r eq 'cr' ) {
                $data =~ s/\x0D\x0A/\x0D/gs;
                $data =~ s/\x0A/\x0D/gs;
            }
            elsif ( $r eq 'crlf' ) {
                $data =~ s/(?:\x0D\x0A|\x0D|\x0A)/\x0D\x0A/sg;
            }
            $this->jcode_convert( \$data, $jcode );
            $this->write_file( $file_path, \$data, 1 );
        }

        #保存
        rename( $file_path, $save_path ) if $file_path ne $save_path;
        $this->set_posted_file_properties( $path, \%properties );
    }

    return 1;
}

######################################################################
#ファイルの確定
sub fix_all_files ($$) {
    my ( $this, $dir ) = @_;
    if (   !$this->{'clipboard_id'}
        || !opendir( DIR, $env::clipboard . '/' . $this->{'clipboard_id'} ) )
    {
        $this->clear_clipboard;
        return 0;
    }
    my $is_existing = ( -d $dir ) ? 1 : 0;
    my @parameters
        = grep( $_ !~ /^\./
            && -d $env::clipboard . '/' . $this->{'clipboard_id'} . '/' . $_,
        readdir(DIR) );
    closedir(DIR);
PARAM:
    foreach my $k (@parameters) {
        my $clipboard
            = $env::clipboard . '/' . $this->{'clipboard_id'} . '/' . $k;
        next PARAM unless opendir( DIR, $clipboard );
        my @attachments = map( $clipboard . '/' . $_,
            grep( $_ !~ /^\./ && -d $clipboard . '/' . $_, readdir(DIR) ) );
        closedir(DIR);
    ATTACH:
        foreach my $path (@attachments) {
            my %properties = $this->get_posted_file_properties($path);
            unlink(
                $this->fileglob(
                    $dir, '^' . $properties{'delete'} . '\\.[0-9a-zA-Z_]+$',
                    'f'
                )
                )
                if $is_existing
                && defined $properties{'delete'}
                && $properties{'delete'} ne '';
            next ATTACH
                if !defined $properties{'name'}
                || $properties{'name'} eq ''
                || !defined $properties{'filename'}
                || $properties{'filename'} eq ''
                || !defined $properties{'savedir'}
                || $properties{'savedir'} eq ''
                || !defined $properties{'savename'}
                || $properties{'savename'} eq ''
                || !-f $properties{'savedir'} . '/' . $properties{'savename'};
            if ( !$is_existing ) {
                if ( !$this->set_dir($dir) ) {
                    $this->clear_clipboard;
                    return 0;
                }
                $is_existing = 1;
            }
            if ( $properties{'basename'} ) {
                my $ext
                    = ( $properties{'savename'} =~ /(\.[0-9a-zA-Z_]+)$/ )
                    ? $1
                    : '.txt';
                if ( $properties{'basename'} eq '<time>' ) {
                    $properties{'savename'}
                        = $this->name_by_time( $ext, $dir );
                }
                else {
                    $properties{'savename'}
                        = $this->name_by_number( $properties{'basename'},
                        $ext, $dir );
                }
            }

            #保存
            if ( $properties{'savedir'} ne $dir ) {
                unlink(
                    $this->fileglob(
                        $dir,
                        '^'
                            . $properties{'exclusivename'}
                            . '\\.[0-9a-zA-Z_]+$',
                        'f'
                    )
                    )
                    if defined $properties{'exclusivename'}
                    && $properties{'exclusivename'} ne '';
                rename(
                    $properties{'savedir'} . '/' . $properties{'savename'},
                    $dir . '/' . $properties{'savename'} );
                $properties{'savedir'} = $dir;
            }
            $this->set_posted_file_properties( $path, \%properties );
        }
    }
    $this->clear_parameters_trash;
    $this->clear_clipboard;
    return 1;
}

sub parameters_trash ($@) {
    my ( $this, @params ) = @_;
    if (@params) {
        $this->{'disposed_parameters'} = []
            if !ref $this->{'disposed_parameters'};
        foreach my $p (@params) {
            push( @{ $this->{'disposed_parameters'} }, $p );
        }
    }
}

sub clear_parameters_trash ($) {
    my $this = shift;
    if ( ref $this->{'disposed_parameters'} ) {
        foreach my $p ( @{ $this->{'disposed_parameters'} } ) {
            undef $$p;
        }
        delete $this->{'disposed_parameters'};
    }
}

######################################################################
#ディレクトリを開けるかチェック
#ディレクトリを開ければ「1」を、ディレクトリを開けなければ「0」を返す。
sub set_dir ($$) {
    my ( $this, $dir ) = @_;
    return 0 if !defined $dir || $dir =~ m!^[\.]{3,}/! || $dir =~ m!://!;
    if ( $dir =~ m!^/! ) {
        $this->path_conv( $dir, 'path' );
        return 0 if $dir !~ /^$cgi::document_root/;
    }
    return 1 if -d $dir;
    return 1 if mkdir( $dir, $env::dir_mode );
    return 1 if mkpath( $dir, 0, $env::dir_mode );
    return 1 if $this->mk_path( $dir, $env::dir_mode );
    return 0;
}

sub mk_path ($$) {
    my ( $this, $dir ) = @_;
    my ( @path, $path );
    @path = split( '/', $dir );
    foreach (@path) {
        $path .= '/' if defined $path && $path ne '';
        $path .= $_;
        next if $_ =~ /^\./;
        mkdir( $path, $env::dir_mode ) unless -d $path;
    }
    return ( -d $dir ) ? 1 : 0;
}

######################################################################
# 書き込み
sub write_file ($$$;$$$) {
    my ( $this, $path, $data, $nocheck_flag, $file_mode, $tategaki_enabled )
        = @_;
    $path = $this->path_change( $path, '', 'rel' )
        if !$nocheck_flag && $path =~ m!^/!;
    local (*FH);
    if ( $tategaki_enabled
        && ( ( ref $data && $$data ) || ( !ref $data && $data ) ) )
    {
        require CMONOS::CGI_Lib::Taketori;
        $data = Taketori->new( { 'css' => 1 } )
            ->text_to_tategaki( ( ( ref $data ) ? $$data : $data ), 1 );
    }
    if ( open( FH, ">>$path" ) ) {
        binmode FH;
        flock( FH, LOCK_EX );
        truncate( FH, 0 );
        seek( FH, 0, 0 );
        if ( ref $data ) {
            my $is_utf8 = 0;
            if ( utf8::is_utf8($$data) ) {
                utf8::encode($$data);
                $is_utf8 = 1;
            }
            print FH $$data;
            utf8::decode($$data) if $is_utf8;
        }
        else {
            utf8::encode($data) if utf8::is_utf8($data);
            print FH $data;
        }
        close(FH);
        chmod(
            (     ( defined $file_mode ) ? $file_mode
                : ( $path =~ /\.(?:cgi|pl)$/ && $env::cgi_file_mode )
                ? $env::cgi_file_mode
                : $env::file_mode
            ),
            $path
        );
        return 1;
    }
    else {
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            'ファイル書き込みに失敗しました。使用可能な空き容量が不足している可能性があります。'
        );
        return 0;
    }
}

#	obj->read_file(path)
sub read_file ($$;$$$$) {
    my ( $this, $path, $nocheck, $tategaki_enabled, $raw_data, $icode ) = @_;
    return ''
        if $ENV{'HTTP_USER_AGENT'}
        && $ENV{'HTTP_USER_AGENT'}
        =~ /(?:Googlebot|Slurp|msnbot|https?:\/\/)/i
        && $path
        =~ /\/(?:navi|recent\\w+|metaindex|trackbacks|related)\.ssi$/;
    my ($data);
    $path = $this->path_change( $path, '', 'rel' )
        if !$nocheck && $path =~ m!^/!;
    local (*FH);
    if ( open( FH, $path ) ) {
        binmode FH;
        flock( FH, LOCK_SH );
        $data = join( '', <FH> );
        close(FH);
        if ($data) {
            if ( !$raw_data ) {
                if ($nocheck) {
                    utf8::decode($data) if !utf8::is_utf8($data);
                }
                else {
                    my $escape_null = 0;
                    $icode = '' if !$icode;
                    if ( $path =~ /\.(?:s?html?|ssi|css|xml|kml)$/i ) {
                        $icode = $this->get_charset_from_html( \$data )
                            if !$icode;
                    }
                    elsif ( $path =~ /\.(?:cgi|pl|pm)$/i ) {
                        $icode
                            = ( $data =~ /^[\x20\x90]*use[\x20\x90]+utf8;/m )
                            ? 'utf8'
                            : ''
                            if !$icode;
                    }
                    elsif ( $path =~ /\.(?:bak|default|dat)$/ ) {
                        $escape_null = 1 if $data =~ s/\x00/&#0;/sg;
                    }
                    $data = $this->jcode_convert( \$data, 'utf8', $icode )
                        if !$icode || $icode ne 'utf8';
                    $data =~ s/&#0;/\x00/sg if $escape_null;
                    utf8::decode($data) if !utf8::is_utf8($data);
                }
            }
            if ($tategaki_enabled) {
                require CMONOS::CGI_Lib::Taketori;
                $data = Taketori->new( { 'css' => 1 } )
                    ->tategaki_to_text($data);
            }
        }
    }
    return ( defined $data ) ? $data : '';
}

# meta 情報から文字コード取得
sub get_charset_from_html ($$) {
    my ( $this, $html ) = @_;
    my $html_ref = ( ref $html ) ? $html : \$html;
    if ( $$html_ref =~ /<head(?![0-9a-zA-Z_])[^>]*>(.*?)<\/head>/is ) {
        my $header = $1;
        if ( $header
            =~ /<meta(?![0-9a-zA-Z_])[^>]*charset=["']?([0-9a-zA-Z_\-]+)["']?[^>]*>/si
            )
        {
            return $this->ename_to_jname($1);
        }
        else {
            while ( $header
                =~ /<meta(?![0-9a-zA-Z_])([^>]*)http-equiv=["']?Content-Type["']?([^>]*)>/sgi
                )
            {
                my $contenttype = $1 . $2;
                if ( $contenttype
                    =~ /[\x20\x09\x0A\x0D]content=["']?(.*?)["']?/si )
                {
                    $contenttype = $1;
                    if ( $contenttype =~ /charset=([0-9a-zA-Z_\-]+)/is ) {
                        return $this->ename_to_jname($1);
                    }
                }
            }
        }
    }
    if ( $$html_ref =~ /^\s*<!DOCTYPE\s+html>/i ) {
        return 'utf8';
    }
    elsif ( $$html_ref
        =~ /<\?xml(?=[\x20\x09\x0A\x0D])[^>]*[\x20\x09\x0A\x0D]encoding=["']?([0-9a-zA-Z_\-]+)["']?[^>]*\?>/i
        )
    {
        return $this->ename_to_jname($1);
    }
    elsif ( $$html_ref =~ /\@charset\x20+["']?([a-zA-Z0-9\-_]+)["']?;/i ) {
        return $this->ename_to_jname($1);
    }
    elsif ( $$html_ref =~ /^[\x20\x09]*use utf8;/m ) {
        return 'utf8';
    }
    return '';
}

######################################################################
#指定ディレクトリ内に同じ名前のファイルを書き込まないような番号付き名前を返す。
sub name_by_number ($$$$) {
    no warnings qw(numeric);
    my ( $this, $name, $ext, $dir ) = @_;
    my ( $filenum, @filenums, $no );
    $no = 0;
    opendir( DIR, $dir );
    foreach my $f ( readdir(DIR) ) {
        if ( $f =~ /^$name([0-9]*)\./ ) {
            $filenum = $1;
            push( @filenums, $filenum );
        }
    }
    closedir(DIR);
    foreach my $n ( sort { $a <=> $b } @filenums ) {
        last if $no < $n || $no > $n;
        $no++;
    }
    $no = sprintf( "%.2d", $no ) if $no;
    $no = '' if $name ne '' && $no == 0;
    return $name . $no . ( ( $ext !~ /^\./ ) ? '.' : '' ) . $ext;
}

######################################################################
#現在時刻を元に名前を付ける。
sub name_by_time ($$$) {
    no warnings qw(numeric);
    my ( $this, $ext, $dir ) = @_;
    my ( @files, $id );
    $id = $this->time_id;

    opendir( DIR, $dir );
    @files = readdir(DIR);
    closedir(DIR);

    while (1) {
        last if grep( $_ =~ /^$id\./, sort { $a <=> $b } @files ) == 0;
        $id++;
    }
    return $id . ( ( $ext !~ /^\./ ) ? '.' : '' ) . $ext;
}

########################################################################
# obj->tag_fix(\html,br,noembed,nostyle);
sub tag_fix ($;$$$$$$$) {
    my ($this,          $in,         $br,
        $noembed,       $nostyle,    $p,
        $nofollow_flag, $js_enabled, $iframe_enabled
    ) = @_;
    my ( $ref, $html, $headfix, $bottomfix, @forbidden, @ignore );
    $headfix = $bottomfix = '';
    if ( ref $in ) {
        if ( ref $in eq 'SCALAR' ) {
            $html = $in;
            $ref  = 1;
        }
        else {
            return 0;
        }
    }
    else {
        $html = \$in;
    }
    return ($ref) ? 0 : '' if !defined $$html;

    #スクリプトを退避
    $$html
        =~ s/(<script(?=[\x09\x0A\x0C\x0D\x20>])[^>]*?>)(.*?)(<\/script(?=[\x09\x0A\x0C\x0D\x20>])[^>]*?>)/$1 . $this->lf_to_entity($this->escape($2)) . $3/gies
        if $js_enabled;

    #コメントを退避
    $$html
        =~ s/(<!--)(.*?)(-->)/$1 . $this->lf_to_entity($this->escape($2)) . $3/ges;

    $$html
        =~ s/(<!)(.*?)(>)/$1 . $this->escape_except_html_entity_and_quot($2) . $3/ges;
    1 while $$html =~ s/(\G|>)([^<]*)>/$1$2&gt;/gs;
    1 while $$html =~ s/<([^>]*)(<|$)/&lt;$1$2/gs;

    #顔文字対策
    $$html =~ s/<(?!\/?[a-zA-Z]+|\!--)([^<>]*)>/&lt;$1&gt;/gs;
    $$html =~ s/<(?!\/?[a-zA-Z]+|\!--)/&lt;/gs;

    #メール
    $$html
        =~ s!<([0-9a-zA-Z_][0-9a-zA-Z_~\-\+\.]*\@[0-9a-zA-Z_~\-]+?\.[0-9a-zA-Z_~\-\.]+)>!&lt;$1&gt;!gs;

    #改行変換前調整
    $$html =~ s/\x0D\x0A/\x0A/g;
    $$html =~ s/\x0D/\x0A/g;
    1 while $$html =~ s/<([^>]*)\x0A+([^<]*)>/<$1 $2>/gs;
    $$html
        =~ s/<div(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>[\x09\x0A\x0C\x0D\x20]*<br(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>[\x09\x0A\x0C\x0D\x20]*<\/div>/\x0A\x0A/igs;
    1 while $$html
        =~ s/<div>((?:(?!(?:<div(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>)).)*?)<\/div>/$1\x0A/igs;
    $$html
        =~ s/<(div(?! [^>]*[^0-9a-zA-Z_]clear:)|p|span(?! [^>]*[^0-9a-zA-Z_]clear:)|strong|em|u|li|dt|dd|ul|ol|font)(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>([\x09\x0A\x0C\x0D\x20]*)<\/\1>/$2/igs
        if !$js_enabled;

    #単独タグ
    foreach my $tag (
        'br',      'hr',   'img',  'input', 'spacer',   'area',
        'bgsound', 'meta', 'link', 'base',  'basefont', 'isindex',
        'frame'
        )
    {
        $$html
            =~ s/<($tag(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*?)[\x09\x0A\x0C\x0D\x20]*\/?>(?:[\x09\x0A\x0C\x0D\x20]*<\/$tag>)?/<$1 \/>/sgi;
    }
    foreach my $tag ( 'embed', 'param' ) {
        $$html
            =~ s/<($tag(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*?)[\x09\x0A\x0C\x0D\x20]*\/?>[\x09\x0A\x0C\x0D\x20]*<\/$tag>/<$1><\/$tag>/sgi;
        $$html
            =~ s/<($tag(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*?)[\x09\x0A\x0C\x0D\x20]*\/?>(?![\x09\x0A\x0C\x0D\x20]*<\/$tag>)/<$1 \/>/sgi;
    }

    #改行変換
    $$html
        =~ s/(<br(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>|)\x0A(?!(?:<table(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<thead(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<tbody(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<tfoot(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<caption(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<tr(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<th(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<td(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/td>|<\/th>|<\/tr>|<\/caption>|<\/tfoot>|<\/tbody>|<\/thead>|<\/table>|<p(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<div(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<h[1-6](?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<hr(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>|<ol(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<ul(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<dl(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<li(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/li>|<option(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/ol>|<\/ul>|<dd(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<dt(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/dd>|<\/dt>|<\/dl>|<pre(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/pre>|<form(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/form>|<\/?param(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>|<\/object>|<\/embed>|<\/?script(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>))/(($1) ? $1 : '<br \/>') . "\x0A"/iseg
        if $br;
    $$html
        =~ s/(<table(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<thead(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<tbody(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<tfoot(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<caption(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<tr(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<th(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<td(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/td>|<\/th>|<\/tr>|<\/caption>|<\/tfoot>|<\/tbody>|<\/thead>|<\/table>|<\/?p(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/?div(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/?h[1-6](?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/?hr(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>|<ol(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<ul(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<dl(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<li(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/li>|<option(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/ol>|<\/ul>|<dd(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<dt(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/dd>|<\/dt>|<\/dl>|<pre(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/pre>|<form(?=[\x09\x0A\x0C\x0D\x20>])[^<>]*?>|<\/form>|<\/?param(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>|<object(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>|<embed(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>|<\/?script(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>)<br(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>/$1/isg;
    $$html =~ s/\x0A//gs if $br && $br == 2;

    #タグが無い場合
    if ( $$html !~ /[<>]/ ) {
        $$html = '<p>' . $$html . '</p>' if defined $p && $p;
        return if defined $ref && $ref;
        return $$html;
    }

#禁止するタグ（開始タグで除外したものを禁止するときは開始タグ除外対象から外す。）
    @forbidden
        = (@env::forbidden_tag) ? @env::forbidden_tag
        : (@default::forbidden_tag) ? @default::forbidden_tag
        : (
        'html',    'head',     'meta',  'link',     'title',  'base',
        'isindex', 'body',     'frame', 'frameset', 'iframe', 'style',
        'form',    'textarea', 'input', 'select',   'option', 'applet',
        'script',  'button'
        );
    push( @forbidden,
        'img',    'spacer', 'area',  'bgsound', 'embed',  'param',
        'object', 'layer',  'video', 'audio',   'source', 'canvas' )
        if $noembed;
    @forbidden = grep( $_ ne 'iframe', @forbidden ) if $iframe_enabled;
    @forbidden = grep( $_ ne 'script', @forbidden ) if $js_enabled;

    #タグチェック除外対象
    @ignore = (
        'p',        'br',      'hr',     'li',    'dt',      'dd',
        'img',      'input',   'spacer', 'area',  'bgsound', 'embed',
        'param',    'input',   'option', 'meta',  'link',    'base',
        'basefont', 'isindex', 'frame',  'video', 'audio',   'source'
    );

    #禁止タグのチェック
    $$html =~ s/<!\-*#[^>]*>//igs;
    foreach my $ftag (@forbidden) {
        $$html
            =~ s/(?:<br(?=[\x09\x0A\x0C\x0D\x20\/>])[^<>]*?>\x0a?|\x0a|)<\/?$ftag(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>//igs;
    }

#小文字化/ダブルクォート/JavaScriptイベント削除/スタイルの削除
    $$html
        =~ s/(<(?!\!)\/?[a-zA-Z]+[^\<\>]*>)/$this->attributes_fix($1,(($js_enabled) ? 0 : 1),$nostyle)/egs;

FIX:
    while ( $$html =~ /<(?!\!)(\/?)([a-z1-6]+)([^\<\>]*)>/gs ) {
        my $tag = $2;
        next FIX if !defined $tag || $tag eq '';

        if ( defined $1 && $1 ) {
            next FIX if grep( $tag =~ /^$_$/i, @ignore );
            $headfix = "<$tag>$headfix" unless $bottomfix =~ s!</\Q$tag\E>!!i;
            next FIX;
        }

        next FIX if grep( $tag =~ /^$_$/i, @ignore );
        $bottomfix = "</$tag>$bottomfix";
    }

    $$html = $headfix . $$html . $bottomfix;

    #閉じタグ追加
    $$html
        =~ s/(<li(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>(.(?!<\/li>))*?)(?=[\x09\x0A\x0C\x0D\x20]*(<li(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>|<\/[ou]l>))/$1<\/li>/sg;
    $$html
        =~ s/(<[ou]l(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>)[\x09\x0A\x0C\x0D\x20]*<\/li>/$1/sg;
    $$html
        =~ s/(<dt(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>(.(?!<\/dt>))*?)(?=[\x09\x0A\x0C\x0D\x20]*(<d[dt](?=[\x09\x0A\x0C\x0D\x20>])[^>]*>|<\/dl>))/$1<\/dt>/sg;
    $$html
        =~ s/(<dd(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>(.(?!<\/dd>))*?)(?=[\x09\x0A\x0C\x0D\x20]*(<d[dt](?=[\x09\x0A\x0C\x0D\x20>])[^>]*>|<\/dl>))/$1<\/dd>/sg;
    $$html
        =~ s/(<dl(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>)[\x09\x0A\x0C\x0D\x20]*<\/d[dt]>/$1/sg;
    $$html
        =~ s/(<option(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>(.(?!<\/option>))*?)(?=[\x09\x0A\x0C\x0D\x20]*(<option(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>|<\/select>))/$1<\/option>/sg;
    $$html
        =~ s/(<a(?=[\x09\x0A\x0C\x0D\x20>])[\x09\x0A\x0C\x0D\x20]+name="([^"]+)"[\x09\x0A\x0C\x0D\x20]*)>/$1 id="$2">/sg;

    # <p>修正
    if ( defined $p && $p ) {
        $$html
            =~ s!<br />[\x09\x0A\x0C\x0D\x20]*<br />[\x09\x0A\x0C\x0D\x20]*!</p><p>!sg;
        $$html = '<p>' . $$html . '</p>';
        $$html
            =~ s/(<code(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>)(.*?)(<\/code>)/$1 . $this->p_to_br($2) . $3/gies;
        $$html
            =~ s/(<pre(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>)(.*?)(<\/pre>)/$1 . $this->p_to_br($2) . $3/gies;
    }
    $$html =~ s/(<p>.*<\/p>)/$this->enparagraph($1)/esgi;

    if ( defined $nofollow_flag && $nofollow_flag ) {
        $$html
            =~ s/(<a[\x09\x0A\x0C\x0D\x20]+[^>]*href=[^>]*>)/$this->set_rel_no_follow($1)/sge;
    }

    #コメントを復元
    $$html
        =~ s/(<!--)(.*?)(-->)/$1 . $this->unescape($this->entity_to_lf($2)) . $3/ges;

    #スクリプトを復元
    $$html
        =~ s/(<script(?=[\x09\x0A\x0C\x0D\x20>])[^>]*?>)(.*?)(<\/script(?=[\x09\x0A\x0C\x0D\x20>])[^>]*?>)/$1 . $this->unescape($this->entity_to_lf($2)) . $3/gies
        if $js_enabled;

    $$html =~ s!(?:<br />[\x0A]*)+$!!si;

    return $$html;
}

sub p_to_br ($$) {
    my ( $this, $html ) = @_;
    $html =~ s!</p><p>!<br /><br />!igs;
    $html =~ s/<p>//sg;
    $html =~ s/<\/p>//sg;
    return $html;
}

#<p>
sub enparagraph ($$) {
    my ( $this, $html ) = @_;
    my $block_start
        = '\!--_[tc]sv|table|caption|thead|tbody|tfoot|tr|th|td|ol|ul|li|dl|dt|dd|div|address|h[0-9]|blockquote|fieldset|form|pre|hr';
    my $block_end
        = '\!--\/_[tc]sv|\/table|\/caption|\/thead|\/tbody|\/tfoot|\/tr|\/th|\/td|\/ol|\/ul|\/li|\/dl|\/dt|\/dd|\/div|\/address|\/h[0-9]|\/blockquote|\/fieldset|\/form|\/pre';
    my $block_allowed_block = 'th|td|li|dd|div|blockquote|fieldset|form';
    my $block               = 'caption|dt|address|h[0-9]|pre';

    $html = $this->paragraph_fix($html);
    $html
        =~ s/(<($block_start|$block_end)(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)([\x09\x0A\x0C\x0D\x20]*<br(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)+/$1/sgo
        ;    #余分な改行を除去
    $html
        =~ s/((?:<br(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>[\x09\x0A\x0C\x0D\x20]*)+)(<($block_start|$block_end)(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)/$2/sgo
        ;    #余分な改行を除去
    $html
        =~ s/(<($block_start|$block_end)(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)/<\/p>$1<p>/sgo
        ;    #ブロックの前後に p を入れる
    $html = $this->paragraph_fix($html);

    $html
        =~ s/<p>[\x09\x0A\x0C\x0D\x20]*(<$block_start(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)/$1/sgo;
    $html
        =~ s/(<$block_end(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)[\x09\x0A\x0C\x0D\x20]*<\/p>/$1/sgo;

    $html
        =~ s/(<($block)(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)(.*?)(<\/\2>)/$1 . $this->p_to_br($3) . $4/sge
        ;    #ブロック要素が許されないブロック要素
    $html
        =~ s/(<($block_allowed_block)(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)(.*?)(<\/\2>)/$1 . $this->p_sweeper($3) . $4/sge
        ;    #ブロック要素が許されるブロック要素

    return $html;
}

sub p_sweeper ($$) {
    my ( $this, $html ) = @_;
    if ( $html !~ m!</p><p>!is ) {
        $html =~ s/<p>//sg;
        $html =~ s/<\/p>//sg;
    }
    return $html;
}

sub paragraph_fix ($$) {
    my ( $this, $html ) = @_;
    $html =~ s/<br \/>(<\/?p>)/$1/sg;
    $html =~ s/(<\/?p>)<br \/>/$1/sg;
    $html =~ s/(<p>)+/<p>/sg;
    $html =~ s/<p>(<p(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)/$1/sg;
    $html =~ s/(<\/p>)+/<\/p>/sg;
    $html
        =~ s/<p>((?:[\x09\x0A\x0C\x0D\x20]*<![^>]+>[\x09\x0A\x0C\x0D\x20]*)+)<\/p>/$1/sg;
    $html =~ s/<p>[\x09\x0A\x0C\x0D\x20]*<\/p>//sg;
    return $html;
}

sub deparagraph ($$) {
    my ( $this, $html ) = @_;
    $html =~ s/<\/p><p>/\x0A\x0A/g;
    $html
        =~ s/<\/p>[\x09\x0A\x0C\x0D\x20]*(<(ol|ul|dl|\!--_[tc]sv|table|div|address|h[0-9]|hr|blockquote|fieldset|form|pre)(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)/\x0A\x0A$1/gs;
    $html
        =~ s/(<(|\!--\/_[tc]sv|\/table|\/ol|\/ul|\/dl|\/div|\/address|\/h[0-9]|hr|\/blockquote|\/fieldset|\/form|\/pre)(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)[\x09\x0A\x0C\x0D\x20]*<p>/$1\x0A\x0A/gs;
    $html =~ s/<p>//sg;
    $html =~ s/<\/p>//sg;
    $html
        =~ s/(<(table|thead|tbody|tfoot|tr|ol|ul|dl|\!--\/_[tc]sv|\!--\/_tate|\/table|\/caption|\/thead|\/tbody|\/tfoot|\/tr|\/th|\/td|\/ol|\/ul|\/li|\/dl|\/dt|\/dd|\/div|\/address|\/h[0-9]|hr|\/blockquote|\/fieldset|\/form|\/pre)(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>(?=<))/$1\x0A/sg;
    return $html;
}

#nofollow
sub set_rel_no_follow ($$) {
    my ( $this, $link ) = @_;
    $link =~ s/[\x09\x0A\x0C\x0D\x20]+rel=".*?"//sgi;
    $link =~ s/>$/ rel="nofollow">/s;
    return $link;
}

#ダブルクォート
sub attributes_fix ($$;$$) {
    my ( $this, $tag, $rmjs, $nostyle ) = @_;
    return $tag if $tag =~ /^<\![\-]+/;
    $tag
        =~ s/<(\/?[a-zA-Z1-6]+)([^>]*)>/$this->_attributes_fix($1,$2,$rmjs,$nostyle)/ges;
    return $tag;
}

#ダブルクォート
sub _attributes_fix ($$$;$$) {
    my ( $this, $tag, $option, $rmjs, $nostyle ) = @_;
    return '<' . $tag . $option . '>'
        if $tag . $option eq 'blockquote class="quotation"';
    $tag = lc $tag;
    my $fixed_option = '';
    while ( $option
        =~ /[\x09\x0A\x0C\x0D\x20]+([0-9a-zA-Z_\-:]+)([\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*("([^"]*)"|["']?([^"'>\x09\x0A\x0C\x0D\x20]*)["']?)|)/g
        )
    {
        if ( $rmjs && $1 =~ /^on/i ) {
            next;
        }
        elsif ( $nostyle && $1 =~ /^(style|class)$/i ) {
            next;
        }
        elsif ( !defined $2 || $2 eq '' ) {
            $fixed_option .= ' ' . ( lc $1 ) . '="' . ( lc $1 ) . '"';
        }
        else {
            my $k = lc $1;
            my $v
                = ( defined $4 && $4 ne '' ) ? $4
                : ( defined $5 && $5 ne '' ) ? $5
                :                              '';
            next
                if $rmjs
                && ( !defined $v
                || $v =~ /^(javascript|vbscript|about):/i
                || $v =~ /<script(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>/
                || ( $k eq 'style' && $v =~ /eval[\x09\x0A\x0C\x0D\x20]*\(/ )
                );
            $fixed_option .= ' ' . $k . '="' . $v . '"';
        }
    }
    $fixed_option = ' ' . $fixed_option if $option =~ /^  /;
    $fixed_option .= ' /' if $option =~ m!/$!;
    return '<' . $tag . $fixed_option . '>';
}

########################################################################
# 複数変数の同時タグ修正
sub tag_fix_multi ($$$$$$@) {
    my ( $this, $br, $noembed, $nostyle, $p, $nofollow, @html ) = @_;
    my (@fixed);
    foreach (@html) {
        if ( ref $_ ) {
            next unless ref $_ eq 'SCALAR';
            $this->tag_fix( $_, $br, $noembed, $nostyle, $p, $nofollow );
        }
        else {
            push( @fixed,
                $this->tag_fix( $_, $br, $noembed, $nostyle, $p, $nofollow )
            );
        }
    }
    return (wantarray) ? @fixed : $fixed[0];
}

########################################################################
#タグの除去
sub detag ($@) {
    my ( $this, @html ) = @_;
    my ( @fixed, $null );
    @fixed = ();
    $null  = undef;
    foreach my $ref (@html) {
        my $html
            = ( !ref $ref ) ? \$ref
            : ( ref $ref eq 'SCALAR' ) ? $ref
            : ( ref $ref eq 'ARRAY' && @$ref && defined $ref->[0] )
            ? \$ref->[0]
            : \$null;
        if ( defined $$html ) {
            $this->delink($html);
            $$html =~ s!<script[^>]*>.*?</script>!!igs;
            $$html =~ s!<style[^>]*>.*?</style>!!igs;
            $$html
                =~ s/<a(?=[\x20\x09\x0A\x0D])[^>]*?[\x20\x09\x0A\x0D]href=["']?((?:https?|mailto|ftps?):[^"'>\x20\x09\x0A\x0D]+)["']?[^>]*?>(.*?)<\/a>/$2 ($1)/igs;
            $$html
                =~ s/<img(?=[\x20\x09\x0A\x0D])[^>]*?[\x20\x09\x0A\x0D]alt=["']?([^"'>\x20\x09\x0A\x0D]+)["']?[^>]*?>/$1/igs;
            $$html =~ s/<(.*?)>//gs;
            $$html =~ s/"/&quot;/g;
            $$html =~ s/</&lt;/g;
            $$html =~ s/>/&gt;/g;
            $$html =~ s/\x0D\x0A/\x0A/gs;
            $$html =~ s/\x0D/\x0A/gs;
        }
        else {
            $$html = '';
        }
        push( @fixed, $$html );
    }
    return (wantarray) ? @fixed : $fixed[0];
}

sub detag_text ($$) {
    my ( $this, $html ) = @_;
    return '' if !defined $html;
    $html =~ s/<.*?>//igs;
    $html =~ s/</&lt;/gs;
    $html =~ s/>/&gt;/gs;
    $html =~ s/[\x0D\x0A]+/ /gs;
    return $html;
}

sub detag_except_anchor ($$) {
    my ( $this, $html ) = @_;
    return '' if !defined $html;
    $html =~ s/<(?!\/?a(?=[\x09\x0A\x0C\x0D\x20>])).*?>//igs;
    $html =~ s/[\x0D\x0A]+/ /gs;
    return $html;
}

sub lf_to_tag ($$;$) {
    my ( $this, $text, $add_flag ) = @_;
    if ($add_flag) {
        $text =~ s/\x0A/<br \/>\x0A/gs;
    }
    else {
        $text =~ s/\x0A/<br \/>/gs;
    }
    return $text;
}

sub tag_to_lf ($$;$) {
    my ( $this, $html, $include_flag ) = @_;
    $html
        =~ s/<div(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>[\x09\x0A\x0C\x0D\x20]*<br(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>[\x09\x0A\x0C\x0D\x20]*<\/div>/\x0A\x0A/igs;
    $html
        =~ s/<div>((?:(?!(?:<div(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>)).)*?)<\/div>/$1\x0A/igs;
    if ($include_flag) {
        $html =~ s/<br(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>[\x0A]?/\x0A/igs;
    }
    else {
        $html =~ s/<br(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>/\x0A/igs;
    }
    return $html;
}

sub lf_to_entity ($$;$) {
    my ( $this, $text, $char_flag ) = @_;
    if ($char_flag) {
        $text =~ s/(\x0D\x0A|\x0A|\x0D)/\\n/gs;
    }
    else {
        $text =~ s/(\x0D\x0A|\x0A|\x0D)/&#10;/gs;
    }
    return $text;
}

sub entity_to_lf ($$;$) {
    my ( $this, $html, $char_flag ) = @_;
    if ($char_flag) {
        $html =~ s/\\n/\x0A/gs;
    }
    else {
        $html =~ s/\&#10;/\x0A/gs;
    }
    return $html;
}

sub lf_to_space ($$) {
    my ( $this, $text ) = @_;
    $text =~ s/[\x0D\x0A]+/ /gs;
    return $text;
}

sub detab ($$) {
    my ( $this, $text ) = @_;
    $text =~ s/\x09/   /gs;
    return $text;
}

########################################################################
#エスケープ
sub escape ($@) {
    my ( $this, @html ) = @_;
    my ( @fixed, $null );
    @fixed = ();
    $null  = undef;
    foreach my $ref (@html) {
        my $html
            = ( !ref $ref ) ? \$ref
            : ( ref $ref eq 'SCALAR' ) ? $ref
            : ( ref $ref eq 'ARRAY' && @$ref && defined $ref->[0] )
            ? \$ref->[0]
            : \$null;
        if ( defined $$html ) {
            $$html =~ s/\&/\&amp;/g;
            $$html =~ s/"/\&quot;/g;
            $$html =~ s/</\&lt;/g;
            $$html =~ s/>/\&gt;/g;
        }
        else {
            $$html = '';
        }
        push( @fixed, $$html );
    }
    return (wantarray) ? @fixed : $fixed[0];
}

#HTMLエンティティはエスケープしない
sub escape_except_html_entity ($@) {
    my ( $this, @html ) = @_;
    my ( @fixed, $null );
    @fixed = ();
    $null  = undef;
    foreach my $ref (@html) {
        my $html
            = ( !ref $ref ) ? \$ref
            : ( ref $ref eq 'SCALAR' ) ? $ref
            : ( ref $ref eq 'ARRAY' && @$ref && defined $ref->[0] )
            ? \$ref->[0]
            : \$null;
        if ( defined $$html ) {
            $$html
                =~ s/\&(?!(?:#[0-9]{2,6}|#x[a-fA-F0-9]+|[a-zA-Z0-9]{2,10});)/\&amp;/g;
            $$html =~ s/"/\&quot;/g;
            $$html =~ s/</\&lt;/g;
            $$html =~ s/>/\&gt;/g;
        }
        else {
            $$html = '';
        }
        push( @fixed, $$html );
    }
    return (wantarray) ? @fixed : $fixed[0];
}

sub escape_except_html_entity_and_quot ($@) {
    my ( $this, @html ) = @_;
    my ( @fixed, $null );
    @fixed = ();
    $null  = undef;
    foreach my $ref (@html) {
        my $html
            = ( !ref $ref ) ? \$ref
            : ( ref $ref eq 'SCALAR' ) ? $ref
            : ( ref $ref eq 'ARRAY' && @$ref && defined $ref->[0] )
            ? \$ref->[0]
            : \$null;
        if ( defined $$html ) {
            $$html
                =~ s/\&(?!(?:#[0-9]{2,6}|#x[a-fA-F0-9]+|[a-zA-Z0-9]{2,10});)/\&amp;/g;
            $$html =~ s/</\&lt;/g;
            $$html =~ s/>/\&gt;/g;
        }
        else {
            $$html = '';
        }
        push( @fixed, $$html );
    }
    return (wantarray) ? @fixed : $fixed[0];
}

sub unescape ($@) {
    my ( $this, @html ) = @_;
    my ( @fixed, $null );
    @fixed = ();
    $null  = undef;
    foreach my $ref (@html) {
        my $html
            = ( !ref $ref ) ? \$ref
            : ( ref $ref eq 'SCALAR' ) ? $ref
            : ( ref $ref eq 'ARRAY' && @$ref && defined $ref->[0] )
            ? \$ref->[0]
            : \$null;
        if ( defined $$html ) {
            $$html =~ s/\&quot;/"/g;
            $$html =~ s/\&lt;/</g;
            $$html =~ s/\&gt;/>/g;
            $$html =~ s/\&#64;/\@/g;
            $$html =~ s/\&#46;/\./g;
            $$html =~ s/\&amp;/&/g;
        }
        else {
            $$html = '';
        }
        push( @fixed, $$html );
    }
    return (wantarray) ? @fixed : $fixed[0];
}

########################################################################
#自動リンク
sub autolink_multi ($$;$$$$$) {
    my ( $this, $html_refs, $nofollow_flag, $protect_flag, $get_title,
        $target, $target2 )
        = @_;
    my (@linked);
    foreach my $html (@$html_refs) {
        push(
            @linked,
            $this->autolink(
                $html,      $nofollow_flag, $protect_flag,
                $get_title, $target,        $target2
            )
        );
    }
    return @linked;
}

sub autolink ($$;$$$$$) {
    my ( $this, $in, $nofollow_flag, $protect_flag, $get_title, $target,
        $target2 )
        = @_;
    my ( $html, $ref ) = ( ref $in ) ? ( $in, 1 ) : ( \$in, 0 );
    return ($ref) ? 0 : '' if ref $html ne 'SCALAR' || !defined $$html;
    $target2 = '_self'
        if ( !defined $target || $target eq '' )
        && ( !defined $target2 || $target2 eq '' );
    $target = '_blank' if !defined $target || $target eq '';
    $this->home_location
        if defined $target2
        && $target2 ne ''
        && ( !defined $cgi::home_location || $cgi::home_location eq '' );
    require CMONOS::CGI_Lib::HTTP_Client if $get_title;
    $$html
        =~ s!([="'][^"<>]*|)(s?https?|s?ftps?)://([0-9a-zA-Z_~\-]+:[0-9a-zA-Z_~\-]+\@|)([0-9a-zA-Z_~\-]+)\.((?:&amp;|[0-9a-zA-Z_~\-/\.#\?\&=\%:])+)(</a>|)!$1 . $this->_autolink($1,$2,$4,$5,$6,$nofollow_flag,$get_title,$target,$target2) . $6!ieg;
    $$html
        =~ s/(<a(?=[\x09\x0A\x0C\x0D\x20>])[^>]+href=['"]?|)(\&#109;\&#97;\&#105;\&#108;\&#116;\&#111;\&#58;|mailto\&#58;|mailto:|)([0-9a-zA-Z_][0-9a-zA-Z_~\-\+\.]*)\@([0-9a-zA-Z_~\-]+?)\.([0-9a-zA-Z_~\-\.]+)(['"]?[^><]*>|[^><]*<\/a>|)/$this->_automail($1,$2,$3,$4,$5,$6,$protect_flag)/ieg;
    return ($ref) ? 1 : $$html;
}

sub _autolink ($$$$$$$$) {
    my ($this,   $attribute_flag, $protocol,      $uri1,
        $uri2,   $linked_flag,    $nofollow_flag, $get_title,
        $target, $target2
    ) = @_;
    my $link = $protocol . '://' . $uri1 . '.' . $this->unescape($uri2);
    return $link if $attribute_flag || $linked_flag;
    my $title = $link;
    if ($get_title) {
        my $res
            = HTTP_Client->new->request( 'GET' => $link, {}, '', '', 4096 );
        if ( $res->is_success ) {
            my $content = $res->content('utf8');
            if ($content
                && ( $content
                        =~ m/<meta\s+name="title"\s+content="(.*?)"[^>]*>/si
                    || $content
                        =~ m/<title(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>(.*?)<\/title(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>/si
                    || $content
                    =~ m/<h[1-6](?=[\x09\x0A\x0C\x0D\x20>])[^>]*>(.*?)<\/h[1-6]>/si
                )
                )
            {
                $title = $this->unescape($1);
            }
        }
        else {
            return $link;
        }
    }
    my $rel = ($nofollow_flag) ? ' rel="nofollow"' : '';
    $target = $target2
        if $target2 ne ''
        && ( $link =~ m!/$!
        || $link =~ /\.(html|htm|shtml|shtm|stm|asp|cgi|pl)$/i )
        && $link =~ /^$cgi::home_location/i;    #ターゲット自動調整
    return
          '<a href="'
        . $this->escape($link)
        . '" target="'
        . $target
        . '" class="autolink"'
        . $rel . '>'
        . $this->escape($title) . '</a>';
}

sub _automail ($$$$$$$;$) {
    my ( $this, $tag, $protocol, $m1, $m2, $m3, $after, $protect_flag ) = @_;
    my $is_linked = ( $after && $after =~ m!<\/a>$!is ) ? 1 : 0;
    return "$protocol$m1\&#64;$m2\&#46;$m3$after"
        if $after && !$is_linked && !$tag;
    if ($protect_flag) {
        my $atmark
            = '<span style="display:none" title="This is a dummy address.">&lt;somebody</span>&#64;<span style="display:none" title="This is a dummy address.">on.the.net&gt;</span>';
        my $m3q = $m3;
        $m3 =~ s/\./&#46;/g;
        $m3q =~ s/\./&quot;+&quot;&#46;&quot;+&quot;/g;
        if ($is_linked) {
            return "$protocol$m1$atmark$m2\&#46;$m3$after";
        }
        elsif ( !defined $tag || $tag eq '' ) {
            return
                "<a href=\"#\" onmouseover=\"this.href=&quot;&#109;&quot;+&quot;&#97;&quot;+&quot;&#105;&quot;+&quot;&#108;&quot;+&quot;&#116;&quot;+&quot;&#111;&quot;+&quot;&#58;&quot;+&quot;$m1&quot;+&quot;&#64;&quot;+&quot;$m2&quot;+&quot;&#46;&quot;+&quot;$m3q&quot;\">$m1$atmark$m2&#46;$m3</a>";
        }
        else {
            $tag =~ s/["']$//;
            $after =~ s/^["']//;
            return
                "$tag\"#\" onmouseover=\"this.href=&quot;&#109;&quot;+&quot;&#97;&quot;+&quot;&#105;&quot;+&quot;&#108;&quot;+&quot;&#116;&quot;+&quot;&#111;&quot;+&quot;&#58;&quot;+&quot;$m1&quot;+&quot;&#64;&quot;+&quot;$m2&quot;+&quot;&#46;&quot;+&quot;$m3q&quot;\"$after";
        }
    }
    else {
        $m3 =~ s/\./\&#46;/g;
        if ($is_linked) {
            return "$protocol$m1\&#64;$m2\&#46;$m3$after";
        }
        elsif ( !defined $tag || $tag eq '' ) {
            return
                "<a href=\"&#109;&#97;&#105;&#108;&#116;&#111;&#58;$m1\&#64;$m2\&#46;$m3\">$m1\&#64;$m2\&#46;$m3</a>";
        }
        else {
            $tag =~ s/["']$//;
            $after =~ s/^["']//;
            return
                "$tag\"&#109;&#97;&#105;&#108;&#116;&#111;&#58;$m1\&#64;$m2\&#46;$m3\"$after";
        }
    }
}

sub delink ($$;$) {
    my ( $this, $in, $delete_title ) = @_;
    my ( $html, $ref ) = ( ref $in ) ? ( $in, 1 ) : ( \$in, 0 );
    if ( ref $html eq 'SCALAR' && defined $$html ) {
        $$html
            =~ s/(?:#" onmouseover="this|javascript:location)\.href=&quot;&#109;&quot;\+&quot;&#97;&quot;\+&quot;&#105;&quot;\+&quot;&#108;&quot;\+&quot;&#116;&quot;\+&quot;&#111;&quot;\+&quot;&#58;&quot;\+&quot;([0-9a-zA-Z_][0-9a-zA-Z_~\-\+\.]*)&quot;\+&quot;&#64;&quot;\+&quot;([0-9a-zA-Z_][0-9a-zA-Z_~\-\+\.]*)&quot;\+&quot;&#46;&quot;\+&quot;([^"]*)&quot;"/{my($m1,$m2,$m3) = ($1,$2,$3); $m3 =~ s!\&quot;\+\&quot;\&#46;\&quot;\+\&quot;!\.!g; 'mailto:' . $m1 . '@' . $m2 . '.' . $m3 . '"'}/ge;
        $$html
            =~ s!<span style="display:none" title="This is a dummy address.">\&lt;somebody</span>\&#64;<span style="display:none" title="This is a dummy address.">on.the.net\&gt;</span>!\@!g;
        $$html =~ s/\&#64;/\@/g;
        $$html =~ s/\&#46;/\./g;
        $$html
            =~ s/<a href="(\&#109;\&#97;\&#105;\&#108;\&#116;\&#111;\&#58;|mailto\&#58;|mailto:|)([^"]+)"[^>]*>\2<\/a>/$this->unescape($2)/segi;
        $$html
            =~ s/<a href="([^"]+)"[^>]* class="autolink"[^>]*>.*?<\/a>/$this->unescape($1)/segi
            if $delete_title;
    }
    return ($ref) ? 1 : $$html;
}

sub autoembed ($$;$$) {
    my ( $this, $in, $width, $height ) = @_;
    my ( $html, $ref ) = ( ref $in ) ? ( $in, 1 ) : ( \$in, 0 );
    return ($ref) ? 0 : '' if ref $html ne 'SCALAR' || !defined $$html;
    $width  = 560 if !$width;
    $height = 315 if !$height;
    $$html
        =~ s!([="'][^"<>]*|)(s?https?)://(www\.youtube\.com\/watch\?.*?v=|youtu\.be\/|www\.nicovideo\.jp\/watch\/|www\.dailymotion\.com\/video\/)([0-9a-zA-Z_\-]+)([0-9a-zA-Z_~\-/\.#\?\&=\%:]*)(</a>|)!$1 . $this->_autoembed($1,$2,$3,$4,$5,$6,$width,$height) . $6!ieg;
    return ($ref) ? 1 : $$html;
}

sub _autoembed ($$$$$$$) {
    my ($this,         $attribute_flag, $protocol,
        $service,      $id,             $query,
        $embeded_flag, $width,          $height
    ) = @_;
    my $link = $protocol . '://' . $service . $id . $query;
    return $link if $attribute_flag || $embeded_flag;
    my $anchor
        = '<a href="'
        . $this->escape($link)
        . '" target="_blank">'
        . $this->escape($link) . '</a>';
    if ( $service =~ /nicovideo/i ) {
        return
            '<script class="autoembeded" type="text/javascript" src="//ext.nicovideo.jp/thumb_watch/'
            . $id . '?w='
            . $width . '&h='
            . $height
            . '"></script><noscript>'
            . $anchor
            . '</noscript>';
    }
    elsif ( $service =~ /dailymotion/i ) {
        $id = $1 if $id =~ /^([0-9a-zA-Z]+)/;
        return
              '<iframe class="autoembeded" frameborder="0" width="'
            . $width
            . '" height="'
            . $height
            . '" src="//www.dailymotion.com/embed/video/'
            . $id
            . '" allowfullscreen>'
            . $anchor
            . '</iframe>';
    }
    else {
        $query =~ s/^\&(?:amp;|)/\?/i;
        $query .= ( ( !$query ) ? '?' : '&' ) . 'rel=0'
            if !$query || $query !~ /rel=/;
        return
              '<iframe class="autoembeded" width="'
            . $width
            . '" height="'
            . $height
            . '" src="//www.youtube.com/embed/'
            . $id
            . $this->escape_except_html_entity($query)
            . '" frameborder="0" allowfullscreen>'
            . $anchor
            . '</iframe>';
    }
    return '';
}

sub deembed ($$;$) {
    my ( $this, $in, $delete_title ) = @_;
    my ( $html, $ref ) = ( ref $in ) ? ( $in, 1 ) : ( \$in, 0 );
    if ( ref $html eq 'SCALAR' && defined $$html ) {
        $$html
            =~ s/<iframe class="autoembeded"[^<>]+><a(?!\w)[^<>]+>([^<>]+)<\/a><\/iframe>/$this->unescape($1)/sgei;
        $$html
            =~ s/<script class="autoembeded"[^<>]+><\/script><noscript><a(?!\w)[^<>]+>([^<>]+)<\/a><\/noscript>/$this->unescape($1)/sgei;
    }
    return ($ref) ? 1 : $$html;
}

sub get_email_from_html ($$) {
    my ( $this, $html ) = @_;
    return (wantarray)
        ? $this->get_address( $this->delink($html) )
        : $this->delink($html);
}

sub address_protector ($$;$) {
    my ( $this, $email, $address_protector ) = @_;
    if ($address_protector) {
        $email =~ s/\./&quot;+&quot;&#46;&quot;+&quot;/g;
        $email =~ s/\@/&quot;+&quot;&#64;&quot;+&quot;/g;
        $email
            = '#" onmouseover="this.href=&quot;&#109;&quot;+&quot;&#97;&quot;+&quot;&#105;&quot;+&quot;&#108;&quot;+&quot;&#116;&quot;+&quot;&#111;&quot;+&quot;&#58;&quot;+&quot;'
            . $email
            . '&quot;';
    }
    else {
        $email =~ s/\./&#46;/g;
        $email =~ s/\@/&#64;/g;
        $email = '&#109;&#97;&#105;&#108;&#116;&#111;&#58;' . $email;
    }
    return $email;
}

sub address_camouflage ($$;$) {
    my ( $this, $email, $address_protector ) = @_;
    return '' if !defined $email;
    $email =~ s/\./&#46;/g;
    if ($address_protector) {
        $email
            =~ s!\@!<span style="display:none" title="This is a dummy address.">&lt;somebody</span>&#64;<span style="display:none" title="This is a dummy address.">on.the.net&gt;</span>!g;
    }
    else {
        $email =~ s/\@/&#64;/g;
    }
    return $email;
}

########################################################################
# 自動リスト化
sub autolist_multi ($@) {
    my ( $this, @html ) = @_;
    my (@listed);
    foreach (@html) {
        push( @listed, $this->autolist($_) );
    }
    return @listed;
}

sub autolist ($$) {
    my ( $this, $original_html ) = @_;
    my ( $html, $ref, @marker_regexps, $start_regexp, $end_regexp );
    ( $html, $ref )
        = ( ref $original_html )
        ? ( $original_html, 1 )
        : ( \$original_html, 0 );
    return ($ref) ? 0 : '' if !defined $$html;

    $$html =~ s/\x0D\x0A/\x0A/gs;
    $$html =~ s/\x0D/\x0A/gs;
    $$html =~ s/<br>/<br \/>/gis;
    return ($ref) ? 1 : $$html if $$html !~ /\x0A/ && $$html !~ /<br \/>/;

#@marker_regexps = ("・","○","●","＊",'\([0-9]+\)\x20*','[0-9]+\.\x20+'); #・○●＊(1)1.
    @marker_regexps = ( "・", '\([0-9]+\)\x20*', '[0-9]+\.\x20+' ); #・(1)1.
    $start_regexp   = '(^|\G|<br \/>\x0a|<br \/>|<p>|\x0a)';
    $end_regexp     = '(\x0a?<br \/>\x0a|\x0a?<br \/>|<\/p>|\x0a)';

    #区切り線
    $$html =~ s/$start_regexp(\-+)$end_regexp/<hr alt="$2" \/>/gs;

    #四段目
    foreach (@marker_regexps) {
        my $marker_regexp = "\x{3000}\x{3000}\x{3000}$_(?!$_)";
        $$html
            =~ s/$start_regexp$marker_regexp(.*?)($end_regexp(?!($marker_regexp|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist("\x{3000}\x{3000}\x{3000}$_",$2)/ges;
    }
    foreach (@marker_regexps) {
        my $marker_regexp = "\x20\x20\x20$_(?!$_)";
        $$html
            =~ s/$start_regexp$marker_regexp(.*?)($end_regexp(?!($marker_regexp|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist("\x20\x20\x20$_",$2)/ges;
    }
    $$html
        =~ s/$start_regexp\-\-\-\-(?![\->])(.*?)($end_regexp(?!(\-\-\-\-|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist('\-\-\-\-',$2)/ges;
    $$html
        =~ s/$start_regexp\+\+\+\+(?!\+)(.*?)($end_regexp(?!(\+\+\+\+|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist('\+\+\+\+',$2)/ges;
    $$html =~ s/\x0A\x0A\x00//sg;

    #三段目
    foreach (@marker_regexps) {
        my $marker_regexp = "\x{3000}\x{3000}$_(?!$_)";
        $$html
            =~ s/$start_regexp$marker_regexp(.*?)($end_regexp(?!($marker_regexp|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist("\x{3000}\x{3000}$_",$2)/ges;
    }
    foreach (@marker_regexps) {
        my $marker_regexp = "\x20\x20$_(?!$_)";
        $$html
            =~ s/$start_regexp$marker_regexp(.*?)($end_regexp(?!($marker_regexp|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist("\x20\x20$_",$2)/ges;
    }
    $$html
        =~ s/$start_regexp\-\-\-(?![\->])(.*?)($end_regexp(?!(\-\-\-|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist('\-\-\-',$2)/ges;
    $$html
        =~ s/$start_regexp\+\+\+(?!\+)(.*?)($end_regexp(?!(\+\+\+|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist('\+\+\+',$2)/ges;
    $$html =~ s/\x0A\x0A\x00//sg;

    #二段目
    foreach (@marker_regexps) {
        my $marker_regexp = "\x{3000}$_(?!$_)";
        $$html
            =~ s/$start_regexp$marker_regexp(.*?)($end_regexp(?!($marker_regexp|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist("\x{3000}$_",$2)/ges;
    }
    foreach (@marker_regexps) {
        my $marker_regexp = "\x20$_(?!$_)";
        $$html
            =~ s/$start_regexp$marker_regexp(.*?)($end_regexp(?!($marker_regexp|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist("\x20$_",$2)/ges;
    }
    $$html
        =~ s/$start_regexp\-\-(?![\->])(.*?)($end_regexp(?!(\-\-|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist('\-\-',$2)/ges;
    $$html
        =~ s/$start_regexp\+\+(?!\+)(.*?)($end_regexp(?!(\+\+|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist('\+\+',$2)/ges;
    $$html =~ s/\x0A\x0A\x00//sg;

    #一段目
    foreach (@marker_regexps) {
        my $marker_regexp = "$_(?!$_)";
        $$html
            =~ s/$start_regexp$marker_regexp(.*?)(\x0a?<br \/>\x0a?<br \/>\x0a?|\x0a?<\/p><p>\x0a?|\x0a\x0a|<\/p>\x0a?$|$)/$this->_autolist($_,$2)/ges;
    }
    $$html
        =~ s/$start_regexp\-(?![\->])(.*?)($end_regexp(?!(\-|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist('\-',$2)/ges;
    $$html
        =~ s/$start_regexp\+(?!\+)(.*?)($end_regexp(?!(\+|<li>|<[uo]l[^]*>|<\/[uo]l>))|$)/$this->_autolist('\+',$2)/ges;
    $$html =~ s/\x0A\x0A\x00//sg;

    return ($ref) ? 1 : $$html;
}

sub _autolist($$$) {
    my ( $this, $marker_regexp, $list ) = @_;
    my $tag = ( $marker_regexp =~ /(\[0-9\]|\\d|\\\+)/ ) ? 'ol' : 'ul';
    $list =~ s/(\x0a?<br \/>\x0a?|\x0a)$marker_regexp/<\/li><li>/sg;
    $list =~ s/(<br \/>|)\x0A/<br \/>/sg;
    $marker_regexp =~ s/\\([\(\)\+\-])/$1/sg;
    return
        "\x0A\x0A\x00<$tag data-originalmarker=\"$marker_regexp\"><li>$list</li></$tag>\x0a";
}

sub delist ($$) {
    my ( $this, $original_html ) = @_;
    my ( $html, $ref )
        = ( ref $original_html )
        ? ( $original_html, 1 )
        : ( \$original_html, 0 );
    $$html =~ s/<hr alt="(\-+)" \/>/\x0A$1\x0A/gs;
    $$html
        =~ s/(<(\/ol|\/ul|\/dl|\!--\/_?[tc]sv|\/table|\/div|\/p|\/address|\/h[0-9]|\/hr|\/blockquote|\/fieldset|\/form|\/pre)(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)[\x09\x0A\x0C\x0D\x20]*(<[ou]l(?=[\x09\x0A\x0C\x0D\x20>])[^>]*>)/$1\x0A$3/gs;
    $$html
        =~ s/(<\/[ou]l>)[\x09\x0A\x0C\x0D\x20]*(<(ol|ul|dl|\!--_?[tc]sv|table|div|p|address|h[0-9]|hr|blockquote|fieldset|form|pre)(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>)/$1\x0A\x0A$2/gs;
    1 while $$html
        =~ s/<([ou]l) (?:data-)?originalmarker="([^"]+)">((.(?!<[ou]l (?:data-)?originalmarker="))*?)<\/\1>/$this->_delist($1,$2,$3)/ges;
    return ($ref) ? 1 : $$html;
}

sub _delist ($$$$$) {
    my ( $this, $tag, $marker, $list, $list_end ) = @_;
    my $text = '';
    my $n    = 0;
    $marker =~ s/\\x20[\+\*]/\x20/g;
    $marker =~ s/\\\././;
    while ( $list =~ /<li>(.*?)<\/li>/sg ) {
        my $data        = $1;
        my $marker_temp = $marker;
        $n++;
        $marker_temp =~ s/(?:\[0-9\]|\\d)\+/$n/;
        $text .= "\x0A" . $marker_temp . $data;
    }
    $text =~ s/\x0A+/\x0A/sg;
    return $text . "\x0A\x0A";
}

########################################################################
# URL書式チェック
sub is_invalid_url ($$) {
    my ( $this, $url ) = @_;
    return (   defined $url
            && !ref $url
            && $url
            =~ m!^s?https?://[0-9a-zA-Z_~\-]+\.[0-9a-zA-Z_~\-/\.#\?\&=\%:,]+$!i
    ) ? 0 : 1;
}

########################################################################
# メールアドレス書式チェック
sub is_invalid_email_address ($$) {
    my ( $this, $address ) = @_;
    return (   defined $address
            && !ref $address
            && $address
            =~ m!^[0-9a-zA-Z_\-\+\.]+\@[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-\.]+$! )
        ? 0
        : 1;
}

sub is_invalid_email_address_header ($$) {
    my ( $this, $address ) = @_;
    return 1 if !defined $address || !$address || ref $address;
    if ( $address =~ /\x0A/ ) {
        $address
            =~ s/[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,\x0A/gs;
        $address =~ s/,?\x0A+/,\x0A/gs;
        $address =~ s/,[\x09\x0A\x0C\x0D\x20]*$//s;
        eval { $_[1] = $address };
    }
    my ( @addresses, $addr, $false );
    @addresses = split( /[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,
        $address );
    foreach $addr (@addresses) {
        if ( $addr =~ /^[^<>]*<([^<>]+)>$/ ) {
            $false++ if $this->is_invalid_email_address($1);
        }
        else {
            $addr =~ s/\(.*\)//g;
            $addr =~ s/[\x09\x0A\x0C\x0D\x20]+//g;
            $false++ if $this->is_invalid_email_address($addr);
        }
    }
    return $false;
}

#メールアドレスのみ取り出す
sub get_address ($@) {
    my ( $this, @addresses ) = @_;
    return (wantarray) ? () : '' if !@addresses;
    my (%email_address);
    my $no = 0;
    foreach my $addr (@addresses) {
        next if !defined $addr;
        if ( ref $addr ) {
            if ( ref $addr eq 'ARRAY' ) {
                foreach my $email (@$addr) {
                    $email_address{$1} = $no++
                        while defined $email
                        && $email
                        =~ /([0-9a-zA-Z_\-\+\.]+\@[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-\.]+)/gs;
                }
            }
            else {
                $email_address{$1} = $no++
                    while defined $$addr
                    && $$addr
                    =~ /([0-9a-zA-Z_\-\+\.]+\@[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-\.]+)/gs;
            }
        }
        else {
            $email_address{$1} = $no++
                while defined $addr
                && $addr
                =~ /([0-9a-zA-Z_\-\+\.]+\@[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-\.]+)/gs;
        }
    }
    return
          (wantarray)
        ? (%email_address)
            ? sort { $email_address{$a} <=> $email_address{$b} }
            keys %email_address
            : ()
        : (%email_address)
        ? ( sort { $email_address{$a} <=> $email_address{$b} }
            keys %email_address )[0]
        : '';
}

########################################################################
# 画像URL書式チェック
sub is_invalid_image ($$;$) {
    my ( $this, $url, $option ) = @_;
    return (
              !defined $url
            || ref $url
            || $url !~ /\.(jpg|jpeg|gif|png)$/i
            || ( defined $option
            && $option eq 'full'
            && $url
            !~ m!^http://[0-9a-zA-Z_~\-]+\.[0-9a-zA-Z_~\-\.]+/[0-9a-zA-Z_~\-/\.#\?\&=\%:]+$!i
            )
    ) ? 1 : 0;
}

########################################################################
#IPチェック
sub client_ip_is_matched ($@) {
    my ( $this, @ip ) = @_;
    @ip = split( /[\x09\x0A\x0C\x0D\x20]+/, $ip[0] )
        if @ip && @ip == 1 && $ip[0] =~ /[\x09\x0A\x0C\x0D\x20]/;
    @ip = grep( defined $_ && $_ ne '', @ip );
    map {
        $_ =~ s/\./\\./g;
        $_ =~ s/\\\\/\\/g;
    } @ip;
    if ( !defined $cgi::client_ip || !$cgi::client_ip ) {
        my $remote_addr = $ENV{'REMOTE_ADDR'};
        $cgi::client_ip   = $this->to_ip($remote_addr);
        $cgi::client_host = $this->to_host($remote_addr);
    }
    return 1 if !@ip;
    return 1
        if grep( eval { $cgi::client_ip =~ /$_/i }
            || eval { $cgi::client_host =~ /$_/i },
        @ip );
    return 0;
}

########################################################################
#Refererチェック
sub referer_is_matched ($;$$$) {
    my ( $this, $ref, $jump, $op ) = @_;
    my (@ref);
    return 1 if $op && !$ENV{'HTTP_REFERER'};
    @ref
        = ( ref $ref )
        ? @$ref
        : ( defined $ref ) ?
        grep( $_ ne '', split( /[\x09\x0A\x0C\x0D\x20]+/, $ref ) )
        : ();
    push( @ref,
        quotemeta( $this->full_path('../../') ),
        quotemeta( $this->full_addr('../../') ) )
        if !@ref;
    ($jump) = grep( $this->is_invalid_url($_) == 0, @ref )
        if !defined $jump || $jump eq '';
    $jump = $this->full_path('../../') if !defined $jump || $jump eq '';
    foreach my $url (@ref) {
        return 1 if eval { $ENV{'HTTP_REFERER'} =~ /$url/i };
    }
    if ( $jump eq 'no' ) {
        return 0;
    }
    elsif ( $jump eq 'error' ) {
        $this->error404;
    }
    else {
        $this->set_alert('こちらから接続して下さい。');
        $this->jump($jump);
    }
}

# alertの設定
sub set_alert ($;@) {
    my ( $this, @alert ) = @_;
    return if !@alert;
    $this->{'onload'} .= $this->mk_alert(@alert);
}

sub mk_alert ($;@) {
    my ( $this, @alert ) = @_;
    return '' if !@alert;
    my ( $alert, $js );
    $js = '';
    foreach $alert (@alert) {
        next if !defined $alert || $alert eq '';
        $alert =~ s/'//g;
        $js .= "alert('" . $this->detag($alert) . "');";
    }
    return $js;
}

########################################################################
# 色指定書式チェック
sub is_invalid_color ($$) {
    my ( $this, $color ) = @_;
    return 1 if !defined $color;
    return 0 if $color =~ /^#[0-9a-fA-F]{6}$/;
    return 0
        if $color
        =~ /^Black|Olive|Teal|Red|Blue|Maroon|Navy|Gray|Lime|Fuchsia|White|Green|Purple|Silver|Yellow|Aqua$/i;
    return 1;
}

sub is_invalid_cscolor ($$) {
    my ( $this, $color ) = @_;
    my (@colors);
    @colors
        = split( /[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/, $color );
    foreach (@colors) {
        return 1 if $this->is_invalid_color($_);
    }
    return 0;
}

########################################################################
# 数字チェック
sub is_nan ($$;$) {
    my ( $this, $num, $nocal ) = @_;
    return 1 if !defined $num;
    if ( $num =~ /^[\+\-]?[0-9\.]+$/s ) {
        if ( ( $num =~ tr/\./\./ ) <= 1 ) {
            if ( !$nocal ) {
                $num += 0;
                eval { $_[1] = $num }
                    if $num =~ /^[\+\-]?[0-9\.]+$/ && $num ne $_[1];
            }
            return 0;
        }
        else {
            return 1;
        }
    }
    else {
        $num = $this->z_to_h($num) if $num !~ /^[\+\-]?[0-9\.]+$/;
        if ( !$nocal && $num !~ /^[\+\-]?[0-9\.]+$/s ) {
            $num =~ s/[^0-9\.+\-\*\/)(]//g;
            $num = eval $num;
            return 1 if !defined $num;
        }
        if (   $num =~ /^[\+\-]?[0-9\.]+$/s
            && ( $num =~ tr/\./\./ ) <= 1
            && ( $num eq $_[1] || eval { $_[1] = $num } ) )
        {
            return 0;
        }
        else {
            return 1;
        }
    }
}

########################################################################
sub is_not_digit ($$) {
    my ( $this, $num, $nocal ) = @_;
    return 1 if !defined $num;
    $num = $this->z_to_h($num);
    if ( !$nocal && $num !~ /^[0-9]+$/s ) {
        $num =~ s/[^0-9\.+\-\*\/)(]//g;
        $num = eval $num;
        return 1 if !defined $num;
    }
    if ( $num =~ /^[0-9]+$/s
        && ( $num eq $_[1] || ( $num ne $_[1] && eval { $_[1] = $num; 1; } ) )
        )
    {
        return 0;
    }
    else {
        return 1;
    }
}

# 半角英数字チェック
sub is_not_word ($$;$) {
    my ( $this, $text, $s ) = @_;
    return 1 if !defined $text;
    if ( ( $s && $text !~ /[^0-9a-zA-Z_\x09\x0A\x0C\x0D\x20\.\,\-]/ )
        || $text !~ /[^0-9a-zA-Z_]/ )
    {
        return 0;
    }
    else {
        $text = $this->z_to_h($text);
        if ((   ( $s && $text !~ /[^0-9a-zA-Z_\x09\x0A\x0C\x0D\x20\.\,\-]/ )
                || $text !~ /[^0-9a-zA-Z_]/
            )
            && eval { $_[1] = $text }
            )
        {
            return 0;
        }
        else {
            return 1;
        }
    }
}

# ひらがなチェック
sub is_not_hiragana ($$;$) {
    my ( $this, $text, $icode ) = @_;
    return 1 if !defined $text;
    if ( $icode && $icode ne 'utf8' && $icode ne 'ascii' ) {
        $text = $cgi::Jcode->set( \$text, $icode )->utf8;
        $icode = $this->ename_to_jname( $cgi::Jcode->icode ) if !$icode;
    }
    else {
        $icode = 'utf8';
    }
    $text = $this->h_to_z( $text, 'utf8' );
    $text =~ s/([\x{30A1}-\x{30F6}])/pack("U",unpack("U",$1) - 96)/ges;
    $text =~ s/^\s+//s;
    $text =~ s/\s+$//s;
    $text =~ s/[\x09\x0A\x0C\x0D\x20]+/\x{3000}/gs;
    if (   $text =~ /[\x{3041}-\x{3096}]/
        && $text =~ /^[\x{3000}\x{3041}-\x{3096}・ー]+$/ )
    {
        $this->jcode_convert( \$text, $icode, 'utf8' ) if $icode ne 'utf8';
        return ( $text eq $_[1] || eval { $_[1] = $text } ) ? 0 : 1;
    }
    else {
        return 1;
    }
}

# カタカナチェック
sub is_not_katakana ($$;$) {
    my ( $this, $text, $icode ) = @_;
    return 1 if !defined $text;
    if ( $icode && $icode ne 'utf8' && $icode ne 'ascii' ) {
        $text = $cgi::Jcode->set( \$text, $icode )->utf8;
        $icode = $this->ename_to_jname( $cgi::Jcode->icode ) if !$icode;
    }
    else {
        $icode = 'utf8';
    }
    $text = $this->h_to_z( $text, 'utf8' );
    $text =~ s/([\x{3041}-\x{3096}])/pack("U",unpack("U",$1) + 96)/ges;
    $text =~ s/^\s+//s;
    $text =~ s/\s+$//s;
    $text =~ s/[\x09\x0A\x0C\x0D\x20]+/\x{3000}/gs;
    if (   $text =~ /[\x{30A1}-\x{30F6}]/
        && $text =~ /^[\x{3000}\x{30A1}-\x{30F6}・ー]+$/ )
    {
        $this->jcode_convert( \$text, $icode, 'utf8' ) if $icode ne 'utf8';
        return ( $text eq $_[1] || eval { $_[1] = $text } ) ? 0 : 1;
    }
    else {
        return 1;
    }
}

# 日本語文字列チェック
sub is_not_japanese ($$;$) {
    my ( $this, $text, $jcode_check ) = @_;
    return 1 if !defined $text;
    $text = $cgi::Jcode->set( \$text )->utf8 if $jcode_check;
    if ( $text
        =~ /[\x{2F00}-\x{2FEF}\x{3041}-\x{3096}\x{30A1}-\x{30F6}\x{4E00}-\x{9FFF}\x{F900}-\x{FAFF}\x{FF66}-\x{FF9D}]{3}/
        )
    {
        return 0;
    }
    else {
        return 1;
    }
}

########################################################################
# パスワード書式チェック
sub is_invalid_password ($$;$$$) {
    my ( $this, $text, $offset, $limit, $pswd ) = @_;
    return 1 if !defined $text;
    $offset = 4           if !defined $offset || !$offset;
    $limit  = $offset + 4 if !defined $limit  || !$limit;
    if ($pswd) {
        $offset--;
        $limit--;
        return 0 if $text =~ /^[a-zA-Z][a-zA-Z0-9]{$offset,$limit}$/s;
    }
    else {
        return 0 if $text =~ /^[a-zA-Z0-9]{$offset,$limit}$/s;
    }
    return 1;
}

########################################################################
# 禁止ワードチェック
sub is_taboo_string ($$$;$) {
    my ( $this, $str, $taboo, $regexp ) = @_;
    if ( $str && defined $taboo && ( ref $taboo || $taboo ne '' ) ) {
        $taboo = [ split( /[\x0A\x0D]+/, $taboo ) ]
            if !ref $taboo && $taboo ne '';
        @$taboo = grep( $_ ne '', @$taboo );
        map( $_ = quotemeta($_), @$taboo ) if !$regexp;
        foreach my $tb (@$taboo) {
            return 1 if eval { $str =~ /$tb/ };
        }
    }
    return 0;
}

########################################################################
#ジャンプ
sub jump ($$;$) {
    my ( $this, $url, $op ) = @_;
    binmode STDOUT, ":utf8";
    if (   $env::location_header_disabled
        || $this->{'onload'}
        || $this->{'Set_Cookie'} )
    {
        $this->print_http_header;
        print '<html><head><meta http-equiv="Refresh" content="0; URL='
            . $this->escape($url) . '" />';
        print
            '<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" /></head>';
        print '<body onload="'
            . ( ( $this->{'onload'} ) ? $this->{'onload'} . ';' : '' )
            . 'location.href=\''
            . $this->escape($url)
            . '\';"></body></html>';
    }
    else {
        $url =~ s/[\x0A\x0D]//gs;
        print 'Location: ' . $url . "\x0A\x0A";
    }
    binmode STDOUT;
    exit if !$op || $op ne 'no';
}

########################################################################
#HTTPヘッダの出力
sub print_http_header ($;$$$) {
    my ( $this, $type, $length, $attachment ) = @_;
    $type = 'text/html' if !defined $type || $type eq '';
    print 'Content-Type: ' . $type . "\x0A";
    print 'Content-Length: ' . $length . "\x0A" if $length;
    if ($attachment) {
        print 'Content-Disposition: attachment';
        if ( $attachment ne '1' ) {
            if ( $attachment =~ /^[\x20-\x7e]+$/ ) {
                print '; filename="' . $attachment . '"';
            }
            else {
                print '; filename*="utf-8\'\''
                  . $this->url_encode($attachment) . '"';
            }
        }
        print "\x0A";
    }
    print 'Last-Modified: '
        . $this->server_gmtime( $this->{'Last_Modified'} ) . "\x0A"
        if $this->{'Last_Modified'};
    print $this->{'Set_Cookie'} if $this->{'Set_Cookie'};
    print "\x0A";
}

sub set_last_modified ($;$) {
    my ( $this, $time ) = @_;
    delete $this->{'Last_Modified'};
    $this->{'Last_Modified'} = $time if $time;
}

sub server_gmtime ($;$) {
    my ( $this, $time ) = @_;
    $time = time if !$time;
    my @week = ( 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat' );
    my @month = (
        'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
        'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'
    );
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday ) = gmtime($time);
    $year = $year + 1900;
    return sprintf( "%s, %d %s %d %02d:%02d:%02d GMT",
        $week[$wday], $mday, $month[$mon], $year, $hour, $min, $sec );
}

########################################################################
#現在時刻
sub present_time ($;$$$) {
    my ( $this, $op, $type, $time ) = @_;
    my ( $presenttime, @week );
    $op = '' if !defined $op;
    $time = time if !defined $time || $time eq '';
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday ) = localtime($time);

    $year += 1900;
    @week = ( '日', '月', '火', '水', '木', '金', '土' );
    $wday = $week[$wday];

    $min = sprintf( "%.2d", $min );

    if ( $type =~ m!/! ) {
        $sec  = sprintf( "%.2d", $sec );
        $mday = sprintf( "%.2d", $mday );
        $mon  = sprintf( "%.2d", $mon );

        $presenttime .= $year . '/'
            if !$op || $op =~ /^(year|)-(year|mon|mday|wday|hour|min|sec|)$/i;
        $presenttime .= $mon . '/'
            if !$op || $op =~ /^(year|mon|)-(mon|mday|wday|hour|min|sec|)$/i;
        $presenttime .= $mday
            if !$op || $op =~ /^(year|mon|mday|)-(mday|wday|hour|min|sec|)$/i;
        $presenttime .= '(' . $wday . ') '
            if !$op || $op =~ /^(year|mon|mday|wday|)-(wday|hour|min|sec|)$/i;
        $presenttime .= $hour . ':' . $min
            if !$op || $op =~ /^(year|mon|mday|wday|hour|min|)-(min|sec|)$/i;
        $presenttime .= ':' . $sec
            if !$op || $op =~ /^(year|mon|mday|wday|hour|min|sec|)-(sec|)$/i;
        return $presenttime;

    }
    else {
        $presenttime
            .= (
            ( $type =~ /j/i ) ? $this->nengo( $year, $mon, $mday ) : $year )
            . '年'
            if !$op || $op =~ /^(year|)-(year|mon|mday|wday|hour|min|sec|)$/i;
        $presenttime
            .= ( ( $type =~ /J/ ) ? $this->tsuki($mon) : ++$mon . '月' )
            if !$op || $op =~ /^(year|mon|)-(mon|mday|wday|hour|min|sec|)$/i;
        $presenttime .= $mday . '日'
            if !$op || $op =~ /^(year|mon|mday|)-(mday|wday|hour|min|sec|)$/i;
        $presenttime .= '(' . $wday . ') '
            if !$op || $op =~ /^(year|mon|mday|wday|)-(wday|hour|min|sec|)$/i;
        $presenttime .= $hour . '時'
            if $op =~ /^(year|mon|mday|wday|hour|)-hour$/i;
        $presenttime .= $hour . ':' . $min
            if !$op || $op =~ /^(year|mon|mday|wday|hour|)-(min|sec|)$/i;
        $presenttime .= $min . '分' if $op =~ /^min-(min|sec|)$/i;
        $presenttime .= ':' . $sec
            if !$op || $op =~ /^(year|mon|mday|wday|hour|)-(sec|)$/i;
        $presenttime .= $sec . '秒' if $op =~ /^(min|sec)-(sec|)$/i;
        $presenttime = $this->kansuuji( $type, $presenttime );
        return $presenttime;

    }

}

sub time_id ($;$) {
    my ( $this, $time ) = @_;
    $time = time if !defined $time || $time eq '';
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday ) = localtime($time);
    $year += 1900;
    $mon++;
    return
          $year
        . sprintf( "%.2d", $mon )
        . sprintf( "%.2d", $mday )
        . sprintf( "%.2d", $hour )
        . sprintf( "%.2d", $min )
        . sprintf( "%.2d", $sec );
}

sub date_to_gmt ($$) {
    my ( $this, $date ) = @_;
    my ( $year, $mon, $mday, $hour, $min, $sec, $tz );
    if ( $date
        =~ /([0-9]{4}|[0-9]{2})-([0-9]{1,2})-([0-9]{1,2})T([0-9]{2}):([0-9]{2})(?::([0-9]{2}))?(?:\.[0-9]+)?(UT|GMT|[ECMP][SD]T|[A-Z]|[+\-][0-9]{2}\:?[0-9]{2})/
        )
    {
        ( $year, $mon, $mday, $hour, $min, $sec, $tz )
            = ( $1, $2, $3, $4, $5, $6, $7 );
    }
    elsif ( $date
        =~ /(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun), )?([0-9]{1,2}) (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) ([0-9]{4}|[0-9]{2}) ([0-9]{2}):([0-9]{2})(?::([0-9]{2}))? (UT|GMT|[ECMP][SD]T|[A-Z]|[+\-][0-9]{2}\:?[0-9]{2})/
        )
    {
        ( $year, $mon, $mday, $hour, $min, $sec, $tz )
            = ( $4, $3, $2, $5, $6, $7, $8 );
        my %monthes
            = qw(Jan 1 Feb 2 Mar 3 Apr 4 May 5 Jun 6 Jul 7 Aug 8 Sep 9 Oct 10 Nov 11 Dec 12);
        $year
            = ( $year < 70 ) ? $year + 2000
            : ( $year < 1000 ) ? $year + 1900
            :                    $year;
        $mon = $monthes{$mon};
    }
    elsif ( $date
        =~ /([0-9]{4})[^0-9]+([0-9]{1,2})[^0-9]+([0-9]{1,2})[^0-9]+([0-9]{1,2})[^0-9]+([0-9]{1,2})[^0-9]+([0-9]{1,2})/
        || $date
        =~ /([0-9]{4})[^0-9]*([0-9]{2})[^0-9]*([0-9]{2})[^0-9]*([0-9]{2})?[^0-9]*([0-9]{2})?[^0-9]*([0-9]{2})?/
        )
    {
        ( $year, $mon, $mday, $hour, $min, $sec )
            = ( $1 + 0, $2 + 0, $3 + 0, $4 + 0, $5 + 0, $6 + 0 );
        return $date
            if ( ( $mon && $mon > 12 )
            || ( $mday && $mday > 31 )
            || ( $hour && $hour > 59 )
            || ( $min  && $min > 59 )
            || ( $sec  && $sec > 59 ) )
            && $date =~ /^[0-9]+$/;
        $mday = $this->max_mday( $year, $mon )
            if $this->max_mday( $year, $mon ) < $mday;
        use Time::Local;
        return timelocal(
            $sec, $min, $hour,
            ( $mday || 1 ),
            ( $mon - 1 ), $year
        );
    }
    else {
        return time;
    }
    if ( $tz && $tz =~ /[A-Z]/ ) {
        my %timezones = (
            'UT'  => '+0000',
            'GMT' => '+0000',
            'EST' => '-0500',
            'EDT' => '-0400',
            'CST' => '-0600',
            'CDT' => '-0500',
            'MST' => '-0700',
            'MDT' => '-0600',
            'PST' => '-0800',
            'PDT' => '-0700',
            'A'   => '-0100',
            'B'   => '-0200',
            'C'   => '-0300',
            'D'   => '-0400',
            'E'   => '-0500',
            'F'   => '-0600',
            'G'   => '-0700',
            'H'   => '-0800',
            'I'   => '-0900',
            'K'   => '-1000',
            'L'   => '-1100',
            'M'   => '-1200',
            'N'   => '+0100',
            'O'   => '+0200',
            'P'   => '+0300',
            'Q'   => '+0400',
            'R'   => '+0500',
            'S'   => '+0600',
            'T'   => '+0700',
            'U'   => '+0800',
            'V'   => '+0900',
            'W'   => '+1000',
            'X'   => '+1100',
            'Y'   => '+1200'
        );
        $tz = $timezones{$tz};
    }
    else {
        $tz =~ s/://;
    }
    $tz
        = ( $tz && $tz =~ /([+\-])([0-9]{2})([0-9]{2})/ )
        ? $1 . $2 * 3600 + $3 * 60
        : 0;
    return timegm( $sec, $min, $hour, $mday, ( $mon - 1 ), $year ) - $tz;
}

########################################################################
# 相対日時
sub date ($;$$$$) {
    my ( $this, $time, $limit, $type, $format ) = @_;
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $gap, $date );
    $type = '' if !defined $type;
    $time = $time->[0] || '' if ref $time && @$time <= 1;
    if ( !$time ) {
        ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday )
            = localtime(time);
        $year += 1900;
        $mon++;
    }
    elsif ( ref $time ) {
        ( $year, $mon, $mday, $hour, $min, $sec ) = @$time;
        $year = '' if !defined $year;
        $mon  = '' if !defined $mon;
        $mday = '' if !defined $mday;
        $hour = '' if !defined $hour;
        $min  = '' if !defined $min;
        $sec  = '' if !defined $sec;
        $yday = $this->get_yday( $year, $mon - 1, $mday ) if $mday ne '';
    }
    elsif ( $type =~ /(gmt|v)/ ) {
        ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday )
            = ( $type =~ /v/ ) ? gmtime($time) : localtime($time);
        $year += 1900;
        $mon++;
    }
    elsif ( $time
        =~ /^([0-9]{4})([0-9]{2}|)([0-9]{2}|)([0-9]{2}|)([0-9]{2}|)([0-9]{2}|)/
        )
    {
        ( $year, $mon, $mday, $hour, $min, $sec )
            = ( $1, $2, $3, $4, $5, $6 );
        $yday = $this->get_yday( $year, $mon - 1, $mday ) if $mday ne '';
    }
    else {
        ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday )
            = localtime($time);
        $year += 1900;
        $mon++;
    }
    $mon  += 0 if $mon ne '';
    $mday += 0 if $mday ne '';
    $hour += 0 if $hour ne '';
    $min  += 0 if $min ne '';
    $sec  += 0 if $sec ne '';
    $mday++ if $mday ne '' && $mday == 0;

    #vCard
    if ( $type =~ /v/ ) {
        return $year
            . ( ( defined $mon && $mon ne '' )
            ? '-' . sprintf( "%.2d", $mon )
            : '' )
            . ( ( defined $mday && $mday ne '' )
            ? '-' . sprintf( "%.2d", $mday )
            : '' )
            . (
            ( $hour ne '' && $type !~ />d/ )
            ? 'T'
                . (
                ( defined $hour && $hour ne '' )
                ? sprintf( "%.2d", $hour )
                : ''
                )
                . (
                ( defined $min && $min ne '' )
                ? ':' . sprintf( "%.2d", $min )
                : ''
                )
                . (
                ( defined $sec && $sec ne '' )
                ? ':' . sprintf( "%.2d", $sec )
                : ''
                )
                . 'Z'
            : ''
            );
    }
    elsif ( ( $type =~ /-/ || $type eq '' ) && !$format ) {
        return $year
            . ( ( defined $mon && $mon ne '' )
            ? '-' . sprintf( "%.2d", $mon )
            : '' )
            . ( ( defined $mday && $mday ne '' )
            ? '-' . sprintf( "%.2d", $mday )
            : '' )
            . (
            ( $hour ne '' && $type !~ />d/ )
            ? ' '
                . (
                ( defined $hour && $hour ne '' )
                ? sprintf( "%.2d", $hour )
                : ''
                )
                . (
                ( defined $min && $min ne '' )
                ? ':' . sprintf( "%.2d", $min )
                : ''
                )
                . (
                ( defined $sec && $sec ne '' )
                ? ':' . sprintf( "%.2d", $sec )
                : ''
                )
            : ''
            );
    }
    $gap = 0;
    if ( defined $yday && $yday ne '' ) {
        $gap = $this->get_difference_of_yday( $year, $yday );
    }
    else {
        $limit = 0;
    }

    #ファイルへの出力時には相対日時を使わない
    $limit = 0 if !$limit || $default::static || $cgi::output;

    if ($format) {
        my $noon = 0;
        $format
            =~ s/\%DATE(.*?)\%_DATE/$this->relative_date($gap,$type,$limit,$1)/iegs;
        $format =~ s/\%Y/$year/gs;
        $format =~ s/\%y/substr($year,-2)/egs;
        $format
            =~ s/\%漢年号/$this->num_to_kazu($this->nengo($year,$mon-1,$mday))/egs;
        $format
            =~ s/\%大年号/$this->num_to_kazu($this->nengo($year,$mon-1,$mday),1,1)/egs;
        $format
            =~ s/\%数年号/$this->num_to_kazu($this->nengo($year,$mon-1,$mday),1)/egs;
        $format =~ s/\%年号/$this->nengo($year,$mon-1,$mday)/egs;
        $format =~ s/\%年/$this->num_to_kazu($year)/egs;
        $format =~ s/\%数年/$this->num_to_kazu($year,1)/egs;
        $format =~ s/\%大年/$this->num_to_kazu($year,1,1)/egs;

        $format
            =~ s/\%MIN/((defined $min && $min ne '') ? sprintf("%.2d", $min) : '')/egs;
        $format =~ s/\%min/$min/gs;

        $format =~ s/\%Month/$this->month_name($mon)/iegs;
        $format =~ s/\%Mon/$this->month_name($mon,1)/iegs;
        $format
            =~ s/\%M/((defined $mon && $mon ne '') ? sprintf("%.2d", $mon) : '')/egs;
        $format =~ s/\%m/$mon/gs;
        $format =~ s/\%月名/$this->tsuki($mon-1)/egs;
        $format =~ s/\%大月/$this->num_to_kazu($mon,1,1)/egs;
        $format =~ s/\%数月/$this->num_to_kazu($mon,1)/egs;
        $format =~ s/\%月/$this->num_to_kazu($mon)/egs;

        $format =~ s/\%Day/$this->mday_number($mday)/iegs;
        $format
            =~ s/\%D/((defined $mday && $mday ne '') ? sprintf("%.2d", $mday) : '')/egs;
        $format =~ s/\%d/$mday/gs;
        $format =~ s/\%大日/$this->num_to_kazu($mday,1,1)/egs;
        $format =~ s/\%数日/$this->num_to_kazu($mday,1)/egs;
        $format =~ s/\%日/$this->num_to_kazu($mday)/egs;

        $format =~ s/\%W/$this->wday_name($year,$mon,$mday)/egs;
        $format =~ s/\%w/$this->wday_name($year,$mon,$mday,1)/egs;
        $format =~ s/\%曜/$this->wday_jname($year,$mon,$mday)/egs;

        $noon = 1 if $format =~ s/\%p/(($hour < 12) ? 'a.m.' : 'p.m.')/egs;
        $noon = 1 if $format =~ s/\%P/(($hour < 12) ? 'A.M.' : 'P.M.')/egs;
        $noon = 1
            if $format =~ s/\%午/(($hour < 12) ? '午前' : '午後')/egs;

        my $h = ( $noon && $hour && $hour >= 12 ) ? $hour - 12 : $hour;
        $format
            =~ s/\%H/((defined $h && $h ne '') ? sprintf("%.2d", $h) : '')/egs;
        $format =~ s/\%h/$h/gs;
        $format =~ s/\%大時/$this->num_to_kazu($h,1,1)/egs;
        $format =~ s/\%数時/$this->num_to_kazu($h,1)/egs;
        $format =~ s/\%時/$this->num_to_kazu($h)/egs;

        $format =~ s/\%大分/$this->num_to_kazu($min,1,1)/egs;
        $format =~ s/\%数分/$this->num_to_kazu($min,1)/egs;
        $format =~ s/\%分/$this->num_to_kazu($min)/egs;

        $format
            =~ s/\%S/((defined $sec && $sec ne '') ? sprintf("%.2d", $sec) : '')/egs;
        $format =~ s/\%s/$sec/gs;
        $format =~ s/\%大秒/$this->num_to_kazu($sec,1,1)/egs;
        $format =~ s/\%数秒/$this->num_to_kazu($sec,1)/egs;
        $format =~ s/\%秒/$this->num_to_kazu($sec)/egs;
        return $format;
    }
    else {
        $hour
            = ( $hour ne '' && $type !~ />d/ )
            ? $this->get_hour( $type, $hour, $min, $sec )
            : '';
        if ( abs($gap) >= $limit ) {
            if ( $default::english || $type =~ /e/ ) {
                if ( $type =~ /c/ ) {
                    $date = $this->get_month_name( $mon, $type )
                        . (
                        ($mday)
                        ? ' '
                            . $this->get_mday_name( $mday, $type )
                            . (
                            ( $type !~ /nw/ )
                            ? ' ('
                                . $this->get_wday_name( $year, $mon, $mday,
                                $type )
                                . ')'
                            : ''
                            )
                            . ', '
                        : ''
                        ) . $year;
                }
                elsif ( $type =~ /\./ ) {
                    $date = ( ($mon) ? sprintf( "%.2d", $mon ) . '.' : '' )
                        . (
                        ($mday)
                        ? sprintf( "%.2d", $mday ) . '.'
                            . $year
                            . (
                            ( $type !~ /nw/ )
                            ? ' ('
                                . $this->get_wday_name( $year, $mon, $mday,
                                $type )
                                . ')'
                            : ''
                            )
                        : ''
                        );
                }
                else {
                    $date = ( ($mon) ? sprintf( "%.2d", $mon ) . '/' : '' )
                        . (
                        ($mday)
                        ? $mday . '/'
                            . $year
                            . (
                            ( $type !~ /nw/ )
                            ? ' ('
                                . $this->get_wday_name( $year, $mon, $mday,
                                $type )
                                . ')'
                            : ''
                            )
                        : ''
                        );
                }
            }
            else {
                if ( $type =~ /c/ ) {
                    $date
                        = $this->get_year_name( $year, $mon, $mday, $type )
                        . $this->get_month_name( $mon, $type )
                        . (
                        ($mday)
                        ? $this->kansuuji( $type, $mday ) . '日'
                            . (
                            ( $type !~ /nw/ )
                            ? '('
                                . $this->get_wday_name( $year, $mon, $mday,
                                $type )
                                . ')'
                            : ''
                            )
                        : ''
                        );
                }
                elsif ( $type =~ /\./ ) {
                    $date
                        = $year
                        . ( ($mon) ? '.' . sprintf( "%.2d", $mon ) : '' )
                        . (
                        ($mday)
                        ? '.'
                            . sprintf( "%.2d", $mday )
                            . (
                            ( $type !~ /nw/ )
                            ? ' ('
                                . $this->get_wday_name( $year, $mon, $mday,
                                $type )
                                . ')'
                            : ''
                            )
                        : ''
                        );
                }
                elsif ( $type =~ /k/i ) {
                    $date
                        = $this->kansuuji( $type, $year )
                        . (
                        ($mon) ? '/' . $this->kansuuji( $type, $mon ) : '' )
                        . (
                        ($mday)
                        ? '/'
                            . $this->kansuuji( $type, $mday )
                            . (
                            ( $type !~ /nw/ )
                            ? ' ('
                                . $this->get_wday_name( $year, $mon, $mday,
                                $type )
                                . ')'
                            : ''
                            )
                        : ''
                        );
                }
                else {
                    $date
                        = $year
                        . ( ($mon) ? '/' . sprintf( "%.2d", $mon ) : '' )
                        . (
                        ($mday)
                        ? '/'
                            . sprintf( "%.2d", $mday )
                            . (
                            ( $type !~ /nw/ )
                            ? ' ('
                                . $this->get_wday_name( $year, $mon, $mday,
                                $type )
                                . ')'
                            : ''
                            )
                        : ''
                        );
                }
            }
        }
        else {
            $date = $this->relative_date( $gap, $type, $limit );
        }
        return $date . $hour;
    }
}

sub datetype_to_format ($$) {
    my ( $this, $type ) = @_;
    my $format = '%DATE';
    if ( $type =~ /v/ ) {
        $format = '%Y-%M-%D' . ( ( $type !~ />d/ ) ? 'T%H:%MIN:%SZ' : '' );
    }
    elsif ( $type =~ /-/ || $type eq '' ) {
        $format
            . '%Y-%M-%D%_DATE'
            . ( ( $type !~ />d/ ) ? '%H:%MIN:%S' : '' );
    }
    else {
        if ( $default::english || $type =~ /e/ ) {
            if ( $type =~ /c/ ) {
                $format .= '%Month %Day'
                    . ( ( $type !~ /nw/ ) ? ' (%w)' : '' ) . ', %Y';
            }
            elsif ( $type =~ /\./ ) {
                $format .= '%M.%D.$Y' . ( ( $type !~ /nw/ ) ? ' (%w)' : '' );
            }
            else {
                $format .= '%M/%D/%Y' . ( ( $type !~ /nw/ ) ? ' (%w)' : '' );
            }
        }
        else {
            if ( $type =~ /c/ ) {
                $format
                    .= ( ( $type =~ /j/i )
                    ? ( $type =~ /K/ )
                            ? '%大年号'
                            : ( $type =~ /k/ ) ? '%漢年号'
                        : '%年号'
                    : ( $type =~ /K/ ) ? '%大年'
                    : ( $type =~ /k/ ) ? '%年'
                    :                    '%Y' )
                    . ( ( $type =~ /J/ ) ? '%月名'
                    : ( $type =~ /K/ ) ? '%大月'
                    : ( $type =~ /k/ ) ? '%月'
                    :                    '%M' )
                    . ( ( $type =~ /K/ ) ? '%大日'
                    : ( $type =~ /k/ ) ? '%日'
                    :                    '%D' )
                    . ( ( $type !~ /nw/ ) ? '(%曜)' : '' );
            }
            elsif ( $type =~ /\./ ) {
                $format
                    .= '%Y.%M.%D' . ( ( $type !~ /nw/ ) ? ' (%曜)' : '' );
            }
            elsif ( $type =~ /K/ ) {
                $format .= '%大年/%大月/%D大日'
                    . ( ( $type !~ /nw/ ) ? ' (%曜)' : '' );
            }
            elsif ( $type =~ /k/ ) {
                $format .= '%年/%月/%D日'
                    . ( ( $type !~ /nw/ ) ? ' (%曜)' : '' );
            }
            else {
                $format
                    .= '%Y/%M/%D' . ( ( $type !~ /nw/ ) ? ' (%曜)' : '' );
            }
        }
        $format .= '%_DATE';
        if ( $type !~ />d/i ) {
            $format .= ' ';
            if ( $default::english || $type =~ /e/ || $type !~ /c/ ) {
                $format .= '%H:%MIN:%S' . ( ( $type =~ /24/ ) ? '' : ' %p' );
            }
            elsif ( $type =~ /K/ ) {
                $format .= ( ( $type =~ /24/ ) ? '' : '%午' )
                    . '%h時%min分%s秒';
            }
            elsif ( $type =~ /k/ ) {
                $format .= ( ( $type =~ /24/ ) ? '' : '%午' )
                    . '%h時%min分%s秒';
            }
            else {
                $format .= ( ( $type =~ /24/ ) ? '' : '%午' )
                    . '%h時%min分%s秒';
            }
        }
    }
    $type =~ s/(vcard|>d|24|nw|[\-\.vecjk\/])//igs;
    return (wantarray) ? ( $format, $type ) : $format;
}

sub relative_date ($$;$$$) {
    my ( $this, $gap, $type, $limit, $date ) = @_;
    $date = '' if !defined $date;
    $type = '' if !defined $type;
    if ( $limit && abs($gap) < $limit ) {
        if ( $gap == 0 ) {
            return ( $default::english || $type =~ /e/ )
                ? '<strong>Today</strong>'
                : '<strong>きょう</strong>';
        }
        elsif ( $gap == -1 ) {
            return ( $default::english || $type =~ /e/ )
                ? '<strong>Yesterday</strong>'
                : '<strong>きのう</strong>';
        }
        elsif ( $gap == 1 ) {
            return ( $default::english || $type =~ /e/ )
                ? '<strong>Tomorrow</strong>'
                : '<strong>あす</strong>';
        }
        elsif ( !$default::english && $type !~ /e/ && $gap == 2 ) {
            return 'あさって';
        }
        elsif ( !$default::english && $type !~ /e/ && $gap == 3 ) {
            return 'しあさって';
        }
        elsif ( !$default::english && $type !~ /e/ && $gap == -2 ) {
            return 'おととい';
        }
        elsif ( $gap < 0 ) {
            $gap = abs($gap);
            return ( $default::english || $type =~ /e/ )
                ? $gap . ' days before'
                : $gap . '日前';
        }
        elsif ( $gap > 0 ) {
            return ( $default::english || $type =~ /e/ )
                ? $gap . ' days after'
                : $gap . '日後';
        }
    }
    return $date;
}

sub nobr_date ($$) {
    my ( $this, $date ) = @_;
    $date
        =~ s/\G(.*?(?:年|月|日|時|分|秒|\)))(?!\))/<span class="nobr">$1<\/span>/g
        if $date !~ /[<>]/;
    return $date;
}

sub is_leap_year ($$) {
    my ( $this, $year ) = @_;
    return 1 if ( $year % 4 == 0 && $year % 100 != 0 ) || $year % 400 == 0;
}

#ツェラーの公式
sub get_wday ($$$$) {
    my ( $this, $year, $mon, $mday ) = @_;
    if ( $mon == 1 || $mon == 2 ) {
        $year--;
        $mon += 12;
    }
    return
        int( $year
            + int( $year / 4 )
            - int( $year / 100 )
            + int( $year / 400 )
            + int( 2.6 * $mon + 1.6 )
            + $mday )
        % 7;
}

sub get_wday_name ($$$$;$) {
    my ( $this, $year, $mon, $mday, $type ) = @_;
    my @week;
    if ( $default::english || ( $type && $type =~ /e/ ) ) {
        return $this->wday_name( $year, $mon, $mday );
    }
    else {
        return $this->wday_jname( $year, $mon, $mday );
    }
}

sub wday_name ($$$$;$) {
    my ( $this, $year, $mon, $mday, $short ) = @_;
    my @week;
    if ($short) {
        @week = ( 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat' );
    }
    else {
        @week = (
            'Sunday',   'Monday', 'Tuesday', 'Wednesday',
            'Thursday', 'Friday', 'Saturday'
        );
    }
    return $week[ $this->get_wday( $year, $mon, $mday ) ];
}

sub wday_jname ($$$$;$) {
    my ( $this, $year, $mon, $mday, $short ) = @_;
    my @week = ( '日', '月', '火', '水', '木', '金', '土' );
    return $week[ $this->get_wday( $year, $mon, $mday ) ];
}

sub get_mday_name ($$;$) {
    my ( $this, $mday, $type ) = @_;
    $type = '' if !defined $type;
    if ( $default::english || $type =~ /e/ ) {
        return
              ( $type =~ /c/ ) ? $this->mday_number($mday)
            : ($mday)          ? $mday
            :                    '';
    }
    else {
        return ($mday)
            ? $this->kansuuji( $type, $mday )
            . ( ( $type =~ /c/ ) ? '日' : '' )
            : '';
    }
}

sub mday_number ($$) {
    my ( $this, $mday ) = @_;
    if ( !$mday ) {
        return '';
    }
    elsif ( $mday =~ /1$/ && $mday !~ /11$/ ) {
        return $mday . 'st';
    }
    elsif ( $mday =~ /2$/ && $mday !~ /12$/ ) {
        return $mday . 'nd';
    }
    elsif ( $mday =~ /3$/ && $mday !~ /13$/ ) {
        return $mday . 'rd';
    }
    else {
        return $mday . 'th';
    }
}

sub get_month_name ($$;$) {
    my ( $this, $mon, $type ) = @_;
    $type = '' if !defined $type;
    if ( $default::english || $type =~ /e/ ) {
        return
              ($mon)
            ? ( $type =~ /c/ )
                ? $this->month_name($mon)
                : $mon
            : '';
    }
    else {
        if ( $type =~ /c/ ) {
            return
                  ($mon)
                ? ( $type =~ /J/ )
                    ? $this->tsuki( $mon - 1 )
                    : $this->kansuuji( $type, $mon ) . '月'
                : '';
        }
        else {
            return ($mon) ? $this->kansuuji( $type, $mon ) : '';
        }
    }
}

sub month_name ($$;$) {
    my ( $this, $mon, $short ) = @_;
    my @month = (
        '',       'January',   'February', 'March',
        'April',  'May',       'June',     'July',
        'August', 'September', 'October',  'November',
        'December'
    );
    return
          ($mon)
        ? ($short)
            ? substr( $month[$mon], 0, 3 )
            : $month[$mon]
        : '';
}

sub get_year_name ($$$$;$) {
    my ( $this, $year, $mon, $mday, $type ) = @_;
    $type = '' if !defined $type;
    if ( $default::english || $type =~ /e/ ) {
        return $year;
    }
    elsif ( $type =~ /c/ ) {
        return $this->kansuuji(
            $type,
            (   ( $type && $type =~ /j/i )
                ? $this->nengo( $year, $mon - 1, $mday )
                : $year
            )
        ) . '年';
    }
    else {
        return $this->kansuuji( $type, $year );
    }
}

sub get_yday ($$$$) {
    my ( $this, $year, $mon, $yday ) = @_;
    my (@mdays);
    $yday = 0 if !$yday;
    $mon += 0;
    @mdays = ( 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
    $mdays[2]++ if $this->is_leap_year($year);
    foreach my $d ( 0 .. $mon ) {
        $yday += $mdays[$d];
    }
    $yday--;
    $yday = 0 if $yday < 0;
    return $yday;
}

sub get_hour ($$$;$$) {
    my ( $this, $type, $hour, $min, $sec ) = @_;
    return '' if !defined $hour || $hour eq '';
    my ( $h, $hj );
    if ( $type =~ /24/ ) {
        $h  = '';
        $hj = '';
    }
    elsif ( $hour >= 12 ) {
        $hour -= 12;
        $h  = ' p.m.';
        $hj = '午後';
    }
    elsif ( $hour ne '' && $hour < 12 ) {
        $h  = ' a.m.';
        $hj = '午前';
    }

    if ( defined $hour && $hour ne '' && defined $min && $min ne '' ) {
        $sec
            = ( defined $sec && $sec ne '' )
            ? ( $default::english || $type =~ /e/ || $type !~ /c/ )
                ? ':' . sprintf( "%.2d", $sec )
                : $sec . '秒'
            : '';
        $hour
            = ( $default::english || $type =~ /e/ || $type !~ /c/ )
            ? ' ' . $hour . ':' . sprintf( "%.2d", $min ) . $sec . $h
            : ' ' . $hj . $hour . '時' . $min . '分' . $sec;
    }
    elsif ( defined $hour && $hour ne '' ) {
        $hour
            = ( $default::english || $type =~ /e/ )
            ? ' ' . $hour . 'h' . $h
            : ' ' . $hj . $hour . '時';
    }
    return ( $default::english || $type =~ /e/ )
        ? $hour
        : $this->kansuuji( $type, $hour );
}

sub get_difference_of_yday ($$$;$) {
    my ( $this, $year_b, $yday_b, $mday ) = @_;
    my ( @localtime, $year_a, $yday_a, $day );

    @localtime = localtime(time);
    $yday_a    = $localtime[7];
    $year_a    = $localtime[5];
    $year_a += 1900;

    $yday_b = $this->get_yday( $year_b, $yday_b, $mday )
        if defined $mday && $mday ne '';
    eval { $_[1] = $year_b };

    $day = 365;
    $day++
        if ( $year_b > $year_a && $this->is_leap_year($year_a) )
        || ( $year_a > $year_b && $this->is_leap_year($year_b) );

    if ( $year_b > $year_a ) {
        return $day - $yday_a + $yday_b;
    }
    elsif ( $year_a > $year_b ) {
        return $yday_b - $yday_a - $day;
    }
    else {
        return $yday_b - $yday_a;
    }
}

#日付チェック
sub is_impossible_mday ($$$$) {
    my ( $this, $y, $m, $d ) = @_;
    if ( $d > $this->max_mday( $y, $m ) ) {
        return 1;
    }
    else {
        return 0;
    }
}

sub max_mday ($$$) {
    my ( $this, $y, $m ) = @_;
    return ( $m == 4 || $m == 6 || $m == 9 || $m == 11 )
        ? 30
        : ( $m == 2 ) ? ( $this->is_leap_year($y) )
            ? 29
            : 28
        : 31;
}

# 漢字化
sub kansuuji ($$@) {
    my ( $this, $option, @num ) = @_;
    return (wantarray) ? @num : $num[0] if $option !~ /k/i;
    my $suushi = ( $option =~ /[KJ]/ ) ? 1 : 0;
    my $daiji  = ( $option =~ /K/ )    ? 1 : 0;
    @num = map( $this->num_to_kazu( $_, $suushi, $daiji ), @num );
    return (wantarray) ? @num : $num[0];
}

sub num_to_kazu ($$;$$) {
    my ( $this, $num, $suushi, $daiji ) = @_;
    $num
        =~ s/([\-\+]?[0-9,]+\.?[0-9,]*)/$this->_num_to_kazu($1,$suushi,$daiji)/ges;
    return $num;
}

sub _num_to_kazu ($$;$$) {
    my ( $this, $num, $suushi, $daiji ) = @_;
    $num =~ s/,//g;
    my @suu = ($daiji)
        ? (
        '〇', '壱', '弐', '参', '四', '五',
        '六', '七', '八', '九'
        )
        : (
        '〇', '一', '二', '三', '四', '五',
        '六', '七', '八', '九'
        );
    if ( !$suushi && !$daiji ) {
        $num =~ s/\./・/g;
        $num =~ s/([0-9])/$suu[$1]/g;
        return ( ( $num =~ s/^\-// ) ? 'マイナス'
            : ( $num =~ s/^\+// ) ? 'プラス'
            :                       '' )
            . $num;
    }
    my $fu
        = ( $num =~ s/^\-// ) ? '負の'
        : ( $num =~ s/^\+// ) ? '正の'
        :                       '';
    my $seisu = '';
    my $syosu = '';
    my @man   = ( '', '十', '百', '千' );
    my @tani  = (
        '',             '万',       '億',       '兆',
        '京',          '垓',       '𥝱',      '穰',
        '溝',          '澗',       '正',       '載',
        '極',          '恒河沙', '阿僧祇', '那由他',
        '不可思議', '無量大数'
    );
    my @syosu_tani = (
        '割',       '分',    '厘',       '毛',
        '糸、絲', '忽',    '微',       '繊',
        '沙',       '塵',    '埃',       '渺',
        '漠',       '模糊', '逡巡',    '須臾',
        '瞬息',    '弾指', '刹那',    '六徳',
        '虚空',    '清浄', '阿頼耶', '阿摩羅',
        '涅槃寂静'
    );
    if ($daiji) {
        $man[1]  = '拾';
        $tani[1] = '萬';
    }
    my ( $int, $dec )
        = ( $num =~ /^([0-9]+)\.?([0-9]*)$/ ) ? ( $1, $2 ) : ( $num, '' );
    if ( $int ne '' ) {
        my $kurai        = 0;
        my $current_tani = '';
        while ( $int =~ s/([0-9])$// ) {
            my $n = $1;
            my $k = $kurai % 4;
            if ( $k == 0 ) {
                if ($n) {
                    $seisu = $suu[$n] . $tani[ int( $kurai / 4 ) ] . $seisu;
                    $current_tani = '';
                }
                else {
                    $current_tani = $tani[ int( $kurai / 4 ) ];
                }
            }
            elsif ($n) {
                $seisu
                    = ( ( $n == 1 && $k < 3 ) ? '' : $suu[$n] )
                    . $man[$k]
                    . $current_tani
                    . $seisu;
                $current_tani = '';
            }
            $kurai++;
        }
        $seisu =~ s/^(拾|百)/壱$1/ if $daiji;
    }
    if ( $dec ne '' ) {
        my $kurai = 0;
        while ( $dec =~ s/^([0-9])// ) {
            $syosu .= $suu[$1] . $syosu_tani[$kurai] if $1;
            $kurai++;
        }
    }
    return $fu . $seisu . ( ( $seisu && $syosu ) ? 'と' : '' ) . $syosu;
}

# 年号化
sub nengo ($$$$) {
    my ( $this, $year, $mon, $mday ) = @_;
    my ( $id, $nengo );
    $mon++;
    $mday = 0 if !$mday;
    $mday += 0;
    $id = $year . sprintf( "%.2d", $mon ) . sprintf( "%.2d", $mday );
    if ( $id - 19890108 > 0 ) {
        $year -= 1988;
        $nengo = '平成';
    }
    elsif ( $id - 19261225 > 0 ) {
        $year -= 1925;
        $nengo = '昭和';
    }
    elsif ( $id - 19120730 > 0 ) {
        $year -= 1911;
        $nengo = '大正';
    }
    elsif ( $id - 19680908 > 0 ) {
        $year -= 1967;
        $nengo = '明治';
    }
    else {
        $nengo = '西暦';
    }
    return $nengo . ( ( $year == 1 ) ? '元' : $year );
}

# 陰暦月名
sub tsuki ($@) {
    my ( $this, @mon ) = @_;
    my (@tsuki);
    @tsuki = (
        '睦月', '如月', '弥生', '卯月',    '皐月', '水無月',
        '文月', '葉月', '長月', '神無月', '霜月', '師走'
    );
    map( $_
            =~ s/(^|[^0-9])([1]{0,1}[0-9](?![0-9]))(月|$)/$1$tsuki[(($1 ne '' || $3 eq '月') ? $2-1 : $2)]/g,
        @mon );
    return (wantarray) ? @mon : $mon[0];
}

# 日付形式メニュー表示
sub print_date_type_menu ($$$;$$$$$) {
    my ( $this, $prefix, $default, $description, $option, $format_required,
        $default_format )
        = @_;
    my $time = time;
    $option = '' if !defined $option;
    print $this->mk_select_menu(
        $prefix . 'datetype',
        $description,
        '', $default,
        '-' . $option => $this->date( $time, 0, 'gmt-' . $option ),
        '/' . $option => $this->date( $time, 0, 'gmt/' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( '/24'
                    . $option => $this->date( $time, 0, 'gmt/24' . $option ) )
        ),
        '/nw' . $option => $this->date( $time, 0, 'gmt/nw' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( '/24nw'
                    . $option =>
                    $this->date( $time, 0, 'gmt/24nw' . $option ) )
        ),
        'c' . $option => $this->date( $time, 0, 'gmtc' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'c24'
                    . $option => $this->date( $time, 0, 'gmtc24' . $option ) )
        ),
        'cnw' . $option => $this->date( $time, 0, 'gmtcnw' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'c24nw'
                    . $option =>
                    $this->date( $time, 0, 'gmtc24nw' . $option ) )
        ),
        'cj' . $option => $this->date( $time, 0, 'gmtcj' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'c24j'
                    . $option => $this->date( $time, 0, 'gmtc24j' . $option )
            )
        ),
        'cjnw' . $option => $this->date( $time, 0, 'gmtcjnw' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'cj24nw'
                    . $option =>
                    $this->date( $time, 0, 'gmtcj24nw' . $option ) )
        ),
        'cjk' . $option => $this->date( $time, 0, 'gmtcjk' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'c24jk'
                    . $option =>
                    $this->date( $time, 0, 'gmtc24jk' . $option ) )
        ),
        'cjknw' . $option => $this->date( $time, 0, 'gmtcjknw' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'cjk24nw'
                    . $option =>
                    $this->date( $time, 0, 'gmtcjk24nw' . $option ) )
        ),
        'cJk' . $option => $this->date( $time, 0, 'gmtcJk' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'c24Jk'
                    . $option =>
                    $this->date( $time, 0, 'gmtc24Jk' . $option ) )
        ),
        'cJknw' . $option => $this->date( $time, 0, 'gmtcJknw' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'cJk24nw'
                    . $option =>
                    $this->date( $time, 0, 'gmtcJk24nw' . $option ) )
        ),
        '.' . $option => $this->date( $time, 0, 'gmt.' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( '.24'
                    . $option => $this->date( $time, 0, 'gmt.24' . $option ) )
        ),
        '.nw' . $option => $this->date( $time, 0, 'gmt.nw' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( '.24nw'
                    . $option =>
                    $this->date( $time, 0, 'gmt.24nw' . $option ) )
        ),
        'e' . $option => $this->date( $time, 0, 'gmte' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'e24'
                    . $option => $this->date( $time, 0, 'gmte24' . $option ) )
        ),
        'enw' . $option => $this->date( $time, 0, 'gmtenw' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'e24nw'
                    . $option =>
                    $this->date( $time, 0, 'gmte24nw' . $option ) )
        ),
        'ec' . $option => $this->date( $time, 0, 'gmtec' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'ec24'
                    . $option => $this->date( $time, 0, 'gmtec24' . $option )
            )
        ),
        'ecnw' . $option => $this->date( $time, 0, 'gmtecnw' . $option ),
        (   ( $option && $option =~ />d/ ) ? ()
            : ( 'ec24nw'
                    . $option =>
                    $this->date( $time, 0, 'gmtec24nw' . $option ) )
        ),
    );
    if ($format_required) {
        my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday )
            = localtime($time);
        $year += 1900;
        $mon++;
        print '<textarea name="'
            . $prefix
            . 'dateformat" id="'
            . $prefix
            . 'dateformat" cols="50" rows="3" wrap="off" class="content-editor-area" title="日付と時刻の書式を設定します。上記メニューよりもこちらのテンプレートが優先されます。">'
            . $this->escape($default_format)
            . '</textarea>';
        print '<div class="toolbox">';
        print
            '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'',
            $prefix,
            'dateformat\').setAutoFit(this.checked);" />フィット</label> ';
        print
            '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\''
            . $prefix
            . 'dateformat\').quotationByMenu(this)" title="これらのパラメータは、書き出し時に対応する日付または時刻に変換されます。">';
        print
            '<option value="">▼日付と時間のパラメータを挿入する</option>';
        print '<option value=""></option>';
        print '<option value="%DATE">%DATE=日付の開始位置</option>';
        print '<option value="%_DATE">%_DATE=日付の終了位置</option>';
        print '<option value=""></option>';
        print '<option value="%Y">%Y=年(4桁) ' . $year . '</option>';
        print '<option value="%y">%y=年(2桁) '
            . substr( $year, -2 )
            . '</option>';
        print '<option value="%年">%年=年(漢字) '
            . $this->num_to_kazu($year)
            . '</option>';
        print '<option value="%数年">%数年=年(漢数詞) '
            . $this->num_to_kazu( $year, 1 )
            . '</option>';
        print '<option value="%年号">%年号=年号(数字) '
            . $this->nengo( $year, $mon - 1, $mday )
            . '</option>';
        print '<option value="%漢年号">%漢年号=年号(漢字) '
            . $this->num_to_kazu( $this->nengo( $year, $mon - 1, $mday ) )
            . '</option>';
        print '<option value="%数年号">%数年号=年(漢数詞) '
            . $this->num_to_kazu( $year, 1 )
            . '</option>';
        print '<option value="%大年">%大年=年(大字) '
            . $this->num_to_kazu( $year, 1, 1 )
            . '</option>';
        print '<option value="%大年号">%大年号=年号(大字) '
            . $this->num_to_kazu( $this->nengo( $year, $mon - 1, $mday ), 1,
            1 )
            . '</option>';
        print '<option value=""></option>';
        print '<option value="%Month">%Month=月の名前(英字) '
            . $this->month_name($mon)
            . '</option>';
        print '<option value="%Mon">%Mon=月の名前(英字略記) '
            . $this->month_name( $mon, 1 )
            . '</option>';
        print '<option value="%M">%M=月(2桁) '
            . sprintf( "%.2d", $mon )
            . '</option>';
        print '<option value="%m">%m=月 ' . $mon . '</option>';
        print '<option value="%月">%月=月(漢字) '
            . $this->num_to_kazu($mon)
            . '</option>';
        print '<option value="%数月">%数月=月(漢数詞) '
            . $this->num_to_kazu( $mon, 1 )
            . '</option>';
        print '<option value="%月名">%月名=日本の月の名前 '
            . $this->tsuki( $mon - 1 )
            . '</option>';
        print '<option value="%大月">%大月=月(大字) '
            . $this->num_to_kazu( $mon, 1, 1 )
            . '</option>';
        print '<option value=""></option>';
        print '<option value="%Day">%Day=日(英字) '
            . $this->mday_number($mday)
            . '</option>';
        print '<option value="%D">%D=日(2桁) '
            . sprintf( "%.2d", $mday )
            . '</option>';
        print '<option value="%d">%d=日 ' . $mday . '</option>';
        print '<option value="%日">%日=日(漢字) '
            . $this->num_to_kazu($mday)
            . '</option>';
        print '<option value="%数日">%数日=日(漢数詞) '
            . $this->num_to_kazu( $mday, 1 )
            . '</option>';
        print '<option value="%大日">%大日=日(大字) '
            . $this->num_to_kazu( $mday, 1, 1 )
            . '</option>';
        print '<option value=""></option>';
        print '<option value="%W">%W=曜日(英字) '
            . $this->wday_name( $year, $mon, $mday )
            . '</option>';
        print '<option value="%w">%w=曜日(英字略記) '
            . $this->wday_name( $year, $mon, $mday, 1 )
            . '</option>';
        print '<option value="%曜">%曜=曜日(漢字) '
            . $this->wday_jname( $year, $mon, $mday )
            . '</option>';
        print '<option value=""></option>';
        print '<option value="%p">%p=a.m./p.m '
            . ( ( $hour < 12 ) ? 'a.m.' : 'p.m.' )
            . '</option>';
        print '<option value="%P">%P=A.M./P.M. '
            . ( ( $hour < 12 ) ? 'A.M.' : 'P.M.' )
            . '</option>';
        print '<option value="%午">%午=午前/午後 '
            . ( ( $hour < 12 ) ? '午前' : '午後' )
            . '</option>';
        print '<option value=""></option>';
        print '<option value="%H">%H=時(2桁) '
            . sprintf( "%.2d", $hour )
            . '</option>';
        print '<option value="%h">%h=時 ' . $hour . '</option>';
        print '<option value="%時">%時=時(漢字) '
            . $this->num_to_kazu($hour)
            . '</option>';
        print '<option value="%数時">%数時=時(漢数詞) '
            . $this->num_to_kazu( $hour, 1 )
            . '</option>';
        print '<option value="%大時">%大時=時(大字) '
            . $this->num_to_kazu( $hour, 1, 1 )
            . '</option>';
        print '<option value=""></option>';
        print '<option value="%MIN">%MIN=分(2桁) '
            . sprintf( "%.2d", $min )
            . '</option>';
        print '<option value="%min">%min=分 ' . $min . '</option>';
        print '<option value="%分">%分=分(漢字) '
            . $this->num_to_kazu($min)
            . '</option>';
        print '<option value="%数分">%数分=分(漢数詞) '
            . $this->num_to_kazu( $min, 1 )
            . '</option>';
        print '<option value="%大分">%大分=分(大字) '
            . $this->num_to_kazu( $min, 1, 1 )
            . '</option>';
        print '<option value=""></option>';
        print '<option value="%S">%S=秒(2桁) '
            . sprintf( "%.2d", $sec )
            . '</option>';
        print '<option value="%s">%s=秒 ' . $sec . '</option>';
        print '<option value="%秒">%秒=秒(漢字) '
            . $this->num_to_kazu($sec)
            . '</option>';
        print '<option value="%数秒">%数秒=秒(漢数詞) '
            . $this->num_to_kazu( $sec, 1 )
            . '</option>';
        print '<option value="%大秒">%大秒=秒(大字) '
            . $this->num_to_kazu( $sec, 1, 1 )
            . '</option>';
        print '</select>';
    }
}

sub mk_select_menu ($$$$$@) {
    my ( $this, $name, $title, $menu_option, $selected_option, @option_pair )
        = @_;
    my ( $menu, $value, $option, $selected_enabled );
    $selected_enabled = 0;
    if ( defined $selected_option ) {
        $selected_enabled = 1;
        $selected_enabled = 2
            if ref $selected_option
            && @$selected_option
            && $menu_option =~ / multiple/;
    }
    $menu
        = '<select'
        . ( ( defined $name && $name ne '' )
        ? ' name="' . $name . '" id="' . $name . '"'
        : '' )
        . ( ( defined $title && $title ne '' )
        ? ' title="' . $this->escape($title) . '"'
        : '' )
        . (
        ( defined $menu_option && $menu_option ne '' ) ? $menu_option : '' )
        . '>';
    while ( ( $value, $option ) = splice( @option_pair, 0, 2 ) ) {
        $menu
            .= '<option value="'
            . $this->escape($value) . '"'
            . (
            (   (   defined $value
                        && ( $selected_enabled == 2
                        && grep( $_ eq $value, @$selected_option ) )
                        || ( $selected_enabled == 1
                        && $value eq $selected_option )
                )
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . ( ( defined $option && $option ne '' )
            ? $this->escape_except_html_entity($option)
            : '' )
            . '</option>';
    }
    $menu .= '</select>';
    return $menu;
}

sub mk_checkbox ($$$$;$$$) {
    my ( $this, $type, $name, $value, $input_value, $title, $input_option )
        = @_;
    $type = 'checkbox' if !defined $type || $type eq '';
    return
          '<input type="'
        . $type
        . '" name="'
        . $name
        . '" id="'
        . $name
        . '" value="'
        . $this->escape($value) . '"'
        . (
        ( defined $value && defined $input_value && $value eq $input_value )
        ? ' checked="checked"'
        : '' )
        . ' />';
}

sub jsquot ($$) {
    my ( $this, $text ) = @_;
    $text
        =~ s/\&(?!(?:#[0-9]{2,6}|#x[a-fA-F0-9]+|[a-zA-Z0-9]{2,10});)/\&amp;/g;
    $text =~ s/"/&quot;/g;
    $text =~ s/</&lt;/g;
    $text =~ s/>/&gt;/g;
    $text =~ s/'/\\'/g;
    $text =~ s/\x0D\x0A/\\n/g;
    $text =~ s/\x0A/\\n/g;
    $text =~ s/\\n/\\r\\n/g if $ENV{'HTTP_USER_AGENT'} =~ /MSIE/;
    return $text;
}

########################################################################
#セッションモジュール呼出
sub use_session ($) {
    my $this = shift;
    return 0;
}

#クッキーデータの取得
sub read_cookie ($;$) {
    my ( $this, $cname ) = @_;
    $cname = lc $main::m
        if ( !defined $cname || $cname eq '' ) && $main::m ne '';
    $cname = 'DATA' if !defined $cname || $cname eq '';
    if ( !defined $cgi::user_info{$cname} || $cgi::user_info{$cname} eq '' ) {
        if (   $env::session_enabled
            && $env::session_enabled == 2
            && $this->use_session )
        {
            $session::id
                = $this->read_cookie_direct( $this->login_cookie_name )
                if !$session::id && !$session::is_deleted;
            $cgi::Session->read_by_uri( $session::id, $ENV{'SCRIPT_NAME'},
                \%cgi::user_info, 1 );
        }
        else {
            $this->read_all_cookie;
        }
    }
    if ( ref $cgi::user_info{$cname} ) {
        $cookie::read{$cname} = 1;
    }
    elsif ( defined $cgi::user_info{$cname} && $cgi::user_info{$cname} ne '' )
    {
        $cookie::read{$cname} = 1
            if $this->url_decode( $cgi::user_info{$cname}, 'main', 1, 'utf8',
            'lf', '', 1 );
    }
    else {
        delete $cgi::user_info{$cname};
    }
}

# クッキーデータの取得
sub read_cookie_direct ($$;$) {
    my ( $this, $cname ) = @_;
    $this->read_all_cookie;
    if ( defined $cgi::user_info{$cname} ) {
        $cookie::read{$cname} = 1;
        return $cgi::user_info{$cname};
    }
    else {
        return '';
    }
}

sub read_all_cookie ($) {
    my $this = shift;
    if ( $ENV{'HTTP_COOKIE'} && !%cgi::user_info ) {
        %cgi::user_info = ();
        foreach ( split( /;[\x09\x0A\x0C\x0D\x20]*/, $ENV{'HTTP_COOKIE'} ) ) {
            my ( $k, $v ) = split( /=/, $_, 2 );
            $cgi::user_info{$k} = $v;
        }
    }
}

########################################################################
#クッキーデータの作成
sub set_cookie ($$$$$@) {
    my ( $this, $lifetime, $cname, $domain, $path, $data ) = @_;
    return '' if !$data || !%$data;
    return $this->set_cookie_direct( $lifetime, $cname,
        $this->url_encoded_query( $data, 1 ),
        $domain, $path, 1 );
}

sub set_cookie_direct ($;$$$$$$) {
    my ( $this, $lifetime, $cname, $cookie, $domain, $path, $session_enabled )
        = @_;
    my ($expires);
    return '' if !defined $cookie || $cookie eq '';
    $cname = ( $main::m ne '' ) ? uc($main::m) : 'DATA'
        if !defined $cname || $cname eq '';
    $path = $this->path_change( $path, $ENV{'SCRIPT_NAME'}, 'abs' )
        if $path ne '' && $path !~ m!^/!;
    if (   $session_enabled
        && $env::session_enabled
        && $env::session_enabled == 2
        && $this->use_session )
    {
        $session::id = $this->read_cookie_direct( $this->login_cookie_name )
            if !$session::id && !$session::is_deleted;
        $session::id
            = $cgi::Session->save_by_name( $session::id, $path, $cname,
            $cookie );
        return ($session::id)
            ? $this->set_cookie_direct( $env::cookie_lifetime,
            $this->login_cookie_name, $session::id, $env::cgi_server,
            $env::cookie_path )
            : '';
    }
    else {
        $path = ' path=' . ( ( $path ne '' ) ? $path : '/' ) . ';';
        $expires = ' expires=' . $this->get_expires($lifetime) . ';'
            if $lifetime;
        if ( defined $domain && $domain ne '' ) {
            $domain = lc $1 if $domain =~ m!^https?://([^/]+)!i;
            $domain = ' domain=' . $domain . ';'
                if defined $domain && $domain ne '';
        }
        $this->{'Set_Cookie'}
            .= 'Set-Cookie: '
            . $cname . '='
            . $cookie . ';'
            . $expires
            . $path
            . $domain
            . ( ( $this->is_ssl ) ? 'secure' : '' ) . "\x0A";
        return $this->{'Set_Cookie'};
    }
}

sub reset_cookie ($) {
    my $this = shift;
    delete $this->{'Set_Cookie'};
    undef $main::set_cookie_id;
    return $this;
}

########################################################################
#クッキー消去期日の計算（$expires = $this->get_expires(limit);　limit日後に消去する場合の値を返す）
sub get_expires ($$) {
    my ( $this, $lifetime ) = @_;
    my ( $time, @week, @month, @gmt );
    $time = time + $lifetime * 86400;
    @week = (
        'Sunday',   'Monday', 'Tuesday', 'Wednesday',
        'Thursday', 'Friday', 'Saturday'
    );
    @month = (
        'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
        'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'
    );
    @gmt = gmtime($time);
    $gmt[0] = sprintf( "%.2d", $gmt[0] );
    $gmt[1] = sprintf( "%.2d", $gmt[1] );
    $gmt[2] = sprintf( "%.2d", $gmt[2] );
    $gmt[3] = sprintf( "%.2d", $gmt[3] );
    $gmt[5] += 1900 if $gmt[5] < 1900;
    return
        "$week[$gmt[6]], $gmt[3]-$month[$gmt[4]]-$gmt[5] $gmt[2]:$gmt[1]:$gmt[0] GMT";
}

########################################################################
#URLを求める
sub my_url ($;$) {
    my ( $this, $is_addr ) = @_;
    if ( defined $env::cgi_server && $env::cgi_server ) {
        return
              ( ( $this->is_ssl ) ? 'https://' : 'http://' )
            . $env::cgi_server
            . $ENV{'SCRIPT_NAME'};
    }
    elsif ($is_addr) {
        return $this->http_my_addr . $ENV{'SCRIPT_NAME'};
    }
    else {
        return $this->http_my_domain . $ENV{'SCRIPT_NAME'};
    }
}

sub is_ssl ($) {
    my $this = shift;
    return ( $ENV{'HTTPS'}
            || ( $ENV{'SERVER_PORT'} && $ENV{'SERVER_PORT'} == 443 ) )
        ? 1
        : 0;
}

########################################################################
#URLを求める
sub full_path ($$;$) {
    my ( $this, $path, $no_fix ) = @_;
    return
          ( defined $path && $path ne '' )
        ? ( $path =~ /^https?:\/\//i )
            ? $path
            : $this->http_my_domain
            . $this->path_change( $path, $ENV{'SCRIPT_NAME'}, 'abs', $no_fix )
        : '';
}

sub full_addr ($$;$) {
    my ( $this, $path, $no_fix ) = @_;
    return
          ( defined $path && $path ne '' )
        ? ( $path =~ /^https?:\/\//i )
            ? $path
            : $this->http_my_addr
            . $this->path_change( $path, $ENV{'SCRIPT_NAME'}, 'abs', $no_fix )
        : '';
}

########################################################################
#IPアドレス->HOSTネーム
sub to_host ($$) {
    my ( $this, $addr ) = @_;
    my ($host_name);
    return '' if !defined $addr;
    return $addr if $addr =~ /[^0-9\.]/;
    $host_name = gethostbyaddr( pack( "C4", split( /\./, $addr ) ), 2 );
    return $addr if !defined $host_name || $host_name eq '';
    return lc($host_name);
}

########################################################################
#HOSTネーム->IPアドレス
sub to_ip ($$) {
    my ( $addr, $ip );
    my ( $this, $host_name ) = @_;
    return '' if !defined $host_name;
    return lc($host_name) if $host_name !~ /[^0-9\.]/;
    $addr = ( gethostbyname($host_name) )[4];
    $ip = sprintf( "%u.%u.%u.%u", unpack( "C*", $addr ) ) if defined $addr;
    return lc($host_name) if !$ip || $ip eq '0.0.0.0';
    return $ip;
}

########################################################################
#ドメイン名取得
sub http_my_domain ($) {
    my $this = shift;
    $cgi::http_domain
        = ( ($env::ssl) ? 'https://' : 'http://' )
        . $env::server_name
        . ( ( $ENV{'SERVER_PORT'} != 80 && $ENV{'SERVER_PORT'} != 443 )
        ? ':' . $ENV{'SERVER_PORT'}
        : '' )
        if !defined $cgi::http_domain || !$cgi::http_domain;
    return $cgi::http_domain;
}

sub http_my_addr ($) {
    my $this = shift;
    return ( ($env::ssl) ? 'https://' : 'http://' )
        . ( ( $ENV{'SERVER_ADDR'} ) ? $ENV{'SERVER_ADDR'}
        : $this->to_ip($env::server_name) )
        . ( ( $ENV{'SERVER_PORT'} != 80 && $ENV{'SERVER_PORT'} != 443 )
        ? ':' . $ENV{'SERVER_PORT'}
        : '' );
}

########################################################################
#ホームディレクトリまでのサーバ内パス･URLパスを求める
sub home_location ($) {
    my $this = shift;
    if (   !defined $cgi::home_location
        || !defined $cgi::user_directory
        || !defined $cgi::document_root )
    {
        my ( $mypath, $d, @dir );
        $cgi::document_root = $this->get_cwd;
        $mypath             = $ENV{'SCRIPT_NAME'};
        $mypath =~ s!/[^/]*$!!;
        @dir = split( '/', $mypath );
        while ( $d = pop(@dir) ) {
            last unless $cgi::document_root =~ s!/$d$!!;
        }
        push( @dir, $d );
        $cgi::user_directory = join( '/', @dir );
        $cgi::home_location = $this->http_my_domain . $cgi::user_directory;
    }
    return $cgi::home_location;
}

sub user_directory ($;$) {
    my ( $this, $dir_required ) = @_;
    $this->home_location if !defined $cgi::user_directory;
    return ( !$dir_required && $cgi::user_directory eq '' )
        ? '/'
        : $cgi::user_directory;
}

sub document_root ($) {
    my $this = shift;
    $this->home_location if !defined $cgi::document_root;
    return $cgi::document_root;
}

sub home_directory ($;$) {
    my ( $this, $dir_required ) = @_;
    my $path = $this->path_change( '../../', $ENV{'SCRIPT_NAME'}, 'abs' );
    return ( $dir_required && $path eq '/' ) ? '' : $path;
}

########################################################################
#URLパスとサーバ内パスを変換する
sub path_conv ($$;$) {
    my ( $this, $path, $op ) = @_;
    $this->home_location
        if !defined $cgi::document_root || !defined $cgi::user_directory;
    $path = $this->path_change( $path, $ENV{'SCRIPT_NAME'}, 'abs' )
        if $path !~ m!^/!;
    return ( $op eq 'url' ) ? $cgi::user_directory : $cgi::document_root
        if $path eq '' || $path eq '/';
    return $path if $path !~ m!^/!;
    return $path
        if $op eq 'url'
        && $cgi::user_directory
        && ( $path =~ /^$cgi::user_directory/
        || $cgi::user_directory =~ /^$path/ );
    return $path if $op eq 'path' && $path =~ /^$cgi::document_root/;
    if ( $cgi::document_root =~ /^$path/ ) {
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            'パスの変換に失敗しました。ネットワークから見えないパスです。'
        );
        return;
    }
    unless ( $path =~ s/^$cgi::document_root/$cgi::user_directory/ ) {
        $path =~ s/^$cgi::user_directory/$cgi::document_root/;
    }
    return $path;
}

######################################################################
# 相対パス<->絶対パス
sub path_change ($$;$$) {
    my ( $this, $url, $base, $op, $no_fix ) = @_;
    return $url
        if ( defined $base && $base =~ /^\./ )
        || !defined $url
        || $url eq ''
        || $url =~ /^#/;
    $base = $ENV{'SCRIPT_NAME'} if !defined $base || $base eq '';
    if ( $url =~ m!^https?://!i || $url =~ m!^/! ) {
        return $url if defined $op && $op eq 'abs';
        my $url_d  = ( $url =~ s!^(https?://.*?/|/)!!i )  ? $1 : '';
        my $base_d = ( $base =~ s!^(https?://.*?/|/)!!i ) ? $1 : '';
        return $url_d . $url
            if $url_d ne $base_d && $url_d ne '/' && $base_d ne '/';
        my @dir = split( '/', $base );
        while ( my $d = shift(@dir) ) {
            last unless $url =~ s!^$d(/|$)!!;
        }
        foreach ( 0 .. $#dir ) {
            $url = '../' . $url;
        }
        $url = './' . $url if $url !~ /^\.\.\//;
    }
    elsif ( $url !~ /^[0-9a-zA-Z_]+:/ ) {
        return $url if defined $op && $op eq 'rel';
        $base =~ s!/[^/]*$!!;
        my ($d);
        ( $d = $url ) =~ s!^\./!!;
        while ( $d =~ s!^\.\./!! ) {
            $base =~ s!/[^/]*$!!;
        }
        $url = $base . ( ( $d ne '' ) ? '/' . $d : '' );
        $url = '/' if !$no_fix && $url eq '';
    }
    return $url;
}

######################################################################
# 起点の変更
sub get_cwd ($) {
    my $this = shift;
    my $cwd;
    if ( $ENV{'SCRIPT_FILENAME'} ) {
        $cwd = $ENV{'SCRIPT_FILENAME'};
        $cwd =~ s!/[^/]*$!!;
    }
    else {
        chdir;
        use Cwd;
        $cwd = cwd() || getcwd;
    }
    return $cwd;
}

sub base_change ($$$;$$) {
    my ( $this, $url, $base, $option, $server ) = @_;
    return $url
        if !defined $url
        || $url =~ /:/
        || $url =~ /\&#58;/
        || $url =~ /^#/
        || ( $option && $option eq 'inherit' && $url =~ m!^/! );
    my $here = ($server) ? $this->get_cwd . '/this.cgi' : $ENV{'SCRIPT_NAME'};
    $here =~ s/[^\/]+$/x.txt/;
    return $this->path_change( $url, $here, 'abs' )
        if defined $option && $option eq 'abs';
    return $url if !defined $base || $base eq '';
    $base .= 'index.html' if $base =~ m!/$!;
    return $this->path_change( $this->path_change( $url, $here, 'abs' ),
        $this->path_change( $base, $here, 'abs' ), 'rel' );
}

#もとに戻す
sub un_base_change ($$$;$$) {
    my ( $this, $url, $base, $option, $server ) = @_;
    return $url
        if !defined $url
        || $url =~ /:/
        || $url =~ /\&#58;/
        || $url =~ /^#/
        || ( $option && $option eq 'inherit' && $url =~ m!^/! );
    my $here = ($server) ? $this->get_cwd . '/this.cgi' : $ENV{'SCRIPT_NAME'};
    $here =~ s/[^\/]+$/x.txt/;
    return $this->path_change(
        $this->path_change(
            $url, $this->path_change( $base, $here, 'abs' ), 'abs'
        ),
        $here,
        ( ($option) ? $option : 'rel' )
    );
}

sub rel_to_abs ($$;$$$$$) {
    my ( $this, $html, $path, $full, $delete_comment, $anchor, $domain ) = @_;
    $domain = ( $domain && $domain =~ m!^(https?://[^/]+)!s ) ? $1 : '';
    $path
        = ($path)
        ? $this->path_change( $path, $ENV{'SCRIPT_NAME'}, 'abs' )
        : $ENV{'SCRIPT_NAME'};

    $html
        =~ s/(<object[\x09\x0A\x0C\x0D\x20]+[^<>]*>.*?<\/object>)/$this->object_rel_to_abs($1,$path,$full,$domain)/seig;
    $html
        =~ s/(<embed[\x09\x0A\x0C\x0D\x20]+[^<>]*>)/$this->embed_rel_to_abs($1,$path,$full,$domain)/seig;

    $html
        =~ s/(<[^<>]*[\x09\x0A\x0C\x0D\x20]+(?:href|src|background|action)=["']?)(?![0-9a-zA-Z_]+:)([^"\x09\x0A\x0C\x0D\x20]+)([^<>]*>)/$1 . (($domain) ? $domain . $this->path_change($2,$path,'abs') : ($full) ? $this->full_path($this->path_change($2,$path,'abs')) : $this->path_change($2,$path,'abs')) . $3/seig;
    $html
        =~ s/(<param[\x09\x0A\x0C\x0D\x20]+name=["']?(?:src|movie)["']?[\x09\x0A\x0C\x0D\x20]+value=["']?)(?![0-9a-zA-Z_]+:)([^"\x09\x0A\x0C\x0D\x20]+)([^<>]*>)/$1 . (($domain) ? $domain . $this->path_change($2,$path,'abs') : ($full) ? $this->full_path($this->path_change($2,$path,'abs')) : $this->path_change($2,$path,'abs')) . $3/seig;
    $html
        =~ s/(<video(?![0-9a-zA-Z_])[^<>]*[\x09\x0A\x0C\x0D\x20]+preview=["']?)(?![0-9a-zA-Z_]+:)([^"\x09\x0A\x0C\x0D\x20]+)([^<>]*>)/$1 . (($domain) ? $domain . $this->path_change($2,$path,'abs') : ($full) ? $this->full_path($this->path_change($2,$path,'abs')) : $this->path_change($2,$path,'abs')) . $3/seig;
    $html
        =~ s/=([^&="':]*%2F[^&="']+)/'=' . $this->urlencoded_rel_to_abs($1,$path,$full,$domain)/seig;
    $html =~ s/[\x09]*<!--(?![\x0A\x0D]).*?-->[\x0A\x0D\x09]*//sg
        if $delete_comment;

    if ( $anchor && $default::id ) {
        my $id = $default::id;
        $id =~ s/^[^a-zA-Z]//;
        $id =~ s/_/-/g;
        if ( $id =~ /^[a-zA-Z]/ ) {
            $html
                =~ s/(<a(?=[\x09\x0A\x0C\x0D\x20])[^<>]*[\x09\x0A\x0C\x0D\x20]+href="\#)a([0-9]+"[^<>]*>)/$1 . $id . $2/seig;
            $html
                =~ s/(<[a-zA-Z][^<>]*[\x09\x0A\x0C\x0D\x20]+id=")a([0-9]+"[^<>]*>)/$1 . $id . $2/seig;
            $html
                =~ s/(<a(?=[\x09\x0A\x0C\x0D\x20])[^<>]*[\x09\x0A\x0C\x0D\x20]+name=")a([0-9]+"[^<>]*>)/$1 . $id . $2/seig;
        }
    }
    return $html;
}

sub object_rel_to_abs ($$;$$$) {
    my ( $this, $html, $path, $full, $domain ) = @_;
    if ( $html
        =~ /<param[\x09\x0A\x0C\x0D\x20]+name=["']?movie["']?[\x09\x0A\x0C\x0D\x20]+value=["']?(?![0-9a-zA-Z_]+:)([^"'\x09\x0A\x0C\x0D\x20]+\/player\.swf)[^<>]*>/si
        )
    {
        my $player_path = $this->path_change( $1, $path, 'abs' );
        $html
            =~ s/(\Wpath=)([^&="':]*%2F[^&="']+)/$1 . $this->urlencoded_rel_to_abs($2,$player_path,$full,$domain)/seig;
    }
    return $html;
}

sub embed_rel_to_abs ($$;$$$) {
    my ( $this, $html, $path, $full, $domain ) = @_;
    if ( $html
        =~ /[\x09\x0A\x0C\x0D\x20]+src=["']?(?![0-9a-zA-Z_]+:)([^"'\x09\x0A\x0C\x0D\x20]+\/player\.swf)/si
        )
    {
        my $player_path = $this->path_change( $1, $path, 'abs' );
        $html
            =~ s/(\Wpath=)([^&="':]*%2F[^&="']+)/$1 . $this->urlencoded_rel_to_abs($2,$player_path,$full,$domain)/seig;
    }
    return $html;
}

sub urlencoded_rel_to_abs ($$;$$$) {
    my ( $this, $url, $path, $full, $domain ) = @_;
    $domain = ( $domain && $domain =~ m!^(https?://[^/]+)!s ) ? $1 : '';
    $url =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
    $url
        = ($domain) ? $domain . $this->path_change( $url, $path, 'abs' )
        : ($full)
        ? $this->full_path( $this->path_change( $url, $path, 'abs' ) )
        : $this->path_change( $url, $path, 'abs' );
    return $this->url_encode($url);
}

########################################################################
# 全角英数字を半角に変換
sub z_to_h ($$;$$) {
    my ( $this, $text, $icode, $kana ) = @_;
    return '' if !defined $text || $text eq '';
    if ( !$icode || ( $icode ne 'utf8' && $icode ne 'ascii' ) ) {
        $text = $cgi::Jcode->set( \$text, $icode )->utf8;
        $icode = $this->ename_to_jname( $cgi::Jcode->icode );
    }
    utf8::decode($text) if !utf8::is_utf8($text);
    $text =~ s/([\x{ff00}-\x{ff5e}])/pack("U",unpack("U",$1) - 65248)/ges;
    $text =~ s/\x{3000}/\x20/gs;
    if ($kana) {
        $text =~ s/・/･/gs;
        $text =~ s/ヲ/ｦ/gs;
        $text =~ s/ァ/ｧ/gs;
        $text =~ s/ィ/ｨ/gs;
        $text =~ s/ゥ/ｩ/gs;
        $text =~ s/ェ/ｪ/gs;
        $text =~ s/ォ/ｫ/gs;
        $text =~ s/ャ/ｬ/gs;
        $text =~ s/ュ/ｭ/gs;
        $text =~ s/ョ/ｮ/gs;
        $text =~ s/ッ/ｯ/gs;
        $text =~ s/ー/ｰ/gs;
        $text =~ s/ア/ｱ/gs;
        $text =~ s/イ/ｲ/gs;
        $text =~ s/ヴ/ｳﾞ/gs;
        $text =~ s/ウ/ｳ/gs;
        $text =~ s/エ/ｴ/gs;
        $text =~ s/オ/ｵ/gs;
        $text =~ s/ガ/ｶﾞ/gs;
        $text =~ s/カ/ｶ/gs;
        $text =~ s/ギ/ｷﾞ/gs;
        $text =~ s/キ/ｷ/gs;
        $text =~ s/グ/ｸﾞ/gs;
        $text =~ s/ク/ｸ/gs;
        $text =~ s/ゲ/ｹﾞ/gs;
        $text =~ s/ケ/ｹ/gs;
        $text =~ s/ゴ/ｺﾞ/gs;
        $text =~ s/コ/ｺ/gs;
        $text =~ s/ザ/ｻﾞ/gs;
        $text =~ s/サ/ｻ/gs;
        $text =~ s/ジ/ｼﾞ/gs;
        $text =~ s/シ/ｼ/gs;
        $text =~ s/ズ/ｽﾞ/gs;
        $text =~ s/ス/ｽ/gs;
        $text =~ s/ゼ/ｾﾞ/gs;
        $text =~ s/セ/ｾ/gs;
        $text =~ s/ゾ/ｿﾞ/gs;
        $text =~ s/ソ/ｿ/gs;
        $text =~ s/ダ/ﾀﾞ/gs;
        $text =~ s/タ/ﾀ/gs;
        $text =~ s/ヂ/ﾁﾞ/gs;
        $text =~ s/チ/ﾁ/gs;
        $text =~ s/ヅ/ﾂﾞ/gs;
        $text =~ s/ツ/ﾂ/gs;
        $text =~ s/デ/ﾃﾞ/gs;
        $text =~ s/テ/ﾃ/gs;
        $text =~ s/ド/ﾄﾞ/gs;
        $text =~ s/ト/ﾄ/gs;
        $text =~ s/ナ/ﾅ/gs;
        $text =~ s/ニ/ﾆ/gs;
        $text =~ s/ヌ/ﾇ/gs;
        $text =~ s/ネ/ﾈ/gs;
        $text =~ s/ノ/ﾉ/gs;
        $text =~ s/バ/ﾊﾞ/gs;
        $text =~ s/パ/ﾊﾟ/gs;
        $text =~ s/ハ/ﾊ/gs;
        $text =~ s/ビ/ﾋﾞ/gs;
        $text =~ s/ピ/ﾋﾟ/gs;
        $text =~ s/ヒ/ﾋ/gs;
        $text =~ s/ブ/ﾌﾞ/gs;
        $text =~ s/プ/ﾌﾟ/gs;
        $text =~ s/フ/ﾌ/gs;
        $text =~ s/ベ/ﾍﾞ/gs;
        $text =~ s/ペ/ﾍﾟ/gs;
        $text =~ s/ヘ/ﾍ/gs;
        $text =~ s/ボ/ﾎﾞ/gs;
        $text =~ s/ポ/ﾎﾟ/gs;
        $text =~ s/ホ/ﾎ/gs;
        $text =~ s/マ/ﾏ/gs;
        $text =~ s/ミ/ﾐ/gs;
        $text =~ s/ム/ﾑ/gs;
        $text =~ s/メ/ﾒ/gs;
        $text =~ s/モ/ﾓ/gs;
        $text =~ s/ヤ/ﾔ/gs;
        $text =~ s/ユ/ﾕ/gs;
        $text =~ s/ヨ/ﾖ/gs;
        $text =~ s/ラ/ﾗ/gs;
        $text =~ s/リ/ﾘ/gs;
        $text =~ s/ル/ﾙ/gs;
        $text =~ s/レ/ﾚ/gs;
        $text =~ s/ロ/ﾛ/gs;
        $text =~ s/ワ/ﾜ/gs;
        $text =~ s/ン/ﾝ/gs;
    }
    $this->jcode_convert( \$text, $icode, 'utf8' )
        if $icode ne 'utf8' && $icode ne 'ascii';
    return $text;
}

sub h_to_z ($$;$$) {
    my ( $this, $text, $icode, $ascii ) = @_;
    return '' if !defined $text || $text eq '';
    if ( !$icode || ( $icode ne 'utf8' && $icode ne 'ascii' ) ) {
        $text = $cgi::Jcode->set( \$text, $icode )->utf8;
        $icode = $this->ename_to_jname( $cgi::Jcode->icode ) if !$icode;
    }
    utf8::decode($text) if !utf8::is_utf8($text);
    if ($ascii) {
        $text =~ s/\x20/\x{3000}/gs;
        $text =~ s/([\x{0020}-\x{007e}])/pack("U",unpack("U",$1) + 65248)/ges;
    }
    $text =~ s/･/・/gs;
    $text =~ s/ｦ/ヲ/gs;
    $text =~ s/ｧ/ァ/gs;
    $text =~ s/ｨ/ィ/gs;
    $text =~ s/ｩ/ゥ/gs;
    $text =~ s/ｪ/ェ/gs;
    $text =~ s/ｫ/ォ/gs;
    $text =~ s/ｬ/ャ/gs;
    $text =~ s/ｭ/ュ/gs;
    $text =~ s/ｮ/ョ/gs;
    $text =~ s/ｯ/ッ/gs;
    $text =~ s/ｰ/ー/gs;
    $text =~ s/ｱ/ア/gs;
    $text =~ s/ｲ/イ/gs;
    $text =~ s/ｳﾞ/ヴ/gs;
    $text =~ s/ｳ/ウ/gs;
    $text =~ s/ｴ/エ/gs;
    $text =~ s/ｵ/オ/gs;
    $text =~ s/ｶﾞ/ガ/gs;
    $text =~ s/ｶ/カ/gs;
    $text =~ s/ｷﾞ/ギ/gs;
    $text =~ s/ｷ/キ/gs;
    $text =~ s/ｸﾞ/グ/gs;
    $text =~ s/ｸ/ク/gs;
    $text =~ s/ｹﾞ/ゲ/gs;
    $text =~ s/ｹ/ケ/gs;
    $text =~ s/ｺﾞ/ゴ/gs;
    $text =~ s/ｺ/コ/gs;
    $text =~ s/ｻﾞ/ザ/gs;
    $text =~ s/ｻ/サ/gs;
    $text =~ s/ｼﾞ/ジ/gs;
    $text =~ s/ｼ/シ/gs;
    $text =~ s/ｽﾞ/ズ/gs;
    $text =~ s/ｽ/ス/gs;
    $text =~ s/ｾﾞ/ゼ/gs;
    $text =~ s/ｾ/セ/gs;
    $text =~ s/ｿﾞ/ゾ/gs;
    $text =~ s/ｿ/ソ/gs;
    $text =~ s/ﾀﾞ/ダ/gs;
    $text =~ s/ﾀ/タ/gs;
    $text =~ s/ﾁﾞ/ヂ/gs;
    $text =~ s/ﾁ/チ/gs;
    $text =~ s/ﾂﾞ/ヅ/gs;
    $text =~ s/ﾂ/ツ/gs;
    $text =~ s/ﾃﾞ/デ/gs;
    $text =~ s/ﾃ/テ/gs;
    $text =~ s/ﾄﾞ/ド/gs;
    $text =~ s/ﾄ/ト/gs;
    $text =~ s/ﾅ/ナ/gs;
    $text =~ s/ﾆ/ニ/gs;
    $text =~ s/ﾇ/ヌ/gs;
    $text =~ s/ﾈ/ネ/gs;
    $text =~ s/ﾉ/ノ/gs;
    $text =~ s/ﾊﾞ/バ/gs;
    $text =~ s/ﾊﾟ/パ/gs;
    $text =~ s/ﾊ/ハ/gs;
    $text =~ s/ﾋﾞ/ビ/gs;
    $text =~ s/ﾋﾟ/ピ/gs;
    $text =~ s/ﾋ/ヒ/gs;
    $text =~ s/ﾌﾞ/ブ/gs;
    $text =~ s/ﾌﾟ/プ/gs;
    $text =~ s/ﾌ/フ/gs;
    $text =~ s/ﾍﾞ/ベ/gs;
    $text =~ s/ﾍﾟ/ペ/gs;
    $text =~ s/ﾍ/ヘ/gs;
    $text =~ s/ﾎﾞ/ボ/gs;
    $text =~ s/ﾎﾟ/ポ/gs;
    $text =~ s/ﾎ/ホ/gs;
    $text =~ s/ﾏ/マ/gs;
    $text =~ s/ﾐ/ミ/gs;
    $text =~ s/ﾑ/ム/gs;
    $text =~ s/ﾒ/メ/gs;
    $text =~ s/ﾓ/モ/gs;
    $text =~ s/ﾔ/ヤ/gs;
    $text =~ s/ﾕ/ユ/gs;
    $text =~ s/ﾖ/ヨ/gs;
    $text =~ s/ﾗ/ラ/gs;
    $text =~ s/ﾘ/リ/gs;
    $text =~ s/ﾙ/ル/gs;
    $text =~ s/ﾚ/レ/gs;
    $text =~ s/ﾛ/ロ/gs;
    $text =~ s/ﾜ/ワ/gs;
    $text =~ s/ﾝ/ン/gs;
    $this->jcode_convert( \$text, $icode, 'utf8' )
        if $icode ne 'utf8' && $icode ne 'ascii';
    return $text;
}

########################################################################
#桁揃え
sub jfold ($$;$$$$$$) {
    my ( $this, $text, $width, $lf, $row, $punc, $quot, $icode ) = @_;
    my ( $wc, $rc );
    return '' if !defined $text || $text eq '';
    $text =~ s/<br(?=[\x09\x0A\x0C\x0D\x20\/>])[^>]*>/\x0A/igs;
    $text =~ s/<\/p><p>/\x0A\x0A/igs;
    $this->detag( \$text );
    $punc  = "\x0A" if !defined $punc  || $punc eq '';
    $width = 70     if !defined $width || !$width;
    $lf    = ''     if !defined $lf;
    $row   = 0      if !defined $row   || $row eq '';
    $quot  = ''     if !defined $quot;
    $rc = $wc = 0;

    if ( $icode && $icode ne 'utf8' && $icode ne 'ascii' ) {
        $text = $cgi::Jcode->set( \$text, $icode )->utf8;
        $icode = $this->ename_to_jname( $cgi::Jcode->icode ) if !$icode;
    }
    else {
        $icode = 'utf8';
    }
    $text = $this->h_to_z( $text, 'utf8' );
    $text
        =~ s/(&[0-9a-zA-Z_\#]+;|\x0D\x0A|.)/$this->split_string($1,\$width,\$lf,\$row,\$punc,\$quot,\$rc,\$wc)/egs;
    $text = $quot . $text;
    if ( ( !$row || $row > $rc ) && $wc > 0 && $width > $wc ) {
        $text .= ' ' x ( $width - $wc ) if $lf ne 'r' && $lf ne 'I';
        $text .= $punc . $quot;
    }
    if ( $row > 0 ) {
        $text
            =~ s/\Q$punc$quot\E.{0,2}[\x09\x0A\x0C\x0D\x20]*(?:\Q$punc$quot\E[\x09\x0A\x0C\x0D\x20]*)*$//s;
        if ( $rc >= $row ) {
            if ( $text
                =~ s/(&[0-9a-zA-Z_\#]+;(?:[\x21-\x7e]{1,2}|.)|(?:[\x21-\x7e]{1,2}|[^\x09\x0A\x0C\x0D\x20]){2}|\x20+)$//s
                )
            {
                $_  = $1;
                $rc = (s/[^\x20-\x7e]//g) * 2;
                $rc += s/[\x20-\x7e]//g;
                $rc -= 3;
                $rc = 0 if $rc < 0;
            }
            else {
                $rc = 0;
            }
            $text .= '...' . ( ' ' x ( $rc || 0 ) );
        }
    }
    else {
        $text =~ s/(?:\Q$punc$quot\E)*$//s;
    }
    $this->jcode_convert( \$text, $icode, 'utf8' )
        if $icode ne 'utf8' && $icode ne 'ascii';
    return $text;
}

sub split_string ($$$$$$$$$) {
    my ( $this, $word, $width, $lf, $row, $punc, $quot, $rc, $wc ) = @_;
    return '' if defined $$row && $$row && $$rc >= $$row;
    if ( $word =~ /[\x0D\x0A]/ ) {
        $word = '';
        if ( $$lf eq 'i' || $$lf eq 'I' ) {
            return '';
        }
        else {
            $word = ' ' x ( $$width - $$wc ) if $$lf ne 'r' && $$width > $$wc;
            $word .= $$punc . $$quot if $$width > $$wc;
            $$wc = 0;
            $$rc++;
            return $word;
        }
    }
    my $plus = ( bytes::length $word >= 2 ) ? 2 : 1;
    $$wc += $plus;
    if ( $$wc == $$width ) {
        $word .= $$punc . $$quot;
        $$wc = 0;
        $$rc++;
    }
    elsif ( $$wc > $$width ) {
        $word
            = ( ( $$lf ne 'r' && $$lf ne 'I' ) ? ' ' : '' )
            . $$punc
            . $$quot
            . $word;
        $$wc = $plus;
        $$rc++;
    }
    return $word;
}

sub cut_string ($$$;$) {
    my ( $this, $str, $max_length, $tag ) = @_;
    return '' if !defined $str;
    $str =~ s/[\x09\x0A\x0C\x0D\x20]+/ /gs;
    $str
        =~ s/<(.*?)>/(($1 =~ m! (href|alt)="(mailto:|)(.*?)"!i) ? ' ' . $3 . ' ' : '')/egs
        if $tag && $tag eq 'no';
    return ( $tag && $tag eq 'no' )
        ? $this->escape_except_html_entity($str)
        : $str
        if !$max_length || bytes::length($str) <= $max_length;
    my $cut_str = '';
    my $length  = 0;
    $max_length -= 3;

    if ( $tag && $tag eq 'i' ) {
        my $over = 0;
        while ( $str =~ /\G(<.*?>|&[#0-9a-zA-Z_]+;|.)/sg ) {
            my $char = $1;
            $cut_str .= $char if !$over || $char =~ /^<.*?>$/s;
            $length
                += ( ( $char =~ /^<.*?>$/s ) ? 0
                : ( $char =~ /^&[#0-9a-zA-Z_]+;$/ ) ? 1
                : ( bytes::length($char) >= 2 )     ? 2
                :                                     1 );
            if ( $length >= $max_length ) {
                $over = 1;
                last;
            }
        }
        if ($over) {
            $cut_str = $this->tag_fix( $cut_str, 2, 1, 1 );
            $cut_str =~ s/((?:<[^>]*?>)*)$/...$1/s;
        }
        return $cut_str;
    }
    else {
        while ( $str =~ /\G(.)/sg ) {
            my $char = $1;
            $cut_str .= $char;
            $length += ( bytes::length($char) >= 2 ) ? 2 : 1;
            return ( ( $tag && $tag eq 'no' )
                ? $this->escape_except_html_entity($cut_str)
                : $cut_str )
                . '...'
                if $length >= $max_length;
        }
        return $cut_str;
    }
}

sub str_length ($$;$) {
    my ( $this, $str, $tag ) = @_;
    return 0 if !defined $str;
    $str =~ s/[\x09\x0A\x0C\x0D\x20]+/ /gs;
    $str
        =~ s/<(.*?)>/(($1 =~ m! (href|alt)="(mailto:|)(.*?)"!i) ? ' ' . $3 . ' ' : '')/egs
        if $tag && $tag eq 'no';
    my $length = 0;
    if ( $tag && $tag eq 'i' ) {
        while ( $str =~ /\G(<.*?>|&[#0-9a-zA-Z_]+;|.)/sg ) {
            my $char = $1;
            $length
                += ( ( $char =~ /^<.*?>$/s ) ? 0
                : ( $char =~ /^&[#0-9a-zA-Z_]+;$/ ) ? 1
                : ( bytes::length($char) >= 2 )     ? 2
                :                                     1 );
        }
    }
    else {
        while ( $str =~ /\G(.)/sg ) {
            $length += ( bytes::length($1) >= 2 ) ? 2 : 1;
        }
    }
    return $length;
}

########################################################################
#引用部分の着色
sub quotation_color ($$;@) {
    my ( $this, $str, @color ) = @_;
    return $str
        if !@color
        || !defined $color[0]
        || ( @color == 1 && $color[0] eq '' );

    @color
        = split( /[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,
        $color[0] )
        if @color == 1 && $color[0] =~ /,/;

    my ( $pat, $op, $count );
    1 while $str =~ s/<([^>]*)\x0A([^<]*)>/<$1$2>/gs;

    $op = '(?![\x09\x0A\x0C\x0D\x20]*(>|\&gt\;|＞))';
    foreach (@color) {
        $count++;
        $pat
            = '(>[\x09\x0A\x0C\x0D\x20]*|\&gt\;[\x09\x0A\x0C\x0D\x20]*|＞[\x09\x0A\x0C\x0D\x20]*)'
            x $count;
        $op = '' if $count == @color;
        $str =~ s/^($pat$op.*?)$/<span style="color:$_">$1<\/span>/mg;
    }
    return $str;
}

########################################################################
#csv/tsv等の分割・結合
sub svsplit ($$$;$) {
    my ( $this, $s, $line, $limit ) = @_;
    return () if !defined $line;
    my (@array);
    if ( $s eq "\x09" ) {
        @array
            = ( defined $limit && $limit ne '' )
            ? split( /\x09/, $line, $limit )
            : split( /\x09/, $line );
        map {
            $_ = '' if !defined $_;
            $_ =~ s/\x0B/\x0A/g;
            $_ =~ s/&#09;/\x09/;
        } @array;
    }
    else {
        $s = ',' if !defined $s || $s eq '';
        use Text::ParseWords;
        @array = quotewords( $s, 1, $line );
        map {
            $_ = '' if !defined $_;
            $_ =~ s/""/"/g;
            $_ =~ s/^"//;
            $_ =~ s/"$//;
        } @array;
    }
    return @array;
}

sub svjoin ($$@) {
    my ( $this, $s, @array ) = @_;
    return '' if !@array;
    if ( $s eq "\x09" ) {
        map {
            $_ =~ s/\t/&#09;/gs;
            $_ =~ s/(\x0D\x0A|\x0D|\x0A)/\x0B/gs;
        } @array;
    }
    else {
        map {
            $_ = '' if !defined $_;
            if ( $_ =~ /$s/ || $_ =~ /["\x09\x0A\x0C\x0D\x20]/ ) {
                $_ =~ s/"/""/g;
                $_ = '"' . $_ . '"';
            }
        } @array;
    }
    return join( $s, @array );
}

########################################################################
#パターンに一致するファイル/ディレクトリのリストを返す
sub fileglob ($$$;$$$$) {
    my ( $this, $dir, $pat, $type, $find, $unmatched, $skipdir ) = @_;
    return if !defined $dir;
    my ( @file, @matched, $name );
    @matched = ();
    $type = 'f' if !defined $type || $type eq '';
    $dir =~ s!/$!!;
    return unless opendir( DIR, $dir );
    @file = grep( $_ !~ /^\.\.?$/, readdir(DIR) );
    closedir(DIR);

    foreach $name (@file) {
        next
            if $skipdir
            && -d $dir . '/' . $name
            && eval { $name =~ /$skipdir/ };
        if ( eval "-$type '$dir/$name'" ) {
            if (   !defined $pat
                || $pat eq ''
                || ( !$unmatched && $name =~ /$pat/ )
                || ( $unmatched  && $name !~ /$pat/ ) )
            {
                push( @matched, $dir . '/' . $name );
            }
        }
        push(
            @matched,
            $this->fileglob(
                $dir . '/' . $name, $pat,       $type,
                $find,              $unmatched, $skipdir
            )
        ) if $find && -d $dir . '/' . $name;
    }
    return (wantarray) ? @matched : $matched[0];
}

########################################################################
#親ディレクトリのパーミッションを設定する
sub chmod_dir ($$$) {
    my ( $this, $mode, $path ) = @_;
    $path =~ s/[^\/]+$//;
    $path = './' if !defined $path || !$path;
    if ( chmod( $mode, $path ) ) {
        return 1;
    }
    else {
        return 0;
    }
}

########################################################################
#ファイルを複製する
sub file_copy ($$$;$) {
    my ( $this, $file, $copy, $file_mode ) = @_;
    my $copied = 0;
    return 0
        if !defined $file || $file eq '' || !defined $copy || $copy eq '';
    return 1 if $file eq $copy;
    if ( $cgi::File_Copy_enabled || eval 'use File::Copy qw/copy/; 1;' ) {
        $copied = 1 if copy( $file, $copy );
        $cgi::File_Copy_enabled = 1;
    }
    if ( !$copied ) {
        local ( *FILE, *COPY );
        if ( open( FILE, $file ) ) {
            binmode FILE;
            flock( FILE, LOCK_SH );
            if ( open( COPY, ">>$copy" ) ) {
                binmode COPY;
                flock( COPY, LOCK_EX );
                truncate( COPY, 0 );
                seek( COPY, 0, 0 );
                my $buf;
                while ( read( FILE, $buf, $env::buffer_size ) ) {
                    print COPY $buf;
                }
                close(COPY);
                $copied = 1;
            }
            close(FILE);
        }
    }
    if ($copied) {
        chmod(
            ( ( defined $file_mode ) ? $file_mode : $env::file_mode ),
            $copy
        );
    }
    else {
        push(
            @{  ( ( ref $cgi::action ) ? $cgi::action : $this )->{'dialog'}
            },
            'ファイルコピーに失敗しました。使用可能な空き容量が不足している可能性があります。'
        );
    }
    return $copied;
}

######################################################################
# ディレクトリをコピー
sub directory_copy ($$$;$$$$$) {
    my ( $this, $dir, $clone, $find, $ignore_array, $maxsize,
        $sizeover_array, $check_path )
        = @_;
    my ( @files, @directories, $entry, $f, $copied );
    $copied = 0;
    local (*DIR);
    return 0 unless $this->set_dir($clone) && opendir( DIR, $dir );
    while ( $entry = readdir(DIR) ) {
        my $path = $dir . '/' . $entry;
        next
            if $entry =~ /^\.(?![0-9a-zA-Z_])/
            || (
               ref $ignore_array
            && @$ignore_array
            && grep( ( !$check_path && eval { $entry =~ /$_/ } )
                    || ( $check_path && eval { $path =~ /$_/ } ),
                @$ignore_array )
            );
        if ( -f $path ) {
            push( @files, $entry );
        }
        elsif ( $find && -d $path ) {
            push( @directories, $entry );
        }
    }
    closedir(DIR);
    foreach $f (@files) {
        if ( !$maxsize || ( -s $dir . '/' . $f ) <= $maxsize ) {
            $copied = 1
                if $this->file_copy( $dir . '/' . $f, $clone . '/' . $f );
        }
        elsif ( ref $sizeover_array eq 'ARRAY' ) {
            push( @$sizeover_array, $dir . '/' . $f );
        }
    }
    if ( $find && @directories ) {
        my $d;
        foreach $d (@directories) {
            $copied = 1
                if $this->directory_copy(
                $dir . '/' . $d,
                $clone . '/' . $d,
                $find, $ignore_array, $maxsize, $sizeover_array, $check_path
                );
        }
    }
    return $copied;
}

########################################################################
# 最小値のキーを返します。
sub minimum ($$) {
    no warnings qw(numeric);
    my ( $this, $hash ) = @_;
    my ( $key, $value );
    while ( my ( $k, $v ) = each %$hash ) {
        ( $key, $value ) = ( $k, $v ) if !defined $value || $v < $value;
    }
    return $key;
}

########################################################################
# 最大値のキーを返します。
sub maximum ($$) {
    no warnings qw(numeric);
    my ( $this, $hash ) = @_;
    my ( $key, $value );
    while ( my ( $k, $v ) = each %$hash ) {
        ( $key, $value ) = ( $k, $v ) if !defined $value || $v > $value;
    }
    return $key;
}

########################################################################
# 新規キーNoを返します。
sub new_number ($$;$) {
    no warnings qw(numeric);
    my ( $this, $ref, $n ) = @_;
    $n += 0;
    if ( ref $ref eq 'HASH' ) {
        $n++ while defined $$ref{$n};
    }
    elsif ( ref $ref eq 'ARRAY' ) {
        $n++ while grep( $_ == $n, @$ref );
    }
    return $n;
}

########################################################################
# ソート
sub sort_by ($$$;$) {
    no warnings qw(numeric);
    my ( $this, $a, $b, $sort_order ) = @_;
    if ( !defined $$a || !defined $$b ) {
        return 0;
    }
    elsif ( defined $sort_order && $sort_order eq 'descend' ) {
        return $$b <=> $$a || uc($$b) cmp uc($$a) || $$b cmp $$a;
    }
    else {
        return $$a <=> $$b || uc($$a) cmp uc($$b) || $$a cmp $$b;
    }
}

########################################################################
# 位取り
sub digit ($$;$$) {
    my ( $this, $str, $s, $n ) = @_;
    my ($p);
    return '' if !defined $str;
    $s = ',' if !defined $s || $s eq '';
    $n = 3 if !$n;
    $p = '[0-9]{' . ( $n + 1 ) . ',}';
    $str =~ s/(\.$p)/$this->n_digit($1,$s,$n)/ge;
    $str =~ s/($p(?=([^0-9]|$)))/$this->p_digit($1,$s,$n)/ge;
    return $str;
}

sub n_digit ($$;$$) {
    my ( $this, $d, $s, $n ) = @_;
    my $p = '[0-9]{' . $n . '}';
    $d =~ s/($p)/$1$s/g;
    return $d;
}

sub p_digit ($$;$$) {
    my ( $this, $d, $s, $n ) = @_;
    my $p = '[0-9]{' . $n . '}';
    1 while $d =~ s/([0-9])($p(?=([^0-9]|$)))/$1$s$2/g;
    return $d;
}

sub round_down ($$$;$) {
    my ( $this, $d, $r, $t ) = @_;
    return $d if !$d || !defined $r || $r eq '';
    $t = 'round' if !$t;
    $r = ( $r - 1 ) * 10;
    if ( $r > 0 ) {
        $d
            = ( $t eq 'round' ) ? int( $d * $r + 0.5 ) / $r
            : ( $t eq 'floor' )             ? int( $d * $r ) / $r
            : ( int( $d * $r ) == $d * $r ) ? int( $d * $r ) / $r
            :                                 int( $d * $r + 1 ) / $r;
    }
    elsif ( $r == 0 ) {
        $d
            = ( $t eq 'round' ) ? int( $d + 0.5 )
            : ( $t eq 'floor' ) ? int($d)
            : ( int($d) == $d ) ? int($d)
            :                     int($d) + 1;
    }
    elsif ( $r < 0 ) {
        $r *= -1;
        $d
            = ( $t eq 'round' ) ? int( $d / $r + 0.5 ) * $r
            : ( $t eq 'floor' )             ? int( $d / $r ) * $r
            : ( int( $d / $r ) == $d / $r ) ? int( $d / $r ) * $r
            :                                 int( $d / $r + 1 ) * $r;
    }
    return $d;
}

########################################################################
#404エラーメッセージ
sub error404 ($;$) {
    my ( $this, $exit ) = @_;
    binmode STDOUT, ":utf8";
    print <<END;
Status: 404 Not Found
Content-Type: text/html

<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html><head>
<title>404 Not Found</title>
</head><body>
<h1>Not Found</h1>
<p>The requested object was not found on this server.</p>
</body></html>
END
    binmode STDOUT;
    exit if !$exit || $exit ne 'no';

    #<hr />
    #<address>$ENV{'SERVER_SIGNATURE'}</address>
}

sub error403 ($;$) {
    my ( $this, $exit ) = @_;
    binmode STDOUT, ":utf8";
    print <<END;
Status: 403 Forbidden
Content-Type: text/html

<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html><head>
<title>403 Forbidden</title>
</head><body>
<h1>Forbidden</h1>
<p>You don't have permission to access $ENV{'SCRIPT_NAME'} on this server.</p>
</body></html>
END
    binmode STDOUT;
    exit if !$exit || $exit ne 'no';

    #<hr />
    #<address>$ENV{'SERVER_SIGNATURE'}</address>
}

#Loginのダミー
sub login ($) {
    my $this = shift;
    $this->error404;
}

#Runのダミー
sub run ($) {
    my $this = shift;
    $this->error404;
}

#304エラーメッセージ
sub error304 ($;$) {
    my ( $this, $exit ) = @_;
    print "Status: 304 Not Modified\x0A\x0A";
    exit if !$exit || $exit ne 'no';
}

########################################################################
#エラーメッセージ
sub dialog ($) {
    my $this = shift;
    if ( ref $this->{'dialog'} && @{ $this->{'dialog'} } ) {
        my @dialog = grep( defined $_ && $_ ne '', @{ $this->{'dialog'} } );
        undef @{ $this->{'dialog'} };
        return @dialog;
    }
    else {
        return ();
    }
}

######################################################################
#ログインCookieネーム
sub login_cookie_name ($;$) {
    my ( $this, $refresh ) = @_;
    $env::login_cookie_name
        = ( $env::register_cgi && $env::register_cgi eq '../pswd/start.cgi' )
        ? $env::home_dir_id . 'id'
        : 'id'
        if !$env::login_cookie_name || $refresh;
    return $env::login_cookie_name;
}

######################################################################
sub DESTROY {
    my $this = shift;
    if ( $this->{'clipboard_expired'} ) {
        if ( $this->{'clipboard_id'} ) {
            if ( -d $env::clipboard . '/' . $this->{'clipboard_id'} ) {
                rmtree $env::clipboard . '/' . $this->{'clipboard_id'};
            }
            if ( -d $env::public_clipboard . '/' . $this->{'clipboard_id'} ) {
                rmtree $env::public_clipboard . '/' . $this->{'clipboard_id'};
            }
        }
    }
    if (   $this->{'is_decoder'}
        && $main::clipboard_id
        && $main::clipboard_id !~ m!/! )
    {
        if ( -d $env::clipboard . '/' . $main::clipboard_id ) {
            rmtree $env::clipboard . '/' . $main::clipboard_id;
        }
        if ( -d $env::public_clipboard . '/' . $main::clipboard_id ) {
            rmtree $env::public_clipboard . '/' . $main::clipboard_id;
        }
    }
    if ( $this->{'upload_check_key'}
        && -f $env::clipboard . '/' . $this->{'upload_check_key'} )
    {
        unlink $env::clipboard . '/' . $this->{'upload_check_key'};
    }
}

#sub import {
BEGIN {

    #MIME-TYPE 設定
    %Common::mime_type = (
        'txt'  => 'text/plain',
        'html' => 'text/html',
        'htm'  => 'text/html',
        'gif'  => 'image/gif',
        'jpg'  => 'image/jpeg',
        'jpeg' => 'image/jpeg',
        'png'  => 'image/png',
        'bmp'  => 'image/bmp',
        'svg'  => 'image/svg+xml',
        'svgz' => 'image/svg+xml',
        'mid'  => 'audio/midi',
        'mpg'  => 'video/mpeg',
        'mov'  => 'video/quicktime',
        'avi'  => 'video/x-msvideo',
        'swf'  => 'application/x-shockwave-flash',
        'dcr'  => 'application/x-director',
        'csv'  => 'text/comma-separated-values',
        'tsv'  => 'text/tab-separated-values',
        'pdf'  => 'application/pdf',
        'hqx'  => 'application/mac-binhex40',
        'lzh'  => 'application/lha',
        'zip'  => 'application/zip',
        'bin'  => 'application/x-macbinary',
        'sit'  => 'application/x-stuffit',
        'ra'   => 'audio/x-pn-realaudio',
        'mp3'  => 'audio/mpeg',
        'm4a'  => 'audio/m4a',
        'pic'  => 'image/pict',
        'pct'  => 'image/pict',
        'pict' => 'image/pict',
        'tif'  => 'image/tiff',
        'tiff' => 'image/tiff',
        'ico'  => 'image/x-icon',
        'mp4'  => 'video/mp4',
        'flv'  => 'video/x-flv',
        'f4v'  => 'video/x-flv',
        '3g2'  => 'video/3gpp2',
        '3gp'  => 'video/3gpp',
        'wmv'  => 'video/x-ms-wmv',
        'asf'  => 'video/x-ms-asf',
        'aac'  => 'audio/aac',
        'm4v'  => 'video/mp4',
        'ogg'  => 'video/ogg',
        'ogv'  => 'video/ogg',
        'oga'  => 'audio/ogg',
        'webm' => 'video/webm',
        'xml'  => 'text/xml',
        'kml'  => 'text/xml',
        'epub' => 'application/epub+zip',
        'ps'   => 'application/postscript',
        'eps'  => 'application/postscript',
        'ai'   => 'application/postscript',
        'psd'  => 'application/vnd.adobe.photoshop',
        'tex'  => 'application/x-tex',
        'tgz'  => 'application/x-compressed',
        'wav'  => 'audio/wav',
        'ppt'  => 'application/vnd.ms-powerpoint',
        'xls'  => 'application/vnd.ms-excel',
        'doc'  => 'application/msword',
        'pptx' =>
            'application/vnd.openxmlformats-officedocument.presentationml.presentation',
        'xlsx' =>
            'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
        'docx' =>
            'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
        'odt'     => 'application/vnd.oasis.opendocument.text',
        'ods'     => 'application/vnd.oasis.opendocument.spreadsheet',
        'odp'     => 'application/vnd.oasis.opendocument.presentation',
        'odg'     => 'application/vnd.oasis.opendocument.graphics',
        'odc'     => 'application/vnd.oasis.opendocument.chart',
        'odf'     => 'application/vnd.oasis.opendocument.formula',
        'odi'     => 'application/vnd.oasis.opendocument.image',
        'odb'     => 'application/vnd.oasis.opendocument.database',
        'key'     => 'application/vnd.apple.keynote',
        'numbers' => 'application/vnd.apple.numbers',
        'pages'   => 'application/vnd.apple.pages',
    );

    # 以下は多くの場合プラグイン表示できるもの
    @Common::html5_audio = ( 'wav', 'mp3', 'm4a',  'aac', 'oga' );
    @Common::html5_video = ( 'ogg', 'ogv', 'webm', 'm4v', 'mp4' );
    @Common::video_plug  = (
        'avi', 'wmv', 'mov', 'mpeg', 'mpg', 'm4v', 'qti', 'qtif',
        'mp4', 'rpm', '3gp', '3g2',  'rts', 'rtsp'
    );
    @Common::image_plug = (
        'bmp',  'dcr', 'dir', 'dxr',  'flc',   'pct', 'pdf', 'pic',
        'pict', 'png', 'pnt', 'pntg', 'psd',   'qcp', 'sdp', 'sgi',
        'spl',  'swf', 'svg', 'svgz', 'targa', 'tga', 'tif', 'tiff',
        'vdo',  'vfw', 'viv'
    );
    @Common::sound_plug = (
        'aif', 'aiff', 'au',  'mp3', 'm4a', 'wav',
        'aac', 'kar',  'mid', 'midi'
    );

#@Common::gdv_plug = ('tiff','tif','bmp','webm','m4v','mp4','mpeg4','3gpp','mov','avi','mpegps','wmv','flv','txt','css','html','php','c','cpp','h','hpp','js','doc','docx','xls','xlsx','ppt','pptx','pdf','pages','ai','psd','dxf','svg','eps','ps','ttf','xps');
    @Common::gdv_plug = (
        'tiff', 'tif', 'txt',   'css', 'html', 'php', 'c',    'cpp',
        'h',    'hpp', 'js',    'doc', 'docx', 'xls', 'xlsx', 'ppt',
        'pptx', 'pdf', 'pages', 'ai',  'dxf',  'eps', 'ps',   'ttf',
        'xps'
    );
}

1;
